#include "upnpd.h"
#include "tool.h"
#include "gateway.h"
#include "sample_util.h"
#include "dll.h"
#include "action.h"

/********************************************************
 *		Global Variable Declaration 		*
 ********************************************************/

extern struct GateEnv gate;

DLL start, end;

char *IGDDeviceType[] = {"urn:schemas-upnp-org:device:InternetGatewayDevice:1",
			"urn:schemas-upnp-org:device:WANDevice:1",
			"urn:schemas-upnp-org:device:WANConnectionDevice:1"};
char *IGDServiceType[] = {"urn:schemas-microsoft-com:service:OSInfo:1",
			 "urn:schemas-upnp-org:service:WANCommonInterfaceConfig:1",
			 "urn:schemas-upnp-org:service:WANIPConnection:1"};



/* Global arrays for storing <service> OSInfo  variable names, values, and defaults */
char *osi_varname[] = {"OSMajorVersion",
		       "OSMinorVersion",
		       "OSBuildNumber",
		       "OSMachineName"};
char osi_varval[DS_OSI_VARCOUNT][DS_MAX_VAL_LEN];
char *osi_varval_def[] = {"2","2","14","LinuxBase"};

/* arrays for storing WANCommonInterfaceConfig's variable names , values, and defaults */
char *cic_varname[] = {	"WANAccessType",
			"Layer1UpstreamMaxBitRate",
			"Layer1DownstreamMaxBitRate",
			"PhysicalLinkStatus",
			"WANAccessProvider",
			"MaximumActiveConnections",
			"NumberOfActiveConnections",
			"ActiveConnectionDeviceContainer",
			"ActiveConnectionServiceID",
			"TotalBytesSent",
			"TotalBytesReceived",
			"TotalPacketsSent",
			"TotalPacketsReceived",
			"X_PersonalFirewallEnabled",
			"X_Uptime",
			"EnableForInternet"};

char cic_varval[DS_WANCIC_VARCOUNT][DS_MAX_VAL_LEN];
char *cic_varval_def[] = {"Ethernet",  	/* WANAccessType : DSL POTS Cable Ethernet */
			  "10000000",
			  "10000000",
			  "Up",	/* PhysicalLinkStatus : Up Down Initializing Unavailable */
			  "WANAccessProvider",
			  "8",		
			  "0",
			  "ActiveConnectionDeviceContainer",
			  "ActiveConnectionServiceID",
			  "100",		/*TotalBytesSent*/
			  "100",          /*TotalBytesReceived*/
			  "100",
			  "100",
			  "0",
			  "0",
			  "1"};


/* arrays for storing WANIPConnction's variable names , values, and defaults */
char *ipconn_varname[] = {"ConnectionType",
			  "PossibleConnectionTypes",
			  "ConnectionStatus",
			  "Uptime",
			  "RSIPAvailable",
			  "NATEnabled",
			  "X_Name",
			  "LastConnectionError",			  
			  "ExternalIPAddress",
			  "RemoteHost",
			  "ExternalPort",
			  "InternalPort",			  			  
			  "PortMappingProtocol",			  
			  "InternalClient",
			  "PortMappingDescription",
			  "PortMappingEnabled",
			  "PortMappingLeaseDuration",
			  "X_PortMappingIndex",
			  "PortMappingNumberOfEntries"};
			  

char ipconn_varval[DS_WANIPConn_VARCOUNT][DS_MAX_VAL_LEN];
char *ipconn_varval_def[] = {"IP_Routed",
			     "IP_Routed",
			     "Connected",
			     "0",		/* uptime*/
			     "0",		/* RSIPAvailable */
			     "1", 		/* nat enabled */
			     "Local Area Connection",
			     "ERROR_NONE",
			     "0.0.0.0",
			     "0.0.0.0",
			     "110", 
			     "110",
			     "TCP",
			     "0.0.0.0",
			     "POP3",
			     "1",
			     "0",
			     "0",
			     "1"};

/* Global structure for storing the state table for this device */
struct IGDService ds_service_table[3];


/* Mutex for protecting the global state table data
   in a multi-threaded, asynchronous environment.
   All functions should lock this mutex before reading
   or writing the state table data. */
pthread_mutex_t DSDevMutex= PTHREAD_MUTEX_INITIALIZER;




/********************************************************************************
 * IGDDeviceStateTableInit
 *
 * Description: 
 *       Initialize the device state table for 
 * 	 this IGDDevice, pulling identifier info
 *       from the description Document.  Note that 
 *       knowledge of the service description is
 *       assumed.  State table variables and default
 *       values are currently hard coded in this file
 *       rather than being read from service description
 *       documents.
 *
 * Parameters:
 *   DescDocName -- The description document name 
 *   DescDocURL -- The description document URL
 *   
 ********************************************************************************/
int IGDDeviceStateTableInit(char* DescDocURL, char* DescDocName)
{
    IXML_Document *DescDoc=NULL;
    int ret = UPNP_E_SUCCESS;
    char *servid_osi=NULL, *evnturl_osi=NULL, *ctrlurl_osi=NULL;
    char *servid_cic=NULL, *evnturl_cic=NULL, *ctrlurl_cic=NULL;    
    char *servid_ipconn=NULL, *evnturl_ipconn=NULL, *ctrlurl_ipconn=NULL;    
    char *udn=NULL;
    char *udn2=NULL;
    char *udn3=NULL;    
    char fullurl[256];
    int i;

    sprintf(fullurl,"%s/%s",DescDocURL,DescDocName);
	printf("fullurl %s \n", fullurl);

    if (UpnpDownloadXmlDoc(fullurl, &DescDoc) != UPNP_E_SUCCESS) {
//    if (UpnpDownloadXmlDoc(DescDocURL, &DescDoc) != UPNP_E_SUCCESS) {
	   
	printf("DeviceStateTableInit -- Error Parsing %s\n", DescDocURL);
	ret = UPNP_E_INVALID_DESC;
    }

#if 1 
    /* Find the OSInfo Service identifiers */
    if (!SampleUtil_FindAndParseService(DescDoc, DescDocURL, IGDServiceType[DS_SERVICE_OSInfo], &servid_osi, &evnturl_osi, &ctrlurl_osi)) {
	printf("IGDDeviceStateTableInit -- Error: Could not find Service: %s\n", IGDServiceType[DS_SERVICE_OSInfo]);

	if (servid_osi) free(servid_osi);
	if (evnturl_osi) free(evnturl_osi);
	if (ctrlurl_osi) free(ctrlurl_osi);

	return(UPNP_E_INVALID_DESC);
    }
    /* Find the Common Interface Config Service identifiers */
    if (!SampleUtil_FindAndParseService(DescDoc, DescDocURL, IGDServiceType[DS_SERVICE_WANCommonInterfaceConfig], &servid_cic, &evnturl_cic, &ctrlurl_cic)) {
	printf("IGDDeviceStateTableInit -- Error: Could not find Service: %s\n", IGDServiceType[DS_SERVICE_WANCommonInterfaceConfig]);

	if (servid_cic) free(servid_cic);
	if (evnturl_cic) free(evnturl_cic);
	if (ctrlurl_cic) free(ctrlurl_cic);

	return(UPNP_E_INVALID_DESC);
    }    

    /* Find the IP Connection identifiers */
    if (!SampleUtil_FindAndParseService(DescDoc, DescDocURL, IGDServiceType[DS_SERVICE_WANIPConnection], &servid_ipconn, &evnturl_ipconn, &ctrlurl_ipconn)) {
	printf("IGDDeviceStateTableInit -- Error: Could not find Service: %s\n", IGDServiceType[DS_SERVICE_WANIPConnection]);

	if (servid_ipconn) free(servid_ipconn);
	if (evnturl_ipconn) free(evnturl_ipconn);
	if (ctrlurl_ipconn) free(ctrlurl_ipconn);

	return(UPNP_E_INVALID_DESC);
    }    

    udn = SampleUtil_GetDocumentItem(DescDoc, "UDN" ,0);
    udn2 = SampleUtil_GetDocumentItem(DescDoc, "UDN" ,1);
    udn3 = SampleUtil_GetDocumentItem(DescDoc, "UDN" ,2);    
        

    strcpy(ds_service_table[DS_SERVICE_OSInfo].UDN, udn);
    strcpy(ds_service_table[DS_SERVICE_OSInfo].ServiceId, servid_osi);
    strcpy(ds_service_table[DS_SERVICE_OSInfo].ServiceType, IGDServiceType[DS_SERVICE_OSInfo]);
    ds_service_table[DS_SERVICE_OSInfo].VariableCount=DS_OSI_VARCOUNT;

    strcpy(ds_service_table[DS_SERVICE_WANCommonInterfaceConfig].UDN, udn2);
    strcpy(ds_service_table[DS_SERVICE_WANCommonInterfaceConfig].ServiceId, servid_cic);
    strcpy(ds_service_table[DS_SERVICE_WANCommonInterfaceConfig].ServiceType, IGDServiceType[DS_SERVICE_WANCommonInterfaceConfig]);
    ds_service_table[DS_SERVICE_WANCommonInterfaceConfig].VariableCount=DS_WANCIC_VARCOUNT;
    for (i=0; i<ds_service_table[DS_SERVICE_WANCommonInterfaceConfig].VariableCount; i++) {
	ds_service_table[DS_SERVICE_WANCommonInterfaceConfig].VariableName[i] = cic_varname[i];
	ds_service_table[DS_SERVICE_WANCommonInterfaceConfig].VariableStrVal[i] = cic_varval[i];
	strcpy(ds_service_table[DS_SERVICE_WANCommonInterfaceConfig].VariableStrVal[i], cic_varval_def[i]);
   }


    strcpy(ds_service_table[DS_SERVICE_WANIPConnection].UDN, udn3);
    strcpy(ds_service_table[DS_SERVICE_WANIPConnection].ServiceId, servid_ipconn);
    strcpy(ds_service_table[DS_SERVICE_WANIPConnection].ServiceType, IGDServiceType[DS_SERVICE_WANIPConnection]);
    ds_service_table[DS_SERVICE_WANIPConnection].VariableCount=DS_WANIPConn_VARCOUNT;
    for (i=0; i<ds_service_table[DS_SERVICE_WANIPConnection].VariableCount; i++) {
	ds_service_table[DS_SERVICE_WANIPConnection].VariableName[i] = ipconn_varname[i];
	ds_service_table[DS_SERVICE_WANIPConnection].VariableStrVal[i] = ipconn_varval[i];
	strcpy(ds_service_table[DS_SERVICE_WANIPConnection].VariableStrVal[i], ipconn_varval_def[i]);
    }

    if (udn) free(udn);
    if (udn2) free(udn2);    
    if (udn3) free(udn3);    
    
    if (servid_osi) free(servid_osi);
    if (evnturl_osi) free(evnturl_osi);
    if (ctrlurl_osi) free(ctrlurl_osi);

    if (servid_cic) free(servid_cic);
    if (evnturl_cic) free(evnturl_cic);
    if (ctrlurl_cic) free(ctrlurl_cic);
    
    if (servid_ipconn) free(servid_ipconn);
    if (evnturl_ipconn) free(evnturl_ipconn);
    if (ctrlurl_ipconn) free(ctrlurl_ipconn);

#endif

    return(ret);
}



/********************************************************************************
 * IGDDeviceHandleSubscriptionRequest
 *
 * Description: 
 *       Called during a subscription request callback.  If the
 *       subscription request is for this device and either its
 *       control service or picture service, then accept it.
 *
 * Parameters:
 *   sr_event -- The subscription request event structure
 *
 ********************************************************************************/
int IGDDeviceHandleSubscriptionRequest(struct Upnp_Subscription_Request *sr_event) 
{
    int i;
    IXML_Document *PropSet;

    pthread_mutex_lock(&DSDevMutex);
  
    for (i=0; i<DS_SERVICE_SERVCOUNT; i++) {
	if ((strcmp(sr_event->UDN,ds_service_table[i].UDN) == 0) &&
	    (strcmp(sr_event->ServiceId,ds_service_table[i].ServiceId) == 0)) {
	switch(i){
	case  DS_SERVICE_WANCommonInterfaceConfig:
            //Another way of doing the same thing.
            PropSet = NULL;
            UpnpAddToPropertySet(&PropSet, ds_service_table[1].VariableName[3],ds_service_table[1].VariableStrVal[3]);
            UpnpAddToPropertySet(&PropSet, ds_service_table[1].VariableName[13],ds_service_table[1].VariableStrVal[13]);
            UpnpAcceptSubscriptionExt(gate.device_handle, sr_event->UDN, sr_event->ServiceId,PropSet,sr_event->Sid);
            ixmlDocument_free(PropSet);
	    break;
	case DS_SERVICE_WANIPConnection:
	    //Another way of doing the same thing.
            PropSet = NULL;
            UpnpAddToPropertySet(&PropSet, ds_service_table[2].VariableName[1],ds_service_table[2].VariableStrVal[1]);
            UpnpAddToPropertySet(&PropSet, ds_service_table[2].VariableName[2],ds_service_table[2].VariableStrVal[2]);
            UpnpAddToPropertySet(&PropSet, ds_service_table[2].VariableName[8],ds_service_table[2].VariableStrVal[8]);
            UpnpAddToPropertySet(&PropSet, ds_service_table[2].VariableName[18],ds_service_table[2].VariableStrVal[18]);
            
            UpnpAcceptSubscriptionExt(gate.device_handle, sr_event->UDN, sr_event->ServiceId,PropSet,sr_event->Sid);
            ixmlDocument_free(PropSet);
	    break;	
		}
	}
    }

    pthread_mutex_unlock(&DSDevMutex);	

    return(1);
}





/********************************************************************************
 * IGDDeviceHandleGetVarRequest
 *
 * Description: 
 *       Called during a get variable request callback.  
 *	 Check if the request is for this device and its services.
 *	 Check current status and save them to status table
 *	 Responds with the variable value
 *
 * Parameters:
 *   cgv_event -- The control get variable request event structure
 *
 ********************************************************************************/
int IGDDeviceHandleGetVarRequest(struct Upnp_State_Var_Request *cgv_event) 
{
    int i, j;
    int gedsar_succeeded = 0;
    cgv_event->CurrentVal = NULL;
  
    pthread_mutex_lock(&DSDevMutex);
  
    for (i=0; i<DS_SERVICE_SERVCOUNT; i++) {
	if ((strcmp(cgv_event->DevUDN,ds_service_table[i].UDN)==0) &&
	    (strcmp(cgv_event->ServiceID,ds_service_table[i].ServiceId)==0)) {
	    /* Request for variable in the IGDDevice Service */
	    for (j=0; j< ds_service_table[i].VariableCount; j++) {
		if (strcmp(cgv_event->StateVarName, ds_service_table[i].VariableName[j])==0) {  
		    
			//check which StateVarName control point requesting 
		    gedsar_succeeded = 1;
		        //find it ,then return success
		    
//		    cgv_event->CurrentVal = (Upnp_DOMString) malloc(sizeof(ds_service_table[i].VariableStrVal[j]));
//		    strcpy(cgv_event->CurrentVal, ds_service_table[i].VariableStrVal[j]);
		    cgv_event->CurrentVal = ixmlCloneDOMString (ds_service_table[i].VariableStrVal[j]);
	    
		        //set value stored in service_table to cgv_event->CurrentVal 
	
		    break;
		}
	    } //end of for loop 
	}
    } //end of for loop
  
    if (gedsar_succeeded) {
	cgv_event->ErrCode = UPNP_E_SUCCESS;
    } else {
	printf("Error in UPNP_CONTROL_GET_VAR_REQUEST callback:\n"); 
	printf("   Unknown variable name = %s\n",  cgv_event->StateVarName); 
	cgv_event->ErrCode = 404;
	strcpy(cgv_event->ErrStr, "Invalid Variable");
    }

    pthread_mutex_unlock(&DSDevMutex);
  
    return(cgv_event->ErrCode == UPNP_E_SUCCESS);
}


/*******************************************************************************
 * IGDDeviceHandleActionRequest
 *
 * Description: 
 *       Called during an action request callback. 
 *	 Check which service is requested.
 *	 Perform the action :
 * 			 set/acquire status table, system calls, set/acquire 
 * 			 portmapping table
 *	 Respond.
 *
 * Parameters:
 *   ca_event -- The control action request event structure
 *
 ******************************************************************************/
int IGDDeviceHandleActionRequest(struct Upnp_Action_Request *ca_event) 
{
    char result_str[500];
   
    /* Defaults if action not found */
    int action_succeeded = -1;
    int err=401;

    ca_event->ErrCode = 0;
    ca_event->ActionResult = NULL;

/********************************
 * WAN Common Interface Config  *
 ********************************/
if ((strcmp(ca_event->DevUDN,ds_service_table[DS_SERVICE_WANCommonInterfaceConfig].UDN)==0) &&
	       (strcmp(ca_event->ServiceID,ds_service_table[DS_SERVICE_WANCommonInterfaceConfig].ServiceId)==0)) {
 
	/* Request for action in the DsDevice WANCommonInterfaceConfig Service  */


	if (strcmp(ca_event->ActionName, "X_GetICSStatistics") == 0) {

		sprintf(result_str, 
			"<u:X_GetICSStatisticsResponse xmlns:u=\"%s\"><TotalBytesSent xmlns:dt=\"urn:schema-microsoft-com:datatypes\" dt:dt=\"ui4\">666</TotalBytesSent><TotalBytesReceived xmlns:dt=\"urn:schema-microsoft-com:datatypes\" dt:dt=\"ui4\">666</TotalBytesReceived><TotalPacketsSent xmlns:dt=\"urn:schema-microsoft-com:datatypes\" dt:dt=\"ui4\">666</TotalPacketsSent><TotalPacketsReceived xmlns:dt=\"urn:schema-microsoft-com:datatypes\" dt:dt=\"ui4\">666</TotalPacketsReceived><Layer1DownstreamMaxBitRate xmlns:dt=\"urn:schema-microsoft-com:datatypes\" dt:dt=\"ui4\">100000000</Layer1DownstreamMaxBitRate><Uptime xmlns:dt=\"urn:schema-microsoft-com:datatypes\" dt:dt=\"ui4\">500</Uptime></u:X_GetICSStatisticsResponse>",
			ca_event->ServiceID);

		ca_event->ActionResult = ixmlParseBuffer(result_str);

		action_succeeded = 1;

	/* GetTotalBytesSent Action */			
	}else if (strcmp(ca_event->ActionName, "GetTotalBytesSent") == 0) {
	
		sprintf(result_str, 
			"<u:%sResponse xmlns:u=\"%s\"><NewTotalBytesSent>%lu</NewTotalBytesSent></u:%sResponse>", 
			ca_event->ActionName,
			ca_event->ServiceID,
			getTotalBytesSent(),
			ca_event->ActionName);
	
		ca_event->ActionResult = ixmlParseBuffer(result_str);
	
		action_succeeded = 1;

	/* GetTotalBytesReceived Action */
	}else if (strcmp(ca_event->ActionName, "GetTotalBytesReceived") == 0) {
	sprintf(result_str, 
			"<u:%sResponse xmlns:u=\"%s\"><NewTotalBytesReceived>%lu</NewTotalBytesReceived></u:%sResponse>", 
			ca_event->ActionName,
			ca_event->ServiceID,
 			getTotalBytesReceived(),
			ca_event->ActionName);
		ca_event->ActionResult = ixmlParseBuffer(result_str);
		action_succeeded = 1;
	
	/* GetTotalPacketsSent Action */
	}else if (strcmp(ca_event->ActionName, "GetTotalPacketsSent") == 0) {
	
		sprintf(result_str, 
			"<u:%sResponse xmlns:u=\"%s\"><NewTotalPacketsSent>%lu</NewTotalPacketsSent></u:%sResponse>", 
			ca_event->ActionName,
			ca_event->ServiceID,
			getTotalPacketsSent(),
			ca_event->ActionName);
		ca_event->ActionResult = ixmlParseBuffer(result_str);
		action_succeeded = 1;

	/* GetTotalPacketsReceived Action */
	}else if (strcmp(ca_event->ActionName, "GetTotalPacketsReceived") == 0) {
	
		sprintf(result_str, 
			"<u:%sResponse xmlns:u=\"%s\"><NewTotalPacketsReceived>%lu</NewTotalPacketsReceived></u:%sResponse>", 
			ca_event->ActionName,
			ca_event->ServiceID,
			getTotalPacketsReceived(),
			ca_event->ActionName);
		ca_event->ActionResult = ixmlParseBuffer(result_str);
		action_succeeded = 1;

	/* GetCommonLinkProperties Action */
	}else if (strcmp(ca_event->ActionName, "GetCommonLinkProperties") == 0) {

		sprintf(result_str, 
			"<u:%sResponse xmlns:u=\"%s\"><NewWANAccessType>DSL</NewWANAccessType><NewLayer1UpstreamMaxBitRate>%d</NewLayer1UpstreamMaxBitRate><NewLayer1DownstreamMaxBitRate>%d</NewLayer1DownstreamMaxBitRate><NewPhysicalLinkStatus>Up</NewPhysicalLinkStatus></u:%sResponse>",
			ca_event->ActionName,
			ca_event->ServiceID,
			gate.UpstreamSpeed*1000,
			gate.DownstreamSpeed*1000,	
			ca_event->ActionName);
	
		ca_event->ActionResult = ixmlParseBuffer(result_str);
		action_succeeded = 1;
	}


/********************************
 *	 WAN IP Connection      *
 ********************************/

} else if ((strcmp(ca_event->DevUDN,ds_service_table[DS_SERVICE_WANIPConnection].UDN)==0) &&
	       (strcmp(ca_event->ServiceID,ds_service_table[DS_SERVICE_WANIPConnection].ServiceId)==0)) {
    
	/* Request for action in the IGDDevice WAN IP Connection Service  */

	if (strcmp(ca_event->ActionName, "SetConnectionType") == 0) {
	
		sprintf(result_str, "<u:%sResponse xmlns:u=\"%s\"/>", 
		ca_event->ActionName,
		ca_event->ServiceID);
  		ca_event->ActionResult = ixmlParseBuffer(result_str);
		action_succeeded = 1;

	/* GetConnectionTypeInfo Action */
	}else  
	if (strcmp(ca_event->ActionName, "GetConnectionTypeInfo") == 0) {
		sprintf(result_str, 
			"<u:%sResponse xmlns:u=\"%s\"><NewConnectionType>IP_Routed</NewConnectionType><NewPossibleConnectionTypes>IP_Routed</NewPossibleConnectionTypes></u:%sResponse>",
		ca_event->ActionName,
		ca_event->ServiceID,
		ca_event->ActionName);
	     	ca_event->ActionResult = ixmlParseBuffer(result_str);
        	action_succeeded = 1;
	/* RequestConnection Action */
	}else if (strcmp(ca_event->ActionName, "RequestConnection") == 0) {
		sprintf(result_str, "<u:%sResponse xmlns:u=\"%s\"/>", 
			ca_event->ActionName,
			ca_event->ServiceID);
	  	ca_event->ActionResult = ixmlParseBuffer(result_str);
		action_succeeded = IGDDeviceRequestConnection();	
	/* ForceTermination */
	}else if (strcmp(ca_event->ActionName, "ForceTermination") == 0) {
		sprintf(result_str, "<u:%sResponse xmlns:u=\"%s\"/>",
			ca_event->ActionName,
			ca_event->ServiceID);
	  	ca_event->ActionResult = ixmlParseBuffer(result_str);
		action_succeeded = IGDDeviceForceTermination();
	/* GetStatusInfo Action */		
	}else if (strcmp(ca_event->ActionName, "GetStatusInfo") == 0) {
		// update Gate.connectionStatus
		getConnectionStatus();
		sprintf(result_str, 
			"<u:GetStatusInfoResponse xmlns:u=\"%s\"><NewConnectionStatus>%s</NewConnectionStatus><NewLastConnectionError>ERROR_NONE</NewLastConnectionError><NewUptime>%d</NewUptime></u:GetStatusInfoResponse>",
			ca_event->ServiceID,
			gate.ConnectionStatus,
			getUptime());
		ca_event->ActionResult = ixmlParseBuffer(result_str);
		action_succeeded = 1; 	    
	/* GetNATRSIPStatus Action */
	}else if (strcmp(ca_event->ActionName, "GetNATRSIPStatus") == 0) {
		sprintf(result_str, 
			"<u:%sResponse xmlns:u=\"%s\"><NewRSIPAvailable>0</NewRSIPAvailable><NewNATEnabled>%d</NewNATEnabled></u:%sResponse>",
			ca_event->ActionName,
			ca_event->ServiceID,
			getNATEnabled(),
			ca_event->ActionName);

		ca_event->ActionResult = ixmlParseBuffer(result_str);
	        action_succeeded = 1;


	/* GetGenericPortMappingEntry Action : get PortMapping according to index */
	}else if (strcmp(ca_event->ActionName, "GetGenericPortMappingEntry") == 0) {
		char *my_pt=SampleUtil_GetDocumentItem(ca_event->ActionRequest
				, "NewPortMappingIndex",0);
		int i=0;
		int len=LinkLength();
//			printf("going to get index %d\n",i);

		
		if(my_pt){
			i=atoi(my_pt);
			free(my_pt);
		}
			
	
	        if(i+1>len)
		{
		        action_succeeded = -4;
		}
		else
		{
		//puts("Searching....in generic\n");							
		DLL tmp;	
		tmp=Search(i);
		if(tmp==NULL)
			{
			//puts("no such index");
			action_succeeded = -2;
			}

		sprintf(result_str, 
			"<u:%sResponse xmlns:u=\"%s\">
			<NewRemoteHost></NewRemoteHost>
			<NewExternalPort>%d</NewExternalPort>
			<NewProtocol>%s</NewProtocol>
			<NewInternalPort>%d</NewInternalPort>
			<NewInternalClient>%s</NewInternalClient>
			<NewEnabled>%d</NewEnabled>
			<NewPortMappingDescription>%s</NewPortMappingDescription>
			<NewLeaseDuration>0</NewLeaseDuration>
			</u:%sResponse>",
			ca_event->ActionName,
			ca_event->ServiceID,
			tmp->data.ExternalPort,
			tmp->data.Protocol,
			tmp->data.InternalPort,
			tmp->data.InternalClient,
			tmp->data.PortMappingEnabled,
			tmp->data.PortMappingDescription,
			ca_event->ActionName);

		ca_event->ActionResult = ixmlParseBuffer(result_str);
		
	        action_succeeded = 1;	    
		}
	} else if (strcmp(ca_event->ActionName, "GetSpecificPortMappingEntry") == 0) {
		char *my_pt=SampleUtil_GetDocumentItem(ca_event->ActionRequest, "NewExternalPort",0);
		int ext_port=atoi(my_pt);
		char *protocol=SampleUtil_GetDocumentItem(ca_event->ActionRequest, "NewProtocol",0);
		int i=0,len=LinkLength();
		int flag=0;
		DLL tmp;
		
		if(my_pt)
			free(my_pt);
			
		for(i=0;i<len;i++)
		{

//		puts("Searching....in specific\n");				
		tmp=Search(i);


			if((tmp->data.ExternalPort==ext_port)&&(strcmp(tmp->data.Protocol, protocol)==0)){
		sprintf(result_str, 
			"<u:%sResponse xmlns:u=\"%s\"><NewInternalPort>%d</NewInternalPort><NewInternalClient>%s</NewInternalClient><NewEnabled>%d</NewEnabled><NewPortMappingDescription>%s</NewPortMappingDescription><NewLeaseDuration>0</NewLeaseDuration></u:%sResponse>",
			ca_event->ActionName,
			ca_event->ServiceID,
			tmp->data.InternalPort,
			tmp->data.InternalClient,	
			tmp->data.PortMappingEnabled,	
			tmp->data.PortMappingDescription,	
			ca_event->ActionName);	
		ca_event->ActionResult = ixmlParseBuffer(result_str);	
			flag=1;
		} //end of if 
	} //end of for

	if(flag==1)
		action_succeeded = 1;	
	else
		action_succeeded = -3;
	
	if(protocol)
		free(protocol);

	} else if (strcmp(ca_event->ActionName, "AddPortMapping") == 0) {
		if(gate.permission==ACCESS_DENY)
		{
			action_succeeded = -1;
		}
		else
		{
		DLL tmp,node;
		//int i,len=LinkLength();
		int i;
		char *my_pt=NULL;
		tmp=AllocNode();
		my_pt=SampleUtil_GetDocumentItem(ca_event->ActionRequest, "NewExternalPort",0);
		tmp->data.ExternalPort=atoi(my_pt);
		if(my_pt) free(my_pt);
		tmp->data.Protocol=SampleUtil_GetDocumentItem(ca_event->ActionRequest, "NewProtocol",0);
		my_pt=SampleUtil_GetDocumentItem(ca_event->ActionRequest, "NewInternalPort",0);
		tmp->data.InternalPort=atoi(my_pt);
		if(my_pt) free(my_pt);
		
		tmp->data.InternalClient=SampleUtil_GetDocumentItem(ca_event->ActionRequest, "NewInternalClient",0);
		my_pt=SampleUtil_GetDocumentItem(ca_event->ActionRequest, "NewEnabled",0);
		tmp->data.PortMappingEnabled=atoi(my_pt);
	       if(my_pt) free(my_pt);
       	       
		tmp->data.PortMappingDescription=SampleUtil_GetDocumentItem(ca_event->ActionRequest, "NewPortMappingDescription",0); 

		action_succeeded = 1;
		
		// Search if the To-Be-Add Portmapping already exist, if do, remove it first 
		for(i=0;i<LinkLength();i++)
		{
			node=Search(i);
			if((node->data.ExternalPort==tmp->data.ExternalPort)&&(strcmp(node->data.Protocol, tmp->data.Protocol)==0))
			{
				/* Ron add for 2 Client want to add the same port XBOX Live */
				if(tmp->data.InternalClient && node->data.InternalClient && strcmp(tmp->data.InternalClient,node->data.InternalClient)!=0 
						&& tmp->data.PortMappingEnabled==1 
						&& node->data.PortMappingEnabled==1){
					action_succeeded = -1;
					break;
				}

				//puts("Found an exist port mapping");
		       		//if(tmp->data.PortMappingEnabled!=0)
		       		delPortForward(tmp->data.Protocol,
       				//gate.ExternalIPAddress,
       				getIPAddress(gate.ext_ifname)?:"0.0.0.0",
       				node->data.ExternalPort,
       				node->data.InternalClient,
       				node->data.InternalPort);	
				Delete(node);
//				print_node(tmp);
				print_link();
				break;
			}
		}
       		if(action_succeeded == 1){
       			// do really work 
       		if(tmp->data.PortMappingEnabled==1)
       		addPortForward(tmp->data.Protocol,
       				getIPAddress(gate.ext_ifname)?:"0.0.0.0",
       				//gate.ExternalIPAddress,
       				tmp->data.ExternalPort,
       				tmp->data.InternalClient,
       				tmp->data.InternalPort);
//		print_node(tmp);	
	
			// Append data to Linked List 
	       		Append(tmp);
			print_link();	       	

			{
				int entries=0;
				char value[128];
				entries = atoi(ds_service_table[2].VariableStrVal[18]);
				sprintf(value, "%d", entries+1);
				IGDDeviceSetServiceTableVar(DS_SERVICE_WANIPConnection, DS_WANIPConn_PortMappingNumberOfEntries, value);
			}
		/* modify according to package we get from certification record */
		sprintf(result_str, "<u:%sResponse xmlns:u=\"%s\"/>", 	
			ca_event->ActionName,
			ca_event->ServiceID);
 	
  		ca_event->ActionResult = ixmlParseBuffer(result_str);
		}
	
		}		
	} 
	else if (strcmp(ca_event->ActionName, "DeletePortMapping") == 0) {
	if(gate.permission==ACCESS_DENY)
	{
		action_succeeded = -1;
	}
	else
	{
		int i;
		DLL tmp;
		char *my_pt=SampleUtil_GetDocumentItem(ca_event->ActionRequest, "NewExternalPort",0);
		
		int ext_port=atoi(my_pt);
		char *protocol=SampleUtil_GetDocumentItem(ca_event->ActionRequest, "NewProtocol",0);

		action_succeeded = 1;
		
		if(my_pt) free(my_pt);
		
		my_pt=SampleUtil_GetDocumentItem(ca_event->ActionRequest, "NewInternalClient",0);
		
		// Search if the To-Be-Delete Portmapping already exist,
		for(i=0;i<LinkLength();i++)
		{
	
			tmp=Search(i);
			
			if((ext_port==tmp->data.ExternalPort)&&(strcmp(protocol, tmp->data.Protocol)==0))
			{
				/* Ron add for 2 Client want to add the same port XBOX Live */
				if(tmp->data.InternalClient 
						&& my_pt 
						&& strcmp(tmp->data.InternalClient,my_pt)!=0 
						&& tmp->data.PortMappingEnabled==1){
					action_succeeded = -1;
					break;
				}

				//puts("Found an exist port mapping");
				/* Delete Port Mapping before free this node */
		       		//if(tmp->data.PortMappingEnabled!=0)		
		       		delPortForward(tmp->data.Protocol,
       				getIPAddress(gate.ext_ifname)?:"0.0.0.0",
       				//gate.ExternalIPAddress,
       				tmp->data.ExternalPort,
       				tmp->data.InternalClient,
       				tmp->data.InternalPort);		

				/* update Linked List */
				Delete(tmp);
				/* update upnptab */
				print_link();				
				break;
			}
	
		  }
		  if(action_succeeded == 1){
			sprintf(result_str, "<u:%sResponse xmlns:u=\"%s\"/>", 
				ca_event->ActionName,
				ca_event->ServiceID);
			ca_event->ActionResult = ixmlParseBuffer(result_str);	
		  }
		
		if(my_pt) free(my_pt);
		if(protocol) free(protocol);
		}//end of permission granted
	} else if (strcmp(ca_event->ActionName, "GetExternalIPAddress") == 0) {
		
		sprintf(result_str, 
		"<u:%sResponse xmlns:u=\"%s\"><NewExternalIPAddress>%s</NewExternalIPAddress></u:%sResponse>",
		ca_event->ActionName,
		ca_event->ServiceID,
       		getIPAddress(gate.ext_ifname)?:"0.0.0.0",
		ca_event->ActionName);

		ca_event->ActionResult = ixmlParseBuffer(result_str);

	        action_succeeded = 1;
	}
}
	/*  ErrCode assignment */
	if (action_succeeded > 0) {
		ca_event->ErrCode = UPNP_E_SUCCESS;
	}
	else
	if (action_succeeded == -1) {
		ca_event->ErrCode = err;
		strcpy(ca_event->ErrStr, "Invalid Action");
		ca_event->ActionResult = NULL;
	}
	else
	if (action_succeeded == -2) {
		ca_event->ErrCode = 713;
		strcpy(ca_event->ErrStr, "SpecifiedArrayIndexInvalid");
		ca_event->ActionResult = NULL;
	}
	else
	if (action_succeeded == -3) {
		ca_event->ErrCode = 714;
		strcpy(ca_event->ErrStr, "NoSuchEntryInArray");
		ca_event->ActionResult = NULL;
	}
	else
	{
		//printf("Error in UPNP_CONTROL_ACTION_REQUEST callback:\n");
		//printf("   Failure while running %s\n",  ca_event->ActionName);
		ca_event->ErrCode = 402;
		strcpy(ca_event->ErrStr, "Invalid Args");
		ca_event->ActionResult = NULL;
	}
	return(ca_event->ErrCode);
}




/********************************************************************************
 * IGDDeviceSetServiceTableVar
 *
 * Description: 
 *       Update the IGDDevice service state table, and notify all subscribed 
 *       control points of the updated state.  Note that since this function
 *       blocks on the mutex DSDevMutex, to avoid a hang this function should 
 *       not be called within any other function that currently has this mutex 
 *       locked.
 *
 * Parameters:
 *   service -- The service number (DS_SERVICE_Layer3Forwarding or DS_SERVICE_PICTURE)
 *   variable -- The variable number (DS_CONTROL_POWER, DS_CONTROL_CHANNEL,
 *                   DS_CONTROL_VOLUME, DS_PICTURE_COLOR, DS_PICTURE_TINT,
 *                   DS_PICTURE_CONTRAST, or DS_PICTURE_BRIGHTNESS)
 *   value -- The string representation of the new value
 *
 ********************************************************************************/
int IGDDeviceSetServiceTableVar(unsigned int service, unsigned int variable, char *value) {
 IXML_Document  *PropSet= NULL;
    if ((service >= DS_SERVICE_SERVCOUNT) || (variable >= ds_service_table[service].VariableCount) || (strlen(value) >= DS_MAX_VAL_LEN)) {
	return(0);
    }

    pthread_mutex_lock(&DSDevMutex);
    
    strcpy(ds_service_table[service].VariableStrVal[variable], value);

    //Using second utility API
    PropSet= UpnpCreatePropertySet(1,ds_service_table[service].VariableName[variable], ds_service_table[service].VariableStrVal[variable]);
    UpnpNotifyExt(gate.device_handle, ds_service_table[service].UDN, ds_service_table[service].ServiceId,PropSet);
    ixmlDocument_free(PropSet);

    pthread_mutex_unlock(&DSDevMutex);

    return(1);

}



/********************************************************************************
 * IGDDeviceCallbackEventHandler
 *
 * Description: 
 *       The callback handler registered with the SDK while registering
 *       root device or sending a search request.  Detects the type of 
 *       callback, and passes the request on to the appropriate procedure.
 *
 * Parameters:
 *   EventType -- The type of callback event
 *   Event -- Data structure containing event data
 *   Cookie -- Optional data specified during callback registration
 *
 ********************************************************************************/
int IGDDeviceCallbackEventHandler(Upnp_EventType EventType, 
			 void *Event, 
			 void *Cookie)
{
    /* Print a summary of the event received */
    SampleUtil_PrintEvent(EventType, Event);
  
    switch ( EventType) {

    case UPNP_EVENT_SUBSCRIPTION_REQUEST:
	IGDDeviceHandleSubscriptionRequest((struct Upnp_Subscription_Request *) Event);
	break;

    case UPNP_CONTROL_GET_VAR_REQUEST:
	IGDDeviceHandleGetVarRequest((struct Upnp_State_Var_Request *) Event);
	break;

    case UPNP_CONTROL_ACTION_REQUEST:
	IGDDeviceHandleActionRequest((struct Upnp_Action_Request *) Event);
	break;

    default:
	   printf("Error in IGDDeviceCallbackEventHandler: unknown event type %d\n", EventType);
    }

    return(0);
}


int IGDconfigure(void)
{
	
	FILE *fp=NULL;
	char line[10000];
	char *cp;
	char *cp2;
	char *name;
	char *value;
	int enable;
	int port;
	char ip[16]="";
	char proto[6]="";
	
	DLL tmp=NULL;

	fp=fopen("/tmp/upnp.conf","r");

	if( fp == NULL )
		return -1;

	while( fgets(line,sizeof(line),fp) !=  NULL)
	{
		/* Trim comment */
		if (( cp = strchr ( line, '#' )) != NULL)
			*cp = '\0';
		/* Split line into words. */
		for ( cp = line; *cp != '\0'; cp = cp2 )
		{
			/* Skip leading whitespace. */
			cp += strspn( cp, " \t\n\r" );
			/* Find next whitespace. */
			cp2 = cp + strcspn( cp, " \t\n\r" );
			/* Insert EOS and advance next-word pointer. */
			while ( *cp2 == ' ' || *cp2 == '\t' || *cp2 == '\n' || *cp2 == '\r' )
			*cp2++ = '\0';
			/* Split into name and value. */
			name = cp;
			value = strchr( name, '=' );
			
			if ( value != (char*) 0 )
				*value++ = '\0';
			/* Interpret. */
	    		if ( strcasecmp( name, "extif" ) == 0 )
			{
				gate.ext_ifname = strdup(value);
			}
			else if ( strcasecmp( name, "intif" ) == 0 )
			{
				gate.int_ifname = strdup(value);
    				strcpy(gate.InternalIPAddress,getIPAddress(gate.int_ifname));
			}
			else if ( strcasecmp( name, "advr_expire" ) == 0 )
			{
				gate.advr_expire=atoi(value)*60*2;
			}
			else if ( strcasecmp( name, "advr_ttl" ) == 0 )
			{	
				gate.advr_ttl=atoi(value);
			}
			else if ( strcasecmp( name, "permission" ) == 0 )
			{
				if(strcmp("enable",value)==0)
					gate.permission=ACCESS_ALLOW;
				else if (strcmp("disable",value)==0)
					gate.permission=ACCESS_DENY;
			}
			else  // Predefine service
			{
				tmp=AllocNode();
				sscanf(value,"%[^:]:%d:%[^:]:%d",proto,&port,ip,&enable);
				tmp->data.PortMappingEnabled=enable;
				tmp->data.InternalPort=port;
				tmp->data.ExternalPort=port;
				
				if(strcmp(ip,"0.0.0.0")==0)
       					tmp->data.InternalClient=strdup(gate.InternalIPAddress);
				else
					tmp->data.InternalClient=strdup(ip);
				
				if(strcasecmp(proto,"tcp")==0)
					tmp->data.Protocol=strdup("TCP");
				else
					tmp->data.Protocol=strdup("UDP");
					
				tmp->data.PortMappingDescription=strdup(name);
				
				if(tmp->data.PortMappingEnabled==1){
					addPortForward(tmp->data.Protocol
							,getIPAddress(gate.ext_ifname)?:"0.0.0.0"
							,tmp->data.ExternalPort
							,tmp->data.InternalClient
							,tmp->data.InternalPort);
				}
				
				Append(tmp);
			}	

		} //end of for loop
	}//end of while loop

	(void) fclose(fp);
	print_link();

	return (0);
}
int IGDdelAllPortForwarding(void)
{

	DLL tmp;
	int i,len=LinkLength();
		
	//remove all Portmapping entry exist in both netfilter table and
        //linked list
	for(i=len-1;i>=0;i--)
	{
		tmp=Search(i);
		if(tmp->data.PortMappingEnabled)
			delPortForward(tmp->data.Protocol,
					getIPAddress(gate.ext_ifname)?:"0.0.0.0",
	       				tmp->data.ExternalPort,
	       				tmp->data.InternalClient,
	       				tmp->data.InternalPort);	
       				
		Delete(tmp);
		print_link();
	}
       	

   return (0);
}
