#include <sys/types.h>
#include <string.h>
#include <malloc.h>

#include "include/snmp_lib.h"
#include "include/snmp_asn1.h"
#include "include/snmp_api.h"

/*
 * Parses the packet received to determine version, either directly
 * from packets version field or inferred from ASN.1 construct.
 */
int snmp_parse_version(u_char * data, size_t length)
{
    u_char          type;
    long            version = SNMPERR_BAD_VERSION;

    data = asn_parse_sequence(data, &length, &type,
                              (ASN_SEQUENCE | ASN_CONSTRUCTOR), "version");
    if (data)
    {
        data =
            asn_parse_int(data, &length, &type, &version, sizeof(version));
        if (!data || type != ASN_INTEGER)
            return SNMPERR_BAD_VERSION;
        
    }
    return version;
}

/*******************************************************************-o-******
 * snmp_comstr_parse
 *
 * Parameters:
 *	*data		(I)   Message.
 *	*length		(I/O) Bytes left in message.
 *	*psid		(O)   Community string.
 *	*slen		(O)   Length of community string.
 *	*version	(O)   Message version.
 *      
 * Returns:
 *	Pointer to the remainder of data.
 *
 *
 * Parse the header of a community string-based message such as that found
 * in SNMPv1 and SNMPv2c.
 */
u_char *snmp_comstr_parse(u_char * data,
                          size_t * length,
                          u_char * psid, size_t * slen, long *version)
{
    u_char          type;
    long            ver;
    size_t          origlen = *slen;

    /*
     * Message is an ASN.1 SEQUENCE.
     */
    data = asn_parse_sequence(data, length, &type,
                              (ASN_SEQUENCE | ASN_CONSTRUCTOR),
                              "auth message");
    if (data == NULL)
        return NULL;
   

    /*
     * First field is the version.
     */
    data = asn_parse_int(data, length, &type, &ver, sizeof(ver));
    *version = ver;
    if (data == NULL)
        return NULL;
    

    /*
     * second field is the community string for SNMPv1 & SNMPv2c 
     */
    data = asn_parse_string(data, length, &type, psid, slen);

    if (data == NULL)
        return NULL;
    
    psid[SNMP_MIN(*slen, origlen - 1)] = '\0';
    return (u_char *) data;

}                               /* end snmp_comstr_parse() */



/*
 * u_char * snmp_parse_var_op(
 * u_char *data              IN - pointer to the start of object
 * oid *var_name             OUT - object id of variable 
 * int *var_name_len         IN/OUT - length of variable name 
 * u_char *var_val_type      OUT - type of variable (int or octet string) (one byte) 
 * int *var_val_len          OUT - length of variable 
 * u_char **var_val          OUT - pointer to ASN1 encoded value of variable 
 * int *listlength          IN/OUT - number of valid bytes left in var_op_list 
 */

u_char *snmp_parse_var_op(u_char * data,
                          oid * var_name,
                          size_t * var_name_len,
                          u_char * var_val_type,
                          size_t * var_val_len,
                          u_char ** var_val, size_t * listlength)
{
    u_char          var_op_type;
    size_t          var_op_len = *listlength;
    u_char         *var_op_start = data;

    data = asn_parse_sequence(data, &var_op_len, &var_op_type,
                              (ASN_SEQUENCE | ASN_CONSTRUCTOR), "var_op");
    if (data == NULL)
    {
        /*
         * msg detail is set 
         */
        return NULL;
    }

    data =
        asn_parse_objid(data, &var_op_len, &var_op_type, var_name,
                        var_name_len);

    if (data == NULL)
        return NULL;
    
    if (var_op_type !=
        (u_char) (ASN_UNIVERSAL | ASN_PRIMITIVE | ASN_OBJECT_ID))
        return NULL;
    *var_val = data;            /* save pointer to this object */
    /*
     * find out what type of object this is 
     */
    data = asn_parse_header(data, &var_op_len, var_val_type);
    if (data == NULL)
        return NULL;
    
    /*
     * XXX no check for type! 
     */
    *var_val_len = var_op_len;
    data += var_op_len;
    *listlength -= (int) (data - var_op_start);
    return data;
}

/*
 * Add object identifier name to SNMP variable.
 * If the name is large, additional memory is allocated.
 * Returns 0 if successful.
 */

int snmp_set_var_objid(netsnmp_variable_list * vp,
                       const oid * objid, size_t name_length)
{
    size_t          len = sizeof(oid) * name_length;

    if (vp->name != vp->name_loc && vp->name != NULL &&
        vp->name_length > (sizeof(vp->name_loc) / sizeof(oid)))
    {
        /*
         * Probably previously-allocated "big storage".  Better free it
         * else memory leaks possible.  
         */
        free(vp->name);
    }

    /*
     * use built-in storage for smaller values 
     */
    if (len <= sizeof(vp->name_loc))
        vp->name = vp->name_loc;
    else
    {
        vp->name = (oid *) malloc(len);
        if (!vp->name)
            return 1;
    }
    if (objid)
        memmove(vp->name, objid, len);
    vp->name_length = name_length;
    return 0;
}

int snmp_realloc(u_char ** buf, size_t * buf_len)
{
    u_char         *new_buf = NULL;
    size_t          new_buf_len = 0;

    if (buf == NULL)
        return 0;
    

    /*
     * The current re-allocation algorithm is to increase the buffer size by
     * whichever is the greater of 256 bytes or the current buffer size, up to
     * a maximum increase of 8192 bytes.  
     */

    if (*buf_len <= 255)
    {
        new_buf_len = *buf_len + 256;
    }
    else if (*buf_len > 255 && *buf_len <= 8191)
    {
        new_buf_len = *buf_len * 2;
    }
    else if (*buf_len > 8191)
    {
        new_buf_len = *buf_len + 8192;
    }

    if (*buf == NULL)
    {
        new_buf = (u_char *) malloc(new_buf_len);
    }
    else
    {
        new_buf = (u_char *) realloc(*buf, new_buf_len);
    }

    if (new_buf != NULL)
    {
        *buf = new_buf;
        *buf_len = new_buf_len;
        return 1;
    }
    else
    {
        return 0;
    }
}

int snmp_realloc_rbuild_var_op(u_char ** pkt, size_t * pkt_len,
                               size_t * offset, int allow_realloc,
                               const oid * var_name, size_t * var_name_len,
                               u_char var_val_type,
                               u_char * var_val, size_t var_val_len)
{
    size_t   start_offset = *offset;
    int      rc = 0;

    /*
     * Encode the value.  
     */

    switch (var_val_type)
    {
        case ASN_INTEGER:
            rc = asn_realloc_rbuild_int(pkt, pkt_len, offset, allow_realloc,
                                        var_val_type, (long *) var_val,
                                        var_val_len);
            break;
        
        case ASN_GAUGE:
        case ASN_COUNTER:
        case ASN_TIMETICKS:
        case ASN_UINTEGER:
            rc = asn_realloc_rbuild_unsigned_int(pkt, pkt_len, offset,
                                                 allow_realloc, var_val_type,
                                                 (u_long *) var_val,
                                                 var_val_len);
            break;

        case ASN_COUNTER64:
            rc = asn_realloc_rbuild_unsigned_int64(pkt, pkt_len, offset,
                                                   allow_realloc, var_val_type,
                                                   (struct counter64 *)
                                                   var_val, var_val_len);
            break;

        case ASN_OCTET_STR:
        case ASN_IPADDRESS:
        case ASN_OPAQUE:
        case ASN_NSAP:
            rc = asn_realloc_rbuild_string(pkt, pkt_len, offset, allow_realloc,
                                           var_val_type, var_val, var_val_len);
            break;
            
        case ASN_OBJECT_ID:
            rc = asn_realloc_rbuild_objid(pkt, pkt_len, offset, allow_realloc,
                                          var_val_type, (oid *) var_val,
                                          var_val_len / sizeof(oid));
            break;
            
        case ASN_NULL:
            rc = asn_realloc_rbuild_null(pkt, pkt_len, offset, allow_realloc,
                                         var_val_type);
            break;
            
        case ASN_BIT_STR:
            rc = asn_realloc_rbuild_bitstring(pkt, pkt_len, offset,
                                              allow_realloc, var_val_type,
                                              var_val, var_val_len);
            break;
            
        case SNMP_NOSUCHOBJECT:
        case SNMP_NOSUCHINSTANCE:
        case SNMP_ENDOFMIBVIEW:
            rc = asn_realloc_rbuild_null(pkt, pkt_len, offset, allow_realloc,
                                         var_val_type);
            break;
            
        default:
            return 0;
    }

    if (rc == 0)
        return 0;

    /*
     * Build the OID.  
     */

    rc = asn_realloc_rbuild_objid(pkt, pkt_len, offset, allow_realloc,
                                  (u_char) (ASN_UNIVERSAL | ASN_PRIMITIVE |
                                            ASN_OBJECT_ID), var_name,
                                  *var_name_len);
    if (rc == 0)
        return 0;

    /*
     * Build the sequence header.  
     */

    rc = asn_realloc_rbuild_sequence(pkt, pkt_len, offset, allow_realloc,
                                     (u_char) (ASN_SEQUENCE |
                                               ASN_CONSTRUCTOR),
                                     *offset - start_offset);
    return rc;
}

int snmp_set_var_typed_value(netsnmp_variable_list * newvar, u_char type,
                             const u_char * val_str, size_t val_len)
{
    newvar->type = type;
    return snmp_set_var_value(newvar, val_str, val_len);
}

int snmp_set_var_value(netsnmp_variable_list * newvar,
                       const u_char * val_str, size_t val_len)
{
    if (newvar->val.string && newvar->val.string != newvar->buf)
        free(newvar->val.string);

    newvar->val.string = 0;
    newvar->val_len = 0;

    /*
     * need a pointer and a length to copy a string value. 
     */
    if (val_str && val_len)
    {
        if (val_len <= sizeof(newvar->buf))
            newvar->val.string = newvar->buf;
        else
        {
            newvar->val.string = (u_char *) malloc(val_len);
            if (!newvar->val.string)
                return 1;
        }
        memmove(newvar->val.string, val_str, val_len);
        newvar->val_len = val_len;
    }
    else if (val_str)
    {
        /*
         * NULL STRING != NULL ptr 
         */
        newvar->val.string = newvar->buf;
        newvar->val.string[0] = '\0';
        newvar->val_len = 0;
    }
    return 0;
}

int count_varbinds(netsnmp_variable_list * var_ptr)
{
    int  count = 0;

    for (; var_ptr != NULL; var_ptr = var_ptr->next_variable)
        count++;

    return count;
}


void snmp_free_var(netsnmp_variable_list * var)
{
    if (!var)
        return;

    if (var->name != var->name_loc)
        SNMP_FREE(var->name);
    if (var->val.string != var->buf)
        SNMP_FREE(var->val.string);

    free((char *) var);
}

void snmp_free_varbind(netsnmp_variable_list * var)
{
    netsnmp_variable_list *ptr;
    while (var) {
        ptr = var->next_variable;
        snmp_free_var(var);
        var = ptr;
    }
}

