/*
 * snmpd.c
 */
/** @defgroup agent The Net-SNMP agent
 * The snmp agent responds to SNMP queries from management stations
 */
/*
 * Copyright 1988, 1989 by Carnegie Mellon University
 * 
 * All Rights Reserved
 * 
 * Permission to use, copy, modify, and distribute this software and its 
 * documentation for any purpose and without fee is hereby granted, 
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in 
 * supporting documentation, and that the name of CMU not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  
 * 
 * CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
 * ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
 * CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
 * ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 * *****************************************************************
 */

#include <string.h>
#include <sys/un.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <unistd.h>
#include <syslog.h>
#include "snmpl_lib.h"
//#include "cm_utils.h"
//#include "cm_comm.h"
//#include "cm_params.h"
//#include "cm_message.h"
#include "input.h"

/* Must be included last */
#if HAVE_DMALLOC_H
#include <dmalloc.h>
#endif

#define TIMETICK         500000L

int             snmp_dump_packet;
int             running = 1;
int             reconfig = 0;

extern char   **argvrestartp;
extern char    *argvrestart;
extern char    *argvrestartname;

#define NUM_SOCKETS 32

/*
 * These definitions handle 4.2 systems without additional syslog facilities.
 */

SNMPL_AGENT_STRUC sSnmplAgnt_priv;
SNMPL_AGENT_STRUC *pSnmp = &sSnmplAgnt_priv;

/* This is the global received buffer. All receives messages can be placed
   here until they get parsed. This buffer is large enough for the configman
   buffer as well as the SNMP PDU */
char  rcvBuffer[CM_MAX_MESSAGE_LEN];

/*******************************************************************-o-******
 * main - Non Windows
 * SnmpDeamonMain - Windows to support windows serivce
 *
 * Parameters:
 *   argc
 *  *argv[]
 *      
 * Returns:
 *  0   Always succeeds.  (?)
 *
 *
 * Setup and start the agent daemon.
 *
 * Also successfully EXITs with zero for some options.
 */
int main(int argc, char *argv[])
{
    char  options[16] = "t";
    int arg, timeoutFlag = 1; 
    int iLoop;
    int retval=0;
    fd_set  master_readfds, readfds;
    int active_fds, max_readfd = 0;
    int recvlen;
    struct sockaddr_in  from_ip;
    socklen_t           from_len;
    struct timeval      select_tv;

    struct sockaddr_in  server_sock;
    int         server_fd;
    int op = 1;
    struct msg_base *msgPtr;
    
#ifdef SIGTERM
    DEBUGMSGTL(("signal", "registering SIGTERM signal handler\n"));
    signal(SIGTERM, snmpl_shutdown);
#endif

#if !NSP_PHASE1
#ifdef SIGINT
    DEBUGMSGTL(("signal", "registering SIGINT signal handler\n"));
    signal(SIGINT, SnmpdShutDown);
#endif
#ifdef SIGHUP
    DEBUGMSGTL(("signal", "registering SIGHUP signal handler\n"));
    signal(SIGHUP, SnmpdReconfig);
#endif
#ifdef SIGUSR1
    DEBUGMSGTL(("signal", "registering SIGUSR1 signal handler\n"));
    signal(SIGUSR1, SnmpdDump);
#endif
#ifdef SIGPIPE
    DEBUGMSGTL(("signal", "registering SIGPIPE signal handler\n"));
    signal(SIGPIPE, SIG_IGN);   /* 'Inline' failure of wayward readers */
#endif
#endif /* if !NSP_PHASE1 */

    /* check any arguments */
    while ((arg = getopt(argc, argv, options)) != EOF)
    {
        switch(arg)
        {
            /* -t: not to time out for debugging */
            case 't':
                timeoutFlag = 0;
                break;
                
            default:
                break;
        }
    }
    
    /* TODO: read in the OID database file */
    
    /* SERVER IMPLEMENTATION */
#ifdef BOARD_
    server_fd=socket(AF_INET, SOCK_DGRAM, 0);
    if(server_fd<0){
        printf("Couldn't create socket\n");
        exit(1);
    }
    server_sock.sin_family=AF_INET;
    server_sock.sin_addr.s_addr = htonl(INADDR_ANY);
    server_sock.sin_port = htons(1610);
    retval = bind(server_fd, (struct sockaddr *) &server_sock, sizeof(server_sock));
    if(retval < 0){
        printf("can't bind socket\n");
        exit(1);
    }
    close(0);
    close(1);
    dup2(server_fd, STDIN_FILENO);
#else
    
    ioctl (STDIN_FILENO, FIONBIO, &op);
    
    // setsockopt(STDIN_FILENO, SOL_SOCKET, SO_TYPE, SOCK_DGRAM, );
#endif

    
    /* Start up the syslog support */
    openlog("snmpd", LOG_CONS, 0);
    syslog(LOG_DEBUG,"Started");

    /* Initialize the data structures */
    memset(pSnmp, 0x00, sizeof(SNMPL_AGENT_STRUC));

    pSnmp->iIdleTime = SNMPL_LISTEN_INET_TIMEOUT;
    
    /* Start the socket with cfgman. This function logs any errors with the
       startup */
    if((pSnmp->iCfgManSocket=comm_start(CM_SNMP_NAME,0)) < 0){
        snmpl_shutdown(1);
        return -1;
    }

    /* Setup the configman input routines */
    setup_cm_input(pSnmp->iCfgManSocket, 20, rcvBuffer, CM_MAX_MESSAGE_LEN);

    if (snmpl_GetCommunityInfo())
    {
        /* Did not get the community name */
        snmpl_shutdown(2);
        return -1;
    }

    /* We have to handle two input fd's, the configman fd for talking with
       configman, and the stdin. stdin is redirected by the inetd server
       to be connected with the raw network socket */
    FD_ZERO(&master_readfds);
    FD_SET(STDIN_FILENO, &master_readfds);
    FD_SET(pSnmp->iCfgManSocket, &master_readfds);
    max_readfd = pSnmp->iCfgManSocket +1;

    iLoop = 1;
    while (iLoop)
    {
        /* Copy the master set into the working set */
        memcpy(&readfds, &master_readfds, sizeof(fd_set));

        /* Set the timeout */
        select_tv.tv_sec = pSnmp->iIdleTime;
        select_tv.tv_usec = 0;

        /* Now, do the select. */
        if (timeoutFlag)
            active_fds=select(max_readfd, &readfds,
                              NULL, NULL, &select_tv);
        else
            active_fds=select(max_readfd, &readfds,
                              NULL, NULL, NULL);
        
        if(active_fds<0){
            /* Error with select call. */
            printf("Error with select call\n");
            iLoop=0;
        }

        /* Check to see if the timeout has expired */
        if( (select_tv.tv_sec == 0) && (select_tv.tv_usec == 0)){
            return iLoop=0;
        }

        if( FD_ISSET(pSnmp->iCfgManSocket, &readfds)){
            /* ConfigMan message. The only time this is a valid
               message is when we have a configuration changed
               notificaiton. */
            recvlen = comm_recvfrom(pSnmp->iCfgManSocket, rcvBuffer,
                                    CM_MAX_MESSAGE_LEN -1, MSG_DONTWAIT,
                    								(struct sockaddr *)&from_ip, &from_len);
            if( recvlen >0){
                rcvBuffer[recvlen]=0;
								msgPtr = msg_parse(rcvBuffer,recvlen);
                if(msgPtr->msg_type == MSG_REPORT)
                  snmpl_GetCommunityInfo();
            }
        }

        if( FD_ISSET(STDIN_FILENO, &readfds)){
            from_len=sizeof(struct sockaddr_in);

            recvlen = comm_recvfrom(STDIN_FILENO, rcvBuffer, CM_MAX_MESSAGE_LEN -1, MSG_DONTWAIT,
                    (struct sockaddr *)&from_ip, &from_len);
            if( recvlen >0){
                rcvBuffer[recvlen]=0;
            }

            /* Ok, we now have a complete PDU in the buffer, with
               IP address info stored in the from_ip struct. Lets
               handle it */
            retval = snmpl_process_snmpPDU(rcvBuffer,
                    recvlen, &from_ip);
        }
    }    

    snmpl_shutdown(retval);
    syslog(LOG_DEBUG,"Stopping");
    closelog();
    return 0;
}
