/*------------------------------------------------------------------------------
 Copyright (c) 2003 Texas Instruments Incorporated
 ALL RIGHTS RESERVED

 "This computer program is subject to a separate license agreement
 signed by Texas Instruments Incorporated and the licensee, and is
 subject to the restrictions therein. No other rights to reproduce,
 use, or disseminate this computer program, whether in part or in
 whole, are granted."

--------------------------------------------------------------------------------
 TI SNMP Transport Interfacing Library

 Module Name:   SNMP Agent

 Module Purpose: provides interface between SNMP Agent and ConfigMan
 Queries Supported:
--------------------------------------------------------------------------------

 Revision History:
 Date and Type of Change.
 08/20/03 -> initial coding
------------------------------------------------------------------------------*/

#include "cm_utils.h"
#include <sys/types.h>
#include <sys/socket.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <syslog.h>
#include <malloc.h>

#include "cm_message.h"
#include "cm_comm.h"
#include "cm_module.h"
#include "include/snmp.h"
#include "include/snmp_lib.h"
#include "include/snmp_asn1.h"
#include "snmpl_lib.h"
#include "input.h"

/* Forward declarations */
static int handle_pdu_commands(struct snmpl_pdu *pdu);
static int snmpl_process_entry(struct snmpl_pdu *pdu,
                netsnmp_variable_list *vp);
static struct map_entry *get_map_entry(int command, netsnmp_variable_list *vp);
static char *indexToStr(oid *pIndexOid, int indexOidLen);
static int oidToInt(oid *dest, char *from, int str_len);

/* The function check a message response to verify proper transaction
   format. If it doesn't have the proper format, it logs the error */
#define     RESPONSE_OK     0
#define     RESPONSE_TIMEOUT    -1
#define     RESPONSE_ERROR      -2
static int  check_response(struct msg_base *msg, char *info)
{
    if(!msg){
        /* Communication error, just dump what we got */
        syslog(LOG_INFO, "Timeout on response: %s", info);
        return RESPONSE_TIMEOUT;
    }

    /* Check for non-transaction */
    if(msg->msg_type != MSG_TRANSACTION){
        syslog(LOG_INFO, "Received non-transaction message: %s", info);
        return RESPONSE_ERROR;
    }

    if(!msg->transaction){
        syslog(LOG_INFO, "Received malformed reply: %s", info);
        return RESPONSE_ERROR;
    }
    return RESPONSE_OK;
}

int snmpl_GetCommunityInfo(void)
{
    int  i, retval;
    struct msg_base     *msg;
    char *pDest, *fromStr;
    
    msgr_sendTransQuery ("snmpd", "snmpcm", NULL,
                         "settings/community");

    msg = get_cm_input();

    if (msg == NULL)
        return -1;

    retval = check_response(msg, "settings/community");

    if (retval < 0)
    {
        if (retval == RESPONSE_TIMEOUT)
            msg_free(msg);
        
        return -1;
    }

    for (i = 0; i < 3; i++)
    {
        pDest = pSnmp->sCommunity[i].cCommunityName;
        
        /* 1st column is the community name, 2nd is access right */
        fromStr = msg_getTransValue(msg, 0, 2*i);
        if (fromStr)
            strcpy(pDest, fromStr);
        else
            pDest = NULL;

        fromStr = msg_getTransValue(msg, 0, 2*i+1);
        pDest = pSnmp->sCommunity[i].cAccessRight;
        if (fromStr)
            strcpy(pDest, fromStr);
        else
            pDest = NULL;
    }

    fromStr = msg_getTransValue(msg, 0, 2*i);
    pSnmp->iIdleTime = atoi(fromStr);
    
    return 0;
}


static void flush_pdu(struct snmpl_pdu *pdu)
{
    netsnmp_variable_list *vp, *vpTemp;

    if(!pdu)
        return;

    /* Loose the strings */
    CLEAR_TEXT(pdu->community);

    /* Clean out the varbinds */
    for (vp = pdu->pVarBind; vp; vp = vpTemp){
        vpTemp = vp->next_variable;
        free(vp);
    }
    pdu->pVarBind = NULL;
}

/* This takes a PDU, and parses its data.
   Returns:
    0 for success
    negitive for error
*/
static int parse_pdu_data (struct snmpl_pdu *pdu, u_char *pData, int *recvlen)
{
    u_char          type;
    u_char          *var_val;
    netsnmp_variable_list   *vp = NULL;
    oid         objid[MAX_OID_LEN];
    size_t          len;
    int         badtype = 0;

    /* retrieve request ID */
    pData = asn_parse_int(pData, recvlen, &type, &pdu->req_id,
            sizeof(pdu->req_id));
    if (!pData){
        syslog(LOG_INFO, "Request ID parsing failed\n");
        return -1;
    }

    /* retrieve error status (non-repeaters for getbulk) */    
    pData = asn_parse_int(pData, recvlen, &type, &pdu->errstat,
            sizeof(pdu->errstat));
    if (!pData){
        syslog(LOG_INFO,"Error status (max-repeaters) parsing failed\n");
        return -1;
    }

    /* retrieve error index (max-repetitions for getbulk) */
    pData = asn_parse_int(pData, recvlen, &type, &pdu->errindex,
            sizeof(pdu->errindex));
    if (!pData){
        syslog(LOG_INFO,"Error index (max-repetitions) parsing failed\n");
        return -1;
    }

    /* get header for variable-bindings sequence */
    pData = asn_parse_sequence(pData, recvlen, &type,
            (ASN_SEQUENCE | ASN_CONSTRUCTOR),
            "varbinds");
    if (!pData){
        syslog(LOG_INFO, "Var-Bind header parsing failed\n");
        return -1;
    }

    /* get each varBind sequence */
    while (*recvlen > 0){
        netsnmp_variable_list *vptemp;
        vptemp = (netsnmp_variable_list *)malloc(sizeof(*vptemp));
        if (!vptemp)
            return -1;

        if (!vp){
            pdu->pVarBind = vptemp;
        } else {
            vp->next_variable = vptemp;
        }
        vp = vptemp;

        vp->next_variable = NULL;
        vp->val.string = NULL;
        vp->name_length = MAX_OID_LEN;
        vp->name = 0;
        vp->index = 0;
        pData = snmp_parse_var_op(pData, objid, &vp->name_length, &vp->type,
                &vp->val_len, &var_val, recvlen);
        if (!pData)
            return -1;
        if (snmp_set_var_objid(vp, objid, vp->name_length))
            return -1;

        len = MAX_PACKET_LENGTH;

        switch ((short) vp->type){
            case ASN_INTEGER:
                vp->val.integer = (long *) vp->buf;
                vp->val_len = sizeof(long);
                asn_parse_int(var_val, &len, &vp->type,
                        (long *) vp->val.integer,
                        sizeof(vp->val.integer));
                break;
            case ASN_COUNTER:
            case ASN_GAUGE:
            case ASN_TIMETICKS:
            case ASN_UINTEGER:
                vp->val.integer = (long *) vp->buf;
                vp->val_len = sizeof(u_long);
                asn_parse_unsigned_int(var_val, &len, &vp->type,
                        (u_long *) vp->val.integer,
                        vp->val_len);
                break;
            case ASN_COUNTER64:
                vp->val.counter64 = (struct counter64 *) vp->buf;
                vp->val_len = sizeof(struct counter64);
                asn_parse_unsigned_int64(var_val, &len, &vp->type,
                        (struct counter64 *) vp->val.
                        counter64, vp->val_len);
                break;
            case ASN_OCTET_STR:
            case ASN_IPADDRESS:
            case ASN_OPAQUE:
            case ASN_NSAP:
                if (vp->val_len < sizeof(vp->buf)){
                    vp->val.string = (u_char *) vp->buf;
                } else {
                    vp->val.string = (u_char *) malloc(vp->val_len);
                }
                if (!vp->val.string)
                    return -1;
                asn_parse_string(var_val, &len, &vp->type, vp->val.string,
                        &vp->val_len);
                break;
            case ASN_OBJECT_ID:
                vp->val_len = MAX_OID_LEN;
                asn_parse_objid(var_val, &len, &vp->type, objid, &vp->val_len);
                vp->val_len *= sizeof(oid);
                vp->val.objid = (oid *) malloc(vp->val_len);
                if (!vp->val.objid)
                    return -1;
                memmove(vp->val.objid, objid, vp->val_len);
                break;
            case SNMP_NOSUCHOBJECT:
            case SNMP_NOSUCHINSTANCE:
            case SNMP_ENDOFMIBVIEW:
            case ASN_NULL:
                break;
            case ASN_BIT_STR:
                vp->val.bitstring = (u_char *) malloc(vp->val_len);
                if (!vp->val.bitstring)
                    return -1;
                asn_parse_bitstring(var_val, &len, &vp->type,
                        vp->val.bitstring, &vp->val_len);
                break;
            default:
                syslog(LOG_INFO, "bad type returned (%x)\n", vp->type);
                badtype = -1;
                break;
        }
    }

    return (badtype);
}

/* Parses the pdu into the pdu structure. After this function is done, all
   necessary data is stored inside the pdu structure. This function returns
   0 if success, or negitive for errors */
static int parse_pdu(struct snmpl_pdu *pdu, char *buffer, int recvlen)
{
    u_char   *pData;
    u_char   community[COMMUNITY_MAX_LEN];
    size_t   community_length = COMMUNITY_MAX_LEN;
    u_char   ucOperation;

    /* Assign our working pointer to the recv buffer */
    pData = (u_char *)buffer;

    /* retrieve version from SNMP PDU */
    pdu->version = snmp_parse_version(pData, recvlen);

    /* retrieve community name from SNMP PDU */
    pData = snmp_comstr_parse (pData, &recvlen, community,
            &community_length, &pdu->version);
    if (!pData) {
        syslog(LOG_INFO, "Community parsing failed\n");
        return -1;
    }

    /* Save off the community name */
    SET_N_TEXT(pdu->community, community, community_length);

    /* get the PDU type (operation) */
    pData = asn_parse_header(pData, &recvlen, &ucOperation);
    if (!pData) {
        syslog(LOG_INFO, "Operation parsing failed\n");
        return -1;
    }
    pdu->command = ucOperation;

    /* Now, just parse the rest of the pdu data */
    return parse_pdu_data(pdu, pData, &recvlen);
}

/* This function will handle a single PDU packet. The return values are:
   -1: Error
   0: Success
*/
int snmpl_process_snmpPDU(char *recvbuf, int recvlen, struct sockaddr_in *from)
{
    int         i;
    struct snmpl_pdu    pdu;

    /* Zero out our pdu structure */
    memset(&pdu, 0, sizeof(struct snmpl_pdu));

    /* Save off the from address info */
    pdu.from = from;

    /* Lets parse the pdu */
    if(parse_pdu(&pdu, recvbuf, recvlen) <0){
        /* Error parsing the pdu. Flush the leftover, and leave */
        flush_pdu(&pdu);
        return -1;
    }

    /* Lets validate some settings. We don't support all options in
       phase I */

    /* only support SNMPv1 and SNMPv2c in Phase I */
    if ((pdu.version != SNMP_VERSION_1) &&
            (pdu.version != SNMP_VERSION_2c)) {
        syslog(LOG_INFO, "Bad version(%ld) in SNMP PDU\n",
                pdu.version);
        flush_pdu(&pdu);
        return -1;
    }

    /* authenticates against configured community names */
    for (i = 0; i < SNMPL_MAX_COMMUNITY_NAMES; i++){
        if ( !(safe_strcmp(pSnmp->sCommunity[i].cCommunityName,
                        pdu.community)))
            break;
    }
    /* community string authentication failed */
    if (i == 3){
        syslog(LOG_INFO, "Community authentication failed\n");
        flush_pdu(&pdu);
        return -1;            
    }

    /* check if operation is supported */
    switch (pdu.command){
        case SNMP_MSG_GET:
        case SNMP_MSG_GETNEXT:
        case SNMP_MSG_GETBULK:
            break;

        case SNMP_MSG_SET:
            /* Check Read-Write access here when supported */
            /* Fall through in phase I */
        default:
            syslog(LOG_INFO, "Not supported SNMP operation(%x)\n",
                    pdu.command);
            flush_pdu(&pdu);
            return -1;
    }

    if(!handle_pdu_commands(&pdu)){
        /* The PDU was handled correctly */
        snmpl_send_response(&pdu);
    }

    flush_pdu(&pdu);
    return 0;
}

oid mib2Oid[] = {1, 3, 6, 1, 2, 1};


/* returns 0 if the PDU should be sent back or -1 if the PDU response
   should be dropped */
static int handle_pdu_commands(struct snmpl_pdu *pdu)
{
    int retval, saved_command;
    netsnmp_variable_list   *prevVp = NULL, *vp = NULL;

    if(!pdu)
        return -1;

    switch (pdu->command){
        case SNMP_MSG_GET:
        case SNMP_MSG_GETNEXT:
        case SNMP_MSG_GETBULK:
            for (vp = pdu->pVarBind; vp; vp = vp->next_variable){
                snmp_set_var_typed_value(vp, ASN_NULL, NULL, 0);
            }
            /* fall through */
        default:
            pdu->vbcount = count_varbinds(pdu->pVarBind);
            break;
    }

    if (pdu->command == SNMP_MSG_GETBULK)
        snmpl_process_getBulk(pdu);


    /* Now, work through the entries, processing each entry. */
    for (vp = pdu->pVarBind; vp; vp = vp->next_variable)
    {
        pdu->handled_index++;  // This index has to be 1-base
        saved_command = pdu->command;
        if (pdu->command == SNMP_MSG_GETBULK)
        {
            if (prevVp)
                snmp_set_var_objid(vp, prevVp->name,
                                   prevVp->name_length);
            
            pdu->command = SNMP_MSG_GETNEXT;
        }
        
        retval=snmpl_process_entry(pdu, vp);
        if(retval<0){
            /* Error with this entry. Check the return code */
            if(retval < -1){
                /* Do not send error packet */
                return -1;
            }
            return 0;   /* Send error packet */
        }

        while (retval > 0)
        {
            retval=snmpl_process_entry(pdu, vp);
        }

        pdu->handled_var = vp;
        pdu->command = saved_command;
        prevVp = vp;
    }

    return 0;
}

void snmp_process_getBulk(snmp_pdu *pdu)
{
    netsnmp_variable_list       *varbind_ptr,
                    *vbsave,
                    *vbptr,
                    **prevNext;
    int bulkcount = 0, bulkrep = 0;
    int i = 0,
        n = 0,
        r = 0;
    int count = count_varbinds(pdu->pVarBind);

    if(!pdu)
        return;

    if (pdu->errstat < 0)
        pdu->errstat = 0;

    if (pdu->errindex < 0)
        pdu->errindex = 0;

    if (pdu->errstat < count)
        n = pdu->errstat;
    else
        n = count;

    if ((r = count - n) < 0){
        r = 0;
        pdu->pBulk = NULL;
    } else {
        pdu->pBulk = (netsnmp_variable_list **) malloc(
                pdu->errindex * r * sizeof(struct varbind_list *));
    }

    prevNext = &(pdu->pVarBind);
    for (varbind_ptr = pdu->pVarBind; varbind_ptr; varbind_ptr = vbsave){
        /*
         * getbulk mess with this pointer, so save it 
         */
        vbsave = varbind_ptr->next_variable;

        if (n > 0) {
            n--;
        } else {
            /*
             * repeate request varbinds on GETBULK.  These will
             * have to be properly rearranged later though as
             * responses are supposed to actually be interlaced
             * with each other.  This is done with the asp->bulkcache. 
             */
            bulkrep = pdu->errindex - 1;
            if (pdu->errindex > 0) {
                vbptr = varbind_ptr;
                pdu->pBulk[bulkcount++] = vbptr;

                for (i = 1; i < pdu->errindex; i++) {
                    vbptr->next_variable =
                        SNMP_MALLOC_STRUCT(variable_list);
                    /*
                     * don't clone the oid as it's got to be
                     * overwwritten anyway 
                     */
                    if (!vbptr->next_variable) {
                        /*
                         * XXXWWW: ack!!! 
                         */
                    } else {
                        vbptr = vbptr->next_variable;
                        vbptr->name_length = 0;
                        vbptr->type = ASN_NULL;
                        pdu->pBulk[bulkcount++] = vbptr;
                    }
                }
                vbptr->next_variable = vbsave;
            } else {
                /*
                 * 0 repeats requested for this varbind, so take it off
                 * the list.  
                 */
                vbptr = varbind_ptr;
                *prevNext = vbptr->next_variable;
                vbptr->next_variable = NULL;
                snmp_free_varbind(vbptr);
                pdu->vbcount--;
                continue;
            }
        }

        /*
         * count the varbinds 
         */
        prevNext = &(varbind_ptr->next_variable);
    }
}


static void modify_requestOID(netsnmp_variable_list *vp,
                              int loc,
                              oid *name, int nameLen,
                              int indexLen)
{
    int i;
    int len = nameLen + indexLen;
    
    if ((vp->name != vp->name_loc) &&
        (vp->name != NULL) &&
        ((len + 6) > vp->name_length))
    {
        if (vp->name)
            free(vp->name);

        vp->name = (oid *) malloc(sizeof(oid) * (len + 6));
    }
    
    // make sure we have OID up to mib2
    vp->name[0] = 1;
    vp->name[1] = 3;
    vp->name[2] = 6;
    vp->name[3] = 1;
    vp->name[4] = 2;
    vp->name[5] = 1;
    
    for (i = 0; i < len; i++)
    {
        vp->name[loc+i] = name[i];
    }
    
    vp->name_length = len + 6;
}


static void fix_GetNextOid(netsnmp_variable_list *vp)
{
    oid mgmt = vp->name[SNMPL_MIB2_MGMT];
    oid group = vp->name[SNMPL_MIB2_GROUP];
    oid table = vp->name[SNMPL_MIB2_TABLE];

    if (mgmt > 2)
        return;
    
    // fill up the mib2 part if not long enough
    if (vp->name_length < SNMPL_MIB2_GROUP)
        modify_requestOID(vp, 0, NULL, 0, 0);

    // missing group. fill it up
    // We must doing GetNext to reach this point
    if (vp->name_length < SNMPL_MIB2_LEAF)
    {
        vp->name[SNMPL_MIB2_LEAF] = 1;
        return;
    }
    
    if (group == 1)  // sysORtable
    {
        if (table == 9)
        {
            if (vp->name_length == 11)
                return;
            
            if (vp->name_length > 11)
            {
                // vp->name[SNMPL_MIB2_TBLOBJ]++;
                vp->name_length = SNMPL_MIB2_INDEX+1;
                return;
            }
            if (vp->name_length < 10)
            {
                vp->name[SNMPL_MIB2_TBLENTRY] = 1;
                vp->name[SNMPL_MIB2_TBLOBJ] = 1;
                vp->name_length = SNMPL_MIB2_TBLOBJ+1;
                return;                
            }
            return;
        }
    }
    else if (group == 2)
    {
        if (table == 2)
        {
            if (vp->name_length == 11)
                return;
            
            if (vp->name_length > 11)
            {
                //vp->name[SNMPL_MIB2_TBLOBJ]++;
                vp->name_length = SNMPL_MIB2_INDEX+1;
                return;
            }
            
            if (vp->name_length < 10)
            {
                vp->name[SNMPL_MIB2_TBLENTRY] = 1;
                vp->name[SNMPL_MIB2_TBLOBJ] = 1;
                vp->name_length = SNMPL_MIB2_TBLOBJ+1;
                return;                
            }
            return;
        }
    }
    else if (group == 4)
    {
        if ((table == 20) || (table == 21))
        {
            if (vp->name_length == 14)
                return;
            if (vp->name_length > 14)
            {
                //vp->name[SNMPL_MIB2_TBLOBJ]++;
                vp->name_length = SNMPL_MIB2_INDEX+4;
                return;
            }
            if (vp->name_length < 10)
            {
                vp->name[SNMPL_MIB2_TBLENTRY] = 1;
                vp->name[SNMPL_MIB2_TBLOBJ] = 1;
                vp->name_length = SNMPL_MIB2_TBLOBJ+1;
                return;                
            }
            return;
        }
        else if (table == 22)
        {
            if (vp->name_length == 15)
                return;
            
            if (vp->name_length > 15)
            {
                //vp->name[SNMPL_MIB2_TBLOBJ]++;
                vp->name_length = SNMPL_MIB2_INDEX+5;
                return;
            }
            if (vp->name_length < 10)
            {
                vp->name[SNMPL_MIB2_TBLENTRY] = 1;
                vp->name[SNMPL_MIB2_TBLOBJ] = 1;
                vp->name_length = SNMPL_MIB2_TBLOBJ+1;
                return;                
            }
            return;
        }
    }

#if 0
    if (vp->name_length > 8)
    {                    
        vp->name_length = SNMPL_MIB2_LEAF+1;
    }
#endif
}


static int check_oidForGet(netsnmp_variable_list *vp,
                           int indexOidLen)
{
    switch(vp->name[SNMPL_MIB2_GROUP])
    {
        case 1:
            // system group OR Table has index of length 1
            if (vp->name[SNMPL_MIB2_TABLE] == 9)
            {
                if (indexOidLen != 1)
                    return -1;
                else
                    return 0;
            }
            if (indexOidLen)
                return -1;
            break;
            
        case 2:               
            // interface group has index of length 1
            if (vp->name[SNMPL_MIB2_TABLE] == 2)
            {
                if (indexOidLen != 1)
                    return -1;
                else return 0;
            }
            if (indexOidLen)
                return -1;
            break;
            
        case 4:
            switch (vp->name[SNMPL_MIB2_TABLE])
            {
                case 20:
                case 21:
                    // ipAddrTbl & ipRtTbl
                    if (indexOidLen != 4)
                        return -1;
                    break;
                    
                case 22:
                    // ipN2MTbl
                    if (indexOidLen != 5)
                        return -1;
                    break;
                    
                default:
                    if (indexOidLen)
                        return -1;
                    break;
            }
            break;
            
        default:
            if (indexOidLen)
                return -1;
            break;
    }
    return 0;
}

int convertStr2PhysAddr(char *dest, char *src)
{
    char *del;
    char *string = src;
    int i, tmp;
    
    del = strchr(string, ':');
    for (i = 1; del != NULL; i++)
    {
        *del = 0;
        tmp = atoi(string);
        *dest = tmp & 0xff;
        dest++;
        string = del+1;
        del = strchr(string, ':');
    }

    if (strlen(string) != 0)
    {
        tmp = atoi(string);
        *dest = tmp & 0xff;
    }
    
    return i;
}

static struct map_entry *get_map_entry(int command, netsnmp_variable_list *vp)
{
    struct map_entry *entry = NULL;
    int result, oidLen;

    fix_GetNextOid(vp);
    
    for (entry = pKeyTbl; entry->module; entry++)
    {
        // search oids past mib2 on mibGroup, mibObject/MibTable,
        // (tableEntry, tableObject if it is table)
        if (command == SNMP_MSG_GET)
        {
            if (vp->name[vp->name_length-1] == 0)
                vp->name_length -= 1;
        }
        
        oidLen = vp->name_length - 6;
        if (vp->name[SNMPL_MIB2_MGMT] > 2)
            return NULL;
        
        if (oidLen > 4)
            oidLen = 4;

        result = snmpl_oid_cmp(entry->name, &vp->name[SNMPL_MIB2_GROUP],
                               entry->nameLen, oidLen);
        if (!result)
        {
            return entry;
        }
        else if (result > 0) 
        {
            if (command == SNMP_MSG_GETNEXT)
            {

                // for get next we want to get the closest one not over
                if (entry != pKeyTbl)
                {
                    --entry;
                    result = snmpl_oid_cmp(entry->name, &vp->name[SNMPL_MIB2_GROUP],
                                           entry->nameLen, oidLen);
                    if (result < 0)
                        return ++entry;
                        
                    return entry;
                }
                
                else
                    return pKeyTbl;
            }
            // we pass the oid and don't need to proceed
            return NULL;
        }
    }

    
#if 0
    entry=(struct map_entry *)malloc(sizeof(struct map_entry));
    if(!entry)
        return NULL;
    
    memset(entry, 0, sizeof(struct map_entry));

    /* TODO: put in real table entries */
    SET_TEXT(entry->module, "sar");
    SET_TEXT(entry->key, "valid_settings/modulation");
    entry->type = ASN_OCTET_STR;
#endif

    return NULL;
}


int snmpl_oid_cmp(oid *in_name1, oid *in_name2, int len1, int len2)
{
    register int    len;
    register const oid *name1 = in_name1;
    register const oid *name2 = in_name2;

    /*
     * len = minimum of len1 and len2 
     */
    if (len1 < len2)
        len = len1;
    else
        len = len2;
    /*
     * find first non-matching OID 
     */
    while (len-- > 0) {
        /*
         * these must be done in seperate comparisons, since
         * subtracting them and using that result has problems with
         * subids > 2^31. 
         */
        if (*(name1) != *(name2)) {
            if (*(name1) < *(name2))
                return -1;
            return 1;
        }
        name1++;
        name2++;
    }
    /*
     * both OIDs equal up to length of shorter OID 
     */
    if (len1 < len2)
        return -1;
    if (len2 < len1)
        return 1;
    return 0;
}


oid *get_IndexOid(int command, netsnmp_variable_list *vp,
                 int *indexLen)
{
    int len = vp->name_length - SNMPL_MIB2_INDEX;
    oid *pIndex = NULL;
    
    if (len < 0)
        len = 0;
    
    // index is contained in the OIDs, retrieve them
    if (len > 0)
        pIndex = &vp->name[SNMPL_MIB2_INDEX];
    
    *indexLen = len;
    return pIndex;
}

void append_OidIndex(netsnmp_variable_list *vp,
                     struct msg_value *values)
{
    oid group = vp->name[SNMPL_MIB2_GROUP];
    oid table = vp->name[SNMPL_MIB2_TABLE];
    
    if (((group == 1) && (table == 9)) ||
        ((group == 2) && (table == 2)))
    {
        // index is located at column 1 and is type of integer
        oid index = atol(values->value[0]);
        vp->name[SNMPL_MIB2_INDEX] = index;
        vp->name_length = SNMPL_MIB2_INDEX + 1;
    }
    else if (group == 4)
    {
        if ((table == 20) || (table == 21))
        {
            // index is located at column 1 and is type of IP address
            union 
            {
                long long ipAddr;
                char ip[8];
            } val;
            
            int  i;

            val.ipAddr = atoll(values->value[0]);
            
            for (i=0; i<4; i++)
                vp->name[SNMPL_MIB2_INDEX+i] = (val.ip[i] & 0xff);
            vp->name_length = SNMPL_MIB2_INDEX + 4;
        }
        else if (table == 22)
        {
            // index is located at both column 1 (integer) and
            // column 2 (IP address)
            union 
            {
                long long ipAddr;
                char ip[8];
            } val;
            
            oid  indexOid;
            int  i;
            
            indexOid = atoll(values->value[0]);
            val.ipAddr = atoll(values->value[1]);

            for (i=0; i<5; i++)
            {
                if (i == 0)
                    vp->name[SNMPL_MIB2_INDEX+i] = indexOid;
                else
                    vp->name[SNMPL_MIB2_INDEX+i] = (val.ip[i-1] & 0xff);
            }
            
            vp->name_length = SNMPL_MIB2_INDEX + 5;
        }
    }

    
#if 0    
    switch(vp->name[SNMPL_MIB2_GROUP])
    {
        case 1:
            // system group OR Table has index of length 1
            if (vp->name[SNMPL_MIB2_TABLE] == 9)
            {
                // index is located at column 1
                oid index = atol(values->value[1]);
                vp->name[SNMPL_MIB2_INDEX] = index;
                vp->name_length = SNMPL_MIB2_INDEX + 1;
            }
            break;
            
        case 2:               
            // interface group has index of length 1
            if (vp->name[SNMPL_MIB2_TABLE] == 2)
            {
                oid index = atol(values->value[1]);
                vp->name[SNMPL_MIB2_INDEX] = index;
                vp->name_length = SNMPL_MIB2_INDEX + 1;
            }
            break;
            
        case 4:
            switch (vp->name[SNMPL_MIB2_TABLE])
            {
                case 20:
                case 21:
                    // ipAddrTbl & ipRtTbl
                    break;
                    
                case 22:
                    // ipN2MTbl
                    break;
                    
                default:
                    break;
            }
            break;
            
        default:
            break;
    }    
#endif
}


//      OIDKEY_MAP_STRUC *pEntry, int varNum)
/* Returns 0 for success, -1 for error with return packet, -2 for error with
   no return packet */
static int snmpl_process_entry(struct snmpl_pdu *pdu,
                               netsnmp_variable_list *vp)
{
    struct map_entry *entry;
    struct msg_base  *msg;
    struct msg_value *values=NULL, *end_value=NULL;
    int               retval, indexOidLen;
    int               modified = 0, response_finished=0;
    char             *pOperation, *key1 = NULL;
    oid              *pIndexOid = NULL;
    char              requestKey[64];
    
    
    if(!pdu || !vp)
        return -1;

    // validate the mib2 OID
    if (pdu->command != SNMP_MSG_GETNEXT)
    {
        if ((strncmp((char *)vp->name, (char *)mib2Oid,
                    sizeof(oid)*6)) ||
            (vp->name_length < SNMPL_MIB2_TBLENTRY))
        {
            pdu->errstat = SNMP_ERR_NOSUCHNAME;
            pdu->errindex = pdu->handled_index;
            return 0;
        }
    }
    
    /* Get the mapping entry */
    entry = get_map_entry(pdu->command, vp);
    
    if(!entry){
        /* Error. Return what we have */
        pdu->errstat = SNMP_ERR_NOSUCHNAME;
        pdu->errindex = pdu->handled_index;
        return -1;
    }
    
    // The OID is supported
    pIndexOid = get_IndexOid(pdu->command, vp, &indexOidLen);
    
    if ((pdu->command == SNMP_MSG_GET) ||
        (pdu->command == SNMP_MSG_GETBULK) ||
        (pdu->command == SNMP_MSG_SET))
    {
        pOperation = "GET";

        if (check_oidForGet(vp, indexOidLen))
        {
            pdu->errstat = SNMP_ERR_NOSUCHNAME;
            pdu->errindex = pdu->handled_index;
            return -1;
        }
    }
    else // this is getNext
    {
        if ((entry+1)->nameLen == 0)
        {
            pdu->errstat = SNMP_ERR_NOSUCHNAME;
            pdu->errindex = pdu->handled_index;            
            return -1;
        }
        
        pOperation = "NEXT";

        if ((retval = snmpl_oid_cmp(entry->name,
                                    &vp->name[SNMPL_MIB2_GROUP],
                                    entry->nameLen,
                                    vp->name_length-6) <= 0))
        {
            if (entry->nameLen == 2)
            {
                entry++;
                modified = 1;
            }
        }
        else
            modified = 1;        
    }
    
    if (modified)
        modify_requestOID(vp, SNMPL_MIB2_GROUP, entry->name,
                          entry->nameLen, indexOidLen);        

    vp->type = entry->type;
    vp->entry = entry;
        
    if (pIndexOid)
    {
        key1 = indexToStr(pIndexOid, indexOidLen);    
        sprintf(requestKey, "%s@%s@%s",
                entry->key, pOperation, key1);
    }
    else
        sprintf(requestKey, "%s@%s",
                entry->key, pOperation);

    if (key1)
        free(key1);
    
#if 0
    /* good OID, convert the PDU to message and send to ConfigMan */
    msg = msgr_makeTransQuery(CM_SNMP_NAME, entry->module,
                              NULL, entry->key);
    if(!msg)
        return -1;
    SET_TEXT(seq, msg->seq);
    msg_send(msg);

#else
    msgr_sendTransQuery("snmpd", entry->module,
                        NULL, requestKey);
    
#endif
    
    while(!response_finished){
        msg=get_cm_input();
        
        /* Check the response, pass in the query for debug */
        retval=check_response(msg, entry->key);
        
        if (retval < 0){     /* Check for error */
            /* If timeout, exit the loop */
            if(retval == RESPONSE_TIMEOUT){
                msg_free(msg);
                return -2;
            }
            /* Else, just get another message */
            continue;
        }

        /* check if error */
        if (!safe_strcmp(msg_getTransKey(msg), "ERROR"))
        {
            vp->type = ASN_NULL;
            pdu->errstat = atoi(msg_getTransValue(msg, 0, 0));
            pdu->errindex = pdu->handled_index;
            msg_free(msg);
            return -1;
        }
        
        if (!safe_strcmp(msg_getTransKey(msg), "NEXT"))
        {
            char *newIndex;
            int  i, cols, sameTable, len;
            struct msg_value *values = msg->transaction->values;
            
            // may need to advance to next group rather
            // than copy the index
            sameTable = atoi(values->value[0]);

            if (!sameTable)
            {
                oid group = vp->name[SNMPL_MIB2_GROUP];
                oid table = vp->name[SNMPL_MIB2_TABLE];
                oid row = vp->name[SNMPL_MIB2_TBLOBJ];
                
                if ( ((group == 1) && (table == 9) && (row < 4))  ||
                     ((group == 2) && (table == 2) && (row < 22)) ||
                     ((group == 4) && (((table == 20) && (row < 4))  ||
                                       ((table == 21) && (row < 13)) ||
                                       ((table == 22) && (row < 4)))) )
                    
                {
                        vp->name[SNMPL_MIB2_TBLOBJ] += 1; //row ++;
                        vp->name_length = SNMPL_MIB2_TBLOBJ + 1;
                        return 1;
                }
                else
                {
                    vp->name[SNMPL_MIB2_TABLE] += 1; //table ++;
                    vp->name_length = SNMPL_MIB2_TABLE+1;
                    return 1;
                }
            }
            else
            {
                int result;
                char tmp[20];
                oid  indexOid[5];

                oid group = vp->name[SNMPL_MIB2_GROUP];
                oid table = vp->name[SNMPL_MIB2_TABLE];
                
                cols = msg_getTransNumCols(msg, 0);
                len = 0;

                // need to consider 2-indicie case
                // use for loop to append it to the vp->name
                // The following code only works for 1 index
                // of type IP address. Need to write a function
                // to determine which table it is and use
                // appropriate type for index and number of index

                if (group == 4)
                {
                    if ((table == 20) || (table == 21))
                    {
                        i = 1;
                        newIndex = values->value[i];
                        if (vp->name_length > SNMPL_MIB2_INDEX)
                        {
                            // OID contains index
                            sprintf(tmp, "%lu.%lu.%lu.%lu",
                                    vp->name[SNMPL_MIB2_INDEX],
                                    vp->name[SNMPL_MIB2_INDEX+1],
                                    vp->name[SNMPL_MIB2_INDEX+2],
                                    vp->name[SNMPL_MIB2_INDEX+3]);
                            result = strcmp(tmp, newIndex);
                            
                            // the last instance of the object
                            if (result == 0)
                            {
                                vp->name[SNMPL_MIB2_TBLOBJ]++;
                                vp->name_length = SNMPL_MIB2_TBLOBJ+1;
                                msg_free(msg);
                                return 1;
                            }
                        }
                        sscanf(newIndex, "%lu.%lu.%lu.%lu",
                               &indexOid[0], &indexOid[1],
                               &indexOid[2], &indexOid[3]);
                        
                        for (i=0; i< 4; i++)
                            vp->name[SNMPL_MIB2_INDEX+i] = indexOid[i];
                    }
                    else if (table == 22)
                    {
                        oid tmpIndex[5];
                        
                        tmpIndex[0] = atol(values->value[1]);
                        newIndex = values->value[2];
                        if (vp->name_length > SNMPL_MIB2_INDEX)
                        {
                            sscanf(newIndex, "%lu.%lu.%lu.%lu",
                                   &tmpIndex[1], &tmpIndex[2],
                                   &tmpIndex[3], &tmpIndex[4]);
                            result = snmpl_oid_cmp(&vp->name[SNMPL_MIB2_INDEX],tmpIndex,
                                                   vp->name_length - SNMPL_MIB2_INDEX, 5);
                            if (result == 0)
                            {
                                vp->name[SNMPL_MIB2_TBLOBJ]++;
                                vp->name_length = SNMPL_MIB2_TBLOBJ+1;
                                msg_free(msg);
                                return 1;
                            }
                        }
                        
                        for (i=0; i< 5; i++)
                            vp->name[SNMPL_MIB2_INDEX+i] = tmpIndex[i];
                    }
                }
                    
#if 0                    
                oidToInt(&(vp->name[SNMPL_MIB2_INDEX + len]),
                         values->value[i+1],
                         strlen(values->value[i+1]));
                len = strlen(values->value[i+1]);
#endif

                msg_free(msg);
                return 1;
            }
        }
        
        /* Message is good transaction. Lets check the sequence
           number */
#if 0
        if(safe_strcmp(msg->transaction->re_seq, seq)){
            /* Error, seq's don't match up. Drop this message */
            continue;
        }
#endif
        if(msg->transaction->type == MSG_TRANS_RESPONSE){
            response_finished++;
        } else if(msg->transaction->type != MSG_TRANS_RESPONSE_CONT){
            /* Not a response, bail out */
            msg_free(msg);
            return -1;
        }

        /* We cheat. We steal the msg_value row from the transaction
           itself. Less work */
        if(msg->transaction->values){
            struct msg_value *row;
            row=msg->transaction->values;
            if(!values){
                values=row;
                end_value=row;
            } else {
                end_value->next=row;
                row->prev=end_value;
            }
            /* Find the new end_value */
            while(end_value->next)
                end_value = end_value->next;

            /* Clean up the message */
            msg->transaction->values=NULL;
            msg_free(msg);
        }
    }

    if (pdu->command == SNMP_MSG_GETNEXT)
        append_OidIndex(vp, values);
    
    /* We now have a complete set of response values */
    switch(vp->type)
    {
        case ASN_INTEGER:
            vp->val.integer = (long *)(vp->buf);    /* Point to the temp storage */
            *vp->val.integer = atol(values->value[entry->colPos]);
//          *vp->val.integer = *((int *)(msg_getTransValue(msg, 0, 0)));
            vp->val_len = sizeof(long);
            break;

        case ASN_COUNTER:
        case ASN_GAUGE:
        case ASN_TIMETICKS:
        case ASN_UINTEGER:
            vp->val.integer = (long *)(vp->buf);    /* Point to the temp storage */
            *vp->val.integer = atol(values->value[entry->colPos]);
            // *vp->val.integer = *((int *)(msg_getTransValue(msg, 0, entry->colPos)));
            vp->val_len = sizeof(u_long);
            break;

        case ASN_COUNTER64:
            {
//              struct counter64 *pVal=(struct counter64 *)(msg_getTransValue(msg, 0, 0));
//              vp->val.counter64->low = pVal->low;
//              vp->val.counter64->high = pVal->high;
//              vp->val_len = sizeof(struct counter64);
                break;
            }

        case ASN_IPADDRESS:            
            vp->val.integer = (long *)(vp->buf);    /* Point to the temp storage */
            *vp->val.integer = atoll(values->value[entry->colPos]);
            vp->val_len = sizeof(long);
            break;

        case ASN_OPAQUE:
        case ASN_NSAP:
            vp->val.string = (vp->buf);    /* Point to the temp storage */
            vp->val_len = strlen(values->value[entry->colPos]);
            memmove(vp->buf, values->value[entry->colPos], vp->val_len);
            vp->buf[vp->val_len] = 0;
            break;

        case ASN_PHYSADDR:
            vp->type = ASN_OCTET_STR;
            vp->val.string = vp->buf;
            vp->val_len = convertStr2PhysAddr(vp->buf, values->value[entry->colPos]);
            break;
            
        case ASN_OCTET_STR:
            vp->val.string = vp->buf;
            if(strlen(values->value[entry->colPos]) < 40){
                vp->val_len = strlen(values->value[entry->colPos]);
                memmove(vp->buf, values->value[entry->colPos], vp->val_len);
            }
            else
            {
                vp->val_len = 40;
                memmove(vp->buf, values->value[entry->colPos], 39);
                vp->buf[40] = 0;
            }
            
            break;

        case ASN_OBJECT_ID:
            vp->val.objid = (oid *)(vp->buf);
            vp->val_len = oidToInt(vp->val.objid,
                                   values->value[entry->colPos],
                                   strlen(values->value[entry->colPos]));
            break;

        default:
            vp->type = ASN_NULL;            
            syslog(LOG_INFO, "Bad type in SNMP Msg (%x)\n", vp->type);
            pdu->errstat = SNMP_ERR_WRONGTYPE;
            pdu->errindex = pdu->handled_index + 1;
            break;
    }

    /* Clean up any left over values */
    while(values){
        int i;
        end_value=values;
        values=values->next;
        for(i = 0; i < end_value->count; i++)
            CLEAR_TEXT(end_value->value[i]);
        free(end_value);
    }

    //    free_map_entry(entry);
    return 0;
}


/* TODO: won't work. We have to sent the resposne to a known ip */
void snmpl_send_response(struct snmpl_pdu *pdu)
{
    u_char *pktbuf = NULL, *packet = NULL;
    size_t  pktbuf_len = 0, offset = 0, length = 0;
    int     result;

    if(!pdu)
        return;

    pdu->command = SNMP_MSG_RESPONSE;

    if ( (pktbuf = malloc(SNMPL_RESP_LEN)) == NULL){
        syslog(LOG_INFO, "Can not allocate packet buffer for SNMP response\n");
        return;
    }

    length = pktbuf_len = SNMPL_RESP_LEN;
    packet = pktbuf;

    result = snmpl_build(pdu, &pktbuf, &pktbuf_len, &offset);

    if (result < 0){
        free (packet);
        syslog (LOG_INFO, "SNMP response PDU encoding failure\n");
        return;
    }

    packet = pktbuf + pktbuf_len - offset;
    length = offset;

    /* send the response */
//  result = write(fileno(stdout), packet, length);
    /* TODO: change to STDOUT, but first have to setup the fd in main */
    sendto(STDIN_FILENO, packet, length, 0, pdu->from, sizeof(struct sockaddr_in));
    free (pktbuf);
}

int snmpl_build(struct snmpl_pdu *pdu, u_char **pkt,
        size_t *pkt_len, size_t *offset)
{
    int    rc;
    long   version;
    size_t length, start_offset = *offset;

    if(!pdu)
        return -1;

    length = *pkt_len;

    /* use reverse asn encoding */
    /* rebuild the var-bind */    
    rc = snmpl_rebuild_varBind(pdu, pkt, pkt_len, offset);
    if (!rc)
        return -1;

    /* rebuild community string */
    rc = asn_realloc_rbuild_string(pkt, pkt_len, offset, 1,
            (u_char) (ASN_UNIVERSAL |
                  ASN_PRIMITIVE |
                  ASN_OCTET_STR),
            pdu->community,
            strlen(pdu->community));
    if (!rc)
        return -1;

    /* rebuild version */
    version = pdu->version;
    rc = asn_realloc_rbuild_int(pkt, pkt_len, offset, 1,
            (u_char) (ASN_UNIVERSAL |
                  ASN_PRIMITIVE |
                  ASN_INTEGER),
            (long *) &version,
            sizeof(version));
    if (!rc)
        return -1;

    /* rebuild the final sequence */
    rc = asn_realloc_rbuild_sequence(pkt, pkt_len, offset, 1,
            (u_char) (ASN_SEQUENCE |
                  ASN_CONSTRUCTOR),
            *offset - start_offset);

    if (!rc)
        return -1;

    return 0;
}

int snmpl_rebuild_varBind(struct snmpl_pdu *pdu, u_char **pkt,
            size_t *pkt_len, size_t *offset)
{
#ifndef VPCACHE_SIZE
#define VPCACHE_SIZE 50
#endif
    netsnmp_variable_list *vpcache[VPCACHE_SIZE];
    netsnmp_variable_list *vp, *tmpvp;
    size_t          start_offset = *offset;
    int             i,
            wrapped = 0,
            notdone, final,
            rc = 0;

    if(!pdu)
        return 0;

    for (vp = pdu->pVarBind, i = VPCACHE_SIZE - 1; vp;
                vp = vp->next_variable, i--) {
        if (i < 0) {
            wrapped = notdone = 1;
            i = VPCACHE_SIZE - 1;
        }
        vpcache[i] = vp;
    }
    final = i + 1;

    do {
        for (i = final; i < VPCACHE_SIZE; i++) {
            vp = vpcache[i];
            rc = snmp_realloc_rbuild_var_op(pkt, pkt_len, offset, 1,
                    vp->name, &vp->name_length,
                    vp->type,
                    (u_char *) vp->val.string,
                    vp->val_len);
            if (!rc)
                return 0;
        }

        if (wrapped) {
            notdone = 1;
            for (i = 0; i < final; i++) {
                vp = vpcache[i];
                rc = snmp_realloc_rbuild_var_op(pkt, pkt_len, offset, 1,
                        vp->name, &vp->name_length,
                        vp->type,
                        (u_char *) vp->val.string,
                        vp->val_len);
                if (!rc)
                    return 0;
            }

            if (!final) {
                tmpvp = vpcache[VPCACHE_SIZE - 1];
            } else {
                tmpvp = vpcache[final - 1];
            }
            wrapped = 0;

            for (vp = pdu->pVarBind, i = VPCACHE_SIZE - 1;
                        vp && vp != tmpvp;
                        vp = vp->next_variable, i--) {
                if (i < 0) {
                    wrapped = 1;
                    i = VPCACHE_SIZE - 1;
                }
                vpcache[i] = vp;
            }
            final = i + 1;
        } else {
            notdone = 0;
        }
    } while (notdone);

    /*
     * Save current location and build SEQUENCE tag and length placeholder for
     * variable-bindings sequence (actual length will be inserted later).  
     */

    rc = asn_realloc_rbuild_sequence(pkt, pkt_len, offset, 1,
            (u_char) (ASN_SEQUENCE |
                  ASN_CONSTRUCTOR),
            *offset - start_offset);
    /*
     * Store fields in the PDU preceeding the variable-bindings sequence.  
     */
    /*
     * Error index (getbulk max-repetitions).  
     */
    rc = asn_realloc_rbuild_int(pkt, pkt_len, offset, 1,
            (u_char) (ASN_UNIVERSAL |
                  ASN_PRIMITIVE |
                  ASN_INTEGER),
            &pdu->errindex,
            sizeof(pdu->errindex));
    if (rc == 0) {
        return 0;
    }

    /*
     * Error status (getbulk non-repeaters).  
     */
    rc = asn_realloc_rbuild_int(pkt, pkt_len, offset, 1,
            (u_char) (ASN_UNIVERSAL |
                  ASN_PRIMITIVE |
                  ASN_INTEGER),
            &pdu->errstat,
            sizeof(pdu->errstat));
    if (rc == 0) {
        return 0;
    }

    /*
     * Request ID.  
     */
    rc = asn_realloc_rbuild_int(pkt, pkt_len, offset, 1,
            (u_char) (ASN_UNIVERSAL |
                  ASN_PRIMITIVE |
                  ASN_INTEGER),
            &pdu->req_id,
            sizeof(pdu->req_id));
    if (rc == 0) {
        return 0;
    }

    /*
     * Build the PDU sequence.  
     */
    rc = asn_realloc_rbuild_sequence(pkt, pkt_len, offset, 1,
            (u_char) pdu->command,
            *offset - start_offset);
    return rc;
}

void snmpl_shutdown(int retval)
{
    close(pSnmp->iCfgManSocket);

    syslog(LOG_INFO, "SNMP shutdown reason(%d)\n", retval);    
}

static char *indexToStr(oid *pIndexOid, int indexOidLen)
{
    int i, len;
    char *dest, *pString = malloc(128);
    char tmp[10];

    dest = pString;

    // begins with "." (eg. ".127.0.0.1" or ".1")
    for (i = 0; i < indexOidLen; i++)
    {
        *dest = '.';
        dest++;
        len = sprintf(tmp,"%d", (int)(*(pIndexOid+i)));
        memmove(dest, tmp, len);
        dest += len;
    }

    // null terminated
    *dest = 0;
    return pString;
}

static int oidToInt(oid *dest, char *from, int str_len)
{
    char *pos, *last = from;
    int   len = 0;
    oid  *to = dest;

    if (!from)
        return 0;
    
    while ((pos = strchr(last, '.')))
    {
        if (pos)
        {
            *to = atoi(pos-1);
            to++;
            pos++; // past the '.'
            last = pos;
            len ++;
        }        
    }

    if ((len == 0) && (str_len == 0))
        return 0;
    
    // one more digit
    *to = atoi(last);
    return ((len + 1)*sizeof(oid));
}


