/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2005 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/*!
                                                                          
    \file  	wcfg_nvs.c                                               
    \brief 	PURPOSE: This module implements non volatile storage API functions
                                                                          
*/

/***********************************************************
	Description:
******************************************************************************

  Non volatile storage (NVS) is located in the boot loader environment 
  variables. NVS storage replaces an EEPROM hardware element removed from 
  TNETW 1350 Baseband MAC boards. NVS storage contains the Baseband MAC 
  board specific data: calibration, radio module parameters etc. The Baseband 
  MAC board is identified by TNETW die id and is associated with the NVS data 
  containing the same die id in the internal table. 
  The NVS software module implements NVS file loading from TFTP server and 
  saving the data in the environment variables during initialization phase.

******************************************************************************/

/***********************************************************
	TODO:
*******************************************************/


#define _WCFGLIB_NVS_C

/***********************************************************
	includes
*******************************************************/

#ifndef VXWORKS
	#include <stdio.h>
	#include <stdlib.h>
	#include <netinet/in.h>
	#include <sys/socket.h>
	#include <arpa/inet.h>
#else
	#include "dda.h"
	#include "dda_trace.h"
	#include "dda_ioctl.h"
	#include "dda_wpaauth_ioctl.h"
	#include "ffs.h"
	#include "defrag.h"
	#include "env.h"
#endif
#include "wcfg_storage.h"
#include "wcfglib.h"
#include "wcfg_nvs.h"

/***********************************************************
	Defines
***********************************************************/

#define NVS_NEXT_BIRST_READ(p) ((p) + sizeof(nvsLen_t) + sizeof(nvsStartAdr_t) + (*((nvsLen_t *) (p)))*sizeof(nvsData_t))
#define NVS_SKIP_END_BIRST(p)  ((p) + sizeof(nvsLen_t) + sizeof(nvsStartAdr_t) + sizeof(nvsData_t))
#define NVS_GET_TABLE2_PTR(p)  ((p) + 2*sizeof(UINT16))
#define NVS_GET_UINT16(p) (*(p) + (*((p)+1) << 8))
#define NVS_GET_UINT32(p) (*(p) + (*((p)+1) << 8) + (*((p)+2) << 16) + (*((p)+3) << 24))

#define WCFG_ERR_MSG(str) \
			printf("Func %s line=%d, Error: %s\n",__FUNCTION__,__LINE__,(str))

#define WCFG_ERR_PAR_MSG(str,par) \
			printf("Func %s line=%d, Error: %s %d\n",__FUNCTION__,__LINE__,(str),(par))

/***********************************************************
	Types
***********************************************************/

typedef unsigned char   nvsLen_t;
typedef UINT16			nvsStartAdr_t;
typedef UINT32			nvsData_t;

/***********************************************************
	Static Variables
***********************************************************/


/***********************************************************
	Local Function Prototypes
***********************************************************/

static int readNvsDieId( tiap_param_eeprom_t * pEeprom );

/*******************************************************************************
		PUBLIC FUNCTION IMPLEMENTATION
********************************************************************************/


/******************************************************************************/

/*! \brief 	Reads non volatile storage located in boot loader env variables
	\param 	pointer to input AP configuration structure
	\param 	pointer to EEPROM structure to fill

	\return 0 if successful and 1 in the case of error
*/
/******************************************************************************/

int wcfg_nvs_read_data( const wcfgd *wd, tiap_param_eeprom_t * pEeprom, 
                        int * pNvsDefault )
{
 tiap_param_die_id_t die_id;
 int    nvs_tftp_load_enable = 1;
 char   nvsFileName[50];
 int    nvsImageSize;
 int 	res;
 FILE   *fp=NULL;

 res = tiap_wcfg_get_die_id( wd, &die_id, 0 /* not from storage */);

 printf("%s: TNETW Die ID=%08X %08X\n",__FUNCTION__, die_id.die_id_hi, die_id.die_id_lo);

 if (res)
 {
	WCFG_ERR_MSG("Failed to get TNETW die ID");
	return 1;
 }

 /* First: Reads TNETW NVS parameters from boot loader environment variables*/
#define EEPROM  "/dev/mtdblock/4"
#define EEPROM_OFFSET (60<<10)
 fp=fopen(EEPROM,"r");

 if(fp){
        fseek(fp,EEPROM_OFFSET,0);
        fread(pEeprom,sizeof(tiap_param_eeprom_t),1,fp);
        fclose(fp);

        if(pEeprom->image[0]==0xffffffff)
                wcfg_nvs_set_default_data( pEeprom ); /* enable to use default NVS next reboot */
        
 }else{
        wcfg_nvs_set_default_data( pEeprom ); /* enable to use default NVS next
reboot */
 }

 return 0;
}

/******************************************************************************/

/*! \brief 	Sets default values for NVS data (EEPROM)
	\param 	pointer to eeprom structure to fill defaults
		
*/
/******************************************************************************/

static char defaultEeprom[] = {
 /*  Data    Address */

      0x02   /*  000  */,
      0x11   /*  001  */,
      0x56   /*  002  */,  
      0x04   /*  003  */,
      0x1B   /*  004  */,
      0x06   /*  005  */,
      0x00   /*  006  */,
      0x12   /*  007  */,
      0x00   /*  008  */,
      0x00   /*  009  */,
      0x00   /*  00A  */,
      0x01   /*  00B  */,
      0x09   /*  00C  */,
      0x56   /*  00D  */,
      0x12   /*  00E  */,
      0x00   /*  00F  */,
      0x00   /*  010  */,
      0x00   /*  011  */,
      0x01   /*  012  */,
      0x0D   /*  013  */,
      0x56   /*  014  */,
      0xA9   /*  015  */,
      0x00   /*  016  */,
      0x00   /*  017  */,
      0x00   /*  018  */,
      0x02   /*  019  */,
      0x6D   /*  01A  */,
      0x54   /*  01B  */,
      0xF5   /*  01C  */,
      0x05   /*  01D  */,
      0x55   /*  01E  */,
      0x28   /*  01F  */,
      0x00   /*  020  */,
      0x08   /*  021  */,
      0x00   /*  022  */,
      0x00   /*  023  */,
      0x01   /*  024  */,
      0x39   /*  025  */,
      0x84   /*  026  */,
      0x00   /*  027  */,
      0x01   /*  028  */,
      0x00   /*  029  */,
      0x00   /*  02A  */,
      0x01   /*  02B  */,
      0x01   /*  02C  */,
      0x85   /*  02D  */,
      0x4C   /*  02E  */,
      0x10   /*  02F  */,
      0x68   /*  030  */,
      0x90   /*  031  */,
      0x01   /*  032  */,
      0x05   /*  033  */,
      0x85   /*  034  */,
      0x06   /*  035  */,
      0x00   /*  036  */,
      0x10   /*  037  */,
      0x00   /*  038  */,
      0x01   /*  039  */,
      0x29   /*  03A  */,
      0x85   /*  03B  */,
      0x07   /*  03C  */,
      0x00   /*  03D  */,
      0x00   /*  03E  */,
      0x00   /*  03F  */,
      0x01   /*  040  */,
      0xC1   /*  041  */,
      0x85   /*  042  */,
      0x00   /*  043  */,
      0x00   /*  044  */,
      0x30   /*  045  */,
      0x00   /*  046  */,
      0x01   /*  047  */,
      0xC5   /*  048  */,
      0x85   /*  049  */,
      0x00   /*  04A  */,
      0x00   /*  04B  */,
      0x00   /*  04C  */,
      0x00   /*  04D  */,
      0x01   /*  04E  */,
      0x11   /*  04F  */,
      0x85   /*  050  */,
      0x00   /*  051  */,
      0x00   /*  052  */,
      0xFF   /*  053  */,
      0xFF   /*  054  */,
      0x01   /*  055  */,
      0x15   /*  056  */,
      0x85   /*  057  */,
      0x00   /*  058  */,
      0x00   /*  059  */,
      0xF0   /*  05A  */,
      0xFF   /*  05B  */,
      0x01   /*  05C  */,
      0xA5   /*  05D  */,
      0x85   /*  05E  */,
      0x00   /*  05F  */,
      0x80   /*  060  */,
      0x00   /*  061  */,
      0x00   /*  062  */,
      0x01   /*  063  */,
      0x09   /*  064  */,
      0x85   /*  065  */,
      0x00   /*  066  */,
      0x00   /*  067  */,
      0x80   /*  068  */,
      0x02   /*  069  */,
      0x01   /*  06A  */,
      0x01   /*  06B  */,
      0x0C   /*  06C  */,
      0x03   /*  06D  */,
      0x00   /*  06E  */,
      0x00   /*  06F  */,
      0x00   /*  070  */,
      0x01   /*  071  */,
      0x21   /*  072  */,
      0x84   /*  073  */,
      0x00   /*  074  */,
      0x00   /*  075  */,
      0x00   /*  076  */,
      0x80   /*  077  */,
      0x01   /*  078  */,
      0x81   /*  079  */,
      0x85   /*  07A  */,
      0x1D   /*  07B  */,
      0x00   /*  07C  */,
      0x03   /*  07D  */,
      0x00   /*  07E  */,
      0x01   /*  07F  */,
      0x55   /*  080  */,
      0x09   /*  081  */,
      0x01   /*  082  */,
      0x00   /*  083  */,
      0x00   /*  084  */,
      0x00   /*  085  */,
      0x01   /*  086  */,
      0xE5   /*  087  */,
      0x58   /*  088  */,
      0x02   /*  089  */,
      0x00   /*  08A  */,
      0x00   /*  08B  */,
      0x00   /*  08C  */,
      0x01   /*  08D  */,
      0xF1   /*  08E  */,
      0x58   /*  08F  */,
      0x08   /*  090  */,
      0x00   /*  091  */,
      0x00   /*  092  */,
      0x00   /*  093  */,
      0x01   /*  094  */,
      0xD5   /*  095  */,
      0x58   /*  096  */,
      0x10   /*  097  */,
      0x00   /*  098  */,
      0x00   /*  099  */,
      0x00   /*  09A  */,
      0x01   /*  09B  */,
      0xB1   /*  09C  */,
      0x58   /*  09D  */,
      0x04   /*  09E  */,
      0x00   /*  09F  */,
      0x00   /*  0A0  */,
      0x00   /*  0A1  */,
      0x00   /*  0A2  */,
      0x00   /*  0A3  */,
      0x00   /*  0A4  */,
      0x00   /*  0A5  */,
      0x00   /*  0A6  */,
      0x00   /*  0A7  */,
      0x00   /*  0A8  */,
      0x0C   /*  0A9  */,
      0x00   /*  0AA  */,
      0xC3   /*  0AB  */,
      0x00   /*  0AC  */,
      0xFE   /*  0AD  */,
      0x00   /*  0AE  */,
      0x08   /*  0AF  */,
      0x01   /*  0B0  */,
      0x14   /*  0B1  */,
      0x01   /*  0B2  */,
      0x26   /*  0B3  */,
      0x01   /*  0B4  */,
      0x44   /*  0B5  */,
      0x01   /*  0B6  */,
      0x4D   /*  0B7  */,
      0x01   /*  0B8  */,
      0x4F   /*  0B9  */,
      0x01   /*  0BA  */,
      0x65   /*  0BB  */,
      0x01   /*  0BC  */,
      0x9F   /*  0BD  */,
      0x01   /*  0BE  */,
      0xAF   /*  0BF  */,
      0x01   /*  0C0  */,
      0xB7   /*  0C1  */,
      0x01   /*  0C2  */,
      0x01   /*  0C3  */,
      0x39   /*  0C4  */,
      0x00   /*  0C5  */,
      0x00   /*  0C6  */,
      0x11   /*  0C7  */,
      0x00   /*  0C8  */,
      0x04   /*  0C9  */,
      0x01   /*  0CA  */,
      0x01   /*  0CB  */,
      0x01   /*  0CC  */,
      0x00   /*  0CD  */,
      0x05   /*  0CE  */,
      0x01   /*  0CF  */,
      0x06   /*  0D0  */,
      0x00   /*  0D1  */,
      0x02   /*  0D2  */,
      0x01   /*  0D3  */,
      0x02   /*  0D4  */,
      0x01   /*  0D5  */,
      0x02   /*  0D6  */,
      0x1E   /*  0D7  */,
      0x00   /*  0D8  */,
      0x0A   /*  0D9  */,
      0x00   /*  0DA  */,
      0x02   /*  0DB  */,
      0x05   /*  0DC  */,
      0x02   /*  0DD  */,
      0x04   /*  0DE  */,
      0x11   /*  0DF  */,
      0x22   /*  0E0  */,
      0x44   /*  0E1  */,
      0x03   /*  0E2  */,
      0x06   /*  0E3  */,
      0x10   /*  0E4  */,
      0x20   /*  0E5  */,
      0x30   /*  0E6  */,
      0x31   /*  0E7  */,
      0x32   /*  0E8  */,
      0x40   /*  0E9  */,
      0x04   /*  0EA  */,
      0x09   /*  0EB  */,
      0x54   /*  0EC  */,
      0x49   /*  0ED  */,
      0x20   /*  0EE  */,
      0x41   /*  0EF  */,
      0x43   /*  0F0  */,
      0x58   /*  0F1  */,
      0x31   /*  0F2  */,
      0x30   /*  0F3  */,
      0x30   /*  0F4  */,
      0x05   /*  0F5  */,
      0x07   /*  0F6  */,
      0x54   /*  0F7  */,
      0x49   /*  0F8  */,
      0x20   /*  0F9  */,
      0x54   /*  0FA  */,
      0x65   /*  0FB  */,
      0x73   /*  0FC  */,
      0x74   /*  0FD  */,
      0x01   /*  0FE  */,
      0x08   /*  0FF  */,
      0x00   /*  100  */,
      0x00   /*  101  */,
      0x00   /*  102  */,
      0x00   /*  103  */,
      0x00   /*  104  */,
      0x00   /*  105  */,
      0x00   /*  106  */,
      0x00   /*  107  */,
      0x05   /*  108  */,
      0x02   /*  109  */,
      0x00   /*  10A  */,
      0x00   /*  10B  */,
      0x37   /*  10C  */,
      0x00   /*  10D  */,
      0x6F   /*  10E  */,
      0x00   /*  10F  */,
      0xA7   /*  110  */,
      0x00   /*  111  */,
      0xE5   /*  112  */,
      0x00   /*  113  */,
      0x04   /*  114  */,
      0x04   /*  115  */,
      0x44   /*  116  */,
      0x10   /*  117  */,
      0x22   /*  118  */,
      0x00   /*  119  */,
      0x45   /*  11A  */,
      0x10   /*  11B  */,
      0x05   /*  11C  */,
      0x00   /*  11D  */,
      0x18   /*  11E  */,
      0x5A   /*  11F  */,
      0x40   /*  120  */,
      0x00   /*  121  */,
      0x14   /*  122  */,
      0x5A   /*  123  */,
      0x0F   /*  124  */,
      0x00   /*  125  */,
      0x02   /*  126  */,
      0x0E   /*  127  */,
      0xA0   /*  128  */,
      0x01   /*  129  */,
      0xE4   /*  12A  */,
      0x00   /*  12B  */,
      0x0A   /*  12C  */,
      0x01   /*  12D  */,
      0x3D   /*  12E  */,
      0x03   /*  12F  */,
      0x46   /*  130  */,
      0x00   /*  131  */,
      0xA4   /*  132  */,
      0x01   /*  133  */,
      0x85   /*  134  */,
      0x09   /*  135  */,
      0x80   /*  136  */,
      0x00   /*  137  */,
      0x0E   /*  138  */,
      0x01   /*  139  */,
      0x0A   /*  13A  */,
      0x01   /*  13B  */,
      0x76   /*  13C  */,
      0x04   /*  13D  */,
      0x46   /*  13E  */,
      0x00   /*  13F  */,
      0xEF   /*  140  */,
      0x01   /*  141  */,
      0x85   /*  142  */,
      0x09   /*  143  */,
      0x01   /*  144  */,
      0x07   /*  145  */,
      0x10   /*  146  */,
      0x00   /*  147  */,
      0x00   /*  148  */,
      0x40   /*  149  */,
      0x00   /*  14A  */,
      0x00   /*  14B  */,
      0x01   /*  14C  */,
      0x00   /*  14D  */,
      0x00   /*  14E  */,
      0x05   /*  14F  */,
      0x04   /*  150  */,
      0x00   /*  151  */,
      0x01   /*  152  */,
      0x01   /*  153  */,
      0x00   /*  154  */,
      0x00   /*  155  */,
      0x00   /*  156  */,
      0x00   /*  157  */,
      0xFF   /*  158  */,
      0xFF   /*  159  */,
      0xFF   /*  15A  */,
      0xFF   /*  15B  */,
      0xFD   /*  15C  */,
      0xFD   /*  15D  */,
      0xFD   /*  15E  */,
      0xFD   /*  15F  */,
      0xFB   /*  160  */,
      0xFB   /*  161  */,
      0xFB   /*  162  */,
      0xFB   /*  163  */,
      0xFB   /*  164  */,
      0x0E   /*  165  */,
      0x04   /*  166  */,
      0xFB   /*  167  */,
      0xFB   /*  168  */,
      0xFB   /*  169  */,
      0xFB   /*  16A  */,
      0x00   /*  16B  */,
      0x00   /*  16C  */,
      0x00   /*  16D  */,
      0xFD   /*  16E  */,
      0x00   /*  16F  */,
      0x01   /*  170  */,
      0x02   /*  171  */,
      0xFD   /*  172  */,
      0x00   /*  173  */,
      0x01   /*  174  */,
      0x02   /*  175  */,
      0xFD   /*  176  */,
      0x09   /*  177  */,
      0x09   /*  178  */,
      0x09   /*  179  */,
      0x09   /*  17A  */,
      0x09   /*  17B  */,
      0x09   /*  17C  */,
      0x09   /*  17D  */,
      0x09   /*  17E  */,
      0x09   /*  17F  */,
      0x09   /*  180  */,
      0x09   /*  181  */,
      0x09   /*  182  */,
      0x00   /*  183  */,
      0x02   /*  184  */,
      0x03   /*  185  */,
      0xFD   /*  186  */,
      0x00   /*  187  */,
      0x01   /*  188  */,
      0x02   /*  189  */,
      0xFD   /*  18A  */,
      0x00   /*  18B  */,
      0x00   /*  18C  */,
      0x00   /*  18D  */,
      0xFD   /*  18E  */,
      0xFB   /*  18F  */,
      0xFB   /*  190  */,
      0xFB   /*  191  */,
      0xFB   /*  192  */,
      0xFB   /*  193  */,
      0xFB   /*  194  */,
      0xFB   /*  195  */,
      0xFB   /*  196  */,
      0xFB   /*  197  */,
      0xFB   /*  198  */,
      0xFB   /*  199  */,
      0xFB   /*  19A  */,
      0xFB   /*  19B  */,
      0xFB   /*  19C  */,
      0xFB   /*  19D  */,
      0xFB   /*  19E  */,
      0x0E   /*  19F  */,
      0x01   /*  1A0  */,
      0x00   /*  1A1  */,
      0x00   /*  1A2  */,
      0x00   /*  1A3  */,
      0x00   /*  1A4  */,
      0x00   /*  1A5  */,
      0x00   /*  1A6  */,
      0x00   /*  1A7  */,
      0x00   /*  1A8  */,
      0x00   /*  1A9  */,
      0x00   /*  1AA  */,
      0x00   /*  1AB  */,
      0x00   /*  1AC  */,
      0x00   /*  1AD  */,
      0x00   /*  1AE  */,
      0x03   /*  1AF  */,
      0x02   /*  1B0  */,
      0xA3   /*  1B1  */,
      0x18   /*  1B2  */,
      0xA3   /*  1B3  */,
      0x18   /*  1B4  */,
      0x07   /*  1B5  */,
      0x00   /*  1B6  */,
      0x01   /*  1B7  */,
      0x20   /*  1B8  */,
      0x00   /*  1B9  */,
      0x00   /*  1BA  */,
      0x00   /*  1BB  */,
      0x00   /*  1BC  */,
      0x00   /*  1BD  */,
      0x00   /*  1BE  */,
      0x00   /*  1BF  */,
      0x00   /*  1C0  */,
      0x00   /*  1C1  */,
      0x00   /*  1C2  */,
      0x00   /*  1C3  */,
      0x00   /*  1C4  */,
      0x00   /*  1C5  */,
      0x00   /*  1C6  */,
      0x00   /*  1C7  */,
      0x00   /*  1C8  */,
      0x00   /*  1C9  */,
      0x00   /*  1CA  */,
      0x00   /*  1CB  */,
      0x00   /*  1CC  */,
      0x00   /*  1CD  */,
      0x00   /*  1CE  */,
      0x00   /*  1CF  */,
      0x00   /*  1D0  */,
      0x00   /*  1D1  */,
      0x00   /*  1D2  */,
      0x00   /*  1D3  */,
      0x00   /*  1D4  */,
      0x00   /*  1D5  */,
      0x00   /*  1D6  */,
      0x00   /*  1D7  */,
      0x00   /*  1D8  */,
	};                

void wcfg_nvs_set_default_data( tiap_param_eeprom_t * pEeprom )
{

 memcpy( pEeprom->image, defaultEeprom, sizeof(defaultEeprom) );
 pEeprom->len = sizeof(defaultEeprom);
 pEeprom->die_id.die_id_hi = 0;				/* default DIE id */
 pEeprom->die_id.die_id_lo = 0;

}


/*******************************************************************************
		LOCAL FUNCTION IMPLEMENTATION
********************************************************************************/

/******************************************************************************/

/*! \brief 	Reads die ID from NVS data (table 2)
	\param 	pointer to eeprom structure conataining image and to return die id
	
	\return 0 in case of success, 1 otherwise
	
*/
/******************************************************************************/

static int readNvsDieId( tiap_param_eeprom_t * pEeprom )
{
 unsigned char * p = (unsigned char *) pEeprom->image;
 nvsLen_t		 len, entry;
 nvsStartAdr_t	 tableIndx;

				/* find end of birst read */
 while (*p)
 {
	p = NVS_NEXT_BIRST_READ(p);
 }
 
 p = NVS_SKIP_END_BIRST(p);		/* skip end of birst read */

 p = NVS_GET_TABLE2_PTR( p );		/* get table 2 address */

 tableIndx = NVS_GET_UINT16(p);

 if (tableIndx >= pEeprom->len)
 {
	printf("%s: Error in NVS File, table 2 adr=%x\n", __FUNCTION__, tableIndx );
	return 1;
 }

 p = (char *)(pEeprom->image) + tableIndx;

 entry = *p; p++;
 len   = *p; p++;
 
 if (entry != 1  ||  len != 8)
 {
	printf("%s: Error in DIE ID Table 2 entry=%x len=%x\n", __FUNCTION__, 
			entry, len);
	return 1;
 }

 pEeprom->die_id.die_id_lo = NVS_GET_UINT32(p);
 p += sizeof(UINT32);
 pEeprom->die_id.die_id_hi = NVS_GET_UINT32(p);

 return 0;
}




