/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/


/*
 * WCFG         An implementation of the wcfglib library.
 *		See wcfg_lib companion document.
 *
 *
 * Version:     0.1.0
 * Authors:     Alexandra Aronov, <alexandraa@ti.com>
 *              Felix Rubinstein, <felixr@ti.com>
 *
 */



/*!
 *
 *   \file  wcfg_storage.c
 *   \brief PURPOSE: wcfg library persistent storage manipulations
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <arpa/inet.h>

#include "wcfg_storage.h"
#include "wcfglib.h"

#define ENV_VAR_FILE_MAX_SIZE  4000

int set_dot11mode(struct wcfg_storage_t *, int dot11mode, int idx);

/*!	\brief wcfg_storage_read
  	\param storage - storage to read into from persistent storage
  	\param storage_path - persistent storage path

  	\return zero upon success, otherwise non-zero
  
	read into storage from persistent storage specified by storage_path.
*/
int wcfg_storage_read(struct storage *storage, const char *storage_path)
{
int fd, nread, expected;

	if( (fd = open(storage_path, O_RDONLY)) == -1 ) {
		fprintf(stderr, "Failed to open %s file: %s\n", storage_path,  strerror(errno));
		return 1;
	}

	expected = sizeof(*storage);
	if( (nread = read(fd, (void *)storage, expected)) == -1 ) {
		fprintf(stderr, "Failed to read %s: %s\n", WCFG_MTDBLOCK, strerror(errno));
		close(fd);
		return 1;
	}
	else if( nread != expected ) {
		fprintf(stderr, "%s: could not read all data (%d %d)\n", __func__, nread, expected);
		close(fd);
		return 1;
	}

	close(fd);
	return 0;
}


/*!	\brief wcfg_storage_write
  	\param storage - storage to store in persistent storage
  	\param storage_path - persistent storage path

  	\return zero upon success, otherwise non-zero
  
	store storage in persistent storage specified by storage_path.
*/
int wcfg_storage_write(const struct storage *storage, const char *storage_path)
{
int fd, nwritten, expected;

	if( (fd = open(storage_path, O_WRONLY)) == -1 ) {
		fprintf(stderr, "Failed to open %s: %s\n", storage_path, strerror(errno));
		return 1;
	}

	((struct storage *)storage)->magic = WCFG_MAGIC;
	expected = sizeof(*storage);
	if( (nwritten = write(fd, (const void *)storage, expected)) == -1 ) {
		fprintf(stderr, "Failed to write to %s: %s\n", storage_path, strerror(errno));
		close(fd);
		return 1;
	}
	else if( nwritten != expected ) {
		fprintf(stderr, "%s: could not save all data (%d %d)\n", __func__, nwritten, expected);
		close(fd);
		return 1;
	}

	close(fd);
	return 0;
}

/*!	\brief get_random_byte
  	\param r - random number

  	\return zero upon success, otherwise non-zero
  
	read random number into r, see urandom(5).

	\note: currently unused.
*/
int get_random_byte(__u8 *r)
{
	int fd;

	if( (fd = open("/dev/urandom", O_RDONLY)) == -1 ) {
		fprintf(stderr, "Failed to open /dev/urandom: %s\n", strerror(errno));
		return 1;
	}

	if( read(fd, r, sizeof(*r)) == -1 ) {
		fprintf(stderr, "Failed to read /dev/urandom: %s\n", strerror(errno));
		close(fd);
		return 1;
	}

	close(fd);
	return 0;
}

/*!	\brief read_hwaddr_from_bootloader
  	\param hwaddr - hardware address

  	\return zero upon success, otherwise non-zero
  
	read access point hardware address from boot loader.
*/
int read_hwaddr_from_bootloader(char *hwaddr)
{
	int fd, nread, i;
	char buf[2000], *p;

	if( (fd = open(WCFG_BOOTLOADER, O_RDONLY)) == -1 ) {
		fprintf(stderr, "Failed to open %s file: %s\n", WCFG_BOOTLOADER,  strerror(errno));
		return 1;
	}

	if( (nread = read(fd, (void *)buf, sizeof(buf))) == -1 ) {
		fprintf(stderr, "Failed to read %s: %s\n", WCFG_BOOTLOADER, strerror(errno));
		close(fd);
		return 1;
	}
	close(fd);


	if( !(p = strstr(buf, WCFG_WLAN_HWADDR)) ) {
		fprintf(stderr, "Boot-loader environment variable %s is not set, "
				"cannot configure wireless LAN driver hardware address\n",
				WCFG_WLAN_HWADDR);
		return 1;
	}

	if( !(p = strtok(p, " \t")) ) {
		fprintf(stderr, "Invalid format of %s environment variable\n", WCFG_WLAN_HWADDR);
		return 1;
	}

	for(i=0; i<ETH_ALEN; i++)
	{
		if( !(p = strtok(NULL, " \t:")) ) {
			fprintf(stderr, "Invalid format of %s environment variable\n", WCFG_WLAN_HWADDR);
			return 1;
		}
		sscanf(p, "%hhx", &hwaddr[i]);
	}
	
	return 0;
}

#ifdef MWCODE

#define WHAL_SSID_STRING_LEN              32
#define WEP_KEY_LEN_MAX                         29
#define DOT11_MAX_PSK_PWD_LEN                               63
#define DOT11_MAX_RADIUS_SECRET_LEN                 63

#define PRIVACY_WEP     1
#define PRIVACY_WPAPSK  2
#define PRIVACY_WPA_8021X 3

#define WIFI_AUTH_OPEN  1
#define WIFI_AUTH_SKEY  2
#define WIFI_AUTH_AUTO  3

typedef struct wifi_s {
        /* char ifname[MAX_IFNAME]; */
        unsigned char if_on;
        char ssid[WHAL_SSID_STRING_LEN+1];
        unsigned char channel;
        unsigned char auth_type;
        unsigned char wep_on; /* 0:off 1:WEP 2:WAP-PSK */
        unsigned char wep_bits; /* 64/128 bits */
        unsigned char key_len[4];
        unsigned char def_key;
        unsigned char key[4][WEP_KEY_LEN_MAX+1];
        unsigned char access_control;
        /* 2004.09.21 add for APDK_5.6 */
        char list[TIAP_PARAM_ACCESS_LIST_SIZE][ETH_ALEN];
        unsigned char broadcast_ssid;
        unsigned char dot11_mode;
        unsigned char dot11_isolated;
        /* 2003.11.7 add for WPA-PSK */
        char pskPwd[DOT11_MAX_PSK_PWD_LEN+1];
	/* 2003.11.20 add for 802.1x */
        unsigned char radiusSecretLength;
        char radiusSecret[DOT11_MAX_RADIUS_SECRET_LEN+1];
        unsigned int radiusPort;
        /* 2004.09.22 Modified for APDK 5.6 */
        char radiusServer[65];
        /* 2004.06.09 add region support. */
	/*up to 2 bytes, differentiate africa, Asia*/
        char region[2];
        /* 2004.10.05 Ron add for G+ 4x mode */
        char g_4x;
} wifi_t;

static wifi_t wifi_conf;

#endif /* MWCODE */

/*!	\brief wcfg_storage_setdefaults_ldb
  	\param ldb - local database structure

	set driver configuration defaults and
	read them into local database structure.
*/
void wcfg_storage_setdefaults_ldb(struct wcfg_storage_t *ldb)
{
	if( read_hwaddr_from_bootloader(ldb->hwaddr) )
	{
		memset(ldb->hwaddr, 0, sizeof(ldb->hwaddr));
		/* first four octets belong to TI */
		ldb->hwaddr[0] = 0x0;
		ldb->hwaddr[1] = 0x50;
		ldb->hwaddr[2] = 0xF1;
		ldb->hwaddr[3] = 0x12;
	}

	ldb->gpio_owner = 1;
	
	ldb->ipaddr = inet_addr("192.168.1.1");
	ldb->netmask = inet_addr("255.255.255.0");
	ldb->default_gw = inet_addr("192.168.1.1");
	
	/*standard*/
	ldb->rts = (wifi_conf.g_4x)?4096:2347; 
	ldb->frag = (wifi_conf.g_4x)?4096:2346;
	ldb->retry_long = 4;
	ldb->retry_short = 7;
	ldb->lifetime_tx = 512; ldb->lifetime_rx = 512;
	
	ldb->txpow_a.hr = txpower_a;
	{
	__s8 txpower[TIAP_PARAM_NUM_TXPOW_LEVELS] = {11, 10, 9, 7, 5};
	memcpy((void *)&ldb->txpow_a.txpower, (void *)txpower, sizeof(ldb->txpow_a.txpower));
	}
	ldb->txpow_g.hr = txpower_g;
	{
	__s8 txpower[TIAP_PARAM_NUM_TXPOW_LEVELS] = {19, 18, 16, 15, 14};
	memcpy((void *)&ldb->txpow_g.txpower, (void *)txpower, sizeof(ldb->txpow_g.txpower));
	}
		
	/*private*/

	/*basic*/
	ldb->replev = 0; 
#ifdef MWCODE
        if(wifi_conf.wep_on==PRIVACY_WEP)
                ldb->priv_opt = 1;
        else
                ldb->priv_opt = 0;
        ldb->beacon_period = 100;
        ldb->dtim_period = 2;
        /* dot11 isolation */
        if(wifi_conf.dot11_isolated)
                ldb->bss_bridge_state = TIAP_PARAM_BSS_BR_BLOCK;
        else
                ldb->bss_bridge_state = TIAP_PARAM_BSS_BR_ENABLE;
        ldb->rate_fallback = 0;
        ldb->radio_cal = 0x1;
        ldb->cal_interval = 300;
        ldb->energy_detect = 1;
        /* Hidden SSID */
        ldb->hidden_ssid = 1 - wifi_conf.broadcast_ssid;
#else
	ldb->priv_opt = 0;
	ldb->beacon_period = 200;
	ldb->dtim_period = 2;
	ldb->bss_bridge_state = TIAP_PARAM_BSS_BR_ENABLE;
	ldb->rate_fallback = 0;
	ldb->radio_cal = 0x1;
	ldb->cal_interval = 300;
	ldb->energy_detect = 0;
	ldb->hidden_ssid = 0;
#endif /* MWCODE */
	ldb->multi_domain_cap = 0;
	ldb->spectrum_mgmt = 0;
	ldb->legacy_sta = 0;
	ldb->local_pow_cons = 0;
	ldb->max_txpow_lev = 1;
	

	/*memcpy(ldb->country_str, "FR ", TIAP_PARAM_COUNTRY_STRING_LEN);*/
	/*memcpy(ldb->country_str, "CNI", TIAP_PARAM_COUNTRY_STRING_LEN);*/

	ldb->oper_state = 1;
	ldb->wone_num = 1;
	ldb->wone_timemod = 1;
	ldb->radar_detect = 0;
	ldb->txop_limit = 3000;
	ldb->bridge_mode = 0;

	/*complex*/

	ldb->auth_tbl[0].algo = TIAP_PARAM_MIB_OPEN_SYSTEM;
	ldb->auth_tbl[0].enable = 1;
	ldb->auth_tbl[1].algo = TIAP_PARAM_MIB_SHARED_KEY;
	ldb->auth_tbl[1].enable = 0;
#ifdef MWCODE
        /* setup Auth Type */
        if (wifi_conf.auth_type == WIFI_AUTH_AUTO)
                ldb->auth_tbl[1].enable = 1;
        else if (wifi_conf.auth_type == WIFI_AUTH_SKEY) {
                ldb->auth_tbl[0].algo = TIAP_PARAM_MIB_SHARED_KEY;
                ldb->auth_tbl[0].enable = 1;
                ldb->auth_tbl[1].algo = TIAP_PARAM_MIB_OPEN_SYSTEM;
                ldb->auth_tbl[1].enable = 0;
        }
#endif
	
	ldb->wd_en.cmd = TIAP_PARAM_WD_ENABLE; ldb->wd_en.value.val16 = 1;
	ldb->wd_rxfree.cmd = TIAP_PARAM_WD_RXFREE; ldb->wd_rxfree.value.val16 = 0;
	ldb->wd_beacons.cmd = TIAP_PARAM_WD_BEACONS; ldb->wd_beacons.value.val16 = 3;
	ldb->wd_txstuck.cmd = TIAP_PARAM_WD_TXSTUCK; ldb->wd_txstuck.value.val16 = 5;
	ldb->wd_fcserrs.cmd = TIAP_PARAM_WD_FCSERR; ldb->wd_fcserrs.value.val16 = 1;
	ldb->wd_aci.cmd = TIAP_PARAM_WD_ACI; ldb->wd_aci.value.val8 = 1;
	ldb->wd_nb.cmd = TIAP_PARAM_WD_NB; ldb->wd_nb.value.val8 = 1;
	
	ldb->wd_interf.cmd = TIAP_PARAM_WD_INTERF;

#if( WLAN_ADAPTOR_VERSION == 1150)
	/* When compiled for 1150 disable interference WA in the driver */
	 ldb->wd_interf.value.val8 = 1; /* ACI by HW */
	 ldb->pd_low_thresh 		= 0x0070;
	 ldb->eeprom_less = 1;
#else
	 ldb->wd_interf.value.val8 = 2; /* ACI by Driver */
	 ldb->pd_low_thresh 		= 0x0090;
	 ldb->eeprom_less = 0;
#endif

	 ldb->input_cca.value = 1;
     ldb->tx_cca.value = 1;
     ldb->qualif_cca.value = 3;
     ldb->stomp_rx.value = 2;
     ldb->stomp_tx.value = 0;

	ldb->aci_state_timeout 	= 60; /* seconds */
	ldb->nb_state_timeout 	= 300; /* seconds */
	ldb->aci_rate_thresh 	= 50;
	ldb->aci_event_thresh   = 4;
	ldb->pd_high_thresh		= 0x00D5;
	ldb->medium_busy_thresh = 75; /* percents */
		
	memset(&ldb->access_list, 0, sizeof(ldb->access_list));
#ifdef MWCODE
        if(wifi_conf.access_control) {
                ldb->access_list.cmd = TIAP_PARAM_ACCESS_LST;
                ldb->access_list.enabled = 1;
                memcpy(ldb->access_list.list, wifi_conf.list, TIAP_PARAM_ACCESS_LIST_SIZE * ETH_ALEN);
        } else
#endif
	ldb->access_list.cmd = TIAP_PARAM_ACCESS_RESET;

#ifdef MWCODE
	ldb->_4x_fs.cmd = TIAP_PARAM_4X_FEATSET; ldb->_4x_fs.value = 1;
        ldb->_4x_4x.cmd = TIAP_PARAM_4X_4X; ldb->_4x_4x.value = wifi_conf.g_4x;
        ldb->_4x_cat.cmd = TIAP_PARAM_4X_CAT; ldb->_4x_cat.value = wifi_conf.g_4x;
        ldb->_4x_pb.cmd = TIAP_PARAM_4X_PKTBURST; ldb->_4x_pb.value =  wifi_conf.g_4x;
        ldb->_4x_mm.cmd = TIAP_PARAM_4X_MIXMOD; ldb->_4x_mm.value =  0;
#else
	ldb->_4x_fs.cmd = TIAP_PARAM_4X_FEATSET; ldb->_4x_fs.value = 1;
	ldb->_4x_4x.cmd = TIAP_PARAM_4X_4X; ldb->_4x_4x.value = 0;
	ldb->_4x_cat.cmd = TIAP_PARAM_4X_CAT; ldb->_4x_cat.value = 0;
	ldb->_4x_pb.cmd = TIAP_PARAM_4X_PKTBURST; ldb->_4x_pb.value = 0;
	ldb->_4x_mm.cmd = TIAP_PARAM_4X_MIXMOD; ldb->_4x_mm.value = 0;
#endif		
	{/* multiple ssid section */

		memset(&ldb->multi_ssid, 0, sizeof(ldb->multi_ssid));
		ldb->multi_ssid.secondary_enabled = 0 ;
	
		/*Initialize primary ssid - first in the table*/
#ifdef MWCODE
		strncpy(ldb->multi_ssid.tbl[0].ssid, wifi_conf.ssid, IW_ESSID_MAX_SIZE);
#else
		strncpy(ldb->multi_ssid.tbl[0].ssid, "TI", IW_ESSID_MAX_SIZE); 
#endif /* MWCODE */
		ldb->multi_ssid.tbl[0].ssid[IW_ESSID_MAX_SIZE]='\0';
		ldb->multi_ssid.tbl[0].ssid_type = TIAP_PARAM_SSID_PRIMARY;
		ldb->multi_ssid.tbl[0].sec_profile_index = 0;
	
		/* Initialize internal ssid for WDS - last in the table */
		/* Add non-printable character (ESC) to assure that there will be no the same public ssid */
		
		strncpy(ldb->multi_ssid.tbl[TIAP_PARAM_NUM_MULTIPLE_SSID-1].ssid,TIAP_WDS_SSID, IW_ESSID_MAX_SIZE+1);
		ldb->multi_ssid.tbl[TIAP_PARAM_NUM_MULTIPLE_SSID-1].ssid_type = TIAP_PARAM_SSID_INTERNAL;
		ldb->multi_ssid.tbl[TIAP_PARAM_NUM_MULTIPLE_SSID-1].sec_profile_index=TIAP_PARAM_NUM_MULTIPLE_SSID-1;
	}
		
	ldb->radio.mask = 0;
	ldb->radio.tbl.type = TIAP_PARAM_PWRTBL;
	{
#ifdef TIWLAN_FW_RADIA_CRT
	__s8 param[TIAP_PARAM_MAX_RADIO_PARAM_POW_TBL] = {0,2,3, 0, 0, 1, 2,-1,-1, 0, 0,-3,-3,-3,-3,-5,-5, -5, -5, -5};
#else
	__s8 param[TIAP_PARAM_MAX_RADIO_PARAM_POW_TBL] = {0,2,31,0,-3,-1, 1,-3,-6,-4,-2,-6,-9,-7,-5,-9,-12,-10,-8,-12};
#endif
	memcpy((void *)&ldb->radio.tbl.param, (void *)param, sizeof(ldb->radio.tbl.param));
	}
	
	memset(&ldb->ch_rang_a, 0, sizeof(ldb->ch_rang_a));
	ldb->ch_rang_a.type = TIAP_PARAM_RANG_A;
	memset(&ldb->ch_rang_g, 0, sizeof(ldb->ch_rang_g));
	ldb->ch_rang_g.type = TIAP_PARAM_RANG_G;
	
	memset(&ldb->vhcf, 0, sizeof(ldb->vhcf));

	ldb->priv_regdomain.band = BAND_DEFAULT;
	ldb->priv_regdomain.domain = 0;
        /*
	 *priv_regdomain.domain will decide the cur_regdomain if it is TIAP_PARAM_RD_WORLD
	 *order by ../src/ddc/core/DB/drvDbRegDomain.c:160
	 *under band_b maybe change to ETSI
         */
	ldb->cur_regdomain[0].band = 0;
#ifdef MWCODE
        /* 2004.06.09 add for region support */
	switch (wifi_conf.region[0]) {
		case 'C':			/*Canada */
			memcpy(ldb->country_str, "CA ", TIAP_PARAM_COUNTRY_STRING_LEN);
			ldb->cur_regdomain[0].type = TIAP_PARAM_RD_FCC;
			break;
		case 'U':			/*USA */
			memcpy(ldb->country_str, "US ", TIAP_PARAM_COUNTRY_STRING_LEN);/*give a default value*/
			ldb->cur_regdomain[0].type = TIAP_PARAM_RD_FCC;
			break;
		case 'F':			/*now ui do not support */
			memcpy(ldb->country_str, "FR ", TIAP_PARAM_COUNTRY_STRING_LEN);/*give a default value*/
			ldb->cur_regdomain[0].type = TIAP_PARAM_RD_FRANCE;
			/*ldb->cur_regdomain[0].type = TIAP_PARAM_RD_WORLD; ??*/
			break;
		case 'J':			/*Japan */
			if (wifi_conf.channel == 14) {
				memcpy(ldb->country_str, "JP ", TIAP_PARAM_COUNTRY_STRING_LEN);
				ldb->cur_regdomain[0].type = TIAP_PARAM_RD_MKK;
			} else {
				memcpy(ldb->country_str, "J4 ", TIAP_PARAM_COUNTRY_STRING_LEN);
				ldb->cur_regdomain[0].type = TIAP_PARAM_RD_MKK1;
				break;
			}
			break;
		case 'A':			/*Asia */
			ldb->cur_regdomain[0].type = TIAP_PARAM_RD_WORLD;
			if (wifi_conf.region[1] == 's') {	/*Asia<korea-rok> */
				memcpy(ldb->country_str, "KR ", TIAP_PARAM_COUNTRY_STRING_LEN);
			} else if (wifi_conf.region[1] == 'f') {	/*Africa<SouthAfrica> */
				memcpy(ldb->country_str, "ZA ", TIAP_PARAM_COUNTRY_STRING_LEN);
			} else if (wifi_conf.region[1] == 'u') {	/*Australia<Australia> */
				memcpy(ldb->country_str, "AU ", TIAP_PARAM_COUNTRY_STRING_LEN);
			}
			break;
		case 'E':			/*Europe <Austria> */
			memcpy(ldb->country_str, "AT ", TIAP_PARAM_COUNTRY_STRING_LEN);
			ldb->cur_regdomain[0].type = TIAP_PARAM_RD_WORLD;
			break;
		case 'S':			/*South America <honduras> */
			memcpy(ldb->country_str, "HN ", TIAP_PARAM_COUNTRY_STRING_LEN);
			ldb->cur_regdomain[0].type = TIAP_PARAM_RD_WORLD;
			break;
		case 'I':			/*Israel <Israel> */
			memcpy(ldb->country_str, "IL ", TIAP_PARAM_COUNTRY_STRING_LEN);
			ldb->cur_regdomain[0].type = TIAP_PARAM_RD_ISRAEL;
			break;
		case 'M':			/*Mexico <Mexico> */
			memcpy(ldb->country_str, "MX ", TIAP_PARAM_COUNTRY_STRING_LEN);
			ldb->cur_regdomain[0].type = TIAP_PARAM_RD_FCC;
			break;
		default:
			memcpy(ldb->country_str, "US ", TIAP_PARAM_COUNTRY_STRING_LEN);/*give a default value*/
			ldb->cur_regdomain[0].type = TIAP_PARAM_RD_ETSI;
			break;
	}
#endif

	ldb->cur_regdomain[1].band = 1;
	ldb->cur_regdomain[1].type = 0x50;

	/*wone index 0*/
	ldb->ch[0].index = 0;
	ldb->ch[0].type = wone_ch;
#ifdef MWCODE
        ldb->ch[0].elem.ch = wifi_conf.channel;
#else
	ldb->ch[0].elem.ch = TIAP_PARAM_DEF_CHANNEL;
#endif	

	ldb->band[0].index = 0;
	ldb->band[0].type = wone_band;
	ldb->band[0].elem.band = 0;
	
	ldb->rate_adapt[0].index = 0;
	ldb->rate_adapt[0].type = wone_rate_adapt;
	ldb->rate_adapt[0].elem.rate_adapt = 1;

	ldb->erp_ti[0].index = 0;
	ldb->erp_ti[0].type = wone_erp_ti; 
	ldb->erp_ti[0].elem.erp_ti = wifi_conf.g_4x; 

	ldb->dot11_mode[0].index = 0;
	ldb->dot11_mode[0].type = wone_dot11_mode;
	ldb->dot11_mode[0].elem.dot11_mode = 0;

	ldb->cwmin_mode[0].index = 0; 
	ldb->cwmin_mode[0].type = wone_cwmin_mode; 
	ldb->cwmin_mode[0].elem.cwmin_mode = 2;

	ldb->usage_time[0].index = 0;
	ldb->usage_time[0].type = wone_usage_time;
	ldb->usage_time[0].elem.usage_time = 20;


#ifdef MWCODE
        set_dot11mode(ldb, wifi_conf.dot11_mode, 0);
#else
	set_dot11mode(ldb, TIAP_PARAM_D11MODE_MIXED, 0);
#endif
	
	/*wone index 1*/
	ldb->ch[1].index = 1;
	ldb->ch[1].type = wone_ch;
	ldb->ch[1].elem.ch = TIAP_PARAM_DEF_A_CHANNEL;
	
	ldb->band[1].index = 1;
	ldb->band[1].type = wone_band;
	ldb->band[1].elem.band = 1;

	ldb->rate_adapt[1].index = 1;
	ldb->rate_adapt[1].type = wone_rate_adapt;
	ldb->rate_adapt[1].elem.rate_adapt = 1;

	ldb->erp_ti[1].index = 1;
	ldb->erp_ti[1].type = wone_erp_ti; 
	ldb->erp_ti[1].elem.erp_ti = wifi_conf.g_4x; 

	ldb->dot11_mode[1].index = 1;
	ldb->dot11_mode[1].type = wone_dot11_mode;
	ldb->dot11_mode[1].elem.dot11_mode = 4;

	ldb->cwmin_mode[1].index = 1;
	ldb->cwmin_mode[1].type = wone_cwmin_mode; 
	ldb->cwmin_mode[1].elem.cwmin_mode = 2; 

	ldb->usage_time[1].index = 1;
	ldb->usage_time[1].type = wone_usage_time;
	ldb->usage_time[1].elem.usage_time = 20;

	set_dot11mode(ldb, TIAP_PARAM_D11MODE_A, 1);

	wcfg_storage_setdefaults_ldb_qos(ldb);

}

/*!	\brief wcfg_storage_setdefaults_ldb_qos
  	\param ldb - local db

	set wme defaults.
*/
void wcfg_storage_setdefaults_ldb_qos(struct wcfg_storage_t *ldb) {
	/* WME */
	ldb->qos.enabled = 0;
	ldb->qos.classifierMode = TIAP_PARAM_QOS_CLASSIFIER_MODE_DIFFSERV; /* Diffserv */

	/* AC for STA */
	/* BK */
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_STA_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_BK].cwMin = 15;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_STA_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_BK].cwMax = 1023;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_STA_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_BK].aifsn = 7;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_STA_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_BK].shortRetryLimit = 7;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_STA_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_BK].longRetryLimit = 2;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_STA_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_BK].ackPolicy = TIAP_PARAM_QOS_ACK_POLICY_ACK_IMMEDIATE;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_STA_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_BK].txopLimit11b = 0;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_STA_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_BK].txopLimit11g = 0;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_STA_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_BK].txMaxLifetime = 512000;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_STA_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_BK].txOpContinuation = 1;
	/* BE */
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_STA_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_BE].cwMin = 15;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_STA_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_BE].cwMax = 1023;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_STA_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_BE].aifsn = 3;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_STA_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_BE].shortRetryLimit = 3;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_STA_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_BE].longRetryLimit = 2;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_STA_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_BE].ackPolicy = TIAP_PARAM_QOS_ACK_POLICY_ACK_IMMEDIATE;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_STA_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_BE].txopLimit11b = 0;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_STA_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_BE].txopLimit11g = 0;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_STA_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_BE].txMaxLifetime = 512000;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_STA_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_BE].txOpContinuation = 1;
	/* VO */
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_STA_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_VI].cwMin = 7;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_STA_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_VI].cwMax = 15;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_STA_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_VI].aifsn = 2;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_STA_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_VI].shortRetryLimit = 2;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_STA_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_VI].longRetryLimit = 2;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_STA_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_VI].ackPolicy = TIAP_PARAM_QOS_ACK_POLICY_ACK_IMMEDIATE;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_STA_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_VI].txopLimit11b = 6016; /* micro seconds */
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_STA_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_VI].txopLimit11g = 3008;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_STA_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_VI].txMaxLifetime = 512000;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_STA_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_VI].txOpContinuation = 1;
	/* VI */
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_STA_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_VO].cwMin = 3;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_STA_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_VO].cwMax = 7;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_STA_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_VO].aifsn = 2;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_STA_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_VO].shortRetryLimit = 2;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_STA_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_VO].longRetryLimit = 2;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_STA_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_VO].ackPolicy = TIAP_PARAM_QOS_ACK_POLICY_ACK_IMMEDIATE;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_STA_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_VO].txopLimit11b = 3264; /* micro seconds */
 	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_STA_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_VO].txopLimit11g = 1504;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_STA_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_VO].txMaxLifetime = 512000;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_STA_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_VO].txOpContinuation = 1;

	/* AC for AP */
	/* BK */
	/******/
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_AP_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_BK].cwMin = 15;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_AP_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_BK].cwMax = 1023;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_AP_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_BK].aifsn = 7;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_AP_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_BK].shortRetryLimit = 7;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_AP_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_BK].longRetryLimit = 2;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_AP_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_BK].ackPolicy = TIAP_PARAM_QOS_ACK_POLICY_ACK_IMMEDIATE;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_AP_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_BK].txopLimit11b = 0;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_AP_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_BK].txopLimit11g = 0;
	/* Tx MSDU life time should be in uSec */
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_AP_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_BK].txMaxLifetime = 5*(ldb->dtim_period*ldb->beacon_period)*1000;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_AP_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_BK].txOpContinuation = 1;

	/* BE */
	/******/
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_AP_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_BE].cwMin = 15;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_AP_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_BE].cwMax = 63;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_AP_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_BE].aifsn = 3;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_AP_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_BE].shortRetryLimit = 3;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_AP_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_BE].longRetryLimit = 2;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_AP_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_BE].ackPolicy = TIAP_PARAM_QOS_ACK_POLICY_ACK_IMMEDIATE;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_AP_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_BE].txopLimit11b = 0;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_AP_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_BE].txopLimit11g = 0;
	/* Tx MSDU life time should be in uSec */
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_AP_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_BE].txMaxLifetime = 5*(ldb->dtim_period*ldb->beacon_period)*1000;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_AP_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_BE].txOpContinuation = 1;

	/* VO */
	/******/
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_AP_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_VI].cwMin = 7;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_AP_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_VI].cwMax = 15;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_AP_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_VI].aifsn = 1;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_AP_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_VI].shortRetryLimit = 2;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_AP_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_VI].longRetryLimit = 2;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_AP_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_VI].ackPolicy = TIAP_PARAM_QOS_ACK_POLICY_ACK_IMMEDIATE;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_AP_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_VI].txopLimit11b = 6016; /* micro seconds */
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_AP_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_VI].txopLimit11g = 3008;
	/* Tx MSDU life time should be in uSec */
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_AP_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_VI].txMaxLifetime = 200000;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_AP_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_VI].txOpContinuation = 1;

	/* VI */
	/******/
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_AP_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_VO].cwMin = 3;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_AP_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_VO].cwMax = 7;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_AP_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_VO].aifsn = 1;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_AP_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_VO].shortRetryLimit = 2;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_AP_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_VO].longRetryLimit = 2;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_AP_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_VO].ackPolicy = TIAP_PARAM_QOS_ACK_POLICY_ACK_IMMEDIATE;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_AP_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_VO].txopLimit11b = 3264; /* micro seconds */
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_AP_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_VO].txopLimit11g = 1504;
	/* Tx MSDU life time should be in uSec */
    ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_AP_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_VO].txMaxLifetime = 100000;
	ldb->qos.edcf_info[TIAP_PARAM_QOS_AC_AP_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_VO].txOpContinuation = 1;

	ldb->qos.diffserv.numOfCP = 8;
	memset(ldb->qos.diffserv.map, TIAP_PARAM_QOS_CLASSIFIER_AC_BE, TIAP_PARAM_QOS_MAX_CP);
	ldb->qos.diffserv.map[0] = 0;
	ldb->qos.diffserv.map[8] = 1;
	ldb->qos.diffserv.map[16] = 2;
	ldb->qos.diffserv.map[24] = 3;
	ldb->qos.diffserv.map[32] = 4;
	ldb->qos.diffserv.map[40] = 5;
	ldb->qos.diffserv.map[48] = 6;
	ldb->qos.diffserv.map[56] = 7;

  /* WDS common */

	ldb->wdsEnable = 0; /* Disable WDS by default */
}



/*!	\brief wcfg_storage_setdefaults_debug
  	\param debug - debug configuration structure

	store defaults from debug configuration structure.
*/
void wcfg_storage_setdefaults_debug(struct tiap_debug_cfg_t * debug ) {
	struct in_addr in_addr;
	
	inet_aton("192.168.1.42", &in_addr);
	debug->debug_server_ip_addr = in_addr.s_addr;

	debug->automatic_fw_load_enabled = 0;
	debug->jtag_enabled = 0;

	debug->wpa_report_sev = 0;
	debug->wpa_report_mod = 0;
	debug->wds_report_sev = 0;
	debug->wds_report_mod = 0;
}

/*!	\brief wcfg_storage_setdefaults_sec_profile
  	\param profile - security profile

	set given security profile defaults.
*/
void wcfg_storage_setdefaults_sec_profile(tiap_sec_profile_t *profile)
{
	int i;

	profile->is_active = 0;
	profile->sec.interval = 3600 * 1000;
#ifdef MWCODE
        profile->sec.wep_def_key_id = wifi_conf.def_key-1;
#else
	profile->sec.wep_def_key_id = 0;
#endif

	for (i=0; i<TIAP_WPA_DEFS_KEY_NAL_LEN; i++)
	{
#ifdef MWCODE
                if((wifi_conf.wep_on == PRIVACY_WEP) && (wifi_conf.key_len[i] >
4))
                        profile->sec.keys[i].len = wifi_conf.key_len[i];
                memset(profile->sec.keys[i].key, 0 , TIAP_WPA_WEP_KEY_LEN_MAX);                 if((wifi_conf.wep_on == PRIVACY_WEP) && (wifi_conf.key_len[i] >
4))
                        memcpy(profile->sec.keys[i].key, wifi_conf.key[i], wifi_conf.key_len[i]);
#else
		profile->sec.keys[i].len = TIAP_WPA_WEP_40_KEY_LEN;
		memset(profile->sec.keys[i].key, 0 , TIAP_WPA_WEP_KEY_LEN_MAX);
#endif
	}


	profile->sec.type = TIAP_PARAM_SECURITY_WEP; 

	profile->sec.wep_auth_type = TIAP_WPA_WEPAUTH_OPEN;
	profile->sec.wep_privacy = 0;

	memcpy(profile->radius.secret, "RADIUS-SECRET",TIAP_WPA_MAX_RADIUS_SECRET_LEN);
	profile->radius.secret_len = strlen("RADIUS-SECRET");
	profile->radius.pre_auth_flag = 0;
	profile->radius.port = 1812;
	
	profile->radius.serv_ip_addr = inet_addr("192.168.100.3");
	profile->psk.pass_phrase_mode = TIAP_WPA_PSK_PARAM_NONE;
	profile->psk.psk_passwd_len = TIAP_WPA_MIN_PSK_PWD_LEN;
	memcpy(profile->psk.psk_passwd, "PSK12345", TIAP_WPA_MIN_PSK_PWD_LEN);

#ifdef MWCODE
        if((wifi_conf.wep_on == PRIVACY_WPAPSK) && (strlen(wifi_conf.pskPwd) > 7)) {
                profile->sec.type = TIAP_PARAM_SECURITY_WPA;
                profile->psk.pass_phrase_mode = TIAP_WPA_PSK_PARAM_ASCII;
                profile->psk.psk_passwd_len = strlen(wifi_conf.pskPwd);
                memcpy(profile->psk.psk_passwd, wifi_conf.pskPwd, strlen(wifi_conf.pskPwd));
        }

        if(wifi_conf.wep_on == PRIVACY_WPA_8021X) {
                profile->sec.type = TIAP_PARAM_SECURITY_WPA | TIAP_PARAM_SECURITY_802_1X;
                memcpy(profile->radius.secret, wifi_conf.radiusSecret,TIAP_WPA_MAX_RADIUS_SECRET_LEN);
                profile->radius.secret_len = strlen(wifi_conf.radiusSecret);
                profile->radius.port = wifi_conf.radiusPort;
                profile->radius.serv_ip_addr = inet_addr(wifi_conf.radiusServer);
        }
#endif
    fprintf(stderr,"wifi_conf.wep_on = %d\n", wifi_conf.wep_on);

	/* 2005.11.09 add for APDK_6.2.0.20 */
        switch (wifi_conf.wep_on) {
            case PRIVACY_WEP:
                if(wifi_conf.wep_bits>64)
                    profile->sec.default_cipher = TIAP_WPA_CIPHER_WEP_104;
                else
                    profile->sec.default_cipher = TIAP_WPA_CIPHER_WEP_40;
                break;
		case PRIVACY_WPAPSK:
        case PRIVACY_WPA_8021X:
            profile->sec.default_cipher = TIAP_WPA_CIPHER_TKIP;
            break;
        default:
            profile->sec.default_cipher = TIAP_WPA_CIPHER_NON;
        }
	
}

/*!	\brief wcfg_storage_setdefaults_wpa
  	\param wpa - security configuration structure

	set security defaults.
*/
void wcfg_storage_setdefaults_wpa(tiap_wpa_auth_t *wpa)
{
	int priv_index;

	memset(wpa,0,sizeof(*wpa));

	for(priv_index = 0;priv_index < TIAP_MAX_NUM_OF_SUPPORTED_PROFILES; priv_index++)
	{
		wcfg_storage_setdefaults_sec_profile(&wpa->profile[priv_index]);
	}
	
	/* Enable first profile - initially it is the one corresponding to primary ssid */
	wpa->profile[0].is_active = 1;

	/* Initialize hidden security profile (the last one) for WDS with hardcoded configuration */
	{
		tiap_sec_profile_t *hidden_profile = &(wpa->profile[TIAP_MAX_NUM_OF_SUPPORTED_PROFILES-1]);
	
		hidden_profile->is_active = 1;
		hidden_profile->sec.type = TIAP_PARAM_SECURITY_WPA2;
		hidden_profile->sec.interval = 0; /*no broadcast in WDS*/
		hidden_profile->psk.pass_phrase_mode = TIAP_WPA_PSK_PARAM_ASCII;
		hidden_profile->psk.psk_passwd_len = TIAP_WPA_MIN_PSK_PWD_LEN;
		memcpy(hidden_profile->psk.psk_passwd, "PSK12345", TIAP_WPA_MIN_PSK_PWD_LEN);
		hidden_profile->sec.default_cipher = TIAP_WPA_CIPHER_CCMP;
	}

}


void wcfg_storage_setdefaults_wds(wdsStorageCfg_s * wdsCfg)
{
    char unusedMac[MAC_ADDR_LEN] = {0x00,0x00,0x00,0x00,0x00,0x00};

    wdsCfg->autoCfg = 0;
    wdsCfg->chSelectMode = 0;
    wdsCfg->hop = 0;
    wdsCfg->crudeMode = 0;
    strncpy(wdsCfg->name, "WDS_TI", WDS_NAME_MAX_LEN);
    wdsCfg->privacy = 0;
    strncpy(wdsCfg->sharedSecret, "Secret", WDS_SHARED_SECRET_STR_LEN);
    wdsCfg->wirelessIfMode = 1;
    memcpy(wdsCfg->downlinkMac0, unusedMac, ETH_ALEN);
    memcpy(wdsCfg->downlinkMac1, unusedMac, ETH_ALEN);
    memcpy(wdsCfg->downlinkMac2, unusedMac, ETH_ALEN);
    memcpy(wdsCfg->downlinkMac3, unusedMac, ETH_ALEN);
    memcpy(wdsCfg->uplinkMac, unusedMac, ETH_ALEN);
}
/*!	\brief wcfg_storage_setdefaults
  	\param stor - storage structure
	set driver configuration defaults.
*/
void wcfg_storage_setdefaults(struct storage *stor)
{
#ifdef MWCODE
        int fd;
        fd=open("/var/tmp/wifi.bin", O_RDONLY);
        read(fd, &wifi_conf, sizeof(wifi_t));
        close(fd);
#endif /* MWCODE */
	wcfg_storage_setdefaults_wpa(&stor->wpa);
	wcfg_storage_setdefaults_ldb(&stor->ldb);
	wcfg_storage_setdefaults_debug(&stor->debug);
	wcfg_storage_setdefaults_wds(&stor->wds);
}

/******************************************************************************/

/*! \brief 	Reads TNETW NVS (non volatile storage) parameters from boot loader
			environment variables

	\param 	pointer to TFTP enable status to return the NVS value
	\param 	pointer to default NVS enable status to return the NVS value
	\param 	pointer to eeprom structure to fill EEPROM image and die id

	\return 0 if success, 1 in the case of error
	
*/
/******************************************************************************/


int wcfg_storage_readNvsData(int * pNvsTftpLoadEnable, int * pNvsDefault,
                             tiap_param_eeprom_t * pEeprom)
{
 char buf[ ENV_VAR_FILE_MAX_SIZE ], * p;
 char nvsEnvName[50];
 int  fd, nread, i,j,k,len;

 *pNvsTftpLoadEnable = 0;
 *pNvsDefault = 0;
 
            /* Read all env variables to the buffer */
 if( (fd = open(WCFG_BOOTLOADER, O_RDONLY)) == -1 ) 
 {
     fprintf(stderr, "%s: Failed to open %s file, err:%s\n", __FUNCTION__,
             WCFG_BOOTLOADER,  strerror(errno));
     return 1;
 }

 if( (nread = read(fd, (void *)buf, sizeof(buf))) == -1 ) 
 {
     fprintf(stderr, "%s: Failed to read %s, err:%s\n", __FUNCTION__, 
             WCFG_BOOTLOADER, strerror(errno));
     close(fd);
     return 1;
 }
 close(fd);

 if( (p = strstr(buf, NVS_DEFAULT_ENV_VAR_NAME)) )   /* read default NVS enable parameter */
 {
     if( (p = strstr(p, "\t")) )
     {
         if (sscanf( p,"%d", pNvsDefault) != 1)
         {
             fprintf(stderr,"%s: Error format var=%s value=%s\n", __FUNCTION__, 
                    NVS_DEFAULT_ENV_VAR_NAME, p);
             *pNvsDefault = 0;
         }
         else
         {
             if (*pNvsDefault) 
             {
                 return 0; /* Use default NVS */
             }
         }
     }
     else
     {
         fprintf(stderr, "%s: Error: invalid format of %s environment variable\n", __FUNCTION__,
                 NVS_DEFAULT_ENV_VAR_NAME);
     }
 }


 if( !(p = strstr(buf, NVS_TFTP_LOAD_ENV_VAR_NAME)) )   /* read TFTP enable parameter */
 {
     fprintf(stderr, "%s: Error: Boot-loader environment variable %s is not set\n", __FUNCTION__,
             NVS_TFTP_LOAD_ENV_VAR_NAME);
     return 1;
 }

 if( !(p = strstr(p, "\t")) )
 {
     fprintf(stderr, "%s: Error: invalid format of %s environment variable\n", __FUNCTION__,
             NVS_TFTP_LOAD_ENV_VAR_NAME);
     return 1;
 }

 if (sscanf( p,"%d", pNvsTftpLoadEnable) != 1)
 {
	 fprintf(stderr,"%s: Error var=%s value=%s\n", __FUNCTION__, 
			NVS_TFTP_LOAD_ENV_VAR_NAME, p);
	 return 1;
 }

 if (*pNvsTftpLoadEnable) 
 {
     return 0; /* TFTP downloading is enabled */
 }
 							   /* Reead eeprom image from env var */
 
 for (i=0, len=0; i<MAX_EEPROM_ENV_CHUNKS; i++) 
 {
	 snprintf( nvsEnvName, sizeof(nvsEnvName)-1, "%s%d", NVS_ENV_VAR_NAME, i);

     if( !(p = strstr(buf, nvsEnvName)) )
     {
		 if( !i )
		 {
             fprintf(stderr, "%s: Boot-loader environment variable %s is not set\n", __FUNCTION__,
                     nvsEnvName);
             return 1;
		 }
		 else break;
     }

     if( !(p = strstr(p, "\t")) ) 
     {
         fprintf(stderr, "%s: Invalid format of %s environment variable\n", __FUNCTION__,
                 nvsEnvName);
         return 1;
     }
      
	 for(k=0, j=0, ++p; k<NVS_ENV_BLOCK_SIZE*2; k++, j+=2)
	 {
		 char c[3];
		 int  value;

		 c[0] = p[j];
		 c[1] = p[j+1];
		 c[2] = 0;

		 if( sscanf(c, "%x", &value) != 1)
			 break;
		 pEeprom->image[ len++] = (char) value;
	 }
 }
 pEeprom->len = len;

 printf("TNETW NVS length=%d\n", pEeprom->len);

 return !(pEeprom->len);
}

/******************************************************************************/

/*! \brief 	Writes Eeprom data  and TFTP load enable/disable parameter 
            to non volatile storage located in boot loader environment variables

	\param 	TFTP load enable/disable parameter
	\param 	pointer to input Eeprom

	\return 0 if success, 1 in the case of error
		
*/
/******************************************************************************/

int wcfg_storage_writeNvsData( int TftpLoadEnable, tiap_param_eeprom_t * pEeprom)
{
 char envStr[ 500 ];
 char * p;
 int  i, j, k;
 int  fd;

 if( (fd = open(WCFG_BOOTLOADER, O_WRONLY)) == -1 ) 
 {
     fprintf(stderr, "%s: Failed to open file %s, err:%s\n", __FUNCTION__,
             WCFG_BOOTLOADER,  strerror(errno));
     return 1;
 }

 sprintf( envStr, "%s\t%d", NVS_TFTP_LOAD_ENV_VAR_NAME, TftpLoadEnable );

 if( write( fd, (void *)envStr, strlen(envStr) + 1) == -1 ) 
 {
     fprintf(stderr, "%s: Failed to write envVar=%s file %s, err:%s\n", __FUNCTION__,
             envStr, WCFG_BOOTLOADER, strerror(errno));
     close(fd);
     return 1;
 }
									/* Delete Eeprom env variables */
 for(i=0; i < MAX_EEPROM_ENV_CHUNKS; i++) 
 {
     sprintf( envStr, "%s%d", NVS_ENV_VAR_NAME, i );

     if( write( fd, (void *)envStr, strlen(envStr) + 1) == -1 ) 
     {
         fprintf(stderr, "%s: Failed to write envVar=%s file %s, err:%s\n", __FUNCTION__,
                 envStr, WCFG_BOOTLOADER, strerror(errno));
         close(fd);
         return 1;
     }
 }
									/* Save Eeprom env variables */
 for(i=0, j=0; i < MAX_EEPROM_ENV_CHUNKS  &&  j < pEeprom->len; i++) 
 {
     p = envStr;
	 p += sprintf( p, "%s%d\t", NVS_ENV_VAR_NAME, i);

	 for( k=0; k < NVS_ENV_BLOCK_SIZE  &&  j < pEeprom->len; k++ )
	 {
		 p += sprintf( p, "%02X", (unsigned char) pEeprom->image[j++] );
	 }

     if( write( fd, (void *)envStr, strlen(envStr) + 1) == -1 ) 
     {
         fprintf(stderr, "%s: Failed to write envVar=%s file %s, err:%s\n", __FUNCTION__,
                 envStr, WCFG_BOOTLOADER, strerror(errno));
         close(fd);
         return 1;
     }     
 }

 close(fd);

 return 0;
}


/******************************************************************************/

/*! \brief 	Reads server IP address from boot loader environment variable
	\param 	pointer to IP address string

	\return 0 if operation is successful or 1 if environment variables is not set
*/
/******************************************************************************/

int wcfg_storage_getServerIpAdr(char * pIpDotAdr)
{
    char buf[ ENV_VAR_FILE_MAX_SIZE ], * p;
    int  fd, nread;

               /* Read all env variables to the buffer */
    if( (fd = open(WCFG_BOOTLOADER, O_RDONLY)) == -1 ) 
    {
        fprintf(stderr, "%s: Failed to open %s file, err:%s\n", __FUNCTION__,
                WCFG_BOOTLOADER,  strerror(errno));
        return 1;
    }

    if( (nread = read(fd, (void *)buf, sizeof(buf))) == -1 ) 
    {
        fprintf(stderr, "%s: Failed to read %s, err:%s\n", __FUNCTION__, 
                WCFG_BOOTLOADER, strerror(errno));
        close(fd);
        return 1;
    }
    close(fd);

    if( !(p = strstr(buf, WCFG_WLAN_SVR_IPDDR)) )   /* read TFTP enable parameter */
    {
        fprintf(stderr, "%s: Error: boot env var %s is not set!\n", __FUNCTION__, 
                WCFG_WLAN_SVR_IPDDR );
        return 1;
    }

    if( !(p = strstr(p, "\t")) )
    {
        fprintf(stderr, "%s: Error: invalid format of %s environment variable\n", __FUNCTION__,
                WCFG_WLAN_SVR_IPDDR);
        return 1;
    }

    strncpy( pIpDotAdr, ++p, INET_ADDRSTRLEN + 1 );

    return 0;
}



/******************************************************************************/

/*! \brief 	Sets env variable enabling/disabling to use default NVS
	\param 	enable (1)/ disable (0) 

	\return 0 if operation is successful or 1 if environment variables is not set
*/
/******************************************************************************/

int wcfg_storage_setDefaultNvs(int enableDefaultNvs)
{
 char envStr[ 100 ];
 int  fd;

 if( (fd = open(WCFG_BOOTLOADER, O_WRONLY)) == -1 ) 
 {
     fprintf(stderr, "%s: Failed to open file %s, err:%s\n", __FUNCTION__,
             WCFG_BOOTLOADER,  strerror(errno));
     return 1;
 }

 sprintf( envStr, "%s\t%d", NVS_DEFAULT_ENV_VAR_NAME, enableDefaultNvs );

 if( write( fd, (void *)envStr, strlen(envStr) + 1) == -1 ) 
 {
     fprintf(stderr, "%s: Failed to write envVar=%s file %s, err:%s\n", __FUNCTION__,
             envStr, WCFG_BOOTLOADER, strerror(errno));
     close(fd);
     return 1;
 }

 close(fd);

 return 0;
}


