/*
 * WCFG         An implementation of the wcfglib library.
 *		See wcfg_lib companion document.
 *
 *
 * Version:     0.1.0
 *
 * Authors:     Alexandra Aronov, <alexandraa@ti.com>
 *              Felix Rubinstein, <felixr@ti.com>
 *
 */



#include <stdio.h>
#include <unistd.h>
#include <libgen.h>
#include <signal.h>
#include <string.h>
#include <errno.h>
#include <sys/wait.h>
#include <stdarg.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/un.h>
#include <linux/route.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <linux/if_bridge.h>

#include "iwlib.h"
#include "wcfglib.h"
#include "dda_wext_ioctl.h"
#include "dda_wpaauth_ioctl.h"
#include "wcfg_ioctl.h"
#include "wcfg_net.h"
#include "debug_report.h"
#include "wcfg_nvs.h"


#include "pform_types_user.h"
#include "Wds/dda_wds.h"

static void set_replev_ldb(wcfgd *wd, __u8 replev);
static void unset_replev_ldb(wcfgd *wd, __u8 replev);
static void disable_replev_ldb(wcfgd *wd);
static int set_radar_detect(wcfgd *wd, __u8 val);
int tiap_wcfg_waitForProcessExit(pid_t pid);


int tftp_get_file(const char * filename, const char * addr, const char * dir) {
	char cmd_str[128];
	struct stat s;

	printf("TFTP: Downloading %s file from %s\n", filename, addr);

	snprintf(cmd_str, 128, "cd %s && tftp -g -r %s %s ", dir, filename, addr);

	puts(cmd_str);

	if (system(cmd_str) == -1) {
		printf("TFTP: Download failed");
		return 1;
	}

	snprintf(cmd_str, 128, "%s/%s", dir, filename);
	if (stat(cmd_str, &s) != 0 || s.st_size <= 0) {
		printf("TFTP: File does not exist or has 0 size. Download probably failed !\n");
		return 1;
	}

	return 0;
}

static int write_pid(pid_t pid, const char * filename) {
	int fd;
	char buf[30];

	if( (fd = open(filename, O_CREAT|O_WRONLY|O_TRUNC)) == -1 ) {
		perror("open: ");
		return 1;
	}

	sprintf(buf, "%d\n", pid);

	if( write(fd, buf, strlen(buf)) == -1 ) {
		perror("write: ");
		close(fd);
		return 1;
	}

	close(fd);
	
	return 0;
}

static int daemonize(void)
{
	/*
	int nullfd;

	fflush(NULL);
	if( (nullfd=open("/dev/null", O_RDWR)) == -1 ) {
		perror("open: ");
		return 1;
	}

	if (dup2(nullfd,0) < 0 || dup2(nullfd,1) < 0 || dup2(nullfd,2) < 0) {
		fprintf(stderr, "wcfglib: Failed to duplicate file descriptors, reason: %s\n",
				strerror(errno));
		close(nullfd);
		return 1;
	}

	close(nullfd);
	*/
	if( setsid() == -1 ) {
		perror("setsid: ");
		return 1;
	}

	return 0;
}

static pid_t start_process(const char *path, const char * pid_path, const char * arg)
{
	pid_t pid = 0, expected_pid;
	int status;
	int i;

	if( !path ) {
        printf("%s : %d no path", __FUNCTION__, __LINE__);
        goto internal_error;
    }
	if( !strlen(path) ) {
        printf("%s : %d path length is zero", __FUNCTION__, __LINE__);
        goto internal_error;
    }

	if( daemonize() ) {
        printf("%s : %d cannot daemonize", __FUNCTION__, __LINE__);
        goto internal_error;
    }

	if( (pid = fork()) == -1 ) {
		fprintf(stderr, "wcfglib: Failed to create a child process, reason: %s\n",
				strerror(errno));
		return -1;
	}
	else if( !pid )
	{
		execl(path, basename((char *)path), arg);
		fprintf(stderr, "Could not execute %s, reason: %s\n", path, strerror(errno));
		if( getppid() == 1 ) /*I'm an orphan*/
		{
			char failure_file[_POSIX_PATH_MAX*2];
			int fd;
			/*Lets touch the file, for debug purpose only*/
			strcpy(failure_file, WCFG_PATHPREFIX);
			strcat(failure_file, path);
			strcat(failure_file, ".failure");
			fprintf(stderr, "Internal error: failed to start %s process "
					"and this failure is unknown to the parent.\n"
					"Dumping %s file.\n", path, failure_file);
			if( (fd = open(failure_file, O_CREAT|O_TRUNC)) == -1 )
				perror("Unfortunate day");
			else close(fd);
		}
		exit(1);
	}

	if( write_pid(pid, pid_path) )
		return -1;

/* We wait WCFG_MAXWAIT seconds, maybe less if signal arrives,
   to verify that the process is execl'ed successfully.
   The calling process, wants to know if it failed to execl'ed.
   Note: there is a slight chance that we still wouldn't know it,
   for example we return and the fork'ed process is failed to execl'ed.
   But at least we know that the executable exists.
 */
	for(i=0, expected_pid=pid; i<WCFG_MAXWAIT; i++)
	{
		printf("%s: wait for the child pid = %d\n", __func__, expected_pid); fflush(NULL);
		if( (pid = waitpid(expected_pid, &status, WNOHANG)) == -1 ) {
			perror("Failed in waiting for the child, reason: ");
			return -1;
		}
		else if( !pid ) /*child has not exited yet*/
			sleep(1);
		else if( expected_pid == pid )
		{
			if( WIFEXITED(status) ) {
				if( WEXITSTATUS(status) == 1 ) /*since child did exit(1)*/
					fprintf(stderr, "The child execl'ed process is exited prematurely %d\n",
							WEXITSTATUS(status));
				else fprintf(stderr, "Internal error %d\n",
						WEXITSTATUS(status));
				return -1;
			}
			else if( WIFSIGNALED(status) ) {
				fprintf(stderr, "The child execl'ed process is %d\n",
						WTERMSIG(status));
				return -1;
			}
		}
		else {
			fprintf(stderr, "Bug: returned pid is not of child %d %d\n", pid, expected_pid);
			return -1;
		}
	}

	return expected_pid;

internal_error:
	fprintf(stderr, "wcfglib: An error occured %s:%d\n", __func__, __LINE__);
	return -1;
}

static pid_t read_pid(const char * filename)
{
	int fd, nbytes;
	char buf[30];
	pid_t pid;

	if( (fd = open(filename, O_RDONLY)) == -1 ) {
		fprintf(stderr, "Failed to open %s  pid file: %s\n", filename, strerror(errno));
		return -1;
	}

	if( (nbytes = read(fd, buf, sizeof(buf))) == -1 ) {
		fprintf(stderr, "Failed to read %s pid file: %s\n", filename, strerror(errno));
		close(fd);
		return -1;
	}

	close(fd);

	if( !nbytes ) {
		fprintf(stderr, "Failed to read %s file: %s\n", filename, strerror(errno));
		return -1;
	}

	/*get wpa authenticator pid*/
	sscanf(buf, "%d", &pid);

	return pid;
}


pid_t pseudo_start(const char *path, const char * pid_path, const char * arg) 
{
	pid_t pid;

	fflush(NULL);
	if( (pid = fork()) == -1) {
		fprintf(stderr, "wcfglib: Failed to create a child process, reason: %s\n",
				strerror(errno));
		return -1;
	}
	else if( pid > 0 ) /*parent*/
	{
		int i, status;
		pid_t expected_pid;

		sleep(WCFG_MAXWAIT);
		for(i=0, expected_pid=pid; i<WCFG_MAXWAIT+1; i++) 
		{
			if( (pid = waitpid(expected_pid, &status, WNOHANG)) == -1 ) {
				perror("Failed in waiting for the child, reason: ");
				return -1;
			}
			else if( !pid ) /*child has not exited yet*/
				sleep(1);
			else if( expected_pid == pid ) {
				return read_pid(pid_path);
		}
		}
		return -1;

	}
	else if( !pid ) { /*child*/
		start_process(path, pid_path, arg); exit(0);
	}
	else {
		printf("\n\n\n\n\n\n\nERROR!!!\n\n\n\n\n\n\n\n"); fflush(NULL);
		return -2;
	}

	return -3;
}

/*mod_apdriver: insmod or rmmod the module specifies by file argument*/
static int mod_apdriver(const char *mod, const char *file)
{
	pid_t pid;
	int status;
	
	if( !strlen(file) )
		return 1;

	if( (pid = fork()) == -1 ) {
		fprintf(stderr, "wcfglib: Failed to create a child process, reason: %s\n",
				strerror(errno));
		return -1;
	}
	else if( !pid ) { /*child*/
		execlp(mod, mod, file, NULL);
		/*want to cover all cases call execl here with a full path to mod*/
		exit(1);
	}

	if( waitpid(pid, &status, 0) == -1 ) {
		fprintf(stderr, "wcfglib: %s\n", strerror(errno));
		return 1;
	}
	else if( WIFEXITED(status) )
		return WEXITSTATUS(status); /*in case of success this one must return 0*/
	else if( WIFSIGNALED(status) )
		return WTERMSIG(status);

	return 1;
}

/*************************************************************************
 *                       wcfg_read_cfg()                  				  *
 *************************************************************************
DESCRIPTION: Read current data common for all user mode process from a 
			 given file descriptor into configuration structure                        
INPUT:       int fd - file desctiptor
OUTPUT:      wcfgd *wd - config struct                                           
RETURN:      zero upon success, non-zero o/w
************************************************************************/
static int wcfg_read_cfg(int fd, wcfgd *wd)
{
	char delim[]="<> \t";
	int nbytes;
	char buf[_POSIX_NAME_MAX+2*IFNAMSIZ+50], *p = NULL;


	if( !wd ) return 1;

	if( (nbytes = read(fd, buf, sizeof(buf))) == -1 ) {
		fprintf(stderr, "Failed to read %s configuration file: %s\n",
				WCFG_CFGFILE, strerror(errno));
		return 1;
	}

	if( !nbytes ) {
		fprintf(stderr, "Failed to read %s configuration file: %s\n",
				WCFG_CFGFILE, strerror(errno));
		return 1;
	}

	if( !(p = strtok(buf, delim)) ) {
		fprintf(stderr, "Invalid file format, %d\n", __LINE__);
		return 1;
	}

	/*get wpa authenticator pid*/
	sscanf(p, "%d", &wd->wpaauth_pid);

	if( !(p = strtok(NULL, delim)) ) {
		fprintf(stderr, "Invalid file format, %d\n", __LINE__);
		return 1;
	}
	sscanf(p, "%d", &wd->wdsmngr_pid);

    if( !(p = strtok(NULL, delim)) ) {
		fprintf(stderr, "Invalid file format, %d\n", __LINE__);
		return 1;
	}

	/*get ap module name*/
	strncpy(wd->apmodule_name, p, _POSIX_NAME_MAX);
	wd->apmodule_name[_POSIX_NAME_MAX] = '\0';

	if( !(p = strtok(NULL, delim)) ) {
		fprintf(stderr, "Invalid file format, %d\n", __LINE__);
		return 1;
	}
	
	/*get driver device name*/
	strncpy(wd->dev_name, p, IFNAMSIZ-1);
	wd->dev_name[IFNAMSIZ-1] = '\0';

	if( !(p = strtok(NULL, delim)) ) {
		fprintf(stderr, "Invalid file format, %d\n", __LINE__);
		return 1;
	}

	/*get bridge device name*/
	if( !isspace(*p) && *p != '\0' ) {
		strncpy(wd->br_name, p, IFNAMSIZ-1);
		wd->br_name[IFNAMSIZ-1] = '\0';
	}

#ifdef TIWLAN_FW_TRACE
	if( !(p = strtok(NULL, delim)) ) {
		fprintf(stderr, "Invalid file format, %d\n", __LINE__);
		return 1;
	}
	sscanf(p, "%d", &wd->fw_trace_pid);
	printf("%s : got FW trace pid : %d\n", __FUNCTION__, wd->fw_trace_pid);
#endif
	printf("wcfglib: Found: wpa pid=%d, wds pid = %d, ap module name %s,\n device ifname %s, bridge ifname %s\n", 
			wd->wpaauth_pid, wd->wdsmngr_pid, wd->apmodule_name, wd->dev_name,
			strlen(wd->br_name)?wd->br_name:"");

	return 0;
}


/*************************************************************************
 *                       wcfg_read_initcfg()                  			 *
 *************************************************************************
DESCRIPTION: Read initial configuration data common for all user mode 
			 processes into intial config struct
INPUT:       NONE
OUTPUT:      wcfginit *init - init config struct                                           
RETURN:      zero upon success, non-zero o/w
************************************************************************/
static int wcfg_read_initcfg(wcfginit *init)
{
	char delim[]="<> \t";
	int fd, nbytes;
	char buf[2*_POSIX_NAME_MAX + (_POSIX_NAME_MAX+_POSIX_PATH_MAX+1)*5 + 50], *p = NULL;


	if( !init ) return 1;

	if( (fd = open(WCFG_INITCFGFILE, O_RDONLY)) == -1 ) {
		fprintf(stderr, "Failed to open %s init configuration file: %s\n",
				WCFG_INITCFGFILE, strerror(errno));
		return 1;
	}

	if( (nbytes = read(fd, buf, sizeof(buf))) == -1 ) {
		fprintf(stderr, "Failed to read %s configuration file: %s\n",
				WCFG_INITCFGFILE, strerror(errno));
		close(fd);
		return 1;
	}
	//	printf("%s Read [%s]\n", __FUNCTION__, buf);
	
	if( !nbytes ) {
		fprintf(stderr, "Failed to read %s configuration file: %s\n",
				WCFG_INITCFGFILE, strerror(errno));
		close(fd);
		return 1;
	}

	if( !(p = strtok(buf, delim)) ) {
		fprintf(stderr, "Invalid file format\n");
		close(fd);
		return 1;
	}

	/*get wpa authenticator executable*/
	sscanf(p, "%s", init->wpaauth_file);

	if( !(p = strtok(NULL, delim)) ) {
		fprintf(stderr, "Invalid file format\n");
		close(fd);
		return 1;
	}

	/*get ap module file*/
	strncpy(init->apmodule_file, p, _POSIX_NAME_MAX);
	init->apmodule_file[_POSIX_NAME_MAX] = '\0';

	if( !(p = strtok(NULL, delim)) ) {
		fprintf(stderr, "Invalid file format\n");
		close(fd);
		return 1;
	}

	/*get driver device name*/
	strncpy(init->dev_name, p, IFNAMSIZ-1);
	init->dev_name[IFNAMSIZ-1] = '\0';

	if( !(p = strtok(NULL, delim)) ) {
		fprintf(stderr, "Invalid file format\n");
		close(fd);
		return 1;
	}

	/*get bridge device name*/
	if( !isspace(*p) && *p != '\0' ) {
		strncpy(init->br_name, p, IFNAMSIZ-1);
		init->br_name[IFNAMSIZ-1] = '\0';
	}

#ifdef TIWLAN_FW_TRACE
	if( !(p = strtok(NULL, delim)) ) {
		fprintf(stderr, "Invalid file format\n");
		close(fd);
		return 1;
	}
	sscanf(p, "%s", init->fw_trace_app_filename);
	printf("%s : FW trace filename : %s\n", __FUNCTION__, init->fw_trace_app_filename);
#endif

	printf("wcfglib: Found: %s, %s, %s, %s\n", 
			init->wpaauth_file, init->apmodule_file, init->dev_name,
			strlen(init->br_name)?init->br_name:"");

	close(fd);
	return 0;
}



static int wcfg_write_initcfg(wcfginit *init)
{
	int fd;
	char buf[1300], tmp[100];

	if( !init ) return 1;

	if( (fd = open(WCFG_INITCFGFILE, O_CREAT|O_WRONLY|O_TRUNC)) == -1 ) {
		perror("open: ");
		return 1;
	}

	sprintf(buf, "<%s> ", init->wpaauth_file);
	sprintf(tmp, "<%s> ", init->apmodule_file);
	strcat(buf, tmp);
	sprintf(tmp, "<%s>", init->dev_name);
	strcat(buf, tmp);
	if( strlen(init->br_name) ) {
		sprintf(tmp, " <%s>", init->br_name);
		strcat(buf, tmp);
	}
#ifdef TIWLAN_FW_TRACE
	sprintf(tmp, " <%s>", init->fw_trace_app_filename);
	strcat(buf, tmp);
#endif
	strcat(buf, "\n");

	//	printf("%s : Wrote [%s]\n", __FUNCTION__, buf);

	if( write(fd, buf, strlen(buf)) == -1 ) {
		perror("write: ");
		close(fd);
		return 1;
	}

	close(fd);
	return 0;
}


static int wcfg_write_cfg(wcfgd *wd)
{
	int fd;
	char buf[_POSIX_NAME_MAX+IFNAMSIZ+50], tmp[100];

	if( !wd ) return 1;

	if( (fd = open(WCFG_CFGFILE, O_CREAT|O_WRONLY|O_TRUNC)) == -1 ) {
		perror("open: ");
		return 1;
	}

	sprintf(buf, "<%d> ", wd->wpaauth_pid);
    sprintf(tmp, "<%d> ", wd->wdsmngr_pid);
    strcat(buf, tmp);
	sprintf(tmp, "<%s> ", wd->apmodule_name);
	strcat(buf, tmp);
	sprintf(tmp, "<%s>", wd->dev_name);
	strcat(buf, tmp);
	if( strlen(wd->br_name) ) {
		sprintf(tmp, " <%s>", wd->br_name);
		strcat(buf, tmp);
	}
#ifdef TIWLAN_FW_TRACE
	sprintf(tmp, " <%d>", wd->fw_trace_pid);
	strcat(buf, tmp);
#endif
	strcat(buf, "\n");
	if( write(fd, buf, strlen(buf)) == -1 ) {
		perror("write: ");
		close(fd);
		return 1;
	}

	close(fd);
	return 0;
}

static void kill_process(pid_t pid) {
	if( !pid ) return;

	printf("wcfglib: Terminating process (pid=%d).\n", pid);
	if( kill(pid, SIGTERM) == -1 )
	{
		sleep(10);
		fprintf(stderr, "wcfglib: process ignored SIGTERM signal, %s\n", 
				strerror(errno));
		if( kill(pid, SIGINT) == -1 ) 
		{
			fprintf(stderr, "wcfglib: process ignored SIGINT signal, %s\n",
					strerror(errno));
trythis:
			kill(pid, SIGKILL);
			fprintf(stderr, "wcfglib: Sent SIGKILL signal to the process.\n");
			sleep(2);
		}
		fflush(stderr);
	}
	else printf("wcfglib: Successfully terminated the process.\n");

	if( kill(pid, SIGTERM) == -1 ) {
		if( errno == ESRCH )
			printf("\nwcfglib: Successfully terminated process.\n\n");
		else goto trythis;
	}

	fflush(stdout);
}

static int config_wpaauth(struct storage *storage)
{
	tiap_wpa_config_t  wpa_config; 

	memcpy(wpa_config.sec_profiles,storage->wpa.profile, sizeof(wpa_config.sec_profiles));
	memcpy(&wpa_config.mssid, &storage->ldb.multi_ssid, sizeof(wpa_config.mssid));
	wpa_config.dbg_report_sev = storage->debug.wpa_report_sev;
	wpa_config.dbg_report_mod = storage->debug.wpa_report_mod;
	return tiap_wcfg_sendIpcMsg(TIAP_SOCKADDR_WPAAUTH, &wpa_config, sizeof(wpa_config));
}

static void deinit_process(wcfgd *wd, int stage)
{
	/*delete the configuration file*/
	unlink(WCFG_CFGFILE);
	unlink(WCFG_INITCFGFILE);
	unlink(WCFG_WPAPIDFILE);

    /*kill the WDS manager process*/
    if( wd->wdsmngr_pid > 1 ) {
        kill_process(wd->wdsmngr_pid);
        if(tiap_wcfg_waitForProcessExit(wd->wdsmngr_pid)) {
            printf("\n\nError waiting for WDS process termination.\n");
        }

    }
    /*kill the wpa authenticator process*/
    if( wd->wpaauth_pid > 1 )
        kill_process(wd->wpaauth_pid);


	if( stage > 1 ) 
	{

		if( tiap_wcfg_dev_set_flags(wd->dev_name, ~IFF_UP, 0) )
			fprintf(stderr, "wcglib: Failed to bring down %s interface\n", wd->dev_name);
		printf("wcfglib: Unloading the ap module.\n");
		if( mod_apdriver("rmmod", wd->apmodule_name) )

			fprintf(stderr, "wcfglib: Failed to unload the ap module.\n");
		else printf("wcfglib: Successfully unloaded the ap module.\n");
	}
}

wcfgd *tiap_wcfg_init(wcfginit *init, char * driver_load_failed) {
	wcfgd *wd = NULL;
	int fd, ret;
	char *p = NULL;
	struct stat s;
	struct in_addr addr;
	pid_t wdsPid;
	tiap_param_eeprom_t eeprom;
    int defaultNvsEnable = 0;


	*driver_load_failed = TRUE;

	/*allocate wcfg descriptor*/
	if( !(wd = malloc(sizeof(wcfgd))) )
		return NULL;
	memset(wd, 0, sizeof(wcfgd));

	memcpy(wd->fw_filename, init->fw_filename, sizeof(wd->fw_filename));
	memcpy(wd->fw_trace_app_filename, init->fw_trace_app_filename, sizeof(wd->fw_trace_app_filename));

#ifdef MWCODE
        printf("\nwcfglib: Setting default parameters.\n");
        wd->storage.defaults = 0; /*once is enough*/
        wcfg_storage_setdefaults(&wd->storage);
#else
	/*store storage file*/
	strncpy(wd->storage_file, init->storage_file, _POSIX_NAME_MAX+_POSIX_PATH_MAX);
	wd->storage_file[_POSIX_NAME_MAX+_POSIX_PATH_MAX] = '\0';

	/*read storage */
	if( wcfg_storage_read(&wd->storage, wd->storage_file) ) {
		free(wd);
		return NULL;
	}

	/*verify storage integrity*/
	printf("Checking storage integrity : %d == %d\n", wd->storage.magic, WCFG_MAGIC);
	if( wd->storage.magic != WCFG_MAGIC ) {
		printf("\nwcfglib: Setting default parameters.\n");
		wd->storage.defaults = 0; /*once is enough*/
		wcfg_storage_setdefaults(&wd->storage);
        if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) ) {
            printf("%s: error storing configuration.\n", __FUNCTION__);
            free(wd);
            return NULL;
        }

	}
#endif /* MWCODE */

	if( wd->storage.defaults == 1 ) {
		wd->storage.defaults = 0;
        if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) ) 
        {
            printf("%s: error storing configuration.\n", __FUNCTION__);
            free(wd);
            return NULL;
        }
		printf("\nwcfglib: Setting default parameters.\n");
	}

	if( (fd = open(WCFG_CFGFILE, O_RDONLY)) == -1 ) /*initialization case*/
	{	
		/* start wpa authenticator always !!! */
		{
			/*start wpa authenticator*/
			if( (wd->wpaauth_pid = pseudo_start(init->wpaauth_file, WCFG_WPAPIDFILE, NULL)) == -1 ) {
				free(wd);
				printf("%s: %d\n", __func__, __LINE__);
				return NULL;
			}

			if( config_wpaauth(&wd->storage) == -1 ) {
				fprintf(stderr, "wcfglib: Failed to configure wpa authenticator.\n");
				kill_process(wd->wpaauth_pid);
				free(wd);
				return NULL;
			}
		}

		if( tiap_wcfg_bridge_config(init->br_name, wd, wd->storage.magic != WCFG_MAGIC) )
		{
			fprintf(stderr, "%s line=%d: Failed to config bridge IP settings\n", 
                    __FUNCTION__, __LINE__);
			free(wd);
			return NULL;
		}

		/* If automatic FW load is enabled, get it */
		if (wd->storage.debug.automatic_fw_load_enabled) {
			addr.s_addr = wd->storage.debug.debug_server_ip_addr;
			if (tftp_get_file("FW.bin", inet_ntoa(addr), WCFG_TMP_DIR)) {
				puts("\n\n\n-------------------------------------------------------\n");
				puts("----------------------------------------------------------\n");
				puts("----------------------------------------------------------\n");
				puts("\nERROR: Automatic FW load failed ! - Will continue anyway\n");
				puts("----------------------------------------------------------\n");
				puts("----------------------------------------------------------\n");
				puts("----------------------------------------------------------\n\n\n");

				exit(1);
			}	else {
				puts("Automaitc FW load : got FW image via TFTP"); 
				snprintf(wd->fw_filename, sizeof(wd->fw_filename), "%s/FW.bin", WCFG_TMP_DIR);
			}
		}
        /* Start WDS manager */
        if(wd->storage.ldb.wdsEnable) {
            printf("%s:%d Start WDS\n", __FUNCTION__, __LINE__); fflush(NULL);
            if(dda_wdsStart(&wdsPid) != OK) {
                fprintf(stderr, "%s : Error starting WDS manager.\n", __FUNCTION__);
            }
            wd->wdsmngr_pid = wdsPid;
        }

		/* Demiurg : check whether module exist */
		if (stat(init->apmodule_file, &s) != 0 || s.st_size <= 0) {
			fprintf(stderr, "wcfglib: Kernel module %s does not exist or invalid !\n", 
					init->apmodule_file);
            /*kill the WDS manager process*/
            if( wd->wdsmngr_pid > 1 ) {
                kill_process(wd->wdsmngr_pid);
                if(tiap_wcfg_waitForProcessExit(wd->wdsmngr_pid)) {
                    printf("\n\nError waiting for WDS process termination.\n");
                }

            }
            /*kill the wpa authenticator process*/
            if( wd->wpaauth_pid > 1 )
                kill_process(wd->wpaauth_pid);
			free(wd);
			return NULL;
		}

		/*insmod ap driver*/
		if( mod_apdriver("insmod", init->apmodule_file) ) 






		{
			if( !strlen(init->apmodule_file) )
				fprintf(stderr, "wcfglib: Empty module name.\n");
			else fprintf(stderr, "wcfglib: Failed to insmod %s, terminating %s.\n",
					init->apmodule_file, init->wpaauth_file);
			deinit_process(wd, 1);
			free(wd);
			return NULL;
		}

#ifdef TIWLAN_DEBUG
		puts("Creating ksyms file !!!");
		system("cat /proc/ksyms > /nfs/ksyms.txt");
#endif

		/*store ap module name*/
		strncpy(wd->apmodule_name, basename(init->apmodule_file), _POSIX_NAME_MAX);
		wd->apmodule_name[_POSIX_NAME_MAX] = '\0';
		if( !(p = strrchr(wd->apmodule_name, '.')) ) {
			fprintf(stderr, "wcfglib: Incorrect ap module name %s.\n", wd->apmodule_name);
			deinit_process(wd, 1);
			free(wd);
			printf("%s: %d\n", __func__, __LINE__);
			return NULL;
		}
		*p = '\0';

		/* store ap device name */
		strncpy(wd->dev_name, init->dev_name, IFNAMSIZ-1);
		wd->dev_name[IFNAMSIZ-1] = '\0';

		/* store bridge device name */
		strncpy(wd->br_name, init->br_name, IFNAMSIZ-1);
		wd->br_name[IFNAMSIZ-1] = '\0';

		if( wcfg_write_initcfg(init) )
		{
			fprintf(stderr, "wcfglib: Failed to write %s configuration file.\n", WCFG_INITCFGFILE);
			deinit_process(wd, 2);
			free(wd);
			printf("%s: %d\n", __func__, __LINE__);
			return NULL;
		}

#ifdef TIWLAN_FW_TRACE
		printf("Starting FW Trace user mode app [%s] \n", wd->fw_trace_app_filename);
		addr.s_addr = wd->storage.debug.debug_server_ip_addr;
		if ((wd->fw_trace_pid = pseudo_start(wd->fw_trace_app_filename, WCFG_FW_TRACE_PID_FILE, inet_ntoa(addr))) == -1 ) {
			printf("%s: %d Failed !\n", __func__, __LINE__);
		}
		printf("FW Trace APP pid = %d\n", wd->fw_trace_pid);
#endif

		if( wcfg_write_cfg(wd) )
		{
			fprintf(stderr, "wcfglib: Failed to write %s configuration file.\n", WCFG_CFGFILE);
			deinit_process(wd, 2);
			free(wd);
			printf("%s: %d\n", __func__, __LINE__);
			return NULL;
		}

		if (wd->storage.ldb.eeprom_less)
        {
            if ( wcfg_nvs_read_data( wd, &eeprom, &defaultNvsEnable ) )
            {
                fprintf(stderr, "%s: NVS Storage ERROR !\n", __FUNCTION__);
                deinit_process(wd, 2);
                free(wd);
                return NULL;
            }
            if (defaultNvsEnable) 
            {
                fprintf(stderr,"%s: Setting default NVS data.\n", __FUNCTION__);
                wcfg_nvs_set_default_data( &eeprom );
            }
        }

		if( (ret = wcfg_ioctl_init(wd, &eeprom)) )
		{
			fprintf(stderr, "wcfglib: Failed to initialize the wcfg library, error=%d\n", ret);
			deinit_process(wd, 2);
			free(wd);
			printf("%s: %d\n", __func__, __LINE__);
			return NULL;
		}
		if( tiap_wcfg_bridge_add_wlan( wd ) )
		{
			fprintf(stderr, "%s line=%d: Failed to add wlan to bridge\n", 
                    __FUNCTION__, __LINE__);
			deinit_process(wd, 2);
			free(wd);
			return NULL;

		}
        /* Configure WDS manager */
        if(wd->storage.ldb.wdsEnable) {
            if(tiap_wcfg_configWds(wd, 0, 0)) {
                /* Take action here */
                fprintf(stderr, "%s : WDS manager does not started.\n", __FUNCTION__);
            }
            dda_wdsAddIfToBridge(wd->br_name);
        }

	}
	else {
		if( (ret = wcfg_read_cfg(fd, wd)) ) {
			printf("%s: %d\n", __func__, __LINE__);
			close(fd);
			return NULL;
		}
		close(fd);
	}

	*driver_load_failed = FALSE;

	return wd;
}

/* Free all library resources. */
int tiap_wcfg_light_deinit(const wcfgd *wd)
{
	
	if( !wd ) return 1;

	/*free wcfg descriptor*/
	free((void *)wd);

	return 0;
}

int tiap_wcfg_deinit(const wcfgd *wd)
{
	int ret = 0;
	__u32 gw;
	struct in_addr in;


	if( !wd ) return 1;

	/*update persistent storage*/
	if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) ) {
		ret = 1;
		goto out;
	}

    /*kill the WDS manager process*/
    if( wd->wdsmngr_pid > 1 ) {
        kill_process(wd->wdsmngr_pid);
        if(tiap_wcfg_waitForProcessExit(wd->wdsmngr_pid)) {
            printf("\n\nError waiting for WDS process termination.\n");
        }

    }
	/*kill the wpa authenticator process*/
	if( wd->wpaauth_pid > 1 )
		kill_process(wd->wpaauth_pid);


#ifdef TIWLAN_FW_TRACE
	printf("killing FW trace app, pid = %d\n", wd->fw_trace_pid);
	if ( wd->fw_trace_pid > 1 )
		kill_process(wd->fw_trace_pid);
#endif
	
	if( tiap_wcfg_dev_set_flags(wd->dev_name, ~IFF_UP, 0) )
		fprintf(stderr, "wcglib: Failed to bring down %s interface\n", wd->dev_name);

	/*unload the ap driver*/
	if( mod_apdriver("rmmod", wd->apmodule_name) ) {

		fprintf(stderr, "wcglib: Failed to unload the ap module from the kernel\n");
		ret = 1;
	}

	/*delete the configuration file*/
	if( unlink(WCFG_CFGFILE) == -1 ) {
		fprintf(stderr, "wcglib: Couldn't remove %s file: %s\n",
				WCFG_CFGFILE, strerror(errno));
		ret = 1;
		goto out;
	}

	if( unlink(WCFG_WPAPIDFILE) == -1 ) {
		fprintf(stderr, "wcglib: Couldn't remove %s file: %s\n",
				WCFG_WPAPIDFILE, strerror(errno));
		ret = 1;
		goto out;
	}

	
	if( tiap_wcfg_get_default_gateway(&gw) ) {
		fprintf(stderr, "wcglib: Warning: Failed to get default route.\n");
		/*		ret = 1;
					goto out; */
	} else {
		/*delete default route*/
		in.s_addr = gw;
		if( tiap_wcfg_set_route(wd->br_name, 0, 0, gw, SIOCDELRT) ) {
			ret = 1;
			goto out;
		}
		printf("wcglib: Deleted default route %s\n", inet_ntoa(in));
	}
	

	fflush(stdout);

out:
	/*free wcfg descriptor*/
	free((void *)wd);

	return ret;
}


/* 
 * Deinitialize and then initialize library.
 * Additionally, read all required configuration parameters so that
 * the next time library will be correctly initialized.
 */
#include <sys/reboot.h>

void tiap_wcfg_system_reboot(const wcfgd *wd) 
{
	sync();
	sleep(1);
	reboot(RB_ENABLE_CAD);
	sync();
	sleep(1);
	reboot(RB_AUTOBOOT);
}

wcfgd *tiap_wcfg_restart(wcfgd *wd, char * driver_load_failed)  {
	wcfginit init;
	int fd;

	memset(&init, 0, sizeof(init));
	strncpy(init.storage_file, WCFG_MTDBLOCK, sizeof(init.storage_file));
	init.storage_file[sizeof(init.storage_file)-1] = '\0';
	memcpy(init.fw_filename, wd->fw_filename, sizeof(wd->fw_filename));

	if( wcfg_read_initcfg(&init) )
		return NULL;
    
	/* It could happen that another usermode process which also uses the 
	   library performed restart and changed child processes pids and so
	   the data in wd structure may be out-of-date so there is a need to 
	   update those parameters from file */
	if( (fd = open(WCFG_CFGFILE, O_RDONLY)) == -1 )
		return NULL;
	if( wcfg_read_cfg(fd, wd) )
	{
		close(fd);
		return NULL;
	}
	close(fd);

	if( tiap_wcfg_deinit(wd) )
		return NULL;

	return tiap_wcfg_init(&init, driver_load_failed);
}



/* The following functions update both local and physical storage.
   On success zero is returned, otherwise nonzero value.

   Note: no syncronization is used for simplicity reasons only.
 */


int tiap_wcfg_set_hwaddr(wcfgd *wd, const __u8 *hwaddr)
{
	memcpy(wd->storage.ldb.hwaddr, hwaddr, ETH_ALEN);

	/* Felix R.
	   The following code is verified and works.
	   It's commented since changing hw addr is not allowed at run time,
	   because of bssid issue.
	 */

	if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;

	return 0;
}

int tiap_wcfg_get_hwaddr(const wcfgd *wd, __u8 *hwaddr)
{
	int s;
	struct ifreq ifr;

	if( (s =  socket(AF_INET, SOCK_DGRAM, 0)) == -1 ) {
		fprintf(stderr, "%d: %s\n", __LINE__, strerror(errno));
		return 1;
	}

	strncpy(ifr.ifr_name, wd->dev_name, IFNAMSIZ);
	ifr.ifr_hwaddr.sa_family = ARPHRD_ETHER;

	if( ioctl(s, SIOCGIFHWADDR, &ifr) < 0) {
		fprintf(stderr, "%d: %s\n", __LINE__, strerror(errno));
        close(s);
		return 1;
	}

	memcpy(hwaddr, ifr.ifr_hwaddr.sa_data, ETH_ALEN);
	close(s);

	return 0;
}



/* Standard */
int tiap_wcfg_siwrts(wcfgd *wd, const __s32 rts, int update_storage)
{
	wd->storage.ldb.rts = rts;

	if( update_storage && wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;

	return 0;
}

int tiap_wcfg_siwfrag(wcfgd *wd, const __s32 frag, int update_storage)
{
	wd->storage.ldb.frag = frag;

	if( update_storage && wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;

	return 0;
}

int tiap_wcfg_siwretry(wcfgd *wd, const __u32 value, const int type, int update_storage)
{
	switch( type )
	{
		case TIAP_PARAM_RETRY_LONG:
			wd->storage.ldb.retry_long = (__u8)value;
			break;
		case TIAP_PARAM_RETRY_SHORT:
			wd->storage.ldb.retry_short = (__u8)value;
			break;
		case TIAP_PARAM_LIFETIME_TX:
			wd->storage.ldb.lifetime_tx = value;
			break;
		case TIAP_PARAM_LIFETIME_RX:
			wd->storage.ldb.lifetime_rx = value;
			break;
		default: return 1;
	}

	if( update_storage && wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;

	return 0;
}

int tiap_wcfg_siwtxpow(wcfgd *wd, const tiap_param_txpower_t *txpow, int update_storage)
{
	if( txpow->hr == txpower_a )
		memcpy(&wd->storage.ldb.txpow_a, (void *)txpow, sizeof(*txpow));
	else if( txpow->hr == txpower_g )
		memcpy(&wd->storage.ldb.txpow_g, (void *)txpow, sizeof(*txpow));

	if( update_storage && wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;

	return 0;
}



/* The following functions read from local storage*/

int tiap_wcfg_giwrts(const wcfgd *wd, __s32 *rts, int from_storage)
{
	int ret = 0, ioctl_sock;

	if (from_storage)
	{
		*rts = wd->storage.ldb.rts;
	}
	else
	{

		if((ioctl_sock = iw_sockets_open()) < 0)
		{
			perror("socket");
			return(-1);
		}

		ret = ioctl_get_rts (rts, ioctl_sock, (char *)wd->dev_name);

		close(ioctl_sock);
	}

	return ret;
}

int tiap_wcfg_giwfrag(const wcfgd *wd, __s32 *frag, int from_storage)
{
	int ret = 0, ioctl_sock;

	if (from_storage)
	{
		*frag = wd->storage.ldb.frag;
	}
	else
	{
		if((ioctl_sock = iw_sockets_open()) < 0)
		{
			perror("socket");
			return(-1);
		}

		ret = ioctl_get_frag (frag, ioctl_sock, (char *)wd->dev_name);

		close(ioctl_sock);
	}

	return ret;
}

int tiap_wcfg_giwretry(const wcfgd *wd, __u32 *value, int type, int from_storage)
{
	int ret = 0, ioctl_sock;

	if (from_storage)
	{
		switch( type )
		{
			case TIAP_PARAM_RETRY_LONG:
				*(__u8 *)value = wd->storage.ldb.retry_long;
				break;
			case TIAP_PARAM_RETRY_SHORT:
				*(__u8 *)value = wd->storage.ldb.retry_short;
				break;
			case TIAP_PARAM_LIFETIME_TX:
				*value = wd->storage.ldb.lifetime_tx;
				break;
			case TIAP_PARAM_LIFETIME_RX:
				*value = wd->storage.ldb.lifetime_rx;
				break;
			default: return 1;
		}
	}
	else
	{


		if((ioctl_sock = iw_sockets_open()) < 0)
		{
			perror("socket");
			return(-1);
		}

		ret = ioctl_get_retry((__s32 *)value, type, ioctl_sock, (char *)wd->dev_name);

		close(ioctl_sock);
	}

	return ret;

}

int tiap_wcfg_giwtxpow(const wcfgd *wd, tiap_param_txpower_t *txpow, int from_storage)
{
	int ret = 0, ioctl_sock;

	if (from_storage)
	{
		if( txpow->hr == txpower_a )
			memcpy(txpow, &wd->storage.ldb.txpow_a, sizeof(*txpow));
		else if( txpow->hr == txpower_g )
			memcpy(txpow, &wd->storage.ldb.txpow_g, sizeof(*txpow));
		else return 1;
	}
	else
	{

		if((ioctl_sock = iw_sockets_open()) < 0)
		{
			perror("socket");
			return(-1);
		}

		ret = ioctl_get_txpow (txpow, ioctl_sock, (char *)wd->dev_name);

		close(ioctl_sock);
	}

	return ret;

}



/* Complex */

int tiap_wcfg_set_basic(wcfgd *wd, const __u32 we_id, void *arg, int update_storage)
{
	int ret = 0, ioctl_sock;

	/* Create a channel to the NET kernel. */
	if((ioctl_sock = iw_sockets_open()) < 0)
	{
		perror("socket");
		return(-1);
	}

	switch( we_id )
	{
		case TIAP_PARAM_REPLEV:
			{
				__s8 replev = *(__s8 *)arg;

				if (replev > 0)
					set_replev_ldb(wd, replev);
				else if (replev < 0)
					    unset_replev_ldb(wd, replev * (-1));
				else
					disable_replev_ldb(wd);
				
				ret = exec_priv_ioctl_set(TIAP_PARAM_REPLEV, arg, sizeof(__s8),ioctl_sock, wd->dev_name);
			}
			break;
		
		case TIAP_PARAM_BEACON_PERIOD:
			wd->storage.ldb.beacon_period = *(__u16 *)arg;
			/* set the MSDU life time acording the new beacon Period Setting */
			wd->storage.ldb.qos.edcf_info[TIAP_PARAM_QOS_AC_AP_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_BE].txMaxLifetime = 
				5*(wd->storage.ldb.beacon_period*wd->storage.ldb.dtim_period)*1000;
			wd->storage.ldb.qos.edcf_info[TIAP_PARAM_QOS_AC_AP_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_BK].txMaxLifetime = 
				5*(wd->storage.ldb.beacon_period*wd->storage.ldb.dtim_period)*1000;
			break;
		case TIAP_PARAM_DTIM_PERIOD:
			wd->storage.ldb.dtim_period = *(__u8 *)arg;
			/* set the MSDU life time acording the new Dtim Period Setting */
			wd->storage.ldb.qos.edcf_info[TIAP_PARAM_QOS_AC_AP_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_BE].txMaxLifetime = 
				5*(wd->storage.ldb.beacon_period*wd->storage.ldb.dtim_period)*1000;
			wd->storage.ldb.qos.edcf_info[TIAP_PARAM_QOS_AC_AP_PARAMS][TIAP_PARAM_QOS_CLASSIFIER_AC_BK].txMaxLifetime = 
				5*(wd->storage.ldb.beacon_period*wd->storage.ldb.dtim_period)*1000;
            break;
		case TIAP_PARAM_BSS_BRIDGE_STATE:
			wd->storage.ldb.bss_bridge_state = *(tiap_param_bss_br_state_t *)arg;
			ret = exec_priv_ioctl_set(TIAP_PARAM_BSS_BRIDGE_STATE, arg,
					sizeof(tiap_param_bss_br_state_t),ioctl_sock, wd->dev_name);
			break;
		case TIAP_PARAM_WONE_TIME_MODE:
			wd->storage.ldb.wone_timemod = *(__u8 *)arg;
			break;
		case TIAP_PARAM_RATE_FALLBACK:
			wd->storage.ldb.rate_fallback = *(__u8 *)arg;
			break;
		case TIAP_PARAM_WONE_NUM:
			wd->storage.ldb.wone_num = *(__u8 *)arg;
			break;
		case TIAP_PARAM_RADIO_CAL:
			wd->storage.ldb.radio_cal = *(__u32 *)arg;
			break;
		case TIAP_PARAM_RADIO_CAL_INT:
			wd->storage.ldb.cal_interval = *(__u16 *)arg;
			break;
		case TIAP_PARAM_ENERGY_DETECT:
			wd->storage.ldb.energy_detect = *(__u8 *)arg;
			break;
		case TIAP_PARAM_HIDDEN_SSID:
			wd->storage.ldb.hidden_ssid = *(__u16 *)arg;
			break;
		case TIAP_PARAM_MULTI_DOMAIN_CAPAB:
			wd->storage.ldb.multi_domain_cap = *(__u8 *)arg;
			break;
		case TIAP_PARAM_SPECTRUM_MGMT:
			wd->storage.ldb.spectrum_mgmt = *(__u8 *)arg;
			break;
		case TIAP_PARAM_LEGACY_STA:
			wd->storage.ldb.legacy_sta = *(__u8 *)arg;
			break;
		case TIAP_PARAM_LOCAL_POWER_CONS:
			wd->storage.ldb.local_pow_cons = *(__s8 *)arg;
			break;
		case TIAP_PARAM_MAX_TX_POWER_LEVEL:
			wd->storage.ldb.max_txpow_lev = *(__u8 *)arg;
			break;
		case TIAP_PARAM_COUNTRY_STR:
			strncpy(wd->storage.ldb.country_str, (char *)arg, TIAP_PARAM_COUNTRY_STRING_LEN);
			break;
		case TIAP_PARAM_OPER_STATE:
			wd->storage.ldb.oper_state = *(__u8 *)arg;
			ret = exec_priv_ioctl_set(TIAP_PARAM_OPER_STATE, arg, sizeof(__u8),ioctl_sock,wd->dev_name);
			break;
		case TIAP_PARAM_RADAR_DETECT_MODE:
			{
				__u8 radar_detect = *(__u8 *)arg;
				ret = set_radar_detect(wd, radar_detect);
			}
			break;
		case TIAP_PARAM_BURST_LIMIT:
			wd->storage.ldb.txop_limit = *(__u16 *)arg;
			break;
		case TIAP_PARAM_AP_BRIDGE_MODE:
			wd->storage.ldb.bridge_mode = *(__u8 *)arg;
			break;
		case TIAP_PARAM_WDS_ENABLE:
			wd->storage.ldb.wdsEnable = *(__u8 *)arg;
            ret = exec_priv_ioctl_set(TIAP_PARAM_WDS_ENABLE, (__u8 *)&wd->storage.ldb.wdsEnable,
					sizeof(__u8), ioctl_sock, wd->dev_name);
            break;

		case TIAP_PARAM_EEPROM_LESS_ENABLE:
			wd->storage.ldb.eeprom_less = *(__u8 *)arg;
			break;

		default:
			ret = 1;
	}

	if ( !ret && update_storage){	
		if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) ) {
			close(ioctl_sock);
			return 1;
		}
	}

	close(ioctl_sock);

	return ret;
}

static void set_replev_ldb(wcfgd *wd, __u8 replev)
{
	wd->storage.ldb.replev |= 1 << replev;
}

static void unset_replev_ldb(wcfgd *wd, __u8 replev)
{

	wd->storage.ldb.replev &= ~(1 << replev);
}

static void disable_replev_ldb(wcfgd *wd)
{

	wd->storage.ldb.replev = 0;
}

int tiap_wcfg_get_basic(wcfgd *wd, const __u32 we_id, void *arg, int from_storage)
{
	int ret = 0, ioctl_sock;

	/* Create a channel to the NET kernel. */
	if((ioctl_sock = iw_sockets_open()) < 0)
	{
		perror("socket");
		return(-1);
	}

	switch( we_id )
	{
		case TIAP_PARAM_REPLEV:
			if (from_storage)
				*(__s32 *)arg = wd->storage.ldb.replev;
			else
				ret = exec_priv_ioctl_get(TIAP_PARAM_REPLEV, arg,sizeof(__s32),ioctl_sock,wd->dev_name);
			break;
		case TIAP_PARAM_BEACON_PERIOD:
			if (from_storage)
				*(__u16 *)arg = wd->storage.ldb.beacon_period;
			else
				ret = exec_priv_ioctl_get(TIAP_PARAM_BEACON_PERIOD, arg,sizeof(wd->storage.ldb.beacon_period),ioctl_sock,wd->dev_name);
			break;
		case TIAP_PARAM_DTIM_PERIOD:
			if (from_storage)
				*(__u8 *)arg = wd->storage.ldb.dtim_period;
			else
				ret = exec_priv_ioctl_get(TIAP_PARAM_DTIM_PERIOD, arg,sizeof(wd->storage.ldb.dtim_period),ioctl_sock,wd->dev_name);
			break;
		case TIAP_PARAM_BSS_BRIDGE_STATE:
			if (from_storage)
				*(tiap_param_bss_br_state_t *)arg = wd->storage.ldb.bss_bridge_state;
			else
				ret = exec_priv_ioctl_get(TIAP_PARAM_BSS_BRIDGE_STATE, arg, sizeof(wd->storage.ldb.bss_bridge_state),ioctl_sock,wd->dev_name);
			break;
		case TIAP_PARAM_WONE_TIME_MODE:
			if (from_storage)
				*(__u8 *)arg = wd->storage.ldb.wone_timemod;
			else
				ret = exec_priv_ioctl_get(TIAP_PARAM_WONE_TIME_MODE, arg, sizeof(wd->storage.ldb.wone_timemod),ioctl_sock,wd->dev_name);
			break;
		case TIAP_PARAM_RATE_FALLBACK:
			if (from_storage)
				*(__u8 *)arg = wd->storage.ldb.rate_fallback;
			else
				ret = exec_priv_ioctl_get(TIAP_PARAM_RATE_FALLBACK, arg, sizeof(wd->storage.ldb.rate_fallback),ioctl_sock,wd->dev_name);
			break;
		case TIAP_PARAM_WONE_NUM:
			if (from_storage)
				*(__u8 *)arg = wd->storage.ldb.wone_num;
			else
				ret = exec_priv_ioctl_get(TIAP_PARAM_WONE_NUM, arg, sizeof(wd->storage.ldb.wone_num),ioctl_sock,wd->dev_name);
			break;
		case TIAP_PARAM_RADIO_CAL:
			if (from_storage)
				*(__u32 *)arg = wd->storage.ldb.radio_cal;
			else
				ret = exec_priv_ioctl_get(TIAP_PARAM_RADIO_CAL, arg, sizeof(wd->storage.ldb.radio_cal),ioctl_sock,wd->dev_name);
			break;
		case TIAP_PARAM_RADIO_CAL_INT:
			if (from_storage)
				*(__u16 *)arg = wd->storage.ldb.cal_interval;
			else
				ret = exec_priv_ioctl_get(TIAP_PARAM_RADIO_CAL_INT, arg, sizeof(wd->storage.ldb.cal_interval),ioctl_sock,wd->dev_name);
			break;
		case TIAP_PARAM_ENERGY_DETECT:
			if (from_storage)
				*(__u8 *)arg = wd->storage.ldb.energy_detect;
			else
				ret = exec_priv_ioctl_get(TIAP_PARAM_ENERGY_DETECT, arg, sizeof(wd->storage.ldb.energy_detect),ioctl_sock,wd->dev_name);
			break;
		case TIAP_PARAM_HIDDEN_SSID:
			if (from_storage)
				*(__u16 *)arg = wd->storage.ldb.hidden_ssid;
			else
				ret = exec_priv_ioctl_get(TIAP_PARAM_HIDDEN_SSID, arg, sizeof(wd->storage.ldb.hidden_ssid),ioctl_sock,wd->dev_name);
			break;
		case TIAP_PARAM_MULTI_DOMAIN_CAPAB:
			if (from_storage)
				*(__u8 *)arg = wd->storage.ldb.multi_domain_cap;
			else
				ret = exec_priv_ioctl_get(TIAP_PARAM_MULTI_DOMAIN_CAPAB, arg, sizeof(wd->storage.ldb.multi_domain_cap),ioctl_sock,wd->dev_name);
			break;
		case TIAP_PARAM_SPECTRUM_MGMT:
			if (from_storage)
				*(__u8 *)arg = wd->storage.ldb.spectrum_mgmt;
			else
				ret = exec_priv_ioctl_get(TIAP_PARAM_SPECTRUM_MGMT, arg, sizeof(wd->storage.ldb.spectrum_mgmt),ioctl_sock,wd->dev_name);
			break;
		case TIAP_PARAM_LEGACY_STA:
			if (from_storage)
				*(__u8 *)arg = wd->storage.ldb.legacy_sta;
			else
				ret = exec_priv_ioctl_get(TIAP_PARAM_LEGACY_STA, arg, sizeof(wd->storage.ldb.legacy_sta),ioctl_sock,wd->dev_name);
			break;
		case TIAP_PARAM_LOCAL_POWER_CONS:
			if (from_storage)
				*(__s8 *)arg = wd->storage.ldb.local_pow_cons;
			else
				ret = exec_priv_ioctl_get(TIAP_PARAM_LOCAL_POWER_CONS, arg, sizeof(wd->storage.ldb.local_pow_cons),ioctl_sock,wd->dev_name);
			break;
		case TIAP_PARAM_MAX_TX_POWER_LEVEL:
			if (from_storage)
				*(__u8 *)arg = wd->storage.ldb.max_txpow_lev;
			else
				ret = exec_priv_ioctl_get(TIAP_PARAM_MAX_TX_POWER_LEVEL, arg, sizeof(wd->storage.ldb.max_txpow_lev),ioctl_sock,wd->dev_name);
			break;
		case TIAP_PARAM_COUNTRY_STR:
			if (from_storage)
				strncpy((char *)arg, wd->storage.ldb.country_str, TIAP_PARAM_COUNTRY_STRING_LEN);
			else
				ret = exec_priv_ioctl_get(TIAP_PARAM_COUNTRY_STR, arg, sizeof(wd->storage.ldb.country_str),ioctl_sock,wd->dev_name);
			break;
		case TIAP_PARAM_OPER_STATE:
			if (from_storage)
				*(__u8 *)arg = wd->storage.ldb.oper_state;
			else
				ret = exec_priv_ioctl_get(TIAP_PARAM_OPER_STATE, arg, sizeof(wd->storage.ldb.oper_state),ioctl_sock,wd->dev_name);
			break;
		case TIAP_PARAM_RADAR_DETECT_MODE:
			if (from_storage)
				*(__u8 *)arg = wd->storage.ldb.radar_detect;
			else
				ret = exec_priv_ioctl_get(TIAP_PARAM_RADAR_DETECT_MODE, arg, sizeof(wd->storage.ldb.radar_detect),ioctl_sock,wd->dev_name);
			break;
		case TIAP_PARAM_BURST_LIMIT:
			if (from_storage)
				*(__u16 *)arg = wd->storage.ldb.txop_limit;
			else
				ret = ioctl_get_burst_limit(arg, sizeof(arg), ioctl_sock, wd->dev_name);
			break;
		case TIAP_PARAM_AP_BRIDGE_MODE:
			if (from_storage)
				*(__u16 *)arg = wd->storage.ldb.bridge_mode;
			else
				ret = exec_priv_ioctl_get(TIAP_PARAM_AP_BRIDGE_MODE, arg,
						sizeof(wd->storage.ldb.bridge_mode),
						ioctl_sock, wd->dev_name);
			break;
			
		case TIAP_PARAM_WDS_ENABLE:
			if (from_storage)
				*(__u8 *)arg = wd->storage.ldb.wdsEnable;
			else ret = exec_priv_ioctl_get(TIAP_PARAM_WDS_ENABLE, arg,
					sizeof(wd->storage.ldb.wdsEnable),
					ioctl_sock, wd->dev_name);
				break;

		case TIAP_PARAM_EEPROM_LESS_ENABLE:
			if (from_storage)
				*(__u8 *)arg = wd->storage.ldb.eeprom_less;
			else
				ret = exec_priv_ioctl_get(TIAP_PARAM_EEPROM_LESS_ENABLE, arg, sizeof(wd->storage.ldb.multi_domain_cap),ioctl_sock,wd->dev_name);
			break;
			
		default:
			ret = 1;
	}

	close(ioctl_sock);

	return ret;
}

int tiap_wcfg_set_auth_type(const wcfgd *wd, const tiap_param_auth_algo_t *tbl, int update_storage)
{
	memcpy(&wd->storage.ldb.auth_tbl, tbl, sizeof(wd->storage.ldb.auth_tbl));

    if(update_storage) {
	if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;
    }

	return 0;
}

int tiap_wcfg_get_auth_type(const wcfgd *wd, tiap_param_auth_algo_t *tbl, int from_storage)
{
	int ret = 0, ioctl_sock;

	if (from_storage)
	{
		memcpy(tbl, wd->storage.ldb.auth_tbl, sizeof(wd->storage.ldb.auth_tbl));
	}
	else
	{
		if((ioctl_sock = iw_sockets_open()) < 0)
		{
			perror("socket");
			return(-1);
		}

		ret = exec_priv_ioctl_get(TIAP_PARAM_AUTH_TYPE, tbl, sizeof(wd->storage.ldb.auth_tbl),ioctl_sock,(char *)wd->dev_name);

		close(ioctl_sock);
	}

	return ret;
}

int tiap_wcfg_set_wd(const wcfgd *wd, const tiap_param_wd_t *w, int update_storage)
{
	switch( w->cmd )
	{
		case TIAP_PARAM_WD_ENABLE:
			memcpy((void *)&wd->storage.ldb.wd_en, (void *)w, sizeof(tiap_param_wd_t));
			break;
		case TIAP_PARAM_WD_INTERF:
			memcpy((void *)&wd->storage.ldb.wd_interf, (void *)w, sizeof(tiap_param_wd_t));
			break;
		case TIAP_PARAM_WD_RXFREE:
			memcpy((void *)&wd->storage.ldb.wd_rxfree, (void *)w, sizeof(tiap_param_wd_t));
			break;
		case TIAP_PARAM_WD_BEACONS:
			memcpy((void *)&wd->storage.ldb.wd_beacons, (void *)w, sizeof(tiap_param_wd_t));
			break;
		case TIAP_PARAM_WD_TXSTUCK:
			memcpy((void *)&wd->storage.ldb.wd_txstuck, (void *)w, sizeof(tiap_param_wd_t));
			break;
		case TIAP_PARAM_WD_FCSERR:
			memcpy((void *)&wd->storage.ldb.wd_fcserrs, (void *)w, sizeof(tiap_param_wd_t));
			break;
		case TIAP_PARAM_WD_ACI:
			memcpy((void *)&wd->storage.ldb.wd_aci, (void *)w, sizeof(tiap_param_wd_t));
			break;
		case TIAP_PARAM_WD_NB:
			memcpy((void *)&wd->storage.ldb.wd_nb, (void *)w, sizeof(tiap_param_wd_t));
			break;
		default:
			return 1;
	}

	if (update_storage){
		if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
			return 1;
	}

	return 0;

}


int tiap_wcfg_get_wd(const wcfgd *wd, const tiap_param_wd_t *w, int from_storage)
{
	int ret = 0, ioctl_sock;

	if (from_storage)
	{
		switch( w->cmd )
		{
			case TIAP_PARAM_WD_ENABLE:
				memcpy((void *)w, (void *)&wd->storage.ldb.wd_en, sizeof(tiap_param_wd_t));
				break;
			case TIAP_PARAM_WD_INTERF:
				memcpy((void *)w, (void *)&wd->storage.ldb.wd_interf, sizeof(tiap_param_wd_t));
				break;
			case TIAP_PARAM_WD_RXFREE:
				memcpy((void *)w, (void *)&wd->storage.ldb.wd_rxfree, sizeof(tiap_param_wd_t));
				break;
			case TIAP_PARAM_WD_BEACONS:
				memcpy((void *)w, (void *)&wd->storage.ldb.wd_beacons, sizeof(tiap_param_wd_t));
				break;
			case TIAP_PARAM_WD_TXSTUCK:
				memcpy((void *)w, (void *)&wd->storage.ldb.wd_txstuck, sizeof(tiap_param_wd_t));
				break;
			case TIAP_PARAM_WD_FCSERR:
				memcpy((void *)w, (void *)&wd->storage.ldb.wd_fcserrs, sizeof(tiap_param_wd_t));
				break;
			case TIAP_PARAM_WD_ACI:
				memcpy((void *)w, (void *)&wd->storage.ldb.wd_aci, sizeof(tiap_param_wd_t));
				break;
			case TIAP_PARAM_WD_NB:
				memcpy((void *)w, (void *)&wd->storage.ldb.wd_nb, sizeof(tiap_param_wd_t));
				break;	
		}
	}
	else
	{

		if((ioctl_sock = iw_sockets_open()) < 0)
		{
			perror("socket");
			return(-1);
		}
		ret = exec_priv_ioctl_get(TIAP_PARAM_WD, (void *)w, sizeof(tiap_param_wd_t),ioctl_sock, (char *)wd->dev_name);

		close(ioctl_sock);
	}

	return ret;
}

int tiap_wcfg_get_interf_wd_config(const wcfgd *wd, tiap_param_interference_wd_t *param, int from_storage)
{
	int ret = 0, ioctl_sock;

	if (from_storage) {
	
		switch(param->cmd)
		{
		case TIAP_PARAM_ACI_TIMEOUT:
			param->value.val32 = wd->storage.ldb.aci_state_timeout ;
			break;
		case TIAP_PARAM_NB_TIMEOUT:
			 param->value.val32 = wd->storage.ldb.nb_state_timeout;
			break;
		case TIAP_PARAM_ACI_RATE_THR:
			param->value.val32 = wd->storage.ldb.aci_rate_thresh;
			break;
		case TIAP_PARAM_ACI_EVENT_THR:
			param->value.val32 = wd->storage.ldb.aci_event_thresh;
			break;
		case TIAP_PARAM_PD_LOW:
			param->value.val16 = wd->storage.ldb.pd_low_thresh;
			break;
		case TIAP_PARAM_PD_HIGH:
			param->value.val16 = wd->storage.ldb.pd_high_thresh;
			break;
		case TIAP_PARAM_MEDIUM_BUSY_THR:
			param->value.val32 = wd->storage.ldb.medium_busy_thresh;
			break;
		}
	}
	else
	{
		 if((ioctl_sock = iw_sockets_open()) < 0)
		{
			perror("socket");
			return(-1);
		}
		ret = exec_priv_ioctl_get(TIAP_PARAM_INTERF_WD_CONFIG, (void *)param, sizeof(tiap_param_interference_wd_t),ioctl_sock, (char *)wd->dev_name);

		close(ioctl_sock);
	
	}
	return 0;
		
}

int tiap_wcfg_set_interf_wd_config(wcfgd *wd, tiap_param_interference_wd_t *param, int update_storage)
{
	switch(param->cmd)
	{
	case TIAP_PARAM_ACI_TIMEOUT:
		wd->storage.ldb.aci_state_timeout = param->value.val32;
		break;
	case TIAP_PARAM_NB_TIMEOUT:
		wd->storage.ldb.nb_state_timeout = param->value.val32;
		break;
	case TIAP_PARAM_ACI_RATE_THR:
		wd->storage.ldb.aci_rate_thresh = param->value.val32;
		break;
	case TIAP_PARAM_ACI_EVENT_THR:
		wd->storage.ldb.aci_event_thresh = param->value.val32;
		break;
	case TIAP_PARAM_PD_LOW:
		wd->storage.ldb.pd_low_thresh = param->value.val16;
		break;
	case TIAP_PARAM_PD_HIGH:
		wd->storage.ldb.pd_high_thresh = param->value.val16;
		break;
	case TIAP_PARAM_MEDIUM_BUSY_THR:
		wd->storage.ldb.medium_busy_thresh = param->value.val32;
		break;
	}

	if(update_storage){
		if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
			return 1;
	}

	return 0;
		
}


void shrink_table(void *tbl, int type, int idx, int tbl_len, int entry_size)
{
	__s8 (*a)[ETH_ALEN] = tbl;
	__s8 (*m)[IW_ESSID_MAX_SIZE+1] = tbl;

	while( idx < (tbl_len - 1) )
	{
		if( !type ) { /*access list*/
			a = tbl;
			memcpy(a[idx], a[idx+1], entry_size);
		}
		else if( type == 1 ) { /*multiple ssid*/
			m = tbl;
			memcpy(m[idx], m[idx+1], entry_size);
		}
		else printf("%s: invalid type = %d\n", __func__, type);
		idx++;
	}

	memset((!type ? a[tbl_len-1] : m[tbl_len-1]), 0, entry_size);
}


int tiap_wcfg_set_qos_ac_params(wcfgd * wd, const tiap_param_qos_t * params, int num, int update_storage)
{
	int i;

	for (i=0; i<4; ++i)
		wd->storage.ldb.qos.edcf_info[num][i] = params->edcf_info[num][i];
	/*
	wd->storage.ldb.qos.edcf_info[num].cwMin = params->cwMin;
	wd->storage.ldb.qos.edcf_info[num].cwMax = params->cwMax;
	wd->storage.ldb.qos.edcf_info[num].aifsn = params->aifsn;
	wd->storage.ldb.qos.edcf_info[num].txopLimit = params->txopLimit;
	wd->storage.ldb.qos.edcf_info[num].txMaxLifetime = params->txMaxLifetime;
	wd->storage.ldb.qos.edcf_info[num].ackPolicy = params->ackPolicy;
	wd->storage.ldb.qos.edcf_info[num].shortRetryLimit = params->shortRetryLimit;
	wd->storage.ldb.qos.edcf_info[num].longRetryLimit = params->longRetryLimit;
	*/

    if(update_storage) {
	if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;
    }


	return 0;
}

int tiap_wcfg_set_qos_params(wcfgd * wd, const tiap_param_qos_t * params, int update_storage) 
{
    if(params->enabled && wd->storage.ldb.wdsEnable) {
        printf("%s : WDS is enabled.\n", __FUNCTION__);
        return 1;
    }
    wd->storage.ldb.qos.enabled = params->enabled;
	wd->storage.ldb.qos.classifierMode = params->classifierMode;

	if (params->classifierMode == TIAP_PARAM_QOS_CLASSIFIER_MODE_DIFFSERV) {
		wd->storage.ldb.qos.diffserv.numOfCP = params->diffserv.numOfCP;
		memcpy(wd->storage.ldb.qos.diffserv.map, params->diffserv.map, sizeof(params->diffserv.map));
	}
		

    if(update_storage) {
	if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;
    }


	return 0;
}

int tiap_wcfg_get_qos_params(wcfgd * wd, tiap_param_qos_t * params, int from_storage) {
	int ret = 0, ioctl_sock;

	if (from_storage)	{
		*params = wd->storage.ldb.qos;
	}	else	{
		if ((ioctl_sock = iw_sockets_open()) < 0) {
			perror("socket");
			return(-1);
		}
		
		ret = exec_priv_ioctl_get(TIAP_PARAM_QOS, params, sizeof(wd->storage.ldb.qos),ioctl_sock,(char *)wd->dev_name);
		
		close(ioctl_sock);
	}

	return ret;
}
																				 
int tiap_wcfg_set_access_list(wcfgd *wd, const tiap_param_access_list_t *tbl, int update_storage)
{
	int ioctl_sock, ret = 0;

	if((ioctl_sock = iw_sockets_open()) < 0)
	{
		perror("socket");
		return(-1);
	}

	switch( tbl->cmd ) /* No matter what is the cmd value only two values are written
			   to NVRAM: TIAP_PARAM_ACCESS_LST (for all cmds except TIAP_PARAM_ACCESS_RESET)
			   and TIAP_PARAM_ACCESS_RESET*/ 
	{
		case TIAP_PARAM_ACCESS_SET_TYPE:
			wd->storage.ldb.access_list.enabled = tbl->enabled;
			wd->storage.ldb.access_list.type = tbl->type;
			wd->storage.ldb.access_list.cmd = TIAP_PARAM_ACCESS_LST;
			break;
		case TIAP_PARAM_ACCESS_LST:
			wd->storage.ldb.access_list.enabled = tbl->enabled;
			wd->storage.ldb.access_list.type = tbl->type;
			wd->storage.ldb.access_list.cmd = tbl->cmd;
			memcpy(wd->storage.ldb.access_list.list, tbl->list,
					sizeof(wd->storage.ldb.access_list.list));
			break;
		case TIAP_PARAM_ACCESS_ADD:
			wd->storage.ldb.access_list.enabled = tbl->enabled;
			ret = exec_priv_ioctl_set(TIAP_PARAM_ACCESS_LIST, 
					(void *)tbl,sizeof(wd->storage.ldb.access_list),
					ioctl_sock, (char *)wd->dev_name);
			if (!ret) /* ioctl succeeded which means that MAC was also checked for validity*/
			{
				/* then add the MAC to the access list in the storage */ 
				int i;
				char hwaddr[ETH_ALEN];

				memset(hwaddr, 0, ETH_ALEN);
				for(i=0; i<TIAP_PARAM_ACCESS_LIST_SIZE; i++)
				{
					/* if such MAC already exists we abort the setting*/
					if( !memcmp(wd->storage.ldb.access_list.list[i], tbl->list[0], ETH_ALEN) ) {
						close(ioctl_sock);
						return 0;
					}
					
					if( !memcmp(wd->storage.ldb.access_list.list[i], hwaddr, ETH_ALEN) )
						break;
				}
				if( i == TIAP_PARAM_ACCESS_LIST_SIZE ) {
					close(ioctl_sock);
					return 1;
				}
				memcpy(wd->storage.ldb.access_list.list[i], tbl->list[0], ETH_ALEN);
				wd->storage.ldb.access_list.cmd = TIAP_PARAM_ACCESS_LST;
			}

			
			break;

		case TIAP_PARAM_ACCESS_DEL_BY_INDEX:
			wd->storage.ldb.access_list.enabled = tbl->enabled;
			wd->storage.ldb.access_list.cmd = TIAP_PARAM_ACCESS_LST;

			shrink_table(wd->storage.ldb.access_list.list, 0,
					tbl->index-1, TIAP_PARAM_ACCESS_LIST_SIZE, ETH_ALEN);
			
			ret = exec_priv_ioctl_set(TIAP_PARAM_ACCESS_LIST, 
					(void *)tbl,sizeof(wd->storage.ldb.access_list),
					ioctl_sock, (char *)wd->dev_name);
			break;

		case TIAP_PARAM_ACCESS_DEL_BY_ADDR:
			wd->storage.ldb.access_list.enabled = tbl->enabled;
			wd->storage.ldb.access_list.cmd = TIAP_PARAM_ACCESS_LST;
			{
				int i;
				for(i=0; i<TIAP_PARAM_ACCESS_LIST_SIZE; i++)
					if( !memcmp(wd->storage.ldb.access_list.list[i], tbl->list[0], ETH_ALEN) )
						break;
				shrink_table(wd->storage.ldb.access_list.list, 0,
					i, TIAP_PARAM_ACCESS_LIST_SIZE, ETH_ALEN);
			}
			ret = exec_priv_ioctl_set(TIAP_PARAM_ACCESS_LIST, (void *)tbl,sizeof(wd->storage.ldb.access_list),ioctl_sock, (char *)wd->dev_name);
			wd->storage.ldb.access_list.cmd = TIAP_PARAM_ACCESS_LST;
			break;

		case TIAP_PARAM_ACCESS_RESET:
			if( tbl->enabled ) {
				close(ioctl_sock);
				return 1;
			}
			memset(wd->storage.ldb.access_list.list, 0, sizeof(wd->storage.ldb.access_list.list));
			wd->storage.ldb.access_list.cmd = TIAP_PARAM_ACCESS_RESET;
			break;
	}
	
	if ( !ret && update_storage ){	
		if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) ) {
			close(ioctl_sock);
			return 1;
		}
	}

	close(ioctl_sock);

	return ret;
}


int tiap_wcfg_get_access_list(wcfgd *wd, tiap_param_access_list_t *tbl, int from_storage)
{
	int ret = 0, ioctl_sock;

	if (from_storage)
	{
		memcpy(tbl, &wd->storage.ldb.access_list, sizeof(wd->storage.ldb.access_list));
	}
	else
	{	
	
		if((ioctl_sock = iw_sockets_open()) < 0)
		{
			perror("socket");
			return(-1);
		}

		ret = exec_priv_ioctl_get(TIAP_PARAM_ACCESS_LIST, tbl, sizeof(wd->storage.ldb.access_list),ioctl_sock,(char *)wd->dev_name);

		close(ioctl_sock);
	}

	return ret;
}

int tiap_wcfg_set_4x(wcfgd *wd, const tiap_param_4x_t *_4x, int idx, const int update_storage)
{
	switch( _4x->cmd )
	{
		case TIAP_PARAM_4X_FEATSET:
			memcpy((void *)&wd->storage.ldb._4x_fs, (void *)_4x, sizeof(wd->storage.ldb._4x_fs));
			/* when setting/resetting FeatureSet, the ERPProtectionTI must be set/reset too */
			((wcfgd *)wd)->storage.ldb.erp_ti[idx].elem.erp_ti = _4x->value;
			break;
		case TIAP_PARAM_4X_4X:
			if ((wd->storage.ldb._4x_4x.cmd   == _4x->cmd) &&
				(wd->storage.ldb._4x_4x.value == _4x->value))
			{
				/* if the given 4x value remained unchanged - do nothing */
				break;
			}
			else
			{
			memcpy((void *)&wd->storage.ldb._4x_4x, (void *)_4x, sizeof(wd->storage.ldb._4x_4x));
			if (_4x->value == 1)
			{
				((wcfgd *)wd)->storage.ldb.frag = TIAP_PARAM_MAX_FRAGMENTATION_TRESHOLD;
				((wcfgd *)wd)->storage.ldb.rts = TIAP_PARAM_RTS_TRESHOLD_MAX;
			}
			else if (_4x->value == 0)
			{

				((wcfgd *)wd)->storage.ldb.frag = TIAP_PARAM_FRAG_TRESH_DEFAULT;
				((wcfgd *)wd)->storage.ldb.rts = TIAP_PARAM_RTS_TRESHOLD_DEFAULT;
			}
			}

			break;
		case TIAP_PARAM_4X_CAT:
			memcpy((void *)&wd->storage.ldb._4x_cat, (void *)_4x, sizeof(wd->storage.ldb._4x_cat));
			break;
		case TIAP_PARAM_4X_PKTBURST:
			memcpy((void *)&wd->storage.ldb._4x_pb, (void *)_4x, sizeof(wd->storage.ldb._4x_pb));
			break;
		case TIAP_PARAM_4X_MIXMOD:
			memcpy((void *)&wd->storage.ldb._4x_mm, (void *)_4x, sizeof(wd->storage.ldb._4x_mm));
			break;
		default:
			return 1;
	}

	if( update_storage && wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;

	return 0;
}

int tiap_wcfg_get_4x(const wcfgd *wd, tiap_param_4x_t *_4x, int from_storage)
{
	int ret = 0, ioctl_sock;

	if (from_storage)
	{
		switch( _4x->cmd )
		{
			case TIAP_PARAM_4X_FEATSET:
				memcpy((void *)_4x, (void *)&wd->storage.ldb._4x_fs, sizeof(wd->storage.ldb._4x_fs));
				break;
			case TIAP_PARAM_4X_4X:
				memcpy((void *)_4x, (void *)&wd->storage.ldb._4x_4x, sizeof(wd->storage.ldb._4x_4x));
				break;
			case TIAP_PARAM_4X_CAT:
				memcpy((void *)_4x, (void *)&wd->storage.ldb._4x_cat, sizeof(wd->storage.ldb._4x_cat));
				break;
			case TIAP_PARAM_4X_PKTBURST:
				memcpy((void *)_4x, (void *)&wd->storage.ldb._4x_pb, sizeof(wd->storage.ldb._4x_pb));
				break;
			case TIAP_PARAM_4X_MIXMOD:
				memcpy((void *)_4x, (void *)&wd->storage.ldb._4x_mm, sizeof(wd->storage.ldb._4x_mm));
				break;
		}
	}

	else
	{
		if((ioctl_sock = iw_sockets_open()) < 0)
		{
			perror("socket");
			return(-1);
		}

		ret = exec_priv_ioctl_get(TIAP_PARAM_4X, _4x, sizeof(tiap_param_4x_t),ioctl_sock,(char *)wd->dev_name);

		close(ioctl_sock);
	}

	return ret;
}

int tiap_wcfg_set_multi_ssid(wcfgd *wd, const tiap_param_multi_ssid_t *multi_ssid, int update_storage)
{
    int i;
    __u32 cipher;
    tiap_param_security_type_t  sec_type;
    tiap_param_auth_algo_t tbl[TIAP_PARAM_MAX_AUTH_ALG_SUPPORTED];

    if(multi_ssid->secondary_enabled && wd->storage.ldb.wdsEnable) {
        printf("%s : WDS is enabled.\n", __FUNCTION__);
        return 1;
    }
    memcpy((void *)&wd->storage.ldb.multi_ssid, (void *)multi_ssid,
				sizeof(wd->storage.ldb.multi_ssid));
	if( update_storage && wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;

    /* If Multiple SSIDs is Enabled and there is 
       a profile configured with static WEP -->
       set the Authentication Algorithm to Both
       Open System and Shared Key.               */ 
    if(multi_ssid->secondary_enabled == 1)
    {
        /* Look for a profile with Static WEP */
        for (i=0; i < TIAP_PARAM_NUM_MULTIPLE_SSID; i++) 
		{
			if( (multi_ssid->tbl[i].ssid_type == TIAP_PARAM_SSID_PRIMARY) ||
                (multi_ssid->tbl[i].ssid_type == TIAP_PARAM_SSID_SECONDARY) )
            {
                if(tiap_wcfg_get_security_type(wd, &sec_type ,i))
                    return 1;

                if(tiap_wcfg_get_default_cipher(wd, &cipher ,i))
                    return 1;
                
                if( (sec_type == TIAP_PARAM_SECURITY_WEP) && 
                    (cipher != TIAP_WPA_CIPHER_NON) )
                {
                    /* Found it! - Set Auth Algo to Both */
                    tbl[0].algo = TIAP_PARAM_MIB_OPEN_SYSTEM;
                    tbl[1].algo = TIAP_PARAM_MIB_SHARED_KEY;
                    tbl[0].enable = TRUE;
                    tbl[1].enable = TRUE;

                    if(tiap_wcfg_set_auth_type(wd, tbl, update_storage))
                        return 1;

                    break;
                }
            }
        }
    }

	return 0;
}

int tiap_wcfg_get_multi_ssid(const wcfgd *wd, tiap_param_multi_ssid_t *multi_ssid, int from_storage)
{
	int ret = 0, ioctl_sock;

	if (from_storage)
		memcpy((void *)multi_ssid, (void *)&wd->storage.ldb.multi_ssid, sizeof(wd->storage.ldb.multi_ssid));
	else
	{
		if((ioctl_sock = iw_sockets_open()) < 0)
		{
			perror("socket");
			return(-1);
		}
		ret = exec_priv_ioctl_get(TIAP_PARAM_MULTI_SSID, (tiap_param_multi_ssid_t *)multi_ssid, sizeof(wd->storage.ldb.multi_ssid),ioctl_sock,(char *)wd->dev_name);

		close(ioctl_sock);
	}

	return ret;
}

int tiap_wcfg_set_cfg_pow_tbl(wcfgd *wd, const tiap_param_radio_t *radio, int update_storage)
{
	int ret = 0, ioctl_sock;

	if((ioctl_sock = iw_sockets_open()) < 0) {
		perror("socket");
		return ioctl_sock;
	}
	
	memcpy((void *)&wd->storage.ldb.radio, (const void *)radio, sizeof(wd->storage.ldb.radio));

	if (update_storage) {
		if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) ) {
			close(ioctl_sock);
			return 1;
		}
	}
	
	ret = exec_priv_ioctl_set(TIAP_PARAM_CFG_POWER_TABLE, (tiap_param_radio_t *)radio,
			sizeof(*radio), ioctl_sock, (char *)wd->dev_name);

	close(ioctl_sock);
	return ret;
}

int tiap_wcfg_set_ch_range(const wcfgd *wd, const tiap_param_channel_rang *ch_rang, int update_storage)
{
	if( ch_rang->type == TIAP_PARAM_RANG_A )
		memcpy((void *)&wd->storage.ldb.ch_rang_a, ch_rang, sizeof(wd->storage.ldb.ch_rang_a));
	else if( ch_rang->type == TIAP_PARAM_RANG_G )
		memcpy((void *)&wd->storage.ldb.ch_rang_g, ch_rang, sizeof(wd->storage.ldb.ch_rang_g));
	else return 1;

    if(update_storage) {
	if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;
    }


	return 0;
}

int tiap_wcfg_get_ch_range(const wcfgd *wd, tiap_param_channel_rang *ch_rang, int from_storage)
{
	int ret = 0, ioctl_sock;

	if (from_storage)
	{
		if( ch_rang->type == TIAP_PARAM_RANG_A )
			memcpy(ch_rang, &wd->storage.ldb.ch_rang_a, sizeof(wd->storage.ldb.ch_rang_a));
		else if( ch_rang->type == TIAP_PARAM_RANG_G )
			memcpy(ch_rang, &wd->storage.ldb.ch_rang_g, sizeof(wd->storage.ldb.ch_rang_g));
	}
	else
	{

		if((ioctl_sock = iw_sockets_open()) < 0)
		{
			perror("socket");
			return(-1);
		}

		ret = exec_priv_ioctl_get(TIAP_PARAM_CHANNEL_RANGE, ch_rang, sizeof(tiap_param_channel_rang),ioctl_sock,(char *)wd->dev_name);

		close(ioctl_sock);
	}

	return ret;
}

int tiap_wcfg_set_vhcf(const wcfgd *wd, const tiap_param_vhcf_t *vhcf, const int update_storage)
{
	memcpy((void *)&wd->storage.ldb.vhcf, vhcf, sizeof(wd->storage.ldb.vhcf));

	if( update_storage && wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;

	return 0;
}

int tiap_wcfg_get_vhcf(const wcfgd *wd, tiap_param_vhcf_t *vhcf, int from_storage)
{
	int ret = 0, ioctl_sock;

	if (from_storage)
	{
		memcpy((void *)vhcf, (void *)&wd->storage.ldb.vhcf, sizeof(wd->storage.ldb.vhcf));
	}
	else
	{
		if((ioctl_sock = iw_sockets_open()) < 0)
		{
			perror("socket");
			return(-1);
		}
		ret = exec_priv_ioctl_get(TIAP_PARAM_VHCF, vhcf, sizeof(tiap_param_vhcf_t),ioctl_sock,(char *)wd->dev_name);
		close(ioctl_sock);
	}

	return ret;
}


int tiap_wcfg_set_wone(wcfgd *wd, const tiap_param_wone_t *wone, const int update_storage)
{
	unsigned int i = wone->index;

	if( i > TIAP_PARAM_WONE_DUAL_BAND - 1 ) return 1;

	switch( wone->type )
	{
		case wone_oper_rate:
			wd->storage.ldb.oper_rate[i].elem.b_legacy = wone->elem.b_legacy;
			break;
		case wone_oper_add:
			wd->storage.ldb.oper_add[i].elem.ofdm_legacy = wone->elem.ofdm_legacy;
			break;
		case wone_oper_ofdm:
			wd->storage.ldb.oper_ofdm[i].elem.ofdm_extend = wone->elem.ofdm_extend;
			break;
		case wone_ch:
			wd->storage.ldb.ch[i].elem.ch = wone->elem.ch;
			break;
		case wone_band:
			wd->storage.ldb.band[i].elem.band = wone->elem.band;
			break;
		case wone_pbcc:
			wd->storage.ldb.pbcc[i].elem.pbcc_mode = wone->elem.pbcc_mode;
			break;
		case wone_rate_adapt:
			wd->storage.ldb.rate_adapt[i].elem.rate_adapt = wone->elem.rate_adapt;
			break;
		case wone_pream_mode:
			wd->storage.ldb.pream_mode[i].elem.pream_mode = wone->elem.pream_mode;
			break;
		case wone_slot_mode:
			wd->storage.ldb.slot_mode[i].elem.slot_mode = wone->elem.slot_mode;
			break;
		case wone_erp_mode:
			wd->storage.ldb.erp_mode[i].elem.erp_mode = wone->elem.erp_mode;
			break;
		case wone_erp_type:
			wd->storage.ldb.erp_type[i].elem.erp_type = wone->elem.erp_type;
			break;
		case wone_erp_ti:
			wd->storage.ldb.dot11_mode[i].elem.erp_ti = wone->elem.erp_ti;
			break;
		case wone_dot11_mode:
			wd->storage.ldb.dot11_mode[i].elem.dot11_mode = wone->elem.dot11_mode;
			break;
		case wone_cwmin_mode:
			wd->storage.ldb.cwmin_mode[i].elem.cwmin_mode = wone->elem.cwmin_mode;
			break;
		case wone_usage_time:
			wd->storage.ldb.usage_time[i].elem.usage_time = wone->elem.usage_time;
			break;

		default:
			return 1;
	}

	if( update_storage && wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;

	return 0;
}

int tiap_wcfg_get_wone(const wcfgd *wd, tiap_param_wone_t *wone, int from_storage)
{
	int ret = 0, ioctl_sock;
	unsigned int i = wone->index;

	if( i > TIAP_PARAM_WONE_DUAL_BAND - 1 ) return 1;

	if (from_storage)
	{
		switch( wone->type )
		{
			case wone_oper_rate:
				wone->elem.b_legacy = wd->storage.ldb.oper_rate[i].elem.b_legacy;
				break;
			case wone_oper_add:
				wone->elem.ofdm_legacy = wd->storage.ldb.oper_add[i].elem.ofdm_legacy;
				break;
			case wone_oper_ofdm:
				wone->elem.ofdm_extend = wd->storage.ldb.oper_ofdm[i].elem.ofdm_extend;
				break;
			case wone_ch:
				wone->elem.ch = wd->storage.ldb.ch[i].elem.ch;
				break;
			case wone_band:
				wone->elem.band = wd->storage.ldb.band[i].elem.band;
				break;
			case wone_pbcc:
				wone->elem.pbcc_mode = wd->storage.ldb.pbcc[i].elem.pbcc_mode;
				break;
			case wone_rate_adapt:
				wone->elem.rate_adapt = wd->storage.ldb.rate_adapt[i].elem.rate_adapt;
				break;
			case wone_pream_mode:
				wone->elem.pream_mode = wd->storage.ldb.pream_mode[i].elem.pream_mode;
				break;
			case wone_slot_mode:
				wone->elem.slot_mode = wd->storage.ldb.slot_mode[i].elem.slot_mode;
				break;
			case wone_erp_mode:
				wone->elem.erp_mode = wd->storage.ldb.erp_mode[i].elem.erp_mode;
				break;
			case wone_erp_type:
				wone->elem.erp_type = wd->storage.ldb.erp_type[i].elem.erp_type;
				break;
			case wone_erp_ti:
				wone->elem.erp_ti = wd->storage.ldb.erp_ti[i].elem.erp_ti;
				break;
			case wone_dot11_mode:
				wone->elem.dot11_mode = wd->storage.ldb.dot11_mode[i].elem.dot11_mode;
				break;
			case wone_cwmin_mode:
				wone->elem.cwmin_mode = wd->storage.ldb.cwmin_mode[i].elem.cwmin_mode;
				break;
			case wone_usage_time:
				wone->elem.usage_time = wd->storage.ldb.usage_time[i].elem.usage_time;
				break;
		}
	}
	else
	{
		if((ioctl_sock = iw_sockets_open()) < 0)
		{
			perror("socket");
			return(-1);
		}

		ret = exec_priv_ioctl_get(TIAP_PARAM_WONE_ELEM, (void *)wone, sizeof(tiap_param_wone_t),
					ioctl_sock,(char *)wd->dev_name);

		close(ioctl_sock);
	}
	
	return ret;


}


void set_dot11mode(struct wcfg_storage_t *ldb, int dot11mode, int idx)
{
	switch( dot11mode )
	{
		case TIAP_PARAM_D11MODE_MIXED:

			ldb->oper_rate[idx].index = idx;
			ldb->oper_rate[idx].type = wone_oper_rate;
			{
				__u8 rates[TIAP_PARAM_RATES_MAX] =
				{ TIAP_PARAM_RATE_BASIC_SUPPORTED, TIAP_PARAM_RATE_BASIC_SUPPORTED,
					TIAP_PARAM_RATE_BASIC_SUPPORTED, TIAP_PARAM_RATE_BASIC_SUPPORTED,
					TIAP_PARAM_RATE_SUPPORTED };
					memcpy((void *)&ldb->oper_rate[idx].elem.b_legacy.rates, (void *)&rates,
							sizeof(ldb->oper_rate[idx].elem.b_legacy.rates));
			}
			ldb->oper_rate[idx].elem.b_legacy.len = 5;


			ldb->oper_add[idx].index = idx;
			ldb->oper_add[idx].type = wone_oper_add;
			{
				__u8 rates[TIAP_PARAM_RATES_MAX] =
				{ TIAP_PARAM_RATE_NOT_SUPPORTED, TIAP_PARAM_RATE_NOT_SUPPORTED,
					TIAP_PARAM_RATE_NOT_SUPPORTED, TIAP_PARAM_RATE_NOT_SUPPORTED };
					memcpy((void *)&ldb->oper_add[idx].elem.ofdm_legacy.rates, (void *)&rates,
							sizeof(ldb->oper_add[idx].elem.b_legacy.rates));
			}
			ldb->oper_add[idx].elem.ofdm_legacy.len = 0;

			ldb->oper_ofdm[idx].index = idx;
			ldb->oper_ofdm[idx].type = wone_oper_ofdm;
			{
				__u8 rates[TIAP_PARAM_RATES_MAX] = 
				{ TIAP_PARAM_RATE_SUPPORTED, TIAP_PARAM_RATE_SUPPORTED,
					TIAP_PARAM_RATE_SUPPORTED, TIAP_PARAM_RATE_SUPPORTED,
					TIAP_PARAM_RATE_SUPPORTED, TIAP_PARAM_RATE_SUPPORTED,
					TIAP_PARAM_RATE_SUPPORTED, TIAP_PARAM_RATE_SUPPORTED };
					memcpy((void *)&ldb->oper_ofdm[idx].elem.ofdm_extend.rates, (void *)&rates,
							sizeof(ldb->oper_ofdm[idx].elem.ofdm_extend.rates));
			}
			ldb->oper_ofdm[idx].elem.ofdm_extend.len = 8;

			ldb->pbcc[idx].index = idx;
			ldb->pbcc[idx].type = wone_pbcc;
			ldb->pbcc[idx].elem.pbcc_mode = 4;

			ldb->pream_mode[idx].index = idx;
			ldb->pream_mode[idx].type = wone_pream_mode;
			ldb->pream_mode[idx].elem.pream_mode = 2;

			ldb->slot_mode[idx].index = idx;
			ldb->slot_mode[idx].type = wone_slot_mode;
			ldb->slot_mode[idx].elem.slot_mode = 2;

			ldb->erp_mode[idx].index = idx;
			ldb->erp_mode[idx].type = wone_erp_mode;
			ldb->erp_mode[idx].elem.erp_mode = 3;

			ldb->erp_type[idx].index = idx;
			ldb->erp_type[idx].type = wone_erp_type;
			ldb->erp_type[idx].elem.erp_type = 1;

			break;

		case TIAP_PARAM_D11MODE_B:

			ldb->oper_rate[idx].index = idx;
			ldb->oper_rate[idx].type = wone_oper_rate;
			{
				__u8 rates[TIAP_PARAM_RATES_MAX] =
				{ TIAP_PARAM_RATE_BASIC_SUPPORTED, TIAP_PARAM_RATE_BASIC_SUPPORTED,
					TIAP_PARAM_RATE_BASIC_SUPPORTED, TIAP_PARAM_RATE_BASIC_SUPPORTED };
					memcpy((void *)&ldb->oper_rate[idx].elem.b_legacy.rates, (void *)&rates,
							sizeof(ldb->oper_rate[idx].elem.b_legacy.rates));
			}
			ldb->oper_rate[idx].elem.b_legacy.len = 4;

			ldb->oper_add[idx].index = idx;
			ldb->oper_add[idx].type = wone_oper_add;
			{
				__u8 rates[TIAP_PARAM_RATES_MAX] =
				{ TIAP_PARAM_RATE_NOT_SUPPORTED, TIAP_PARAM_RATE_NOT_SUPPORTED,
					TIAP_PARAM_RATE_NOT_SUPPORTED, TIAP_PARAM_RATE_NOT_SUPPORTED };
					memcpy((void *)&ldb->oper_add[idx].elem.ofdm_legacy.rates, (void *)&rates,
							sizeof(ldb->oper_add[idx].elem.b_legacy.rates));
			}
			ldb->oper_add[idx].elem.ofdm_legacy.len = 0;

			ldb->oper_ofdm[idx].index = idx;
			ldb->oper_ofdm[idx].type = wone_oper_ofdm;
			memset((void *)&ldb->oper_ofdm[idx].elem.ofdm_extend.rates, 0,
					sizeof(ldb->oper_ofdm[idx].elem.ofdm_extend.rates));
			ldb->oper_ofdm[idx].elem.ofdm_extend.len = 0;

			ldb->pbcc[idx].index = idx;
			ldb->pbcc[idx].type = wone_pbcc;
			ldb->pbcc[idx].elem.pbcc_mode = 0;

			ldb->pream_mode[idx].index = idx;
			ldb->pream_mode[idx].type = wone_pream_mode;
			ldb->pream_mode[idx].elem.pream_mode = 2; /* local stations */

			ldb->slot_mode[idx].index = idx;
			ldb->slot_mode[idx].type = wone_slot_mode;
			ldb->slot_mode[idx].elem.slot_mode = 2; /* local stations */

			ldb->erp_mode[idx].index = idx;
			ldb->erp_mode[idx].type = wone_erp_mode;
			ldb->erp_mode[idx].elem.erp_mode = 0; /* no protection */

			ldb->erp_type[idx].index = idx;
			ldb->erp_type[idx].type = wone_erp_type;
			ldb->erp_type[idx].elem.erp_type = 0;

			break;

		case TIAP_PARAM_D11MODE_B_PLUS:

			ldb->oper_rate[idx].index = idx;
			ldb->oper_rate[idx].type = wone_oper_rate;
			{
				__u8 rates[TIAP_PARAM_RATES_MAX] =
				{ TIAP_PARAM_RATE_BASIC_SUPPORTED, TIAP_PARAM_RATE_BASIC_SUPPORTED,
					TIAP_PARAM_RATE_BASIC_SUPPORTED, TIAP_PARAM_RATE_BASIC_SUPPORTED,
					TIAP_PARAM_RATE_SUPPORTED };
					memcpy((void *)&ldb->oper_rate[idx].elem.b_legacy.rates, (void *)&rates,
							sizeof(ldb->oper_rate[idx].elem.b_legacy.rates));
			}
			ldb->oper_rate[idx].elem.b_legacy.len = 5;

			ldb->oper_add[idx].index = idx;
			ldb->oper_add[idx].type = wone_oper_add;
			{
				__u8 rates[TIAP_PARAM_RATES_MAX] =
				{ TIAP_PARAM_RATE_NOT_SUPPORTED, TIAP_PARAM_RATE_NOT_SUPPORTED,
					TIAP_PARAM_RATE_NOT_SUPPORTED, TIAP_PARAM_RATE_NOT_SUPPORTED };
					memcpy((void *)&ldb->oper_add[idx].elem.ofdm_legacy.rates, (void *)&rates,
							sizeof(ldb->oper_add[idx].elem.b_legacy.rates));
			}
			ldb->oper_add[idx].elem.ofdm_legacy.len = 0;

			ldb->oper_ofdm[idx].index = idx;
			ldb->oper_ofdm[idx].type = wone_oper_ofdm;
			{
				__u8 rates[TIAP_PARAM_RATES_MAX] = 
				{ TIAP_PARAM_RATE_NOT_SUPPORTED, TIAP_PARAM_RATE_NOT_SUPPORTED,
					TIAP_PARAM_RATE_NOT_SUPPORTED, TIAP_PARAM_RATE_NOT_SUPPORTED };
					memcpy((void *)&ldb->oper_ofdm[idx].elem.ofdm_extend.rates, (void *)&rates,
							sizeof(ldb->oper_ofdm[idx].elem.ofdm_extend.rates));
			}
			ldb->oper_ofdm[idx].elem.ofdm_extend.len = 0;

			ldb->pbcc[idx].index = idx;
			ldb->pbcc[idx].type = wone_pbcc;
			ldb->pbcc[idx].elem.pbcc_mode = 4;

			ldb->pream_mode[idx].index = idx;
			ldb->pream_mode[idx].type = wone_pream_mode;
			ldb->pream_mode[idx].elem.pream_mode = 0;

			ldb->slot_mode[idx].index = idx;
			ldb->slot_mode[idx].type = wone_slot_mode;
			ldb->slot_mode[idx].elem.slot_mode = 0;

			ldb->erp_mode[idx].index = idx;
			ldb->erp_mode[idx].type = wone_erp_mode;
			ldb->erp_mode[idx].elem.erp_mode = 0;

			ldb->erp_type[idx].index = idx;
			ldb->erp_type[idx].type = wone_erp_type;
			ldb->erp_type[idx].elem.erp_type = 0;

			break;


		case TIAP_PARAM_D11MODE_OFDM:

			ldb->oper_rate[idx].index = idx;
			ldb->oper_rate[idx].type = wone_oper_rate;
			{
				__u8 rates[TIAP_PARAM_RATES_MAX] =
				{ TIAP_PARAM_RATE_BASIC_SUPPORTED, TIAP_PARAM_RATE_BASIC_SUPPORTED,
					TIAP_PARAM_RATE_BASIC_SUPPORTED, TIAP_PARAM_RATE_BASIC_SUPPORTED };
					memcpy((void *)&ldb->oper_rate[idx].elem.b_legacy.rates, (void *)&rates,
							sizeof(ldb->oper_rate[idx].elem.b_legacy.rates));
			}
			ldb->oper_rate[idx].elem.b_legacy.len = 4;

			ldb->oper_add[idx].index = idx;
			ldb->oper_add[idx].type = wone_oper_add;
			memset((void *)&ldb->oper_add[idx].elem.ofdm_legacy.rates, TIAP_PARAM_RATE_NOT_SUPPORTED,
					sizeof(ldb->oper_add[idx].elem.b_legacy.rates));
			ldb->oper_add[idx].elem.ofdm_legacy.len = 0;

			ldb->oper_ofdm[idx].index = idx;
			ldb->oper_ofdm[idx].type = wone_oper_ofdm;
			{
				__u8 rates[TIAP_PARAM_RATES_MAX] = 
				{ TIAP_PARAM_RATE_BASIC_SUPPORTED, TIAP_PARAM_RATE_SUPPORTED,
					TIAP_PARAM_RATE_BASIC_SUPPORTED, TIAP_PARAM_RATE_SUPPORTED,
					TIAP_PARAM_RATE_BASIC_SUPPORTED, TIAP_PARAM_RATE_SUPPORTED,
					TIAP_PARAM_RATE_SUPPORTED, TIAP_PARAM_RATE_SUPPORTED };
					memcpy((void *)&ldb->oper_ofdm[idx].elem.ofdm_extend.rates, (void *)&rates,
							sizeof(ldb->oper_ofdm[idx].elem.ofdm_extend.rates));
			}
			ldb->oper_ofdm[idx].elem.ofdm_extend.len = 8;

			ldb->pbcc[idx].index = idx;
			ldb->pbcc[idx].type = wone_pbcc;
			ldb->pbcc[idx].elem.pbcc_mode = 0;

			ldb->pream_mode[idx].index = idx;
			ldb->pream_mode[idx].type = wone_pream_mode;
			ldb->pream_mode[idx].elem.pream_mode = 2;

			ldb->slot_mode[idx].index = idx;
			ldb->slot_mode[idx].type = wone_slot_mode;
			ldb->slot_mode[idx].elem.slot_mode = 2;

			ldb->erp_mode[idx].index = idx;
			ldb->erp_mode[idx].type = wone_erp_mode;
			ldb->erp_mode[idx].elem.erp_mode = 3;

			ldb->erp_type[idx].index = idx;
			ldb->erp_type[idx].type = wone_erp_type;
			ldb->erp_type[idx].elem.erp_type = 1;

			break;


		case TIAP_PARAM_D11MODE_A:

			ldb->oper_rate[idx].index = idx;
			ldb->oper_rate[idx].type = wone_oper_rate;
			{
				__u8 rates[TIAP_PARAM_RATES_MAX] =
				{ TIAP_PARAM_RATE_NOT_SUPPORTED, TIAP_PARAM_RATE_NOT_SUPPORTED,
					TIAP_PARAM_RATE_NOT_SUPPORTED, TIAP_PARAM_RATE_NOT_SUPPORTED };
					memcpy((void *)&ldb->oper_rate[idx].elem.b_legacy.rates, (void *)&rates,
							sizeof(ldb->oper_rate[idx].elem.b_legacy.rates));
			}
			ldb->oper_rate[idx].elem.b_legacy.len = 0;

			ldb->oper_add[idx].index = idx;
			ldb->oper_add[idx].type = wone_oper_add;
			{
				__u8 rates[TIAP_PARAM_RATES_MAX] =
				{ TIAP_PARAM_RATE_BASIC_SUPPORTED, TIAP_PARAM_RATE_SUPPORTED,
					TIAP_PARAM_RATE_BASIC_SUPPORTED, TIAP_PARAM_RATE_SUPPORTED,
					TIAP_PARAM_RATE_BASIC_SUPPORTED, TIAP_PARAM_RATE_SUPPORTED,
					TIAP_PARAM_RATE_SUPPORTED, TIAP_PARAM_RATE_SUPPORTED };
					memcpy((void *)&ldb->oper_add[idx].elem.ofdm_legacy.rates, (void *)&rates,
							sizeof(ldb->oper_add[idx].elem.b_legacy.rates));
			}
			ldb->oper_add[idx].elem.ofdm_legacy.len = 8;

			ldb->oper_ofdm[idx].index = idx;
			ldb->oper_ofdm[idx].type = wone_oper_ofdm;
			{
				__u8 rates[TIAP_PARAM_RATES_MAX] = 
				{ TIAP_PARAM_RATE_NOT_SUPPORTED, TIAP_PARAM_RATE_NOT_SUPPORTED,
					TIAP_PARAM_RATE_NOT_SUPPORTED, TIAP_PARAM_RATE_NOT_SUPPORTED };
					memcpy((void *)&ldb->oper_ofdm[idx].elem.ofdm_extend.rates, (void *)&rates,
							sizeof(ldb->oper_ofdm[idx].elem.ofdm_extend.rates));
			}
			ldb->oper_ofdm[idx].elem.ofdm_extend.len = 0;

			ldb->pbcc[idx].index = idx;
			ldb->pbcc[idx].type = wone_pbcc;
			ldb->pbcc[idx].elem.pbcc_mode = 0;

			ldb->pream_mode[idx].index = idx;
			ldb->pream_mode[idx].type = wone_pream_mode;
			ldb->pream_mode[idx].elem.pream_mode = 0;

			ldb->slot_mode[idx].index = idx;
			ldb->slot_mode[idx].type = wone_slot_mode;
			ldb->slot_mode[idx].elem.slot_mode = 1;

			ldb->erp_mode[idx].index = idx;
			ldb->erp_mode[idx].type = wone_erp_mode;
			ldb->erp_mode[idx].elem.erp_mode = 0;

			ldb->erp_type[idx].index = idx;
			ldb->erp_type[idx].type = wone_erp_type;
			ldb->erp_type[idx].elem.erp_type = 1;

			break;

		default: return;
	}
}


int tiap_wcfg_set_dot11mode(wcfgd *wd, int dot11mode, int wone_index, int update_storage)
{
	
	set_dot11mode(&wd->storage.ldb, dot11mode, wone_index);
	wd->storage.ldb.dot11_mode[wone_index].elem.dot11_mode = dot11mode;
	
	if( dot11mode == TIAP_PARAM_D11MODE_A) {
            
		__u8 band;

		band = wd->storage.ldb.band[wone_index].elem.band;
		switch (wd->storage.ldb.cur_regdomain[band].type)
        {
            case TIAP_PARAM_RD_FCC:
			case TIAP_PARAM_RD_US:
				wd->storage.ldb.cur_regdomain[TIAP_PARAM_RADIO_BAND_5GHZ].band = TIAP_PARAM_RADIO_BAND_5GHZ;
                wd->storage.ldb.cur_regdomain[TIAP_PARAM_RADIO_BAND_5GHZ].type = TIAP_PARAM_RD_US;
                break;

            case TIAP_PARAM_RD_IC:
            case TIAP_PARAM_RD_ETSI:
            case TIAP_PARAM_RD_SPAIN:
            case TIAP_PARAM_RD_FRANCE:
            case TIAP_PARAM_RD_MKK:
            case TIAP_PARAM_RD_MKK1:
            case TIAP_PARAM_RD_ISRAEL:
			case TIAP_PARAM_RD_WORLD:
				wd->storage.ldb.cur_regdomain[TIAP_PARAM_RADIO_BAND_5GHZ].band = TIAP_PARAM_RADIO_BAND_5GHZ;
                wd->storage.ldb.cur_regdomain[TIAP_PARAM_RADIO_BAND_5GHZ].type = TIAP_PARAM_RD_WORLD;
                break;
            default:
				break;
        }
    
		wd->storage.ldb.ch[wone_index].elem.ch = TIAP_PARAM_DEF_A_CHANNEL;
		wd->storage.ldb.band[wone_index].elem.band = TIAP_PARAM_RADIO_BAND_5GHZ;	
	}
	else {
		__u8 wone_nr, radar_detect;

		if (wd->storage.ldb.band[wone_index].elem.band == TIAP_PARAM_RADIO_BAND_5GHZ)
		{
			wd->storage.ldb.ch[wone_index].elem.ch = TIAP_PARAM_DEF_CHANNEL;
			wd->storage.ldb.band[wone_index].elem.band = TIAP_PARAM_RADIO_BAND_2_4GHZ;
		}
		/* in all modes except TIAP_PARAM_D11MODE_A (in single band)
		   radar detect should be disabled*/

		/* check if AP is in single band */
		if (tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, FROM_STORAGE))
			return 1;

		/* if it is - disable radar detect */
		if(wone_nr == TIAP_PARAM_WONE_SINGLE_BAND)
		{
			radar_detect = 0;
			if (tiap_wcfg_set_basic(wd, TIAP_PARAM_RADAR_DETECT_MODE, 
						&radar_detect, UPDATE_STORAGE)) /*only for you*/
				return 1;
			printf("wcfglib: Setting radar detect mode to Disabled.\n");
		}

	}

	if (update_storage)
	{
		if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
			return 1;
	}

	return 0;
}

int tiap_wcfg_get_dot11mode(wcfgd *wd, __u8 *dot11_mode, int wone_index, int from_storage)
{
	tiap_param_wone_t wone;
	int ret = 0;


	if (from_storage)
	{
		*dot11_mode = wd->storage.ldb.dot11_mode[wone_index].elem.dot11_mode;
	}
	else
	{
		int ioctl_sock;

		if((ioctl_sock = iw_sockets_open()) < 0)
		{
			perror("socket");
			return(-1);
		}

		wone.type = wone_dot11_mode;
		wone.index = wone_index;

		ret = exec_priv_ioctl_get(TIAP_PARAM_WONE_ELEM, &wone, 
				sizeof(wone),ioctl_sock, wd->dev_name);

		*dot11_mode = wone.elem.dot11_mode;
		close(ioctl_sock);
	}

	return ret;  


}

int tiap_wcfg_config_wpaauth(const wcfgd *wd, const tiap_wpa_auth_t *wa, int update_storage)
{
	memcpy((void *)&wd->storage.wpa, wa, sizeof(wd->storage.wpa));

    if(update_storage) {
	if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;
    }


	return 0;
}



int tiap_wcfg_get_sta_table(wcfgd *wd, tiap_param_statbl_t *sta_tbl)
{
	int ioctl_sock, ret;

	if((ioctl_sock = iw_sockets_open()) < 0)
	{
		perror("socket");
		return(-1);
	}

	ret = exec_priv_ioctl_get(TIAP_PARAM_STA_TBL, sta_tbl, 
			sizeof(tiap_param_statbl_t)*TIAP_PARAM_MAX_STATIONS,ioctl_sock, wd->dev_name);

	close(ioctl_sock);

	return ret;  
}


int tiap_wcfg_get_actual_ssid_ind(tiap_param_multi_ssid_t *multi_ssid, int ssid_seq_num, 
								  int *actual_index)
{
	int i, num_active = 0;

	for (i=0; i<TIAP_PARAM_NUM_MULTIPLE_SSID; i++) 
	{
		/* Omit non-active and internal since it is not configurable so user cannot see it */
		if ((multi_ssid->tbl[i].ssid_type == TIAP_PARAM_SSID_NON_ACTIVE) ||
            (multi_ssid->tbl[i].ssid_type == TIAP_PARAM_SSID_INTERNAL))
			continue;
			
		if (num_active == ssid_seq_num) 
			break;		

		num_active++;
	}

	if (i == TIAP_PARAM_NUM_MULTIPLE_SSID) 
	{
		printf("%s: failed to get actual ssid index\n",__func__);
		return 1;
	}

	*actual_index = i;
	return 0;

}

int tiap_wcfg_set_wep_default_key_id(wcfgd *wd, __u8 key_id,  int profile_index, int update_storage)
{
    int sec_profile_index, ssid_index; 

	if( tiap_wcfg_get_actual_ssid_ind(&wd->storage.ldb.multi_ssid, profile_index, &ssid_index) )
		return 2;

	sec_profile_index = wd->storage.ldb.multi_ssid.tbl[ssid_index].sec_profile_index;
	wd->storage.wpa.profile[sec_profile_index].sec.wep_def_key_id = key_id;

    if(update_storage) {
	if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;
    }
	
	return 0;

}

int tiap_wcfg_get_wep_default_key_id(wcfgd *wd, __u8 *key_id,int profile_index)
{
	int sec_profile_index, ssid_index; 

	if( tiap_wcfg_get_actual_ssid_ind(&wd->storage.ldb.multi_ssid, profile_index, &ssid_index) )
		return 2;

	/* retrieve the corresponding security profile index from multi ssid table */
	sec_profile_index = wd->storage.ldb.multi_ssid.tbl[ssid_index].sec_profile_index;
	*key_id = wd->storage.wpa.profile[sec_profile_index].sec.wep_def_key_id;

	return 0;
}
	

int tiap_wcfg_set_def_wep_key(wcfgd *wd, int key_idx, tiap_wpa_wep_key_t *wep_key,
		int profile_index, const int update_storage)
{
	int sec_profile_index, ssid_index; 

	if( key_idx > TIAP_WPA_DEFS_KEY_NAL_LEN-1 )
		return 1;

	if( tiap_wcfg_get_actual_ssid_ind(&wd->storage.ldb.multi_ssid, profile_index, &ssid_index) )
		return 2;

	/* retrieve the corresponding security profile index from multi ssid table */
	sec_profile_index = wd->storage.ldb.multi_ssid.tbl[ssid_index].sec_profile_index;

	wd->storage.wpa.profile[sec_profile_index].sec.keys[key_idx].len = wep_key->len;
	memcpy(wd->storage.wpa.profile[sec_profile_index].sec.keys[key_idx].key, wep_key->key, wep_key->len);

	if( update_storage && wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;
	
	return 0;
}

int tiap_wcfg_get_def_wep_key(wcfgd *wd, int key_idx, tiap_wpa_wep_key_t *wep_key, int profile_index)
{
	int sec_profile_index, ssid_index; 

	if( key_idx > TIAP_WPA_DEFS_KEY_NAL_LEN-1 )
		return 1;
	
	if( tiap_wcfg_get_actual_ssid_ind(&wd->storage.ldb.multi_ssid, profile_index, &ssid_index) )
		return 2;

	/* retrieve the corresponding security profile index from multi ssid table */
	sec_profile_index = wd->storage.ldb.multi_ssid.tbl[ssid_index].sec_profile_index;
	wep_key->len = wd->storage.wpa.profile[sec_profile_index].sec.keys[key_idx].len;
	memcpy(wep_key->key, wd->storage.wpa.profile[sec_profile_index].sec.keys[key_idx].key, wep_key->len);
	
	return 0;
}

void  print_mac(const char *mac)
{
	fprintf(stderr,"%02hhx:%02hhx:%02hhx:%02hhx:%02hhx:%02hhx\n", 
			     mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]); 
}


int tiap_wcfg_set_cur_regdomain(wcfgd *wd, tiap_param_cur_reg_domain_t *cur_regdomain,
		const int update_storage)
{
	int band;

	band = cur_regdomain->band;
	wd->storage.ldb.cur_regdomain[band] = *cur_regdomain;

	if( update_storage && wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;

	return 0;
}


int tiap_wcfg_get_cur_regdomain(wcfgd *wd, tiap_param_cur_reg_domain_t *cur_regdomain, int from_storage)
{
	int ioctl_sock, ret = 0;

	if (from_storage){
		int band;
		
		band = cur_regdomain->band;
		*cur_regdomain = wd->storage.ldb.cur_regdomain[band]; 
	}
	else
	{
		if((ioctl_sock = iw_sockets_open()) < 0)
		{
			perror("socket");
			return(-1);
		}

		ret = exec_priv_ioctl_get(TIAP_PARAM_CURR_REG_DOMAIN, cur_regdomain, 
				sizeof(*cur_regdomain), ioctl_sock, wd->dev_name);

		close(ioctl_sock);
	}

	return ret;  

}


int tiap_wcfg_set_priv_regdomain(wcfgd *wd, tiap_param_priv_reg_domain_t *priv_regdomain,
		const int update_storage)
{
	wd->storage.ldb.priv_regdomain = *priv_regdomain;

	if( update_storage && wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;

	return 0;
}


int tiap_wcfg_get_priv_regdomain(wcfgd *wd, tiap_param_priv_reg_domain_t *priv_regdomain, int from_storage)
{
	int ioctl_sock, ret = 0;

	if (from_storage)
		*priv_regdomain = wd->storage.ldb.priv_regdomain;
	else
	{
		if((ioctl_sock = iw_sockets_open()) < 0)
		{
			perror("socket");
			return(-1);
		}

		ret = exec_priv_ioctl_get(TIAP_PARAM_PRIV_REG_DOMAIN, priv_regdomain,
				sizeof(*priv_regdomain), ioctl_sock, wd->dev_name);

		close(ioctl_sock);
	}

	return ret;  

}

int tiap_wcfg_get_priv_net_stats(wcfgd *wd, tiap_param_net_stats *stats)
{
	int ioctl_sock, ret;

	if((ioctl_sock = iw_sockets_open()) < 0) {
		perror("socket");
		return(-1);
	}

	ret = exec_priv_ioctl_get(TIAP_PARAM_NET_STATS, stats, sizeof(*stats), ioctl_sock, wd->dev_name);
	close(ioctl_sock);

	return ret;
}

int tiap_wcfg_set_defaults(wcfgd *wd)
{
	wcfg_storage_setdefaults(&wd->storage);
	wd->storage.defaults = 1;

	if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;

	return 0;
}

int tiap_wcfg_set_psk_passphrase_mode(wcfgd *wd, int value,int profile_index, int update_storage)
{
	int sec_profile_index, ssid_index; 

	if( tiap_wcfg_get_actual_ssid_ind(&wd->storage.ldb.multi_ssid, profile_index, &ssid_index) )
		return 2;

	/* retrieve the corresponding security profile index from multi ssid table */
	sec_profile_index = wd->storage.ldb.multi_ssid.tbl[ssid_index].sec_profile_index;
	wd->storage.wpa.profile[sec_profile_index].psk.pass_phrase_mode = value;
    if(update_storage) {
	if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;
    }


	return 0;

}

int tiap_wcfg_get_psk_passphrase_mode(wcfgd *wd, int *value,int profile_index)
{
	int sec_profile_index, ssid_index; 

	if( tiap_wcfg_get_actual_ssid_ind(&wd->storage.ldb.multi_ssid, profile_index, &ssid_index) )
		return 2;

	/* retrieve the corresponding security profile index from multi ssid table */
	sec_profile_index = wd->storage.ldb.multi_ssid.tbl[ssid_index].sec_profile_index;
	*value = wd->storage.wpa.profile[sec_profile_index].psk.pass_phrase_mode;

	return 0;
}


int tiap_wcfg_set_psk_passwd(wcfgd *wd, __s8 *passwd, __u8 passwd_len, int profile_index, 
                             int update_storage)
{
	int sec_profile_index, ssid_index; 

	if( tiap_wcfg_get_actual_ssid_ind(&wd->storage.ldb.multi_ssid, profile_index, &ssid_index) )
		return 2;

	/* retrieve the corresponding security profile index from multi ssid table */
	sec_profile_index = wd->storage.ldb.multi_ssid.tbl[ssid_index].sec_profile_index;
	wd->storage.wpa.profile[sec_profile_index].psk.psk_passwd_len = passwd_len;
	memcpy(wd->storage.wpa.profile[sec_profile_index].psk.psk_passwd, passwd, passwd_len);
    if(update_storage) {
	if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;
    }

	return 0;

}

int tiap_wcfg_get_psk_passwd(wcfgd *wd, __s8 *passwd, __u8 *passwd_len,int profile_index)
{
	int sec_profile_index, ssid_index; 

	if( tiap_wcfg_get_actual_ssid_ind(&wd->storage.ldb.multi_ssid, profile_index, &ssid_index) )
		return 2;

	/* retrieve the corresponding security profile index from multi ssid table */
	sec_profile_index = wd->storage.ldb.multi_ssid.tbl[ssid_index].sec_profile_index;

	*passwd_len = wd->storage.wpa.profile[sec_profile_index].psk.psk_passwd_len;
	memcpy(passwd, wd->storage.wpa.profile[sec_profile_index].psk.psk_passwd, 
		   wd->storage.wpa.profile[sec_profile_index].psk.psk_passwd_len);

	return 0;
}

int tiap_wcfg_set_default_cipher(wcfgd *wd, __u32 cipher,int profile_index, int update_storage)
{
	int sec_profile_index, ssid_index; 

	if( tiap_wcfg_get_actual_ssid_ind(&wd->storage.ldb.multi_ssid, profile_index, &ssid_index) )
		return 2;

	/* retrieve the corresponding security profile index from multi ssid table */
	sec_profile_index = wd->storage.ldb.multi_ssid.tbl[ssid_index].sec_profile_index;

	wd->storage.wpa.profile[sec_profile_index].sec.default_cipher = cipher;
    if(update_storage) {
	if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;
    }


	return 0;

}

int tiap_wcfg_get_default_cipher(wcfgd *wd, __u32 *cipher,int profile_index)
{
	int sec_profile_index, ssid_index; 

	if( tiap_wcfg_get_actual_ssid_ind(&wd->storage.ldb.multi_ssid, profile_index, &ssid_index) )
		return 2;

	/* retrieve the corresponding security profile index from multi ssid table */
	sec_profile_index = wd->storage.ldb.multi_ssid.tbl[ssid_index].sec_profile_index;

	*cipher = wd->storage.wpa.profile[sec_profile_index].sec.default_cipher ;

	return 0;
}

int tiap_wcfg_set_gtk_interval(wcfgd *wd, __u32 intvl,int profile_index, int update_storage)
{
	int sec_profile_index, ssid_index;

	if( tiap_wcfg_get_actual_ssid_ind(&wd->storage.ldb.multi_ssid, profile_index, &ssid_index) )
		return 2;

	/* retrieve the corresponding security profile index from multi ssid table */
	sec_profile_index = wd->storage.ldb.multi_ssid.tbl[ssid_index].sec_profile_index;

	wd->storage.wpa.profile[sec_profile_index].sec.interval = intvl;
    if(update_storage) {
	if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;
    }

	return 0;

}

int tiap_wcfg_get_gtk_interval(wcfgd *wd, __u32 *intvl,int profile_index)
{
	int sec_profile_index, ssid_index; 

	if( tiap_wcfg_get_actual_ssid_ind(&wd->storage.ldb.multi_ssid, profile_index, &ssid_index) )
		return 2;

	/* retrieve the corresponding security profile index from multi ssid table */
	sec_profile_index = wd->storage.ldb.multi_ssid.tbl[ssid_index].sec_profile_index;

	*intvl = wd->storage.wpa.profile[sec_profile_index].sec.interval;
	
	return 0;	
}

int tiap_wcfg_set_radius_port(wcfgd *wd, __u16 port,int profile_index, int update_storage)
{
	int sec_profile_index, ssid_index; 

	if( tiap_wcfg_get_actual_ssid_ind(&wd->storage.ldb.multi_ssid, profile_index, &ssid_index) )
		return 2;

	/* retrieve the corresponding security profile index from multi ssid table */
	sec_profile_index = wd->storage.ldb.multi_ssid.tbl[ssid_index].sec_profile_index;

	wd->storage.wpa.profile[sec_profile_index].radius.port = port;
	
    if(update_storage) {
	if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;
    }
	
	return 0;
}

int tiap_wcfg_get_radius_port(wcfgd *wd, __u16 *port,int profile_index)
{
	int sec_profile_index, ssid_index; 

	if( tiap_wcfg_get_actual_ssid_ind(&wd->storage.ldb.multi_ssid, profile_index, &ssid_index) )
		return 2;

	/* retrieve the corresponding security profile index from multi ssid table */
	sec_profile_index = wd->storage.ldb.multi_ssid.tbl[ssid_index].sec_profile_index;

	*port = wd->storage.wpa.profile[sec_profile_index].radius.port;
	
	return 0;	
}

int tiap_wcfg_debug_jtag(wcfgd *wd, char enable) {
	wd->storage.debug.jtag_enabled = enable;

	if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;
	
	return 0;
}

int tiap_wcfg_debug_automatic_fw_load(wcfgd *wd, char enable) {
	wd->storage.debug.automatic_fw_load_enabled = enable;

	if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;
	
	return 0;
}

int tiap_wcfg_debug_set_server_ip(wcfgd *wd, __u32 ipaddr) {
	wd->storage.debug.debug_server_ip_addr = ipaddr;

	if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;
	
	return 0;
}

int tiap_wcfg_set_radius_ipaddr(wcfgd *wd, __u32 ipaddr,int profile_index, int update_storage)
{
	int sec_profile_index, ssid_index; 

	if( tiap_wcfg_get_actual_ssid_ind(&wd->storage.ldb.multi_ssid, profile_index, &ssid_index) )
		return 2;

	/* retrieve the corresponding security profile index from multi ssid table */
	sec_profile_index = wd->storage.ldb.multi_ssid.tbl[ssid_index].sec_profile_index;
	wd->storage.wpa.profile[sec_profile_index].radius.serv_ip_addr = ipaddr;
	
    if(update_storage) {
	if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;
    }
	
	return 0;
}

int tiap_wcfg_get_radius_ipaddr(wcfgd *wd, __u32 *ipaddr,int profile_index)
{
	int sec_profile_index, ssid_index; 

	if( tiap_wcfg_get_actual_ssid_ind(&wd->storage.ldb.multi_ssid, profile_index, &ssid_index) )
		return 2;

	/* retrieve the corresponding security profile index from multi ssid table */
	sec_profile_index = wd->storage.ldb.multi_ssid.tbl[ssid_index].sec_profile_index;

	*ipaddr = wd->storage.wpa.profile[sec_profile_index].radius.serv_ip_addr ;

	return 0;
}

int tiap_wcfg_get_radius_secret(wcfgd *wd, __s8 *secret, __u8 *len,int profile_index)
{
	int sec_profile_index, ssid_index; 

	if( tiap_wcfg_get_actual_ssid_ind(&wd->storage.ldb.multi_ssid, profile_index, &ssid_index) )
		return 2;

	/* retrieve the corresponding security profile index from multi ssid table */
	sec_profile_index = wd->storage.ldb.multi_ssid.tbl[ssid_index].sec_profile_index;

	*len = wd->storage.wpa.profile[sec_profile_index].radius.secret_len;

	memcpy(secret, wd->storage.wpa.profile[sec_profile_index].radius.secret, *len);

	return 0;
}

int tiap_wcfg_set_radius_secret(wcfgd *wd, __s8 *secret, __u8 len,int profile_index, int update_storage)
{
	int sec_profile_index, ssid_index; 

	if( len > TIAP_WPA_MAX_RADIUS_SECRET_LEN )
		return 1;

	if( tiap_wcfg_get_actual_ssid_ind(&wd->storage.ldb.multi_ssid, profile_index, &ssid_index) )
		return 2;

	/* retrieve the corresponding security profile index from multi ssid table */
	sec_profile_index = wd->storage.ldb.multi_ssid.tbl[ssid_index].sec_profile_index;

	wd->storage.wpa.profile[sec_profile_index].radius.secret_len = len;
	memcpy(wd->storage.wpa.profile[sec_profile_index].radius.secret, secret, len);
	/*wd->storage.wpa.profile[sec_profile_index].radius.secret[len] = '\0';*/
    if(update_storage) {
	if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;
    }
	
	return 0;
}


int tiap_wcfg_set_pre_auth_flag(wcfgd *wd, int flag,int profile_index, int update_storage)
{
	int sec_profile_index, ssid_index; 

	if( flag != 0 && flag != 1)
		return 1;

	if( tiap_wcfg_get_actual_ssid_ind(&wd->storage.ldb.multi_ssid, profile_index, &ssid_index) )
		return 2;

	/* retrieve the corresponding security profile index from multi ssid table */
	sec_profile_index = wd->storage.ldb.multi_ssid.tbl[ssid_index].sec_profile_index;

	wd->storage.wpa.profile[sec_profile_index].radius.pre_auth_flag = flag;
    if(update_storage) {
        if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
            return 1;
    }
	
	return 0;
}


int tiap_wcfg_get_pre_auth_flag(wcfgd *wd, __u8* flag ,int profile_index)
{
	int sec_profile_index, ssid_index; 

	if( tiap_wcfg_get_actual_ssid_ind(&wd->storage.ldb.multi_ssid, profile_index, &ssid_index) )
		return 2;

	/* retrieve the corresponding security profile index from multi ssid table */
	sec_profile_index = wd->storage.ldb.multi_ssid.tbl[ssid_index].sec_profile_index;

	*flag = wd->storage.wpa.profile[sec_profile_index].radius.pre_auth_flag;

	return 0;
}


int tiap_wcfg_set_dual_ap(wcfgd *wd, __u8 wone_nr, int update_storage)
{

	if (wone_nr == wd->storage.ldb.wone_num)
	{
	      printf("AP stays in the same wone mode\n");
	      return 0;
	}

	if ( wone_nr < wd->storage.ldb.wone_num )
	{
		if( tiap_wcfg_set_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, UPDATE_STORAGE) )
			return 1;
	}
	else if (wone_nr > wd->storage.ldb.wone_num )
	{
		int beacon_period;

		if( tiap_wcfg_set_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, !UPDATE_STORAGE) )
			return 1;

		if ( tiap_wcfg_set_dot11mode(wd, TIAP_PARAM_D11MODE_MIXED, 0, !UPDATE_STORAGE ) )
			return 1;
		if ( tiap_wcfg_set_dot11mode(wd, TIAP_PARAM_D11MODE_A, 1, !UPDATE_STORAGE ) )
			return 1;

		beacon_period = TIAP_PARAM_DEF_WONE_BEACON_PERIOD;

		if ( tiap_wcfg_set_basic(wd, TIAP_PARAM_BEACON_PERIOD,
					  &beacon_period, update_storage) )
						
			return 1;

	}

	return 0;

}

static int set_radar_detect(wcfgd *wd, __u8 val)
{
	__u8 wone_nr, dot11_mode;

	if (val) /* enable */
	{
		/* Allow to enable radar detect only if AP works in TIAP_PARAM_D11MODE_A 
		   if AP works in dual band it is garanteered that one of two bands is TIAP_PARAM_D11MODE_A*/
		
		/* So we check if AP is in single band */
		if (tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, FROM_STORAGE))
			return 1;
		
		if(wone_nr == TIAP_PARAM_WONE_SINGLE_BAND)
		{
			if (tiap_wcfg_get_dot11mode(wd, &dot11_mode, 0, FROM_STORAGE))
				return 1;
			
			if (dot11_mode != TIAP_PARAM_D11MODE_A)
			{
				printf("Radar detect cannot be enabled while AP is not in A_Only mode\n");
				return 1;
			}

		}
	}
			

	/* in all other cases radar detect can be set */
	wd->storage.ldb.radar_detect = val;

	return 0;
}

int tiap_wcfg_set_gpio_owner(wcfgd *wd, __u8 val, int update_storage)
{
	wd->storage.ldb.gpio_owner = val;
	
    if(update_storage) {
	if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;
    }

	return 0;
}


int tiap_wcfg_enableWds(wcfgd *wd, __u8 enable, int update_storage)
{

    if(enable && wd->storage.ldb.wdsEnable) {
        printf("WDS is already enabled.\n");
        return (-1);
    }

    if(!enable && !wd->storage.ldb.wdsEnable) {
        printf("WDS is already disabled.\n");
        return (-1);
    }

    if(!enable) {
        tiap_wcfg_set_basic(wd, TIAP_PARAM_WDS_ENABLE, &enable, update_storage);
        dda_wdsRelease(0);
        wd->wdsmngr_pid = 0;
    }
    else {
        pid_t wdsPid;

        if(wd->storage.ldb.multi_ssid.secondary_enabled) {
            fprintf(stderr, "%s : Multiple SSID is enabled.\n", __FUNCTION__);
            dda_wdsRelease(0);
            return (1);
        }
        if(wd->storage.ldb.qos.enabled) {
            fprintf(stderr, "%s : WME is enabled.\n", __FUNCTION__);
            dda_wdsRelease(0);
            return (1);
        }

        printf("Start WDS\n");
        if(dda_wdsStart(&wdsPid) != OK) {
            fprintf(stderr, "%s : Error starting WDS manager.\n", __FUNCTION__);
        }
        wd->wdsmngr_pid = wdsPid;
        /* WDS pid set */
        if(ioctl_set_wds_pid(wd->dev_name, wd->wdsmngr_pid) != 0) {
            fprintf(stderr, "%s %d: ioctl_set_wds_pid returns error.\n", __FUNCTION__, __LINE__);
            dda_wdsRelease(0);
            return (-1);
        }
        
		tiap_wcfg_set_basic(wd, TIAP_PARAM_WDS_ENABLE, &enable, update_storage);
		if(tiap_wcfg_configWds(wd, 0, 0)) {
            /* Take action here */
            fprintf(stderr, "%s %d: tiap_wcfg_configWds returns error.\n", __FUNCTION__, __LINE__);
            dda_wdsRelease(0);
            return (-1);
        }
        dda_wdsAddIfToBridge(wd->br_name);
        
    }

    if( wcfg_write_cfg(wd) )
    {
        fprintf(stderr, "%s %d: Failed to write %s configuration file.\n", __FUNCTION__, __LINE__, WCFG_CFGFILE);
        dda_wdsRelease(0);
        return (-1);
    }

    return (0);
}
int tiap_wcfg_configWds(const wcfgd *wd, int restart, int update_storage)
{
    wdsMngrCfg_s cfg;

    if(wd->storage.ldb.multi_ssid.secondary_enabled) {
        fprintf(stderr, "%s : Multiple SSID is enabled.\n", __FUNCTION__);
        dda_wdsRelease(0);
        return (1);
    }
    if(wd->storage.ldb.qos.enabled) {
        fprintf(stderr, "%s : WME is enabled.\n", __FUNCTION__);
        dda_wdsRelease(0);
        return (1);
    }

    cfg.hop = wd->storage.wds.hop;
    cfg.chSelectMode = wd->storage.wds.chSelectMode;
    cfg.privacy = wd->storage.wds.privacy;
    cfg.autoCfg = wd->storage.wds.autoCfg;
    cfg.wirelessIfMode = wd->storage.wds.wirelessIfMode;
    cfg.crudeMode = wd->storage.wds.crudeMode;
    strncpy(cfg.name, wd->storage.wds.name, WDS_NAME_MAX_LEN);
    strncpy(cfg.sharedSecret, wd->storage.wds.sharedSecret, WDS_SHARED_SECRET_STR_LEN);
    memcpy(cfg.downlinkMac0, wd->storage.wds.downlinkMac0, ETH_ALEN);
    memcpy(cfg.downlinkMac1, wd->storage.wds.downlinkMac1, ETH_ALEN);
    memcpy(cfg.downlinkMac2, wd->storage.wds.downlinkMac2, ETH_ALEN);
    memcpy(cfg.downlinkMac3, wd->storage.wds.downlinkMac3, ETH_ALEN);
    memcpy(cfg.uplinkMac, wd->storage.wds.uplinkMac, ETH_ALEN);

    memcpy(cfg.bss_id, wd->storage.ldb.hwaddr, ETH_ALEN);
    strncpy(cfg.brgDevName, wd->br_name, WDS_NAME_MAX_LEN);
    strncpy(cfg.wlanDevName, wd->dev_name, WDS_NAME_MAX_LEN);

    if(restart) {
        if(dda_wdsRestart(TRUE) != OK) {
            fprintf(stderr, "%s : Error restart WDS manager.\n", __FUNCTION__);
            dda_wdsRelease(0);
            return (1);
        }
    }
    if(dda_wdsPassConfig(&cfg) != OK) {
        fprintf(stderr, "%s : Error configure WDS manager.\n", __FUNCTION__);
        dda_wdsRelease(0);
        return (1);
    }
    if(update_storage) {
        if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
            return (1);
    }
    return (0);
}

int tiap_wcfg_setWdsDefaults(wcfgd *wd)
{
    wcfg_storage_setdefaults_wds(&wd->storage.wds);
    if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
        return (1);
    return (0);

}

int tiap_wcfg_wdsHopSet(wcfgd *wd, __u8 hop, int update_storage)
{
    if((hop != 0) && (hop != 1)) {
        printf("Hop number must be 0 for root device or 1 for any other.\n");
        return (1);
    }
    wd->storage.wds.hop = hop;
    if(update_storage) {
        if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
            return (1);
    }
    return (0);
}

void tiap_wcfg_wdsHopGet(const wcfgd *wd, __u8 *pHop)
{
    *pHop = wd->storage.wds.hop;
}

int tiap_wcfg_wdsChSelModeSet(wcfgd *wd, __u8 chSelectMode, int update_storage)
{
	if( (chSelectMode) != 0 && (chSelectMode) != 1)
 {
        return (1);
    }
    wd->storage.wds.chSelectMode = chSelectMode;
    if(update_storage) {
        if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
            return (1);
    }
    return (0);
}

void tiap_wcfg_wdsChSelModeGet(const wcfgd *wd, __u8 *pChSelectMode)
{
    *pChSelectMode = wd->storage.wds.chSelectMode;
}

int tiap_wcfg_wdsAutoCfgSet(wcfgd *wd, __u8 autoCfg, int update_storage)
{
	if( (autoCfg) != 0 && (autoCfg) != 1)
 {
        return (1);
    }
    wd->storage.wds.autoCfg = autoCfg;
    if(update_storage) {
        if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
            return (1);
    }
    return (0);
}

void tiap_wcfg_wdsAutoCfgGet(const wcfgd *wd, __u8 *pAutoCfg)
{
    *pAutoCfg = wd->storage.wds.autoCfg;
}

int tiap_wcfg_wdsPrivacySet(wcfgd *wd, __u8 privacy, int update_storage)
{
	if( (privacy) != 0 && (privacy) != 1)
 {
        return (1);
    }
    wd->storage.wds.privacy = privacy;
    if(update_storage) {
        if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
            return (1);
    }
    return (0);
}

void tiap_wcfg_wdsPrivacyGet(const wcfgd *wd, __u8 *pPrivacy)
{
    *pPrivacy = wd->storage.wds.privacy;
}


int tiap_wcfg_wdsNameSet(wcfgd *wd, const __u8 *name, int update_storage)
{
	if( strlen(name) > WDS_NAME_MAX_LEN)
 {
        printf("Name too long. Max %d.\n", WDS_NAME_MAX_LEN);
        return (1);
    }
    strncpy(wd->storage.wds.name, name, WDS_NAME_MAX_LEN);
    if(update_storage) {
        if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
            return (1);
    }
    return (0);
}

void tiap_wcfg_wdsNameGet(const wcfgd *wd, __u8 *name)
{
    strncpy(name, wd->storage.wds.name, WDS_NAME_MAX_LEN);
}


int tiap_wcfg_wdsWirelessIfModeSet(wcfgd *wd, __u8 wirelessIfMode, int update_storage)
{
	if( (wirelessIfMode) != 0 && (wirelessIfMode) != 1)
    {
        return (1);
    }
    wd->storage.wds.wirelessIfMode = wirelessIfMode;
    if(update_storage) {
        if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
            return (1);
    }
    return (0);
}

void tiap_wcfg_wdsWirelessIfModeGet(const wcfgd *wd, __u8 *pWirelessIfMode)
{
    *pWirelessIfMode = wd->storage.wds.wirelessIfMode;
}

int tiap_wcfg_wdsCrudeModeSet(wcfgd *wd, __u8 crudefMode, int update_storage)
{
	if( (crudefMode) != 0 && (crudefMode) != 1)
 {
        return (1);
    }
    wd->storage.wds.crudeMode = crudefMode;
    if(update_storage) {
        if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
            return (1);
    }
    return (0);
}

void tiap_wcfg_wdsCrudeModeGet(const wcfgd *wd, __u8 *pCrudeMode)
{
    *pCrudeMode = wd->storage.wds.crudeMode;
}


int tiap_wcfg_wdsSharedSecretSet(wcfgd *wd, const __u8 *sharedSecret, int update_storage)
{
	if( strlen(sharedSecret) > WDS_SHARED_SECRET_STR_LEN)
 {
        printf("Shared secret too long. Max %d.\n", WDS_SHARED_SECRET_STR_LEN);
        return (1);
    }
    strncpy(wd->storage.wds.sharedSecret, sharedSecret, WDS_SHARED_SECRET_STR_LEN);
    if(update_storage) {
        if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
            return (1);
    }
    return (0);
}

void tiap_wcfg_wdsSharedSecretGet(const wcfgd *wd, __u8 *sharedSecret)
{
    strncpy(sharedSecret, wd->storage.wds.sharedSecret, WDS_SHARED_SECRET_STR_LEN);
}

int tiap_wcfg_wdsUlMacSet(wcfgd *wd, __u8 *mac, int update_storage)
{
    memcpy(wd->storage.wds.uplinkMac, mac, ETH_ALEN);
    if(update_storage) {
        if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
            return (1);
    }
    return (0);
}

void tiap_wcfg_wdsUlMacGet(const wcfgd *wd, __u8 *mac)
{
    memcpy(mac, wd->storage.wds.uplinkMac, ETH_ALEN);
}

int tiap_wcfg_wdsDl0MacSet(wcfgd *wd, __u8 *mac, int update_storage)
{
    memcpy(wd->storage.wds.downlinkMac0, mac, ETH_ALEN);
    if(update_storage) {
        if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
            return (1);
    }
    return (0);
}

void tiap_wcfg_wdsDl0MacGet(const wcfgd *wd, __u8 *mac)
{
    memcpy(mac, wd->storage.wds.downlinkMac0, ETH_ALEN);
}

int tiap_wcfg_wdsDl1MacSet(wcfgd *wd, __u8 *mac, int update_storage)
{
    memcpy(wd->storage.wds.downlinkMac1, mac, ETH_ALEN);
    if(update_storage) {
        if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
            return (1);
    }
    return (0);
}

void tiap_wcfg_wdsDl1MacGet(const wcfgd *wd, __u8 *mac)
{
    memcpy(mac, wd->storage.wds.downlinkMac1, ETH_ALEN);
}

int tiap_wcfg_wdsDl2MacSet(wcfgd *wd, __u8 *mac, int update_storage)
{
    memcpy(wd->storage.wds.downlinkMac2, mac, ETH_ALEN);
    if(update_storage) {
        if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
            return (1);
    }
    return (0);
}

void tiap_wcfg_wdsDl2MacGet(const wcfgd *wd, __u8 *mac)
{
    memcpy(mac, wd->storage.wds.downlinkMac2, ETH_ALEN);
}

int tiap_wcfg_wdsDl3MacSet(wcfgd *wd, __u8 *mac, int update_storage)
{
    memcpy(wd->storage.wds.downlinkMac3, mac, ETH_ALEN);
    if(update_storage) {
        if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
            return (1);
    }
    return (0);
}

void tiap_wcfg_wdsDl3MacGet(const wcfgd *wd, __u8 *mac)
{
    memcpy(mac, wd->storage.wds.downlinkMac3, ETH_ALEN);
}

#define TIAP_SOCKADDR_WDS_REPORT "/var/tmp/wdsresponse"

/* we need this function since init_aton function returns different values in different OS */
int tiap_inet_aton(char *str, struct in_addr * inetAddress)
{
	if(inet_aton(str, inetAddress))
		return 0; //OK;
	else
		return 1;//NOK;
}

int tiap_wcfg_wdsPrepareReportSocket(wdsMngrReportRequest_s *pRequest)
{
    int sockRep;
    int retCode;

    pRequest->domain = AF_UNIX;
    pRequest->type = SOCK_DGRAM;
    pRequest->protocol = 0;
    sockRep = socket(pRequest->domain, pRequest->type, pRequest->protocol);
    if (sockRep < 0) {
        printf("Error creating socket for WDS report: %s.\n", strerror(errno));
        return (sockRep);
    }
    /* Create name. */
    pRequest->addr.sun_family = AF_UNIX;
    pRequest->addrLen = sizeof(struct sockaddr_un);
    sprintf(pRequest->addr.sun_path,"%s", TIAP_SOCKADDR_WDS_REPORT);
    /* Bind the UNIX domain address to the created socket */
    retCode = bind(sockRep, (struct sockaddr *) &(pRequest->addr), pRequest->addrLen);
    if (retCode < 0) {
        printf("Error binding WDS report socket: %s", strerror(errno));
        close(sockRep);
        unlink(pRequest->addr.sun_path);
        return(retCode);
    }
    return (sockRep);
}
int tiap_wcfg_wdsReadResponse(void * pResponse, int size, int sock)
{
    u32 sockAddrSize;
    fd_set readFdSet;
    int retCode;
    struct timeval timeOut;
    struct sockaddr_un from;

    FD_ZERO(&readFdSet);
    FD_SET(sock, &readFdSet);

    sockAddrSize = sizeof(from);
    timeOut.tv_usec = 0;
    timeOut.tv_sec = 1;   /* One second timeout */
    while(1) {
        retCode = select(FD_SETSIZE, &readFdSet, NULL, NULL, &timeOut);
        if(retCode < 0) {
            if(errno == EINTR) {
                /* Ignore interrupted by signal */
                continue;
            }
            printf("%s : Error returned by select: %s.", __FUNCTION__, strerror(errno));
            break;
        }
        if(retCode == 0) {
            printf("%s : Timeout.", __FUNCTION__);
            break;

        }
        if(FD_ISSET(sock, &readFdSet)) {
            if (recvfrom(sock, pResponse, size, 0, 
                         (struct sockaddr *)&from, &sockAddrSize) < 0) {
                printf("Error reading hidden WPA instance: %s", strerror(errno));
                break;
            }
            close(sock);
            return (OK);
        }
    }
    close(sock);

    return (NOK);
}

void tiap_wcfg_wdsReport(const wcfgd *wd)
{
    wdsMngrReportRequest_s  request;
    wdsMngrReport_s         report;
    int sockRep;
    int retCode;
    int i;

    if(wd->storage.ldb.wdsEnable) {
        sockRep = tiap_wcfg_wdsPrepareReportSocket(&request);
        if(sockRep < 0) {
            return;
        }
        dda_wdsGetReport(&request);
        retCode = tiap_wcfg_wdsReadResponse((void *)&report, sizeof(wdsMngrReport_s), sockRep);
        unlink(request.addr.sun_path);
        if (retCode != OK) {
            printf("Error reading WDS report message: %s", strerror(errno));
            return;
        }
        else {
            printf("WDS version %d.%d.%d.\n", report.wdsLibVerMajor, 
                   report.wdsLibVerMinor, report.wdsLibVerPatch); 
            printf("WDS state:%s. DL number:%d. Actual hop #%d\n", 
                   wds_stateToString[report.wdsMngrState], report.dlNumber,report.actualHopNum);
            printf("Configuration:\nName %s, %s, auto config %s.\n", report.wdsName, 
                   (report.wdsHop) ? "not root": "root", (report.autoCfg ? "enbled" : "disabled" ));
            printf("Ch select %s, privacy %s, shared secret %s\n",  
                   (report.chSelect ? "auto" : "manual" ), (report.privacy ? "enbled" : "disabled" ), 
                   report.sharedSecret);
            printf("BSS_ID <%hhX:%hhX:%hhX:%hhX:%hhX:%hhX>\n", report.bssId[0], report.bssId[1],
                   report.bssId[2], report.bssId[3], report.bssId[4], report.bssId[5]);
            printf("\nUplink:\n");
            printf("State: %s, link partner <%hhX:%hhX:%hhX:%hhX:%hhX:%hhX>, RSSI %d\n", 
                   wds_hdlrStateToStr[report.uplink.state], report.uplink.linkPartner[0], 
                   report.uplink.linkPartner[1], report.uplink.linkPartner[2], 
                   report.uplink.linkPartner[3], report.uplink.linkPartner[4], 
                   report.uplink.linkPartner[5], report.uplink.rssi);
            for(i = 0; i < WDS_MAX_DL_NUMBER;i++) {
                printf("\nDownlink %d:\n", i);
                printf("State: %s, link partner <%hhX:%hhX:%hhX:%hhX:%hhX:%hhX>, RSSI %d\n", 
                       wds_hdlrStateToStr[report.downlink[i].state], report.downlink[i].linkPartner[0], 
                       report.downlink[i].linkPartner[1], report.downlink[i].linkPartner[2], 
                       report.downlink[i].linkPartner[3], report.downlink[i].linkPartner[4], 
                       report.downlink[i].linkPartner[5], report.downlink[i].rssi);
            }

        }
    }
    else {
        printf("WDS is disabled.\n");
    }
}

void tiap_wcfg_wdsStat(const wcfgd *wd)
{
    wdsMngrReportRequest_s  request;
    wdsMngrStat_s           stat;
    int sockRep;
    int retCode;
    int devNum, i;

    if(wd->storage.ldb.wdsEnable) {
        sockRep = tiap_wcfg_wdsPrepareReportSocket(&request);
        if(sockRep < 0) {
            return;
        }

        dda_wdsGetStat(&request);
        retCode = tiap_wcfg_wdsReadResponse((void *)&stat, sizeof(wdsMngrStat_s), sockRep);
        unlink(request.addr.sun_path);
        if (retCode != OK) {
            printf("Error reading WDS stat message: %s", strerror(errno));
            return;
        }
        else {
            printf("\nWDS protocol messages send and receive statistics:\n");
            printf("TX: WDS protocol messages %d.\n", stat.mngrStat.sentProtocolMsgs);
            printf("TX: WDS protocol messages fails %d.\n", stat.mngrStat.sentProtocolMsgsFails);
            printf("RX: WDS protocol messages %d.\n", stat.mngrStat.recvProtocolMsgs);

            printf("\nWDS switch statistics:\n");
            for (i = 0; i < WDS_LINKS_NUMBER; i++) {
                printf("%d : %s, MAC %02hx:%02hx:%02hx:%02hx:%02hx:%02hx, Cookie %p, used %u\n",
                        i, (stat.switchTalbe[i].inUse ? "inUse" : "Free"),
                        stat.switchTalbe[i].mac[0], stat.switchTalbe[i].mac[1], 
                        stat.switchTalbe[i].mac[2], stat.switchTalbe[i].mac[3], 
                        stat.switchTalbe[i].mac[4], stat.switchTalbe[i].mac[5], 
                        stat.switchTalbe[i].pCookie, stat.switchTalbe[i].used);
            }

            printf("\nWDS driver send/receive statistics:\n");
            printf("TX packets %d, staIndexNotFound %d, staIndexNotAuthorized %d, TX fails %d.\n",
                   stat.drvStat.tx_pkts, stat.drvStat.tx_staIndexNotFound, 
                   stat.drvStat.tx_staIndexNotAuthorized, stat.drvStat.tx_fails);
            printf("RX packets %d, wrong RA %d, srcNotAuthorized %d, TA not found %d.\n",
                   stat.drvStat.rx_pkts, stat.drvStat.rx_wrongRA, 
                   stat.drvStat.rx_srcNotAuthorized, stat.drvStat.rx_TAnotFound);

            printf("\nWDS network interfaces statistics:\n");

            for (devNum = 0; devNum < WDS_LINKS_NUMBER; devNum++) {
                if(devNum == 0) {
                    printf("Uplink : ");
                }
                else {
                    printf("Downlink%d : ", devNum - 1);
                }
                printf("%s\n", stat.linkStat[devNum].devName);
                printf("RX packets:%u errors:%u dropped:%u\n",
                       stat.linkStat[devNum].ifStat.rx_packets, stat.linkStat[devNum].ifStat.rx_errors, 
                       stat.linkStat[devNum].ifStat.rx_dropped);
                printf("TX packets:%u errors:%u dropped:%u\n",
                       stat.linkStat[devNum].ifStat.tx_packets, stat.linkStat[devNum].ifStat.tx_errors, 
                       stat.linkStat[devNum].ifStat.tx_dropped);

                printf("Management:\nlink establised: %d, link fails: %d, secure fails: %d,  Keep alive ID: %d\n",
                       stat.linkStat[devNum].linkEstablished, stat.linkStat[devNum].linkFailed, 
                       stat.linkStat[devNum].secureFailed, stat.linkStat[devNum].lastKeepALiveId);
            }
        }
    }
    else {
        printf("WDS is disabled.\n");
    }
}

void tiap_wcfg_wdsInfoDb(const wcfgd *wd)
{
    wdsMngrReportRequest_s  request;
    wdsMngrInfoRecord_s     maintainDb[WDS_MNGR_INFO_TABLE_SIZE];
    int sockRep;
    int retCode;
    int index, i;
    struct in_addr addr;


    if(wd->storage.ldb.wdsEnable) {
        sockRep = tiap_wcfg_wdsPrepareReportSocket(&request);
        if(sockRep < 0) {
            return;
        }

        dda_wdsGetInfoDb(&request);
        retCode = tiap_wcfg_wdsReadResponse((void *)&maintainDb, sizeof(maintainDb), sockRep);
        unlink(request.addr.sun_path);
        if (retCode != OK) {
            printf("Error reading WDS stat message: %s", strerror(errno));
            return;
        }
        else {
            for(i = 0; i < WDS_MNGR_INFO_TABLE_SIZE; i++) {
                if(WDS_IS_MAC_NOT_ZERO_m(maintainDb[i].sender)) {
                    printf("Sender %02hhX:%02hhX:%02hhX:%02hhX:%02hhX:%02hhX ",
                           maintainDb[i].sender[0], maintainDb[i].sender[1], 
                           maintainDb[i].sender[2], maintainDb[i].sender[3], 
                           maintainDb[i].sender[4], maintainDb[i].sender[5]);
                    addr.s_addr = maintainDb[i].ip;
                    printf("IP  %s ", inet_ntoa(addr));
                    printf("UL %02hhX:%02hhX:%02hhX:%02hhX:%02hhX:%02hhX rssi %d.\n", 
                           maintainDb[i].ul.mac[0], maintainDb[i].ul.mac[1],
                           maintainDb[i].ul.mac[2], maintainDb[i].ul.mac[3], 
                           maintainDb[i].ul.mac[4], maintainDb[i].ul.mac[5],
                           maintainDb[i].ul.rssi);
                    for(index = 0; index < WDS_MAX_DL_NUMBER; index++) {
                        if(!WDS_IS_MAC_NOT_ZERO_m(maintainDb[i].dl[index].mac))
                            continue;
                        printf("\t\t\t\tDL %02hhX:%02hhX:%02hhX:%02hhX:%02hhX:%02hhX rssi %d.\n", 
                               maintainDb[i].dl[index].mac[0], maintainDb[i].dl[index].mac[1],
                               maintainDb[i].dl[index].mac[2], maintainDb[i].dl[index].mac[3], 
                               maintainDb[i].dl[index].mac[4], maintainDb[i].dl[index].mac[5],
                               maintainDb[i].dl[index].rssi);
    
                    }
                }
            }
        }
    }
    else {
        printf("WDS is disabled.\n");
    }
}

const char * tiap_wcfg_wdsLinkStateGet(const wcfgd *wd, int link/* 0 - uplink, 1-4 - downlinks*/)
{
    wdsMngrReportRequest_s  request;
    wdsMngrReport_s         report;
    int sockRep;
    int retCode;
    const char * pStr;

    if(wd->storage.ldb.wdsEnable) {
        sockRep = tiap_wcfg_wdsPrepareReportSocket(&request);
        if(sockRep < 0) {
            return ("None");
        }
        dda_wdsGetReport(&request);
        retCode = tiap_wcfg_wdsReadResponse((void *)&report, sizeof(wdsMngrReport_s), sockRep);
        unlink(request.addr.sun_path);
        if (retCode != OK) {
            printf("Error reading WDS report message: %s", strerror(errno));
            return ("None");
        }
        else {
            switch(link) {
            case 0: /* uplink */
                pStr = wds_hdlrStateToStr[report.uplink.state];
                break;
            case 1: /* downlink0 */
            case 2: /* downlink1 */
            case 3: /* downlink2 */
            case 4: /* downlink3 */
                pStr =  wds_hdlrStateToStr[report.downlink[link - 1].state];
                break;
            default: 
                pStr = "None";
                break;
            }
        }
    }
    else {
        pStr = "None";
    }
    return (pStr);
}

const char * tiap_wcfg_wdsMngrStateGet(const wcfgd *wd, int link/* ignored */)
{
    wdsMngrReportRequest_s  request;
    wdsMngrReport_s         report;
    int sockRep;
    int retCode;
    const char * pStr;

    if(wd->storage.ldb.wdsEnable) {
        sockRep = tiap_wcfg_wdsPrepareReportSocket(&request);
        if(sockRep < 0) {
            return ("None");
        }
        dda_wdsGetReport(&request);
        retCode = tiap_wcfg_wdsReadResponse((void *)&report, sizeof(wdsMngrReport_s), sockRep);
        unlink(request.addr.sun_path);
        if (retCode != OK) {
            printf("Error reading WDS report message: %s", strerror(errno));
            return ("None");
        }
        else {
            pStr = wds_stateToString[report.wdsMngrState];
        }
    }
    else {
        pStr = "None";
    }
    return (pStr);
}


void tiap_wcfg_wdsSeveritySet(const wcfgd *wd, UINT32 error, UINT32 warning, UINT32 info)
{
    wds_ddaPrintSeveritySet(error, warning, info);
}

void tiap_wcfg_wdsRestart(const wcfgd *wd)
{
    dda_wdsRestart(FALSE);
}

int tiap_wcfg_get_num_of_multi_ssid(wcfgd *wd)
{
	int i, num_of_ssids = 0;
	tiap_param_multi_ssid_t multi_ssid;
	
	if( tiap_wcfg_get_multi_ssid(wd, &multi_ssid, FROM_STORAGE) ) {
		printf("Failed to get multiple ssid status.\n");
		return 0;
	}
	
	for (i =0; i < TIAP_PARAM_NUM_MULTIPLE_SSID; i++) 
	{
		if (multi_ssid.tbl[i].ssid_type == TIAP_PARAM_SSID_NON_ACTIVE ||
			multi_ssid.tbl[i].ssid_type == TIAP_PARAM_SSID_INTERNAL) 
			continue;
		
			num_of_ssids++;
	}

	return num_of_ssids;
}


int tiap_wcfg_get_primary_vlan(wcfgd *wd, __u16 *vlan_id,  int from_storage)
{
	int i;
	tiap_param_multi_ssid_t multi_ssid;

	if ( tiap_wcfg_get_multi_ssid(wd, &multi_ssid, from_storage))
	{
		printf("%s: failed to get multiple ssid table\n",__FUNCTION__);
		return 1;
	}
    
	for (i=0; i < TIAP_PARAM_NUM_MULTIPLE_SSID; i++) 
	{
		if (multi_ssid.tbl[i].ssid_type == TIAP_PARAM_SSID_PRIMARY) 
		{
			*vlan_id = multi_ssid.tbl[i].vlan;
			break;
		}
	}

	if ( i == TIAP_PARAM_NUM_MULTIPLE_SSID ) 
	{
		printf("%s: primary vlan not found\n",__FUNCTION__);
		return 1;
	}
	
	return 0;
}

int tiap_wcfg_set_primary_vlan(wcfgd *wd,__u16 vlan_id, int update_storage)
{
	int i;
	tiap_param_multi_ssid_t *multi_ssid = &wd->storage.ldb.multi_ssid;

	for (i=0; i < TIAP_PARAM_NUM_MULTIPLE_SSID; i++) 
	{
		if (multi_ssid->tbl[i].ssid_type == TIAP_PARAM_SSID_PRIMARY)
		{
			multi_ssid->tbl[i].vlan = vlan_id;
			break;
		}
	}
	if (i == TIAP_PARAM_NUM_MULTIPLE_SSID) 
	{
		printf("%s: primary vlan not found\n",__FUNCTION__);
		return 1;
	}

	if( update_storage && wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;

	return 0;
}

int tiap_wcfg_addMultiSsid(wcfgd *wd, tiap_param_multi_ssid_elem_t *elem, int update_storage)
{
	int i, max_num_elems, num_elems;
	tiap_param_multi_ssid_t *multi_ssid = &wd->storage.ldb.multi_ssid;

	switch (elem->ssid_type) 
	{
	
	case TIAP_PARAM_SSID_PRIMARY:
		max_num_elems = TIAP_PARAM_MAX_PRIMARY_SSIDS;
		break;
	case TIAP_PARAM_SSID_SECONDARY:
		max_num_elems = TIAP_PARAM_MAX_SECONDARY_SSIDS;
		break;
	case TIAP_PARAM_SSID_INTERNAL:
		max_num_elems = TIAP_PARAM_MAX_INTERNAL_SSIDS;
		break;
	default:
		max_num_elems = 0;
		printf("Wrong ssid profile type\n");
		return 1;
		break;
	
	}

	num_elems = 0;
	for (i=0; i<TIAP_PARAM_NUM_MULTIPLE_SSID; i++ ) 
	{
		if (multi_ssid->tbl[i].ssid_type == elem->ssid_type)
			num_elems++;
	}

	if (num_elems >= max_num_elems) {
		printf("%s: Failed to add SSID. No more SSIDs of that type are allowed\n",__FUNCTION__);
		return 1;
	}

	for (i=0; i<TIAP_PARAM_NUM_MULTIPLE_SSID; i++) 
	{
		if (multi_ssid->tbl[i].ssid_type == TIAP_PARAM_SSID_NON_ACTIVE) 
		{
			__u32 free_profile_ind;

			/* assign  security profile corresponfing to the new ssid */
			if (tiap_wcfg_get_freeSecProfileIndex(wd, &free_profile_ind)) 
			{
				printf("No free security profile is available\n");
				return 1;
			}
			
			wd->storage.wpa.profile[free_profile_ind].is_active = 1;
			elem->sec_profile_index = free_profile_ind;

			memcpy(&multi_ssid->tbl[i], elem, sizeof(tiap_param_multi_ssid_elem_t));
			break;
		}
	}

	if( update_storage && wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;

	return 0;
}

int tiap_wcfg_delMultiSsid(wcfgd *wd,  int ssid_id, int update_storage)
{
	tiap_param_multi_ssid_t *multi_ssid = &wd->storage.ldb.multi_ssid;
	int sec_profile_index;

	if (ssid_id > TIAP_PARAM_NUM_MULTIPLE_SSID - 1) 
	{
		printf("Wrong multiple SSID index\n");
		return 1;
	}

	multi_ssid->tbl[ssid_id].ssid_type = TIAP_PARAM_SSID_NON_ACTIVE;
	memset(multi_ssid->tbl[ssid_id].ssid, 0, IW_ESSID_MAX_SIZE+1);
	multi_ssid->tbl[ssid_id].vlan = 0;
	
	/* free corresponding security profile*/
	sec_profile_index= multi_ssid->tbl[ssid_id].sec_profile_index;
	wcfg_storage_setdefaults_sec_profile(&wd->storage.wpa.profile[sec_profile_index]);
	
	if( update_storage && wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;

	return 0;

}


int tiap_wcfg_lookUpMultiSsidByType(tiap_param_multi_ssid_t *multi_ssid, tiap_param_ssid_type_t type,
					   tiap_param_multi_ssid_elem_t *elem, int *elem_ind)
{
	int i;
	for (i=0; i < TIAP_PARAM_NUM_MULTIPLE_SSID; i++) 
	{
		if (multi_ssid->tbl[i].ssid_type == type) {
			memcpy(elem, &multi_ssid->tbl[i], sizeof(tiap_param_multi_ssid_elem_t));
			*elem_ind = i;
			break;
		}
	}
	if (i == TIAP_PARAM_NUM_MULTIPLE_SSID) 
	{
		printf("%s: Failed to find multi ssid element of type %d\n",__FUNCTION__,type);
		return NOK;
	}
	return OK;
}


int tiap_wcfg_lookUpMultiSsidByVlan(tiap_param_multi_ssid_t *multi_ssid, __u16 vlan,
					   tiap_param_multi_ssid_elem_t *elem, int *elem_ind)
{
	int i;
	for (i=0; i < TIAP_PARAM_NUM_MULTIPLE_SSID; i++) 
	{
		if ( (multi_ssid->tbl[i].ssid_type != TIAP_PARAM_SSID_NON_ACTIVE) &&
			 (multi_ssid->tbl[i].vlan == vlan))
		{
			if( elem )
				memcpy(elem, &multi_ssid->tbl[i], sizeof(tiap_param_multi_ssid_elem_t));
			if (elem_ind) 
				*elem_ind = i;
			break;
		}
	}
	if (i == TIAP_PARAM_NUM_MULTIPLE_SSID) 
	{
		return NOK;
	}
	return OK;
}

int tiap_wcfg_lookUpMultiSsidBySsid(tiap_param_multi_ssid_t *multi_ssid, __u8 *ssid,
					   tiap_param_multi_ssid_elem_t *elem, int *elem_ind)
{
	int i;
	for (i=0; i < TIAP_PARAM_NUM_MULTIPLE_SSID; i++) 
	{
		if ( (multi_ssid->tbl[i].ssid_type != TIAP_PARAM_SSID_NON_ACTIVE) &&
			 (!strcmp(multi_ssid->tbl[i].ssid, ssid)) )
		{
			if( elem ) 
				memcpy(elem, &multi_ssid->tbl[i], sizeof(tiap_param_multi_ssid_elem_t));
			if( elem_ind ) 
				*elem_ind = i;
			break;
		}
	}
	if (i == TIAP_PARAM_NUM_MULTIPLE_SSID) 
	{
		return NOK;
	}
	return OK;
}

/****************************************************************************************/
/*! \brief Checks if one of the profiles is configured to work with the given Cipher. 

	\param wd     		    pointer to the storage
	\param cipher           the cipher to look for
	\param isCipherExists	an output buffer: TRUE is the cipher exists in one of the 
                                              profiles, FALSE otherwise.
    \param profile_index    an output buffer: the index of the profile with the given 
                                              cipher (if exists).
	
	\return 0 on Success, 1 otherwise.
*/
/****************************************************************************************/
int tiap_wcfg_isCipherExistsInMultiSsid(wcfgd *wd, __u32 cipher, BOOL *isCipherExists, 
                                        int *profile_index)
{
	int i;
    int active_profile_ind = 0;
    __u32 ind_cipher;
    tiap_param_multi_ssid_t multi_ssid;
    tiap_param_security_type_t  sec_type;
	
	if ( tiap_wcfg_get_multi_ssid(wd, &multi_ssid, FROM_STORAGE))
	{
		printf("%s: failed to get multiple ssid table\n",__FUNCTION__);
		return 1;
	}

	if (!multi_ssid.secondary_enabled) 
	{
		*isCipherExists = FALSE;
		return 0;
	}

	for (i=0; i < TIAP_PARAM_NUM_MULTIPLE_SSID; i++) 
	{
		if ( (multi_ssid.tbl[i].ssid_type != TIAP_PARAM_SSID_NON_ACTIVE) &&
			 (multi_ssid.tbl[i].ssid_type != TIAP_PARAM_SSID_INTERNAL))
		{
            if(!tiap_wcfg_get_default_cipher(wd, &ind_cipher, active_profile_ind))
            {
                if(ind_cipher & cipher)
                {
                    if(!tiap_wcfg_get_security_type(wd, &sec_type ,active_profile_ind))
                    {
                        if(sec_type != TIAP_PARAM_SECURITY_802_1X)
                        {
                            break;
                        }
                    }
                    else
                    {
                        return 1;
                    }
                }
            }
            else
            {
                return 1;
            }
            active_profile_ind++;
		}
	}

	if (i == TIAP_PARAM_NUM_MULTIPLE_SSID) 
	{
		*isCipherExists = FALSE;
	}
    else
    {
        *isCipherExists = TRUE;
        *profile_index = active_profile_ind;
    }

	return 0;
}

/****************************************************************************************/
/*! \brief Checks if one of the profiles is configured to work with Pre-Authentication. 

	\param wd     		    pointer to the storage
	\param isPreAuthOn  	an output buffer: TRUE is Pre-Auth is ON in one of the 
                                              profiles, FALSE otherwise.
    \param profile_index    an output buffer: the index of the profile with Pre-Auth 
                                              ON (if exists).
	
	\return 0 on Success, 1 otherwise.
*/
/****************************************************************************************/
int tiap_wcfg_isPreAuthOnInMultiSsid(wcfgd *wd, BOOL *isPreAuthOn, int *profile_index)
{
	int     i;
    int     active_profile_ind = 0;
    __u8    sec_profile_index;
    tiap_param_multi_ssid_t multi_ssid;
	
	if ( tiap_wcfg_get_multi_ssid(wd, &multi_ssid, FROM_STORAGE))
	{
		printf("%s: failed to get multiple ssid table\n",__FUNCTION__);
		return 1;
	}

	if (!multi_ssid.secondary_enabled) 
	{
		*isPreAuthOn = FALSE;
		return 0;
	}

	for (i=0; i < TIAP_PARAM_NUM_MULTIPLE_SSID; i++) 
	{
        
		if ( (multi_ssid.tbl[i].ssid_type != TIAP_PARAM_SSID_NON_ACTIVE) &&
		     (multi_ssid.tbl[i].ssid_type != TIAP_PARAM_SSID_INTERNAL) )
		{
            sec_profile_index = multi_ssid.tbl[i].sec_profile_index;
            if( (wd->storage.wpa.profile[sec_profile_index].radius.pre_auth_flag) &&
                (wd->storage.wpa.profile[sec_profile_index].sec.type & TIAP_PARAM_SECURITY_WPA2) )
            {
                break;
            }
            active_profile_ind++;
		}
	}

	if (i == TIAP_PARAM_NUM_MULTIPLE_SSID) 
	{
		*isPreAuthOn = FALSE;
	}
    else
    {
        *isPreAuthOn = TRUE;
        *profile_index = active_profile_ind;
    }
    return 0;
}

/*Warning: essid must be at least IW_ESSID_MAX_SIZE+1 size long*/
int tiap_wcfg_get_primary_ssid(wcfgd *wd, __u8 *ssid,  int from_storage)
{
	int i;
	tiap_param_multi_ssid_t multi_ssid;
	
	if ( tiap_wcfg_get_multi_ssid(wd, &multi_ssid, from_storage))
	{
		printf("%s: failed to get multiple ssid table\n",__FUNCTION__);
		return 1;
	}
    

   for (i=0; i < TIAP_PARAM_NUM_MULTIPLE_SSID; i++) 
	{
		if (multi_ssid.tbl[i].ssid_type == TIAP_PARAM_SSID_PRIMARY) 
		{
			strncpy(ssid, multi_ssid.tbl[i].ssid, IW_ESSID_MAX_SIZE);
			ssid[IW_ESSID_MAX_SIZE] = '\0';
			break;
		}
	}
	if (i == TIAP_PARAM_NUM_MULTIPLE_SSID) {
		printf("%s: primary SSID not found\n",__FUNCTION__);
		return 1;
	}

    return 0;
}
	

int tiap_wcfg_set_primary_ssid(wcfgd *wd, __u8 *ssid, const int update_storage)
{
	tiap_param_multi_ssid_elem_t primary;
	int primary_ind;

	if (tiap_wcfg_lookUpMultiSsidByType(&wd->storage.ldb.multi_ssid,
							   TIAP_PARAM_SSID_PRIMARY, &primary, &primary_ind) == NOK)
	{
		printf("%s: failed to find current primary ssid\n",__FUNCTION__);
		return 1;
	}

	strncpy(primary.ssid, ssid, IW_ESSID_MAX_SIZE);
	primary.ssid[IW_ESSID_MAX_SIZE] = '\0';

	memcpy(&wd->storage.ldb.multi_ssid.tbl[primary_ind], &primary, sizeof(primary));

	if( update_storage && wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;

	return 0;
}

int tiap_wcfg_get_primary_profileIndex(wcfgd *wd, int *profile_index,  int from_storage)
{
	int i;
	tiap_param_multi_ssid_t multi_ssid;
	
	if ( tiap_wcfg_get_multi_ssid(wd, &multi_ssid, from_storage))
	{
		printf("%s: failed to get multiple ssid table\n",__FUNCTION__);
		return 1;
	}
    

   for (i=0; i < TIAP_PARAM_NUM_MULTIPLE_SSID; i++) 
	{
		if (multi_ssid.tbl[i].ssid_type == TIAP_PARAM_SSID_PRIMARY) 
		{
            *profile_index = multi_ssid.tbl[i].sec_profile_index;
			break;
		}
	}
	if (i == TIAP_PARAM_NUM_MULTIPLE_SSID) {
		printf("%s: primary SSID not found\n",__FUNCTION__);
		return 1;
	}

    return 0;
}

int tiap_wcfg_set_security_type(wcfgd *wd, tiap_param_security_type_t sec_type, int profile_index, 
                                int update_storage)
{
    __u32 cipher;
	int sec_profile_index, ssid_index; 
    int from_storage = update_storage;
    tiap_param_multi_ssid_t multi_ssid;
    tiap_param_auth_algo_t tbl[TIAP_PARAM_MAX_AUTH_ALG_SUPPORTED];

	if( tiap_wcfg_get_actual_ssid_ind(&wd->storage.ldb.multi_ssid, profile_index, &ssid_index) )
		return 2;

	/* retrieve the corresponding security profile index from multi ssid table */
	sec_profile_index = wd->storage.ldb.multi_ssid.tbl[ssid_index].sec_profile_index;

	wd->storage.wpa.profile[sec_profile_index].sec.type =  sec_type;
    if( !(sec_type & TIAP_PARAM_SECURITY_WPA2) )
    {
        wd->storage.wpa.profile[sec_profile_index].radius.pre_auth_flag = 0;
    }

    if(update_storage) {
	if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;
    }

    if(tiap_wcfg_get_multi_ssid(wd, &multi_ssid, from_storage))
            return 1;

    if(tiap_wcfg_get_default_cipher(wd, &cipher ,profile_index))
                    return 1;
                
    /* Note: When Multiple SSIDs is enabled the AP doesn't  *
     *       suppot Shared Key Authentication Algorithm,    *
     *       only Open System or Both. When a profile with  *
     *       static WEP exists in Multi SSIDs, the AP works *
     *       with Both.                                     */
    if( (sec_type == TIAP_PARAM_SECURITY_WEP) &&
        (cipher != TIAP_WPA_CIPHER_NON) )
    {
        /* If profile is being configured to static WEP and
           Multiple SSIDs is Enabled --> Set Authentication
           Algorithm to Both Open System & Shared Key.      */
        if(multi_ssid.secondary_enabled == 1)
        {
            tbl[0].algo = TIAP_PARAM_MIB_OPEN_SYSTEM;
            tbl[1].algo = TIAP_PARAM_MIB_SHARED_KEY;
            tbl[0].enable = TRUE;
            tbl[1].enable = TRUE;
            
            if(tiap_wcfg_set_auth_type(wd, tbl, update_storage))
                return 1;
        }     
    }
    else
    {
        /* If profile is being configured to security other than static WEP and
           Multiple SSIDs is Disabled --> Set Authentication Algorithm to Open System.
           Note: Shared Key Auth is supported on;y in static WEP profiles.             */
        if(multi_ssid.secondary_enabled == 0)
        {
            tbl[0].algo = TIAP_PARAM_MIB_OPEN_SYSTEM;
            tbl[1].algo = TIAP_PARAM_MIB_SHARED_KEY;
            tbl[0].enable = TRUE;
            tbl[1].enable = FALSE;
            
            if(tiap_wcfg_set_auth_type(wd, tbl, update_storage))
                return 1;
        }
    }

	return 0;
}

int tiap_wcfg_get_security_type(wcfgd *wd, tiap_param_security_type_t  *sec_type ,int profile_index)
{
	int sec_profile_index, ssid_index; 

	if( tiap_wcfg_get_actual_ssid_ind(&wd->storage.ldb.multi_ssid, profile_index, &ssid_index) )
		return 2;

	/* retrieve the corresponding security profile index from multi ssid table */
	sec_profile_index = wd->storage.ldb.multi_ssid.tbl[ssid_index].sec_profile_index;

	*sec_type = wd->storage.wpa.profile[sec_profile_index].sec.type;
	return 0;
}

int tiap_wcfg_get_freeSecProfileIndex(wcfgd *wd, int *sec_profile_index)
{
	int i;

	for (i=0; i < TIAP_MAX_NUM_OF_SUPPORTED_PROFILES; i++) 
	{
		if ( !wd->storage.wpa.profile[i].is_active )
		{
			*sec_profile_index = i;
			return 0;
		}
	}

	return 1;
}

#define _GNU_SOURCE
#include <fcntl.h>
#include <time.h>

int tiap_wcfg_waitForProcessExit(pid_t pid)
{
    char dirName[128];
    struct stat fileStat;
    int iterations = 0;
    struct timespec requested, remaining;


    sprintf(dirName, "/proc/%d",pid);
    while((stat(dirName, &fileStat) == 0) && (iterations < 20)) {
        requested.tv_sec = 0;
        requested.tv_nsec = 100000; /* 100 msecs wait*/
        remaining.tv_sec = 0;
        requested.tv_nsec = 0;
        if((nanosleep(&requested, &requested) != 0) && (errno != EINTR)) {
            printf("\n\n%s : nanosleep error %s.\n", __FUNCTION__, strerror(errno));
            return (1);
        }

        iterations++;
    }
    if(errno == ENOENT) {
        return (0);
    }
    return (1);
}

int tiap_wcfg_sendIpcMsg(char *sock_path, void *msg, int msg_len)
{
	int sock;
	struct sockaddr_un sockaddr;

	/* Create socket on which to send. */
	sock = socket(AF_UNIX, SOCK_DGRAM, 0);
	if (sock < 0) {
		perror("opening datagram socket");
		return -1;
	}

	memset(&sockaddr, 0, sizeof(struct sockaddr_un));
	/* Construct name of socket to send to. */
	sockaddr.sun_family = AF_UNIX;
	strcpy(sockaddr.sun_path, sock_path);

	/* Send message. */
	if (sendto(sock, msg, msg_len, 0,
				(struct sockaddr *)&sockaddr, sizeof(struct sockaddr_un)) < 0) {
		close(sock);
		perror("writing to ipc socket");
		return -1;
	}
	close(sock);

	return 0;

}

int tiap_wcfg_wpaSetSeverity(wcfgd *wd, UINT32 severity)
{
	dbgMsg_T msg;

	switch(severity)
	{
	case 0:
			wd->storage.debug.wpa_report_sev = 0;
			break;
	case 1:
	case 2:
	case 3:
			wd->storage.debug.wpa_report_sev |= (1 << severity);
			break;
	case -1:
	case -2:
	case -3:		
			wd->storage.debug.wpa_report_sev &= ~(1 << (severity*(-1)));
			break;
	default:
			printf("%s: Invalid input value %d\n",__func__, severity);
			return 1;
			break;

	}

	msg.type = SET_SEVERITY;
	msg.content.severity = wd->storage.debug.wpa_report_sev;

	if (tiap_wcfg_sendIpcMsg(DBG_WPA_SOCK_PATH, &msg, sizeof(msg)))
		return 1;
	
	if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;
	
	return 0;
}

int tiap_wcfg_wpaGetSeverity(wcfgd *wd, UINT32 *severity)
{
	*severity= wd->storage.debug.wpa_report_sev;
	return 0;
}

int tiap_wcfg_wpaSetRepModule(wcfgd *wd, UINT32 mod)
{
	dbgMsg_T msg;

	
	switch(mod)
	{
	case 0:
			wd->storage.debug.wpa_report_mod = 0;
			break;
	case 1:
	case 2:
	case 3:
	case 4:
			wd->storage.debug.wpa_report_mod  |= (1 << mod);
			break;
	case -1:
	case -2:
	case -3:		
	case -4:
			wd->storage.debug.wpa_report_mod  &= ~(1 << (mod*(-1)));
			break;
	default:
			printf("%s:Invalid input value %d\n", __func__, mod);
			return 1;
			break;

	}
	msg.type = SET_REP_MODULE;
	msg.content.module = wd->storage.debug.wpa_report_mod;

	if (tiap_wcfg_sendIpcMsg(DBG_WPA_SOCK_PATH, &msg, sizeof(msg)))
		return 1;
	
	if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
		return 1;
	
	return 0;
}

int tiap_wcfg_wpaPrintStaTbl(int profile_index)
{
	dbgMsg_T msg;

	msg.type = PRINT_PROF_TBLS;
	msg.content.profile_ind = profile_index;

	if (tiap_wcfg_sendIpcMsg(DBG_WPA_SOCK_PATH, &msg, sizeof(msg))) 
		return 1;

	return 0;
}

int tiap_wcfg_wpaPrintProfStatistics(int profile_index)
{
	dbgMsg_T msg;

	msg.type = PRINT_PROF_STAT;
	msg.content.profile_ind = profile_index;

	if (tiap_wcfg_sendIpcMsg(DBG_WPA_SOCK_PATH, &msg, sizeof(msg))) 
		return 1;

	return 0;
}

int tiap_wcfg_wpaPrintProfCfg(int profile_index)
{
	dbgMsg_T msg;

	msg.type = PRINT_PROF_CFG;
	msg.content.profile_ind = profile_index;

	if (tiap_wcfg_sendIpcMsg(DBG_WPA_SOCK_PATH, &msg, sizeof(msg))) 
		return 1;

	return 0;
}

int tiap_wcfg_wpaPrintPmkCacheList(void)
{
	dbgMsg_T msg;

	msg.type = PRINT_PMK_CACHE;

	if (tiap_wcfg_sendIpcMsg(DBG_WPA_SOCK_PATH, &msg, sizeof(msg))) 
		return 1;

	return 0;
}

int tiap_wcfg_wpaGetRepModule(wcfgd *wd, UINT32 *mod)
{
	*mod = wd->storage.debug.wpa_report_mod;
	return 0;
}

int tiap_wcfg_wpaGetPSKinHexFormat(int profile_index, char *psk)
{
	dbgMsg_T msg;
    int sockIpc;
    struct sockaddr_un name;
    int retCode;
    fd_set activeFdSet, readFdSet;
    msgReport_t     report;
    struct timeval timeOut;

	msg.type = GET_PSK;
    msg.reportMsgType = RECV_PSK;
    msg.content.profile_ind = profile_index;
    msg.domain = AF_UNIX;
    msg.SocketType = SOCK_DGRAM;
    msg.protocol = 0;
    msg.addr.sun_family = AF_UNIX;
    sprintf(msg.addr.sun_path,"%s", DBG_WPA_SOCK_PATH);
    msg.addrLen = sizeof(struct sockaddr_un);

	if (tiap_wcfg_sendIpcMsg(DBG_WPA_SOCK_PATH, &msg, sizeof(msg)))
    {
		return 1;
    }

    /* Create socket for inter process communications. */
    sockIpc = socket(AF_UNIX, SOCK_DGRAM, 0);
    if (sockIpc < 0) 
    {
        fprintf(stderr, "%s : Error creating socket for inter process communications.\n", __FUNCTION__);
        return 1;
    }
    /* Create name. */
    name.sun_family = AF_UNIX;

    sprintf((char*)name.sun_path,"%s", DBG_WPA_SOCK_PATH);

    unlink((char*)name.sun_path);
    /* Bind the UNIX domain address to the created socket */
    retCode = bind(sockIpc, (struct sockaddr *) &name, sizeof(struct sockaddr_un));
    if (retCode < 0 ) 
    {
        fprintf(stderr, "%s : Error binding socket for inter process communications.\n", __FUNCTION__);
        close(sockIpc);
        return 1;
    }
    /* Initialize the set of active sockets. */
    FD_ZERO(&activeFdSet);
    FD_SET(sockIpc, &activeFdSet);
    timeOut.tv_usec = 0;
    timeOut.tv_sec = 10;
    while(1) 
    {
        readFdSet = activeFdSet;
        retCode = select(FD_SETSIZE, &readFdSet, NULL, NULL, &timeOut);
        if(retCode < 0) 
        {
            if(errno == EINTR) 
            {
                /* Ignore interrupted by signal */
                continue;
            }
            fprintf(stderr, "Error returned by select: %s.\n", strerror(errno));
            close(sockIpc);
            return 1;
        }
        if(FD_ISSET(sockIpc, &readFdSet)) 
        {
            /* Inter process communications. */
            retCode = recvfrom(sockIpc, (void *)&report, sizeof(msgReport_t), 0, NULL, NULL);
            if (retCode < 0) 
            {
                fprintf(stderr, "Error reading IPC message.\n");
                close(sockIpc);
                return 1;
            }
            else 
            {
                if(report.msgType == RECV_PSK) 
                {
                    memcpy(psk, report.psk, PSK_LEN);
                    close(sockIpc);
                    return 0;
                }
                else 
                {
                    /* Ignore all but RECV_PSK */
                    continue;                
                }
            }
        }
        else
        {
            fprintf(stderr, "No response from WPA - Select timed-out.\n");
            close(sockIpc);
            return 1;
        }
    }
}

int tiap_wcfg_set_bitArgs(wcfgd *wd, tiap_param_bit_args_t *bit_args, int update_storage)
{
    int ret = 0, ioctl_sock;

	/* Create a channel to the NET kernel. */
	if((ioctl_sock = iw_sockets_open()) < 0)
	{
		perror("socket");
		return(-1);
	}

	ret = exec_priv_ioctl_set(TIAP_PARAM_BIT, bit_args, sizeof(tiap_param_bit_args_t),
                              ioctl_sock,wd->dev_name);

	close(ioctl_sock);

	return ret;
}

int tiap_wcfg_get_hw_interf_param(const wcfgd *wd, tiap_param_hw_interf_t *param, int from_storage)
{
	int ret = 0, ioctl_sock;

	if (from_storage) {
	
		switch(param->cmd)
		{
		case TIAP_PARAM_INPUT_CCA:
			memcpy(param, &wd->storage.ldb.input_cca, sizeof(tiap_param_hw_interf_t)) ;
			break;
		case TIAP_PARAM_TX_CCA:
			 memcpy(param, &wd->storage.ldb.tx_cca, sizeof(tiap_param_hw_interf_t)) ;
			break;
		case TIAP_PARAM_QUALIF_CCA:
			memcpy(param, &wd->storage.ldb.qualif_cca, sizeof(tiap_param_hw_interf_t)) ;
			break;
		case TIAP_PARAM_STOMP_RX:
			memcpy(param, &wd->storage.ldb.stomp_rx, sizeof(tiap_param_hw_interf_t)) ;
			break;
		case TIAP_PARAM_STOMP_TX:
			memcpy(param, &wd->storage.ldb.stomp_tx, sizeof(tiap_param_hw_interf_t)) ;
			break;
		default:
			printf("%s: Invalid HW interference parameter\n",__func__);
			return -1;
			break;

		}
	}
	else
	{
		 if((ioctl_sock = iw_sockets_open()) < 0)
		{
			perror("socket");
			return(-1);
		}
		ret = exec_priv_ioctl_get(TIAP_PARAM_HW_INTERF_CFG, (void *)param, sizeof(tiap_param_hw_interf_t),ioctl_sock, (char *)wd->dev_name);

		close(ioctl_sock);
	
	}
	return ret;
		
}

int tiap_wcfg_set_hw_interf_param(wcfgd *wd, tiap_param_hw_interf_t *param, int update_storage)
{
	switch(param->cmd)
	{
	case TIAP_PARAM_INPUT_CCA:
		memcpy(&wd->storage.ldb.input_cca, param, sizeof (tiap_param_hw_interf_t));
		break;
	case TIAP_PARAM_TX_CCA:
		memcpy(&wd->storage.ldb.tx_cca, param, sizeof (tiap_param_hw_interf_t));
		break;
	case TIAP_PARAM_QUALIF_CCA:
		memcpy(&wd->storage.ldb.qualif_cca, param, sizeof (tiap_param_hw_interf_t));
		break;
	case TIAP_PARAM_STOMP_RX:
		memcpy(&wd->storage.ldb.stomp_rx, param, sizeof (tiap_param_hw_interf_t));
		break;
	case TIAP_PARAM_STOMP_TX:
		memcpy(&wd->storage.ldb.stomp_tx, param, sizeof (tiap_param_hw_interf_t));
		break;
	default:
		printf("%s: Invalid interference parameter: %d\n",__func__, param->cmd);
		break;
	}

	if(update_storage){
		if( wcfg_storage_write(&wd->storage, (const char *)wd->storage_file) )
			return 1;
	}

	return 0;
		
}

int tiap_wcfg_get_die_id(const wcfgd *wd, tiap_param_die_id_t *param, int from_storage)
{
    int ret = 0, ioctl_sock;
    
    if((ioctl_sock = iw_sockets_open()) < 0)
    {
        perror("socket");
        return(-1);
    }
    ret = exec_priv_ioctl_get(TIAP_PARAM_DIE_ID, (void *)param, 
                              sizeof(tiap_param_die_id_t), ioctl_sock, (char *)wd->dev_name);

    close(ioctl_sock);
    
    return ret;
}

/******************************************************************************/

/*! \brief 	Gets file via TFTP
	\param 	pointer to file name
	\param 	pointer to TFTP server IP address
	\param 	pointer to buffer to get file
	\param 	buffer size
	
	\return positive bin file size if operation is succefful or negative error
*/
/******************************************************************************/
/*
int tiap_tftp_read_file(const char * filename, const char * addr, char * pBuf, int bufSize) 
{
    char   srcFileName[60];
    int    fd, size;

	snprintf(srcFileName, sizeof(srcFileName), "%s/%s", WCFG_TMP_DIR, filename );

	if (tftp_get_file( filename, addr, WCFG_TMP_DIR ))
    {
		printf("%s: TFTP Failed !\n", __FUNCTION__);
		return -1;
    }

    fd = open( srcFileName, O_RDONLY);
    if ((fd = open( srcFileName, O_RDONLY)) == -1 ) 
    {
        printf("%s: Error reading file %s\n", __FUNCTION__, srcFileName );
		return -1;
    }

    size = read(fd, pBuf, bufSize );

    close(fd);

    return size;
}
*/


int tiap_notftp_read_file(const char * filename, char * pBuf, int bufSize) 
{
    char   srcFileName[60];
    int    fd, size;

	snprintf(srcFileName, sizeof(srcFileName), "%s/%s", "/lib/modules", filename );

/*	if (tftp_get_file( filename, addr, WCFG_TMP_DIR ))
    {
		printf("%s: TFTP Failed !\n", __FUNCTION__);
		return -1;
    }
*/
    fd = open( srcFileName, O_RDONLY);
    if ((fd = open( srcFileName, O_RDONLY)) == -1 ) 
    {
        printf("%s: Error reading file %s\n", __FUNCTION__, srcFileName );
		return -1;
    }

    size = read(fd, pBuf, bufSize );

    close(fd);

    return size;
}


