/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2004 Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |** 
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*																		   */
/*	 MODULE:	hiddenProfileIfc.c									       */
/*   PURPOSE:	Contains API that should be called from another process    */
/*				in order to communicate with hidden profile				   */
/*																		   */
/*														                   */
/*																		   */					
/***************************************************************************/
/***************************************************************************/
#include <stdio.h>

#ifdef VXWORKS
#include "socket.h"
#else
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/un.h>
#include <net_utils.h>
#endif

#include <errno.h>
#include <ipc.h>
#include <kos/knet.h>
#include <ipc.h>
#include <802.1x/wep_port.h>
#include <wpa/wpa_port.h>
#include <802.1x/eap_radius.h>
#include <wpa/wpa_info_elem.h>
#include "dda_wpa.h"
#include "hiddenSecurityProfileAPI.h"
#include "securityProfile.h"

u32 hidden_profile_PrepareRespSocket(u32 *pSock);
u32 hidden_profile_GetResp(u32 *pSock, hiddenWpaInstanceMsg_T *pMsg);



/******************************************************************************
*                      sendMsgToHiddenProfInstance()							  *
*******************************************************************************
DESCRIPTION:    Send a messsage to WPA instance corresponding to the hidden profile
				 via domain socket
                                                                         
INPUTS:			msg - a pointer to message
				msgLen - message length

OUTPUT:			None

RETURNS:		0 in case of success, -1 o/w                                             
******************************************************************************/
static u32 sendMsgToHiddenProfInstance(void *msg, u32 msgLen)
{
	u32 sock;	
	
#ifdef VXWORKS	
	struct sockaddr_in wpas_sockaddr;
	int sockAddrSize = sizeof(struct sockaddr_in);

    /* Create socket on which to send. */
	sock = socket(AF_INET, SOCK_DGRAM, 0);
	if (sock < 0) {
		perror("opening datagram socket");
		return -1;
	}
#else /* ifdef VXWORKS */
	struct sockaddr_un wpas_sockaddr;
	int sockAddrSize = sizeof(struct sockaddr_un);

	/* Create socket on which to send. */
	sock = socket(AF_UNIX, SOCK_DGRAM, 0);
	if (sock < 0) {
		perror("opening datagram socket");
		return -1;
	}
#endif  /* ifdef VXWORKS */

#ifdef VXWORKS
    bzero((char *)&wpas_sockaddr, sockAddrSize);
    wpas_sockaddr.sin_len = sockAddrSize;
    wpas_sockaddr.sin_family = AF_INET;
    wpas_sockaddr.sin_port = htons(TO_WPA_HIDDEN_PROFILE_SOCK_PORT);
    wpas_sockaddr.sin_addr.s_addr = inet_addr(IPC_LOCAL_INET_ADDR);
    if(wpas_sockaddr.sin_addr.s_addr == ERROR) {
	    printf("%s : failed in inet_addr error: %s\n",__FUNCTION__, strerror(errno));
        close(sock);
        return (NOK);
    }
#else  /* ifdef VXWORKS */
	/* Construct name of socket to send to. */
	wpas_sockaddr.sun_family = AF_UNIX;
	strcpy(wpas_sockaddr.sun_path, TO_WPA_HIDDEN_PROFILE_SOCK);
#endif  /* ifdef VXWORKS */

	/* Send message. */
	if (sendto(sock, msg, msgLen, 0,
				(struct sockaddr *)&wpas_sockaddr, sockAddrSize) < 0) {
		perror("sending message to hidden WPA instance ");
		return -1;
	}
	close(sock);

	return 0;
}

/******************************************************************************
*                      hidden_profile_SecureAuthStart()							  *
*******************************************************************************
DESCRIPTION:    External API function.
				Triggers the start of secure authentication for hidden profile

INPUTS:			linkParams - secure link parameters

OUTPUT:			None

RETURNS:		OK/NOK                                           
******************************************************************************/
u32 hidden_profile_SecureAuthStart(dynamicSecureCfg_T *linkParams)
{
	hiddenWpaInstanceMsg_T msg;
	u32 ret;

	msg.type = HIDDEN_PROF_AUTH_START;
	/* TODO : check NULL */
	msg.u.confParams = *linkParams;
   
	ret = sendMsgToHiddenProfInstance((hiddenWpaInstanceMsg_T *)&msg, sizeof(msg));
	if (ret) 
	{
		printf("FAILED to send START message to hidden WPA authenticator instance\n");
		return NOK;
	}
   	return OK;
}

/******************************************************************************
*                      hidden_profile_SecureSuplStart()							  *
*******************************************************************************
DESCRIPTION:    External API function.
				Triggers the start of WPA supplicant for hidden profile

INPUTS:			linkParams - secure link parameters

OUTPUT:			None

RETURNS:		OK/NOK                                           
******************************************************************************/
u32 hidden_profile_SecureSuplStart(dynamicSecureCfg_T *linkParams)
{
	hiddenWpaInstanceMsg_T msg;
	u32 ret;

	msg.type = HIDDEN_PROF_SUPL_START;
	/* TODO : check NULL */
	msg.u.confParams = *linkParams;
   
	ret = sendMsgToHiddenProfInstance((hiddenWpaInstanceMsg_T *)&msg, sizeof(msg));
	if (ret) 
	{
		printf("Failed to send START message to hidden WPA supplicant instance \n");
		return NOK;
	}

	return OK;
}


/******************************************************************************
*                      hidden_profile_Init()							  *
*******************************************************************************
DESCRIPTION:    External API function.
				init

INPUTS:			linkParams - secure link parameters

OUTPUT:			None

RETURNS:		OK/NOK                                           
******************************************************************************/
u32 hidden_profile_Init(initHiddenProfConf_T *initParams)
{
	hiddenWpaInstanceMsg_T msg;
    hiddenWpaInstanceMsg_T response;
	u32 ret;
    u32  sock;
    if(hidden_profile_PrepareRespSocket(&sock) != OK) {
        printf("Can not prepare socket for response.\n");
        return NOK;
    }
    msg.type = HIDDEN_PROF_INIT;
	msg.u.initParams = *initParams;

	ret = sendMsgToHiddenProfInstance((hiddenWpaInstanceMsg_T *)&msg, sizeof(msg));
	if (ret) 
	{
		printf("Failed to send Init message to hidden WPA instance\n");
        close(sock);
		return NOK;
	}
    if(hidden_profile_GetResp(&sock, &response) != OK) {
        printf("Can not get response.\n");
        return NOK;

    }

    if(response.type == HIDDEN_PROF_INIT_COMPLETE) {
        return OK;
    }
    printf("%s : %d Wrong response type from hidden WPA instance\n", __FUNCTION__, __LINE__);
    return NOK;
}

/******************************************************************************
*                      hidden_profile_AuthClose()							  *
*******************************************************************************
DESCRIPTION:    External API function.
				Closes secure  link on authenticator side

INPUTS:			linkParams - secure link parameters

OUTPUT:			None

RETURNS:		OK/NOK                                           
******************************************************************************/
u32 hidden_profile_AuthLinkClose(dynamicSecureCfg_T *linkParams)
{
	hiddenWpaInstanceMsg_T msg;
    hiddenWpaInstanceMsg_T response;
	u32 ret;
    u32  sock;

    if(hidden_profile_PrepareRespSocket(&sock) != OK) {
        printf("Can not prepare socket for response.\n");
        return NOK;
    }

    msg.type = HIDDEN_PROF_AUTH_CLOSE;
	/* TODO : check NULL */
	msg.u.confParams = *linkParams;
   
	ret = sendMsgToHiddenProfInstance((hiddenWpaInstanceMsg_T *)&msg, sizeof(msg));
	if (ret) 
	{
		printf("FAILED to send CLOSE message to hidden WPA authenticator instance\n");
        close(sock);
		return NOK;
	}
    if(hidden_profile_GetResp(&sock, &response) != OK) {
        printf("Can not get response.\n");
        return NOK;

    }

    if(response.type == HIDDEN_PROF_LINK_CLOSE_COMPLETE) {
        return OK;
    }
    printf("%s : %d Wrong response type from hidden WPA instance\n", __FUNCTION__, __LINE__);
    return NOK;
}


/******************************************************************************
*                      hidden_profile_SuplLinkClose()							  *
*******************************************************************************
DESCRIPTION:    External API function.
				Closes secure  link on supplicant side  

INPUTS:			linkParams - secure link parameters

OUTPUT:			None

RETURNS:		OK/NOK                                           
******************************************************************************/
u32 hidden_profile_SuplLinkClose(dynamicSecureCfg_T *linkParams)
{
	hiddenWpaInstanceMsg_T msg;
    hiddenWpaInstanceMsg_T response;
	u32 ret;
    u32  sock;

    if(hidden_profile_PrepareRespSocket(&sock) != OK) {
        printf("Can not prepare socket for response.\n");
        return NOK;
    }

	msg.type = HIDDEN_PROF_SUPL_CLOSE;
	/* TODO : check NULL */
	msg.u.confParams = *linkParams;
   
	ret = sendMsgToHiddenProfInstance((hiddenWpaInstanceMsg_T *)&msg, sizeof(msg));
	if (ret) 
	{
		printf("FAILED to send CLOSE message to hidden WPA supplicant instance\n");
        close(sock);
        return NOK;
	}
    if(hidden_profile_GetResp(&sock, &response) != OK) {
        printf("Can not get response.\n");
        return NOK;

    }

    if(response.type == HIDDEN_PROF_LINK_CLOSE_COMPLETE) {
        return OK;
    }
    printf("%s : %d Wrong response type from hidden WPA instance\n", __FUNCTION__, __LINE__);
    return NOK;
}

/******************************************************************************
*                      hidden_profile_PrepareRespSocket()					  *
*******************************************************************************
DESCRIPTION:   Creates and binds a socket to which hidden instance would send 
			   the reponses. Should be called from communicating process 				

INPUTS:			pSock

OUTPUT:			None

RETURNS:		OK/NOK                                           
******************************************************************************/
u32 hidden_profile_PrepareRespSocket(u32 *pSock)
{
#ifdef VXWORKS
    struct sockaddr_in sockAddr;

    /* Create socket from which to read. */
    *pSock = socket(AF_INET, SOCK_DGRAM, 0);
    if (*pSock < 0) {
        printf("WPA HIDDEN INST.:error opening datagram socket for WDS\n");
        return (NOK);
    }
    /* Create name. */
	sockAddr.sin_family = AF_INET;
    bzero((char *)&sockAddr, sizeof(sockAddr));
    sockAddr.sin_len = sizeof(sockAddr);
    sockAddr.sin_family = AF_INET;
    sockAddr.sin_port = htons(TO_WPA_HIDDEN_PROFILE_SOCK_RESP_PORT);
    sockAddr.sin_addr.s_addr = inet_addr(IPC_LOCAL_INET_ADDR);
    if(sockAddr.sin_addr.s_addr == ERROR) {
	    printf("%s : inet_addr returns error: %s\n",__FUNCTION__, strerror(errno));
        close(*pSock);
        return (NOK);
    }

#else /* ifdef VXWORKS */
    struct sockaddr_un sockAddr;

    /* Create socket from which to read. */
    *pSock = socket(AF_UNIX, SOCK_DGRAM, 0);
    if (*pSock < 0) {
        printf("WPA HIDDEN INST.:error opening datagram socket for WDS\n");
        return (NOK);
    }
    /* Create name. */
    sockAddr.sun_family = AF_UNIX;

    strcpy((char*)sockAddr.sun_path,TO_WPA_HIDDEN_PROFILE_SOCK_RESP);

    unlink(TO_WPA_HIDDEN_PROFILE_SOCK_RESP	);
#endif /* ifdef VXWORKS*/  
    /* Bind address to the created socket */
    if (bind(*pSock, (struct sockaddr *)&sockAddr, sizeof(sockAddr)) < 0 )
	{
		  printf("WPA HIDDEN INST.:error binding datagram socket\n");
          close(*pSock);
          return (NOK);
    }

    return (OK);
}

/******************************************************************************
*                          hidden_profile_GetResp()					          *
*******************************************************************************
DESCRIPTION:    
				

INPUTS:			

OUTPUT:			None

RETURNS:		OK/NOK                                           
******************************************************************************/
u32 hidden_profile_GetResp(u32 *pSock, hiddenWpaInstanceMsg_T *pMsg)
{
    u32 sockAddrSize;
    fd_set readFdSet;
    int retCode;
    struct timeval timeOut;
#ifdef VXWORKS
    struct sockaddr_in from;
#else /* ifdef VXWORKS */
    struct sockaddr_un from;
#endif /* ifdef VXWORKS */
    FD_ZERO(&readFdSet);
    FD_SET(*pSock, &readFdSet);

    sockAddrSize = sizeof(from);
    timeOut.tv_usec = 0;
    timeOut.tv_sec = 10;
    while(1) {
        retCode = select(FD_SETSIZE, &readFdSet, NULL, NULL, &timeOut);
        if(retCode < 0) {
            if(errno == EINTR) {
                /* Ignore interrupted by signal */
                continue;
            }
            printf("%s : Error returned by select: %s.", __FUNCTION__, strerror(errno));
            break;
        }
        if(retCode == 0) {
            printf("%s : Timeout.", __FUNCTION__);
            break;

        }
        if(FD_ISSET(*pSock, &readFdSet)) {
            if (recvfrom(*pSock, (void *) pMsg, sizeof(*pMsg), 0, 
                         (struct sockaddr *)&from, &sockAddrSize) < 0) {
                printf("Error reading hidden WPA instance: %s", strerror(errno));
                break;
            }
            close(*pSock);
            return (OK);
        }
    }
    close(*pSock);
    return (NOK);
}

