/* JUNGO_COPYRIGHT_1 */

#include <time.h>
#include <rg_memory.h>
#include <kos/knet.h>
#include <rg_error.h>
#include <rg_config.h>
#include "wpa_info_elem.h"
#include "os_includes.h"
#include "wpa_log.h"
#include "wpa_common.h"
#include "debug_report.h"

#define WPA_IE_HDR_OUI 0x01
#define WPA_IE_VERSION 0x01

#define WPA_IE_ID      0xdd
#define WPA2_IE_ID     0x30

#define WPA_IE_AUTH_NONE  0x00
#define WPA_IE_AUTH_8021X 0x01
#define WPA_IE_AUTH_PSK   0x02

#define WPA_IE_CIPHER_USE_GROUP_KEY 0x00
#define WPA_IE_CIPHER_WEP_40  	    0x01
#define WPA_IE_CIPHER_TKIP          0x02
#define WPA_IE_CIPHER_WRAP          0x03
#define WPA_IE_CIPHER_CCMP          0x04
#define WPA_IE_CIPHER_WEP_104       0x05


/* capabilties */
#define WPA_IE_CAP_PRE_AUTH_BIT     0x0001

static u8 wpa_ie_oui[] = {0x00, 0x50, 0xf2};
static u8 wpa2_ie_oui[] = {0x00, 0x0f, 0xac};

#define WPA_OUI_TYPE      0
#define WPA2_OUI_TYPE     1


#pragma pack(1)
typedef struct {
    u8 oui[3];
    u8 suite;
} suite_selector_t;

typedef struct {
    u8 id;
    u8 len;      /* Represented as little-endian */
    u8 oui[4];
    u16 version; /* Represented as little-endian */
} wpa_ie_hdr_t;

typedef struct {
    u8 id;
    u8 len;      /* Represented as little-endian */
    u16 version; /* Represented as little-endian */
} wpa2_ie_hdr_t;

#pragma pack()

static int bit_count(int bitmap)
{
    int i, count = 0;
    
    for (i = 0; i < sizeof(int) * 8; i++)
    {
		if (bitmap & (1 << i))
			count++;
    }
	
    return count;
}

/* Writes a suite selector to buf and returns the position following the written
* suite selector
*/
static u8 *add_suite_selector(u8 *buf, u8 suite,u8 oui_type)
{
    suite_selector_t *selector = (suite_selector_t *)buf;
	
	if(oui_type == WPA2_OUI_TYPE)
		memcpy(selector->oui, wpa2_ie_oui, sizeof(selector->oui));
    else
		memcpy(selector->oui, wpa_ie_oui, sizeof(selector->oui));
	
	selector->suite = suite;
	
    return buf + sizeof(*selector);
}

wpa_info_elem_t *wpa_rsn_elem_open(wpa_cipher_t multicast_cipher, 
								   wpa_cipher_t unicast_ciphers,
								   wpa_auth_suite_t auth_suites,
								   UINT8 pre_auth_flag)
{
    u8 wpa_ie_len;
    wpa2_ie_hdr_t *wpa_ie_hdr;
    u8 *wpa_ie_tail;
    u16 num_unicast_ciphers = 1;/*bit_count(unicast_ciphers);*/
    u16 num_auth_suites = bit_count(auth_suites);
	
    DEBUG_PRINT_m(SEV_INFO, MOD_WPA_INIT,"Building information element. Multicast cipher=%X, unicast "
		"cipher=%X, authentication suites=%X", multicast_cipher, 
		unicast_ciphers, auth_suites);
    
    wpa_ie_len = 
		/* Size of header */
		sizeof(wpa2_ie_hdr_t) +
		/* Multicast cipher */
		sizeof(suite_selector_t) + 
		/* Unicast cipher and authentication suite lists */
		(2 * sizeof(u16)) + (sizeof(suite_selector_t) * (num_unicast_ciphers + 
		num_auth_suites)) + 2;
	
    wpa_ie_hdr = (wpa2_ie_hdr_t *)zalloc_l(wpa_ie_len);
    if (!wpa_ie_hdr)
		return NULL;
	
    /* Fill information element header */
    wpa_ie_hdr->id = WPA2_IE_ID;
	
    /* Length of ID and length fields is not included in length */
    wpa_ie_hdr->len = wpa_ie_len - (sizeof(wpa_ie_hdr->id) + 
		sizeof(wpa_ie_hdr->len));
    wpa_ie_hdr->version = knet_cpu_to_le16(WPA_IE_VERSION);
	
    /* Find where IE header ends and start adding things after it */
    wpa_ie_tail = (u8 *)wpa_ie_hdr + sizeof(*wpa_ie_hdr);
	/* add multi cipher */
	if (multicast_cipher == WPA_CIPHER_TKIP)
		wpa_ie_tail = add_suite_selector(wpa_ie_tail, WPA_IE_CIPHER_TKIP,WPA2_OUI_TYPE);
    else 
		wpa_ie_tail = add_suite_selector(wpa_ie_tail, WPA_IE_CIPHER_CCMP,WPA2_OUI_TYPE);
	
    /* Add unicast cipher suites */
    *((u16 *)wpa_ie_tail) = knet_cpu_to_le16(num_unicast_ciphers);
    wpa_ie_tail += sizeof(u16);
	
	if (unicast_ciphers == WPA_CIPHER_TKIP)
		wpa_ie_tail = add_suite_selector(wpa_ie_tail, WPA_IE_CIPHER_TKIP,WPA2_OUI_TYPE);
    else 
		wpa_ie_tail = add_suite_selector(wpa_ie_tail, WPA_IE_CIPHER_CCMP,WPA2_OUI_TYPE);
	
	
    /* Add authenticated key management suites */
	/* TODO : add 80211 support (radius)*/
    *((u16 *)wpa_ie_tail) = knet_cpu_to_le16(num_auth_suites);
    wpa_ie_tail += sizeof(u16);
	
    if (auth_suites & WPA_AUTH_8021X)
		wpa_ie_tail = add_suite_selector(wpa_ie_tail, WPA_IE_AUTH_8021X,WPA2_OUI_TYPE);
    else if (auth_suites & WPA_AUTH_PSK)
		wpa_ie_tail = add_suite_selector(wpa_ie_tail, WPA_IE_AUTH_PSK,WPA2_OUI_TYPE);
	
	
	/* Capability information field 
	add pre authentication */
	if(pre_auth_flag)
	{
		*((u16 *)wpa_ie_tail) = knet_cpu_to_le16(WPA_IE_CAP_PRE_AUTH_BIT);
		wpa_ie_tail += sizeof(u16);
	}
	
    wpa_ie_tail = (u8 *)wpa_ie_hdr + sizeof(*wpa_ie_hdr);
	
    DEBUG_PRINT_m(SEV_INFO, MOD_WPA_INIT,"Information element=\n%s", wpa_log_format_buf((u8 *)wpa_ie_hdr, 
		wpa_ie_len));
	
    return (wpa_info_elem_t *)wpa_ie_hdr;
}


wpa_info_elem_t *wpa_info_elem_open(wpa_cipher_t multicast_cipher, 
									wpa_cipher_t unicast_ciphers, wpa_auth_suite_t auth_suites)
{
    u8 wpa_ie_len;
    wpa_ie_hdr_t *wpa_ie_hdr;
    u8 *wpa_ie_tail;
    u16 num_unicast_ciphers = bit_count(unicast_ciphers);
    u16 num_auth_suites = bit_count(auth_suites);
	
    DEBUG_PRINT_m(SEV_INFO, MOD_WPA_INIT,"Building information element. Multicast cipher=%X, unicast "
		"cipher=%X, authentication suites=%X", multicast_cipher, 
		unicast_ciphers, auth_suites);
	
	
    
    wpa_ie_len = 
		/* Size of header */
		sizeof(wpa_ie_hdr_t) +
		/* Multicast cipher */
		sizeof(suite_selector_t) + 
		/* Unicast cipher and authentication suite lists */
		(2 * sizeof(u16)) + (sizeof(suite_selector_t) * (num_unicast_ciphers + 
		num_auth_suites));
	
    wpa_ie_hdr = (wpa_ie_hdr_t *)zalloc_l(wpa_ie_len);
    if (!wpa_ie_hdr)
		return NULL;
	
	
	
    /* Fill information element header */
    wpa_ie_hdr->id = WPA_IE_ID;
    memcpy(&wpa_ie_hdr->oui, wpa_ie_oui, sizeof(wpa_ie_hdr->oui));
    wpa_ie_hdr->oui[3] = WPA_IE_HDR_OUI;
    
    /* Length of ID and length fields is not included in length */
    wpa_ie_hdr->len = wpa_ie_len - (sizeof(wpa_ie_hdr->id) + 
		sizeof(wpa_ie_hdr->len));
    wpa_ie_hdr->version = knet_cpu_to_le16(WPA_IE_VERSION);
	
    /* Find where IE header ends and start adding things after it */
    wpa_ie_tail = (u8 *)wpa_ie_hdr + sizeof(*wpa_ie_hdr);
	
    /* Add multicast cipher suite */
    if (multicast_cipher & WPA_CIPHER_WEP_104)
		wpa_ie_tail = add_suite_selector(wpa_ie_tail, WPA_IE_CIPHER_WEP_104,WPA_OUI_TYPE);
    else if (multicast_cipher & WPA_CIPHER_WEP_40)
		wpa_ie_tail = add_suite_selector(wpa_ie_tail, WPA_IE_CIPHER_WEP_40,WPA_OUI_TYPE);
    else if (multicast_cipher & WPA_CIPHER_TKIP)
		wpa_ie_tail = add_suite_selector(wpa_ie_tail, WPA_IE_CIPHER_TKIP,WPA_OUI_TYPE);
	
    /* Add unicast cipher suites */
    *((u16 *)wpa_ie_tail) = knet_cpu_to_le16(num_unicast_ciphers);
    wpa_ie_tail += sizeof(u16);
	
    if (unicast_ciphers & WPA_CIPHER_TKIP)
		wpa_ie_tail = add_suite_selector(wpa_ie_tail, WPA_IE_CIPHER_TKIP,WPA_OUI_TYPE);
    else if (unicast_ciphers & WPA_CIPHER_CCMP)
		wpa_ie_tail = add_suite_selector(wpa_ie_tail, WPA_IE_CIPHER_CCMP,WPA_OUI_TYPE);
	
    /* Add authenticated key management suites */
    *((u16 *)wpa_ie_tail) = knet_cpu_to_le16(num_auth_suites);
    wpa_ie_tail += sizeof(u16);
	
    if (auth_suites & WPA_AUTH_8021X)
		wpa_ie_tail = add_suite_selector(wpa_ie_tail, WPA_IE_AUTH_8021X,WPA_OUI_TYPE);
    if (auth_suites & WPA_AUTH_PSK)
		wpa_ie_tail = add_suite_selector(wpa_ie_tail, WPA_IE_AUTH_PSK,WPA_OUI_TYPE);
    
	/* Capability information field is omitted because we always set it to 0
		* anyway
	*/
	
	
    DEBUG_PRINT_m(SEV_INFO, MOD_WPA_INIT,"Information element=\n%s", wpa_log_format_buf((u8 *)wpa_ie_hdr, 
		wpa_ie_len));
	
    return (wpa_info_elem_t *)wpa_ie_hdr;
}

void wpa_info_elem_close(wpa_info_elem_t *t)
{
    free(t);
}

int wpa_info_elem_get_len(wpa_info_elem_t *t)
{
	wpa_ie_hdr_t *wpa_ie_hdr = (wpa_ie_hdr_t *)t;

	return wpa_ie_hdr->len + sizeof(wpa_ie_hdr->id) + sizeof(wpa_ie_hdr->len);
}

int wpa_info_elem_get_info_type(wpa_info_elem_t *t)
{
	wpa_ie_hdr_t *wpa_ie_hdr = (wpa_ie_hdr_t *)t;
	if( wpa_ie_hdr->id == WPA_IE_ID)
		return WPA_STA_TYPE_WPA;
    else if(wpa_ie_hdr->id == WPA2_IE_ID)
		return WPA_STA_TYPE_WPA2;
	else
		return -1;
}



int wpa_info_elem_parse(wpa_info_elem_t *t, wpa_cipher_t *multicast_cipher, 
						wpa_cipher_t *unicast_ciphers, wpa_auth_suite_t *auth_suites, int len)
{
	wpa_ie_hdr_t *wpa_ie_hdr = (wpa_ie_hdr_t *)t;
	volatile u8 *cur;
	u8 *wpa_ie_end = (u8 *)wpa_ie_hdr + len;
	suite_selector_t *suite_selector;
	volatile int suite_count;
	int i;
	
	DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Parsing information element=\n%s", wpa_log_format_buf((u8 *)t, 
		len));
	
		/* In case some of the items are not specified in the information element,
		* the spec defines the following default values
	*/
	*multicast_cipher = WPA_CIPHER_TKIP;
	*unicast_ciphers = WPA_CIPHER_TKIP;
	*auth_suites = WPA_AUTH_8021X;
	
	/* Make sure information element header makes sense */
	if ((wpa_ie_hdr->len + 2 != len) || 
		(wpa_ie_hdr->len < 6) || 
		(wpa_ie_hdr->id != WPA_IE_ID) ||
		memcmp(wpa_ie_hdr->oui, wpa_ie_oui, sizeof(wpa_ie_oui)) ||
		(wpa_ie_hdr->oui[3] != WPA_IE_HDR_OUI) || 
		(knet_le16_to_cpu(wpa_ie_hdr->version) != WPA_IE_VERSION)) 
	{
		DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Bad information element header");
		return -1;
	}
	
	/* Start going over the information element, cur points to the current
	* position, we advance it as we search the information element
	*/
	cur = (u8 *)wpa_ie_hdr + sizeof(*wpa_ie_hdr);
	
	if ((cur + sizeof(suite_selector_t)) > wpa_ie_end)
		return 0;
	
	suite_selector = (suite_selector_t *)cur;
	
	/* Validate suite selector's oui */
	if (memcmp(suite_selector->oui, wpa_ie_oui, sizeof(wpa_ie_oui)))
        return -1;
	
	/* Convert IE suite value to one of our bits */
	switch (suite_selector->suite)
	{
	case WPA_IE_CIPHER_TKIP:
		*multicast_cipher = WPA_CIPHER_TKIP;
		break;
	case WPA_IE_CIPHER_WEP_40:
		*multicast_cipher = WPA_CIPHER_WEP_40;
		break;
	case WPA_IE_CIPHER_WEP_104:
		*multicast_cipher = WPA_CIPHER_WEP_104;
		break;
	case WPA_IE_CIPHER_WRAP:
		*multicast_cipher = WPA_CIPHER_WRAP;
		break;
	case WPA_IE_CIPHER_CCMP:
		*multicast_cipher = WPA_CIPHER_CCMP;
		break;
	default:
		return -1;
		break;
	}
	
	cur += sizeof(suite_selector_t);
	
	if ((cur + sizeof(u16)) > wpa_ie_end)
		return 0;
	
	/* Get number of unicast suites */
	{
		/* Avoid exceptions, read 16bit value that is suppose to be little endian */
		/* replaces: suite_count = knet_le16_to_cpu(*((u16 *)cur)); */
		u8 lsb = *((u8*)cur);
		u8 msb = *(((u8*)cur)+1);
		u16 val = (((u16)msb) << 8) | ((u16)lsb);
		suite_count = val;
	}
	
	if ((cur + suite_count * sizeof(suite_selector_t)) > wpa_ie_end)
		return -1;
	
	cur += sizeof(u16);
	
	/* Unicast suites are specified in the IE so lose the default */
	*unicast_ciphers = 0;
	
	for (i = 0; i < suite_count; i++)
	{
		suite_selector_t *suite_selector = (suite_selector_t *)cur;
		
		/* Validate suite selector's oui */
		if (memcmp(suite_selector->oui, wpa_ie_oui, sizeof(wpa_ie_oui)))
			return -1;
		
		/* Convert IE suite value to one of our bits */
		switch (suite_selector->suite)
		{
		case WPA_IE_CIPHER_TKIP:
			*unicast_ciphers |= WPA_CIPHER_TKIP;
			break;
		case WPA_IE_CIPHER_WRAP:
			*unicast_ciphers |= WPA_CIPHER_WRAP;
			break;
		case WPA_IE_CIPHER_CCMP:
			*unicast_ciphers |= WPA_CIPHER_CCMP;
			break;
		default:
			return -1;
			break;
		}
		
		cur += sizeof(suite_selector_t);
	}
	
	if ((cur + sizeof(u16)) > wpa_ie_end)
		return 0;
	
	/* Authenticated key management suites are specified so lose the default */
	*auth_suites = 0;
	
	/* Get number of authenticated key management suites */
	{
		/* Avoid exceptions, read 16bit value that is suppose to be little endian */
		/* replaces: suite_count = knet_le16_to_cpu(*((u16 *)cur)); */
		u8 lsb = *((u8*)cur);
		u8 msb = *(((u8*)cur)+1);
		u16 val = (((u16)msb) << 8) | ((u16)lsb);
		suite_count = val;
	}
	
	if ((cur + suite_count * sizeof(suite_selector_t)) > wpa_ie_end)
		return -1;
	
	cur += sizeof(u16);
	
	for (i = 0; i < suite_count; i++)
	{
		suite_selector_t *suite_selector = (suite_selector_t *)cur; 
		
		/* Validate suite selector's oui */
		if (memcmp(suite_selector->oui, wpa_ie_oui, sizeof(wpa_ie_oui)))
			return -1;
		
		/* Convert IE suite value to one of our bits */
		switch (suite_selector->suite)
		{
		case WPA_IE_AUTH_NONE:
			break;
		case WPA_IE_AUTH_8021X:
			*auth_suites |= WPA_AUTH_8021X;
			break;
		case WPA_IE_AUTH_PSK:
			*auth_suites |= WPA_AUTH_PSK;
			break;
		default:
			return -1;
			break;
		}
		
		cur += sizeof(suite_selector_t);
	}
	
	return 0;
}


int wpa2_info_elem_parse(wpa_info_elem_t *t, wpa_cipher_t *multicast_cipher, 
						 wpa_cipher_t *unicast_ciphers, wpa_auth_suite_t *auth_suites,
						 u8 *pre_auth_flag,u8 **pmk_id_list,u8 *num_of_ids,int len)
{
	wpa2_ie_hdr_t *wpa_ie_hdr = (wpa2_ie_hdr_t *)t;
	volatile u8 *cur;
	u8 *wpa_ie_end = (u8 *)wpa_ie_hdr + len ,*tmp;
	suite_selector_t *suite_selector;
	volatile int suite_count;
	int i;
	
	DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Parsing information element=\n%s", wpa_log_format_buf((u8 *)t, 
		len));
/*	printf("Parsing information element=\n%s", wpa_log_format_buf((u8 *)t, 
		len));*/
	
		/* In case some of the items are not specified in the information element,
		* the spec defines the following default values
	*/
	*multicast_cipher = WPA_CIPHER_CCMP;
	*unicast_ciphers = WPA_CIPHER_CCMP;
	*auth_suites = WPA_AUTH_8021X;
	
	/* Make sure information element header makes sense */
	if ((wpa_ie_hdr->len + 2 != len) || 
		(wpa_ie_hdr->len < 4) || 
		(wpa_ie_hdr->id != WPA2_IE_ID) 
		)
	{
		DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Bad information element header");
		return -1;
	}
	
	
	
	/* Start going over the information element, cur points to the current
	* position, we advance it as we search the information element
	*/
	cur = (u8 *)wpa_ie_hdr + sizeof(*wpa_ie_hdr);

	if ((cur + sizeof(suite_selector_t)) > wpa_ie_end)
		return 0;
	
	suite_selector = (suite_selector_t *)cur;
	
	
	/* Validate suite selector's oui */
	if (memcmp(suite_selector->oui, wpa2_ie_oui, sizeof(wpa2_ie_oui)))
	{
        DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Bad oui on first broadcast suite");
        return -1;
	}
	
	
	/* Convert IE suite value to one of our bits */
	switch (suite_selector->suite)
	{
	case WPA_IE_CIPHER_TKIP:
		*multicast_cipher = WPA_CIPHER_TKIP;
		break;
	case WPA_IE_CIPHER_CCMP:
		*multicast_cipher = WPA_CIPHER_CCMP;
		break;
	default:
		DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"unknown broadcast cipher suite");
		return -1;
		break;
	}
	
	
	cur += sizeof(suite_selector_t);

	if ((cur + sizeof(u16)) > wpa_ie_end)
		return 0;
	
	
	/* Get number of unicast suites */
	{
		/* Avoid exceptions, read 16bit value that is suppose to be little endian */
		/* replaces: suite_count = knet_le16_to_cpu(*((u16 *)cur)); */
		u8 lsb = *((u8*)cur);
		u8 msb = *(((u8*)cur)+1);
		u16 val = (((u16)msb) << 8) | ((u16)lsb);
		suite_count = val;
	}
	
	if ((cur + suite_count * sizeof(suite_selector_t)) > wpa_ie_end)
	{
		DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Bad number of unicast suites %d",suite_count);
		return -1;
	}
	
	
	cur += sizeof(u16);
	
	/* Unicast suites are specified in the IE so lose the default */
	*unicast_ciphers = 0;
	
	for (i = 0; i < suite_count; i++)
	{
		suite_selector_t *suite_selector = (suite_selector_t *)cur;
		
		/* Validate suite selector's oui */
		if (memcmp(suite_selector->oui, wpa2_ie_oui, sizeof(wpa2_ie_oui)))
		{
			DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Bad oui for unicast suite");
			return -1;
		}
		
		
		/* Convert IE suite value to one of our bits */
		switch (suite_selector->suite)
		{
		case WPA_IE_CIPHER_CCMP:
			*unicast_ciphers |= WPA_CIPHER_CCMP;
			break;
		case WPA_IE_CIPHER_TKIP:
			*unicast_ciphers |= WPA_CIPHER_TKIP;
			break;
			
		default:
			DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"unknown unicast suite type");
			return -1;
			break;
		}
		
		cur += sizeof(suite_selector_t);
	}
	
	
	if ((cur + sizeof(u16)) > wpa_ie_end)
	{
		return 0;
	}
	
	
	/* Authenticated key management suites are specified so lose the default */
	*auth_suites = 0;
	
	/* Get number of authenticated key management suites */
	{
		/* Avoid exceptions, read 16bit value that is suppose to be little endian */
		/* replaces: suite_count = knet_le16_to_cpu(*((u16 *)cur)); */
		u8 lsb = *((u8*)cur);
		u8 msb = *(((u8*)cur)+1);
		u16 val = (((u16)msb) << 8) | ((u16)lsb);
		suite_count = val;
	}
	
	if ((cur + suite_count * sizeof(suite_selector_t)) > wpa_ie_end)
		return -1;
	
	
	cur += sizeof(u16);
	
	for (i = 0; i < suite_count; i++)
	{
		suite_selector_t *suite_selector = (suite_selector_t *)cur; 
		
		/* Validate suite selector's oui */
		if (memcmp(suite_selector->oui, wpa2_ie_oui, sizeof(wpa2_ie_oui)))
			return -1;
		
		/* Convert IE suite value to one of our bits */
		switch (suite_selector->suite)
		{
		case WPA_IE_AUTH_NONE:
			break;
		case WPA_IE_AUTH_8021X:
			*auth_suites |= WPA_AUTH_8021X;
			break;
		case WPA_IE_AUTH_PSK:
			*auth_suites |= WPA_AUTH_PSK;
			break;
		default:
			return -1;
			break;
		}
		
		cur += sizeof(suite_selector_t);
	}



	if ((cur + sizeof(u16)) > wpa_ie_end)
		return 0;


	/* capabilities */
	{
		u16 cap = knet_le16_to_cpu(*((u16 *)cur));
		if(cap & WPA_IE_CAP_PRE_AUTH_BIT )
		{
			*pre_auth_flag = 1;
		}
	}



	cur += sizeof(u16);

	if ((cur + sizeof(u16)) > wpa_ie_end)
		return 0;
	



	/* pmk cache list*/
	*num_of_ids = knet_le16_to_cpu(*((u16 *)cur));

	cur += sizeof(u16);
	
	/* run on all pmk id's and update output pmk_id_list*/
	*pmk_id_list = (u8*)malloc(16 * (*num_of_ids) );
	tmp = *pmk_id_list;	
	for(i=0;i< *num_of_ids;i++)
	{
		memcpy(tmp,(u8*)cur,16);
		tmp += 16;
		if ((cur + sizeof(u16)) > wpa_ie_end)
		{
			*num_of_ids = i;
			DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"wpa2_info_elem_parse : different between num of pmkid elements and info element size\n");
			return 0;
		}
		cur+=16;
	}

	return 0;
}


int wpa_info_elem_get_pmk_id(wpa_info_elem_t *t,u8 **pmk_id,u8 *num_pmk_id)
{
	wpa_cipher_t     multicast_cipher; 
	wpa_cipher_t     unicast_ciphers;
	wpa_auth_suite_t auth_suites;
	u8 pre_auth_flag;
	int info_elem_buf_len;
	
	if(!t)
		return -1;
	
	info_elem_buf_len = wpa_info_elem_get_len(t);

	if(( wpa2_info_elem_parse(t, &multicast_cipher, &unicast_ciphers,
		&auth_suites,&pre_auth_flag,pmk_id,num_pmk_id,info_elem_buf_len)) == -1)
	{
		DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"wpa_info_elem_get_pmk_id:error parsing station info element\n");
		return -1;
	}
	
	if(!*num_pmk_id)
		return -1; 

	return 0;
}
