/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2004 Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |** 
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*																									*/
/*	 MODULE:	wpa_supplicant.c	   									   */
/*   PURPOSE:	WPA-2 supplicant functions.			   	 				   */
/*																									*/
/***************************************************************************/
/***************************************************************************/

#include <stdio.h>
#ifndef VXWORKS
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/un.h>
#include <net_utils.h>
#endif /* ifndef VXWORKS */
#include <kos/knet.h>
#include <ipc.h>
#include "eapol.h"
#include "wpa_supplicant.h"
#include "wpa_key_utils.h"
#include "wpa_assoc.h"
#include "wpa_info_elem.h"
#include "wpaIfc.h"
#include "os_libc_extra.h"
#include "wpa_log.h"
#include "debug_report.h"

#define IS_PAIRWISE_KEY(key_desc) (ntohs((key_desc)->info) & KEY_INFO_PAIRWISE_BIT)

/* supplicant state machine states*/
typedef enum
{
	SUPPL_WAIT_FOR_CFG,
	SUPPL_WAIT_EAPOL_KEY,
	SUPPL_AUTHENTICATED,
}WpasState_T;

/* received EAPOL-key check status codes */
typedef enum
{
	EAPOL_KEY_OK,         /* proceed */
	EAPOL_KEY_DISCARD,    /* silently discard the packet */
	EAPOL_KEY_FATAL_FAIL     /* terminate the handshake and report FAILED status to WDS*/
}EapolKeyStatus_T;

/* parameters derived from 4-way handshake*/
typedef struct
{
	u64 replayCount;
	wpa_nonce_t snonce;
	wpa_ptk_t ptk;
}HandshakeParams_T;

/* WPA Supplicant local DB*/
   
static ether_mac_t			 authMAC;
static wpa_pmk_t			 pmk;

/* Parameters derived during 4-way handshake */
static HandshakeParams_T handshakePars;

/* driver interface callbacks*/
static int (*sendEapolCB)(auth1x_pc_t *t, void *buf, int len,int encrypt ,u16 eth_type);

/* static functions */
static u32 isEapolKey_Msg1(eapol_key_desc_t *eapolKeyDesc);
static EapolKeyStatus_T verifyEapolKey_3(wpa_ptk_t *ptk, eapol_pkt_t *eapolPkt, wpa_port_t *wpaPort);
static STATUS	 verifyReplayCount(u64 replayCounter, eapol_key_desc_t *eapolKeyDesc);
static eapol_pkt_t *supplicCreateEapolKey( wpa_ptk_t *ptk, u32 keyInfoBits,
										   u8 *keyData, u32 keyDataLen, wpa_port_t *port);
static void 		supplicSendEapolKey(eapol_pkt_t *eapolPkt);


/******************************************************************************
*                      wpas_InitSupplicant()								  *
*******************************************************************************
DESCRIPTION:    Initializes the WPA supplicant parameters
                                                                         
INPUTS:			None

OUTPUT:			None

RETURNS:		None                                                    
******************************************************************************/
void wpas_InitSupplicant(void)
{
    /* Reset authenticator MAC*/
	memset(authMAC, 0, sizeof(authMAC));
	
	/* Clear PMK */
	memset(&pmk, 0, sizeof(pmk));
}

/******************************************************************************
*                      wpas_OpenNewAssoc()								  *
*******************************************************************************
DESCRIPTION:    Initializes the WPA supplicant new assoc paramss
                                                                         
INPUTS:			wpaPort - WPA port params
				pCfg -  aussociation params

OUTPUT:			None

RETURNS:		OK/NOK                                                    
******************************************************************************/
void wpas_OpenNewAssoc(wpa_port_t *wpaPort, dynamicSecureCfg_T *pCfg)
{
    /* Copy the received configuration to the local DB*/
	memcpy(authMAC, pCfg->secPartyMac, sizeof(pCfg->secPartyMac));
	
	/* We work in WPA PSK hence set PMK to be the same as PSK*/
	memcpy(&pmk, &wpaPort->psk, sizeof(pmk));
 
	DEBUG_PRINT_m(SEV_INFO, MOD_WPA_HIDDEN,"WPA SUPPL: received configuration:\n"
			  "			  auth MAC: %02X%02X%02X%02X%02X%02X\n PMK = %s",
			  authMAC[0],authMAC[1], authMAC[2],
			  authMAC[3], authMAC[4],authMAC[5], 
			  wpa_log_format_buf((u8 *)&pmk, sizeof(pmk)));

}


/******************************************************************************
*                      wpas_SupplGetEvent()							  *
*******************************************************************************
DESCRIPTION:    Implements the WPA supplicant side of 4-way handshake for  WPA 
                                                                         
INPUTS:			event -supplicant event
				eapolPkt - received EAPOL-key frame or NULL to indicate configuration
						   received	or abort message received
				wpaPort - params of corresponding WPA profile 
OUTPUT:			None

RETURNS:		None                                               
******************************************************************************/
void wpas_SupplGetEvent(wpasEvent_T event, eapol_pkt_t *eapolPkt, wpa_port_t *wpaPort)
{
	/*wpa_port_t *wpaPort = secProfileData -> pWpaPortContext;*/
	static wpa_nonce_t preANonce;/* previous message ANonce used for detecting retransmissions*/
	static u8   havePtk = FALSE; /* indicates if PTK have been already generated for current handshk*/
	static u8   haveSnonce = FALSE; /* indicates if Snonce have been already generated for current handshk*/
	static u8   ptkInstalled = FALSE;/* indicates if PTK have been already installed for current handshk*/ 
	static wpa_ptk_t tPtk; /* temporary ptk until valid message 3 arrived
							  used to stop attakers changing the supplicant's PTK
							  by sending message 1 */

	static WpasState_T 		 supplCurState = SUPPL_WAIT_FOR_CFG;

    switch (event) 
    {
        case WPAS_RESTART:    /* prepare for a new handshake no matter what is current state*/
            /* We'll need only send_eapol field from this struct */
            {
                auth1x_pc_ops_t portCBs;
    
                /* prepare for a new handshake */
                handshakePars.replayCount = 0;
                havePtk = FALSE;
                haveSnonce = FALSE;
                /*initialize previous seen ANonce to 0*/
                memset(&preANonce, 0 , sizeof(preANonce));
    
                /* Initialize a callback for sending EAPOLs - the same as for authenticator*/
                wdrv_wpaApPortInitCallbacks(&portCBs);
                sendEapolCB = portCBs.send_eapol;
                if (!sendEapolCB) {
                    printf("WPA SUPPL: NULL send eapol callback\n");
                    return;
                }
    
                supplCurState = SUPPL_WAIT_EAPOL_KEY;
                DEBUG_PRINT_m(SEV_INFO, MOD_WPA_HIDDEN,"WPA SUPPL: Ready for 4-way handshake. Current state WAIT_EAPOL_KEY\n");
                return;
            }
            break;
        case WPAS_EAPOL:
            if (supplCurState == SUPPL_WAIT_FOR_CFG) {
                DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_HIDDEN,"WPA SUPPL: Error: EAPOL-key received prior to receiving configuration\n");
                return;
            }
            if (!eapolPkt) /* This indicates that current handshake should be aborted*/
            {
                DEBUG_PRINT_m(SEV_WARNING, MOD_WPA_HIDDEN,"WPA SUPPL: NULL EAPOL packet received\n");
                supplCurState = SUPPL_WAIT_FOR_CFG;
            }
            /*process the eapol*/
            {
    			eapol_key_desc_t *eapolKeyDesc = (eapol_key_desc_t *)eapolPkt->pkt_body;
    	
                /* Verify the replay counter */
    			if ( (verifyReplayCount(handshakePars.replayCount, eapolKeyDesc)) == NOK) {
    				DEBUG_PRINT_m(SEV_INFO, MOD_WPA_HIDDEN,"WPA SUPPL: Invalid replay counter - packet discarded \n");
    				return;
    			}
    	
    			/* If it is not a pairwise key packet - discard it - not used in this implementation */
    			if ( ! IS_PAIRWISE_KEY(eapolKeyDesc)) {
    				DEBUG_PRINT_m(SEV_INFO, MOD_WPA_HIDDEN,"WPA SUPPL: Group key EAPOL received - packet discarded\n");
    				return;
    			}
    
    			/* If EAPOL-key msg 1 received*/
    			if ( isEapolKey_Msg1(eapolKeyDesc) == TRUE)
    			{
    				wpa_nonce_t ANonce;
    				eapol_pkt_t *eapolPktMsg2;
    
    				DEBUG_PRINT_m(SEV_INFO, MOD_WPA_HIDDEN,"WPA SUPPL: processing EAPOL key message 1\n");
    				
    				/* Remember the last valid packet replay counter */
    				handshakePars.replayCount = knet_ntohll(eapolKeyDesc->replay_count);
    	
    				/* Generate SNonce once for a new handshake*/
    				if (haveSnonce == FALSE) 
    				{
    					DEBUG_PRINT_m(SEV_INFO, MOD_WPA_HIDDEN,"WPA SUPPL: generating sNonce\n");
    					memcpy(&handshakePars.snonce, &wpaPort->key_counter, sizeof(handshakePars.snonce));
    					wpa_key_utils_inc((u8 *)&wpaPort->key_counter, sizeof(wpaPort->key_counter));
    					haveSnonce = TRUE;
    				}
    
    				/* Don't generate new PTK if it is  a retransmission*/
    				memcpy(&ANonce, &eapolKeyDesc->nonce, sizeof(ANonce));
    				if (memcmp(&preANonce, &ANonce, sizeof(preANonce))) 
    				{
    					DEBUG_PRINT_m(SEV_INFO, MOD_WPA_HIDDEN,"WPA SUPPL: deriving the PTK\n");
    					/* Derive the PTK*/
    					tPtk.temporal_key.dataSize = WPA_AES_KEY_LEN;
    					wpa_key_utils_calc_ptk(&tPtk, &pmk, &ANonce,
    									    &handshakePars.snonce, authMAC,
    										wpaPort->params.mac);
    					havePtk = TRUE;
    
    					/* Remember last ANonce*/
    					memcpy(&preANonce, &ANonce, sizeof(preANonce));
    				}
    	
    				/* Create EAPOL-key message 2*/
    				eapolPktMsg2 = supplicCreateEapolKey(&tPtk, 
    							   KEY_INFO_PAIRWISE_BIT | KEY_INFO_VERSION_2_BIT|
    							   KEY_INFO_MIC_BIT,
    							   (u8 *)wpaPort->ap_rsn_info_elem, 
    							   wpa_info_elem_get_len(wpaPort->ap_rsn_info_elem),
    							   wpaPort);
    
    				/* Send EAPOL-key message 2*/
    				DEBUG_PRINT_m(SEV_INFO, MOD_WPA_HIDDEN,"WPA SUPPL: sending EAPOL-key message 2\n");
    				supplicSendEapolKey(eapolPktMsg2);
    
    			}
    
    			else /* it's EAPOL-key message 3 */
    			{
    				EapolKeyStatus_T eapolSt;
    	
    				DEBUG_PRINT_m(SEV_INFO, MOD_WPA_HIDDEN,"WPA SUPPL: processing EAPOL key message 3\n");
    
    				/* If still don't have the PTK discard the message*/
    				if (havePtk == FALSE) 
    				{
    					DEBUG_PRINT_m(SEV_WARNING, MOD_WPA_HIDDEN,"WPA SUPPL: Message 3 received prior to valid message 1\n");
    					return;
    				}
    				if (memcmp(&preANonce, &eapolKeyDesc->nonce, sizeof(preANonce) ) )
    				{
    					DEBUG_PRINT_m(SEV_INFO, MOD_WPA_HIDDEN,"WPA SUPPL: ANonce not identical to message 1 - discarding packet\n");
    					return;
    				}
    
    				eapolSt = verifyEapolKey_3(&tPtk, eapolPkt, wpaPort);
    				/* If critical error occured - invalid RSN IE*/
    				if (eapolSt == EAPOL_KEY_FATAL_FAIL ) 
    				{
    					DEBUG_PRINT_m(SEV_INFO, MOD_WPA_HIDDEN,"WPA SUPPL: Fatal error in message 3 - terminating the handshake\n");
    					/* Terminate current 4-way handshake*/
    					supplCurState = SUPPL_WAIT_FOR_CFG;
    					/* Report the failure status*/
    					authStatusReport(authMAC, SEC_AUTH_STATUS_FAILURE);
    					return;
    				}
    				else if (eapolSt == EAPOL_KEY_OK) 
    					{
    						eapol_pkt_t *eapolPktMsg4;
    						
    						DEBUG_PRINT_m(SEV_INFO, MOD_WPA_HIDDEN,"WPA SUPPL: message 3 is OK\n");
    
    						/* Remember the last valid packet replay counter */
    						handshakePars.replayCount = knet_ntohll(eapolKeyDesc->replay_count);
    	
    						/* set PTK from TPTK (it's valid, so it's no longer temporary) */
    						memcpy(&handshakePars.ptk, &tPtk, sizeof(handshakePars.ptk));
    
    						/* Create EAPOL-key message 4*/
    						eapolPktMsg4 = supplicCreateEapolKey(&tPtk, 
    								   KEY_INFO_PAIRWISE_BIT | KEY_INFO_VERSION_2_BIT |
    
    								   KEY_INFO_MIC_BIT | KEY_INFO_SECURE_BIT,
    								   NULL, 0, wpaPort);
    	
    						/* Send message 4*/
    						DEBUG_PRINT_m(SEV_INFO, MOD_WPA_HIDDEN,"WPA SUPPL: sending EAPOL-key message 4\n");
    						supplicSendEapolKey (eapolPktMsg4);
    	
    						/* install temporal key into the driver */
    						DEBUG_PRINT_m(SEV_INFO, MOD_WPA_HIDDEN,"WPA SUPPL: installing temporal key size=%d data =\n%s\n", handshakePars.ptk.temporal_key.dataSize,
											 wpa_log_format_buf((u8 *)&handshakePars.ptk.temporal_key.data.aes.key, 
																handshakePars.ptk.temporal_key.dataSize));
							
                            
    						wpaPort->params.ap_dev_ops->set_mapping_key(wpaPort->port_profile_index,
    																	authMAC, WPA_CIPHER_CCMP,
    																	&handshakePars.ptk.temporal_key);
    	
							ptkInstalled = TRUE;
    						authStatusReport(authMAC, SEC_AUTH_STATUS_OK);
    
    						/* transit to inital state  */
    						supplCurState = SUPPL_AUTHENTICATED;
    					}
    	
    			}
    		}
		
            break;
        case WPAS_ABORT:    
            DEBUG_PRINT_m(SEV_INFO, MOD_WPA_HIDDEN,"WPA SUPPL: Aborting current handshake. Current state - WAIT_FOR_CFG\n");
            supplCurState = SUPPL_WAIT_FOR_CFG;
            return;
            break;
		case WPAS_CLOSE_AUTH:
			if (supplCurState != SUPPL_AUTHENTICATED)
			{
				DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_HIDDEN,"WPA SUPPL:Close Authentication event received while not authenticated yet\n");
				return;
			}
			else 
			{
				DEBUG_PRINT_m(SEV_INFO, MOD_WPA_HIDDEN,"WPA SUPPL: Closing authentication and removing keys. Current state - WAIT_FOR_CFG\n");
				
				if (ptkInstalled == TRUE) 
				{
					wpaPort->params.ap_dev_ops->remove_mapping_key(NULL/*IGNORED*/, authMAC, WPA_CIPHER_CCMP);
					ptkInstalled = FALSE;
				}
				havePtk = FALSE;
				haveSnonce = FALSE;
				
				supplCurState = SUPPL_WAIT_FOR_CFG;
			}
			break;
        default:
            /* we shouldn't  reach here*/
            DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_HIDDEN,"Unknown WPA SUPPLICANT event\n");
            break;

    }
}

/******************************************************************************
*                      wpas_IsEapolToSupplicant()							  *
*******************************************************************************
DESCRIPTION:    Determines if the  EAPOL received by WPA process is addressed 
				to the supplicant by mac address
                                                                         
INPUTS:			srcMac - packet source MAC

OUTPUT:			None

RETURNS:		FALSE/TRUE                                               
******************************************************************************/
u32 wpas_IsPktToSupplicant(ether_mac_t srcMac)
{
    if ( !(memcmp(srcMac, authMAC, sizeof(ether_mac_t)))) 
		return TRUE;
	else 
		return FALSE;
}

/******************************************************************************
*                      verifyEapolKey_1()							  *
*******************************************************************************
DESCRIPTION:    Verifies if the given eapol-key descriptor is  message 1
                                                                          
INPUTS:			
				eapolKeyDesc - received EAPOL-key descriptor

OUTPUT:			None

RETURNS:		FALSE/TRUE                                         
******************************************************************************/
static u32 isEapolKey_Msg1(eapol_key_desc_t *eapolKeyDesc)
{
	/* Only message 1 has MIC bit not set */
	if ( ntohs(eapolKeyDesc->info) & KEY_INFO_MIC_BIT ) 
		return FALSE;
	else
		return TRUE;
}

/******************************************************************************
*                      verifyReplayCount()							  *
*******************************************************************************
DESCRIPTION:    Verifies if the given EAPOL packet contains valid replay counter
                                                                          
INPUTS:			replayCounter - current replay counter of the last received valid eapol-key
				eapolKeyDesc - received EAPOL-key descriptor

OUTPUT:			None

RETURNS:		OK/NOK                                         
******************************************************************************/
static STATUS verifyReplayCount(u64 replayCounter, eapol_key_desc_t *eapolKeyDesc)
{
	if(replayCounter >= knet_ntohll(eapolKeyDesc->replay_count))
		return NOK;
	else
		return OK;
}

/******************************************************************************
*                      verifyEapolKey_3()							  *
*******************************************************************************
DESCRIPTION:    Verifies if the given eapol-key descriptor is valid message 3
				from 4-way handshake
                                                                          
INPUTS:			PTK
				eapolKeyBody - received EAPOL-key descriptor

OUTPUT:			None

RETURNS:		OK/NOK                                          
******************************************************************************/
static EapolKeyStatus_T verifyEapolKey_3(wpa_ptk_t *ptk, eapol_pkt_t *eapolPkt, wpa_port_t *wpaPort)
{
	 eapol_key_desc_t *eapolKeyDesc = (eapol_key_desc_t *)eapolPkt->pkt_body;
	 wpa_assoc_t assoc; /* required in order to use the existing functions*/
	 u8 decryptedKeyData[1024];
	 u32 decryptDataLen, encryptDataLen;
	 BOOL ret;

	 /* Verify the MIC */
	  memcpy(&assoc.ptk, ptk, sizeof(assoc.ptk));
	  assoc.sta_type = WPA_STA_TYPE_WPA2;
	  if (mic_check(&assoc, eapolPkt))
	  {
	  	  DEBUG_PRINT_m(SEV_INFO, MOD_WPA_HIDDEN,"WPA SUPPL: MIC failure - discarding a packet\n");	  
		  return EAPOL_KEY_DISCARD;
	  }

	 /* Information element received in packet must be identical to local IE */
	  
	  /* Decrypt the key data */
	  
	  encryptDataLen = ntohs(eapolKeyDesc->data_len);
	  /* must have at least 2 blocks*/
	  if (encryptDataLen < 16) 
	  {
		  DEBUG_PRINT_m(SEV_INFO, MOD_WPA_HIDDEN,"WPA SUPPL: invalid RSN IE length - 4-way handshake failed\n");	  
		  return EAPOL_KEY_FATAL_FAIL;
	  }
	  
      /* compute output size (truncating input size to multiple of 8 bytes)
	     Decrypted data length is 8 bytes less */
	  decryptDataLen = (encryptDataLen & ~7) - 8;

	  if (decryptDataLen < wpa_info_elem_get_len(wpaPort->ap_rsn_info_elem)) 
	  {
		  DEBUG_PRINT_m(SEV_INFO, MOD_WPA_HIDDEN,"WPA SUPPL: invalid RSN IE length - 4-way handshake failed\n");	  
		  return EAPOL_KEY_FATAL_FAIL;
	  }

	  ret = wpa_aes_Decrypt(decryptedKeyData, decryptDataLen, eapolKeyDesc->data,
					  (u8 *)&ptk->eapol_key_encryption_key, EAPOL_KEY_ENCRYPTION_KEY_SIZE,DefaultIV());

	  if (ret == FALSE) 
	  {
		  DEBUG_PRINT_m(SEV_INFO, MOD_WPA_HIDDEN,"WPA SUPPL:Failed to decrypt key data - 4-way handshake failed\n");	  
		  return EAPOL_KEY_FATAL_FAIL;
	  }
	 
	  if ( memcmp(wpaPort->ap_rsn_info_elem, decryptedKeyData, wpa_info_elem_get_len(wpaPort->ap_rsn_info_elem)) != 0 )
	  {
	  	  DEBUG_PRINT_m(SEV_INFO, MOD_WPA_HIDDEN,"WPA SUPPL: invalid RSN IE - 4-way handshake failed\n");	  
		  return EAPOL_KEY_FATAL_FAIL;
	  }
	 

	return EAPOL_KEY_OK;
}

/******************************************************************************
*                      supplicCreateEapolKey()							  *
*******************************************************************************
DESCRIPTION:    Creates EAPOL-key message for supplicant (2 or 4)
                                                                         
INPUTS:			PTK
				keyInfoBits - key info bits
				keyData - a pointer to key data

OUTPUT:			None

RETURNS:		Created EAPOL packet                                              
******************************************************************************/

static eapol_pkt_t *supplicCreateEapolKey( wpa_ptk_t *ptk, u32 keyInfoBits,
										   u8 *keyData, u32 keyDataLen, wpa_port_t *port)
{
	/* these definitions are required for using existing authenticator function*/
	wpa_assoc_t assoc; 
	
	
	/* initialize required parameters*/
    assoc.port =  port;
	memcpy (assoc.sta_mac, authMAC, sizeof(ether_mac_t));
	assoc.port = port;
	assoc.sta_type = WPA_STA_TYPE_WPA2;
	memcpy(&assoc.ptk, ptk, sizeof(assoc.ptk));

	/* function that creates eapol packet was written initially for authenticator, so it
	increments the given replay counter. But the supplicant should include the same replay
	counter as in previous packet, so we put replay counter - 1 and it will remain unchanged*/
	assoc.ap_to_sta_replay_count = handshakePars.replayCount - 1;

	return create_eapol_key_pkt(&assoc, keyInfoBits,

								0/*key index*/, 0/*rsc*/,
								0/*key length*/, &handshakePars.snonce,
								0 /*key iv*/, keyData,
								keyDataLen);

}


/******************************************************************************
*                      supplicSendEapolKey()							  *
*******************************************************************************
DESCRIPTION:    Sends EAPOL-key 
                                                                         
INPUTS:			eapolPkt - eapol packet pointer

OUTPUT:			None

RETURNS:		None                                              
******************************************************************************/
static void supplicSendEapolKey(eapol_pkt_t *eapolPkt)
{
	u32 pktLen = sizeof(eapol_pkt_t) - 1 + ntohs(eapolPkt->pkt_body_len);
    u32 ret;
	
	ret = sendEapolCB(NULL/*ignored*/, eapolPkt, pktLen,
											   FALSE/*unencrypted*/, htons(0x888e)/*ignored*/);
	if (ret != pktLen) {
		DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_HIDDEN,"WPA supplicant: failed to send EAPOL-key\n");
	}
   
}	


/******************************************************************************
*                      wpas_HandleEapol()							  *
*******************************************************************************
DESCRIPTION:    Handle EAPOL packet received
                                                                         
INPUTS:			eapol - a pointer to eapol packet
				eapolDataLen - eapol packet length

OUTPUT:			None

RETURNS:		OK/NOK                                         
******************************************************************************/
u32 wpas_HandleEapol(eapol_pkt_t *eapol, u32 eapolDataLen, wpa_port_t *wpaPort)
{
	if (!eapol) {
		 DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_HIDDEN,"WPA SUPPL: NULL EAPOL packet received\n");
		 return NOK;
	}
	
	if (eapolDataLen < sizeof(eapol_pkt_t)-1 || 
			eapolDataLen < sizeof(eapol_pkt_t)-1+ntohs(eapol->pkt_body_len))
	{
			DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_HIDDEN,"WPA SUPPL: Incorrect EAPoL packet length (%d)",eapolDataLen );
			return NOK;
	}
		
	if (eapol->proto_ver != EAPOL_VER)
	{
		DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_HIDDEN,"WPA SUPPL: Incorrect EAPoL packet version (%d)", eapol->proto_ver );
		return NOK;
	}
	
	if (eapol->pkt_type != ET_KEY)
	{
		DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_HIDDEN,"WPA SUPPL: non-EAPOL-key packet received\n" );
		return NOK;
	}
	
	wpas_SupplGetEvent(WPAS_EAPOL, eapol, wpaPort);

	return OK;
}


