/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998,1999,2000 Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*																			*/
/*	  MODULE:	powerSave.c	                                                */ 
/*   PURPOSE:	Power Save module implementation.	 					    */ 
/*																			*/
/***************************************************************************/
#include "wlan_pal.h"
#include "envList.h"
#include "whal.h"
#include "QoS/QoSClassifier.h"
#include "drvDbMain.h"
#include "drvDbHash.h"
#include "tx.h"

#define GetAssocId(x) (x+1)


#define MAX_PS_ENV_PER_STA                      32
#define PS_AGING_TIMEOUT                         6   /* 1 MIN (uints=SEC*10) */
#define WME_AC_ENV_LIST_SIZE				   	16

typedef struct {
   UINT32     sentPkts[WDRV_QOS_MAX_AC];
   EnvList_T *envListPtr[WDRV_QOS_MAX_AC];   /* Evelop list pointers.           */
   UINT16	  currNumberOfEnvInAllLists;
   UINT8      psStatus;     /* Power Save Status (True,False) */
   UINT8      prevTxFail;   /* The previos Tx fail )SRAM Overflow */
} PsEntry_T;

static PsEntry_T *psTable; /* The main Power Save table. */
UINT8 PsPrevTxFailCounter = 0;

static UINT32 agingTimeStamp;   /* The PS agingTimeStamp counter. */
static UINT32 numOfStaInPsMode;
static UINT32 numOfStaInWonePsMode[MAX_WONE_NET];
/* Definition for Bitmap control. */
#define MAX_VIRT_BITMAP_LEN 251
#define PART_VIRT_BITMAP_LEN 9	/* enough for 64 stations */
static UINT8 partVirtBitmap[MAX_VIRT_BITMAP_LEN]; 

UINT32 psPacketsExpired = 0;
UINT32 psPacketsMissed  = 0;
UINT32 psPacketsCleaned = 0;

static BOOL firstTimeAfterRecoveryFlag = FALSE;


/*************************************************************************
 *                        wdrv_psInit                                    *
 *************************************************************************
DESCRIPTION: This function initializes the Power save module.                 
                                                                                                   
INPUT:      numOfStations - Number of supported station.

OUTPUT:      

RETURN:      OK : Initiation succesfull.
             NOK: Initiation unsuccesfull.                                                                                                                                             
************************************************************************/
STATUS wdrv_psInit(UINT16 numOfStations )
{ 
   int index,wone_index,i;
   wdrv_QoSACParams_t wmeACParams;

   /* Init the main table. */
   if((psTable=(PsEntry_T*)PAL_memAlloc( 0,sizeof(PsEntry_T)*numOfStations,0,0)) == NULL)
   {
      wdrv_utilsReport(RL_TX, PRI_EMERGENCY,
       		"PS: Not enough memory to allocate psTable.");
      return NOK;   
   }

   wmeACParams = wdrv_dbGetWMEParams ();

   /* For each STA, init the envelop list, and set ps status to false. */     
   for( index=0; index < numOfStations; index++ )
   {
	   if(wmeACParams.enabled)
	   {
		   for (i = 0; i < WDRV_QOS_MAX_AC; ++i)
		   {
			   if( (psTable[index].envListPtr[i] = wdrv_txNewEnvList(WME_AC_ENV_LIST_SIZE))==NULL)
			   {
				   wdrv_utilsReport(RL_TX, PRI_EMERGENCY,
									"TX: Not enough memory to PS EnvList");
				   return NOK;
			   }
			   psTable[index].sentPkts[i] = 0;

		   }
	   }
	   else
	   {
		   if( (psTable[index].envListPtr[0] = wdrv_txNewEnvList(MAX_PS_ENV_PER_STA))==NULL)
		   {
			   wdrv_utilsReport(RL_TX, PRI_EMERGENCY,
								"TX: Not enough memory to PS EnvList");
			   return NOK;
		   }
		   psTable[index].sentPkts[0] = 0;

		   psTable[index].envListPtr[1] = NULL;
		   psTable[index].envListPtr[2] = NULL;
		   psTable[index].envListPtr[3] = NULL;

	   }

	   psTable[index].psStatus = FALSE;
	   psTable[index].prevTxFail = FALSE;
	   psTable[index].currNumberOfEnvInAllLists = 0;
   }

   PAL_memSet( partVirtBitmap, 0 , MAX_VIRT_BITMAP_LEN );

   numOfStaInPsMode = 0;
    for (wone_index = 0 ;wone_index < MAX_WONE_NET; wone_index++)
        numOfStaInWonePsMode[wone_index] = 0;
   agingTimeStamp = 0;

   return OK;
}

/*************************************************************************
 *                        wdrv_psUnload                                    *
 *************************************************************************
DESCRIPTION: This function deinitializes the Power save module.                 
                                                                                                   
INPUT:      numOfStations - Number of station in the system 

OUTPUT:      

RETURN:                                                                                                                                                   
************************************************************************/
void wdrv_psUnload(UINT16 numOfStations)
{
	int i,j;
	TxEnvelop_T *envelop;

	if(psTable)
	{
		for(i=0; i<numOfStations; i++)
		{
			for (j = 0; j < WDRV_QOS_MAX_AC; ++j)
			{
				if(psTable[i].envListPtr[j])
				{
					while(TRUE)
					{
						if(wdrv_txEnvListGetFirst(psTable[i].envListPtr[j], &envelop) == OK)
							PAL_memFree(0,(void *)envelop,0);
						else
							break;
					}
					
					wdrv_txFreeEnvList(psTable[i].envListPtr[j]);
				}
			}
		}
	
		PAL_memFree(0,(void *)psTable,0);
	}
}

/*************************************************************************
 *                        wdrv_psSetPsStatus                             *
 *************************************************************************
DESCRIPTION: This function called to report to the power save module of
			 changing ps status of specific STA.                 
                                                                                                   
INPUT:      staIndex - The station index.
			woneIndex - The station wone index.
			status - the new STA's power save status.

OUTPUT:      

RETURN:  TRUE  - PS in ON .  
         FALSE - PS is OFF.                                                                                                                                              
************************************************************************/
inline void wdrv_psSetPsStatus( UINT16 staIndex, UINT8 woneIndex, UINT16 status )
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
   
   /* 
   * in case there a change in the station Power Save status -> wake up the Tx task 
   * else -> do nothing
   */
   if( psTable[staIndex].psStatus != status ) {
       PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
       wdrv_txSendPsChangeStateMsg( staIndex, status, woneIndex ) ;
   }
   else
       PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}
    
      
/*************************************************************************
 *                        wdrv_psPowerSaveStatusChangeFunc                *
 *************************************************************************
DESCRIPTION: This function called to update power sae status of specific STA.
                                                                                                   
INPUT:      staIndex - The station index.
		    status - The new status.
			woneIndex - The station wone index.

OUTPUT:      

RETURN:                                                                                                                                                
************************************************************************/
void wdrv_psPowerSaveStatusChangeFunc(UINT32 staIndex, UINT16 status, UINT8 woneIndex)
{

   if( psTable[staIndex].psStatus == status )
      return;

   psTable[staIndex].psStatus = status;

   if( status == TRUE){
      numOfStaInPsMode++;
      numOfStaInWonePsMode[woneIndex]++;
      if( numOfStaInPsMode == 1 ){
          if (REPORT_LEVEL(RL_TX))
               wdrv_utilsReport(RL_TX,PRI_INFORMATION,"PS: Set Power save mode to True\n");
          whal_apiSetBssPs( TRUE );
      }

      /* If this station was flaged as TxFail (last out of PS couldn't send all the packets),
         then we need to reset this flag, so the packets will not be transmitted to the open air */
      if(psTable[staIndex].prevTxFail == TRUE) {
          PsPrevTxFailCounter--;
          psTable[staIndex].prevTxFail = FALSE;
      }
   }

   else{ /* status == FALSE */
      /* The station has become active send PS-wake to Tx. */
      numOfStaInPsMode--;
      numOfStaInWonePsMode[woneIndex]--;
      if( numOfStaInPsMode == 0 ){
         if (REPORT_LEVEL(RL_TX))
            wdrv_utilsReport(RL_TX,PRI_INFORMATION,"PS: Set Power save mode to False\n");
         whal_apiSetBssPs( FALSE );
      }
   }
}

/*************************************************************************
 *                        wdrv_psGetStaStatus                            *
 *************************************************************************
DESCRIPTION: This function returns the power save status of the station.                
                                                                                                   
INPUT:      staIndex - The station index.

OUTPUT:      

RETURN: TRUE  - PS in ON .  
        FALSE - PS is OFF.                                                                                                                                               
************************************************************************/
UINT16 wdrv_psGetStaStatus( UINT16 stationIndex )
{
	unsigned int flags;
	UINT8 status;
	
    if((stationIndex == INVALID_INDEX) || (stationIndex >= MAX_STATIONS))
        return FALSE;
    else
	{
		PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); 
		status = psTable[ stationIndex ].psStatus;
		PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags); 
	}

	return status;
}

/*************************************************************************
 *                        wdrv_updateRecoveryFlag                        *
 *************************************************************************
DESCRIPTION: This function updates the 'firstTimeAfterRecoveryFlag' flag. 
             This flag is used to determine if to update the virtual bitmap
             when receiving a packet to put in the queue.
                                                                                                   
INPUT:                  

OUTPUT:      

RETURN:                                                                                                                                                     
************************************************************************/
void wdrv_updateRecoveryFlag(void)
{
    firstTimeAfterRecoveryFlag = TRUE;
}

/*************************************************************************
 *                        wdrv_psStoreEnv                                *
 *************************************************************************
DESCRIPTION: This function saves the envelop in the corresponding PS envelop list.                
                                                                                                   
INPUT:      staIndex - The station index.
            envelopPtr - Pointer to the envelop to save.            

OUTPUT:      

RETURN:      OK : The envelop has been inserted, there was enough place in 
                  the list.
             NOK: The list was full, the first envelop is dropt, and returned
                  by *envelopPtr.                                                                                                                                                
************************************************************************/
STATUS wdrv_psStoreEnv( UINT16 staIndex, TxEnvelop_T **envelopPtr, wdrv_QoSClassifierAC_t ac )
{
   STATUS retVal;
   unsigned int flags;
   BOOL wasQueueFull = FALSE;

   (*envelopPtr)->expirTick = agingTimeStamp + PS_AGING_TIMEOUT;
   if((*envelopPtr)->txFlags & WDRV_TX_EAPOL_FRAME)
	   retVal = wdrv_txEnvListInsFirst( psTable[staIndex].envListPtr[ac], envelopPtr, &wasQueueFull);
   else
	   retVal= wdrv_txEnvListIns( psTable[staIndex].envListPtr[ac], envelopPtr);

   if( (retVal==OK) && (wasQueueFull == FALSE) )
       psTable[staIndex].currNumberOfEnvInAllLists++;

   /*Note: There is a need to update the virtual bitmap even   */
   /*      if it's not the frist packet to put in the queue.   */
   /*      That is if this is the first packet after recovery, */
   /*      becuase the Beacon is not updated even if there     */
   /*      are already packets in the queue.                   */
   PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); 
   if(( psTable[staIndex].psStatus == TRUE)&&
      ( ( psTable[staIndex].currNumberOfEnvInAllLists == 1 ) || (firstTimeAfterRecoveryFlag) ))
   {
       firstTimeAfterRecoveryFlag = FALSE;
	   PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags); 
       
	   updatePartVirtBitmap(staIndex, TRUE );
   }
   else
	   PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags); 
      
   return retVal;
}


/*************************************************************************
 *                        wdrv_psWatchEnv                                *
 *************************************************************************
DESCRIPTION: This function returns an indication whether there is at least
			 one envelope in the list. In case it is, the first envelope
			 pointer is returned in the envelopPtr.
                                                                                                   
INPUT:       staIndex - The station index.
                        

OUTPUT:      envelopPtr - Pointer to the first envelop in the list.

RETURN:      OK : There was an envelop in the list, and it is assigned 
                  to the *envelopPtr.
             NOK: The list was empty, *envelopPtr is trush.                                                                                                                                                 
************************************************************************/
STATUS wdrv_psWatchEnv( UINT16 staIndex, TxEnvelop_T **envelopPtr, wdrv_QoSClassifierAC_t ac)
{
   if(psTable[staIndex].envListPtr[ac])
	   return( wdrv_txEnvListWatchFirst( psTable[staIndex].envListPtr[ac], 
                                      envelopPtr));
   else
	   return NOK;
}

/*************************************************************************
 *                        wdrv_psGetEnv                                  *
 *************************************************************************
DESCRIPTION: This function retrieves envelope from the list.               
                                                                                                   
INPUT:       staIndex - The station index.
                        

OUTPUT:      envelopPtr - Pointer to the retrievde envelope. 

RETURN:      OK : There was an envelop in the list, and it is assigned 
                  to the *envelopPtr.
             NOK: The list was empty, *envelopPtr is trush.                                                                                                                                                 
************************************************************************/
STATUS wdrv_psGetEnv( UINT16 staIndex, TxEnvelop_T **envelopPtr, wdrv_QoSClassifierAC_t ac)
{
    STATUS retVal;
    unsigned int flags;

	if(psTable[staIndex].envListPtr[ac])
	{
		retVal = wdrv_txEnvListGetFirst( psTable[staIndex].envListPtr[ac], 
                                     envelopPtr);
	}
	else
	{
		*envelopPtr = NULL;
		return NOK;
	}

    if(retVal == OK)
	{

		PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); 
		psTable[staIndex].sentPkts[ac]++;
		psTable[staIndex].currNumberOfEnvInAllLists--;

		if(psTable[staIndex].prevTxFail == TRUE)
		{
		   PsPrevTxFailCounter --;
		   psTable[staIndex].prevTxFail = FALSE;
		}

		if(psTable[staIndex].currNumberOfEnvInAllLists == 0)
		{
			PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags); 
			updatePartVirtBitmap(staIndex, FALSE );
		}
		else
			PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags); 
	}

    return retVal;
}
/****************************************************************************
 *                      wdrv_psResetStaCounters		                        *
 ****************************************************************************
 * DESCRIPTION: reset counters for a specific station
 *
 * INPUTS:
 *
 * OUTPUT: 	       
 *		
 * RETURNS:
 ****************************************************************************/

void wdrv_psResetStaCounters( UINT16 staIndex)
{
	int i;

	for (i = 0; i < WDRV_QOS_MAX_AC; ++i)
	{
        psTable[staIndex].sentPkts[i] = 0;
        if (psTable[staIndex].envListPtr[i])
			psTable[staIndex].envListPtr[i]->maxStoredEnv = 0;
	}

	psTable[staIndex].currNumberOfEnvInAllLists = 0;
}


/*************************************************************************
 *                        wdrv_psPsOnAndPksInQueue                       *
 *************************************************************************
DESCRIPTION: This function returns TRUE if the station is in PS mode and
             there are more packets in the PS queue.                 
                                                                                                   
INPUT:       staIndex - The station index.

OUTPUT:      

RETURN: TRUE  - Station in PS and more packets to send .  
        FALSE - Otherwise.                                                                                                                                               
************************************************************************/
UINT16 wdrv_psPsOnAndPksInQueue( UINT16 stationIndex )
{
   unsigned int flags;

   PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); 
   if(( psTable[ stationIndex ].psStatus == TRUE ) &&
      ( psTable[ stationIndex ].currNumberOfEnvInAllLists != 0))
   {
	  PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags); 
      return TRUE;
   }
   else 
   {
	  PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags); 
	  return FALSE;
   }
}


/*************************************************************************
 *                        wdrv_psGetTotalStaInPSMode                     *
 *************************************************************************
DESCRIPTION: Returns the number of station currently in PS mode.

INPUT :
OUTPUT:      

RETURN: The number of packets in PS mode.                                                                                                                                               
************************************************************************/
UINT32  wdrv_psGetTotalStaInPSMode( void )
{
   return numOfStaInPsMode;
}


/*************************************************************************
 *                        wdrv_psGetTotalWoneStaInPSMode                     *
 *************************************************************************
DESCRIPTION: Returns the number of station currently in PS mode for wone.

INPUT : wone_Index : index of wone on which the number of STAs in PS requested.
OUTPUT:      

RETURN: The number of packets in PS mode for wone.                                                                                                                                               
************************************************************************/
UINT32  wdrv_psGetTotalWoneStaInPSMode( UINT8 wone_Index )
{
    return numOfStaInWonePsMode[wone_Index];
    
}

/*************************************************************************
 *                        updatePartVirtBitmap                           *
 *************************************************************************
DESCRIPTION: This function updates partial virtual bitmap according to the
			 current station PS status.

INPUT:		 staIndex - The station index.
			 status - The station status.
OUTPUT:      

RETURN: OK                                                                                                                                               
************************************************************************/
STATUS updatePartVirtBitmap(UINT16 staIndex, UINT16 status )
{
   UINT8 bitMapControl;
   UINT8 length=0;
   UINT8 offset=0;
   UINT16 assocId;
   UINT8  woneIndex;
   unsigned int flags;

   assocId =  GetAssocId(staIndex);

   wdrv_dbGetStationWoneIndex(staIndex, &woneIndex);
   if(woneIndex >= MAX_WONE_NET)
   {
	   printf("updatePartVirtBitmap: staIndex=%d, woneIndex=%d\n",staIndex,woneIndex);
	   return NOK; 
   }

   if (REPORT_LEVEL(RL_TX))
      printf("PS : Virtual BitMap change :: assocId=%d  status=%d\n",assocId,status);

	

   PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
   /*---------------------------------------------------*/
   /*   Set the Bitmap                                  */
   /*---------------------------------------------------*/
   if(  status == TRUE ){
      
       if( (partVirtBitmap[assocId/8] & (1<<(assocId % 8))) == TRUE) {
         /* Bit is already set, don't do anything.  */
         PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
         return OK;
       }
      else {
         /* Bit=0, need to set it to one.. */ 
         partVirtBitmap[assocId/8] |= 1<<(assocId % 8);
	  }
   }


   else{ /* STATUS == FALSE */
      
       if((partVirtBitmap[assocId/8] & (1<<(assocId % 8))) == FALSE) {
         /* Bit is already set to FALSE, don't do anything.  */
         PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
         return OK;
       }      
      else { 
         /* Bit=1, need to turn it off. */ 
         partVirtBitmap[assocId/8] &= ~(1<<(assocId%8));
	  }
   }

   /*---------------------------------------------------*/
   /*   Set the length and the offset fields            */
   /*---------------------------------------------------*/
   offset = 0; /* offset is always 0: don't support partial bitmap */
   if ( numOfStaInPsMode )
	   length = PART_VIRT_BITMAP_LEN;
   else
	   length = 1; /* send only minimal number of bytes */

   bitMapControl = offset << 1; /* reserve first bit for Traffic Indicator bit */

   PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */

   whal_apiSetTim(woneIndex, bitMapControl, partVirtBitmap , length);

   return OK;
}


/*************************************************************************
 *                        wdrv_txPowerSaveAgingFunc                      *
 *************************************************************************
DESCRIPTION: Power Save aging management function.

OUTPUT:      

RETURN:                                                                                                                                               
************************************************************************/
void wdrv_txPowerSaveAgingFunc( void )
{
   TxEnvelop_T  *envelopPtr;
   EnvList_T    *staEnvListPtr;
   UINT16        staTreated = 0;
   UINT16        stationIndex = 0;
   UINT16		acId = 0;
   unsigned int flags;

   agingTimeStamp++;
   PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags);
   while( (staTreated < numOfStaInPsMode) &&  (stationIndex < MAX_STATIONS))
   {
      /* Looping through all the stations in PS mode */
      if( psTable[ stationIndex ].psStatus == TRUE )
	  {
		  for(acId = 0 ; acId < WDRV_QOS_MAX_AC ; acId++)
		  {

			 staEnvListPtr = psTable[stationIndex].envListPtr[acId];
			 if(staEnvListPtr)
			 {
				 while( wdrv_txEnvListWatchFirst( staEnvListPtr, &envelopPtr) == OK )
				 { 
					/* For any STA in PS we loop through the packets  */ 
							 /* in its envelop list.                           */    
					 if( envelopPtr->expirTick < agingTimeStamp )
					 {
						 psPacketsExpired++;
						/* Time to expire the packet !!! */
						if(REPORT_LEVEL(RL_TX))
							 wdrv_utilsReport(RL_DB, PRI_DEBUG, "PS: envelop expired !");
						wdrv_txEnvListGetFirst( staEnvListPtr, &envelopPtr );
						wlan_memMngrFreeMSDU( (envelopPtr)->msduPtr->handle  );
						wdrv_txFreeEnv( &envelopPtr );
						psTable[stationIndex].currNumberOfEnvInAllLists--;
						if(psTable[stationIndex].currNumberOfEnvInAllLists == 0)
							updatePartVirtBitmap(stationIndex, FALSE ); 

					 }
					 else 
					 { /* There is a packet but it is not expired yet -> exit this STA */
						 break;
					 }
				 }
			  }  
		  } /* For AC */
          staTreated++;
      } /* Staion is PS mode */
	  else
	  {	 
		  for(acId = 0 ; acId < WDRV_QOS_MAX_AC ; acId++)
		  {
			 staEnvListPtr = psTable[stationIndex].envListPtr[acId];
			 if(staEnvListPtr)
				if(staEnvListPtr->CurrNumOfEnv > 0)
					psPacketsMissed++;
		  }	
	  }
      stationIndex++;
   } /* While */
   PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);
}


/*************************************************************************
 *                        wdrv_powerSavePrintDb							 *
 *************************************************************************
DESCRIPTION: Returns PS module statistics

INPUT : buf - buffer to be filled with the statistics
OUTPUT:      

RETURN: The length of data in the buffer.                                                                                                                                              
************************************************************************/
int wdrv_powerSavePrintDb( char *buf )
{
   int stationIndex,acId,len = 0;

   len += sprintf(buf +len,"-------------------------- Power Save -------------------------\n");
   len += sprintf(buf +len, "First 8 bytes of Virtual Bitmap: %x %x %x %x %x %x %x %x\n", 
                    partVirtBitmap[0],partVirtBitmap[1],partVirtBitmap[2],partVirtBitmap[3],
                    partVirtBitmap[4],partVirtBitmap[5],partVirtBitmap[6],partVirtBitmap[7]);

   for( stationIndex=0; stationIndex < MAX_STATIONS; stationIndex++ )
   {
	   if( psTable[ stationIndex ].psStatus == TRUE ) 
	   {
		   len += sprintf(buf +len,"\nStaInd    %6d    \n", stationIndex);
		   len += sprintf(buf +len,"----------------- \n"); 
		   len += sprintf(buf +len,"currNumberOfEnvInAllLists = %d \n",psTable[ stationIndex ].currNumberOfEnvInAllLists); 
		   
		   len += sprintf(buf +len,"   AC       StoredPkst      SentPkts    MaxStored       \n");
		   len += sprintf(buf +len,"-------    ------------    -----------  ---------        \n"); 
		   for(acId = 0 ; acId < 4 ; acId++)
		   {
			   if(psTable[ stationIndex ].envListPtr[acId])
			   {
				   len += sprintf(buf +len,"%5d    %9d     %8d      %5d\n",acId , 
								  psTable[ stationIndex ].envListPtr[acId]->CurrNumOfEnv, 
								  psTable[ stationIndex ].sentPkts[acId],
								  psTable[ stationIndex ].envListPtr[acId]->maxStoredEnv);   
			   }
		   }
	   }
   }
   len += sprintf(buf +len,"-------------------------------------------------------------\n");
	
   return len;
}


/****************************************************************************
 *                      wdrv_powerSaveCleanStaList()                       *
 ****************************************************************************
 * DESCRIPTION: Clean all the packets stored in a station power save list 
 *              and set the PS status to FALSE, used for disconnecting a 
 *              staion from the AP.

 * INPUTS: stationIndex - The station index
 *		   woneIdx - The station wone index
 * 
 * OUTPUT:
 * 
 * RETURNS:
 ****************************************************************************/
void wdrv_powerSaveCleanStaList( UINT16 staIndex, UINT8 woneIdx )
{
  wdrv_txSendPsCleanStaListMsg( staIndex, woneIdx ) ;
}

/****************************************************************************
 *                      wdrv_powerSaveCleanStaListFunc()                       *
 ****************************************************************************
 * DESCRIPTION: Clean all the packets stored in a station power save list 
 *              and set the PS status to FALSE, used for disconnecting a 
 *              staion from the AP.

 * INPUTS: stationIndex - The station Index
 *		
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
void wdrv_powerSaveCleanStaListFunc( UINT16 stationIndex )
{
   TxEnvelop_T *envelopPtr;
   UINT8 acId;
 
   for(acId = 0 ; acId < WDRV_QOS_MAX_AC ; acId++)
   {
	   while( wdrv_psGetEnv( stationIndex, &envelopPtr, acId) == OK )
	   {
		  wlan_memMngrFreeMSDU( (envelopPtr)->msduPtr->handle  );
		  wdrv_txFreeEnv( &envelopPtr );
		  psPacketsCleaned++;
	   }
   }
   
   wdrv_psResetStaCounters(stationIndex);

   return;
}


/****************************************************************************
 *                      wdrv_psGetPsPrevTxFail()							*
 ****************************************************************************
 * DESCRIPTION: Return the status of previous Ps unicast packet send 
 *
 * INPUTS: stationIndex - The station Index
 *		      
 *		
 * 
 * OUTPUT:	prevTxFail - Indecate if this station failed to send previous 
 *          unicast packet
 * 
 * RETURNS:
 ****************************************************************************/
void  wdrv_psGetPsPrevTxFail( UINT8 stationIndex ,UINT8 *prevTxFail)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); 
    *prevTxFail = psTable[stationIndex].prevTxFail;
    PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags); 
}

/****************************************************************************
 *                      wdrv_psSetPsPrevTxFail()							*
 ****************************************************************************
 * DESCRIPTION: Set the PS Prev send fail flag.
 *
 * INPUTS: stationIndex - The station Index
 *		      
 *		
 * RETURNS:
 ****************************************************************************/
void  wdrv_psSetPsPrevTxFail( UINT8 stationIndex)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); 
    if(psTable[stationIndex].prevTxFail == FALSE)
		PsPrevTxFailCounter ++;
    psTable[stationIndex].prevTxFail = TRUE;
	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags); 
}

/****************************************************************************
 *                      wdrv_psGetPsPrevTxFailCounter()                       *
 ****************************************************************************
 * DESCRIPTION: return the number of station with PS Prev send fail flag on.
 * 
 * INPUTS:
 *
 * OUTPUT: 	PsTxFailurs - Number of station with PS Prev send fail flag on	      
 *		
 * RETURNS:
 ****************************************************************************/
inline void  wdrv_psGetPsPrevTxFailCounter(UINT8 *PsTxFailurs)
{
   *PsTxFailurs = PsPrevTxFailCounter;
}

/****************************************************************************
 *                      wdrv_psGetEnvList			                        *
 ****************************************************************************
 * DESCRIPTION: return a specific PsEnvList for a specific ac and station
 *
 * INPUTS:
 *
 * OUTPUT: 	       
 *		
 * RETURNS:
 ****************************************************************************/
STATUS wdrv_psGetEnvList( UINT16 staIndex, EnvList_T **envelopListPtr, wdrv_QoSClassifierAC_t ac)
{
	*envelopListPtr = psTable[staIndex].envListPtr[ac];

	if ( (psTable[staIndex].envListPtr[ac] == NULL) ||
		 (psTable[staIndex].envListPtr[ac]->CurrNumOfEnv == 0) )
        return NOK;

	return OK;

}





