/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2003 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/****************************************************************************
 *
 *   MODULE:  whalAcxCmd.c
 *   PURPOSE: ACX100 commands handler
 *      The object support the followinf ACX100 commands:
 *          StartBss
 *          EnableRx
 *          EnableTx
 *          InitMemory
 *          ProbeRespTemplate
 *          BeaconTemplate
 * 
 ****************************************************************************/

#include "wlan_pal.h"
#include "whalCommon.h"
#include "AcxCommands.h"
#include "whalAcxMbox.h"
#include "whalParams.h"
#include "whalFrameBuilder.h"
#include "whalAcxCmd.h"
#include "whalTrace.h"
#include "AcxDataIf.h"
#include "AcxCommands.h"
#include "public_security_keys.h"
#include "whalAcxRegs.h"
#include "QoS/QoSClassifier.h"
#include "drvDbMain.h"
#include "public_dcs.h"
#include "mgmtSsidHandler.h" 

/****************************************************************************
 *                      whal_acxCmdStartBss()
 ****************************************************************************
 * DESCRIPTION: Construct the StartBss command fileds and send it to the mailbox
 * 
 * INPUTS: 
 *          UINT8   WoneIndex   Index of wOne Network.
 * 
 * OUTPUT:  None
 * 
 * RETURNS: OK or NOK
 ****************************************************************************/
int whal_acxCmdStartBss(UINT8 WoneIndex)
{
    AcxCmd_StartBss_T AcxCmd_StartBss[MAX_WONE_NET];
    AcxCmd_StartBss_T *pCmd = &AcxCmd_StartBss[WoneIndex];
    dot11_SSID_T *pWlanElm_Ssid = whal_ParamsGetElm_Ssid();
    GenParams_T *pGenParams = whal_ParamsGetGenParams(WoneIndex);
    UINT8 *BssId;
    int i;

    pCmd->BeaconInterval = ENDIAN_HANDLE_WORD(pGenParams->BeaconInterval);
    pCmd->DtimInterval = pGenParams->DtimInterval; 
    pCmd->BSS_Type   = 0;
    pCmd->BSS_Type  |=   pGenParams->BssType & BSS_TYPE_MASK; /* AP */
    pCmd->BSS_Type  |=  (pGenParams->Band <<  BSS_TYPE_BAND_OFFSET)
                                            & BSS_TYPE_BAND_MASK;
    pCmd->BSS_Type  |=  (WoneIndex        <<  BSS_TYPE_WONE_IDX_OFFSET)
                                            & BSS_TYPE_WONE_IDX_MASK;

    /*
     * BasicRateSet
     * The ACX100 uses this field to determine the rate at which to transmit 
     * control frame responses (such as ACK or CTS frames)
     */
    pCmd->BasicRateSet = pGenParams->BasicRateSet;
    pCmd->BasicRateSet = ENDIAN_HANDLE_WORD(pCmd->BasicRateSet);
    pCmd->ChannelNumber = pGenParams->StartRadioChannel;

    /*
     * ACX100_Tx_Rate
     * This field indicates the rate at which the ACX100 transmits 
     * Beacon, Probe Response, RTS, and PS-Poll frames
     */
    pCmd->TxCtlFrameRate = pGenParams->ACX100_GenTxRate;
    pCmd->TxMgtFrameRate = pGenParams->ACX100_GenTxRate;
    /*
     * PreambleOpt
     * indicates the PLCP preamble type to use when transmitting 
     * Beacon, Probe Response, RTS, and PS-Poll frames.
     */
    pCmd->TxCtlFrameModulation = pGenParams->ACX100_GenTxPreample;
    pCmd->TxMgtFrameModulation = pGenParams->ACX100_GenTxPreample;

    pCmd->JoinCtrl = 0;
    /* BSS ID - reversed order (see ACX100 spec) */
    BssId = whal_ParamsGetBssId();
    for (i=0; i<MAC_ADDR_SIZE; i++)
        pCmd->BssId[i] = BssId[MAC_ADDR_SIZE-1-i];

    /* SSID string */ 
    pCmd->SSID_length = pWlanElm_Ssid->hdr.eleLen;
    PAL_memCopy((UINT8 *)(pCmd->SSID),(UINT8 *)pWlanElm_Ssid->serviceSetId, pWlanElm_Ssid->hdr.eleLen);

    /*
     * Send the command 
     */
#ifdef TIWLAN_DEBUG
    if (whal_trcIsEnable())
        whal_trcAddCtrl("CMD-", "StartBss --------", pGenParams->BeaconInterval);
#endif
    return (whal_acxMboxCommand(CMD_JOIN, (char *)pCmd, sizeof(*pCmd), INIT_TIMEOUT));
}

/****************************************************************************
 *                      whal_acxCmdReset()
 ****************************************************************************
 * DESCRIPTION: Reset the ACX100
 * 
 * INPUTS: None 
 * 
 * OUTPUT:  None
 * 
 * RETURNS: OK or NOK
 ****************************************************************************/
int whal_acxCmdReset()
{
    /*
     * no parameters 
     */
    
    /*
     * Send the command 
     */
    return (whal_acxMboxCommand(CMD_ACX_RESET, NULL, 0, NO_WAIT));
}

/****************************************************************************
 *                      whal_acxCmdSleep()
 ****************************************************************************
 * DESCRIPTION: Set SLEEP command on the ACX100
 * 
 * INPUTS: None 
 * 
 * OUTPUT:  None
 * 
 * RETURNS: OK or NOK
 ****************************************************************************/
int whal_acxCmdSleep()
{
    /*
     * no parameters 
     */
    
    /*
     * Send the command 
     */
    return (whal_acxMboxCommand(CMD_SLEEP, NULL, 0, NO_WAIT));
}

/****************************************************************************
 *                      whal_acxCmdWake()
 ****************************************************************************
 * DESCRIPTION: Set WAKE command on the ACX100
 * 
 * INPUTS: None 
 * 
 * OUTPUT:  None
 * 
 * RETURNS: OK or NOK
 ****************************************************************************/
int whal_acxCmdWake()
{
    /*
     * no parameters 
     */
    
    /*
     * Send the command 
     */
    return (whal_acxMboxCommand(CMD_WAKE, NULL, 0, NO_WAIT));
}

/****************************************************************************
 *                      whal_acxCmdEnableRx()
 ****************************************************************************
 * DESCRIPTION: Construct the EnableRx command fileds and send it to the mailbox
 * 
 * INPUTS: None 
 * 
 * OUTPUT:  None
 * 
 * RETURNS: OK or NOK
 ****************************************************************************/
int whal_acxCmdEnableRx()
{
    AcxCmd_EnableRx_T AcxCmd_EnableRx;
    AcxCmd_EnableRx_T *pCmd = &AcxCmd_EnableRx;

    /*
     * Set command fields 
     */
    PAL_memSet(pCmd, 0, sizeof(*pCmd));

    /* Same channel for Tx and Rx -> overide channel parameter in EnableTx command */
    /*pCmd->ChannelNumber = whal_ParamsGetRadioChannel(0);        */
    
    /*
     * Send the command 
     */
    return (whal_acxMboxCommand(CMD_ENABLE_RX, (char *)pCmd, sizeof(*pCmd), INIT_TIMEOUT));
}

/****************************************************************************
 *                      whal_acxCmdEnableTx()
 ****************************************************************************
 * DESCRIPTION: Construct the EnableTx command fileds and send it to the mailbox
 * 
 * INPUTS: None 
 * 
 * OUTPUT:  None
 * 
 * RETURNS: OK or NOK
 ****************************************************************************/
int whal_acxCmdEnableTx()
{
    AcxCmd_EnableTx_T AcxCmd_EnableTx;
    AcxCmd_EnableTx_T *pCmd = &AcxCmd_EnableTx;

    /*
     * Set command fields 
     */
    PAL_memSet(pCmd, 0, sizeof(*pCmd));

    /* Same channel for Tx and Rx -> overide channel parameter in EnableRx command */
    pCmd->ChannelNumber = whal_ParamsGetRadioChannel(0);

    /*
     * Send the command 
     */
    return (whal_acxMboxCommand(CMD_ENABLE_TX, (char *)pCmd, sizeof(*pCmd), INIT_TIMEOUT));
    return OK;
}

/****************************************************************************
 *                      whal_acxCmdDisableRx()
 ****************************************************************************
 * DESCRIPTION: Construct the DisableRx command fileds and send it to the mailbox
 * 
 * INPUTS: None 
 * 
 * OUTPUT:  None
 * 
 * RETURNS: OK or NOK
 ****************************************************************************/
int whal_acxCmdDisableRx()
{
    /*
     * no parameters 
     */
    
    /*
     * Send the command 
     */
    return (whal_acxMboxCommand(CMD_DISABLE_RX, NULL, 0, INIT_TIMEOUT));
}

/****************************************************************************
 *                      whal_acxCmdDisableTx()
 ****************************************************************************
 * DESCRIPTION: Construct the DisableTx command fileds and send it to the mailbox
 * 
 * INPUTS: None 
 * 
 * OUTPUT:  None
 * 
 * RETURNS: OK or NOK
 ****************************************************************************/
int whal_acxCmdDisableTx()
{
    /*
     * no parameters 
     */
    
    /*
     * Send the command 
     */
    return (whal_acxMboxCommand(CMD_DISABLE_TX, NULL, 0, INIT_TIMEOUT));
}

/****************************************************************************
 *                      whal_acxCmdRxReset()
 ****************************************************************************
 * DESCRIPTION: Reset the ACX100 Receiving path
 * 
 * INPUTS: None 
 * 
 * OUTPUT:  None
 * 
 * RETURNS: OK or NOK
 ****************************************************************************/
int whal_acxCmdRxReset(void)
{
    /*
     * no parameters 
     */
    
    /*
     * Send the command 
     */
    return (whal_acxMboxCommand(CMD_RX_RESET, NULL, 0, NO_WAIT));
}

/****************************************************************************
 *                      whal_acxCmdBeaconTemplate()
 ****************************************************************************
 * DESCRIPTION: Build the beacon template command fileds and send it to the mailbox
 * 
 * INPUTS: 
 *          UINT8   WoneIndex   Index of wOne Network.
 *
 * OUTPUT:  None
 * 
 * RETURNS: OK or NOK
 ****************************************************************************/
int whal_acxCmdBeaconTemplate(UINT8 WoneIndex)
{
    AcxCmd_ConfBeacon_T AcxCmd_ConfBeacon[MAX_WONE_NET];
    AcxCmd_ConfBeacon_T *pCmd = &AcxCmd_ConfBeacon[WoneIndex];
    int FrameSize;

    /*
     * Set configuration fields 
     */
    whal_frameBuildBeacon(WoneIndex, &pCmd->BeaconFrame, &FrameSize);
    pCmd->len = ENDIAN_HANDLE_WORD(FrameSize);
    /*
     * Send the command 
     */
    return (whal_acxMboxCommand(((AcxMboxCommand_T)WoneIndex << 8) | CMD_CONF_BEACON, (char *)pCmd, FrameSize+sizeof(pCmd->len), NO_WAIT));
}

/****************************************************************************
 *                      whal_acxCmdProbeRespTemplate()
 ****************************************************************************
 * DESCRIPTION: Build the probe response template command fileds and send it to the mailbox
 * 
 * INPUTS: None 
 * 
 * OUTPUT:  None
 * 
 * RETURNS: OK or NOK
 ****************************************************************************/
int whal_acxCmdProbeRespTemplate(UINT8 WoneIndex)
{
    AcxCmd_ConfProbeResp_T AcxCmd_ConfProbeResp[MAX_WONE_NET];
    AcxCmd_ConfProbeResp_T *pCmd = &AcxCmd_ConfProbeResp[WoneIndex];
    int FrameSize;
    UINT16 HiddenSSID;
    UINT16 MultipleSsidFlag;

    /*
     * Set configuration fields 
     */
        
    whal_frameBuildProbeResp(WoneIndex, 
                             &pCmd->ProbeRespFrame, 
                             &FrameSize, 
                             whal_ParamsGetBcastMac(),
                             whal_ParamsGetElm_Ssid());
    pCmd->len = ENDIAN_HANDLE_WORD(FrameSize);


    wdrv_mgmt_ssidGetHiddenSsidStatus(&HiddenSSID);
    wdrv_mgmt_ssidGetMultipleSsidStatus(&MultipleSsidFlag);
    if (HiddenSSID || MultipleSsidFlag)
    {
        /* Set the TX_FRM_CTL to stop replying to Probe Requests automatically */
        whal_apiMacWrite(TX_FRM_CTL, 0x00080000);   
    }

    /*
     * Send the command 
     */
     
    return (whal_acxMboxCommand(((AcxMboxCommand_T)WoneIndex << 8) | CMD_CONF_PROBE_RESP, (char *)pCmd, FrameSize+sizeof(pCmd->len), NO_WAIT));
}

/****************************************************************************
 *                      whal_acxCmdTimTemplate()
 ****************************************************************************
 * DESCRIPTION: Build the tim template fileds and send it to the mailbox
 * 
 * INPUTS: None 
 * 
 * OUTPUT:  None
 * 
 * RETURNS: OK or NOK
 ****************************************************************************/
int whal_acxCmdTimTemplate(UINT8 WoneIndex)
{
    AcxCmd_ConfTim_T AcxCmd_ConfTim[MAX_WONE_NET];
    AcxCmd_ConfTim_T *pCmd = &AcxCmd_ConfTim[WoneIndex];
    int FrameSize;

    /*
     * Set configuration fields 
     */
    whal_frameBuildBeaconFromTim(WoneIndex, pCmd->infoElements, &FrameSize);
    
    pCmd->len = ENDIAN_HANDLE_WORD(FrameSize);
    
    /*
     * Send the command 
     */

    return (whal_acxMboxCommand(((AcxMboxCommand_T)WoneIndex << 8) | CMD_CONF_TIM, (char *)pCmd, FrameSize+sizeof(pCmd->len), NO_WAIT));
}


/****************************************************************************
 *                      whal_acxCmdWoneToggleTemplate()
 ****************************************************************************
 * DESCRIPTION: Build toggle packet template and send it to the mailbox
 * 
 * INPUTS: None 
 * 
 * OUTPUT:  None
 * 
 * RETURNS: OK or NOK
 ****************************************************************************/
int whal_acxCmdWoneToggleTemplate(void)
{
    AcxCmd_ConfToggle_T toggle_cfg;
    UINT8       BSSID[6];
    
    toggle_cfg.toggle.fc = ENDIAN_HANDLE_WORD(DOT11_FC_CTS | DOT11_FC_FROM_DS);
    
    wdrv_dbGetBssid(BSSID);
    PAL_memCopy((void*)&toggle_cfg.toggle.address1, (void*)BSSID,6); /* BSSID */

    toggle_cfg.len = ENDIAN_HANDLE_WORD( sizeof(toggle_cfg.toggle.fc) +
                                         sizeof(toggle_cfg.toggle.duration) +
                                         sizeof(toggle_cfg.toggle.address1) );
    
    return (whal_acxMboxCommand(0x116, (char *)&toggle_cfg, toggle_cfg.len + sizeof(toggle_cfg.len), INIT_TIMEOUT));
}

/****************************************************************************
 *                      whal_acxCmdWepSetKey()
 ****************************************************************************
 * DESCRIPTION: Construct the WepMgmt command fileds and send it to the mailbox
 * 
 * INPUTS: Action - add/remove key
 *         MacAddr - relevant only for mapping keys
 *         EntryIndex - relevant only for TKIP
 *         KeySize - key size
 *         KeyType - default/mapping/TKIP
 *         KeyId - relevant only for default keys
 *         Key - key data
 *         AcSeqNum16 - security sequence num
 *         AcSeqNum32 - security sequence num
 * 
 * OUTPUT:  None
 * 
 * RETURNS: OK or NOK
 ****************************************************************************/
int whal_acxCmdSetKey(int Action, char *MacAddr, int ssidProfile, int KeySize, int KeyType, 
											int KeyId, char *Key, UINT16 AcSeqNum16[WDRV_QOS_MAX_AC], UINT32 AcSeqNum32[WDRV_QOS_MAX_AC])
{
    AcxCmd_SetKey_T AcxCmd_SetKey;
    AcxCmd_SetKey_T *pCmd = &AcxCmd_SetKey;
		int i;

    PAL_memSet(pCmd, 0, sizeof(*pCmd));

    /* Fill the command fields */
    PAL_memCopy(pCmd->MacAddr, MacAddr, MAC_ADDR_SIZE);
    PAL_memCopy(pCmd->Key, Key, KeySize);
    pCmd->Action = ENDIAN_HANDLE_WORD((UINT16)Action);
    pCmd->KeySize = (UINT8)KeySize;
    pCmd->KeyType = (UINT8)KeyType;
    pCmd->KeyId = (UINT8)KeyId;
    pCmd->ssidProfile = (UINT8)ssidProfile;
	
	for (i=0; i<WDRV_QOS_MAX_AC; ++i) {
		pCmd->AcSeqNum16[i] = ENDIAN_HANDLE_WORD(AcSeqNum16[i]);
		pCmd->AcSeqNum32[i] = ENDIAN_HANDLE_LONG(AcSeqNum32[i]);
	}

    if(REPORT_LEVEL(RL_HAL)) 
    {
        /* debug for tkip keys */
        wdrv_utilsReport(RL_HAL, PRI_DEBUG,"\nwhal_acxCmdSetKey: Action %d, KeySize:%d, KeyType: %d, KeyId: %d, SSID INDEX:%d\n",pCmd->Action,pCmd->KeySize,pCmd->KeyType ,pCmd->KeyId,pCmd->ssidProfile);

        wdrv_utilsReport(RL_HAL, PRI_DEBUG, "whal_acxCmdSetKey: MacAddress %x,%x,%x,%x,%x,%x\n",pCmd->MacAddr[0],pCmd->MacAddr[1],pCmd->MacAddr[2],pCmd->MacAddr[3],pCmd->MacAddr[4],pCmd->MacAddr[5]);
        if(pCmd->KeySize == 8)
            wdrv_utilsReport(RL_HAL, PRI_DEBUG, "Key: %x,%x,%x,%x,%x,%x,%x,%x\n ",pCmd->Key[0],pCmd->Key[1],pCmd->Key[2],pCmd->Key[3],pCmd->Key[4],pCmd->Key[5],pCmd->Key[6],pCmd->Key[7]);
        else
            wdrv_utilsReport(RL_HAL, PRI_DEBUG, "Key: %x,%x,%x,%x,%x,%x,%x,%x,%x,%x,%x,%x,%x,%x,%x,%x\n ",pCmd->Key[0],pCmd->Key[1],pCmd->Key[2],pCmd->Key[3],pCmd->Key[4],pCmd->Key[5],pCmd->Key[6],pCmd->Key[7],
            pCmd->Key[8],pCmd->Key[9],pCmd->Key[10],pCmd->Key[11],pCmd->Key[12],pCmd->Key[13],pCmd->Key[14],pCmd->Key[15]);

    }
    /* Send the command */
    return whal_acxMboxCommand(CMD_SET_KEY, (char *)pCmd, sizeof(*pCmd), NO_WAIT);
}

/****************************************************************************
 *                      whal_acxCmdRadioInit()
 ****************************************************************************
 * DESCRIPTION: Send RADIO_INIT Command
 * 
 * INPUTS: None 
 * 
 * OUTPUT:  None
 * 
 * RETURNS: OK or NOK
 ****************************************************************************/
int whal_acxCmdRadioInit(UINT32 Addr, UINT32 NumBytes)
{
    AcxCmd_RadioInit_T AcxCmd_RadioInit;
    AcxCmd_RadioInit_T *pCmd = &AcxCmd_RadioInit;

    /*
     * Set command fields 
     */
    PAL_memSet(pCmd, 0, sizeof(*pCmd));

    /* Set parameters */
    pCmd->Addr      = ENDIAN_HANDLE_LONG(Addr);
    pCmd->NumBytes  = ENDIAN_HANDLE_LONG(NumBytes);
    
    /*
     * Send the command 
     */
    return (whal_acxMboxCommand(CMD_RADIO_INIT, (char *)pCmd, sizeof(*pCmd), 100));
}

/****************************************************************************
 *                      whal_acxCmdRadioCalibrate()
 ****************************************************************************
 * DESCRIPTION: Send RADIO_INIT Command
 * 
 * INPUTS: None 
 * 
 * OUTPUT:  None
 * 
 * RETURNS: OK or NOK
 ****************************************************************************/
int whal_acxCmdRadioCalibrate(UINT32 Option)
{
    AcxCmd_RadioCalibration_T AcxCmd_RadioCalibration;
    AcxCmd_RadioCalibration_T *pCmd = &AcxCmd_RadioCalibration;

    /*
     * Set command fields 
     */
    PAL_memSet(pCmd, 0, sizeof(*pCmd));

    /* Set parameters */
    pCmd->Option        = ENDIAN_HANDLE_LONG(Option);
    pCmd->Reserved      = ENDIAN_HANDLE_LONG(0);
    
    /*
     * Send the command, wait 100ms 
     */

    return (whal_acxMboxCommand(CMD_RADIO_CALIBRATE, (char *)pCmd, sizeof(*pCmd), NO_WAIT));
}

/****************************************************************************
 *                      whal_acxCmdBeaconActive()
 ****************************************************************************
 * DESCRIPTION: Send CMD_SET_BCN_ACTIVE Command
 * 
 * INPUTS: Option 0 - supress beacons, 1 - activate beacons  
 * 
 * OUTPUT:  None
 * 
 * RETURNS: OK or NOK
 ****************************************************************************/
int whal_acxCmdBeaconActive(UINT32 Option)
{
    return (whal_acxMboxCommand(CMD_SET_BCN_ACTIVE, (char *)&Option, sizeof(Option), NO_WAIT));
}

/****************************************************************************
 *                      RADAR DETECTION 
 ****************************************************************************/

     
/****************************************************************************
 * DESCRIPTION: whal_acxCmdSendRDStopCmd
 * 
 * INPUTS:  
 *    
 * OUTPUT:  time_diff_evctor  ,number of sample events
 * 
 * RETURNS: OK or NOK
 ****************************************************************************/

int whal_acxCmdSendRDStopCmd(UINT16 *p_time_vector,UINT8 *events_num)
{
    int i;
    radar_detection_events_info Info;
 
    /* TODO:fill message header*/
    Info.ElmHdr.Id = ACX_DYNAMIC_CHANNEL_SELC; 
    Info.ElmHdr.Len = sizeof(Info) - sizeof(ElmHdr_T);

    if(whal_acxMboxInterrogate((char*)&Info, sizeof(radar_detection_events_info)) != OK)
        return NOK;

    for(i = 0; i < Info.events_num; i++)
        Info.events_vec[i] = ENDIAN_HANDLE_WORD(Info.events_vec[i]);

    memcpy((char*)p_time_vector,(char*)Info.events_vec,Info.events_num * sizeof(short));
    *events_num = Info.events_num;

    return OK;
}

/****************************************************************************
 *                      whal_acxCmdSendRDStartCmd()
 ****************************************************************************
 * DESCRIPTION: sends start scan for channel command
 * 
 * INPUTS:  channel
 * 
 * OUTPUT:  None
 * 
 * RETURNS: OK or NOK
 ****************************************************************************/

int whal_acxCmdSendRDStartCmd(UINT8 channel,UINT8 band)
{   
    DcsStart_t  cmd;

    cmd.channel = channel;
    cmd.band    = band;

    return (whal_acxMboxCommand(CMD_DCS_START, (char *)&cmd, sizeof(cmd), NO_WAIT));
}

/****************************************************************************
 *                   whal_acxCrtRadioCalibrateNeed ()
 ****************************************************************************
 * DESCRIPTION: Get CRT calibration state from ACX Device
 * 
 * INPUTS: None 
 * 
 * OUTPUT:  None
 * 
 * RETURNS: None
 ****************************************************************************/

static AcxElm_RunTimeCalState_T    AcxElm_CrtState;

void whal_acxCrtRadioCalibrateNeedAsyncCallback(BOOL status, UINT8 * buf)
{
    UINT32  calState = ENDIAN_HANDLE_LONG(AcxElm_CrtState.RunTimeCalState);

    if (calState == RUNTIME_CALIBRATION_NEEDED) 
    {
        whal_acxCmdRadioCalibrate(RADIO_CALIBRATE_STATE_1);
        
        if(REPORT_LEVEL(RL_MGMT))
        {
            wdrv_utilsReport( RL_MGMT,
                              PRI_INFORMATION,
                              "WDRV_MGMT: Radio Calibrate Option %x is executed\n",
                              RADIO_CALIBRATE_STATE_1 );
        }
    }

}

void whal_acxCrtRadioCalibrateNeedAsync(void) 
{
    memset(&AcxElm_CrtState , 0, sizeof(AcxElm_RunTimeCalState_T));
    /*
     * Set information element header 
     */
    AcxElm_CrtState.ElmHdr.Id  = ACX_CAL_ASSESSMENT;
    AcxElm_CrtState.ElmHdr.Len = sizeof(AcxElm_RunTimeCalState_T) - sizeof(ElmHdr_T); 
    
    whal_acxMboxInterrogateNoWait( CMD_INTERROGATE,
                                   &AcxElm_CrtState,
                                   sizeof(AcxElm_CrtState), 
                                   (whal_userMboxCB)    whal_acxCrtRadioCalibrateNeedAsyncCallback,
                                   (UINT8 *)            &AcxElm_CrtState );
}

/****************************************************************************/

