/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2003 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/****************************************************************************
 *
 *   MODULE:  whalAcxConfig.c
 *   PURPOSE: Handle the ACX100 command and information mailboxes
 *   
 ****************************************************************************/

#include "whalCommon.h"
#include "whalAcxAccess.h"
#include "whalInterrupt.h"
#include "whalAcxRegs.h"
#include "whalAcxMbox.h"
#include "whalParams.h"
#include "dda_contexts.h"
#include "dda_contexts.h"
#include "whal.h"

 
/****************************************************************************
 *                      OBJECT DATA BASE + INTERNALS
 ****************************************************************************
 *
 ****************************************************************************/
static whalMboxQueue_T mBoxQ[MBOX_QUEUE_NUM_ENTRIES];
static UINT32 currMboxQoutInd;
static UINT32 currMboxQinInd;
static UINT32 currMboxQcount;
static whal_userMboxCB  currCmdCB;
static UINT8  *currCmdReplyBuf;
static UINT32 currCmdLen;
static AcxMboxCommand_T currCmdType;
static UINT32 currCmdTimeStamp;
static CmndMbox_T *pCmdMbox;
static InfoMbox_T *pInfoMbox;
static UINT32   UseInterrupt;
static UINT32   CmdCompleteCount;
static BOOL     mboxOn;
static UINT32  CmdCompleteFromIntrCount = 0;
static UINT32   PrevNotCompleteCount;
static UINT32   CmdOnProcess;
static UINT32  WaitTicks;
static UINT32  MaxTicks;

static UINT16   whal_acxMboxGetStatus(void);
static UINT16   whal_acxMboxGetIE(void);

static void whal_acxMboxSetCommand(AcxMboxCommand_T Cmd);
static void whal_acxMboxSetElmHdr(UINT16 *pElmHdr);
static int      whal_acxMboxWaitComplete(UINT32 Timeout);
static int whal_acxMboxSendCmd(AcxMboxCommand_T MboxCmd, char *ParamsBuf, int ParamsLen,
                               UINT32 Timeout, whal_userMboxCB userCB, UINT8 *replyBuf);
static int whal_acxSetCmd2Mbox (AcxMboxCommand_T cmdType, UINT8 * cmdBuf, whal_userMboxCB cb,
                               UINT8 *replyBuf, UINT32 cmdLen, UINT32 Timeout);
static int whal_acxCopyCmd2Queue (AcxMboxCommand_T cmdType, UINT8 * cmdBuf,
                           whal_userMboxCB cb, UINT8 *replyBuf, UINT32 cmdLen);
char * whal_acxMboxStatusToString(UINT16 status);
void whal_acxTraceMboxCommand(AcxMboxCommand_T cmdType, UINT16 status);

static char mboxErrBuff[120];

typedef struct  {
    AcxMboxCommand_T cmd;
    UINT16  status;
    UINT32  timeStamp;
    char cmdParamsBuff[MAX_CMND_PARAMS];
} mboxCmdTraceEntry;

#define WHAL_ACX_TRACE_SIZE 10

mboxCmdTraceEntry mboxCmdErrorTrace[WHAL_ACX_TRACE_SIZE];
UINT32 g_mboxTraceIndex = 0;
UINT32 g_mboxErrorsNumber = 0;


char *mboxErrorCodes[] = {
    "CMD_MAILBOX_IDLE"              /* =  0  */,
    "CMD_STATUS_SUCCESS"            /* =  1  */,
    "CMD_STATUS_UNKNOWN_CMD"        /* =  2  */,
    "CMD_STATUS_UNKNOWN_IE"         /* =  3  */,
    "CMD_STATUS_RX_BUSY"            /* = 13  */,
    "CMD_STATUS_INVALID_PARAM"      /* = 14  */,
    "CMD_STATUS_TEMPLATE_TOO_LARGE" /* = 15  */,
    "CMD_STATUS_OUT_OF_MEMORY"      /* = 16  */,
    "CMD_STATUS_STA_TABLE_FULL"     /* = 17  */,
    "CMD_STATUS_RADIO_ERROR"        /* = 18  */,
    "CMD_STATUS_WRONG_NESTING"      /* = 19  */,
    "CMD_STATUS_IN_PROCESS"         /* = 20  */,

};

#define NO_USER_CB NULL
#define NO_REPLY_BUF NULL

/****************************************************************************
 *                      whal_acxMboxConfigure()
 ****************************************************************************
 * DESCRIPTION: Send configure command with its information element parameter
 * 
 * INPUTS:  
 *      void    *ParamsBuf      The configure parameters buffer
 *      UINT32  ParamsLen       The length of the configure parameters buffer
 *      UINT32  Timeout         Command complete timeout in msec
 * 
 * OUTPUT:  None
 * 
 * RETURNS: OK or NOK
 ****************************************************************************/
int whal_acxMboxConfigure( void *ParamsBuf, UINT32 ParamsLen, UINT32 Timeout)
{
    return( whal_acxMboxSendCmd( CMD_CONFIGURE, ParamsBuf, ParamsLen, Timeout, NO_USER_CB, NO_REPLY_BUF) );
}

/****************************************************************************
 *                      whal_acxMboxInterrogate()
 ****************************************************************************
 * DESCRIPTION: Send interrogate command with its information element parameter
 * 
 * INPUTS:  
 *      void    *ParamsBuf      The parameters buffer for the answer
 *      UINT32  ParamsLen       The length of the parameters buffer
 * 
 * OUTPUT:  None
 * 
 * RETURNS: OK or NOK
 ****************************************************************************/
int whal_acxMboxInterrogate( void *ParamsBuf, UINT32 ParamsLen)
{
    while(whal_acxMboxSendCmd( CMD_INTERROGATE, ParamsBuf, ParamsLen, READ_TIMEOUT, NO_USER_CB, ParamsBuf) == NOK)
    {
        PAL_sleepLong(10);
    }
    return OK;
}

/****************************************************************************
 *                      whal_acxMboxInterrogateNoWait()
 ****************************************************************************
 * DESCRIPTION: Send interrogate command with its information element parameter
 *              Core may use this function only when interrogate command is sent
 *              in run time phase (not during initialization). Basically this 
 *              function is used by different statistics ioctls.
 * 
 * INPUTS:  
 *      void    *ParamsBuf      The parameters buffer for the answer
 *      UINT32  ParamsLen       The length of the parameters buffer
 * 
 * OUTPUT:  None
 * 
 * RETURNS: OK or NOK
 ****************************************************************************/
int whal_acxMboxInterrogateNoWait(  AcxMboxCommand_T Cmd,void *ParamsBuf, UINT32 ParamsLen, whal_userMboxCB userCB, UINT8 *replyBuf)
{   
    whal_acxMboxSendCmd( Cmd, ParamsBuf, ParamsLen, 0, userCB, replyBuf);
    whal_acxIntrEnable(INTR_CMD_COMPLETE);
    return OK;
}

/****************************************************************************
 *                      whal_acxMboxSendCmd()
 ****************************************************************************
 * DESCRIPTION: Send command to the ACX100 command mailbox
 * 
 * INPUTS:  
 *      AcxMboxCommand_T MboxCmd    Command type
 *      char    *ParamsBuf      The command parameters buffer
 *      int     ParamsLen       The length of the command parameters buffer
 *      UINT32  Timeout         Command complete timeout in msec
 * 
 * OUTPUT:  None
 * 
 * RETURNS: OK or NOK
 ****************************************************************************/
int whal_acxMboxCommand(AcxMboxCommand_T MboxCmd, char *ParamsBuf,
                        int ParamsLen, UINT32 Timeout)
{
    /* Overide timeout parameter only in test commands, to wait for test results */
    return( whal_acxMboxSendCmd( MboxCmd, ParamsBuf, ParamsLen, Timeout, NO_USER_CB, NO_REPLY_BUF) );
}

/****************************************************************************
 *                      whal_acxMboxChangeSts()
 ****************************************************************************
 * DESCRIPTION:	Enable/Disable mbox (if disabled, no caommands sent to the  FW)
 * 
 * INPUTS:	
 *		BOOL    sts				Status
 * 
 * OUTPUT:	None
 * 
 * RETURNS:
 ****************************************************************************/
void whal_acxMboxSetStatus(BOOL sts)
{
	mboxOn = sts;
}

/****************************************************************************
 *                      whal_acxMboxChangeSts()
 ****************************************************************************
 * DESCRIPTION:	Get mbox status (if disabled, no caommands sent to the  FW)
 * 
 * INPUTS:	
 *		BOOL    *status				Status
 * 
 * OUTPUT:	None
 * 
 * RETURNS:
 ****************************************************************************/
/*void whal_acxMboxGetStatus(BOOL *status)
{
	*status = mboxOn;
}
*/

/****************************************************************************
 *                      whal_acxMboxSendCmd()
 ****************************************************************************
 * DESCRIPTION: Send command to the ACX100 command mailbox
 * 
 * INPUTS:  
 *      AcxMboxCommand_T MboxCmd    Command type
 *      char    *ParamsBuf      The command parameters buffer
 *      int     ParamsLen       The length of the command parameters buffer
 *      UINT32  Timeout         Command complete timeout in msec
 *      void    *userCB         user callback function
 *      UINT8   *replyBuf       buffer provided with interrogate command
 *
 *  
 * Note  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 *       In case of IOCTL which sends interrogate command to the FW,
 *       the timeout should be 0 and it is the user's responsibility 
 *       to wait for the return of info from the FW and to wake up in the callback
 *       !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 *
 * 
 * OUTPUT:  None
 * 
 * RETURNS: OK or NOK
 ****************************************************************************/
static int whal_acxMboxSendCmd(AcxMboxCommand_T MboxCmd, char *ParamsBuf, int ParamsLen,
                        UINT32 Timeout, whal_userMboxCB userCB, UINT8 *replyBuf)
{
    UINT16 MboxStatus;
    BOOL   cmpltStatus;
    BOOL   prevCommandInProcess = FALSE;
    unsigned int flags = 0;

	if(!mboxOn)
		return OK;

    if (ParamsLen > MAX_CMND_PARAMS ) {
        printf("%s: Warning Mbox command is too long: %d. Truncating it to %d\n",__func__,ParamsLen,MAX_CMND_PARAMS);
        ParamsLen = MAX_CMND_PARAMS;
    }
    /* disable mailbox command complete interrupt */
    whal_acxIntrDisable(INTR_CMD_COMPLETE);

    PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); 
    if (CmdOnProcess)
    {
        /* Check if the previous command completed */
        cmpltStatus = whal_acxIntrCheckAndAck(INTR_CMD_COMPLETE);
        if(cmpltStatus)
        {
            CmdCompleteCount++;
            CmdOnProcess = 0;
            
            MboxStatus = whal_acxMboxGetStatus();

            /* call cb function and report the status */
            if (MboxStatus != CMD_STATUS_SUCCESS)
            {
                sprintf(mboxErrBuff,"MBOX status = %d(%s) for command 0x%X (IE[0x%X])\n", 
                        MboxStatus, whal_acxMboxStatusToString(MboxStatus), currCmdType, 
                        whal_acxMboxGetIE());
                whal_acxTraceMboxCommand(currCmdType, MboxStatus);
                printf("%s",mboxErrBuff);
#ifdef DISABLE_MB_RECOVERY            
                whal_apiWaOptionReset();
                whal_acxMboxSetStatus(FALSE);
                printf("***************************** \n");
                printf("Disable MBOX and hal recovery \n");
                printf("***************************** \n");
#endif
                if(currCmdCB)
                    currCmdCB(FALSE, currCmdReplyBuf);
            }
            else
            {
                /* in case of interrogate, read the info returned by FW */
                if (currCmdType == CMD_INTERROGATE || currCmdType == CMD_TEST)
                    if(currCmdReplyBuf)
                        whal_acxMemCopy( currCmdReplyBuf, pCmdMbox->parameters, currCmdLen);

                /* return the status and the info in case of interrogate command */
                if(currCmdCB)
                    currCmdCB(TRUE, currCmdReplyBuf);   
            }

            /* be aware that a callback could have sent already a command the mailbox */
            if (!CmdOnProcess) 
            {
                if(currMboxQcount > 0)
                {
                    /* copy command from the queue into the mailbox */
                    whal_acxSetCmd2Mbox(mBoxQ[currMboxQoutInd].cmdType,
                                        mBoxQ[currMboxQoutInd].cmdBuf,
                                        mBoxQ[currMboxQoutInd].cb,
                                        mBoxQ[currMboxQoutInd].replyBuf,
                                        mBoxQ[currMboxQoutInd].cmdLen,
                                        0);

                    /* advance out index */
                    currMboxQoutInd = (currMboxQoutInd+1) & MBOX_QUEUE_SIZE_MASK;
                    currMboxQcount--;
                    
                    /* it is not allowed to copy command with timeout to the queue */
                    if(MboxCmd == CMD_INTERROGATE && Timeout > 0)
                    {                   
                        goto exit;
                    }
                        
                    /* copy current command into the queue */
                    whal_acxCopyCmd2Queue(MboxCmd, ParamsBuf, userCB, replyBuf, ParamsLen);
                }
                else
                {
                    /* copy current command into the mailbox */
                    whal_acxSetCmd2Mbox(MboxCmd,ParamsBuf,userCB,replyBuf,ParamsLen,Timeout);
                }
            }
            else /* mailbox is busy - copy current command into the queue */
            {
                /* it is not allowed to copy command with timeout to the queue */
                if(MboxCmd == CMD_INTERROGATE && Timeout > 0)
                {
                    goto exit;
                }
                    
                /* copy current command into the queue */
                whal_acxCopyCmd2Queue(MboxCmd, ParamsBuf, userCB, replyBuf, ParamsLen);
            }

        }
        else
        {
            if(PALSRV_getTick() - currCmdTimeStamp > PALSRV_msec2ticks(MBOX_CMD_FAILURE_THRESHOLD))
            {
                MboxStatus = whal_acxMboxGetStatus();
                /* discard the command */
                sprintf(mboxErrBuff,"failed to wait for MBOX command completion:\nMBOX status = %d(%s) for command 0x%X (IE[0x%X])\n", 
                        MboxStatus, whal_acxMboxStatusToString(MboxStatus), currCmdType, 
                        whal_acxMboxGetIE());
                whal_acxTraceMboxCommand(currCmdType, MboxStatus);
                printf("%s",mboxErrBuff);
                /* disable all kinds of recoveries */
#ifdef DISABLE_MB_RECOVERY            
                whal_apiWaOptionReset();
                whal_acxMboxSetStatus(FALSE);
                printf("***************************** \n");
                printf("Disable MBOX and hal recovery \n");
                printf("***************************** \n");
#endif
                CmdOnProcess = 0;
                 
                if(currCmdCB)
                    currCmdCB(FALSE, currCmdReplyBuf);              
                
                /* be aware that a callback could have sent already a command the mailbox */
                if (!CmdOnProcess) 
                {
                    if(currMboxQcount > 0)
                    {
                        /* copy command from the queue into the mailbox */
                        whal_acxSetCmd2Mbox(mBoxQ[currMboxQoutInd].cmdType,
                                            mBoxQ[currMboxQoutInd].cmdBuf,
                                            mBoxQ[currMboxQoutInd].cb,
                                            mBoxQ[currMboxQoutInd].replyBuf,
                                            mBoxQ[currMboxQoutInd].cmdLen,
                                            0);

                        /* advance out index */
                        currMboxQoutInd = (currMboxQoutInd+1) & MBOX_QUEUE_SIZE_MASK;
                        currMboxQcount--;
                        
                        /* it is not allowed to copy command with timeout to the queue */
                        if(MboxCmd == CMD_INTERROGATE && Timeout > 0)
                        {
                            goto exit;
                        }
                            
                        /* copy current command into the queue */
                        whal_acxCopyCmd2Queue(MboxCmd, ParamsBuf, userCB, replyBuf, ParamsLen);
                    }
                    else
                    {
                        /* copy current command into the mailbox */
                        whal_acxSetCmd2Mbox(MboxCmd,ParamsBuf,userCB,replyBuf,ParamsLen,Timeout);
                    }
                }
                else /* mailbox is busy - copy current command into the queue */
                {
                    /* it is not allowed to copy command with timeout to the queue */
                        if(MboxCmd == CMD_INTERROGATE && Timeout > 0)
                        {
                            goto exit;
                        }
                            
                        /* copy current command into the queue */
                        whal_acxCopyCmd2Queue(MboxCmd, ParamsBuf, userCB, replyBuf, ParamsLen);
                }
            }
            else
            {
                if(currMboxQcount == MBOX_QUEUE_NUM_ENTRIES)
                {
                    printf("The mailbox queue is full, drop current command: 0x%X\n", MboxCmd);
                }
                else
                {                               
                    /* it is not allowed to copy command with timeout to the queue */
                    if(MboxCmd == CMD_INTERROGATE && Timeout > 0)
                    {
                        goto exit;
                    }
                        
                    /* copy current command into the queue */
                    whal_acxCopyCmd2Queue(MboxCmd, ParamsBuf, userCB, replyBuf, ParamsLen);
                }
                prevCommandInProcess = TRUE;
            }
        }       
    }
    else
    {
        /* copy command from the queue into the mailbox */
        if(currMboxQcount > 0)
        {
            /*printf("CmdOnProcess=FALSE when mailbox is not empty !!!!!!!!!!!!!!!\n");*/
            whal_acxSetCmd2Mbox(mBoxQ[currMboxQoutInd].cmdType,
                                mBoxQ[currMboxQoutInd].cmdBuf,
                                mBoxQ[currMboxQoutInd].cb,
                                mBoxQ[currMboxQoutInd].replyBuf,
                                mBoxQ[currMboxQoutInd].cmdLen,
                                0);

            /* advance out index */
            currMboxQoutInd = (currMboxQoutInd+1) & MBOX_QUEUE_SIZE_MASK;
            currMboxQcount--;

            /* it is not allowed to copy command with timeout to the queue */
            if(MboxCmd == CMD_INTERROGATE && Timeout > 0)
            {
                goto exit;
            }
                
            /* copy current command into the queue */
            whal_acxCopyCmd2Queue(MboxCmd, ParamsBuf, userCB, replyBuf, ParamsLen);
        }
        else
        {
            /* copy current command into the mailbox */
            whal_acxSetCmd2Mbox(MboxCmd, ParamsBuf, userCB, replyBuf, ParamsLen, Timeout);
        }               
    }

    /* before quiting check once again whether the command is completed, because 
       mailbox command complete interrupt disabled in this function */
    if(prevCommandInProcess)
    {
        cmpltStatus = whal_acxIntrCheckAndAck(INTR_CMD_COMPLETE);

        if(cmpltStatus)
        {
            CmdCompleteCount++;
            CmdOnProcess = 0;

            /* in case of interrogate, read the info returned by FW */
            if (currCmdType == CMD_INTERROGATE || currCmdType == CMD_TEST)
                if(currCmdReplyBuf)
                    whal_acxMemCopy( currCmdReplyBuf, pCmdMbox->parameters, currCmdLen);

            /* return the status and the info in case of interrogate command */
            if(currCmdCB)
                currCmdCB(TRUE, currCmdReplyBuf);   

            /* be aware that a callback could have sent already a command the mailbox */
            if (!CmdOnProcess) 
            {
                /* copy command from the queue into the mailbox */
                whal_acxSetCmd2Mbox(mBoxQ[currMboxQoutInd].cmdType,
                                mBoxQ[currMboxQoutInd].cmdBuf,
                                mBoxQ[currMboxQoutInd].cb,
                                mBoxQ[currMboxQoutInd].replyBuf,
                                mBoxQ[currMboxQoutInd].cmdLen,
                                0);
            

                /* advance out index */
                currMboxQoutInd = (currMboxQoutInd+1) & MBOX_QUEUE_SIZE_MASK;
                currMboxQcount--;               
            }
        }
    }

    /* in case there are some commands in the queue, enable interrupt */
    if(currMboxQcount > 0)
        whal_acxIntrEnable(INTR_CMD_COMPLETE);

    PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags); 
    return OK;

exit:
    /*printf("interrogate with timeout !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n");*/
    PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags); 
    return NOK;
}


static int whal_acxSetCmd2Mbox (AcxMboxCommand_T cmdType, UINT8 * cmdBuf, whal_userMboxCB cb,
                         UINT8 *replyBuf, UINT32 cmdLen, UINT32 Timeout)
{
    static AcxMboxCommand_T lastCmdType=0;
    static UINT16 LastElmHdrId=0;
    BOOL   Status;

    CmdOnProcess = 1;

    currCmdTimeStamp = PALSRV_getTick();
    currCmdType = cmdType;
    currCmdCB = cb;
    currCmdReplyBuf = replyBuf;
    currCmdLen = cmdLen;
    
    switch (currCmdType)
    {
        case CMD_INTERROGATE:
            /* 
             * Copy the element header only, the answer will have the parameters
             */
            whal_acxMboxSetElmHdr( (UINT16 *)cmdBuf);
            LastElmHdrId = ((ElmHdr_T *)cmdBuf)->Id;
            break;

        case CMD_CONFIGURE:
            /* 
             * Copy the element header and the parameters
             * The header is copied separately because of ENDIAN problems
             */
            whal_acxMboxSetElmHdr( (UINT16 *)cmdBuf);
            whal_acxMemCopy(pCmdMbox->parameters+4, cmdBuf+4, cmdLen-4);                                                        
            LastElmHdrId = ((ElmHdr_T *)cmdBuf)->Id;
            break;

        default:
            /* 
             * Other commands - copy the parameters
             */
            whal_acxMemCopy( pCmdMbox->parameters, cmdBuf, cmdLen);
            LastElmHdrId = 0;
            break;
    }

    whal_acxMboxSetCommand(currCmdType);
    lastCmdType = currCmdType;

    whal_acxIntrTriggerCmd();

    /* in case Timeout is greater then 0, we have to wait for command completion */
    if (Timeout == 0)
        return OK;
    else
    {
        Status = whal_acxMboxWaitComplete(Timeout);
        if (Status != OK)
        {
            sprintf(mboxErrBuff,"whal_acxMboxSendCmd: ERROR waiting for command complete %d %d\n", lastCmdType, LastElmHdrId);
            whal_acxTraceMboxCommand(currCmdType, Status);
            printf("%s",mboxErrBuff);
#ifdef DISABLE_MB_RECOVERY            
            /* disable all kinds of recoveries */
            whal_apiWaOptionReset();
            whal_acxMboxSetStatus(FALSE);
            printf("***************************** \n");
            printf("Disable MBOX and hal recovery \n");
            printf("***************************** \n");
#endif

            if(cb)
                cb(FALSE, NULL);
            return NOK;
        }
        else
        {
            /* in case of interrogate, read the info returned by FW */
            if (cmdType == CMD_INTERROGATE || cmdType == CMD_TEST) 
                if(replyBuf)
                    whal_acxMemCopy( replyBuf, pCmdMbox->parameters, cmdLen);

            /* return the status and the info in case of interrogate command */
            if(cb)
                cb(TRUE, replyBuf); 
        }
    }

    return OK;
}


static int whal_acxCopyCmd2Queue (AcxMboxCommand_T cmdType, UINT8 * cmdBuf,
                           whal_userMboxCB cb, UINT8 *replyBuf, UINT32 cmdLen)
{
    mBoxQ[currMboxQinInd].cmdType = cmdType;
    PAL_memCopy(mBoxQ[currMboxQinInd].cmdBuf, cmdBuf, cmdLen);
    mBoxQ[currMboxQinInd].cmdLen = cmdLen;
    mBoxQ[currMboxQinInd].cb = cb;
    mBoxQ[currMboxQinInd].replyBuf = replyBuf;

    currMboxQinInd = (currMboxQinInd+1) & MBOX_QUEUE_SIZE_MASK;
    currMboxQcount++;

    return OK;
}





/****************************************************************************
 *                      whal_acxMboxReadParameters()
 ****************************************************************************
 * DESCRIPTION: Read the parameters from the ACX100 mailbox
 * 
 * INPUTS:  
 *      char    *ParamsBuf      The parameters buffer 
 *      int     ParamsLen       The length of the parameters buffer
 * 
 * OUTPUT:  None
 * 
 * RETURNS: OK or NOK
 ****************************************************************************/
int whal_acxMboxReadParameters( char *ParamsBuf, int ParamsLen )
{
    whal_acxMemCopy( ParamsBuf, (UINT8 *)pCmdMbox->parameters, ParamsLen);
    return OK;
}

/****************************************************************************
 *                      whal_acxMboxInit()
 ****************************************************************************
 * DESCRIPTION: Init the acxMbox object
 *      Get the mailbox pointers from the scratch pad registers and Set the 
 *      object pointers
 * 
 * INPUTS:  None
 * 
 * OUTPUT:  None
 * 
 * RETURNS: OK or NOK
 ****************************************************************************/
int whal_acxMboxInit()
{
    UINT32 Mbox_Offset;

    whal_acxMboxCreate();

    UseInterrupt = whal_ParamsGetGenParams(0)->UseMboxInterrupt;
    
    /* 
     * Get the command mailbox pointer
     */
    Mbox_Offset  = whal_acxRegRead(REG_COMMAND_MAILBOX_PTR); 
    pCmdMbox = (CmndMbox_T *)((UINT32)ACX100_MEM_ADDRESS + Mbox_Offset);      

    /* 
     * Get the information mailbox pointer
     */
    Mbox_Offset  = whal_acxRegRead(REG_INFO_MAILBOX_PTR); 
    pInfoMbox = (InfoMbox_T *)((UINT32)ACX100_MEM_ADDRESS + Mbox_Offset);  
    
    /*if (UseInterrupt)*/
        whal_acxIntrEnable(INTR_CMD_COMPLETE);

    PAL_memSet(mBoxQ, 0, sizeof(mBoxQ));
    currMboxQoutInd = 0;
    currMboxQinInd = 0;
    currMboxQcount = 0;
    currCmdCB = NULL;
    CmdOnProcess = 0;
	mboxOn = TRUE;

    printf("whal_acxMboxInit: pCmdMbox=0x%x, pInfoMbox=0x%x\n", (int)pCmdMbox, (int)pInfoMbox);

    return OK;
}

/****************************************************************************
 *                      whal_acxMboxCreate()
 ****************************************************************************
 * DESCRIPTION: Create the acxMbox object
 *      Create semaphores and initiate vars
 * 
 * INPUTS:  None
 * 
 * OUTPUT:  None
 * 
 * RETURNS: OK or NOK
 ****************************************************************************/
int whal_acxMboxCreate()
{
    CmdCompleteCount = 0;   
    CmdOnProcess = 0; 
    UseInterrupt = 0;

    return OK;
}

/****************************************************************************
 *                      whal_acxMboxDelete()
 ****************************************************************************
 * DESCRIPTION: Delete the acxMbox Delete semaphores
 * 
 * INPUTS:  None
 * 
 * OUTPUT:  None
 * 
 * RETURNS: OK or NOK
 ****************************************************************************/
int whal_acxMboxUnload()
{
    PAL_memSet(mBoxQ, 0, sizeof(mBoxQ));
    currMboxQoutInd = 0;
    currMboxQinInd = 0;
    currMboxQcount = 0;
    currCmdCB = NULL;
    CmdOnProcess = 0;

    return OK;
}

/****************************************************************************
 *                      whal_acxMboxSetCommand()
 ****************************************************************************
 * DESCRIPTION: Set the command type on the ACX100 command mailbox
 *      use ACX100 access macros because of the ENDIAN problem
 * 
 * INPUTS:  
 *      AcxMboxCommand_T Cmd    The command type    
 * 
 * OUTPUT:  None
 * 
 * RETURNS: 
 ****************************************************************************/
void whal_acxMboxSetCommand(AcxMboxCommand_T Cmd)
{
    whal_acxMemSetU16(&pCmdMbox->CmdType, Cmd);
    whal_acxMemSetU16(&pCmdMbox->Status, 0);
}

void whal_acxMboxSetCommandStatus(UINT16 StatusVal)
{
    whal_acxMemSetU16(&pCmdMbox->Status, StatusVal);
}

/****************************************************************************
 *                      whal_acxMboxGetStatus()
 ****************************************************************************
 * DESCRIPTION: Read from mailbox the status of the last processed command
 
 * 
 * INPUTS:  None
 * 
 * OUTPUT:  None
 * 
 * RETURNS: The command status
 ****************************************************************************/
static UINT16 whal_acxMboxGetStatus(void)
{
    return (whal_acxMemGetU16(&pCmdMbox->Status));
}

/****************************************************************************
 *                      whal_acxMboxGetIE()
 ****************************************************************************
 * DESCRIPTION: Read from mailbox the Information Element ID
 * 
 * INPUTS:  None
 * 
 * OUTPUT:  None
 * 
 * RETURNS: Information Element ID
 ****************************************************************************/
static UINT16 whal_acxMboxGetIE(void)
{
    return (whal_acxMemGetU16(&pCmdMbox->parameters[0]));
}


/****************************************************************************
 *                      whal_acxMboxSetElmHdr()
 ****************************************************************************
 * DESCRIPTION: Set the information elemet header in the parameter field
 *      used for Configure and Interrogate commands only
 *      use ACX100 access macros because of the ENDIAN problem
 * 
 * INPUTS:  
 *      UINT16 *pElmHdr     The information element header
 * 
 * OUTPUT:  None
 * 
 * RETURNS: 
 ****************************************************************************/
static void whal_acxMboxSetElmHdr(UINT16 *pElmHdr)
{
    whal_acxMemSetU16(&pCmdMbox->parameters[0], pElmHdr[0]);
    whal_acxMemSetU16(&pCmdMbox->parameters[2], pElmHdr[1]);
}

/****************************************************************************
 *                      whal_acxMboxWaitComplete()
 ****************************************************************************
 * DESCRIPTION: Wait till the command complete
 *      The wait is on semaphore which the interrupt handler (command complete)
 *      will give
 * 
 * INPUTS:  
 *      UINT32  Timeout     Command complete timeout in ticks
 * 
 * OUTPUT:  None
 * 
 * RETURNS: OK, ERROR
 ****************************************************************************/
static int  whal_acxMboxWaitComplete(UINT32 Timeout)
{
    int Status = NOK;
    int i;

    /* check if complete */
    if (whal_acxIntrCheckAndAck(INTR_CMD_COMPLETE))
    {
        CmdCompleteCount++;
        CmdOnProcess = 0;
        return( OK );
    }

    if (Status == NOK)
        PrevNotCompleteCount++;
    
    for (i=0; i<(int)Timeout; i++)
    {
        PAL_sleepLong(10);

        if (whal_acxIntrCheckAndAck(INTR_CMD_COMPLETE))
        {
            CmdCompleteCount++;
            CmdOnProcess = 0;
            Status = OK;
            break;
        }
    }

    return (Status);
}

/****************************************************************************
 *                      wdrv_mailbox_getEvent()
 ****************************************************************************
 * DESCRIPTION: Called by DDA on mailbox event reception (the completed command 
 *              and the mailbox queue are not handled in the interrupt, 
 *              but in the context supplied by DDA)
 * 
 * INPUTS:  event
 * 
 * OUTPUT:  None
 * 
 * RETURNS: OK or NOK
 ****************************************************************************/
int wdrv_mailbox_getEvent(UINT32 event)
{
    UINT16 MboxStatus, IE;

    /* handle duplication of interrupt and set command */
    if(CmdOnProcess == 0)
        return OK;
        
    /* call cb function and report the status */
    MboxStatus = whal_acxMboxGetStatus();
    IE         = whal_acxMboxGetIE();

    if (MboxStatus != CMD_STATUS_SUCCESS)
    {
        if (MboxStatus != CMD_MAILBOX_IDLE)
        {
            sprintf(mboxErrBuff,"Mbox complete interrupt:\nMBOX status = %d(%s) for command 0x%X (IE[0x%X])\n", 
                    MboxStatus, whal_acxMboxStatusToString(MboxStatus), currCmdType, 
                    whal_acxMboxGetIE());
            whal_acxTraceMboxCommand(currCmdType, MboxStatus);
            printf("%s",mboxErrBuff);
            
            CmdCompleteCount++;
            CmdCompleteFromIntrCount++;
            CmdOnProcess = 0;

            if(currCmdCB)
                currCmdCB(FALSE, currCmdReplyBuf);
        }
    }
    else
    {
        CmdCompleteCount++;
        CmdCompleteFromIntrCount++;
        CmdOnProcess = 0;

        /* in case of interrogate, read the info returned by FW */
        if (currCmdType == CMD_INTERROGATE || currCmdType == CMD_TEST) 
            if(currCmdReplyBuf)
                whal_acxMemCopy( currCmdReplyBuf, pCmdMbox->parameters, currCmdLen);

        /* return the status and the info in case of interrogate command */
        if(currCmdCB)
            currCmdCB(TRUE, currCmdReplyBuf);   
    }

    /* be aware that a callback could have sent already a command the mailbox */
    if (!CmdOnProcess) 
    {
        /* copy command from the queue into the mailbox */
        if(currMboxQcount > 0)
        {
            whal_acxSetCmd2Mbox(mBoxQ[currMboxQoutInd].cmdType,
                                mBoxQ[currMboxQoutInd].cmdBuf,
                                mBoxQ[currMboxQoutInd].cb,
                                mBoxQ[currMboxQoutInd].replyBuf,
                                mBoxQ[currMboxQoutInd].cmdLen,
                                0);
            currMboxQoutInd = (currMboxQoutInd+1) & MBOX_QUEUE_SIZE_MASK;
            currMboxQcount--;

            /* enable the interrupt in case of interrogate*/
            if(currCmdType == CMD_INTERROGATE)
                whal_acxMboxIntrMask(INTR_CMD_COMPLETE, TRUE);
        }
    }

    return OK;
}

int whal_acxMboxSetComplete()
{
    wdrv_DDA_mailbox_setEvent(0);
    return OK;
}

/****************************************************************************
 *                      whal_acxMboxPrint()
 ****************************************************************************
 * DESCRIPTION: Print mbox information
 * 
 * INPUTS:  None
 * 
 * OUTPUT:  
 *          char *  buf         pointer to buffer for formatted information.
 * 
 * RETURNS: Length of the formatted data.
 ****************************************************************************/
int whal_acxMboxPrint(char *buf)
{
    int len = 0;
    len +=sprintf(buf+len,"whal_acxMboxPrint: Addr=0x%x, CmdOnProcess=%d\n", 
        (int)pCmdMbox, CmdOnProcess);
    len +=sprintf(buf+len,"CompleteCount=%d, PrevNotCompleteCount=%d\n",
        CmdCompleteCount, PrevNotCompleteCount);
    len +=sprintf(buf+len,"WaitTicks=%d, MaxTicks=%d\n",
        (int)WaitTicks, (int)MaxTicks);

    return len;
}
 /*****************************************************************************
 * DESCRIPTION: Print mbox Errorinformation
 * 
 * INPUTS:  None
 * 
 * OUTPUT:  
 * 
 * RETURNS:  
 *****************************************************************************/
void whal_acxMboxErrorPrint(void)
{
   printf("Reason: %s",mboxErrBuff);
}


 /*****************************************************************************
 * DESCRIPTION: Convert mbox error code to string
 * 
 * INPUTS:  mbox status
 * 
 * OUTPUT:  
 * 
 * RETURNS: pointer to the error code string
 *****************************************************************************/
char * whal_acxMboxStatusToString(UINT16 status)
{
    char *retStr = "UNKNOWN";
    if(status <= 3) {
        retStr = mboxErrorCodes[status];
    }
    else {
        if(status <= 20) {
            retStr = mboxErrorCodes[status - 9];
        }
    }
    return (retStr);

}

/*****************************************************************************
 * DESCRIPTION: Add mail box command, status and parameters to trace
 * 
 * INPUTS:  mbox command, mbox status
 * 
 * OUTPUT:  
 * 
 * RETURNS: 
 *****************************************************************************/
void whal_acxTraceMboxCommand(AcxMboxCommand_T cmdType, UINT16 status)
{
    mboxCmdErrorTrace[g_mboxTraceIndex].cmd = cmdType;
    mboxCmdErrorTrace[g_mboxTraceIndex].status = status;
    mboxCmdErrorTrace[g_mboxTraceIndex].timeStamp = PAL_GET_TIMESTAMP();
    whal_acxMemCopy( mboxCmdErrorTrace[g_mboxTraceIndex].cmdParamsBuff, 
                     pCmdMbox->parameters, MAX_CMND_PARAMS);
    g_mboxTraceIndex = (g_mboxTraceIndex + 1) % WHAL_ACX_TRACE_SIZE;
    g_mboxErrorsNumber++;
}

 /*****************************************************************************
 * DESCRIPTION: Print mail box errors
 * 
 * INPUTS:  printParams - if TRUE print also parameters
 * 
 * OUTPUT:  
 * 
 * RETURNS: 
 *****************************************************************************/
int whal_acxMboxErrorTracePrint(char *buf, UINT32 printParams)
{
    UINT16 *pBuff;
    int i, j, length = 0;
    UINT32 index;

    index = (g_mboxTraceIndex) ? (g_mboxTraceIndex - 1) : (WHAL_ACX_TRACE_SIZE - 1);
    length += sprintf(buf + length, "\nTotal errors number %d.", g_mboxErrorsNumber);
    for(j = 0; j < WHAL_ACX_TRACE_SIZE; j++) {
        length += sprintf(buf + length, "\nCommand 0x%X, status 0x%X - %s, timestamp %u", 
               mboxCmdErrorTrace[index].cmd, mboxCmdErrorTrace[index].status, 
               whal_acxMboxStatusToString(mboxCmdErrorTrace[index].status),
                          mboxCmdErrorTrace[index].timeStamp);
        if(printParams) {
            pBuff = (UINT16 *)mboxCmdErrorTrace[index].cmdParamsBuff;
            for(i = 0; i < MAX_CMND_PARAMS/2; i ++) {
                if(i % 8 == 0) {
                    length += sprintf(buf + length, "\n%04d : ", i);
                }
                length += sprintf(buf + length, "%04X ", *(pBuff + i));
            }
            length += sprintf(buf + length, "\n");
        }
        index = (index) ? (index - 1) : (WHAL_ACX_TRACE_SIZE - 1);
    }
    length += sprintf(buf + length, "\n");

    return (length);
}


