/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2003 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/****************************************************************************
 *
 *   MODULE:  whalParams.c
 *   PURPOSE: Holds all the whal parameters
 *		The whal parameters are seperate into the following category:
 *		DmaParams	holds Rx and Tx queue and buffers parameters
 *		IdParams	holds the AP MACs (Bssid, AP MAC address) and the SSID string
 *		WlanParams	holds information elements for beacon (SSID, Rates, Channel)
 *		GenParams	holds general parameters such as fragment threshold 
 * 
 ****************************************************************************/
#include "wlan_pal.h"
#include "whalAcxAccess.h"
#include "whalCommon.h"
#include "whalParams.h"
#include "whalAcxMbox.h"
#include "whalAcxConfig.h"
#include "whalTrace.h"
#include "AcxInfoElements.h"
#include "802_11Defs.h"
#include "QoS/QoSClassifier.h"
#include "drvDbMain.h"
#include "AcxDataIf.h"
#include "QoS/QoSClassifier.h"
#include "drvDbRegDomain.h"
#include "whalAcxRegs.h"
#include "drvDbHash.h"
#include "drvDbRegDomain.h"
#include "whalAcxCmd.h"
#include "mgmtSsidHandler.h" 


/****************************************************************************
 *                      OBJECT DATA BASE
 ****************************************************************************
 *
 *	The object holds one structure that contains all the parameters 
 *
 ****************************************************************************
 *
 * !!! Warning !!! The pWhalParams always should point the first 
 *                 WhalParams_Obj instance.
 *
 ****************************************************************************/
WhalParams_T WhalParams_Obj[MAX_WONE_NET], *pWhalParams;
NvParams_T *whal_ParamsGetNvParams(void);

/****************************************************************************
 *                      whal_ParamsInit()
 ****************************************************************************
 * DESCRIPTION:	Initiate all the parameters to the default values
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_ParamsInit()
{
	int Status;
	pWhalParams = &WhalParams_Obj[0];

	PAL_memSet(pWhalParams, 0, sizeof(WhalParams_Obj));

	whal_ParamsInitNvParams();
	/*
	 * initiate the object elements 
	 */
	Status = whal_ParamsInitGenParams();
	if (Status != OK)
	{
		printf("whal_ParamsInit: ERROR in whal_ParamsInitGenParams\n");
		return NOK;
	}
	whal_ParamsInitDmaParams();
	whal_ParamsInitIdParams();
	whal_ParamsInitWlanParams();
	whal_ParamsInitWepParams();

	return OK;
}

/*
 * -------------------------------------------------------------------------- 
 *			DmaParams_T - Rx/Tx Queues and Bufs params
 * -------------------------------------------------------------------------- 
 */

#define DEFAULT_0_NUM_STATIONS_PRIVACY_ON			64

#define DEFAULT_0_NUM_STATIONS_PRIVACY_OFF			16
#define DEFAULT_0_RX_BLOCKS_RATIO		8	 /* 5% unit, 40% for rx pool, the rest 60% is for tx pool */
#define DEFAULT_0_TX_XFER_RATIO			6	 /* 5% unit, 30% for tx pool, the rest 70% is for fragmentation pool */

#define DEFAULT_0_RX_BLOCKS_RATIO_WITH_PRIVACY		8	 /* 5% unit, 40% for rx pool, the rest 60% is for tx pool */
#define DEFAULT_0_TX_XFER_RATIO_WITH_PRIVACY		7	 /* 5% unit, 35% for tx pool, the rest 65% is for fragmentation pool */

#define DEFAULT_0_RXQ_PRIORITY			0   /* low 0 .. 15 high  */
#define DEFAULT_0_RXQ_TYPE			      7	 /* All frames, Data/Ctrl/Mgmt -- not documented well */


#define DEFAULT_0_WONE_RX_BLOCKS_RATIO		7	 /* 5% unit, 35% for rx pool, the rest 65% is for tx pool */
#define DEFAULT_0_WONE_TX_XFER_RATIO		4	 /* 5% unit, 20% for tx pool, the rest 80% is for fragmentation pool */
#define DEFAULT_0_WONE_RX_BLOCKS_RATIO_WITH_PRIVACY		7	 /* 5% unit, 35% for rx pool, the rest 65% is for tx pool */
#define DEFAULT_0_WONE_TX_XFER_RATIO_WITH_PRIVACY		5	 /* 5% unit, 25% for tx pool, the rest 75% is for fragmentation pool */


#define BROADCAST_QUEUE_TYPE	0x80

/* defaults for large queue size */
#define DEFAULT_0_BLOCK_SIZE                256     /* 4096 */ /* 1522 */
#define DEFAULT_0_NUM_RX_BLOCKS			    52      /* In ACX SDRAM */
#define DEFAULT_0_NUM_TX_BLOCKS             50      /* In ACX SDRAM */
#define DEFAULT_0_WME_NUM_RX_BLOCKS			    35      /* In ACX SDRAM */
#define DEFAULT_0_WME_NUM_TX_BLOCKS             60      /* In ACX SDRAM */


#define DEFAULT_0_NUM_RX_HOST_DESC          128     /* In Host SDRAM */
#define DEFAULT_0_NUM_TX_HOST_DESC          256     /* In Host SDRAM */

#define DEFAULT_0_NUM_RX_DESC			    32      /* In ACX SRAM */
#define DEFAULT_0_NUM_UCAST_TX_DESC         32  /*64*/  /* In ACX SRAM */
#define DEFAULT_0_NUM_BCAST_TX_DESC         32  /* In ACX SRAM */
#define DEFAULT_0_UCAST_PRIORITY            0   /* low 0 .. 15 high */
#define DEFAULT_0_BCAST_PRIORITY            BROADCAST_QUEUE_TYPE | 1 /* bit 7 must be set */   /* low 0 .. 15 high */
#define DEFAULT_0_POOL_START                0xAB00

#define DEFAULT_0_WONE_NUM_TX_HOST_DESC		32  /* In Host SDRAM */
#define DEFAULT_0_WONE_NUM_RX_DESC		    16  /* In ACX SRAM */
#define DEFAULT_0_WONE_NUM_UCAST_TX_DESC	4   /*64*/  /* In ACX SRAM */
#define DEFAULT_0_WONE_NUM_BCAST_TX_DESC	8   /* In ACX SRAM */


#define DEFAULT_0_WONE_4X_NUM_UCAST_TX_DESC   4
#define DEFAULT_0_WONE_4X_NUM_BCAST_TX_DESC   8

#define DEFAULT_1_NUM_STATIONS			16
#define DEFAULT_1_RX_BLOCKS_RATIO		8	/* 5% unit, 50% for rx pool, the rest 50% is for tx pool */
#define DEFAULT_1_TX_XFER_RATIO			10	/* 5% unit, 75% for tx pool, the rest 15% is for fragmentation pool */
#define DEFAULT_1_RXQ_PRIORITY			0  /* QUEUE_PRI_LOWEST */
#define DEFAULT_1_RXQ_TYPE				   7	/* All frames, Data/Ctrl/Mgmt */

/* defaults for reduced queue size (large FW code) */
#define DEFAULT_1_BLOCK_SIZE			   256

#define DEFAULT_1_NUM_RX_BLOCKS			    50 /* In ACX SDRAM */
#define DEFAULT_1_NUM_TX_BLOCKS			   100 /* In ACX SDRAM */
#define DEFAULT_1_WME_NUM_RX_BLOCKS			    50 /* In ACX SDRAM */
#define DEFAULT_1_WME_NUM_TX_BLOCKS			   100 /* In ACX SDRAM */

#define DEFAULT_1_NUM_RX_HOST_DESC          64 /* In Host SDRAM */
#define DEFAULT_1_NUM_TX_HOST_DESC          64 /* In Host SDRAM */
#define DEFAULT_1_NUM_RX_DESC			    32 /* In ACX SRAM */
#define DEFAULT_1_NUM_UCAST_TX_DESC		/* 24   */ 16
#define DEFAULT_1_NUM_BCAST_TX_DESC		/* 8    */ 16
#define DEFAULT_1_UCAST_PRIORITY		   0
#define DEFAULT_1_BCAST_PRIORITY		BROADCAST_QUEUE_TYPE | 1
#define DEFAULT_1_POOL_START			   0xBB00

/* Demiurg: WME */
#define DEFAULT_0_NUM_UCAST_WME_TX_DESC 15
#define DEFAULT_0_NUM_BCAST_WME_TX_DESC 10


/****************************************************************************
 *                      whal_ParamsInitDmaParams()
 ****************************************************************************
 * DESCRIPTION:	Initiate DmaParams to the default values
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
void whal_ParamsInitDmaParams()
{
	DmaParams_T *	pDmaParams;
	UINT8 			WoneIndex,
					ac,
					TotalWones,
					WoneTxQueues,
					type,
					priority,
                    active;
    TxQueueId_T 	txQid;

    UINT32          defRxBlocksRatio,
                    defTxXferRatio,
                    defNumTxHostDesc,
                    defNumRxDesc,
                    defUcastTxDesc,
                    defBcastTxDesc;

    BOOL            enableConcat;
    BOOL            privacySupport;

	/*
	 * Initiate DMA parameters
	 */
//	wdrv_dbPrivacySupport(0,&privacySupport); 
	/* TODO :check if we have at list one profile with privecy*/
	if(wdrv_mgmt_ssidIs1PrivecyCfg())
		privacySupport = OK;
	else
		privacySupport = NOK;
	 
	wdrv_dbGetNumSupWone(&TotalWones);
   if (TotalWones == WONE_SINGLE_BAND)
   {
		if (privacySupport == OK)
      {
          defRxBlocksRatio = DEFAULT_0_RX_BLOCKS_RATIO_WITH_PRIVACY;
          defTxXferRatio   = DEFAULT_0_TX_XFER_RATIO_WITH_PRIVACY;
		} else
      {
      defRxBlocksRatio = DEFAULT_0_RX_BLOCKS_RATIO;
      defTxXferRatio   = DEFAULT_0_TX_XFER_RATIO;

      }
      defNumTxHostDesc    = DEFAULT_0_NUM_TX_HOST_DESC;
      defNumRxDesc    =  DEFAULT_0_NUM_RX_DESC;

		if (wdrv_dbGetWMEEnabled())
		{
			defUcastTxDesc = DEFAULT_0_NUM_UCAST_WME_TX_DESC;
			defBcastTxDesc = DEFAULT_0_NUM_BCAST_WME_TX_DESC;
		} else
		{
      defUcastTxDesc   = DEFAULT_0_NUM_UCAST_TX_DESC;
      defBcastTxDesc   = DEFAULT_0_NUM_BCAST_TX_DESC;
		}

   } else
   {
        if (privacySupport == OK)
        {
            defRxBlocksRatio = DEFAULT_0_WONE_RX_BLOCKS_RATIO_WITH_PRIVACY;
            defTxXferRatio   = DEFAULT_0_WONE_TX_XFER_RATIO_WITH_PRIVACY;
        } else
        {
      defRxBlocksRatio = DEFAULT_0_WONE_RX_BLOCKS_RATIO;
      defTxXferRatio   = DEFAULT_0_WONE_TX_XFER_RATIO;
        }

      defNumTxHostDesc    = DEFAULT_0_WONE_NUM_TX_HOST_DESC;
      defNumRxDesc    =  DEFAULT_0_WONE_NUM_RX_DESC;

      wdrv_dbGetConcatEnable(&enableConcat);
      if (enableConcat == TRUE)
      {
         defUcastTxDesc   = DEFAULT_0_WONE_4X_NUM_UCAST_TX_DESC;
         defBcastTxDesc   = DEFAULT_0_WONE_4X_NUM_BCAST_TX_DESC;
		} else
		{
			if (wdrv_dbGetWMEEnabled())
			{
				defUcastTxDesc = DEFAULT_0_NUM_UCAST_WME_TX_DESC;
				defBcastTxDesc = DEFAULT_0_NUM_BCAST_WME_TX_DESC;
			} else
      {
      defUcastTxDesc   = DEFAULT_0_WONE_NUM_UCAST_TX_DESC;
      defBcastTxDesc   = DEFAULT_0_WONE_NUM_BCAST_TX_DESC;
			}
		}       
	}

	for (WoneIndex = 0; WoneIndex < TotalWones; WoneIndex++)
	{	
		pDmaParams = &WhalParams_Obj[WoneIndex].DmaParams;
		wdrv_dbGetNumSupWoneQueues( WoneIndex, &WoneTxQueues );

		printf("%s:%d wone = %d queues = %d\n", __FUNCTION__, __LINE__, WoneIndex, WoneTxQueues);

		if (whal_ParamsGetNvParams()->QueueSizesType == 0)
		{
				
			if (privacySupport == OK)
                pDmaParams->NumStations		= DEFAULT_0_NUM_STATIONS_PRIVACY_ON;
			else
                pDmaParams->NumStations		= DEFAULT_0_NUM_STATIONS_PRIVACY_OFF;

			pDmaParams->RxBlocksRatio	= defRxBlocksRatio;
			pDmaParams->TxXfrBlocksRatio= defTxXferRatio;
			pDmaParams->RxQueue_Priority= DEFAULT_0_RXQ_PRIORITY;
			pDmaParams->RxQueue_Type	= DEFAULT_0_RXQ_TYPE;

			if (wdrv_dbGetWMEEnabled()) {
				pDmaParams->NumRxBlocks		= DEFAULT_0_WME_NUM_RX_BLOCKS;
				pDmaParams->NumTxBlocks		= DEFAULT_0_WME_NUM_TX_BLOCKS;
			} else {
				pDmaParams->NumRxBlocks		= DEFAULT_0_NUM_RX_BLOCKS;
				pDmaParams->NumTxBlocks		= DEFAULT_0_NUM_TX_BLOCKS;
			}

			pDmaParams->NumRxHostDesc   = DEFAULT_0_NUM_RX_HOST_DESC;
			pDmaParams->NumTxHostDesc   = defNumTxHostDesc;

			pDmaParams->RxQueue_NumDesc = defNumRxDesc;
			
			for (ac=0; ac < WDRV_MAX_TX_QUEUE ; ac++)                              
			{
				wdrv_dbGetAcxQindex (WoneIndex, ac, &type, &priority,&active,&txQid);
				
                pDmaParams->TxQueuStatus [ac] = active;

                if (active == TRUE)
                {
                    pDmaParams->TxQPriority  [ac]    = priority;
				
				if (type)
				{
                        pDmaParams->TxNumDesc   [ac]    = defBcastTxDesc;
                        pDmaParams->TxQPriority [ac]    |= BROADCAST_QUEUE_TYPE;
                    } else
				{
						pDmaParams->TxNumDesc   [ac]    =  defUcastTxDesc;
                    }
				}
				
			}
			
			pDmaParams->MemoryPoolStart	= DEFAULT_0_POOL_START;
		} else
		{
			pDmaParams->NumStations		  = DEFAULT_1_NUM_STATIONS;
			pDmaParams->RxBlocksRatio	  = DEFAULT_1_RX_BLOCKS_RATIO;
			pDmaParams->TxXfrBlocksRatio = DEFAULT_1_TX_XFER_RATIO;
			pDmaParams->RxQueue_Priority = DEFAULT_1_RXQ_PRIORITY;
			pDmaParams->RxQueue_Type	  = DEFAULT_1_RXQ_TYPE;

			if (wdrv_dbGetWMEEnabled()) {
				pDmaParams->NumRxBlocks		= DEFAULT_1_WME_NUM_RX_BLOCKS;
				pDmaParams->NumTxBlocks		= DEFAULT_1_WME_NUM_TX_BLOCKS;
			} else {
				pDmaParams->NumRxBlocks		 = DEFAULT_1_NUM_RX_BLOCKS;
				pDmaParams->NumTxBlocks		 = DEFAULT_1_NUM_TX_BLOCKS;
			}

			pDmaParams->NumRxHostDesc       = DEFAULT_1_NUM_RX_HOST_DESC;
			pDmaParams->NumTxHostDesc       = DEFAULT_1_NUM_TX_HOST_DESC;

			pDmaParams->RxQueue_NumDesc = DEFAULT_1_NUM_RX_DESC;
			
            for (ac=0; ac < WDRV_MAX_TX_QUEUE; ac++)
            {
                wdrv_dbGetAcxQindex (WoneIndex, ac, &type, &priority,&active,&txQid);

                pDmaParams->TxQueuStatus [ac] = active;

                if (active == TRUE)
			{
                    pDmaParams->TxQPriority [ac]    = priority;
				
				if (type)
				{
                        pDmaParams->TxNumDesc   [ac]    = DEFAULT_1_NUM_BCAST_TX_DESC;
                        pDmaParams->TxQPriority [ac]    |= BROADCAST_QUEUE_TYPE;
                    } else
				{
                        pDmaParams->TxNumDesc   [ac]    = DEFAULT_1_NUM_UCAST_TX_DESC;
                    }
				}
			}
			pDmaParams->MemoryPoolStart = DEFAULT_1_POOL_START;
		}
	}
}

/****************************************************************************
 *                      whal_ParamsGetDmaParams()
 ****************************************************************************
 * DESCRIPTION:	return DmaParams pointer
 ****************************************************************************/
DmaParams_T *whal_ParamsGetDmaParams(UINT8 WoneIndex) 
{	
	return (&WhalParams_Obj[WoneIndex].DmaParams); 
}

/*
 * -------------------------------------------------------------------------- 
 *							Identity params
 * -------------------------------------------------------------------------- 
 */
UINT8   DefaultBssId[MAC_ADDR_SIZE]			= {0x00,0x02,0x03,0x04,0x05,0x06};
UINT8   DefaultSrcMac[MAC_ADDR_SIZE]		= {0x00,0x02,0x03,0x04,0x05,0x06};
UINT8   DefaultBcastMac[MAC_ADDR_SIZE]		= {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};
#define DEFAULT_AP_SSID	"CBC-AP--"

/****************************************************************************
 *                      whal_ParamsInitIdParams()
 ****************************************************************************
 * DESCRIPTION:	Initiate IdParams to the default values
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
void whal_ParamsInitIdParams()
{
	IdParams_T *pIdParams = &WhalParams_Obj[0].IdParams;
	char ssid[WHAL_SSID_STRING_LEN+1];
	UINT8 macAddr[6];
	char bssId[6];
	
	/*
	 * Read the ID Params from the NVRAM.
	 */
	wdrv_mgmt_ssidGetPrimarySsid(ssid);
	wdrv_dbGetOperationMacAddress(macAddr);
	wdrv_dbGetBssid(bssId);

	/*
	 * Initiate ID parameters
	 */
	PAL_memCopy(pIdParams->BssId,		bssId,				MAC_ADDR_SIZE);
	PAL_memCopy(pIdParams->SrcMacAddr,	macAddr,			MAC_ADDR_SIZE);
	PAL_memCopy(pIdParams->BcastMacAddr, DefaultBcastMac,	MAC_ADDR_SIZE);

	PAL_memCopy(pIdParams->Ssid, ssid, strlen(ssid));
}

/* The following functions are moved to whalParams.h as inline functions */
/****************************************************************************
 *                      whal_ParamsGetBssId()
 ****************************************************************************
 * DESCRIPTION:	return the Bss Id
 ****************************************************************************/
/****************************************************************************
 *                      whal_ParamsGetSrcMac()
 ****************************************************************************
 * DESCRIPTION:	return the AP mac address
 ****************************************************************************/
/****************************************************************************
 *                      whal_ParamsGetBcastMac()
 ****************************************************************************
 * DESCRIPTION:	return the broadcast mac address
 ****************************************************************************/
/****************************************************************************
 *                      whal_ParamsGetGenParams()
 ****************************************************************************
 * DESCRIPTION:	return pointer to the GenParams
 ****************************************************************************/

/* 
 * -------------------------------------------------------------------------- 
 *							General params
 * -------------------------------------------------------------------------- 
 */

#define DEFAULT_FRAGMENT_THRESHOLD		MAX_FRAGMENTATION_THRESHOLD
#define DEFAULT_RTS_THRESHOLD			2347
#define DEFAULT_BEACON_INTERVAL			300
#define DEFAULT_START_RADIO_CHANNEL		6
#define DEFAULT_REG_DOMAIN				0x10		/* FCC */
#define DEFAULT_DTIM_INTERVAL			5
#define DEFAULT_BASIC_RATE_SET			0x0F		/* bitmap 1,2,5.5,11,22 */
#define DEFAULT_SUPPORTED_RATE_SET		0x0F
#define DEFAULT_BSS_TYPE				BSS_TYPE_AP
#define DEFAULT_ACX100_TX_RATE			RATE_1
#define DEFAULT_ACX100_TX_PREMPLE		PREAMBLE_OPT_LONG /* no beacon when short ?! */
#define DEFAULT_RATE_FALLBACK			RATE_FALLBACK_3_RETRIES
#define DEFAULT_MAX_TX_MSDU_LIFETIME	512
#define DEFAULT_SHORT_RETRY_LIMIT		7
#define DEFAULT_LONG_RETRY_LIMIT		4
#define DEFAULT_RX_CONFIG_OPTION		(/*CFG_INT_FCS_ERROR |*/ RX_CFG_ENABLE_ANY_DEST_MAC | RX_CFG_ENABLE_ONLY_MY_BSSID | ((1ul)<<13)) /*Get Info header*/
#define DEFAULT_RX_CONFIG_OPTION_MY_MAC_ANY_BSSID (RX_CFG_ENABLE_ONLY_MY_DEST_MAC | RX_CFG_ENABLE_ANY_BSSID | ((1ul)<<13))
#define DEFAULT_RX_FILTER_OPTION		(/*CFG_RX_RCTS_ACK |*//*CFG_RX_PREQ_EN |*/ CFG_RX_MGMT_EN | CFG_RX_DATA_EN | CFG_RX_CTL_EN |/*CFG_RX_BCN_EN |*/CFG_RX_AUTH_EN |CFG_RX_ASSOC_EN )
#define DEFAULT_RX_FILTER_OPTION_PROBE	(/*CFG_RX_RCTS_ACK |*/  CFG_RX_PREQ_EN |   CFG_RX_MGMT_EN | CFG_RX_DATA_EN | CFG_RX_CTL_EN |/*CFG_RX_BCN_EN |*/CFG_RX_AUTH_EN |CFG_RX_ASSOC_EN )
#define DEFAULT_CAPABILITIES			(DOT11_CAPS_ESS)
#define DEFAULT_USE_MBOX_INTERRUPT		0
#define DEFAULT_SUPPORT_4X				0

#define DEFAULT_PBCC_DYNAMIC_ENABLE			0
#define DEFAULT_PBCC_DYNAMIC_IGNORE_MCAST	0
#define DEFAULT_PBCC_DYNAMIC_INTERVAL		500	/* ticks */

#define DEFAULT_BAND	RADIO_BAND_2_4GHZ
/****************************************************************************
 *                      whal_ParamsInitGenParams()
 ****************************************************************************
 * DESCRIPTION:	Initiate GenParams to the default values
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	
 ****************************************************************************/
int whal_ParamsInitGenParams(void)
{
	GenParams_T *pGenParams = &WhalParams_Obj[0].GenParams;
	UINT8 RatesLen;
	UINT8 Rates[DOT11_SUPPORTED_RATES_MAX+DOT11_EXT_SUPPORTED_RATES_MAX];
	UINT16 RateMask=0;
	UINT8 RateFallback;
	int i;
	UINT8 PbccAlgorithmType;
	BOOL  Support4x;
	UINT8 ERPprotectionTI;
	UINT8 ERPProtectionMode;
	UINT8 ERPProtectionType;
	UINT8 minBasicOFDMrate;
    UINT16 HiddenSSID;
    UINT16 MultipleSsidFlag;
	UINT8 WoneIndex, TotalWones;
    UINT16 pdLowThresh;
	UINT16 rtsThresh;


	/*
	 * Initiate general parameters
	 */

    wdrv_dbGetNumSupWone(&TotalWones);
	
    for (WoneIndex = 0; WoneIndex < TotalWones; WoneIndex++)
    {	
        pGenParams = &WhalParams_Obj[WoneIndex].GenParams;
		 	 	
        wdrv_dbGetDot11Mode(WoneIndex, &pGenParams->Dot11Mode);
        wdrv_dbGetOperationFragmentationThreshold(&pGenParams->FragmentThreshold);

		/* The RTS threshold in ACX HAL is not less than MIN_RTS_THRESHOLD */
        wdrv_dbGetOperationRTSThreshold(&rtsThresh);
		pGenParams->RtsThreshold = (rtsThresh >= WHAL_PARAM_MIN_RTS_THRESH) ? rtsThresh : WHAL_PARAM_MIN_RTS_THRESH ;

        wdrv_dbGetOperationShortRetryLimit(&pGenParams->ShortRetryLimit);
        wdrv_dbGetOperationLongRetryLimit(&pGenParams->LongRetryLimit);
        wdrv_dbGetStationConfigBeaconPeriod(&pGenParams->BeaconInterval);
        wdrv_dbGetStationConfigDtimPeriod(&pGenParams->DtimInterval);
        wdrv_dbGetOperationMaxTransmitMSDULifetime(&pGenParams->MaxTxMsduLifetime);
        wdrv_dbGetCwMinWinSize(&pGenParams->CWmin);
#if( WLAN_ADAPTOR_VERSION == WLAN_ADAPTOR_VERSION_1150)
		pGenParams->enEnergyDetect = 0; 	/* Disable Energy Detection */
#else
        wdrv_dbGetEnergyDetect(&pGenParams->enEnergyDetect);
#endif

        wdrv_dbGetEepromLess(&pGenParams->enEepromLess);
        
		/* at init time PD is configured to low threshold*/
		wdrv_dbGetPacketDetectLowThresh(&pdLowThresh);
		pGenParams->packetDetectThresh = pdLowThresh;


        wdrv_mgmt_ssidGetHiddenSsidStatus(&HiddenSSID);
        wdrv_mgmt_ssidGetMultipleSsidStatus(&MultipleSsidFlag);
        
        /* PBCC hal params init*/
        wdrv_dbGetPBCCAlgorithmMode(0,&PbccAlgorithmType);
        if(PbccAlgorithmType == PBCC_ALGO_DYNAMIC || PbccAlgorithmType == PBCC_ALGO_ENHANCED_DYNAMIC)
        {
            pGenParams->PbccDynamicEnable = TRUE;
        }
	
        /* get support flag for 4x features */
        wdrv_dbGet4xEnable(&Support4x);
        pGenParams->Support4x = Support4x;
	
        wdrv_dbGetERPProtectionTI(0,&ERPprotectionTI);
        pGenParams->SupportTIcapTemplate = Support4x | ERPprotectionTI;
	
        /* ERP hal params init */
        wdrv_dbGetERPProtectionMode(WoneIndex,&ERPProtectionMode);
        if (ERPProtectionMode == PROTECTION_MODE_DYNAMIC || ERPProtectionMode == PROTECTION_MODE_ENHANCED_DYNAMIC)
        {
            pGenParams->ERPProtectionEnable = TRUE;
        }
        else
        {
            pGenParams->ERPProtectionEnable = FALSE;
        }

        wdrv_dbGetERPProtectionType(WoneIndex,&ERPProtectionType);
        pGenParams->ERPProtectionType = ERPProtectionType;

        pGenParams->BssType				= DEFAULT_BSS_TYPE;
	
		wdrv_dbGetRateFallback(&RateFallback);
		pGenParams->RateFallback		= (UINT8)RateFallback;
	
		/*pGenParams->RxConfigOption = DEFAULT_RX_CONFIG_OPTION;*/
		pGenParams->RxConfigOption = DEFAULT_RX_CONFIG_OPTION_MY_MAC_ANY_BSSID;

        /* NewPlcp - for MIC failure from PLCP header */
        pGenParams->RxConfigOption |= CFG_RX_RAW; 

		/* status + plcp */
		pGenParams->RxStatusHdrLen = BLOCK_HEADER_SIZE + ACX_RX_STAT_HDR_LEN + ACX_RX_PLCP_HDR_LEN;

		pGenParams->RxFilterOption = DEFAULT_RX_FILTER_OPTION_PROBE;
 
		if (HiddenSSID || MultipleSsidFlag)
        {
            if (HiddenSSID)
                printf("\n\nHidden SSID is activated\n");
            if (MultipleSsidFlag)
                printf("\n\nMultiple SSID are enabled\n");
        }

        if ((pGenParams->PbccDynamicEnable) || (pGenParams->ERPProtectionEnable))
            pGenParams->RxFilterOption |= CFG_RX_BCN_EN;

        pGenParams->UseMboxInterrupt	= DEFAULT_USE_MBOX_INTERRUPT;

	/* ----------- WONE --------------- */

#ifndef NO_PBCC_CHECK
		if (whal_ParamsGetNvParams()->BeaconPatch != 0)
			pGenParams->ACX100_GenTxRate = whal_ParamsGetNvParams()->BeaconPatch_Rate;
#endif

		/*
	 	 * set acx111 basic rates
		 */

        wdrv_dbGetBand(WoneIndex ,&pGenParams->Band); 

        wdrv_dbGetStationConfigOperationalAllRateSet(WoneIndex,Rates, &RatesLen);	

		/*
		 * BasicRateSet
		 * The ACX100 uses this field to determine the rate at which to transmit 
		 * control frame responses (such as ACK or CTS frames)
		 */
        pGenParams->BasicRateSet = 0;
        for (i=0; i<RatesLen; i++)
        {
            if ((Rates[i] & 0x80) != 0x80)
                continue;

			switch (Rates[i] & 0x7F)
            {
                case DOT11_RATE_1:   
                    RateMask=ACX_RATE_1;   
                    break;
                case DOT11_RATE_2:   
                    RateMask=ACX_RATE_2;   
                    break;
				case DOT11_RATE_5_5: 
                    RateMask=ACX_RATE_5_5; 
                    break;
				case DOT11_RATE_6:   
                    RateMask=ACX_RATE_6;   
                    break;
				case DOT11_RATE_9:   
                    RateMask=ACX_RATE_9;   
                    break;
				case DOT11_RATE_11:  
                    RateMask=ACX_RATE_11;  
                    break;
				case DOT11_RATE_12:  
                    RateMask=ACX_RATE_12;  
                    break;
				case DOT11_RATE_18:  
                    RateMask=ACX_RATE_18;  
                    break;
				case DOT11_RATE_22:  
                    RateMask=ACX_RATE_22;  
                    break;
				case DOT11_RATE_24:  
                    RateMask=ACX_RATE_24;  
                    break;
				case DOT11_RATE_36:  
                    RateMask=ACX_RATE_36;  
                    break;
				case DOT11_RATE_48:  
                    RateMask=ACX_RATE_48;  
                    break;
				case DOT11_RATE_54:  
                    RateMask=ACX_RATE_54;  
                    break;
				default:
					printf("whal_ParamsInitGenParams: ERROR unsupported rate=0x%x wdrv_dbGetStationConfigOperationalRateSet\n", Rates[i]);  
			}
			pGenParams->BasicRateSet |= RateMask;
		}

          
        if(RatesLen == 0)
        {
            printf("\nERROR >>>>>>>>>>>>>>>>>>> RatesLen == 0 <<<<<<<<<<<<<<<<<<<<<<\n");
            printf("\nERROR >>>>>>>>>>>>>>>>>>> RatesLen == 0 <<<<<<<<<<<<<<<<<<<<<<\n");
            printf("\nERROR >>>>>>>>>>>>>>>>>>> RatesLen == 0 <<<<<<<<<<<<<<<<<<<<<<\n");
            printf("\nERROR >>>>>>>>>>>>>>>>>>> RatesLen == 0 <<<<<<<<<<<<<<<<<<<<<<\n");
            pGenParams->BasicRateSet = (ACX_RATE_6 | ACX_RATE_12 |ACX_RATE_24);
        }

		/*
		 * ACX100_GenTxRate
		 * This field indicates the rate at which the ACX100 transmits 
		 * Beacon, Probe Response, RTS, and PS-Poll frames
		 * 802.11 b/g - Set it to the 2Mbps
		 * 802.11 a - Set it to min OFDM basic rate (6 MBPS)
		 */

        if (pGenParams->Band == RADIO_BAND_2_4GHZ)
        {
			pGenParams->ACX100_GenTxRate = RATE_2;
			/* WiFi mode optimizations */
			if (wdrv_dbGetApBridgeMode())
				pGenParams->ACX100_GenTxRate = RATE_11;
        } 
        else   /* RADIO_BAND_5GHZ */
        {
            wdrv_dbGetMinOFDMbasicRate(WoneIndex,&minBasicOFDMrate);
		    switch(minBasicOFDMrate & 0x7F)
            {
               case DOT11_RATE_6:
                     pGenParams->ACX100_GenTxRate = RATE_6MBPS;
                     break;
               case DOT11_RATE_9:						
                     pGenParams->ACX100_GenTxRate = RATE_9MBPS;
                     break;
               case DOT11_RATE_12:						
                     pGenParams->ACX100_GenTxRate = RATE_12MBPS;
                     break;
               case DOT11_RATE_18: 
                     pGenParams->ACX100_GenTxRate = RATE_18MBPS; 
                     break;
               case DOT11_RATE_24:  
                     pGenParams->ACX100_GenTxRate = RATE_24MBPS;
                     break;
               case DOT11_RATE_36:	  
                     pGenParams->ACX100_GenTxRate = RATE_36MBPS;
                     break;
               case DOT11_RATE_48:	
                     pGenParams->ACX100_GenTxRate = RATE_48MBPS;
                     break;
               case DOT11_RATE_54:	 
                     pGenParams->ACX100_GenTxRate = RATE_54MBPS;
                     break;
               default:
					pGenParams->ACX100_GenTxRate = RATE_6MBPS;
			}
        }
	
		/* Beacon, Probe Response, RTS and PS-Poll send by the ACX100 will send only in LONG PREAMBLE */
		
        if (pGenParams->Band == RADIO_BAND_2_4GHZ)
            pGenParams->ACX100_GenTxPreample = (UINT8)PREAMBLE_OPT_LONG;
        else
            pGenParams->ACX100_GenTxPreample = ((UINT8)PREAMBLE_OPT_LONG) | 0x40;

        wdrv_dbGetApCapabilities(WoneIndex,&pGenParams->BeaconCapabilities);
		{
			int primSsidInd;
			wdrv_mgmt_ssidGetPrimarySsidIndex(&primSsidInd);
			wdrv_mgmt_ssidUpdateCapPrivProfileIndex(&pGenParams->BeaconCapabilities,primSsidInd);
		}
        wdrv_dbGetApCapabilities(WoneIndex,&pGenParams->ProbResCapabilities);
        wdrv_dbGetChannel       (WoneIndex,&pGenParams->StartRadioChannel);
        wdrv_dbGetdot11CurrentRegDomain	(pGenParams->Band, &pGenParams->CurrRegDomain);
        wdrv_dbGetPowerLevel    (pGenParams->StartRadioChannel, &pGenParams->PowerLevel);

	        /* ----------- WONE --------------- */
    }

	
	return OK;
}

/****************************************************************************
 *                      whal_ParamsGetEeepromLess()
 ****************************************************************************
 * DESCRIPTION:	return the eeprom less mode
 ****************************************************************************/
UINT16 whal_ParamsGetEeepromLess()
{
	return (pWhalParams->GenParams.enEepromLess);
}

/****************************************************************************
 *                      whal_ParamsGetBeaconInterval()
 ****************************************************************************
 * DESCRIPTION:	return the beacon interval
 ****************************************************************************/
UINT16 whal_ParamsGetBeaconInterval()
{
	return (pWhalParams->GenParams.BeaconInterval);
}

/****************************************************************************
 *                      whal_ParamsGetRadioChannel()
 ****************************************************************************
 * DESCRIPTION:	return the radio channel 
 ****************************************************************************/
UINT8 whal_ParamsGetRadioChannel(UINT8 WoneIndex)
{
	return (WhalParams_Obj[WoneIndex].GenParams.StartRadioChannel);
}

/****************************************************************************
 *                      whal_ParamsSetRadioChannel()
 ****************************************************************************
 * DESCRIPTION:	set the radio channel 
 ****************************************************************************/
void whal_ParamsSetRadioChannel(UINT8 WoneIndex, int Channel)
{
	WhalParams_Obj[WoneIndex].GenParams.StartRadioChannel = (UINT8)Channel;
}

/****************************************************************************
 *                      whal_ParamsSetFragmentThreshold()
 ****************************************************************************
 * DESCRIPTION:	set the fragmentation threshold 
 ****************************************************************************/
void whal_ParamsSetFragmentThreshold(int FragSize)
{
	pWhalParams->GenParams.FragmentThreshold = FragSize;
}

/****************************************************************************
 *                      whal_ParamsSetRtsThreshold()
 ****************************************************************************
 * DESCRIPTION:	set the rts threshold 
 ****************************************************************************/
void whal_ParamsSetRtsThreshold(int RtsSize)
{
	pWhalParams->GenParams.RtsThreshold = RtsSize;
}

/****************************************************************************
 *                      whal_ParamsSetShortRetryLimit()
 ****************************************************************************
 * DESCRIPTION:	set the short retry limit 
 ****************************************************************************/
void whal_ParamsSetShortRetryLimit(UINT8 Val)
{
	pWhalParams->GenParams.ShortRetryLimit = Val;
}

/****************************************************************************
 *                      whal_ParamsSetShortRetryLimit()
 ****************************************************************************
 * DESCRIPTION:	set the long retry limit 
 ****************************************************************************/
void whal_ParamsSetLongRetryLimit(UINT8 Val)
{
	pWhalParams->GenParams.LongRetryLimit = Val;
}

static  UINT8 log2(UINT32 n) {
	UINT8 log = 0;

	if (!n)
		return 0;

	while (n > 1) {
		n >>= 1;
		++log;
	}
	
	return log;
}

/*
 * -------------------------------------------------------------------------- 
 *							Wlan params
 * -------------------------------------------------------------------------- 
 */
/****************************************************************************
 *                      whal_ParamsInitWlanParams()
 ****************************************************************************
 * DESCRIPTION:	Initiate WlanParams to the default values
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
void whal_ParamsInitWlanParams()
{
	GenParams_T *		pGenParams 		= &WhalParams_Obj[0].GenParams;
	WlanElmParams_T *	pWlanElmParams;
	UINT8 ERPProtectionMode, PreambleMode, GSlotTimeMode, pbccMode;
    UINT8 Dot11Mode;
	UINT8 WoneIndex, TotalWones, band, length;
    wlan_bandType bandType;
	/*
	 * Initiate Wlan parameters
	 */

	wdrv_dbGetNumSupWone(&TotalWones);
	
	for (WoneIndex = 0; WoneIndex < TotalWones; WoneIndex++)
	{
        UINT8 tmpV;

		pWlanElmParams 	= &WhalParams_Obj[WoneIndex].WlanElmParams;
		
		/* SSID */
		pWlanElmParams->WlanElm_Ssid.hdr.eleId = DOT11_SSID_ELE_ID;
		pWlanElmParams->WlanElm_Ssid.hdr.eleLen = strlen( WhalParams_Obj[0].IdParams.Ssid);
		PAL_memCopy(pWlanElmParams->WlanElm_Ssid.serviceSetId, WhalParams_Obj[0].IdParams.Ssid, strlen(WhalParams_Obj[0].IdParams.Ssid));
	
		/* Supported rates */
		pWlanElmParams->WlanElm_SupportedRates.hdr.eleId = DOT11_SUPPORTED_RATES_ELE_ID;
			wdrv_dbGetStationConfigOperationalRateSet(WoneIndex,pWlanElmParams->WlanElm_SupportedRates.rates,
	                       &pWlanElmParams->WlanElm_SupportedRates.hdr.eleLen);
	
		/* Extended Supported rates */
		pWlanElmParams->WlanElm_ExtSupportedRates.hdr.eleId = DOT11_EXT_SUPPORTED_RATES_ELE_ID;
			wdrv_dbGetStationConfigOperationalOFDMRateSet(WoneIndex,pWlanElmParams->WlanElm_ExtSupportedRates.rates,
		                   &pWlanElmParams->WlanElm_ExtSupportedRates.hdr.eleLen);
	
		/* current channel */
		pWlanElmParams->WlanElm_DsParams.hdr.eleId = DOT11_DS_PARAMS_ELE_ID;
		pWlanElmParams->WlanElm_DsParams.hdr.eleLen = DOT11_DS_PARAMS_ELE_LEN;
			pWlanElmParams->WlanElm_DsParams.currChannel = whal_ParamsGetRadioChannel(WoneIndex);
	
		/* TIM */
		pWlanElmParams->WlanElm_Tim.hdr.eleId = DOT11_TIM_ELE_ID;
		pWlanElmParams->WlanElm_Tim.hdr.eleLen = 4;
		pWlanElmParams->WlanElm_Tim.dtimCount = 0;
		pWlanElmParams->WlanElm_Tim.dtimPeriod = WhalParams_Obj[0].GenParams.DtimInterval;
		pWlanElmParams->WlanElm_Tim.bmapControl = 0;
		PAL_memSet(pWlanElmParams->WlanElm_Tim.partialVirtualBmap, 0, DOT11_PARTIAL_VIRTUAL_BITMAP_MAX);
	
        /* Country Information */
        PAL_memSet(&pWlanElmParams->WlanElm_CountryInformation,0,sizeof(pWlanElmParams->WlanElm_CountryInformation));

        pWlanElmParams->WlanElm_CountryInformation.hdr.eleId = DOT11_COUNTRY_INFORMATION_ELE_ID;
        wdrv_dbGetdot11CountryString(pWlanElmParams->WlanElm_CountryInformation.CountryString);

        /* Locate compatible band for Regulatory Domain */
        wdrv_dbGetBand(WoneIndex, &band);
        bandType = (band == RADIO_BAND_2_4GHZ) ? WLAN_BAND_B : WLAN_BAND_A;
        length = sizeof(pWlanElmParams->WlanElm_CountryInformation.channelInfo);
        if (wdrv_dbFillChannelsInformation(bandType,(UINT8 *)&pWlanElmParams->WlanElm_CountryInformation.channelInfo,&length) == ERROR)
        {
            printf("PANIC!!\nPANIC!! Country information is shorter to contain the information - enlarge size!!\nPANIC!\n");
        }
        pWlanElmParams->WlanElm_CountryInformation.hdr.eleLen = DOT11_COUNTRY_STRING_LENGTH+length;
        if (((DOT11_COUNTRY_STRING_LENGTH+length) %2) != 0) 
            pWlanElmParams->WlanElm_CountryInformation.hdr.eleLen++;    /* padding of one byte. */
            
		
		/* WPA rsn/ssn */
		wdrv_mgmt_ssidGetRsne(0,&pWhalParams->WlanElmParams.WlanElm_Rsne);
		wdrv_mgmt_ssidGetSsn(0,&pWhalParams->WlanElmParams.WlanElm_Ssn);
		
		pWlanElmParams->WlanElm_Cf.hdr.eleId	= DOT11_CF_ELE_ID;
		pWlanElmParams->WlanElm_Cf.hdr.eleLen	= DOT11_CF_PARAMS_ELE_LEN;
		pWlanElmParams->WlanElm_Cf.cfpCount		= 0;
		pWlanElmParams->WlanElm_Cf.cfpPeriod	= 1;
		pWlanElmParams->WlanElm_Cf.cfpMaxDuration	= 256;
		pWlanElmParams->WlanElm_Cf.cfpDurRemain		= 256;
	
		if (pGenParams->SupportTIcapTemplate == 1)
			whal_ParamsSetElm_TIcapSupport();
		else
			whal_ParamsResetElm_TIcapSupport();
	
		/* Init ERP IE */
		pWlanElmParams->WlanElm_ERP.hdr.eleId  = DOT11_ERP_ELE_ID;
		pWlanElmParams->WlanElm_ERP.hdr.eleLen = DOT11_ERP_ELE_LEN;
		pWlanElmParams->WlanElm_ERP.erp = 0;

		wdrv_dbGetDot11Mode(WoneIndex, &Dot11Mode);
		if ((Dot11Mode == DOT11_MODE_B_ONLY) || (Dot11Mode == DOT11_MODE_B_ONLY_PLUS) || (Dot11Mode == DOT11_MODE_A_ONLY))
		{
			pWlanElmParams->ERP_Enabled = 0;
		}
		else
		{
			pWlanElmParams->ERP_Enabled = 1;
		}

		/* ERP Protection - Start without protection if not always enabled */
		wdrv_dbGetERPProtectionMode(WoneIndex,&ERPProtectionMode);
      if (ERPProtectionMode == PROTECTION_MODE_ENABLED)
      {
         tmpV = 1;
      }else
      {
         if((ERPProtectionMode == PROTECTION_MODE_LOCAL_STATIONS) ||
            (ERPProtectionMode == PROTECTION_MODE_DYNAMIC       ) ||
            (ERPProtectionMode == PROTECTION_MODE_ENHANCED_DYNAMIC))
         {
            /*  Dynamic ERP protection - test current state */
            BOOL erpSup;
            wdrv_dbGetCurrentERPProtectionMode( &erpSup);
            tmpV = (erpSup ? 1:0);
         }else
         {
            tmpV = 0;
         }
      }
      
      {
         /* ERP Protection - Not Present bit */
         BOOL erpNonP;
         wdrv_dbGetCurrentERPNonPresent( &erpNonP);
         whal_ParamsSetElm_ERPProtection( WoneIndex, erpNonP ? 1:0, tmpV);
      }
      
      /* ERP Preamble */
      wdrv_dbGetPreambleMode(WoneIndex,&PreambleMode);
      if (PreambleMode == PREAMBLE_MODE_LONG)
      {
         /* start with long preamble if configured */
         tmpV = 1;
      }else if( PreambleMode == PREAMBLE_MODE_SHORT)
      {
         /* start with short preamble if configured */
         tmpV = 0;
      }else
      {
      /* PreambleMode == PREAMBLE_MODE_LOCAL_STATIONS
      * Preamble mode is dictated according to current state of the STAs.
         */
         BOOL shrtPreambSup;
         wdrv_dbGetCurrentShortPreambleSupport( &shrtPreambSup);
         tmpV = (shrtPreambSup? 0:1);
      }
      whal_ParamsSetElm_ERPLongPreambleOnly(WoneIndex, tmpV);
      
      /* G - Mode Short/Long Slot time 9/20usec */
      wdrv_dbGetSlotTimeMode( WoneIndex, &GSlotTimeMode);
      if( GSlotTimeMode == SLOTTIME_MODE_ENABLED)
      {
         /* Start with short slot time */
         tmpV = 1;
      }else if( (GSlotTimeMode == SLOTTIME_MODE_LOCAL_STATIONS) ||
         (GSlotTimeMode == SLOTTIME_MODE_ENHANCED_DYNAMIC) )
      {
         /* Determine it according to current state of the system */
         BOOL currentShortSlotTimeSupport;
         wdrv_dbGetCurrentShortSlotTimeSupport( &currentShortSlotTimeSupport);
         tmpV = (currentShortSlotTimeSupport ? 1:0);
      }else
      {
         tmpV = 0;
      }
      /* set the beacon & probe response capabilities bit field */
      if( tmpV == 0)
      {
         pGenParams->BeaconCapabilities  &= ~DOT11_CAPS_SLOTTIME;
         pGenParams->ProbResCapabilities &= ~DOT11_CAPS_SLOTTIME;
      }else
      {
         pGenParams->BeaconCapabilities  |= DOT11_CAPS_SLOTTIME;
         pGenParams->ProbResCapabilities |= DOT11_CAPS_SLOTTIME;
      }
      
      /* PBCC capabilities */
      wdrv_dbGetPBCCAlgorithmMode( WoneIndex, &pbccMode);
      if( pbccMode == PBCC_ALGO_ENABLED)
      {
         tmpV = 1;
      }else
      {
         if( (pbccMode == PBCC_ALGO_LOCAL_STATIONS) ||
            (pbccMode == PBCC_ALGO_DYNAMIC) ||
            (pbccMode == PBCC_ALGO_ENHANCED_DYNAMIC))
         {
            BOOL shrtPreambSup;
            wdrv_dbGetCurrentPBCCSupportMode( &shrtPreambSup);
            tmpV = (shrtPreambSup ? 1:0);
         }else
         {
            /* PBCC disabled */
            tmpV = 0;
         }
      }
      /* Set the beacon & probe response capabilities bit field */
      if( tmpV == 0)
      {
         pGenParams->BeaconCapabilities  &= ~DOT11_CAPS_PBCC;
         pGenParams->ProbResCapabilities &= ~DOT11_CAPS_PBCC;
      }else
      {
         pGenParams->BeaconCapabilities  |= DOT11_CAPS_PBCC;
         pGenParams->ProbResCapabilities |= DOT11_CAPS_PBCC;
      }

			/* Demiurg: WME */
			if (wdrv_dbGetWMEEnabled()) {
				int i;
				wdrv_QoSACParams_t wmeACParams;

				pWlanElmParams->WlanElm_WME_IE.hdr.eleId = DOT11_WME_IE_ELE_ID;
				pWlanElmParams->WlanElm_WME_IE.hdr.eleLen = DOT11_WME_IE_ELE_LEN;
				pWlanElmParams->WlanElm_WME_IE.OUI[0] = DOT11_WME_IE_ELE_OUI_0;
				pWlanElmParams->WlanElm_WME_IE.OUI[1] = DOT11_WME_IE_ELE_OUI_1;
				pWlanElmParams->WlanElm_WME_IE.OUI[2] = DOT11_WME_IE_ELE_OUI_2;
				pWlanElmParams->WlanElm_WME_IE.OUIType = DOT11_WME_IE_ELE_OUI_TYPE;
				pWlanElmParams->WlanElm_WME_IE.OUISubtype = DOT11_WME_IE_ELE_OUI_SUBTYPE;
				pWlanElmParams->WlanElm_WME_IE.version = DOT11_WME_IE_ELE_VERSION;
				pWlanElmParams->WlanElm_WME_IE.QoSInfo = 0;
				
				pWlanElmParams->WlanElm_WME_PE.ie.hdr.eleId = DOT11_WME_PE_ELE_ID;
				pWlanElmParams->WlanElm_WME_PE.ie.hdr.eleLen = DOT11_WME_PE_ELE_LEN;
				pWlanElmParams->WlanElm_WME_PE.ie.OUI[0] = DOT11_WME_PE_ELE_OUI_0;
				pWlanElmParams->WlanElm_WME_PE.ie.OUI[1] = DOT11_WME_PE_ELE_OUI_1;
				pWlanElmParams->WlanElm_WME_PE.ie.OUI[2] = DOT11_WME_PE_ELE_OUI_2;
				pWlanElmParams->WlanElm_WME_PE.ie.OUIType = DOT11_WME_PE_ELE_OUI_TYPE;
				pWlanElmParams->WlanElm_WME_PE.ie.OUISubtype = DOT11_WME_PE_ELE_OUI_SUBTYPE;
				pWlanElmParams->WlanElm_WME_PE.ie.version = DOT11_WME_PE_ELE_VERSION;
				pWlanElmParams->WlanElm_WME_PE.ie.QoSInfo = 0;
				pWlanElmParams->WlanElm_WME_PE.reserved = 0;

				wmeACParams=wdrv_dbGetWMEParams();

				for (i=0; i<WDRV_QOS_MAX_AC; ++i) {
					pWlanElmParams->WlanElm_WME_PE.ac[i].ACI_AIFSN = wmeACParams.sta_edcfParams[i].aifsn | ((i & 0x3) << 5);
					pWlanElmParams->WlanElm_WME_PE.ac[i].ECWmin_max = log2(wmeACParams.sta_edcfParams[i].cwMin + 1) | (log2(wmeACParams.sta_edcfParams[i].cwMax + 1) << 4);
					wmeACParams.sta_edcfParams[i].txopLimit = (Dot11Mode == DOT11_MODE_B_ONLY || Dot11Mode == DOT11_MODE_B_ONLY_PLUS) ? 
																											 wmeACParams.sta_edcfParams[i].txopLimit11b : wmeACParams.sta_edcfParams[i].txopLimit11g;
					pWlanElmParams->WlanElm_WME_PE.ac[i].Txoplimit = PAL_LE2UINT16((wmeACParams.sta_edcfParams[i].txopLimit)/32); /* units of TXop are 32u */
				}

			}

   }/*endFor*/
}


/****************************************************************************
 *                      whal_ParamsGetElm_Ssid()
 ****************************************************************************
 * DESCRIPTION:	return the SSID info element
 ****************************************************************************/
dot11_SSID_T  *whal_ParamsGetElm_Ssid()
{
	return (&pWhalParams->WlanElmParams.WlanElm_Ssid);
}


/****************************************************************************
 *                      whal_ParamsGetElm_SupportedRates()
 ****************************************************************************
 * DESCRIPTION:	return the Supported rates info element
 ****************************************************************************/
dot11_supRates_T  *whal_ParamsGetElm_SupportedRates(UINT8 WoneIndex)
{
	return (&WhalParams_Obj[WoneIndex].WlanElmParams.WlanElm_SupportedRates);
}


/****************************************************************************
 *                      whal_ParamsGetElm_CountryInfo()
 ****************************************************************************
 * DESCRIPTION:	return the Country Information info element
 ****************************************************************************/
extern dot11_CountryInfo_T *whal_ParamsGetElm_CountryInfo(UINT8 WoneIndex)
{
    return (&WhalParams_Obj[WoneIndex].WlanElmParams.WlanElm_CountryInformation);
}


/****************************************************************************
 *                      whal_ParamsGetElm_ExtSupportedRates()
 ****************************************************************************
 * DESCRIPTION:	return the Extended Supported rates info element
 ****************************************************************************/
dot11_extSupRates_T  *whal_ParamsGetElm_ExtSupportedRates(UINT8 WoneIndex)
{
	return (&WhalParams_Obj[WoneIndex].WlanElmParams.WlanElm_ExtSupportedRates);
}

/****************************************************************************
 *                      whal_ParamsGetElm_ERP()
 ****************************************************************************
 * DESCRIPTION:	return the ERP info element
 ****************************************************************************/
dot11_ERP_T  *whal_ParamsGetElm_ERP(UINT8 WoneIndex)
{
	if (WhalParams_Obj[WoneIndex].WlanElmParams.ERP_Enabled)
	{
        	return (&WhalParams_Obj[WoneIndex].WlanElmParams.WlanElm_ERP);
	}
	else
	{
		return NULL;
	}
}

/****************************************************************************
 *                      whal_ParamsGetElm_DsParams()
 ****************************************************************************
 * DESCRIPTION:	return the DsParameters info element
 ****************************************************************************/
dot11_dsParams_T  *whal_ParamsGetElm_DsParams(UINT8 WoneIndex)
{
	return (&WhalParams_Obj[WoneIndex].WlanElmParams.WlanElm_DsParams);
}

/****************************************************************************
 *                      whal_ParamsGetElm_TIcapSupport()
 ****************************************************************************
 * DESCRIPTION:	return the TIcap info element
 ****************************************************************************/
dot11_TIcap_T  *whal_ParamsGetElm_TIcapSupport()
{
	return (&pWhalParams->WlanElmParams.WlanElm_TIcapSupport);
}

/****************************************************************************
 *                      whal_ParamsSetElm_TIcapSupport()
 ****************************************************************************
 * DESCRIPTION:	set the TIcap info element
 ****************************************************************************/
int whal_ParamsSetElm_TIcapSupport()
{
	WlanElmParams_T *pWlanElmParams = &WhalParams_Obj[0].WlanElmParams;
	UINT8			len;

	wdrv_dbGetTIInfoElement(pWlanElmParams->WlanElm_TIcapSupport.cap, &len);
	pWlanElmParams->WlanElm_TIcapSupport.hdr.eleId  = DOT11_TI_CAP_ELE_ID;
	pWlanElmParams->WlanElm_TIcapSupport.hdr.eleLen = len;

	return OK;
}

/****************************************************************************
 *                      whal_ParamsSetElm_Rates()
 ****************************************************************************
 * DESCRIPTION:	set the Rates & Ext Rates info elements
 ****************************************************************************/
int whal_ParamsSetElm_Rates(UINT8 WoneIndex)
{
	WlanElmParams_T *pWlanElmParams = &WhalParams_Obj[0].WlanElmParams;

    /* Supported rates */
    pWlanElmParams->WlanElm_SupportedRates.hdr.eleId = DOT11_SUPPORTED_RATES_ELE_ID;
    wdrv_dbGetStationConfigOperationalRateSet(WoneIndex,pWlanElmParams->WlanElm_SupportedRates.rates,
        &pWlanElmParams->WlanElm_SupportedRates.hdr.eleLen);
    
    /* Extended Supported rates */
    pWlanElmParams->WlanElm_ExtSupportedRates.hdr.eleId = DOT11_EXT_SUPPORTED_RATES_ELE_ID;
    wdrv_dbGetStationConfigOperationalOFDMRateSet(WoneIndex,pWlanElmParams->WlanElm_ExtSupportedRates.rates,
        &pWlanElmParams->WlanElm_ExtSupportedRates.hdr.eleLen);
    return OK;
}

/****************************************************************************
 *                      whal_ParamsSetElm_TIcapSupport()
 ****************************************************************************
 * DESCRIPTION:	set the TIcap info element into a given one
 ****************************************************************************/
int whal_ParamsSetElm_TIcapPrivateSupport(dot11_TIcap_T* TIcapStruct)
{
	WlanElmParams_T *pWlanElmParams = &WhalParams_Obj[0].WlanElmParams;
	
	PAL_memCopy(pWlanElmParams->WlanElm_TIcapSupport.cap,TIcapStruct->cap,TIcapStruct->hdr.eleLen);
	pWlanElmParams->WlanElm_TIcapSupport.hdr.eleId  = DOT11_TI_CAP_ELE_ID;
	pWlanElmParams->WlanElm_TIcapSupport.hdr.eleLen = TIcapStruct->hdr.eleLen;

	return OK;
}

/****************************************************************************
 *                      whal_ParamsSetElm_ERPProtection()
 ****************************************************************************
 * DESCRIPTION:	set the ERP info element
 ****************************************************************************/
int whal_ParamsSetElm_ERPProtection(UINT8 WoneIndex, UINT8 nonERPpresent, UINT8 useProtection)
{
	unsigned int		flags;
	WlanElmParams_T *pWlanElmParams = &WhalParams_Obj[WoneIndex].WlanElmParams;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */

	pWlanElmParams->WlanElm_ERP.erp &= ~(0x3);
	pWlanElmParams->WlanElm_ERP.erp |= nonERPpresent | (useProtection<<1);
	                                                                           
   	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */

	return OK;
}

/****************************************************************************
 *                      whal_ParamsSetElm_ERPLongPreambleOnly()
 ****************************************************************************
 * DESCRIPTION:	set the ERP info element
 ****************************************************************************/
int whal_ParamsSetElm_ERPLongPreambleOnly(UINT8 WoneIndex, UINT8 longPreambleOnly)
{
	unsigned int		flags;
	WlanElmParams_T *pWlanElmParams = &WhalParams_Obj[WoneIndex].WlanElmParams;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */

	pWlanElmParams->WlanElm_ERP.erp &= ~(0x4);
	pWlanElmParams->WlanElm_ERP.erp |= (longPreambleOnly<<2);
	
   	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */

	return OK;
}

/****************************************************************************
 *                      whal_ParamsResetElm_TIcapSupport()
 ****************************************************************************
 * DESCRIPTION:	reset the TIcapSupport info element
 ****************************************************************************/
int whal_ParamsResetElm_TIcapSupport()
{
	WlanElmParams_T *pWlanElmParams = &WhalParams_Obj[0].WlanElmParams;

	pWlanElmParams->WlanElm_TIcapSupport.hdr.eleId  = 0;
	pWlanElmParams->WlanElm_TIcapSupport.hdr.eleLen = 0;
	
	return OK;
}

/****************************************************************************
 *                      whal_ParamsGetElm_Tim()
 ****************************************************************************
 * DESCRIPTION:	return the TIM info element
 ****************************************************************************/
dot11_tim_T  *whal_ParamsGetElm_Tim()
{
	return (&pWhalParams->WlanElmParams.WlanElm_Tim);
}

/****************************************************************************
 *                      whal_ParamsSetElm_Tim()
 ****************************************************************************
 * DESCRIPTION:	set the TIM info element
 ****************************************************************************/
int whal_ParamsSetElm_Tim(char BmapControl, char *PartialBmapVec, int PartialBmapLen)
{
	UINT8 offset;
	unsigned int		flags;
	WlanElmParams_T *pWlanElmParams = &WhalParams_Obj[0].WlanElmParams;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */

	pWlanElmParams->WlanElm_Tim.hdr.eleId = DOT11_TIM_ELE_ID;
	pWlanElmParams->WlanElm_Tim.hdr.eleLen = 3+PartialBmapLen;
	pWlanElmParams->WlanElm_Tim.bmapControl = BmapControl;
	/* retrieve the offset: */
	/* first bit is reserved for Traffic Indicator bit */
	/* Bitmap Offset subfield of the Bitmap Control field is offset/2 */
	offset = ( BmapControl >> 1 ) * 2; 
	PAL_memCopy(pWlanElmParams->WlanElm_Tim.partialVirtualBmap, PartialBmapVec + offset, PartialBmapLen);

   	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */

#ifdef TIWLAN_DEBUG
	if (whal_trcIsEnable())
		whal_trcAddCtrl("CMD-", "Send Tim, Len=", PartialBmapLen);
#endif

	return OK;
}

/* Demiurg: WME */

dot11_WME_IE_t * whal_ParametersGetElm_WME_IE(void) {
	return &pWhalParams->WlanElmParams.WlanElm_WME_IE;
}

dot11_WME_PE_t * whal_ParametersGetElm_WME_PE(void) {
	return &pWhalParams->WlanElmParams.WlanElm_WME_PE;
}

/****************************************************************************
 *                      whal_ParamsGetElm_Rsne()
 ****************************************************************************
 * DESCRIPTION:	return the RSN info element
 ****************************************************************************/
dot11_rsne_T  *whal_ParamsGetElm_Rsne()
{
    if (pWhalParams->WlanElmParams.WlanElm_Rsne.hdr.eleLen)
		return (&pWhalParams->WlanElmParams.WlanElm_Rsne);
	else
		return NULL;
}

/****************************************************************************
 *                      whal_ParamsGetElm_Ssn()
 ****************************************************************************
 * DESCRIPTION:	return the CF info element
 ****************************************************************************/
dot11_rsne_T  *whal_ParamsGetElm_Ssn(void)
{
    if (pWhalParams->WlanElmParams.WlanElm_Ssn.hdr.eleLen)
		return (&pWhalParams->WlanElmParams.WlanElm_Ssn);
	else
		return NULL;
}

/****************************************************************************
 *                      whal_ParamsGetElm_Cf()
 ****************************************************************************
 * DESCRIPTION:	return the CF info element
 ****************************************************************************/
dot11_cfParams_T  *whal_ParamsGetElm_Cf()
{
	return (&pWhalParams->WlanElmParams.WlanElm_Cf);
}

/****************************************************************************
 *                      whal_ParamsGetMemMap()
 ****************************************************************************
 * DESCRIPTION:	get the current memory map status
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	pointer to the MemMap object
 ****************************************************************************/
AcxElm_MemoryMap_T *whal_ParamsGetMemMap()
{
	return (&pWhalParams->AcxElm_MemoryMap);
}

void whal_ParamsPrintMemMap()
{
	AcxElm_MemoryMap_T AcxElm_MemoryMap, *pMemMap;

	pMemMap = &AcxElm_MemoryMap;
    whal_acxConfigRead_MemoryMap( pMemMap ); 

	/* print the memory map */
	printf ("whal_ParamsPrintMemMap:\n");
	printf("\tCode  (0x%08x, 0x%08x)\n\tWep   (0x%08x, 0x%08x)\n\tSta   (0x%08x, 0x%08x)\n\tTmpl  (0x%08x, 0x%08x)\n\tQueue (0x%08x, 0x%08x)\n\tPool  (0x%08x, 0x%08x)\n", 
			(int)pMemMap->CodeStart, (int)pMemMap->CodeEnd, 
			(int)pMemMap->WEPCacheStart, (int)pMemMap->WEPCacheEnd, 
			(int)pMemMap->staTableStart, (int)pMemMap->staTableEnd, 
			(int)pMemMap->PacketTemplateStart, (int)pMemMap->PacketTemplateEnd, 
			(int)pMemMap->QueueMemoryStart, (int)pMemMap->QueueMemoryEnd, 
			(int)pMemMap->PacketMemoryPoolStart, (int)pMemMap->PacketMemoryPoolEnd);

    printf("\tDbg1  (0x%08x, 0x%08x)\n\tDbg2  (0x%08x, 0x%08x)\n\tTxBlocks (%d)\n\tRxBlocks (%d)\n",
           (int)pMemMap->DebugBuffer1Start,    (int)pMemMap->DebugBuffer1End,
           (int)pMemMap->DebugBuffer2Start,    (int)pMemMap->DebugBuffer2End,
           (int)pMemMap->NumTxMemBlocks,       (int)pMemMap->NumRxMemBlocks );
}

/****************************************************************************
 *                      whal_ParamsGetConfigOptions()
 ****************************************************************************
 * DESCRIPTION:	return the config options structure pointer
 *		The config options are read after reset from the ACX100 
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	pointer to the MemMap object
 ****************************************************************************/
AcxElm_ConfigOptions_T *whal_ParamsGetConfigOptions()
{
	return (&pWhalParams->AcxElm_ConfigOptions);
}

int whal_ParamsAcxPrintParams(char *buf)
{
	AcxElm_ConfigOptions_T			*pCfg = &pWhalParams->AcxElm_ConfigOptions;
    AcxElm_ConfigOptionsFixed_T		*pFix = &pWhalParams->AcxElm_ConfigOptions.ConfigOptionsFixed;
	int i,len = 0;

	/*
	 * Print the fixed field
	 */
	len +=sprintf(buf+len,"--------  ConfigOptionsFixed ----------\n");
	len +=sprintf(buf+len,"NVSVersion       = ");
	for (i=0; i<8; i++)
		len +=sprintf(buf+len,"%02x ", pFix->NVSVersion[i]);
	len +=sprintf(buf+len,"\n");

	len +=sprintf(buf+len,"ProductId        = %s\n", pCfg->ProductId.Table );
	len +=sprintf(buf+len,"ManufactureId    = %s\n", pCfg->ManufactureId.Table );
	len +=sprintf(buf+len,"dot11MacAddress  = %02x:%02x:%02x:%02x:%02x:%02x\n", pFix->dot11MacAddress[5], pFix->dot11MacAddress[4], pFix->dot11MacAddress[3], pFix->dot11MacAddress[2], pFix->dot11MacAddress[1], pFix->dot11MacAddress[0]);
	len +=sprintf(buf+len,"ProbeDelay       = %d \n", pFix->ProbeDelay );
	len +=sprintf(buf+len,"EndofMemory      = 0x%x \n", pFix->EndofMemory );
	len +=sprintf(buf+len,"dot11CCAModeSupported               = %d \n", pFix->dot11CCAModeSupported );
	len +=sprintf(buf+len,"dot11DiversitySupport               = %d \n", pFix->dot11DiversitySupport );
	len +=sprintf(buf+len,"dot11ShortPreambleOptionImplemented = %d \n", pFix->dot11ShortPreambleOptionImplemented );
	len +=sprintf(buf+len,"dot11PBCCOptionImplemented          = %d \n", pFix->dot11PBCCOptionImplemented );
	len +=sprintf(buf+len,"dot11ChanneAgilityPresent           = %d \n", pFix->dot11ChanneAgilityPresent );
	len +=sprintf(buf+len,"dot11PHYType                        = %d \n", pFix->dot11PHYType );
	len +=sprintf(buf+len,"dot11TempType                       = %d \n", pFix->dot11TempType );

	/*
	 * Print acx elements
	 */
	len +=sprintf(buf+len,"--------    ACX100 elements  ----------\n");
	len +=sprintf(buf+len,"AntennaList          (%d) = ", pCfg->AntennaList.Num );
	for (i=0; i<pCfg->AntennaList.Num; i++)
		len +=sprintf(buf+len,"0x%02x ", pCfg->AntennaList.Table[i]);
	len +=sprintf(buf+len,"\n");

	len +=sprintf(buf+len,"SupportedPowerLevels (%d) = ", pCfg->SupportedPowerLevels.Num );
	for (i=0; i<pCfg->SupportedPowerLevels.Num; i++)
		len +=sprintf(buf+len,"0x%02x ", pCfg->SupportedPowerLevels.Table[i]);
	len +=sprintf(buf+len,"\n");

	len +=sprintf(buf+len,"SupportedDataRates   (%d) = ", pCfg->SupportedDataRates.Num );
	for (i=0; i<pCfg->SupportedDataRates.Num; i++)
		len +=sprintf(buf+len,"0x%02x ", pCfg->SupportedDataRates.Table[i]);
	len +=sprintf(buf+len,"\n");

	len +=sprintf(buf+len,"SupportedRegDomains  (%d) = ", pCfg->SupportedRegDomains.Num );
	for (i=0; i<pCfg->SupportedRegDomains.Num; i++)
		len +=sprintf(buf+len,"0x%02x ", pCfg->SupportedRegDomains.Table[i]);
	len +=sprintf(buf+len,"\n");
	return len;

}
/****************************************************************************
 *                      whal_ParamsGetAcxVersion()
 ****************************************************************************
 * DESCRIPTION:	return the acx100 version structure pointer
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	pointer to the MemMap object
 ****************************************************************************/
AcxElm_AcxVersion_T *whal_ParamsGetAcxVersion()
{
	return (&pWhalParams->AcxVersion);
}

void whal_ParamsPrintFwVersion()
{
    int len = 0;
    UINT8 eepromVer[255];

    len = whal_apiGetEEPROMVersion(eepromVer,FALSE);

	printf("----------------------------------------------------------------\n");
	printf("            ACX Firmaware Version = %s\n", pWhalParams->AcxVersion.FwVersion);
    printf("            %s", eepromVer);
	printf("----------------------------------------------------------------\n");

}

/*
 * -------------------------------------------------------------------------- 
 *							Wep parameters
 * -------------------------------------------------------------------------- 
 */

/****************************************************************************
 *                      whal_ParamsInitWepParams()
 ****************************************************************************
 * DESCRIPTION:	Initiate WepParams to the default values
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
void whal_ParamsInitWepParams()
{
	WepParams_T *pWepParams = &WhalParams_Obj[0].WepParams;
	UINT16 i;
	UINT16 KeyLength;

	/*
	 * Initiate WEP parameters
	 */

	/* Default keys, and KeyId */
	for (i=0; i<WEP_NUM_DEFAULT_KEYS; i++)
	{
		wdrv_dbGetDefaultWepKey(i, &KeyLength, pWepParams->WepDefaultKey[i].Key);
		pWepParams->WepDefaultKey[i].KeySize = KeyLength;
	}
	
	wdrv_dbGetPrivacyWEPDefaultKeyId(&pWepParams->DefaultKeyId);
}

/****************************************************************************
 *                      whal_ParamsGetWepParams()
 ****************************************************************************
 * DESCRIPTION:	return pointer to the WepParams
 ****************************************************************************/
WepParams_T *whal_ParamsGetWepParams()
{
	return (&pWhalParams->WepParams);
}

int whal_ParamsGetWepDefaultKey(int KeyId, char *Key)
{
	PAL_memCopy(Key, pWhalParams->WepParams.WepDefaultKey[KeyId].Key, WEP_KEY_LEN_MAX);
	return OK;
}

int whal_ParamsGetWepDefaultKeySize(int KeyId, int *Size)
{
	*Size = pWhalParams->WepParams.WepDefaultKey[KeyId].KeySize;
	return OK;
}

int whal_ParamsGetWepDefaultKeyId(int *KeyId)
{
	*KeyId = pWhalParams->WepParams.DefaultKeyId;
	return OK;
}

/*
 * -------------------------------------------------------------------------- 
 *							NV parameters
 * -------------------------------------------------------------------------- 
 */

/****************************************************************************
 *                      whal_ParamsInitNvParams()
 ****************************************************************************
 * DESCRIPTION:	Get NV params
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	
 ****************************************************************************/

int UglyHackGlobalConfiguredFWNumber;

void whal_ParamsInitNvParams()
{
	/* init default values to */
	WhalParams_Obj[0].NvParams.Magic[0] = 'H';
	WhalParams_Obj[0].NvParams.Magic[1] = '1';

	WhalParams_Obj[0].NvParams.TraceEnable      = 0 ;
	WhalParams_Obj[0].NvParams.PlcpPatch        = 0;
	WhalParams_Obj[0].NvParams.FwNumber         = UglyHackGlobalConfiguredFWNumber;
	WhalParams_Obj[0].NvParams.QueueSizesType   = 0;
	WhalParams_Obj[0].NvParams.LoadRadio		= 0;
	WhalParams_Obj[0].NvParams.RadioNumber		= 0;
	WhalParams_Obj[0].NvParams.UseIntrThreshold = 0;
	WhalParams_Obj[0].NvParams.TraceOut		= 0;
	WhalParams_Obj[0].NvParams.TraceFile		= 0;

	WhalParams_Obj[0].NvParams.ProbePatch		= 0;
	WhalParams_Obj[0].NvParams.ProbePatch_Rate  = 0;
	WhalParams_Obj[0].NvParams.ProbePatch_Caps  = 0;
	WhalParams_Obj[0].NvParams.ProbePatch_Spare = 0;

	WhalParams_Obj[0].NvParams.BeaconPatch      = 0;
	WhalParams_Obj[0].NvParams.BeaconPatch_Rate = 0;
	WhalParams_Obj[0].NvParams.BeaconPatch_Caps = 0;
	WhalParams_Obj[0].NvParams.BeaconPatch_Spare= 0;

	WhalParams_Obj[0].NvParams.PcfPatch         = 0;
	WhalParams_Obj[0].NvParams.PcfPatch_Caps    = 0;
	WhalParams_Obj[0].NvParams.PcfPatch_Spare[0]= 0;
	WhalParams_Obj[0].NvParams.PcfPatch_Spare[1]= 0;
}

/****************************************************************************
 *                      whal_ParamsGetNvParams()
 ****************************************************************************
 * DESCRIPTION:	return pointer to the NvParams
 ****************************************************************************/
NvParams_T *whal_ParamsGetNvParams()
{
	return (&pWhalParams->NvParams);
}

#ifndef NO_PBCC_CHECK
/****************************************************************************
 *                      whal_ParamsGetNv_ProbeRespSupportedRates()
 ****************************************************************************
 * DESCRIPTION:	return the Supported rates info element for Probe response
 ****************************************************************************/
dot11_supRates_T  *whal_ParamsGetNv_ProbeRespSupportedRates()
{
	dot11_supRates_T *pRates = &WhalParams_Obj[0].WlanElmParams.WlanElm_SupportedRates_Probe;
	int i;

	pRates->hdr.eleId = DOT11_SUPPORTED_RATES_ELE_ID;
	for (i=0; i<8; i++)
	{
		if (pWhalParams->NvParams.ProbePatch_SuppRates[i] != 0)
			pRates->rates[i] = pWhalParams->NvParams.ProbePatch_SuppRates[i];
		else
		{
			pRates->hdr.eleLen = i;
			break;
		}
	}

	return (pRates);
}

/****************************************************************************
 *                      whal_ParamsGetNv_BeaconSupportedRates()
 ****************************************************************************
 * DESCRIPTION:	return the Supported rates info element for Beacon
 ****************************************************************************/
dot11_supRates_T  *whal_ParamsGetNv_BeaconSupportedRates()
{
	dot11_supRates_T *pRates = &WhalParams_Obj[0].WlanElmParams.WlanElm_SupportedRates_Beacon;
	int i;

	pRates->hdr.eleId = DOT11_SUPPORTED_RATES_ELE_ID;
	for (i=0; i<8; i++)
	{
		if (pWhalParams->NvParams.BeaconPatch_SuppRates[i] != 0)
			pRates->rates[i] = pWhalParams->NvParams.BeaconPatch_SuppRates[i];
		else
		{
			pRates->hdr.eleLen = i;
			break;
		}
	}

	return (pRates);
}


#endif

/****************************************************************************
 *                      whal_ParamsSetNv()
 ****************************************************************************
 * DESCRIPTION:	Set NV params
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	
 ****************************************************************************/
void whal_ParamsSetNv(int Offset, UINT8 Val)
{
	((char *)&WhalParams_Obj[0].NvParams)[Offset] = Val;
}


/****************************************************************************
 *                      MAC Address functions
 ****************************************************************************
 * 
 ****************************************************************************/
int genMacCompare(char *Mac1, char *Mac2)
{
	register int MacSize=6;
	/* (!!!) improve it by comparing 16 bit words */
	while(MacSize--)
		if (*Mac1++ != *Mac2++)
			return NOK;
	return OK;
}


/****************************************************************************
 *                      whal_printHalParams()
 ****************************************************************************
 * DESCRIPTION:	print hal params.
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	
 ****************************************************************************/
void whal_printHalParams( void ) {
	printf("FragmentThreshold    : %10d \n", WhalParams_Obj[0].GenParams.FragmentThreshold );
	printf("RtsThreshold         : %10d \n", WhalParams_Obj[0].GenParams.RtsThreshold	);
	printf("ShortRetryLimit      : %10d \n", WhalParams_Obj[0].GenParams.ShortRetryLimit );
	printf("LongRetryLimit       : %10d \n", WhalParams_Obj[0].GenParams.LongRetryLimit );
	printf("BeaconInterval       : %10d \n", WhalParams_Obj[0].GenParams.BeaconInterval );
}
/****************************************************************************
 *                      whal_ParamsSetBssType()
 ****************************************************************************
 * DESCRIPTION:	Set BSS type.
 * 
 * INPUTS:	bssType - AP or infrastructure
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	
 ****************************************************************************/

void whal_ParamsSetBssType(UINT32 bssType)
{
	GenParams_T *pGenParams = &WhalParams_Obj[0].GenParams;
	UINT8 WoneIndex, TotalWones;
    wdrv_dbGetNumSupWone(&TotalWones);
	
    for (WoneIndex = 0; WoneIndex < TotalWones; WoneIndex++) {	
        pGenParams = &WhalParams_Obj[WoneIndex].GenParams;
        pGenParams->BssType	= bssType;
    }
}




