/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2004 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/
/***************************************************************************
 *
 *   MODULE:  whalTxDesc.c
 *   PURPOSE: WLAN chip Tx descriptor object - functional implementation
 *   
 ***************************************************************************/


/****************************
 * HAL Include files
 ****************************/
#include "acx_hw.h"
#include "whalCommon.h"
#include "AcxDataIf.h"
#include "whalAcxAccess.h"
#include "whalAcxRegs.h"
#include "whalTxMemCalc.h"
#include "whalTxDesc.h"
#include "AcxCommands.h"
#include "whalHostDesc.h"

/***************************
 * Function Implementation
 ***************************/

/****************************************************************************
 *                      whal_txDesc_Init()
 ****************************************************************************
 * DESCRIPTION:
 * Initializes WLAN chip Tx desctiptor, setting OWN and FIRST FRAG bits.
 * The "Next" field is not set because it is already initialized by FW to
 * point to the next descriptor in the WLAN chip Tx queue.
 * 
 * INPUT :  None
 * OUTPUT:  TxDesc_T *pTxDesc = Pointer of descriptor to process
 *
 * RETURN:  None
 ****************************************************************************/
void whal_txDesc_Init(TxDesc_T *pTxDesc)
{
   TxDesc_T localTxDesc;

   /* Initialize all fields to 0 */
   PAL_memSet(&localTxDesc, 0, sizeof(localTxDesc));

   /* Override only what is necessary. */
   localTxDesc.Ctl         = CTL_OWN;
   localTxDesc.Rate        = BIT_RATE_1MBPS;

   /* Skip Next field and copy */
   whal_acxMemCopy(4+(char *)pTxDesc, 4+(char *)&localTxDesc, sizeof(TxDesc_T)-4);
}/* END whal_txDesc_Init() */

/****************************************************************************
 *                      whal_txDesc_reinit()
 ****************************************************************************
 * DESCRIPTION:
 * Reinitializes WLAN chip Tx desctiptor, by setting OWN and FIRST FRAG bits.
 * 
 * INPUT :  None
 * OUTPUT:  TxDesc_T *pTxDesc = Pointer of descriptor to process
 * 
 * RETURN:  None
 ****************************************************************************/
void whal_txDesc_reinit(TxDesc_T *pTxDesc)
{
   /* Override only what is necessary */
   whal_acxMemSetU16( (UINT32)(&(pTxDesc->Ctl)) , CTL_OWN);
   whal_acxMemSetU16( (UINT32)(&(pTxDesc->Rate)), BIT_RATE_1MBPS);
   
   /* Place 0's in fields we do not use for setMpduInfo, for safety */
   whal_acxMemClrU32(&pTxDesc->ACXMemPtr);
   
   /* clear all status / failure bits */
   whal_acxMemClrU16(&pTxDesc->Status);
   whal_acxMemClrU16(&pTxDesc->RTSFailures);
   /* clear unused reserved */
}/* END whal_txDesc_reinit() */


/****************************************************************************
 *                      whal_txDesc_SetMpduInfoMaster()
 ****************************************************************************
 * DESCRIPTION:
 * Sets the descriptor's fileds before transmission in Master mode.
 *
 *    FIRST_FRAG  - set only in the first fragment
 *    MORE_FRAG   - set in all fragments except the last fragment
 *    MpduLen     - The size of the MPDU
 *    AcxMemPtr   - The pointer to where the data placed in WLAN chip SRAM
 * 
 * INPUT :
 *    UINT32    hostDescAddr  = Address of host descriptors in shared memory
 *    int       MpduLen       = Length of MPDU in bytes.
 *    UINT8     MpduNumMemBlks= Number of memory blocks required for the MPDU
 *    UINT16    FragThreshold = Fragmentation threshold.
 *    UINT16    Rate          = Rate required for the transmission 
 *    UINT32    MpduFlags     = MPDU flags, setting transmission attributes.
 *    char      *DstMac       = Destination MAC address.
 *    UINT32    lifetime      = Life time of the packet.
 * 
 * OUTPUT:
 *    TxDesc_T *pTxDesc =  Pointer of WLAN chip Tx descriptor to process
 * 
 * RETURN:  int: 0 if OK
 ****************************************************************************/
int whal_txDesc_SetMpduInfoMaster
(
   TxDesc_T *pTxDesc,
   UINT32    hostDescAddr,
   int       MpduLen,
   UINT8     MpduNumMemBlks,
   UINT16    FragThreshold,
   UINT16    Rate,
   UINT32    MpduFlags,
   char      *DstMac,
   UINT32    lifetime
)
{
   /* Declare local TxDesc */
   register UINT8 Ctl2 = 0;

   /* enable burst in case there is no fragmentation */
    if ((MpduFlags & MPDU_FLAGS_TO_BE_FRAGMENTED) == 0 )
      Ctl2 |= CTL2_BURST;

    /* Check Mpdu flags */
   if (MpduFlags & MPDU_FLAGS_RTS_SET) 
      Ctl2 |= CTL2_RTS;
   
   if (MpduFlags & MPDU_FLAGS_ERP_PROTECTION_SET) 
      Ctl2 |= CTL2_ERP;

#if 0 /* No support */  
   if (MpduFlags & MPDU_FLAGS_TI_PROTECTION_SET) 
      Ctl2 |= CTL2_TI;
#endif

   /* Demiurg: WME */
   if (MpduFlags & MPDU_FLAGS_QOS) 
      Ctl2 |= CTL2_QOS;

   	if( MpduFlags & MPDU_FLAGS_QOS_NO_ACK_POLICY)
      Ctl2 |= CTL2_NO_ACK;


    if (TEST_MULTICAST_ADDRESS(DstMac))
    {
        Ctl2 |= CTL2_BCAST;
    }

   whal_acxMemSetByte((UINT32)pTxDesc, TX_CTL2_BYTE_OFFSET, Ctl2);

   if (MpduFlags & MPDU_FLAGS_SHORT_PREAMBLE_SET) 
      Rate |= 0x8000;

   whal_acxMemSetU16( &pTxDesc->Rate, Rate);

   /* save the Rate also in the workspace for later checking */
    *(UINT16 *)(&((HostDesc_T *)(hostDescAddr))->DriverWorkspace[0]) = Rate;
    *(UINT16 *)(&((HostDesc_T *)(hostDescAddr))->DriverWorkspace[2]) = *(UINT16 *)(&DstMac[0]);
    *(UINT16 *)(&((HostDesc_T *)(hostDescAddr))->DriverWorkspace[4]) = *(UINT16 *)(&DstMac[2]);
    *(UINT16 *)(&((HostDesc_T *)(hostDescAddr))->DriverWorkspace[6]) = *(UINT16 *)(&DstMac[4]);

   /* clear ant. diversity field */
   whal_acxMemClrU8((UINT32)pTxDesc + TX_ANT_DIVERSITY_OFFSET);

   whal_acxMemSetU16(&pTxDesc->MPDULen, (UINT16)MpduLen);


   /* Host mem pointer points to the HOST SDRAM. The pointer is converted for the ACX */
#if( WLAN_BUS_TYPE == WLAN_BUS_TYPE_VLYNQ)
   whal_acxMemSetU32( &pTxDesc->HostMemPtr, VLYNQ_HOST2ACX111_SDRAM_PTR(hostDescAddr));
#elif( WLAN_BUS_TYPE == WLAN_BUS_TYPE_PCI)
   whal_acxMemSetU32( &pTxDesc->HostMemPtr, PCI_HOST2ACX111_SDRAM_PTR(hostDescAddr));
#else
#error   whalTxDesc.c -- Check bus type.
#endif

    {
        TxDesc_T   *pTxDescOffset = 0; /* Used for offset calculation !!! */
        UINT32      CurrentTimestamp;
        UINT16      Duration;

        CurrentTimestamp = whal_acxRegRead( ACX_MAC_TIMESTAMP_REG )/*wdrv_QoSGetTimestamp()*/;
        CurrentTimestamp+= lifetime; /* 300000;  300 msec timeout */

        whal_acxMemSetU32( &pTxDesc->ExpiryTime, CurrentTimestamp );


        whal_acxMemSetU16( &pTxDesc->FragThreshold, FragThreshold );
        whal_acxMemSetByte((UINT32)pTxDesc, (int)&pTxDescOffset->NumMemBlocks , MpduNumMemBlks);

        Duration = TxMemCalcComputeDuration((UINT16)MpduLen, Rate, ((MpduFlags & MPDU_FLAGS_QOS_NO_ACK_POLICY) != 0));

        whal_acxMemSetU16( &pTxDesc->DurationInAir, Duration );

    }

   return 0;
}/* END whal_txDesc_SetMpduInfoMaster() */

/****************************************************************************
 *                      whal_txDesc_GetStatus()
 ****************************************************************************
 * DESCRIPTION:
 * Retrieves the "Status" fields from the WLAN chip Tx descriptor.
 *
 * INPUT :  TxDesc_T *pTxDesc =  Pointer of WLAN chip Tx descriptor
 * OUTPUT:  UINT8 *AckFail = Pointer to "AckFail" field
 *          UINT8 *RtsFail = Pointer to "RtsFail" field
 *          UINT8 *RtsSucc = Pointer to "RtsSucc" field
 * 
 * RETURN:  UINT8: status byte
 ****************************************************************************/
UINT8 whal_txDesc_GetStatus(TxDesc_T *pTxDesc, UINT8 *AckFail, UINT8 *RtsFail, UINT8 *RtsSucc)
{
   register UINT16 StatusWord = whal_acxMemGetU16( (UINT32)pTxDesc + TX_STATUS_WORD_OFFSET);
   register UINT16 RtsCounters = whal_acxMemGetU16( (UINT32)pTxDesc + TX_STATUS_WORD_OFFSET+2);

   *AckFail = StatusWord & 0xff;
   *RtsFail = RtsCounters >> 8;
   *RtsSucc = RtsCounters & 0xff;

   /* Return status byte */
   return( (UINT8)(StatusWord >> 8));
}/* END whal_txDesc_GetStatus() */

/****************************************************************************
 *                      whal_txDesc_GetRate()
 ****************************************************************************
 * DESCRIPTION: Retrieves current value of "Rate" field of the Tx descriptor.
 *
 * INPUT :  TxDesc_T *pTxDesc =  Pointer of WLAN chip Tx descriptor
 * OUTPUT:  None
 *
 * RETURN:  UINT16: "Rate" field
 ****************************************************************************/
UINT16 whal_txDesc_GetRate(TxDesc_T *pTxDesc)
{
   return ((UINT16)whal_acxMemGetU16(&pTxDesc->Rate));
}/* END whal_txDesc_GetRate() */

/****************************************************************************
 *                      whal_txDesc_GetCtl()
 ****************************************************************************
 * DESCRIPTION: Retrieves current value of "Ctl" field of the Tx descriptor.
 *
 * INPUT :  TxDesc_T *pTxDesc =  Pointer of WLAN chip Tx descriptor
 * OUTPUT:  None
 *
 * RETURN:  UINT8: "Ctl" field
 ****************************************************************************/
UINT8 whal_txDesc_GetCtl(TxDesc_T *pTxDesc)
{
   return(whal_acxMemGetByte((UINT32)pTxDesc, TX_CTL_WORD_OFFSET));
}/* END whal_txDesc_GetCtl() */

/****************************************************************************
 *                      whal_txDesc_SetCtl()
 ****************************************************************************
 * DESCRIPTION:   Sets the "Ctl" field in a WLAN chip Tx descriptor.
 *
 * INPUT :  TxDesc_T *pTxDesc =  Pointer of WLAN chip Tx descriptor
 *          UINT8 Val         =  Value to set
 * OUTPUT:  None
 *
 * RETURN:  None
 ****************************************************************************/
void whal_txDesc_SetCtl(TxDesc_T *pTxDesc, UINT8 Val)
{
   whal_acxMemSetByte((UINT32)pTxDesc, TX_CTL_WORD_OFFSET, Val);
}/* END whal_txDesc_SetCtl() */

/****************************************************************************
 *                      whal_txDesc_Read()
 ****************************************************************************
 * DESCRIPTION:
 * Copies Rx descriptor from WLAN chip into a buffer in Host's memory,
 * swapping relevant fields.
 * 
 * INPUT :  TxDesc_T *pSrcDesc   = Pointer of WLAN chip Tx descriptor
 * OUTPUT:  TxDesc_T *pDstDesc   = Pointer to destination buffer in the Host
 * 
 * RETURN:  int: OK
 ****************************************************************************/
int whal_txDesc_Read(TxDesc_T *pDstDesc, TxDesc_T *pSrcDesc)
{
   whal_acxMemCopy( (char *)pDstDesc, (char *)pSrcDesc, sizeof(TxDesc_T));

   /* 
    * Convert only 16 and 32 bit fields 
    * (!!!) can copy only relevat fields
    */
   pDstDesc->next        = (UINT8 *)(ENDIAN_HANDLE_LONG((int)(pDstDesc->next)));
   pDstDesc->HostMemPtr  = (UINT8 *)(ENDIAN_HANDLE_LONG((int)(pDstDesc->HostMemPtr)));
   pDstDesc->ACXMemPtr   = (UINT32 *)(ENDIAN_HANDLE_LONG((int)(pDstDesc->ACXMemPtr)));
   pDstDesc->MPDULen     = ENDIAN_HANDLE_WORD(pDstDesc->MPDULen);

   return OK;
}/* END whal_txDesc_Read() */

/****************************************************************************
 *                      whal_txDesc_ResetCtlOwn()
 ****************************************************************************
 * DESCRIPTION:
 * Resets OWN bit in the "Ctl" field on the Tx descriptor
 *
 * INPUT :  TxDesc_T *pTxDesc =  Pointer of WLAN chip Tx descriptor
 * OUTPUT:  None
 *
 * RETURN:  None
 ****************************************************************************/
void whal_txDesc_ResetCtlOwn(TxDesc_T *pTxDesc)
{
   whal_acxMemResetByteBits((UINT32)pTxDesc, TX_CTL_BYTE_OFFSET, CTL_OWN);
}/* END whal_txDesc_ResetCtlOwn() */

/****************************************************************************
 *                      whal_txDesc_GetHostNext()
 ****************************************************************************
 * DESCRIPTION:
 * Retrieves the pointer of the "Next" WLAN chip Tx descriptor in the ring
 * buffer maintained by the WLAN FW.
 *
 * INPUT :  TxDesc_T *pTxDesc =  Pointer of WLAN chip Tx descriptor
 * OUTPUT:  None
 *
 * RETURN:  TxDesc_T *: Pointer to next descriptor.
 ****************************************************************************/
TxDesc_T *whal_txDesc_GetHostNext(TxDesc_T *pTxDesc)
{
   return ( (TxDesc_T *) (whal_acxMemGetU32(&pTxDesc->next) + ACX100_MEM_ADDRESS) );
}/* END whal_txDesc_GetHostNext() */

/****************************************************************************
 *                      whal_txDesc_SetHostNext()
 ****************************************************************************
 * DESCRIPTION:
 * Sets the host address in the "Next" WLAN chip Tx descriptor.
 *
 * INPUT :  TxDesc_T *pTxDesc = Pointer of current WLAN chip Tx descriptor
 *          UINT32 HostAddr   = Address of host descriptor.
 * OUTPUT:  None
 *
 * RETURN:  None
 ****************************************************************************/
void whal_txDesc_SetHostNext(TxDesc_T *pTxDesc, UINT32 HostAddr)
{
   whal_acxMemSetU32(&pTxDesc->next, HostAddr - ACX100_MEM_ADDRESS);
}/* END whal_txDesc_SetHostNext() */

/****************************************************************************
 *                      whal_txDesc_GetAcxBufAddr()
 ****************************************************************************
 * DESCRIPTION: Returns the pointer to the frame data in the host memory
 *
 * NOTE:        The Tx Descriptor MUST be an image in SDRAM of the TxDesc 
 *              in SRAM
 *
 * INPUT :  TxDesc_T pTxDesc = WLAN chip Tx descriptor
 * OUTPUT:  None
 *
 * RETURN:  UINT32 : Pointer to frame data in the host memory.
 ****************************************************************************/
UINT32 whal_txDesc_GetAcxBufAddr(TxDesc_T pTxDesc)
{
   /* MASTER MODE */
    UINT32 HostMemPtr,BlkPtr;

#if( WLAN_BUS_TYPE == WLAN_BUS_TYPE_VLYNQ)
    HostMemPtr = VLYNQ_ACX111_TO_HOST_SDRAM_PTR_CACHED(pTxDesc.HostMemPtr);
    BlkPtr = (UINT32)VLYNQ_ACX111_TO_HOST_SDRAM_PTR_CACHED(((HostDesc_T *)HostMemPtr)->blk_ptr);
    return BlkPtr;

#elif( WLAN_BUS_TYPE == WLAN_BUS_TYPE_PCI)
    HostMemPtr = PCI_ACX111_TO_HOST_SDRAM_PTR_UNCACHED(pTxDesc.HostMemPtr);
    BlkPtr = (UINT32)PCI_ACX111_TO_HOST_SDRAM_PTR_CACHED(((HostDesc_T *)HostMemPtr)->blk_ptr);
    return BlkPtr;
#else
   #error   whalHostDesc.c -- Check bus type.
#endif   /* WLAN_BUS_TYPE */

}/* END whal_txDesc_GetAcxBufAddr() */

/****************************************************************************
 *                      whal_txDesc_GetHostMemPtr()
 ****************************************************************************
 * DESCRIPTION:
 * Returns the pointer of the host descriptor pointed to by the WLAN chip
 * Tx descriptor.
 *
 * INPUT :  TxDesc_T pTxDesc = WLAN chip Tx descriptor
 * OUTPUT:  None
 *
 * RETURN:  UINT32 : Pointer to the host descriptor.
 ****************************************************************************/
UINT32 whal_txDesc_GetHostMemPtr(TxDesc_T *pTxDesc)
{
    return (whal_acxMemGetU32(&pTxDesc->HostMemPtr));
}/* END whal_txDesc_GetHostMemPtr() */

/****************************************************************************
 *                      whal_txDesc_Print()
 ****************************************************************************
 * DESCRIPTION:   Prints descriptor's main fields into a buffer.
 *
 * INPUT :  TxDesc_T pTxDesc = WLAN chip Tx descriptor
 * OUTPUT:  char *buf =  Pointer to output buffer, must be large enough.
 *
 * RETURN:  int : Total length of output string
 ****************************************************************************/
int whal_txDesc_Print(TxDesc_T *pHwTxDesc ,char *buf)
{
   TxDesc_T localTxDesc;
   whal_acxMemCopy( (char *)&localTxDesc, (char *)pHwTxDesc, sizeof(TxDesc_T));

   localTxDesc.next        = (UINT8 *)(ENDIAN_HANDLE_LONG((int)(localTxDesc.next)));
   localTxDesc.ACXMemPtr   = (UINT32 *)(ENDIAN_HANDLE_LONG((int)(localTxDesc.ACXMemPtr)));
   localTxDesc.MPDULen     = ENDIAN_HANDLE_WORD(localTxDesc.MPDULen);

   return sprintf( buf,
      "AcxMemPtr=0x%04x, Next=0x%04x, MpduLen=0x%04x, Ctl=0x%02x:0x%02x, Stt=0x%02x\n",
      (UINT32)localTxDesc.ACXMemPtr, (UINT32)localTxDesc.next, localTxDesc.MPDULen,
      localTxDesc.Ctl, localTxDesc.Ctl2, localTxDesc.Status);
}/* END whal_txDesc_Print() */


