/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2004 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/
/****************************************************************************
 *
 *   MODULE : whalTxMaster.c
 *   PURPOSE: Manages the transmission of MSDU's to the WLAN chip when
 *            configured to "Master Mode".
 *   
 ****************************************************************************/

/****************************
 * HAL Include files
 ****************************/
#include "wlan_pal.h"
#include "acx_hw.h"
#include "whalCommon.h"
#include "whal.h"
#include "whalTxQueue.h"
#include "whalTxBufs.h"
#include "whalInterrupt.h"
#include "whalAcxAccess.h"
#include "whalParams.h"
#include "memMngr.h"
#include "whalTx.h"
#include "whalTxMemCalc.h"
#include "whalTrace.h"
#include "whalHostDesc.h"
#include "whalHostQueue.h"
#include "whalInterrupt.h"
#include "QoS/QoSClassifier.h"
#include "drvDbMain.h"
#include "dda.h"

#define TKIP_TAIL_BYTES 4
#define AES_TAIL_BYTES  8

/**********************
 * External Functions
 **********************/
extern UINT32 whal_txQueue_GetNumUsed(TxQueueId_T Qid);

/****************************************************************************
 *                      OBJECT DATA BASE + INTERNALS
 ****************************************************************************
 *
 ****************************************************************************/
TxStatus_Callback_T TxStatusCB=NULL;
TxDataUser_Callback_T TxDataUserCB=NULL;
Recv4xWackInfo_Callback_T Recv4xWackInfoCB=NULL;
UINT8 LastRecv4xWackInfo = 0xaa;
int TxFrameCounter [NUM_TX_QUEUES];
HostQueue_Obj   *txHostQueueArray[NUM_TX_QUEUES];


/***************************
 * Function Implementation
 ***************************/

/****************************************************************************
 *                      whal_txSendMsdu()
 ****************************************************************************
 * DESCRIPTION: Fragments MSDU into MPDUs and Sends (Copies) to the WLAN chip.
 * 
 *    For each fragment, the function allocates WLAN chip buffers from the 
 *		TxBufs object to copy the MPDU data blocks and allocate buffer 
 *		descriptors from the TxQueue object to point to the buffers and set 
 *    the appropriate bits to inform the WLAN chip that the MPSU is ready for
 *		transmission.
 *    The function trigger TX_PROC interrupt to the WLAN chip only after all 
 *    the MPDUs are ready in the WLAN chip memory.
 *    The function also set the Queue indicator filed to inform the WLAN chip
 *		in which queue there is a bufferrs to process.
 *		Broadcast frames are not fragmented
 *
 * INPUT :  
 *    TxQueueId_T TxQid = The queue id
 *    MEM_HANDLE *Msdu_Handle =
 *			The MSDU descriptor which contain Header block and link list of 
 *			Data blocks. The Header block contain the 802.11 header and can 
 *			have more data (LLC+SNAP for data frame or Management frame body
 *			for short frames. Each data block has offset filed which indicates
 *			where the data is actually started.
 *    TxAttr_T *pTxAttr = Pointer to Tx attributes
 * 
 * OUTPUT:	None
 * 
 * RETURN:  int: OK or NOK.
 ****************************************************************************/
int whal_txSendMsdu(TxQueueId_T TxQid, MEM_HANDLE Msdu_Handle, TxAttr_T *pTxAttr)
{
   TxDesc_T       *pTxDesc;
   mem_BD_T       *pBD;
   UINT32          totalSize, hostDescAddr;
   HostDescAttr_T  hostDscAttr;
   HostQueue_Obj  *pTxHostQ;
   dot11_header_T *pMacHdr;
   UINT32 	       msduSize, MpduSize;
   int             NumFrags;
   UINT32          MpduFlags=0;
   unsigned int    flags;
   mem_BD_T		   *firstBDptr;
   UINT32          moreBytes = 0;
   UINT16          fc;

   uint8                            Fragmented      = 0, MpduNumMemBlks = 0;
   uint16                           FragThreshold   = 0, txOpLimit = 0;
   FragmentationDecisionData_t      FragData;

   pTxHostQ =  txHostQueueArray[TxQid];
   if( Msdu_Handle == NULL ){
      
       printf("whal_txSendMsdu() -- ERROR! Msdu_Handle = NULL\n");
       return NOK;
   }
   PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags);  /* Begin critical section */

   /* Check if there are free descriptors */
   if(pTxHostQ->NumOfDesc - pTxHostQ->CurrNumDescInQ < 2)
   {
      PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);   /* End critical section */
	   printf("whal_txSendMsdu() - No free tx host descriptors\n");
	   return NOK;
   }

   /*
    * Allocate WLAN chip Tx descriptor
    */
   pTxDesc = whal_txQueue_Alloc(TxQid);
   if (pTxDesc == NULL)
   {
    /* This should never happen because we request for resources before we decide to send
      * but if for some reason there is no WLAN chip descriptor then we must free the MSDU and 
      * return.
      */
      PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);   /* End critical section */
      printf("whal_txSendMsdu: ERROR, no more AcxDesc descriptors ...\n");  
      return NOK;
   }

   /* set ssid index */
   pTxDesc->ssidProfile =  pTxAttr->ssid_index;
   
   /* Swap the frame control and the sequence control */
   pMacHdr = (dot11_header_T *)memMgr_MsduHdrAddr(Msdu_Handle);

   fc = pMacHdr->fc;
   pMacHdr->fc = ENDIAN_HANDLE_WORD(pMacHdr->fc);
   /*
    *  Insert every buffer to a host descriptor.
    */
   msduSize = memMgr_MsduDataSize( Msdu_Handle );

   pBD =  memMgr_MsduDataAddr( Msdu_Handle );
   if( pBD == NULL ){
      PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);    /* End critical section */
        printf("whal_txSendMsdu() -- ERROR! pBD = NULL\n");
        return NOK;
    }

   totalSize = 0;
   
   hostDescAddr = whal_HostQueue_getNextToInsertDescAddr( pTxHostQ);
   hostDscAttr.msduPtr = (UINT32) Msdu_Handle;

   NumFrags = 1;

   if ((fc & DOT11_FC_TO_DS) && (fc & DOT11_FC_FROM_DS))
   {
	   FragData.mpduHeaderLength    = sizeof(dot11_header_T);
   }
   else
   {
	   FragData.mpduHeaderLength    = sizeof(dot11_AP_header_T);
   }

   /* need this setting here for WPA padding calculation */
   if (pTxAttr->QoS)
	   FragData.mpduHeaderLength += WLAN_QOS_HDR_LEN;

   
   /* In case of NULL data don't add the padding for data encryption */
   if (FragData.mpduHeaderLength != msduSize)
   {
       /* =============================================== */
	   /* adding 4 bytes for IV after mac header          */
	   /* In case of AES - insert 8 bytes padding for MIC */
	   /* =============================================== */
	   
	   if((pTxAttr->CipherType == CIPHER_TKIP) || 
		  (pTxAttr->CipherType == CIPHER_CCMP))  
	   {
		   int paddingSize = 0;
		   
		   if (pTxAttr->CipherType == CIPHER_TKIP)
		   {
			   paddingSize = WPA_IV_SIZE;
		  /* adjust RTS threshold offset taking into consideration 4 tailing bytes added by Firmware */
			 moreBytes = paddingSize + TKIP_TAIL_BYTES;
		   }
		   else /* AES */
		   {
			   paddingSize = MIC_LEN;
		  /* adjust RTS threshold offset taking into consideration 8 tailing bytes added by Firmware */
			 moreBytes = paddingSize + AES_TAIL_BYTES;
		   }
	
		   if( pBD->data == NULL ) /* Just to make sure. */
			  pBD->data = pBD->dataBuf->data;
	
	
		  /* Copy data with padding size offset, increasing BD length  */
		  {
			 UINT32  i;
			 char   *pTmpSrc;
			 char   *pTmpDst;
			 UINT32  limit;
	
			 limit   = pBD->length - FragData.mpduHeaderLength;
			 pTmpSrc = pBD->data + pBD->dataOffset + pBD->length - 1;
			 pTmpDst = pTmpSrc + paddingSize;
		  
			 /* copy data  with IV_SIZE offset */
			 for( i=0; i < limit; i++ /*,pTmpSrc,pTmpDst*/)
				*pTmpDst-- = *pTmpSrc--;
		  
			 /* pad IV with 0 */
			 for( i=0; i<paddingSize; i++)
				*pTmpDst-- = 0;
		  }
		  /* increase BD length */
		  pBD->length += paddingSize;
		  /* increase msdu size */
		  memMgr_MsduDataSize( Msdu_Handle ) += paddingSize;
	   }/* endIf */
   }
   
   /* =============================================== */
   
   firstBDptr = pBD;

    do {
      if( pBD->data == NULL ) /* Just to make sure. */
         pBD->data = pBD->dataBuf->data;

      hostDscAttr.blk_ptr    = (UINT32)( pBD->data) + pBD->dataOffset;
      hostDscAttr.blk_offset = 0;
      hostDscAttr.ctrl       = 0;
			hostDscAttr.blk_len = pBD->length;
      hostDscAttr.status     = 0;

      whal_HostQueue_insert( pTxHostQ, &hostDscAttr );
      /*
       * Only the first descriptor has to keep MSDU pointer,
       * for the "free" callback.
       */
      hostDscAttr.msduPtr = 0;
      totalSize +=  pBD->length;

	  PAL_cacheFlush( 0, (void *)hostDscAttr.blk_ptr, hostDscAttr.blk_len);

      pBD = pBD->nextBDPtr;
      /*
       * TODO: Check up and clean the mess in case of mismatch.
       * Exception hazard, or infinite loop. Report if something gets wrong.
       */
   } while (pBD);

   MpduSize = totalSize;

   {
       UINT16   txFragThreshold;
			 
			 txOpLimit = whal_txQueueGetTxOpLimit(TxQid);
       wdrv_dbGetOperationFragmentationThreshold(&txFragThreshold);

       FragData.dataLength              = totalSize;
       FragData.driverFragmentationOn   = FALSE;
       FragData.encryptionType          = pTxAttr->CipherType;
       FragData.memBlkSize              = 252;
       FragData.unicastFrame            = TEST_UNICAST_ADDRESS(pMacHdr->address1.addr);
       FragData.txRate                  = (UINT32) pTxAttr->Rate;
       FragData.txCtlRate               = whal_ParamsGetGenParams(0)->ACX100_GenTxRate;
       FragData.txCts                   = (pTxAttr->ProtectionType == TXATTR_PROTECTION_CTS_TOSELF);
       FragData.txRts                   = (pTxAttr->ProtectionType == TXATTR_PROTECTION_RTS_CTS);
       FragData.tx4X                    = FALSE;
       FragData.txNoAck                 = pTxAttr->ackPolicy;
       FragData.txOpLimit               = txOpLimit;
       FragData.originalFragThreshold   = txFragThreshold;
      
       /* In case of NULL data, no need to calculate # of Memoey Blocks to alloccate. */
       if (FragData.mpduHeaderLength != msduSize)
       {
           NumMemBlocksTxAllocation( &Fragmented, &MpduNumMemBlks, &FragThreshold, &FragData);
       }
       else
       {
           MpduNumMemBlks = 1;
       }
   }

   /*
    * prepare Mpdu flags
    */
   if (txOpLimit)
   {
       MpduFlags |= MPDU_FLAGS_TX_OP_ENABLED;
   }
   
   if (Fragmented)
   {
       MpduFlags |= MPDU_FLAGS_TO_BE_FRAGMENTED;
   }

   if (pTxAttr->ShortPreamble)
      MpduFlags |= MPDU_FLAGS_SHORT_PREAMBLE_SET;

   /* First check the RtsThreshold, if no need for RTS/CTS check if protection needed */
   if( ((MpduSize+moreBytes) > (whal_ParamsGetGenParams(0)->RtsThreshold-FCS_FIELD_SIZE)) &&
		(pTxAttr->Broadcast == FALSE))
   {
		  MpduFlags |= MPDU_FLAGS_RTS_SET;
   }
   else
   {
	   switch(pTxAttr->ProtectionType)
	   {
	   case TXATTR_PROTECTION_RTS_CTS:
		   MpduFlags |= MPDU_FLAGS_RTS_SET;
		   break;
	   case TXATTR_PROTECTION_CTS_TOSELF:
		   MpduFlags |= MPDU_FLAGS_ERP_PROTECTION_SET;
		   break;
	   case TXATTR_PROTECTION_TI_TRICK:
		   MpduFlags |= MPDU_FLAGS_TI_PROTECTION_SET;
		   break;
	   default: /* no protection in TxAttr */
		   MpduFlags |= TXATTR_PROTECTION_NONE;
		   break;
	   }
   }/* endIf */

	 /* Demiurg: WME */
	 if (pTxAttr->QoS)
		 MpduFlags |= MPDU_FLAGS_QOS;

  /*
   * set the descriptor fields (OWN, FIRST_FRAG, MORE_FRAGS, RTS, RATE, ...)
   */
#if 0
     if ((fc & DOT11_FC_TO_DS) && (fc & DOT11_FC_FROM_DS))
     {
         printf( "whal_txSendMsdu() --\n pTxDesc = %p\n hostDescAddr = %p\n"
                 " MpduSize = %u\n NumBlks = %u\n Flags = 0x%08x\nPrivacy %s.\n",
                 pTxDesc, hostDescAddr, MpduSize, MpduNumMemBlks, MpduFlags, 
                 (fc & DOT11_FC_WEP) ? "ON" : "OFF");
         printf(" Attr: Bcast %x CIPHER %x Lifetime %x Prot %x QoS %x \n == Rate %x Fallback %x Preamble %x SSID %x\n", 
                    pTxAttr->Broadcast,
                    pTxAttr->CipherType,
                    pTxAttr->lifetime,
                    pTxAttr->ProtectionType,
                    pTxAttr->QoS,
                    pTxAttr->Rate,
                    pTxAttr->RateWithFallback,
                    pTxAttr->ShortPreamble,
                    pTxAttr->ssid_index );
     }
#endif

   {
		 UINT32 lifetime = pTxAttr->lifetime + PALSRV_ticks2usec(((mem_MSDU_T*) Msdu_Handle)->timestamp) - PALSRV_ticks2usec(PALSRV_getTick());

		 if ( pTxAttr->ackPolicy == QOS_ACK_POLICY_NO_ACK)
			MpduFlags |= MPDU_FLAGS_QOS_NO_ACK_POLICY;


       whal_txDesc_SetMpduInfoMaster( pTxDesc, hostDescAddr, MpduSize,
                                      MpduNumMemBlks,FragThreshold,
                                      pTxAttr->RateWithFallback, 
                                      MpduFlags, pMacHdr->address1.addr,lifetime);

   }															 
   
   /* We should only reset CTL_OWN bit to make FW take over the descriptor,
    * other bits are reserved (0)
    */
   whal_acxMemClrU8( (UINT32)pTxDesc + TX_CTL_BYTE_OFFSET);

#ifdef TIWLAN_DEBUG
   if (whal_trcIsEnable())
      whal_trcAddTx(pTxDesc, (UINT32) (firstBDptr->data + firstBDptr->dataOffset),"SEND");
#endif
   /*
    * Trigger the WLAN chip once for all the MPDUs
    */
   whal_acxIntrTriggerTxProc();

   /*
    * Increment number of Tx MSDU
    */
   TxFrameCounter[TxQid]++;
   PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  

   bm_trace(11,0,0);

   return 0;
}/* END whal_txSendMsdu() */

/****************************************************************************
 *                      whal_txCheckSize()
 ****************************************************************************
 * DESCRIPTION:
 * Checks if there is enough memory/descriptors available in the WLAN chip
 * to handle the MSDU, given its length.
 * 
 * INPUT :  
 *    TxQueueId_T TxQid   = Queue id
 *    UINT32      MsduLen = Length of the requested buffer
 *
 * OUTPUT:  None
 * 
 * RETURN:  int: OK or NOK
 ****************************************************************************/
int whal_txCheckSize(TxQueueId_T TxQid, UINT32 MsduLen)
{
    UINT32 NumOfHostDescInQ;

	/*
    * check WLAN chip FW descriptors
	 */
	if (whal_txQueue_GetNumFree(TxQid) <= 0) 
        {
#if 0 /* This can happen, no need to print error msg */
		puts("whal_txCheckSize() -- Out of Acx descriptors. \n");
#endif
		return NOK;
	}

    whal_HostDesc_getFreeBufsNum( txHostQueueArray[TxQid], &NumOfHostDescInQ );
    if( NumOfHostDescInQ < 2 ) {
        /*printf("whal_txGetBufsNum: Out of host descriptors, num of free host desc = %d, num of free acx desc = %d\n",NumOfHostDescInQ,whal_txQueue_GetNumFree(TxQid));*/
        return NOK;
    }

	return OK;
}/* END whal_txCheckSize() */

/****************************************************************************
 *                      whal_txGetBufsNum()
 ****************************************************************************
 * DESCRIPTION:
 * Checks if there is enough memory/descriptors for the MSDU returning the
 * FreeHostDescNum and CurrUsedAcxDescNum.
 * 
 * INPUT :  TxQueueId_T TxQid   = Queue id
 * OUTPUT:  UINT32 *FreeHostDescNum    = Pointer to store the number of free
 *                                       descriptors in the Host Queue
 *          UINT32 *CurrUsedAcxDescNum = Pointer to store the number of
 *                                       currently used descriptors in
 *                                       WLAN chip Queue.
 * RETURN:  int: OK or NOK
 ****************************************************************************/
int whal_txGetBufsNum (TxQueueId_T TxQid, UINT32 *FreeHostDescNum, UINT32 *CurrUsedAcxDescNum)
{
    /* Get number of Host Q free descriptors */
    whal_HostDesc_getFreeBufsNum (txHostQueueArray[TxQid], FreeHostDescNum );
    if(*FreeHostDescNum < MIN_NUM_OF_BDs_IN_MSDU)
	{
		/*printf("whal_txGetBufsNum: Out of host descriptors, num of free host desc = %d, num of free acx desc = %d\n",*FreeHostDescNum,whal_txQueue_GetNumFree(TxQid));*/
        return NOK;
	}
    
    if (whal_txQueue_GetNumFree(TxQid) == 0)
		return NOK;
    
    /* Get number of ACX Q currently used descriptors */
    *CurrUsedAcxDescNum = whal_txQueue_GetNumUsed(TxQid);

    return OK;
}/* END whal_txGetBufsNum () */

/****************************************************************************
 *                      whal_txGetHostQsize()
 ****************************************************************************
 * DESCRIPTION:   Gets the number of descriptors (size) of the Host Queue
 * 
 * INPUT :  TxQueueId_T TxQid   = Queue id
 * OUTPUT:  none
 * 
 * RETURN: int: OK or NOK
 ****************************************************************************/
int whal_txGetHostQsize(TxQueueId_T TxQid, UINT32 *queueSize)
{
	*queueSize = txHostQueueArray[TxQid]->NumOfDesc;
	return OK;
}/* END whal_txGetHostQsize() */

/****************************************************************************
 *                      whal_txInit()
 ****************************************************************************
 * DESCRIPTION:   Initializes Tx queues and Tx bufs objects.
 *
 * INPUT :  none
 * OUTPUT:  None
 * 
 * RETURN:  int: OK or NOK
 ****************************************************************************/
int whal_txInit()
{
	UINT8			WoneIndex, ac,TotalWones ;
	TxQueueId_T 	TxQid;
	
	TxStatusCB = NULL;
   
	wdrv_dbGetNumSupWone(&TotalWones);
	for (WoneIndex = 0,TxQid = 0; WoneIndex < TotalWones; WoneIndex++)
	{
        for (ac=0; ac< WDRV_MAX_TX_QUEUE ; ac++)
        {
            if (wdrv_dbGetAcxGlobalQindex(WoneIndex, ac, &TxQid) == OK)
						{
							UINT16 txOpLimit;
							TxFrameCounter[TxQid] = 0;
                            
							if(wdrv_dbGetWMEEnabled())
							{
								txOpLimit = wdrv_dbGetTxOPLimit(WoneIndex, TxQid);
							}
							else
							{
                                txOpLimit =0;
							}
							/* Init queue in ACX SRAM */
							whal_txQueue_Init(TxQid, whal_ParamsGetDmaParams(WoneIndex)->TxNumDesc[ac], txOpLimit, wdrv_QoSACtoACID[ac]);
							/* Init host descriptors queue */
							txHostQueueArray[TxQid] =  whal_HostQueue_init( whal_ParamsGetDmaParams(WoneIndex)->NumTxHostDesc, 0 );
         }/*endIf*/
      }/*endFor ac */
   }/*endFor WoneIndex */
   
	return OK;
}/* END whal_txInit() */

 /****************************************************************************
 *                      whal_txMasterUnload()
 ****************************************************************************
 * DESCRIPTION:   Unloads Tx Host Queues.
 *
 * INPUT :  none
 * OUTPUT:  None
 * 
 * RETURN:  none
 ****************************************************************************/
void whal_txMasterUnload()
{
	TxQueueId_T 	TxQid = 0;
	UINT8 			WoneIndex, LocalQid, TotalWones, WoneTxQueues;
	
   	wdrv_dbGetNumSupWone(&TotalWones);
   	
	for (WoneIndex = 0; WoneIndex < TotalWones; WoneIndex++)
	{
		wdrv_dbGetNumSupWoneQueues( WoneIndex, &WoneTxQueues );
		
		for (LocalQid=0; LocalQid<WoneTxQueues; LocalQid++,TxQid++)                              
	    {                                                                  
			whal_HostQueue_unload(txHostQueueArray[TxQid], 0);
			txHostQueueArray[TxQid] = NULL;
		}	
	}
}/* END whal_txMasterUnload() */

/****************************************************************************
 *                      whal_txClean()
 ****************************************************************************
 * DESCRIPTION:
 * Cleans the already transmitted Queue descriptors and buffers.
 * The function checks for TX_PROC interrupts and calls the TxQueue object 
 * to reclaim all transmitted Queues descriptors and Data buffers.
 *
 * INPUT :  TxQueueId_T TxQid = Queue id
 * OUTPUT:  None
 * 
 * RETURN:  int: OK. 
 ****************************************************************************/
int whal_txClean(TxQueueId_T TxQid)
{
	UINT8  WoneIndex, LocalQid, TotalWones, WoneTxQueues;
	
	if (TxQid == ALL_TX_QUEUES)
	{
	   	wdrv_dbGetNumSupWone(&TotalWones);
   	
		for (WoneIndex = 0, TxQid = 0; WoneIndex < TotalWones; WoneIndex++)
		{
			wdrv_dbGetNumSupWoneQueues( WoneIndex, &WoneTxQueues );
		
			for (LocalQid=0; LocalQid<WoneTxQueues; LocalQid++,TxQid++)                              
	    	{                                                                  
				whal_txQueue_Free(TxQid, txHostQueueArray[TxQid]);
			}
		}	
	}
   else {
   /* Clean ACX queue */
		whal_txQueue_Free( TxQid, txHostQueueArray[TxQid] );
   }

	return OK;
}/* END whal_txClean() */

/****************************************************************************
 *                      whal_txCleanAll()
 ****************************************************************************
 * DESCRIPTION:   Frees Tx unicast and broadcast host queue MSDUs
 *
 * INPUT :  None
 * OUTPUT:  None
 * 
 * RETURN:  int: OK. 
 ****************************************************************************/
int whal_txCleanAll(void)
{
	TxQueueId_T TxQid;
	UINT8 		WoneIndex, LocalQid, TotalWones, WoneTxQueues;

	wdrv_dbGetNumSupWone(&TotalWones);
   	
	for (WoneIndex = 0, TxQid = 0; WoneIndex < TotalWones; WoneIndex++)
	{
		wdrv_dbGetNumSupWoneQueues( WoneIndex, &WoneTxQueues );
		
		for (LocalQid=0; LocalQid<WoneTxQueues; LocalQid++,TxQid++)                              
	   	{                                      
			whal_HostQueue_CleanAll(txHostQueueArray[TxQid]);
		}
	}	

	return OK;
}/* END whal_txCleanAll() */

/****************************************************************************
 *                      whal_txHandleTxData()
 ****************************************************************************
 * DESCRIPTION:
 * Cleans the already transmitted Queue descriptors and buffers.
 * The function calls the TX_DATA user callback to reclaim all transmitted
 * Queues descriptors and Data buffers.
 *
 * INPUT :  None
 * OUTPUT:  None
 * 
 * RETURN:  none
 ****************************************************************************/
void whal_txHandleTxData()
{
	/* call user defined callback */
	if (TxDataUserCB != NULL)
		TxDataUserCB();	
}/* END whal_txHandleTxData() */

/****************************************************************************
 *                      whal_txSetTxDataUserCB()
 ****************************************************************************
 * DESCRIPTION:   Sets the user callback for TX_DATA Interrupt.
 *
 * INPUT :  TxDataUser_Callback_T externalTxDataUserCB = Callback address
 * OUTPUT:  None
 * 
 * RETURN:  int: OK. 
 ****************************************************************************/
int whal_txSetTxDataUserCB(TxDataUser_Callback_T externalTxDataUserCB)
{
	TxDataUserCB = externalTxDataUserCB;
   return OK;
}/* END whal_txSetTxDataUserCB() */

/****************************************************************************
 *                      whal_txReset()
 ****************************************************************************
 * DESCRIPTION:
 * Resets the object and the requested tx queues and buffers
 *
 * NOTE  : This function is now empty, only returns OK.
 * 
 * INPUT :  TxQueueId_T TxQid = The queue id
 * OUTPUT:  None
 * 
 * RETURN:  int: OK.
 ****************************************************************************/
int whal_txReset(TxQueueId_T TxQid)
{
#if 0
	if (TxQid == ALL_TX_QUEUES)
	{
		whal_txQueue_Reset(UcastTxQid);
		whal_txQueue_Reset(BcastTxQid);
		whal_txBufs_Reset();
	}
	else
		whal_txQueue_Reset(TxQid);
#endif
	return OK;
}/* END whal_txReset() */

/****************************************************************************
 *                      whal_txSetStatusCB()
 ****************************************************************************
 * DESCRIPTION:   Sets the callback for the return Tx status.
 *
 * INPUT :  TxStatus_Callback_T externalTxStatusCB = Callback address
 * OUTPUT:  None
 * 
 * RETURN:  int: OK
 ****************************************************************************/
int whal_txSetStatusCB(TxStatus_Callback_T externalTxStatusCB)
{
	TxStatusCB = externalTxStatusCB;
   return OK;
}/* END whal_txSetStatusCB() */

/****************************************************************************
 *                      whal_txSetStatus()
 ****************************************************************************
 * DESCRIPTION:	update(callback) Tx status of the transmitted frame.
 * 
 * INPUT :  
 *    UINT8 *DestMacAddress = Destination MAC address of the tx frame
 *    UINT16 ActualTxRate   = Actual transmitted rate
 *    UINT8  TxStatus       = Transmission's status
 * 
 * OUTPUT:	None
 * 
 * RETURN:  none
 ****************************************************************************/
void whal_txSetStatus(UINT8 *DestMacAddress, UINT16 ActualTxRate, UINT8 TxStatus)
{
	if (TxStatusCB != NULL)
		TxStatusCB(DestMacAddress, ActualTxRate, TxStatus);
}/* END whal_txSetStatus() */

/****************************************************************************
 *                      whal_txSet4xWackInfoCB()
 ****************************************************************************
 * DESCRIPTION:   Sets the callback for the return Wack Info.
 *
 * INPUT :  Recv4xWackInfo_Callback_T external4xWackInfoCB = callback address
 * OUTPUT:  None
 * 
 * RETURN:  int: OK. 
 ****************************************************************************/
int whal_txSetRecv4xWackInfoCB(Recv4xWackInfo_Callback_T externalRecv4xWackInfoCB)
{
	Recv4xWackInfoCB = externalRecv4xWackInfoCB;
   return OK;
}/* END whal_txSetRecv4xWackInfoCB() */

/****************************************************************************
 *                      whal_txSetRecv4xWackInfo()
 ****************************************************************************
 * DESCRIPTION:   Updates (callback) Tx status of the transmitted frame.
 * 
 * INPUT :  
 *    UINT8 *DestMacAddress = Destination MAC address of the tx frame
 *    UINT8 WackInfo        = Wack info from the FW
 *    UINT8 TxStatus        = Transmission's status
 * 
 * OUTPUT:  none
 * 
 * RETURN:  none
 ****************************************************************************/
void whal_txSetRecv4xWackInfo(UINT8 *DestMacAddress, UINT8 Recv4xWackInfo)
{
	if ( (Recv4xWackInfoCB != NULL) && (Recv4xWackInfo != LastRecv4xWackInfo) )
	{
		LastRecv4xWackInfo = Recv4xWackInfo;
		Recv4xWackInfoCB(DestMacAddress, Recv4xWackInfo);
	}
}/* END whal_txSetRecv4xWackInfo() */

/****************************************************************************
 *                      whal_txGetRecv4xWackParams()
 ****************************************************************************
 * DESCRIPTION:   Gets 4x Wack parameters.
 * 
 * INPUT :  none
 * OUTPUT:  UINT32 *CallbackAddr   = Set to the address of the callback
 *          UINT8  *Recv4xWackInfo = Will be set to LastRecv4xWackInfo
 * 
 * RETURN:  none
 ****************************************************************************/
void whal_txGetRecv4xWackParams(UINT32 *CallbackAddr, UINT8 *Recv4xWackInfo)
{
	*Recv4xWackInfo = LastRecv4xWackInfo;
	*CallbackAddr = (UINT32)Recv4xWackInfoCB;
}/* END whal_txGetRecv4xWackParams() */

/****************************************************************************
 *                      whal_txTxFrameCount()
 ****************************************************************************
 * DESCRIPTION:   Returns the Tx frame counter
 *
 * INPUT :  TxQueueId_T TxQid = The queue id
 * OUTPUT:  none
 *
 * RETURN:  int: Tx frame counter
 ****************************************************************************/
int whal_txTxFrameCount(TxQueueId_T TxQid)
{
	return (TxFrameCounter[TxQid]);
}/* END whal_txTxFrameCount() */

/****************************************************************************
 *                      whal_txInfoPrint()
 ****************************************************************************
 * DESCRIPTION:   Prints Tx queue/bufs info.
 *
 * INPUT :  int Qid       = The queue id
 *          BOOL QprintEn = Set this flag to enable printing
 * OUTPUT:  char *buf     = pointer for output storage. Caller must allocate
 *                          enough space for output.
 * 
 * RETURN:  int: length of output buffer.
 ****************************************************************************/
int whal_txInfoPrint(int Qid, BOOL QprintEn, char *buf)
{
	UINT8			WoneIndex, TotalWones, ac, WoneTxQueues, Type, Priority, TotalQueues,Active;
	TxQueueId_T 	TxQid;
	int len = 0;

	wdrv_dbGetNumSupWone(&TotalWones);
	for (WoneIndex = 0, TotalQueues = 0; WoneIndex < TotalWones; WoneIndex++)
	{
		wdrv_dbGetNumSupWoneQueues( WoneIndex, &WoneTxQueues );
		TotalQueues +=	WoneTxQueues;
	}

   if( TotalQueues <= Qid ) return 0;

   len+=whal_txQueue_InfoPrint(Qid,buf);

	for (WoneIndex = 0,TxQid = 0; WoneIndex < TotalWones; WoneIndex++)
	{
		wdrv_dbGetNumSupWoneQueues( WoneIndex, &WoneTxQueues );

        for (ac=0; ac < WDRV_MAX_TX_QUEUE; ac++)
        {
            if (wdrv_dbGetAcxGlobalQindex(WoneIndex,ac,&TxQid)==OK)
		{
			if (TxQid == Qid)
			{
                    wdrv_dbGetAcxQindex( WoneIndex, ac, &Type, &Priority,&Active,&TxQid);
                }
			}
		}
	}

#if 0
   if( Type )
   {
      len+=sprintf(buf+len," Broadcast Host Queue:");
      len+=sprintf(buf+len,"=======================");
   }
   else
   {
      len+=sprintf(buf+len," Unicast Host Queue:");
      len+=sprintf(buf+len,"=====================");
   }
#endif
   
   if(QprintEn)
	   len+=whal_HostQueue_show( txHostQueueArray[Qid],buf );


   len += sprintf(buf+len,"\n\nHost Queue Statistics: \n");
   len +=sprintf(buf+len,"\tCurrNumDescInQ\t\t%u\n", txHostQueueArray[Qid]->CurrNumDescInQ );
   len +=sprintf(buf+len,"\tMaxCurrNumDescInQ\t%u\n", txHostQueueArray[Qid]->MaxCurrNumDescInQ );
   len +=sprintf(buf+len,"\tMinCurrNumDescInQ\t%u\n\n", txHostQueueArray[Qid]->MinCurrNumDescInQ );

   len +=sprintf(buf+len,"\ttargetAborts\t\t%u\n", txHostQueueArray[Qid]->statistics.targetAborts);
   len +=sprintf(buf+len,"\tmasterAborts\t\t%u\n", txHostQueueArray[Qid]->statistics.masterAborts);
   len +=sprintf(buf+len,"\treqBeforeComplete\t%u\n", txHostQueueArray[Qid]->statistics.reqBeforeComplete);
   len +=sprintf(buf+len,"\tunexpectHostError\t%u\n", txHostQueueArray[Qid]->statistics.unexpectHostError);
   len +=sprintf(buf+len,"\tunexpectAcxError\t%u\n", txHostQueueArray[Qid]->statistics.unexpectAcxError);
   len +=sprintf(buf+len,"\tpciNotAbled\t\t%u\n\n", txHostQueueArray[Qid]->statistics.pciNotAbled);
   
   return len;
}/* END whal_txInfoPrint() */

 
