/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2004 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/
/****************************************************************************
 *
 *   MODULE:  whalTxQueue.c
 *   PURPOSE: WLAN chip Tx Queues manager
 *
 *   The object manages Unicast and Broadcast Tx queues. each queue consists
 *   of MPDU descriptors located in the ACX100 memory.
 *   The Object holds 2 pointers (pFree, pAlloc) to manage the alloc/free
 *   of WLAN chip FW descriptors which are organized in a link list.
 *
 *   Functions: 
 *       Allocation - is from pFree pointer
 *       Free/Reclaim - is from pAlloc pointer.
 *   
 ****************************************************************************/


/****************************
 * HAL Include files
 ****************************/
#include "wlan_pal.h"
#include "whalCommon.h"
#include "whal.h"
#include "AcxCommands.h"
#include "AcxDataIf.h"
#include "whalParams.h"
#include "whalAcxAccess.h"
#include "whalTx.h"
#include "whalTxQueue.h"
#include "whalTxBufs.h"
#include "whalTrace.h"
#include "whalHostQueue.h"
#include "whalAcxRegs.h"
#include "whalTxDesc.h"
#include "QoS/QoSClassifier.h"
#include "drvDbMain.h"
#include "drvDbHash.h"
#include "mgmtSsidHandler.h"

/****************************
 * External data variables
 ****************************/
extern void whal_apiRestartWatchDog_PacketTransmitted(TxQueueId_T Qid);
/****************************************************************************
 *                      OBJECT DATA BASE
 ****************************************************************************
 *
 * WLAN chip Mpdu descriptor
 *    The WLAN chip descriptor holds MPDU information (Size, Tx rate, ...)
 *    and a pointer to the internal data blocks to be transmitted.
 *    pNext and Control(for LAST block bit) and are initiate by the FW
 *    as a link list.
 *
 * Local Data Base
 *    TxQueueDb[NUM_TX_QUEUES]   
 *          Holds the Queues objects data bases. The Queues are:
 *             1. Unicast queue
 *             2. Broadcast queue
 *    Each queue is handled as link list where pFree points always to free 
 *    descriptor on the queue and pAlloc points to allocated descriptors 
 *    that after transmission should be reclaimed.
 *
 ****************************************************************************/

TxQueue_T TxQueueDb[NUM_TX_QUEUES];
QueueIndicator_T *pTxQIndicator;
QueueIndicator_T *pTxHostQIndicator;

/****************************
 * Functional Implementation
 ****************************/

/****************************************************************************
 *                      whal_txQueue_Alloc()
 ****************************************************************************
 * DESCRIPTION:   Allocates WLAN FW descriptor from the pFree list
 * 
 * INPUT :  TxQueueId_T Qid  = The Id of the Queue
 * OUTPUT:  none
 * 
 * RETURN:  TxDesc_T *: Allocated Tx queue pointer. NULL if failed.
 ****************************************************************************/
TxDesc_T *whal_txQueue_Alloc(TxQueueId_T Qid)
{
   TxQueue_T *pTxQ = &TxQueueDb[Qid];
   TxDesc_T  *pReturnDesc = NULL;

   if (pTxQ->NumFree <= 0)
   {
		/*printf("whal_txQueue_Alloc: WARNNING no more descriptors, num=%d\n",pTxQ->NumFree);*/
      return NULL;
   }

   if ( (whal_txDesc_GetCtl(pTxQ->pFree) & CTL_OWN) == 0)
   {
      printf("whal_txQueue_Alloc: ERROR Desc in the free list but not CTL_OWN ?!\n");
      return NULL;
   }

   /* descriptor address to return */
   pReturnDesc = pTxQ->pFree;

   /* point to the next free descriptor */
   pTxQ->pFree = whal_txDesc_GetHostNext(pTxQ->pFree);
   pTxQ->NumFree--;

   return pReturnDesc;
}/* END whal_txQueue_Alloc() */

/****************************************************************************
 *                      whal_txQueue_Free()
 ****************************************************************************
 * DESCRIPTION:
 * Frees WLAN FW descriptors from the pAlloc list, the function also frees
 * associated data blocks (pointed to by AcxMemPtr field) by calling the
 * TxBufs object.
 * 
 * INPUT :  TxQueueId_T Qid  = The Id of the Queue
 *          HostQueue_Obj* phostQueue = Pointer to host queue
 * OUTPUT:  none
 * 
 * RETURN:  int: OK or NOK
 ****************************************************************************/
int whal_txQueue_Free(TxQueueId_T Qid, HostQueue_Obj* phostQueue)
{
   register TxQueue_T *pTxQ = &TxQueueDb[Qid];
   UINT8 TxStatus;
   UINT16 ActualTxRate;
   UINT8 DestMacAddress[MAC_ADDR_LEN];
   UINT8 AckFail, RtsFail, RtsSucc;
   HostDesc_T *        HostDescPtr = NULL;
   UINT8 AcSecuritySeqNumLsb;
   UINT16 staIndex;
   UINT32 currFreeNum = pTxQ->NumFree;

   while (pTxQ->NumDesc > pTxQ->NumFree)
   {
      register TxDesc_T *pAlloc = pTxQ->pAlloc;
      UINT16 SendTxRate = 0;
      
      if (pAlloc == 0)
      {
         printf("whal_txQueue_Free error 1\n");
         return NOK;
      }
      
      /*
       * stop if the ACX100 didn't send yet the first allocated MPDU descriptor, will occur at 1st time. 
       */
      if((whal_txDesc_GetCtl(pAlloc) & (CTL_OWN | CTL_ACXDONE)) != (CTL_OWN | CTL_ACXDONE))
      {
         break;
      }
      
#ifdef TIWLAN_DEBUG
      if (whal_trcIsEnable())
         whal_trcAddTc(pAlloc, " END");
#endif
      /* 
       * Increment counters - (!!!) add error threshold for FATAL ERROR and reboot
       */
      TxStatus = whal_txDesc_GetStatus(pAlloc, &AckFail, &RtsFail, &RtsSucc);
      TxStatus = pAlloc->Status;
      
      pTxQ->AckFailCounter += AckFail;
      pTxQ->RtsFailCounter += RtsFail;
      pTxQ->RtsSuccCounter += RtsSucc;
      if (AckFail > 0)
         pTxQ->RetryCounter++;
      if (AckFail > 1)
         pTxQ->MultipleRetryCounter++;
      
      if ( TxStatus == STAT_TX_SUCCESS)
      {
         pTxQ->SuccCounter++;
      }
      else
      {
         if( TxStatus & STAT_RETRY)
            pTxQ->RetryFailCounter++;
         else
            pTxQ->OtherFailCounter++;
         
         if( TxStatus & STAT_DMA_ERR)
         {
            UINT32 dma_st;    /* Track what is wrong with the DMA */
            
            pTxQ->DmaErrorCounter++;
            
            /* Monitor DMA#1 */
            dma_st = whal_acxRegRead( ACX_DMA_STATUS1_REG );
            switch( dma_st)
            {
            case  DMA_IN_PROGRESS   : pTxQ->DmaErrorInProgress++    ;  break;
#if( WLAN_ADAPTOR_VERSION == WLAN_ADAPTOR_VERSION_1130)
            case  DMA_NO_ERROR      : pTxQ->DmaErrorNoErr_spur++    ;  break;
            case  DMA_TARGET_ABORT  : pTxQ->DmaErrorTargetAbort++   ;  break;
            case  DMA_MASTER_ABORT  : pTxQ->DmaErrorMasterAbort++   ;  break;
#endif   /* 1130 */
            case  DMA_CLEARED_ABORT : pTxQ->DmaErrorClearedAbort++  ;  break;
            case  DMA_HOST_END_ABORT: pTxQ->DmaErrorHostEndAbort++  ;  break;
#if( WLAN_ADAPTOR_VERSION == WLAN_ADAPTOR_VERSION_1150)
            case  DMA_PREV_ERROR    : pTxQ->DmaErrorPrevError++     ;  break;
            case  DMA_LOCAL_READ_ERROR: pTxQ->DmaLocalReadError++   ;  break;
            case  DMA_LOCAL_WRITE_ERROR: pTxQ->DmaLocalWriteError++ ;  break;
            case  DMA_HOST_OWN_ERROR: pTxQ->DmaErrorHostOwnError++  ;  break;
#else  /* not 1150 */
            case  DMA_ACX_END_ABORT : pTxQ->DmaErrorAcxEndAbort++   ;  break;
            case  DMA_PCI_DISABLED  : pTxQ->DmaPciDisabled++        ;  break;
#endif /* WLAN_ADAPTOR_VERSION == WLAN_ADAPTOR_VERSION_1150 */
            default:                  pTxQ->DmaErrorUnspecified++   ;  break;
            }
         }
         
         if( TxStatus & STAT_MSDU_TIMEOUT)
            pTxQ->TimeoutErrorCounter++;
      }

      /*
       * Call Rate Adaptaion if needed
       */
      ActualTxRate = whal_txDesc_GetRate(pAlloc);

      HostDescPtr = (HostDesc_T *)whal_acxMemGetU32((UINT32)&pAlloc->HostMemPtr);
      
#if( WLAN_BUS_TYPE == WLAN_BUS_TYPE_VLYNQ)
      HostDescPtr = (HostDesc_T *)VLYNQ_ACX111_TO_HOST_SDRAM_PTR_CACHED(HostDescPtr);
#elif( WLAN_BUS_TYPE == WLAN_BUS_TYPE_PCI)
      HostDescPtr = (HostDesc_T *)PCI_ACX111_TO_HOST_SDRAM_PTR_CACHED(HostDescPtr);
#else
#error    whalHostqueue.c -- Check bus type.
#endif   /* WLAN_BUS_TYPE  */
      
      if (HostDescPtr)
      {
         SendTxRate                      = *(UINT16 *)(&HostDescPtr->DriverWorkspace[0]);
         *(UINT16 *)(&DestMacAddress[0]) = *(UINT16 *)(&HostDescPtr->DriverWorkspace[2]);
         *(UINT16 *)(&DestMacAddress[2]) = *(UINT16 *)(&HostDescPtr->DriverWorkspace[4]);
         *(UINT16 *)(&DestMacAddress[4]) = *(UINT16 *)(&HostDescPtr->DriverWorkspace[6]);
      }
      
      if ((ActualTxRate != SendTxRate) || (TxStatus == STAT_RETRY))
      {
         whal_txSetStatus(DestMacAddress, ActualTxRate, TxStatus);
      }
			
		if (wdrv_dbFindStaIndex(DestMacAddress, &staIndex) == OK || (DestMacAddress[0] & 1)) 
		{
			if (whal_HostQueue_GetSecurity(phostQueue)) 
			{
				wpa_sta_type_t sta_type;
                int profile_index;

				if (DestMacAddress[0] & 1) /* broadcast packet */
				{
				   profile_index = whal_acxMemGetByte((UINT32)pAlloc, &(((TxDesc_T *)0)->ssidProfile));
				}
				else /* in case of unicast the ssid field in descriptor may be non-intialized, so we can't use it */
				{
					wdrv_mgmt_ssidGetStationSsidIndex(staIndex, &profile_index);
				}
				wdrv_mgmt_ssidGetSecurityType(profile_index, &sta_type);
				
				/* in case of WEP cipher sequence number is not used */
				if ( !(sta_type & (WPA_STA_TYPE_WEP_8021X | WPA_STA_TYPE_WEP_LEGACY)) )
				{
					AcSecuritySeqNumLsb = whal_acxMemGetByte((UINT32)pAlloc, TX_AC_SEC_SEQ_NUM_LSB_OFFSET);
					wdrv_dbSetSecuritySeqNum(staIndex, AcSecuritySeqNumLsb, pTxQ->acID,  DestMacAddress[0] & 1);
				}
			}
		}

	  /* Free the Host Descr and MSDU */
      if( phostQueue ->CurrNumDescInQ )
         whal_HostQueue_freeMsdu( phostQueue );      
      
      /*
       * re-initiate all descriptor fields (except pNext field)
       */
      whal_txDesc_reinit(pAlloc);
      pTxQ->NumFree++;
      
      /* move to the next descriptor (cyclic) */
      pTxQ->pAlloc = whal_txDesc_GetHostNext(pAlloc);
   }/* endWhile*/

   if(currFreeNum != pTxQ->NumFree) {
	   /*printf("%s: Qid=%d NumOfFreeDesc=%d\n",__FUNCTION__,Qid,pTxQ->NumFree);*/
	  whal_apiRestartWatchDog_PacketTransmitted(Qid);
	  }
   
   return OK;
}/* END whal_txQueue_Free() */

/****************************************************************************
 *                      whal_txQueue_Free()
 ****************************************************************************
 * DESCRIPTION:
 * Initiates the selected queue object. In this stage there is FW has not
 * allocated internal memory in the WLAN chip (i.e. no address).
 * 
 * INPUT :  TxQueueId_T Qid  = The Id of the Queue
 *          int NumTxAcxDesc = Number of descriptors required
 *          UINT16 txOpLimit = limit of the Tx queue
 *          UINT8 acID = AC ID
 * OUTPUT:  none
 * 
 * RETURN:  int: OK or NOK
 ****************************************************************************/
int whal_txQueue_Init(TxQueueId_T Qid, int NumTxAcxDesc, UINT16 txOpLimit, UINT8 acID)
{
   TxQueue_T *pTxQ = &TxQueueDb[Qid];
   
   PAL_memSet((void *)pTxQ, 0, sizeof(*pTxQ));
   
   pTxQ->NumDesc = NumTxAcxDesc;
   pTxQ->txOpLimit = txOpLimit;
	 pTxQ->acID = acID;
   
   return OK;
}/* END whal_txQueue_Init() */

/****************************************************************************
 *                      whal_txQueue_Configure()
 ****************************************************************************
 * DESCRIPTION:
 * Configures the selected Queue object and initiates the Discriptors
 * Queue linked list in the WLAN chip internal SRAM.
 * 
 * INPUT :  TxQueueId_T Qid  = The Id of the Queue
 *          UINT32 TxQueue_OffsetAddr = Number of descriptors required
 * OUTPUT:  none
 * 
 * RETURN:  UINT32: size of the queue.
 ****************************************************************************/
UINT32 whal_txQueue_Config(TxQueueId_T Qid, UINT32 TxQueue_OffsetAddr)
{
   TxQueue_T *pTxQ = &TxQueueDb[Qid];

   pTxQ->NumFree = pTxQ->NumDesc;

   /*
    * Save parameters
    */
   pTxQ->HwBaseAddr = (TxDesc_T *)(TxQueue_OffsetAddr + ACX100_MEM_ADDRESS );
   pTxQ->pFree = pTxQ->HwBaseAddr;
   pTxQ->pAlloc = pTxQ->HwBaseAddr;

   printf("whal_txQueue_Init: [Qid=%1d] HwBaseAddr=0x%08x, NumDesc=%d\n", Qid, (UINT32)pTxQ->HwBaseAddr, pTxQ->NumDesc);   

   return (whal_txQueue_GetSize(Qid));
}/* END whal_txQueue_Config() */

/****************************************************************************
 *                      whal_txIndicatorSetAddr()
 ****************************************************************************
 * DESCRIPTION:   Set the QueueIndicator address
 * 
 * INPUT :  UINT32 QueueIndicatorAddress = The Indicator address
 * OUTPUT:  none
 * 
 * RETURN:  none
 ****************************************************************************/
void whal_txIndicatorSetAddr(UINT32 QueueIndicatorAddress)
{
   pTxQIndicator = (QueueIndicator_T *)(QueueIndicatorAddress+ACX100_MEM_ADDRESS);
   whal_acxMemClrU32(&pTxQIndicator->indicator); 
   whal_acxMemClrU16(&pTxQIndicator->fw_lock); 
   whal_acxMemClrU16(&pTxQIndicator->host_lock);

   pTxHostQIndicator = (QueueIndicator_T *)(QueueIndicatorAddress+ACX100_MEM_ADDRESS+sizeof(QueueIndicator_T));
   whal_acxMemClrU32(&pTxHostQIndicator->indicator); 
   whal_acxMemClrU16(&pTxHostQIndicator->fw_lock); 
   whal_acxMemClrU16(&pTxHostQIndicator->host_lock);
}/* END whal_txIndicatorSetAddr() */

/****************************************************************************
 *                      whal_txIndicatorSet()
 ****************************************************************************
 * DESCRIPTION:
 * Sets the specific Queue Indicator, according to the following sequence: 
 *    Wait for FW to unlock 
 *    Lock from host side
 *    Set the indicator
 *    Unlock from host side
 * 
 * INPUT :  TxQueueId_T Qid = The Id of the Queue
 * OUTPUT:  none
 * 
 * RETURN:  int: OK or NOK
 ****************************************************************************/
int whal_txIndicatorSet(TxQueueId_T Qid)
{
   int Count;

   /* Wait for FW to unlock */
   Count = 0;
   while( whal_acxMemGetRawU16( &pTxQIndicator->fw_lock))
   {
      if (Count++ > QIND_MAX_COUNT)
      {
         printf("whal_txIndicatorSet: FATAL_ERROR firmware lock the queue indicator[1]\n");
         return NOK;
      }
      PAL_sleepLong(QIND_DELAY_VALUE);
   }

   /* Host Lock*/
   whal_acxMemSetU16(&pTxQIndicator->host_lock, 1);

   /* Check FW Lock again, just in case we were in race ! */
   Count = 0;
   while( whal_acxMemGetRawU16( &pTxQIndicator->fw_lock))
   {
      if (Count++ > QIND_MAX_COUNT)
      {
         printf("whal_txIndicatorSet: FATAL_ERROR firmware lock the queue indicator[2]\n");
         return NOK;
      }
      PAL_sleepLong(QIND_DELAY_VALUE);
   }

   /* Set the specific queue indicator */
   whal_acxMemSetU32_Bits(&pTxQIndicator->indicator, (0x1 << Qid));

   /* Host UnLock */
   whal_acxMemClrU16( &pTxQIndicator->host_lock);

   return OK;
}/* END whal_txIndicatorSet() */

/****************************************************************************
 *                      whal_txQueue_GetSize()
 ****************************************************************************
 * DESCRIPTION:
 * Returns the size of the Tx queue allocated internally in the WLAN chip.
 * INPUT :  TxQueueId_T Qid = The Id of the Queue
 * OUTPUT:  none
 * 
 * RETURN:  UINT32: Total size in bytes
 ****************************************************************************/
UINT32 whal_txQueue_GetSize(TxQueueId_T Qid)
{
   TxQueue_T *pTxQ = &TxQueueDb[Qid];
   return (pTxQ->NumDesc * sizeof(TxDesc_T));
}/* END whal_txQueue_GetSize() */

/****************************************************************************
 *                      whal_txQueue_GetDescNum()
 ****************************************************************************
 * DESCRIPTION:   Returns the total number of descriptors avaialble in the
 *                WLAN chip.
 *
 * INPUT :  TxQueueId_T Qid = The Id of the Queue
 * OUTPUT:  none
 * 
 * RETURN:  int: number of descriptors
 ****************************************************************************/
int whal_txQueue_GetDescNum(TxQueueId_T Qid)
{
   TxQueue_T *pTxQ = &TxQueueDb[Qid];
   return (int)(pTxQ->NumDesc);
}/* END whal_txQueue_GetDescNum() */

/****************************************************************************
 *                      whal_txQueue_GetBaseAddr()
 ****************************************************************************
 * DESCRIPTION:   Returns the WLAN chip Tx queue address
 * INPUT :  TxQueueId_T Qid = The Id of the Queue
 * OUTPUT:  none
 * 
 * RETURN:  UINT32: Base address of Tx queue.
 ****************************************************************************/
UINT32 whal_txQueue_GetBaseAddr(TxQueueId_T Qid)
{
   TxQueue_T *pTxQ = &TxQueueDb[Qid];
   return ((UINT32)pTxQ->HwBaseAddr-ACX100_MEM_ADDRESS);
}/* END whal_txQueue_GetBaseAddr() */

/****************************************************************************
 *                      whal_txQueue_GetNumFree()
 ****************************************************************************
 * DESCRIPTION:   Returns number of free descriptors in WLAN chip Tx queue.
 * INPUT :  TxQueueId_T Qid = The Id of the Queue
 * OUTPUT:  none
 * 
 * RETURN:  UINT32: Number of free descriptors.
 ****************************************************************************/
UINT32 whal_txQueue_GetNumFree(TxQueueId_T Qid)
{
   TxQueue_T *pTxQ = &TxQueueDb[Qid];
   return ((UINT32)pTxQ->NumFree);
}/* END whal_txQueue_GetNumFree() */

/****************************************************************************
 *                      whal_txQueue_GetNumUsed()
 ****************************************************************************
 * DESCRIPTION:   Returns number of used descriptors in WLAN chip Tx queue 
 * INPUT :  TxQueueId_T Qid = The Id of the Queue
 * OUTPUT:  none
 * 
 * RETURN:  UINT32: Number of used descriptors.
 ****************************************************************************/
UINT32 whal_txQueue_GetNumUsed(TxQueueId_T Qid)
{
   TxQueue_T *pTxQ = &TxQueueDb[Qid];
   return ((UINT32)(pTxQ->NumDesc - pTxQ->NumFree));
}/* END whal_txQueue_GetNumUsed() */

/****************************************************************************
 *                      whal_txQueue_GetFailedCount()
 ****************************************************************************
 * DESCRIPTION:   Return the Failed counter
 * INPUT :  TxQueueId_T Qid = The Id of the Queue
 * OUTPUT:  none
 * 
 * RETURN:  int: RetryFailCounter
 ****************************************************************************/
int whal_txQueue_GetRetryFailCount(TxQueueId_T Qid)
{
   TxQueue_T *pTxQ = &TxQueueDb[Qid];
   return (pTxQ->RetryFailCounter);
}/* END whal_txQueue_GetRetryFailCount() */

/****************************************************************************
 *                      whal_txQueue_GetRetryCount()
 ****************************************************************************
 * DESCRIPTION:   Returns the "Retry"s counter
 * INPUT :  TxQueueId_T Qid = The Id of the Queue
 * OUTPUT:  none
 * 
 * RETURN:  int: Retry counter value.
 ****************************************************************************/
int whal_txQueue_GetRetryCount(TxQueueId_T Qid)
{
   TxQueue_T *pTxQ = &TxQueueDb[Qid];
   return (pTxQ->RetryCounter);
}/* END whal_txQueue_GetRetryCount() */

/****************************************************************************
 *                      whal_txQueue_GetMultipleRetryCount()
 ****************************************************************************
 * DESCRIPTION:   Returns the "Multiple Retry" counter.
 * INPUT :  TxQueueId_T Qid = The Id of the Queue
 * OUTPUT:  none
 * 
 * RETURN:  int: Value of "Multiple Retry" counter.
 ****************************************************************************/
int whal_txQueue_GetMultipleRetryCount(TxQueueId_T Qid)
{
   TxQueue_T *pTxQ = &TxQueueDb[Qid];
   return (pTxQ->MultipleRetryCounter);
}/* END whal_txQueue_GetMultipleRetryCount() */

/****************************************************************************
 *                      whal_txQueue_GetAckFailCount()
 ****************************************************************************
 * DESCRIPTION:   Returns the "Ack Fail" counter
 * INPUT :  TxQueueId_T Qid = The Id of the Queue
 * OUTPUT:  none
 * 
 * RETURN:  int: Value of "Multiple Retry" counter.
 ****************************************************************************/
int whal_txQueue_GetAckFailCount(TxQueueId_T Qid)
{
   TxQueue_T *pTxQ = &TxQueueDb[Qid];
   return (pTxQ->AckFailCounter);
}/* END whal_txQueue_GetAckFailCount() */

/****************************************************************************
 *                      whal_txQueue_GetRtsFailCount()
 ****************************************************************************
 * DESCRIPTION:   Returns the RTS failures counter
 * INPUT :  TxQueueId_T Qid = The Id of the Queue
 * OUTPUT:  none
 * 
 * RETURN:  int: Value of "RTS failures" counter.
 ****************************************************************************/
int whal_txQueue_GetRtsFailCount(TxQueueId_T Qid)
{
   TxQueue_T *pTxQ = &TxQueueDb[Qid];
   return (pTxQ->RtsFailCounter);
}/* END whal_txQueue_GetRtsFailCount() */

/****************************************************************************
 *                      whal_txQueue_GetRtsSuccCount()
 ****************************************************************************
 * DESCRIPTION:   Returns the "RTS Success" counter
 * INPUT :  TxQueueId_T Qid = The Id of the Queue
 * OUTPUT:  none
 * 
 * RETURN:  int: Value of "RTS Success" counter.
 ****************************************************************************/
int whal_txQueue_GetRtsSuccCount(TxQueueId_T Qid)
{
   TxQueue_T *pTxQ = &TxQueueDb[Qid];
   return (pTxQ->RtsSuccCounter);
}/* END whal_txQueue_GetRtsSuccCount() */

/****************************************************************************
 *                      whal_txQueue_GetSuccCount()
 ****************************************************************************
 * DESCRIPTION:   Returns the "Success" counter
 * INPUT :  TxQueueId_T Qid = The Id of the Queue
 * OUTPUT:  none
 * 
 * RETURN:  int: Value of "Success" counter.
 ****************************************************************************/
int whal_txQueue_GetSuccCount(TxQueueId_T Qid)
{
   TxQueue_T *pTxQ = &TxQueueDb[Qid];
   return (pTxQ->SuccCounter);
}/* END whal_txQueue_GetSuccCount() */

/****************************************************************************
 *                      whal_txQueue_Print()
 ****************************************************************************
 * DESCRIPTION:
 * Prints the TxQueue list and it's main fields, assuming output buffer is
 * large enough to hold output.
 *
 * INPUT :  TxQueueId_T Qid = The Id of the Queue
 * OUTPUT:  char *buf = Output buffer
 * 
 * RETURN:  int: length of output.
 ****************************************************************************/
int whal_txQueue_Print(TxQueueId_T Qid, char *buf)
{
   TxQueue_T *pTxQ = &TxQueueDb[Qid];
   TxDesc_T *pHwTxDesc = pTxQ->HwBaseAddr;
   int Idx,len = 0;


   for (Idx=0; Idx<pTxQ->NumDesc; Idx++)
   {
      len +=sprintf(buf+len,"%02x: ", Idx);
      len +=whal_txDesc_Print(&pHwTxDesc[Idx], buf + len);
   }

   return len;
}/* END whal_txQueue_Print() */

/****************************************************************************
 *                      whal_txQueue_InfoPrint()
 ****************************************************************************
 * DESCRIPTION:   Prints the TxQueue Info, assuming output buffer is
 * large enough to hold output.
 *
 * INPUT :  TxQueueId_T Qid = The Id of the Queue
 * OUTPUT:  char *buf = Output buffer
 * 
 * RETURN:  int: length of output.
 ****************************************************************************/
int whal_txQueue_InfoPrint(TxQueueId_T Qid ,char *buf)
{
   TxQueue_T *pTxQ = &TxQueueDb[Qid];
   int len = 0;

   len +=sprintf(buf+len,"TxQueue Id=%d ----------------------------------------------\n", Qid);
   len +=sprintf(buf+len,"Addr=0x%x, NumDesc=0x%x (DescSize=0x%x)\n", (UINT32)pTxQ->HwBaseAddr, pTxQ->NumDesc, sizeof(TxDesc_T));
   len +=sprintf(buf+len,"pFree=0x%x, pAlloc=0x%x, NumFree=0x%x\n", (UINT32)pTxQ->pFree, (UINT32)pTxQ->pAlloc, pTxQ->NumFree);
   len +=sprintf(buf+len,"Counters:\n Success = 0x%x, Retry = 0x%x, Multiretry = 0x%x\n", 
      pTxQ->SuccCounter, pTxQ->RetryCounter, pTxQ->MultipleRetryCounter);
   len +=sprintf(buf+len,"Fails   :\n Retry   = 0x%x, Other = 0x%x, Timeout = 0x%x, DMA Err = 0x%x\n",
      pTxQ->RetryFailCounter, pTxQ->OtherFailCounter,
      pTxQ->TimeoutErrorCounter, pTxQ->DmaErrorCounter);

#if( WLAN_ADAPTOR_VERSION == WLAN_ADAPTOR_VERSION_1150)
   len += sprintf( buf+len, "DMA Errors:\n"
      " InProgress = 0x%x\n"
      " Cleared    = 0x%x\n"
      " Host end   = 0x%x\n"
      " Prev Err   = 0x%x\n"
      " Read err   = 0x%x\n"
      " Write Err  = 0x%x\n"
      " Host Own   = 0x%x\n"
      " Other      = 0x%x\n\n",
      pTxQ->DmaErrorInProgress   ,
      pTxQ->DmaErrorClearedAbort ,
      pTxQ->DmaErrorHostEndAbort ,
      pTxQ->DmaErrorPrevError  ,
      pTxQ->DmaLocalReadError  ,
      pTxQ->DmaLocalWriteError  ,
      pTxQ->DmaErrorHostOwnError  ,
      pTxQ->DmaErrorUnspecified
      );
#else   /* WLAN_ADAPTOR_VERSION != 1150 */
   len +=sprintf( buf+len, "DMA Errors:\n"
      " InProgress = 0x%x\n"
      " Spur       = 0x%x\n"
      " Target     = 0x%x\n"
      " Master     = 0x%x\n"
      " Cleared    = 0x%x\n"
      " Host end   = 0x%x\n"
      " ACX end    = 0x%x\n"
      " PCI disable= 0x%x\n"
      " Other      = 0x%x\n\n",
      pTxQ->DmaErrorInProgress   ,
      pTxQ->DmaErrorNoErr_spur ,
      pTxQ->DmaErrorTargetAbort  ,
      pTxQ->DmaErrorMasterAbort  ,
      pTxQ->DmaErrorClearedAbort ,
      pTxQ->DmaErrorHostEndAbort ,
      pTxQ->DmaErrorAcxEndAbort  ,
      pTxQ->DmaPciDisabled  ,
      pTxQ->DmaErrorUnspecified
      );
#endif   /* WLAN_ADAPTOR_VERSION  */

   return len;
}/* END whal_txQueue_InfoPrint() */

/****************************************************************************
 *                      whal_txQueueGetTxOpLimit()
 ****************************************************************************
 * DESCRIPTION:   TX OP limit (WME), time limit to transmit the packet.
 *
 * INPUT :  TxQueueId_T Qid = The Id of the Queue
 * OUTPUT:
 * 
 * RETURN:  UINT16: TX OP limit.
 ****************************************************************************/
UINT16 whal_txQueueGetTxOpLimit(TxQueueId_T Qid)
{
   TxQueue_T *pTxQ = &TxQueueDb[Qid];
   return pTxQ->txOpLimit;
}/* END whal_txQueueGetTxOpLimit() */

