/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/*!
 *                                                                         
 *   \file  dda_wpa_interface.c                                                
 *   \brief PURPOSE: implementation of WPA interface													       
 *                                                                         
 */


/***************************************************************************/
/*           	includes                                                   */
/***************************************************************************/


#include "wlan_pal.h"
#include "whal.h"
#include "802_11Defs.h"
#include "QoS/QoSClassifier.h"
#include "QoS/QoSACParams.h"

#include "dda_wpa_interface.h"
#include <WPA/util/linux_ipc_kernel.h>
#include "dda.h"
#include "mgmtSm.h"
#include "QoS/QoSClassifier.h"
#include "drvDbMain.h"
#include "drvDbHash.h"
#include "AcxInfoElements.h"
#include "mgmtIf.h"
#include "bssbridge.h"
#include "tx.h"
#include "dda_contexts.h"
#include "whalApi.h"
#include "mgmtSsidHandler.h"
#include "dda_wpaauth_ioctl.h"



/***************************************************************************/
/*           	static vars                                                */
/***************************************************************************/


DDA_DECLARE_WAIT_MECHANISM(WaitForWpaModule);
#define WAIT_FOR_WPA_MODULE_TIMEOUT     10000


/*******************************************************************************/
/*! \brief Gets ethernet type from the given packet buffer

	\param eth_buf	 a pointer to the ethernet packet
	
	\return ethernet type
*/  		                                                    
/********************************************************************************/
static UINT16 get_eth_type(u8 *eth_buf)
{	
	return (((UINT16)eth_buf[12]) << 8 | eth_buf[13]);
}

/*******************************************************************************************/
/*! \brief Converts an Ethernet packets to MSDU
    	\param buf 		input buffer - a pointer to ethernet packet 
	\param dataLen  ethernet packet length
	\param DA 		ignored
	\param eth_type         eth type
	
	\return  a pointer to MSDU

*/    
/*******************************************************************************************/
static mem_MSDU_T *formatEthernetToMsdu(char *buf, int dataLen, char *DA, u16 eth_type)
{
	mem_MSDU_T*		MSDUPtr;
	mem_BD_T		*BD;
	char			*dataP; /**bufP;*/
	struct sk_buff *skb;

	if(wlan_memMngrAllocMSDU(&MSDUPtr, dataLen) != OK)
		return NULL;

	MSDUPtr->dataLen = 0;
	MSDUPtr->headerLen = 0;/*sizeof(EthernetHeader_T);*/

	BD = MSDUPtr->firstBDPtr;

	BD->dataOffset = 0;

	BD->length = dataLen;

	dataP = (char *)(BD->data + BD->dataOffset);

	/* alloc and fill skb*/
	skb = alloc_skb(dataLen, GFP_ATOMIC);		
	memcpy(skb->data,buf,dataLen);
	
	BD->bufToken = (UINT32)skb;
	
	BD->data = skb->data;

	return MSDUPtr; 
}

	   
/*******************************************************************************************/
/*! \brief Converts an Ethernet packets to an EAPOL packet

	\param buf 		input buffer - a pointer to ethernet packet 
	\param dataLen  ethernet packet length
	\param DA 		ignored
	\param eth_type         eth type
	
	\return  a pointer to EAPOL MSDU
*/                             
/*******************************************************************************************/
static mem_MSDU_T *formatEthernetToEapol(char *buf, int dataLen, char *DA, UINT16 eth_type, UINT32 apToAp)	
{
	mem_MSDU_T*		MSDUPtr;
	UINT32			totalLen;
    UINT32          headerLen;
	dot11_header_T 	*pdot11Header;
	Wlan_LlcHeader_T	*pLlcHeader;
	mem_BD_T		*BD;
	char			*dataP, *bufP;
	char			apMac[MAC_ADDR_LEN];

    headerLen = (apToAp) ? sizeof(dot11_header_T) : WLAN_HDR_LEN;
    totalLen = headerLen + sizeof(Wlan_LlcHeader_T) + dataLen - sizeof(EthernetHeader_T); 

	if (wlan_memMngrAllocMSDU(&MSDUPtr, totalLen) != OK)
		return NULL;

	MSDUPtr->dataLen = totalLen;
	MSDUPtr->headerLen = headerLen;

	BD = MSDUPtr->firstBDPtr;

	BD->dataOffset = 0;

	BD->length = totalLen - dataLen;

	pdot11Header = (dot11_header_T *)(BD->data + BD->dataOffset) ;

	/* build frame control first */
	pdot11Header->fc = 0;
	
	pdot11Header->fc |= DOT11_FC_PROT_VERSION;

	pdot11Header->fc |= DOT11_FC_TYPE_DATA;

	if(apToAp) {
        pdot11Header->fc |= (TRUE << DOT11_FC_TO_DS_SHIFT);
    }

	pdot11Header->fc |= (TRUE << DOT11_FC_FROM_DS_SHIFT);

	pdot11Header->fc |= (FALSE << DOT11_FC_MORE_FRAG_SHIFT);

	pdot11Header->fc |= (FALSE << DOT11_FC_RETRY_SHIFT);

	pdot11Header->fc |= (FALSE << DOT11_FC_PWR_MGMT_SHIFT);

	pdot11Header->fc |= (FALSE << DOT11_FC_MORE_DATA_SHIFT);

	pdot11Header->fc |= (FALSE << DOT11_FC_WEP_SHIFT);

	pdot11Header->fc |= (FALSE << DOT11_FC_ORDER_SHIFT);

	/* Build the addresses */
	wdrv_dbGetBssid(apMac);

    memcpy(&pdot11Header->address1, buf, MAC_ADDR_LEN);		/* dest */
	memcpy(&pdot11Header->address2, apMac, MAC_ADDR_LEN);	/* BSSID */
    if(apToAp) {
        memcpy(&pdot11Header->address3, buf, MAC_ADDR_LEN);	/* dest */
        memcpy(&pdot11Header->address4, apMac, MAC_ADDR_LEN);	/* src */
    }
    else {
        memcpy(&pdot11Header->address3, apMac, MAC_ADDR_LEN);	/* src */
    }


	pLlcHeader = (Wlan_LlcHeader_T *)(BD->data + BD->dataOffset + headerLen);

		/* Now build the LLC header. */
	pLlcHeader->DSAP = SNAP_CHANNEL_ID;
	pLlcHeader->SSAP = SNAP_CHANNEL_ID;
	pLlcHeader->Control = LLC_CONTROL_UNNUMBERED_INFORMATION;
	pLlcHeader->OUI[0] = SNAP_OUI_RFC1042_BYTE0;
	pLlcHeader->OUI[1] = SNAP_OUI_RFC1042_BYTE1;
	pLlcHeader->OUI[2] = SNAP_OUI_RFC1042_BYTE2;
	pLlcHeader->Type = htons(eth_type);


	bufP = buf + 14;

	dataP = (char *)(BD->data + BD->dataOffset + headerLen + sizeof(Wlan_LlcHeader_T));

	wlan_memMngrFillField(&dataP, bufP, dataLen, &BD);

	return MSDUPtr; 
}

/*******************************************************************************************/
/*! \brief Sends hello message to WPA process
	
	\return  OK/NOK
*/
/*******************************************************************************************/
STATUS dda_wpa_drv_send_hello(void)
{
	wpaMessage_T	msg;
	UINT32 rc;

	msg.type = WPA_DRV_HELLO;

	rc = ipc_connect_buf_write((void *)&msg, sizeof(wpaMessage_T));
	if (rc)
	{
		printk("WDRV_WPA_KERNEL: (%s) Error in 'ipc_write_port'. Error Code = %d\n",__func__, rc);
		return NOK;
	}
	return OK;

}

/*******************************************************************************************/
/*! \brief Restores mapping keys for each station in recovery list 
	
	\return  OK
*/
/*******************************************************************************************/
STATUS dda_wpa_restart_module(void)
{
	int i,key_idx;
	char mac[6];
	wpa_temporal_key_t  broadcast_key, sta_key;
	UINT8 cipher, wepDefKeyId;
    UINT16 keySize;
    UINT8  inUse;
	UINT8  wep_def_key[WPA_MAX_WEP_KEY_LEN];
    wpa_sta_type_t  type;

	/* reconfig all stations keys */
	for(i = 0; i<MAX_STATIONS; i++)
	{
        wdrv_dbGetKeyInUse(i, &inUse);
		if(inUse == TRUE)
		{
			wdrv_dbGetStaMacAddress(i, mac);
            wdrv_dbGetStationEncryptionKey(i, &keySize, (UINT8 *)&sta_key, &cipher);
			dda_wpa_set_mapping_key(mac, cipher, &sta_key);	
		}
	}
	
	
	for(i=0; i<NUM_MULTIPLE_SSID; i++)
	{
		if (wdrv_mgmt_ssidIsActive(i) == TRUE) 
		{
            wdrv_mgmt_ssidGetSecurityType(i, &type);
			wdrv_mgmt_ssidGetBroadcastSuiteType(&cipher,i);	
			/* reconfig AES/TKIP broadcast keys*/
			if( (cipher == WPA_CIPHER_TKIP) || (cipher == WPA_CIPHER_CCMP) || 
                ((cipher == CIPHER_WEP) && (type & WPA_STA_TYPE_WEP_8021X)) )
			{
				wdrv_mgmt_ssidGetBroadcastKey(i,&broadcast_key,&key_idx);
				dda_wpa_set_default_key(key_idx,cipher,type,i,&broadcast_key);
			}
			/* reconfig WEP default keys */
			else if (cipher == CIPHER_WEP) 
			{
				for (key_idx=0; key_idx< WEP_NUM_DEFAULT_KEYS; key_idx++ ) 
				{
					 wdrv_dbGetDefaultWepKey(key_idx, &keySize, wep_def_key);
					 whal_apiSetDefaultKey(WEP_ACTION_ADD, (UINT8)key_idx, keySize, wep_def_key, i);
				}
				wdrv_dbGetWepDefaultKeyId(&wepDefKeyId);
				whal_apiSetDefaultKeyId(wepDefKeyId);
			}
		}
	}
	return OK;
}

/*******************************************************************************************/
/*! \brief Notify the WPA process that new station connected
	
	\param idx 	station index

	\return  OK/NOK
*/
/*******************************************************************************************/
STATUS dda_wpa_notify_station_connect(UINT16 idx)
{

	char   mac[MAC_ADDR_LEN];
	wpaMessage_T	msg;
	UINT32 rc;
	int stat;


	if((stat=wdrv_mgmt_ssidIsStationNeedAuthSupprt(idx)) == NOK)
	{
		wdrv_dbGetStaMacAddress(idx, mac);
		wdrv_mgmtMacAuthorize(mac);
		return OK;
	}
	else if(stat == 0xff)
		return NOK;
	else  
	{
		if(wdrv_mgmt_ssidGetStationSsidIndex(idx,&msg.profile_index ) != OK)
		{
			printk("dda_wpa_notify_station_connect:cant find station index %d ,profile_index\n",idx);
			return NOK;
		}
		
		msg.type = NOTIFY_STATION_CONNECT;
		wdrv_dbGetStaMacAddress(idx, (char*)msg.content.stationConnect.mac);
		msg.content.stationConnect.index = idx;
		wdrv_dbGetStationRsnIe(idx,&(msg.content.stationConnect.rsne));
		
		rc = ipc_connect_buf_write((void *)&msg, sizeof(wpaMessage_T));
		if (rc == NOK)
		{
			printk("WDRV_WPA_KERNEL: (%s) Error in 'ipc_write_port'. Error Code = %d\n",__func__, rc);
			return NOK;
		}
	}
	return OK;
}

/*******************************************************************************************/
/*! \brief Notify the WPA process that station disconnected
	
	\param idx 	station index

	\return  OK/NOK
*/
/*******************************************************************************************/
STATUS dda_wpa_notify_station_disconnect(UINT16 idx) {
	wpaMessage_T	msg;
	UINT32 rc;
	int stat;

	if((stat =wdrv_mgmt_ssidIsStationNeedAuthSupprt(idx)) == NOK)
	{
		return OK;	
	}
	else if(stat == 0xff)
		return NOK;

	if(wdrv_mgmt_ssidGetStationSsidIndex(idx,&msg.profile_index ) != OK)
	{
		printk("dda_wpa_notify_station_disconnect:cant find station index %d ,profile_index\n",idx);
		return NOK;
	}
	
	msg.type = NOTIFY_STATION_DIS_CONNECT;
	wdrv_dbGetStaMacAddress(idx, msg.content.stationDisConnect.mac);
	
	rc = ipc_connect_buf_write((void *)&msg, sizeof(wpaMessage_T));
	if (rc)
	{
		printk("WDRV_WPA_KERNEL: (%s) Error in 'ipc_write_port'. Error Code = %d\n",__func__, rc);
		return NOK;
	}

	return OK;
}


/*******************************************************************************************/
/*! \brief Notify the WPA process that first data from station received
	
	\param idx 		station index
	\param isEapol	a flag which is TRUE if first packet received was Eapol packet

	\return  OK/NOK
*/
/*******************************************************************************************/
STATUS dda_wpa_first_dataFromStation(UINT16 idx, BOOL isEapol) {

	wpaMessage_T	msg;
	UINT32 rc;
	int stat;


	if(wdrv_mgmt_ssidGetStationSsidIndex(idx,&msg.profile_index ) != OK)
	{
		printk("dda_wpa_first_dataFromStation:cant find station profile.Index %d\n",idx);
		return NOK;
	}

	if((stat =wdrv_mgmt_ssidIsStationNeedAuthSupprt(idx)) == NOK)
	{
		return OK;	
	}
	else if(stat == 0xff)
		return NOK;

	
	msg.type = FIRST_DATA_FROM_STATION;
	wdrv_dbGetStaMacAddress(idx, msg.content.firstDataFromStation.mac);
	msg.content.firstDataFromStation.isEapol = isEapol;
		
	rc = ipc_connect_buf_write((void *)&msg, sizeof(wpaMessage_T));
	if (rc)
	{
		printk("WDRV_WPA_KERNEL: (%s) Error in 'ipc_write_port'. Error Code = %d\n",__func__, rc);
		return NOK;
	}

	return OK;
}

/*******************************************************************************************/
/*! \brief Notify the WPA process that MIC falure occured for some station
	
	\param idx 	station index

	\return  OK/NOK
*/
/*******************************************************************************************/
STATUS dda_wpa_rxMicFailure(UINT16 idx) {

	wpaMessage_T	msg;
	UINT32 rc;
	int stat;
	
	FUNC;
	
	if((stat =wdrv_mgmt_ssidIsStationNeedAuthSupprt(idx)) == NOK)
	{
		return OK;	
	}
	else if(stat == 0xff)
		return NOK;

	if(wdrv_mgmt_ssidGetStationSsidIndex(idx,&msg.profile_index ) != OK)
	{
		printk("dda_wpa_rxMicFailure:cant find station index %d ,profile_index\n",idx);
		return NOK;
	}
	
	msg.type = MIC_FAILURE;
	wdrv_dbGetStaMacAddress(idx, msg.content.micFailure.mac);
	
	rc = ipc_connect_buf_write((void *)&msg, sizeof(wpaMessage_T));
	if (rc)
	{
		printk("WDRV_WPA_KERNEL: (%s) Error in 'ipc_write_port'. Error Code = %d\n",__func__, rc);
		return NOK;
	}

	return OK;
}

/*******************************************************************************************/
/*! \brief Sends security attack flag to WPA
	
	\param index 	station index
	\param flag	    security attack flag

	\return  OK/NOK
*/
/*******************************************************************************************/
STATUS   dda_wpa_SetStationSecAttack(UINT16 index,BOOL flag) {
	wpaMessage_T	msg;
	UINT32 rc;
	int stat;
	
	if((stat =wdrv_mgmt_ssidIsStationNeedAuthSupprt(index)) == NOK)
	{
		return OK;	
	}
	else if(stat == 0xff)
		return NOK;


	if(wdrv_mgmt_ssidGetStationSsidIndex(index,&msg.profile_index ) != OK)
	{
		printk("dda_wpa_SetStationSecAttack:cant find station index %d ,profile_index\n",index);
		return NOK;
	}
	
	msg.type = SET_STA_ATTACK;
	msg.content.StaAttack.flag  = flag;
	msg.content.StaAttack.index = index;
	
	rc = ipc_connect_buf_write((void *)&msg, sizeof(wpaMessage_T));
	if (rc)
	{
		printk("WDRV_WPA_KERNEL: (%s) Error in 'ipc_write_port'. Error Code = %d\n",__func__, rc);
		return NOK;
	}

	return OK;

}


/*******************************************************************************************/
/*! \brief Asks WPA if a given station can associate
	
	\param index 				station index
	\param info_elem		station information element
	\param info_elem_len	iformation element length

	\return  OK/NOK
*/
/*******************************************************************************************/
STATUS dda_wpa_canStationAss(UINT16 index,UINT8 *info_elem, int info_elem_len)
{

	
	wpaMessage_T	msg;
	int rc;
	int stat;
	
	if((stat =wdrv_mgmt_ssidIsStationNeedAuthSupprt(index)) == NOK)
	{
		dda_wpa_canStationAssRes(index,1);
		setBcastState(TRUE);
		return OK;	
	}
	else if(stat == 0xff)
		return NOK;

	if(wdrv_mgmt_ssidGetStationSsidIndex(index,&msg.profile_index ) != OK)
	{
		printk("dda_wpa_canStationAss:cant find station index %d ,profile_index\n",index);
		return NOK;
	}

	msg.type = CAN_STA_ASS;
	

	msg.content.canStaAss.index = index;
	msg.content.canStaAss.info_elem_len = info_elem_len;

	memcpy(msg.content.canStaAss.info_elem,info_elem,info_elem_len);
	
	rc = ipc_connect_buf_write((void *)&msg, sizeof(wpaMessage_T));
	
	if (rc)
	{
		printk("WDRV_WPA_KERNEL: (%s) Error in 'ipc_write_port'. Error Code = %d\n",__func__, rc);
		return NOK;
	}

	return OK;
}



/****************************  driver api for user cmd    ***********************************************/

/*******************************************************************************************/
/*! \brief receive a msg from wpa user mode process,update driver according
			to cmd type
	
	\param	message  input message buffer
	\param  len		 message len

	\return  OK/NOK
*/
/*******************************************************************************************/
STATUS dda_wpa_receive(void *message, UINT32 len) {

	wpaMessage_T *msg = (wpaMessage_T *)message;
	unsigned int flags;

	if(len != sizeof(wpaMessage_T))
	{
		printk("dda_wpa_receive:receive msg with error len %d,expected %d\n",len,sizeof(wpaMessage_T));
		return NOK; 
    }

	switch(msg->type)
	{
	case EAPOL_FRAME:
		dda_wpa_send_eapol(msg->content.eapolFrame.pEapol,
						   msg->content.eapolFrame.eapolDataLen,
						   msg->content.eapolFrame.encrypted);
		break;
	case SET_DEFAULT_KEY:	
		dda_wpa_set_default_key(msg->content.setDefaultKey.index,
								msg->content.setDefaultKey.cipher,
                                msg->content.setDefaultKey.staType,
								msg->content.setDefaultKey.profile_index ,
								&msg->content.setDefaultKey.temporal_key);
		break;
	case SET_DEFAULT_KEY_ID:	
		dda_wpa_set_default_key_id(msg->content.setDefaultKeyId.index);
		break;
	case REMOVE_DEFAULT_KEY:
        dda_wpa_remove_default_key(msg->content.removeDefaultKey.index,
								   msg->content.removeDefaultKey.cipher,
								   msg->content.removeDefaultKey.profile_index);
		break;
	case SET_MAP_KEY:
        /*printk("\n%s : Set cipher.MAC %02hX:%02hX:%02hX:%02hX:%02hX:%02hX cipher = %d.\n", __FUNCTION__, 
               msg->content.setMapKey.mac[0], msg->content.setMapKey.mac[1], msg->content.setMapKey.mac[2], 
               msg->content.setMapKey.mac[3], msg->content.setMapKey.mac[4], msg->content.setMapKey.mac[5], 
               msg->content.setMapKey.cipher);*/
        dda_wpa_set_mapping_key(msg->content.setMapKey.mac,
								msg->content.setMapKey.cipher,
								&(msg->content.setMapKey.temporal_key));
		break;
	case REMOVE_MAP_KEY:
		dda_wpa_remove_mapping_key(msg->content.removeMapKey.mac,
								   msg->content.removeMapKey.cipher
								  );
		break;
	case DEAUTH_STA:
		dda_wpa_deauth_sta(msg->content.deAssAuthSta.mac);
		break;
	case DISASS_STA:
		dda_wpa_disassoc_sta(msg->content.deAssAuthSta.mac);
		break;
	case BLOCK_UNICAST:
		dda_wpa_block_unicast_traffic(msg->content.blockUcast.block,msg->content.blockUcast.mac);
		break;
	case BLOCK_MULTYCAST:
		dda_wpa_block_multicast_traffic(msg->content.blockMcast.block);
		break;
	case CAN_STA_ASS_RES:
		PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); 
		dda_wpa_canStationAssRes(msg->content.canStaAssRes.index,msg->content.canStaAssRes.flag);
		PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags); 
		break;
	case WPA_HELLO:
		{
			int i;
			for(i = 0; i < TIAP_MAX_NUM_OF_SUPPORTED_PROFILES;i++)
			{
				/* wpa info elem*/
				if(msg->content.hello.info_len)
					wdrv_mgmt_ssidSetSsn(msg->content.hello.info_len[i] ,msg->content.hello.info[i],i);
				else
					wdrv_mgmt_ssidSetSsn(0,NULL,i);
				
				/* rsn info elem*/
				if(msg->content.hello.rsn_len)
					wdrv_mgmt_ssidSetRsne(msg->content.hello.rsn_len[i] ,msg->content.hello.rsn[i],i);
				else
					wdrv_mgmt_ssidSetRsne(0,NULL,i);

				/* priv_opt_implement*/
				if(msg->content.hello.cipher[i] == 0/*TIAP_WPA_CIPHER_NON*/)
					wdrv_mgmt_ssidSetPrivOpt(i,FALSE);
				else
					wdrv_mgmt_ssidSetPrivOpt(i,TRUE);

                /* Security Type */
                wdrv_mgmt_ssidSetSecurityType(i, msg->content.hello.secType[i]);
			}

			PAL_sleepLong(1);/* just in case the driver not waiting for hello yet...*/
			dda_wpa_receive_hello();
			break;
		}
        
    case WPA_SET_PROFILE_PRIV_OPT:
        wdrv_mgmt_ssidSetPrivOpt(msg->content.profilePrivOpt.profile_index,
                                 msg->content.profilePrivOpt.priv_opt);
        break;
	default:
		printk("dda_wpa_receive:receive unknown msg to handle ,type %d\n",msg->type);
		
		return NOK; 
	}

	return OK;
}

/*******************************************************************************************/
/*! \brief Send to the driver the response from WPA if station can associate
	
	\param	index  station index
	\param  ret	   response flag

*/
/*******************************************************************************************/
void dda_wpa_canStationAssRes(UINT16 index,int ret)
{

	mgmtMessage_T message;
	message.type = WPA_ASSOC_REQ_REP;
	message.content.wpaAssCheckRep.index = index;
	message.content.wpaAssCheckRep.flag  = ret;

	wdrv_DDA_mgmt_setEvent(&message);
	return;
}


/*******************************************************************************************/
/*! \brief Sets station default keys according to the cipher type
	
	\param  index 	 		station index
	\param	cipher   		wpa cipher
	\param  temporal_key	input keys structure

*/
/*******************************************************************************************/
void dda_wpa_set_default_key(int index,wpa_cipher_t cipher,wpa_sta_type_t staType,int profile_index,wpa_temporal_key_t *temporal_key)
{
	char    bcastMac[MAC_ADDR_LEN] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};
    int     wepKeyLen = 0;

	if (index > 3)
	{
		printf("%s: invalid default key index: %d!\n",__FUNCTION__, index);
		return;
	}
	
	/* if cipher is TKIP then we have to configure the key in one of the mapping keys, 
		and in addition to set the broadcast MIC key */
	if (cipher == WPA_CIPHER_TKIP)
	{	
		char key_buf[WPA_TKIP_ENCRYPTION_KEY_LEN*2];
		
       /* must be last incase of restart workaround*/
		memcpy(key_buf,temporal_key->data.tkip.tkip_key,16);
		memcpy(&key_buf[16],temporal_key->data.tkip.rx_mic_key,8);
		memcpy(&key_buf[24],temporal_key->data.tkip.tx_mic_key,8);
   
        /* mic failure workaround - first insert dummy key for icv fail at station, then install mic key 
           and then the right tkip key */
        whal_apiSetMappingKey(KEY_ADD_OR_REPLACE,
							  bcastMac,
							  WPA_TKIP_ENCRYPTION_KEY_LEN*2,
							  KEY_TKIP_MIC_GROUP,
							  profile_index/*ENTRY_INDEX_IGNORE*/,
							  index, key_buf);

        wdrv_mgmt_ssidSetBroadcastSuiteType(CIPHER_TKIP,profile_index);
		if(REPORT_LEVEL(RL_WPA)) 
		{
			wdrv_utilsReport(RL_MGMT, PRI_DEBUG, "set_default_key: index :%d tx group key %x,%x,%x,%x,%x,%x,%x,%x,%x,%x,%x,%x,%x,%x,%x,%x\n",index,temporal_key->data.tkip.tkip_key[0],temporal_key->data.tkip.tkip_key[1],temporal_key->data.tkip.tkip_key[2],temporal_key->data.tkip.tkip_key[3],temporal_key->data.tkip.tkip_key[4],temporal_key->data.tkip.tkip_key[5],temporal_key->data.tkip.tkip_key[6],temporal_key->data.tkip.tkip_key[7], temporal_key->data.tkip.tkip_key[8],temporal_key->data.tkip.tkip_key[9],temporal_key->data.tkip.tkip_key[10],temporal_key->data.tkip.tkip_key[11],temporal_key->data.tkip.tkip_key[12],temporal_key->data.tkip.tkip_key[13],temporal_key->data.tkip.tkip_key[14],temporal_key->data.tkip.tkip_key[15]);
			wdrv_utilsReport(RL_MGMT, PRI_DEBUG, "set_default_key: tx group mic %x,%x,%x,%x,%x,%x,%x,%x\n",temporal_key->data.tkip.tx_mic_key[0],temporal_key->data.tkip.tx_mic_key[1],temporal_key->data.tkip.tx_mic_key[2],temporal_key->data.tkip.tx_mic_key[3],temporal_key->data.tkip.tx_mic_key[4],temporal_key->data.tkip.tx_mic_key[5],temporal_key->data.tkip.tx_mic_key[6],temporal_key->data.tkip.tx_mic_key[7]);
		}

	}

	else if(cipher == WPA_CIPHER_CCMP)
	{
      /* DB must be set before FW incase of restart workaround */
        wdrv_mgmt_ssidSetBroadcastSuiteType(CIPHER_CCMP,profile_index);
	    wdrv_mgmt_ssidSetBroadcastKey(profile_index,temporal_key,index); 	

		/* mic failure workaround - first insert dummy key for icv fail at station, then install mic key 
           and then the right tkip key */
        whal_apiSetMappingKey(KEY_ADD_OR_REPLACE, bcastMac, WPA_AES_KEY_LEN, KEY_AES_GROUP, profile_index, index, temporal_key->data.aes.key);
		if(REPORT_LEVEL(RL_WPA)) 
		{
			wdrv_utilsReport(RL_MGMT, PRI_DEBUG, "set_default_key: profile_index = %d index :%d tx group key %x,%x,%x,%x,%x,%x,%x,%x,%x,%x,%x,%x,%x,%x,%x,%x\n",profile_index,index,temporal_key->data.tkip.tkip_key[0],temporal_key->data.tkip.tkip_key[1],temporal_key->data.tkip.tkip_key[2],temporal_key->data.tkip.tkip_key[3],temporal_key->data.tkip.tkip_key[4],temporal_key->data.tkip.tkip_key[5],temporal_key->data.tkip.tkip_key[6],temporal_key->data.tkip.tkip_key[7], temporal_key->data.tkip.tkip_key[8],temporal_key->data.tkip.tkip_key[9],temporal_key->data.tkip.tkip_key[10],temporal_key->data.tkip.tkip_key[11],temporal_key->data.tkip.tkip_key[12],temporal_key->data.tkip.tkip_key[13],temporal_key->data.tkip.tkip_key[14],temporal_key->data.tkip.tkip_key[15]);
			wdrv_utilsReport(RL_MGMT, PRI_DEBUG, "set_default_key: tx group mic %x,%x,%x,%x,%x,%x,%x,%x\n",temporal_key->data.tkip.tx_mic_key[0],temporal_key->data.tkip.tx_mic_key[1],temporal_key->data.tkip.tx_mic_key[2],temporal_key->data.tkip.tx_mic_key[3],temporal_key->data.tkip.tx_mic_key[4],temporal_key->data.tkip.tx_mic_key[5],temporal_key->data.tkip.tx_mic_key[6],temporal_key->data.tkip.tx_mic_key[7]);
		}
	}
	else
	{
		wdrv_mgmt_ssidSetBroadcastSuiteType(CIPHER_WEP,profile_index);
        
        switch(cipher) 
        {
        case WPA_CIPHER_WEP_40:
            wepKeyLen = WPA_WEP_40_KEY_LEN;
        	break;
        case WPA_CIPHER_WEP_104:
            wepKeyLen = WPA_WEP_104_KEY_LEN;
        	break;
        case WPA_CIPHER_WEP_232:
            wepKeyLen = WPA_WEP_232_KEY_LEN;
            break;
        default:
        }
		
        if(staType & WPA_STA_TYPE_WEP_8021X)
        {
            wdrv_mgmt_ssidSetBroadcastKey(profile_index,temporal_key,index);
            whal_apiSetMappingKey(KEY_ADD_OR_REPLACE, bcastMac, wepKeyLen, KEY_WEP_GROUP, profile_index, index, temporal_key->data.wep.wep_key);
        }
        else
        {
            wdrv_dbSetDefaultWepKey((UINT8)index, wepKeyLen, temporal_key->data.wep.wep_key);
            whal_apiSetDefaultKey(WEP_ACTION_ADD, (UINT8)index, wepKeyLen, temporal_key->data.wep.wep_key, profile_index);
        }
        if(REPORT_LEVEL(RL_WPA)) 
        {
            wdrv_utilsReport(RL_MGMT, PRI_DEBUG, "set_default_key WEP_%d  key at index %d\n\n",
                wepKeyLen*8, index);
        }
    }
}

/*******************************************************************************************/
/*! \brief Sets station default key ID
	
	\param  index 	 		station index

*/
/*******************************************************************************************/
void dda_wpa_set_default_key_id(int index)
{
	if (index > 3)
	{
		printf("%s: invalid group key index: %d!\n",__FUNCTION__, index);
		return;
	}

    wdrv_dbSetWepDefaultKeyId((UINT8)index);
	whal_apiSetDefaultKeyId((UINT8)index);
}
/*******************************************************************************************/
/*! \brief Removes station default keys according to the cipher type
	
	\param  index 	 		station index
	\param	cipher   		wpa cipher
   
*/
/*******************************************************************************************/
void dda_wpa_remove_default_key(int index,wpa_cipher_t cipher,int profile_index)
{
	char bcastMac[MAC_ADDR_LEN] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};
	wpa_temporal_key_t	temporal_key;

	if (index > 3) 
	{
		printf("%s: invalid default key index: %d!\n",__FUNCTION__, index);
		return;
	}

	memset(&temporal_key, 0, sizeof(wpa_temporal_key_t));
	
	/* if cipher is TKIP then we have to remove the key from one of the mapping keys, 
			and in addition to reset the broadcast MIC key */
	wdrv_mgmt_ssidSetBroadcastSuiteType(CIPHER_NONE,profile_index);

	if (cipher == WPA_CIPHER_TKIP)
	{	
        /* DB must be set before FW incase of restart workaround */
        whal_apiSetMappingKey(KEY_ADD_OR_REPLACE,
							  bcastMac,
							  WPA_TKIP_ENCRYPTION_KEY_LEN*2,
							  KEY_TKIP_MIC_GROUP,
							  profile_index/*ENTRY_INDEX_IGNORE*/,
							  index, (char*)&temporal_key);
	}
	else if (cipher == WPA_CIPHER_CCMP)
	{
        whal_apiSetMappingKey(KEY_REMOVE_V, bcastMac, WPA_TKIP_ENCRYPTION_KEY_LEN, KEY_AES_GROUP,profile_index, index, temporal_key.data.aes.key);
	}  	
	else
	{
		if (cipher == WPA_CIPHER_WEP_40)
		{
            wdrv_dbSetDefaultWepKey((UINT8)index, 0, NULL);
			whal_apiSetDefaultKey(KEY_REMOVE_V, (UINT8)index, WPA_WEP_40_KEY_LEN, temporal_key.data.wep.wep_key, profile_index);
		}
		else if (cipher == WPA_CIPHER_WEP_104)
		{
            wdrv_dbSetDefaultWepKey((UINT8)index,0,NULL);
			whal_apiSetDefaultKey(KEY_REMOVE_V, (UINT8)index, WPA_WEP_104_KEY_LEN, temporal_key.data.wep.wep_key, profile_index);
		}
		else if (cipher == WPA_CIPHER_WEP_232)
		{
            wdrv_dbSetDefaultWepKey((UINT8)index,0,NULL);
			whal_apiSetDefaultKey(KEY_REMOVE_V, (UINT8)index, WPA_WEP_232_KEY_LEN, temporal_key.data.wep.wep_key, profile_index);
		}
	}
}

/*******************************************************************************************/
/*! \brief Sets station mapping keys according to the cipher type
	
	\param  mac 	 		station mac
	\param	cipher   		wpa cipher
	\param  temporal_key	keys structure

*/
/*******************************************************************************************/
void dda_wpa_set_mapping_key(char *mac,wpa_cipher_t cipher,wpa_temporal_key_t *temporal_key)
{
	UINT16	index;

	if ( wdrv_dbFindStaIndex(mac, &index) != OK) 
	{
		printf("%s: Couldn't find STA index, %02hX:%02hX:%02hX:%02hX:%02hX:%02hX\n",
			   __func__,mac[0], mac[1],mac[2],mac[3],mac[4],mac[5]);
		return; 
    }

	/* if cipher is TKIP then we have to set the station TX & RX MIC key in addition 
		to configuring the key to the HAL */
	if (cipher == WPA_CIPHER_TKIP)
	{	
		char key_buf[WPA_TKIP_ENCRYPTION_KEY_LEN*2];
		
       /* must be last incase of restart workaround*/
        wdrv_dbSetStationEncryptionKey(index, CIPHER_TKIP, sizeof(*temporal_key),(UINT8 *)temporal_key);
		memcpy(key_buf,temporal_key->data.tkip.tkip_key,16);
		memcpy(&key_buf[16],temporal_key->data.tkip.rx_mic_key,8);
		memcpy(&key_buf[24],temporal_key->data.tkip.tx_mic_key,8);


        whal_apiSetMappingKey(KEY_ADD_OR_REPLACE,
							  mac,
							  WPA_TKIP_ENCRYPTION_KEY_LEN*2,
							  KEY_TKIP_MIC_PAIRWISE/*KEY_TKIP_PAIRWISE*/,
							  0/*profile_index*/,
							  0,
							  key_buf);

		if(REPORT_LEVEL(RL_WPA)) 
		{
			wdrv_utilsReport(RL_MGMT, PRI_DEBUG, "set_mapping_key TX: %x,%x,%x,%x,%x,%x,%x,%x\n",temporal_key->data.tkip.tx_mic_key[0],temporal_key->data.tkip.tx_mic_key[1],temporal_key->data.tkip.tx_mic_key[2],temporal_key->data.tkip.tx_mic_key[3],temporal_key->data.tkip.tx_mic_key[4],temporal_key->data.tkip.tx_mic_key[5],temporal_key->data.tkip.tx_mic_key[6],temporal_key->data.tkip.tx_mic_key[7]);
			wdrv_utilsReport(RL_MGMT, PRI_DEBUG, "set_mapping_key RX: %x,%x,%x,%x,%x,%x,%x,%x\n",temporal_key->data.tkip.rx_mic_key[0],temporal_key->data.tkip.rx_mic_key[1],temporal_key->data.tkip.rx_mic_key[2],temporal_key->data.tkip.rx_mic_key[3],temporal_key->data.tkip.rx_mic_key[4],temporal_key->data.tkip.rx_mic_key[5],temporal_key->data.tkip.rx_mic_key[6],temporal_key->data.tkip.rx_mic_key[7]);
		}
	}
	else if (cipher == WPA_CIPHER_CCMP)
	{
		/* must be last incase of restart workaround*/
        wdrv_dbSetStationEncryptionKey(index, CIPHER_CCMP, sizeof(*temporal_key),(UINT8 *)temporal_key);
	    whal_apiSetMappingKey(KEY_ADD_OR_REPLACE, mac, WPA_AES_KEY_LEN, KEY_AES_PAIRWISE, 0/*profile_index*/, 0, temporal_key->data.aes.key);
  
		if(REPORT_LEVEL(RL_WPA)) 
		{
			wdrv_utilsReport(RL_MGMT, PRI_DEBUG, "set_mapping_key CCMP  key: %x,%x,%x,%x,%x,%x,%x,%x\n%x,%x,%x,%x,%x,%x,%x,%x\n\n",temporal_key->data.aes.key[0],temporal_key->data.aes.key[1],temporal_key->data.aes.key[2],temporal_key->data.aes.key[3],temporal_key->data.aes.key[4],temporal_key->data.aes.key[5],temporal_key->data.aes.key[6],temporal_key->data.aes.key[7]
																			 ,temporal_key->data.aes.key[8],temporal_key->data.aes.key[9],temporal_key->data.aes.key[10],temporal_key->data.aes.key[11],temporal_key->data.aes.key[12],temporal_key->data.aes.key[13],temporal_key->data.aes.key[14],temporal_key->data.aes.key[15]);
		}

	}

	else
	{
		if (cipher == WPA_CIPHER_WEP_40)
		{
            wdrv_dbSetStationEncryptionKey(index, cipher, WPA_WEP_40_KEY_LEN,temporal_key->data.wep.wep_key);

            whal_apiSetMappingKey(KEY_ADD_OR_REPLACE, mac, WPA_WEP_40_KEY_LEN, KEY_WEP_ADDR, ENTRY_INDEX_IGNORE, 0, temporal_key->data.wep.wep_key);

            if(REPORT_LEVEL(RL_WPA)) 
            {
                wdrv_utilsReport(RL_MGMT, PRI_DEBUG, "set_mapping_key WEP_40  key\n\n");
            }

		}
		else if (cipher == WPA_CIPHER_WEP_104)
		{
            wdrv_dbSetStationEncryptionKey(index, cipher, WPA_WEP_104_KEY_LEN,temporal_key->data.wep.wep_key);

            whal_apiSetMappingKey(KEY_ADD_OR_REPLACE, mac, WPA_WEP_104_KEY_LEN, KEY_WEP_ADDR, ENTRY_INDEX_IGNORE, 0, temporal_key->data.wep.wep_key);

            if(REPORT_LEVEL(RL_WPA)) 
            {
                wdrv_utilsReport(RL_MGMT, PRI_DEBUG, "set_mapping_key WEP_104  key\n\n");
            }
		}
		else if (cipher == WPA_CIPHER_WEP_232)
		{
            wdrv_dbSetStationEncryptionKey(index, cipher, WPA_WEP_232_KEY_LEN,temporal_key->data.wep.wep_key);

            whal_apiSetMappingKey(KEY_ADD_OR_REPLACE, mac, WPA_WEP_232_KEY_LEN, KEY_WEP_ADDR, ENTRY_INDEX_IGNORE, 0, temporal_key->data.wep.wep_key);

            if(REPORT_LEVEL(RL_WPA)) 
            {
                wdrv_utilsReport(RL_MGMT, PRI_DEBUG, "set_mapping_key WEP_232  key\n\n");
            }
		}
	}
}

/*******************************************************************************************/
/*! \brief Removes station mapping keys according to the cipher type
	
	\param  mac 	 		station MAC
	\param	cipher   		wpa cipher


*/
/*******************************************************************************************/
void dda_wpa_remove_mapping_key(ether_mac_t mac,wpa_cipher_t cipher)
{
	UINT16	index;
	wpa_temporal_key_t	temporal_key;
    BOOL    staRemoved = FALSE;

	memset(&temporal_key, 0, sizeof(wpa_temporal_key_t));
	
	if(wdrv_dbFindStaIndex(mac, &index) != OK) {
        staRemoved = TRUE;
	}

	/* if cipher is TKIP then we have to remove the station TX & RX MIC key in addition 
		to removing the key from the HAL */
	if (cipher == WPA_CIPHER_TKIP)
	{	
        if(!staRemoved) {
            wdrv_dbSetStationEncryptionKey(index, CIPHER_NONE, 0, NULL);
    		wdrv_dbSetStationTxMicKey(index, NULL);
    		wdrv_dbSetStationRxMicKey(index, NULL);
        }
  
        whal_apiSetMappingKey(KEY_REMOVE_V, mac, WPA_TKIP_ENCRYPTION_KEY_LEN*2,KEY_TKIP_MIC_PAIRWISE/*KEY_TKIP_PAIRWISE*/, ENTRY_INDEX_IGNORE, index, (char*)&temporal_key);
	}
	else if (cipher == WPA_CIPHER_CCMP)
	{	
        if(!staRemoved) {
            wdrv_dbSetStationEncryptionKey(index, CIPHER_NONE, 0, NULL);
        }
	    whal_apiSetMappingKey(KEY_REMOVE_V, mac, WPA_AES_KEY_LEN, KEY_AES_PAIRWISE, ENTRY_INDEX_IGNORE, 0, temporal_key.data.aes.key);
	}
	else
	{
		if (cipher == WPA_CIPHER_WEP_40)
		{
            if(!staRemoved) {
                wdrv_dbSetStationEncryptionKey(index, CIPHER_NONE, 0,NULL);
            }
            whal_apiSetMappingKey(KEY_REMOVE_V, mac, WPA_WEP_40_KEY_LEN, KEY_WEP_ADDR, ENTRY_INDEX_IGNORE, index, temporal_key.data.wep.wep_key);
		}
		else if (cipher == WPA_CIPHER_WEP_104)
		{
            if(!staRemoved) {
                wdrv_dbSetStationEncryptionKey(index, CIPHER_NONE, 0,NULL);
            }
            whal_apiSetMappingKey(KEY_REMOVE_V, mac, WPA_WEP_104_KEY_LEN, KEY_WEP_ADDR, ENTRY_INDEX_IGNORE, index, temporal_key.data.wep.wep_key);
		}
		else if (cipher == WPA_CIPHER_WEP_232)
		{
            if(!staRemoved) {
                wdrv_dbSetStationEncryptionKey(index, CIPHER_NONE, 0,NULL);
            }
            whal_apiSetMappingKey(KEY_REMOVE_V, mac, WPA_WEP_232_KEY_LEN, KEY_WEP_ADDR, ENTRY_INDEX_IGNORE, index, temporal_key.data.wep.wep_key);
		}
	}
}

/*******************************************************************************************/
/*! \brief Sends Deauthentication message to station in response to the message received from WPA
	
	\param  mac 	 		station MAC address

*/
/*******************************************************************************************/
void dda_wpa_deauth_sta(ether_mac_t mac)
{
	wdrv_mgmtDeAuthMng(mac, STATUS_UNSPECIFIED);
}

/*******************************************************************************************/
/*! \brief Sends Dissasociation message to station in response to the message received from WPA
	
	\param  mac 	 		station MAC address

*/
/*******************************************************************************************/
void dda_wpa_disassoc_sta(ether_mac_t mac)
{
	wdrv_mgmtDisAssocMng(mac, STATUS_UNSPECIFIED);
}

/*******************************************************************************************/
/*! \brief Sets block multicast traffic flag 

	\param block	input flag
*/
/*******************************************************************************************/
void dda_wpa_block_multicast_traffic(int block)
{
	if (block)
		setBcastState(FALSE);
	else
		setBcastState(TRUE);

}

/*******************************************************************************************/
/*! \brief Sets block unicast traffic flag 

	\param block	input flag
	\param mac		MAC of the station to be blocked
*/
/*******************************************************************************************/
void dda_wpa_block_unicast_traffic(int block,ether_mac_t mac)
{
	if (block)
		wdrv_mgmtMacDeAuthorize(mac);
    else {
		wdrv_mgmtMacAuthorize(mac);
        PAL_sleepLong(10);
    }
}

/*******************************************************************************************/
/*! \brief Gets max number of mapping keys

	\return  number of mapping keys
*/
/*******************************************************************************************/
int  dda_wpa_get_max_num_of_mapping_keys(void)
{
	return MAX_STATIONS;
}

/*******************************************************************************************/
/*! \brief Sends an EAPOL packet to WLAN

	\param buf		input EAPOL packet 
	\param len		input packet length
	\param encrypt	an encryption flag
	
	\return  len upon success, 0 otherwise
*/
/*******************************************************************************************/
int  dda_wpa_send_eapol(void *buf, int len, int encrypt)
{
	UINT16 idx;
	ULONG txFlags;
	int rc;

	UINT16 eth_type;

	char DA[MAC_ADDR_LEN];
	mem_MSDU_T *pMsdu = NULL;
    UINT8 wone_index = 0;
    drvDbHashStaType_e staType;

	if(REPORT_LEVEL(RL_WPA)) 
		wdrv_utilsReport(RL_WPA, PRI_EMERGENCY, "WDRV_WPA_KERNEL: 'send_eapol()' \n");

	memcpy(DA, buf, MAC_ADDR_LEN);	
	eth_type = get_eth_type(buf);

	if(eth_type == EAPOL_TYPE)
	{
		rc = wdrv_dbFindStaIndex(DA, &idx); 
		
		if (rc != OK)
		{
			if(REPORT_LEVEL(RL_WPA)) 
				wdrv_utilsReport(RL_WPA, PRI_EMERGENCY, "WDRV_WPA_KERNEL: error in 'send_eapol()': destination has no index \n");
			return 0;
		}
		else if (!wdrv_mgmtIsStaAssoc(idx))
		{
			if(REPORT_LEVEL(RL_WPA)) 
				wdrv_utilsReport(RL_WPA, PRI_EMERGENCY, "WDRV_WPA_KERNEL: error in 'send_eapol()': destination is not associated\n");
			return 0;
		}

        rc = wdrv_dbGetStationType(idx,  &staType);
        if (rc != OK)
        {
            if(REPORT_LEVEL(RL_WPA)) 
                wdrv_utilsReport(RL_WPA, PRI_EMERGENCY, "WDRV_WPA_KERNEL: error in 'send_eapol()': sta type read error. \n");
            return 0;
        }

		pMsdu = formatEthernetToEapol(buf, len, DA, eth_type, 
                                      (staType == DRV_DB_HASH_STA_TYPE_STATION) ? FALSE : TRUE);
		if (pMsdu == NULL)
		{
			if(REPORT_LEVEL(RL_WPA)) 
				wdrv_utilsReport(RL_WPA, PRI_EMERGENCY, "WDRV_WPA_KERNEL: MSDU is NULL\n");
			return 0;
		}
		
		wdrv_dbGetStationWoneIndex(idx, &wone_index);
		if (wone_index >= MAX_WONE_NET)
		{
			printf("%s: staIndex=%d, woneIndex = %d  !!!\n",__func__,idx,wone_index);
			wlan_memMngrFreeMSDU(pMsdu->handle);
			return 0;
		}
		
		/* Send it to the Tx module, we always send it in the mgmt queue */
		txFlags = WDRV_TX_HIGH_PRIORITY | WDRV_TX_UNICAST_MSDU | WDRV_TX_EAPOL_FRAME;
		
		if (encrypt == FALSE)
			txFlags |= WDRV_TX_FORCE_UN_ENCRYPTED; /* We send EAPOLs via the mgmt queue. */
		
		wdrv_txSendMsdu(pMsdu, idx, txFlags,wone_index);
	}
	else if(eth_type == EAPOL_ETHER_PRE_AUTH_TYPE)
	{
		pMsdu = formatEthernetToMsdu(buf, len, DA,EAPOL_ETHER_PRE_AUTH_TYPE);
		if(wdrv_bssBridgeMsduToBridge(pMsdu) != OK)
		{
			wdrv_utilsReport(RL_WPA, PRI_EMERGENCY, "WDRV_WPA(send eapol): error sending eapol to end.\n");
			return 0;
		}
	}
	else if(REPORT_LEVEL(RL_WPA)) 
			wdrv_utilsReport(RL_WPA, PRI_EMERGENCY, "WDRV_WPA(send eapol): MSDU with error ether type\n");

	return len;
}

/*******************************************************************************************/
/*! \brief Waits for hello message from WPA process

	\return  OK/NOK
*/
/*******************************************************************************************/
STATUS dda_wpa_wait_for_wpaHello(void)
{
	if((wdrv_DDA_sleep_timeout((void*)&WaitForWpaModule ,WAIT_FOR_WPA_MODULE_TIMEOUT)) == 0)
		return NOK;
	return OK;
}

/*******************************************************************************************/
/*! \brief A callback which is called upon reception of hello message from WPA
		   Initializes wpa recovery structure. Wakes up a waiting mechanism. 
*/
/*******************************************************************************************/
void   dda_wpa_receive_hello(void) {
	wdrv_DDA_wake((void*)&WaitForWpaModule);
}

