/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/******************************************************************************/

/*! \file  dda_EventQueue.c                                               
    \brief This module implements DDA Event Queue and Pool                                                                          
*/
/******************************************************************************/

#define _DDA_EVENT_QUEUE_C

#include "wlan_pal.h"
#include "dda.h"
#include "dda_trace.h"
#include "dda_EventQueue.h"

/***********************************************************
	defs
***********************************************************/

#define EVENT_ALIGNMENT  4

#define EVQ_AUDIT
#define EVQ_DEBUG
#undef  EVQ_TEST

/***********************************************************
	macros
***********************************************************/

/***********************************************************
	static variables
***********************************************************/

/***********************************************************
	Local Function Prototypes
***********************************************************/

static EVQ_POOL_P  EventPoolCreate( UINT32 numOfEvents, int eventDataSize );

static STATUS      EventPoolDelete( EVQ_POOL_P pEventPool );

static EVQ_LIST_P  ListCreate( const EVQ_ACTION_FUN_P pActionFun );

static STATUS      ListDelete( const EVQ_LIST_P pList );

static STATUS      ListPut( const EVQ_LIST_P pList, 
                            const EVQ_EVENT_P pEvent );

static EVQ_EVENT_P ListTake( const EVQ_LIST_P pList );

static EVQ_EVENT_P ListTakeAll( const EVQ_LIST_P pList );

static EVQ_EVENT_P EventCreate( const EVQ_POOL_P pEvPool );

static void        EventDelete( const EVQ_POOL_P pEvPool, 
                                const EVQ_EVENT_P pEvent );

static void        EventDeleteAll( const EVQ_POOL_P pEventPool, 
                                   const EVQ_EVENT_P pEvent );

static STATUS      BuildLists( const EVQ_QUEUE_P pEvQueue, 
                               const int         numOfLists,
                               const EVQ_ACTION_FUN_P actionFun[] );

static STATUS      DeleteLists( const EVQ_QUEUE_P pEvQueue );

static void        EventSet( const EVQ_EVENT_P      pEvent, 
                             const EVQ_EVENT_TYPE_T eventType,
                             const UINT8 *          message,
                             const UINT32           messageLen );


/*******************************************************************************
		PUBLIC FUNCTION IMPLEMENTATION
********************************************************************************/

/******************************************************************************/

/*! \brief 	Creates event queue with lists and pool of events
	\param 	numOfLists      - number of event lists in the queue
	\param 	maxNumOfEvents  - number of events in the pool
	\param 	maxNumOfEvents  - number of events in the pool
	\param 	eventDataSize   - size of data field of event structure
	\param 	actionFun[]     - array of action function pointers for each event list
	\return pointer to created queue structure or NULL in case of error

Allocates memory for queue and event pool, creates waiting semaphore
*/
/******************************************************************************/

EVQ_QUEUE_P EVQ_Create( const UINT32 numOfLists, 
					    const UINT32 numOfEvents, 
						const UINT32 maxProcessNum,
                        const UINT32 eventDataSize,
                        const EVQ_ACTION_FUN_P actionFun[] )
{
 EVQ_QUEUE_P	pEvQueue;
 STATUS         status;

 pEvQueue = PAL_memAlloc(0,sizeof(EVQ_QUEUE_S), 4 /* alignment*/, 0);
 if (pEvQueue == NULL)
 {
    DDA_LOG(DDA_LOG_ERR, "dda_EventQueue: Can't allocate memory for queue\n");
    goto EvqCreateError;
 }
 PAL_memSet(pEvQueue, 0, sizeof(EVQ_QUEUE_S));
 
 pEvQueue->pEventPool = EventPoolCreate( numOfEvents, eventDataSize );
 if (pEvQueue->pEventPool == NULL)
 {
	DDA_LOG(DDA_LOG_ERR, "dda_EventQueue: Can't create event pool\n");
	goto EvqCreateError;
 }
 
 status = BuildLists( pEvQueue, numOfLists, actionFun );
 if (status != OK)
 {
	DDA_LOG(DDA_LOG_ERR, "dda_EventQueue: Can't create event lists\n");
    goto EvqCreateError;
 }
 
 pEvQueue->waitSem = semBCreate( SEM_Q_FIFO, SEM_EMPTY );
 if (pEvQueue->waitSem == NULL)
 {
	DDA_LOG(DDA_LOG_ERR, "dda_EventQueue: Can't create waiting semaphore\n");
    goto EvqCreateError;
 }

 pEvQueue->numOfEvents   = numOfEvents;
 pEvQueue->maxProcessNum = maxProcessNum;
 pEvQueue->eventDataSize = eventDataSize;
 pEvQueue->numOfLists    = numOfLists;
 
 return (pEvQueue);

EvqCreateError:
 if (pEvQueue)
 {
    if (pEvQueue->pEventPool) EventPoolDelete(pEvQueue->pEventPool);
    if (pEvQueue->waitSem)    semDelete(pEvQueue->waitSem);
	PAL_memFree( 0, pEvQueue, 0);
 }
 return NULL;

}

/******************************************************************************/

/*! \brief 	puts event into queue
    \param  eventType  - event type
    \param  listNum    - number of the queue list to add event
	\param	message    - message to be copied into event data field 
	\param	messageLen - size of data
	\return status of operation OK or ERROR

Allocates event, sets event fields, adds event to the list tail and 
signals queue semaphore
*/
/******************************************************************************/

STATUS EVQ_PutEvent( const EVQ_QUEUE_P      pEvQueue,
                     const EVQ_EVENT_TYPE_T eventType,
                     const UINT32           listNum, 
                     const UINT8 *          message,
                     const UINT32           messageLen )
{
 EVQ_EVENT_P pEvent;
 STATUS      status;

#ifdef EVQ_AUDIT
 if (listNum >= pEvQueue->numOfLists) 
 {
     DDA_LOG_ARG( DDA_LOG_ERR, "EVQ_PutEvent: Invalid list number=%u max=%u\n", 
                  listNum, pEvQueue->numOfLists, 0,0,0,0 );
     return ERROR;
 }
 if (messageLen > pEvQueue->eventDataSize) 
 {
     DDA_LOG_ARG(DDA_LOG_ERR, "EVQ_PutEvent: Too long msg len=%u max=%u\n", 
                 messageLen, pEvQueue->eventDataSize, 0,0,0,0);
     return ERROR;
 }
#endif

 DDA_LOG_ARG(DDA_LOG_TRACE, "%s: type=%u list=%u msg=%x len=%u\n", __func__, 
		 eventType, listNum, message, messageLen, 0 );

 pEvent = EventCreate( pEvQueue->pEventPool );
 if (pEvent == NULL) 
 {
     DDA_LOG_ARG(DDA_LOG_ERR, "EVQ_PutEvent: No free events, listNum=%u\n",
                 listNum, 0,0,0,0,0 );
     return ERROR; 
 }
 EventSet( pEvent, eventType, message, messageLen );
 ListPut( pEvQueue->eventLists[listNum], pEvent );
 DTR_TRACE( DTR_TYPE_SET_A, eventType, *((UINT32*)message), *((UINT32*)message+1) );
 
 status = semGive(pEvQueue->waitSem);
 if (status != OK) 
 {
     DDA_LOG(DDA_LOG_ERR, "EVQ_PutEvent: Semaphore give error\n");
     return ERROR;
 }
 return OK;
  
}

/******************************************************************************/

/*! \brief 	Waits for event and processes it
	\param 	pEvQueue - queue pointer
	\return status of operation OK or ERROR

Performs action function associated with event, then frees the event
*/
/******************************************************************************/

STATUS EVQ_WaitAndProcess( const EVQ_QUEUE_P  pEvQueue )
{
 EVQ_EVENT_P pEvent;
 EVQ_EVENT_P pHead;
 STATUS      status;
 int         eventCount;
 int         i;
/* int         maxProcessNum = pEvQueue->maxProcessNum; */
 
 DDA_LOG_ARG(DDA_LOG_TRACE, "%s: WaitForSem\n", __func__, 0,0,0,0,0);
 status = semTake(pEvQueue->waitSem, WAIT_FOREVER );
 DDA_LOG_ARG(DDA_LOG_TRACE, "%s: getSem\n", __func__, 0,0,0,0,0);
 
 if (status != OK) 
 {
     DDA_LOG(DDA_LOG_ERR, "EVQ_WaitAndProcess: Semaphore take error\n");
     return ERROR;
 }
 
 while (TRUE) /* alternative : while (--maxProcessNum)  */
 {
     eventCount = 0;
     for( i=0; i < pEvQueue->numOfLists; i++ )
     {
        pHead = ListTakeAll(pEvQueue->eventLists[i]);
        pEvent = pHead;
        while(pEvent)
        {
			DDA_LOG_ARG( DDA_LOG_TRACE, "%s: Event list=%u type=%x\n", __func__, 
					 i, pEvent->type, 0,0,0);

            DTR_TRACE( DTR_TYPE_GET_A, pEvent->type, 
                       *((UINT32*)(&pEvent->data)), *(((UINT32*)(&pEvent->data)+1)) );
            status = ((pEvQueue->eventLists[i])->pActionFun)( &pEvent->data );
            if (status != OK) 
            {
                DDA_LOG_ARG(DDA_LOG_ERR, 
                            "EVQ_WaitAndProcess: Action #%u failed status=%x data=%x\n",
                            i, status, pEvent->data, 0,0,0 );
            }
            pEvent = pEvent->next;
            eventCount++;
        }
        EventDeleteAll(pEvQueue->pEventPool, pHead);
     }
     if (!eventCount) break;
 }
 return OK;
  
}

/******************************************************************************/

/*! \brief 	Deletes event queue structure, event pool and lists
	\param 	pEvQueue - queue pointer
	\return status of operation OK or ERROR
*/
/******************************************************************************/

STATUS EVQ_Delete( const EVQ_QUEUE_P pEvQueue )
{

 DeleteLists( pEvQueue );
 EventPoolDelete(pEvQueue->pEventPool);
 semDelete(pEvQueue->waitSem);
 PAL_memFree( 0, pEvQueue, 0);
 return OK;

}


/******************************************************************************
		LOCAL FUNCTION IMPLEMENTATION
*******************************************************************************/

/******************************************************************************/

/*! \brief 	Creates event pool
	\param 	maxNumOfEvents - number of events in the pool
	\param 	eventDataSize  - size of event data field
	\return pointer to created event pool or NULL in case of error

Allocates memory for event pool and sets event pool structure, 
event memory block size is multiple of 4 bytes for better performance
*/
/******************************************************************************/

static EVQ_POOL_P EventPoolCreate( UINT32 numOfEvents, int eventDataSize )
{
 EVQ_POOL_P pEventPool;
 int i;
 UINT32 eventSize = sizeof(EVQ_EVENT_S) + eventDataSize - sizeof(void *);

 pEventPool = (EVQ_POOL_P) PAL_memAlloc( 0, sizeof(EVQ_POOL_S), 4, 0);
 if (pEventPool == NULL) 
 {
     DDA_LOG(DDA_LOG_ERR, "EventPoolCreate: Failed to allocate pool structure\n");
     goto EventPoolCreateError;
 }

 pEventPool->eventArray = (EVQ_EVENT_P *) PAL_memAlloc( 0, 
                           sizeof(EVQ_EVENT_P)*numOfEvents, 4, 0);
 if (pEventPool->eventArray == NULL) 
 {
     DDA_LOG(DDA_LOG_ERR, "EventPoolCreate: Failed to allocate event array\n");
     goto EventPoolCreateError;
 }
 PAL_memSet(pEventPool->eventArray, 0, sizeof(EVQ_EVENT_P) * numOfEvents);
 
            /* Event memory block is 4 byte alligned for better performance */
 if (eventSize & (EVENT_ALIGNMENT - 1)) 
 {
     eventSize = (eventSize & ~(EVENT_ALIGNMENT - 1)) + EVENT_ALIGNMENT;
 }
 pEventPool->poolAddr = PAL_memAlloc(0, eventSize * numOfEvents, 4, 0);
 if (pEventPool->poolAddr == NULL) 
 {
     DDA_LOG(DDA_LOG_ERR, "EventPoolCreate: Failed to allocate event pool\n");
     goto EventPoolCreateError;
 }

 for( i=0; i < numOfEvents; i++)
 {
     pEventPool->eventArray[i] = (EVQ_EVENT_P) (pEventPool->poolAddr + eventSize * i);
 }
 
 pEventPool->poolSize  = numOfEvents;
 pEventPool->dataSize  = eventDataSize;
 pEventPool->freeIndex = 0;
 DDA_LOG_ARG( DDA_LOG_POOL, "Created Pool at=%x AlignedSize=%u PoolSize=%u\n",
              pEventPool->poolAddr, eventSize, eventSize * numOfEvents, 0,0,0 );
 
 return (pEventPool);

EventPoolCreateError:
 if (pEventPool) 
 {
     if (pEventPool->eventArray)
     {
         PAL_memFree( 0, pEventPool->eventArray, 0 );
     }
     if (pEventPool->poolAddr) 
     {
         PAL_memFree( 0, pEventPool->poolAddr, 0 );
     }
     PAL_memFree( 0, pEventPool, 0 );
 }
 return NULL;

}

/******************************************************************************/

/*! \brief 	Deletes event pool
	\param 	pEventPool - number of event lists in the queue
	\param 	eventDataSize  - size of event data field
	\return status of operation OK
*/
/******************************************************************************/

static STATUS EventPoolDelete( EVQ_POOL_P pEventPool )
{
 
 PAL_memFree( 0, pEventPool->eventArray, 0 );
 PAL_memFree( 0, pEventPool->poolAddr, 0 );
 PAL_memFree( 0, pEventPool, 0 );
 
 return OK;

}

/******************************************************************************/

/*! \brief 	Builds a number of event list
	\param 	pEvQueue    - queue pointer
	\param 	numOfLists  - number of lists to create
	\param 	actionFun[] - array of action function pointers corresponding to lists
	\return status of operation OK

Allocates memory for event lists and sets list stuctures	
*/
/******************************************************************************/

static STATUS BuildLists( const EVQ_QUEUE_P pEvQueue, 
                   const int         numOfLists,
                   const EVQ_ACTION_FUN_P actionFun[] )
{
 int i;
 EVQ_LIST_P pList;

 pEvQueue->eventLists = (EVQ_LIST_P *) PAL_memAlloc( 0, 
                         sizeof(EVQ_LIST_P)*numOfLists, 4, 0);
 
 if (pEvQueue->eventLists == NULL) 
 {
     DDA_LOG(DDA_LOG_ERR, "BuildLists: Failed to allocate list array\n");
     return ERROR;
 }
 
 for( i=0; i < numOfLists; i++)
 {
     pList = ListCreate( actionFun[i] );
     if (pList == NULL) 
     {
         DDA_LOG(DDA_LOG_ERR, "BuildLists: Failed to allocate list\n");
         PAL_memFree( 0, pEvQueue->eventLists, 0 );
         return ERROR;
     }
     pEvQueue->eventLists[i] = pList;
 }
 
 return OK;

}


/******************************************************************************/

/*! \brief 	Deletes all event queue lists
	\param 	pEvQueue        - queue pointer
	\return status of operation OK

Frees list structures memory	
*/
/******************************************************************************/

static STATUS DeleteLists( const EVQ_QUEUE_P pEvQueue )
{
 int i;

 for( i=0; i < pEvQueue->numOfLists; i++)
 {
     ListDelete(pEvQueue->eventLists[i]);
 }
 PAL_memFree( 0, pEvQueue->eventLists, 0 );

 return OK;

}

/******************************************************************************/

/*! \brief 	creates event list
	\param 	pActionFun - pointer to list action function
	\return pointer to the created list or NULL in case of allocation error

Allocates memory for the list and sets list structure	
*/
/******************************************************************************/

static EVQ_LIST_P ListCreate( const EVQ_ACTION_FUN_P pActionFun )
{
 EVQ_LIST_P pList;

 pList = PAL_memAlloc( 0, sizeof(EVQ_LIST_S), 4, 0);
 if (pList == NULL) 
 {
     DDA_LOG(DDA_LOG_ERR, "dda_EventQueue: Failed to allocate list\n");
     return NULL;
 }
 pList->pActionFun = pActionFun;
 pList->head = NULL;
 pList->tail = NULL;
 pList->count = 0;
 pList->maxCount = 0;
 return (pList);

}

/******************************************************************************/

/*! \brief 	Puts event to the list tail
	\param 	pList   - pointer to the list control structure
	\param	pEvent  - pointer to the event
	\return status of operation OK or ERROR
*/
/******************************************************************************/

static STATUS ListPut( const EVQ_LIST_P pList, const EVQ_EVENT_P pEvent )
{
 unsigned int flags;

 PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); 
 if (pList->tail)   /* is list not empty ? */
 {
     pList->tail->next=pEvent;
     pList->tail = pEvent;
 }
 else
 {
     pList->tail = pEvent;
     pList->head = pEvent;
 }
 pEvent->next = NULL;
 if (++pList->count > pList->maxCount) 
 {
     pList->maxCount = pList->count; /* update list statistics */
 }
 PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags); 
 
 return OK;

}

/******************************************************************************/

/*! \brief 	Takes event from the list head
	\param 	pList   - pointer to the list control structure
	\return pointer to the event or NULL if list is empty

Event is removed from the list head	
*/
/******************************************************************************/

static EVQ_EVENT_P ListTake( const EVQ_LIST_P pList )
{
 EVQ_EVENT_P pEvent = NULL;
 unsigned int flags;

 PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); 
 if (pList->head)   /* is list not empty ? */
 {
     pEvent = pList->head;
     pList->head = pEvent->next;
     if (pList->head == NULL) 
     {
         pList->tail = NULL;
     }
     pList->count--;
 }
 PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags); 
 
 return pEvent;

}


/******************************************************************************/

/*! \brief 	Takes all the events from the list
	\param 	pList   - pointer to the list control structure
	\return pointer to the list head or NULL if list is empty

All events are removed from the list	
*/
/******************************************************************************/

static EVQ_EVENT_P ListTakeAll( const EVQ_LIST_P pList )
{
 EVQ_EVENT_P pHead;
 unsigned int flags;

 PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); 
 
 pHead = pList->head;
 pList->head = NULL;
 pList->tail = NULL;
 pList->count = 0;
 
 PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags); 
 
 return pHead;
}

/******************************************************************************/

/*! \brief 	Deletes event list
	\param 	pList - pointer to the list control structure
	\return status of operation OK

Frees memory of list structure	
*/
/******************************************************************************/

static STATUS ListDelete( const EVQ_LIST_P pList )
{

 PAL_memFree( 0, pList, 0);
 return OK;

}

/******************************************************************************/

/*! \brief 	Creates event from the event pool
	\param 	pEvPool - pointer to the event pool
	\return pointer the event or NULL if there are no free events in the pool
*/
/******************************************************************************/

static EVQ_EVENT_P EventCreate( const EVQ_POOL_P pEvPool )
{
 EVQ_EVENT_P pEvent;
 unsigned int flags;

 PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); 
 if (pEvPool->freeIndex < pEvPool->poolSize)
 {
     pEvent = pEvPool->eventArray[ pEvPool->freeIndex++ ];
 }
 else
 {
     pEvent = NULL;
 }
 PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags); 
 
 return pEvent; 
}

/******************************************************************************/

/*! \brief 	Assigns event structure fields
	\param	pEvent      - pointer to the event to set
	\param	eventType   - type of event
	\param	message     - event data
	\param  messageLen  - event data length
	\return void
*/
/******************************************************************************/

static void EventSet( const EVQ_EVENT_P      pEvent, 
					  const EVQ_EVENT_TYPE_T eventType,
					  const UINT8 *          message,
					  const UINT32           messageLen )
{
 pEvent->type = eventType;
 PAL_memCopy( &pEvent->data, message, messageLen );
 pEvent->dataLen = messageLen;
}

/******************************************************************************/

/*! \brief 	Returns event to the event pool
	\param	pEvPool  - pointer to the event pool
	\param	pEvent   - pointer to the event to free
	\return void
*/
/******************************************************************************/

static void EventDelete( const EVQ_POOL_P pEvPool, const EVQ_EVENT_P pEvent )
{
 unsigned int flags;
 
 PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); 
 pEvPool->eventArray[ --pEvPool->freeIndex ] = pEvent;
 PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags); 

}

/******************************************************************************/

/*! \brief 	Returns event chain to the event pool
	\param	pEvPool  - pointer to the event pool
	\param	pHead       - pointer to the first event of the chain
	\return void
*/
/******************************************************************************/

static void EventDeleteAll( const EVQ_POOL_P pEventPool, const EVQ_EVENT_P pHead )
{
 unsigned int flags;
 EVQ_EVENT_P pEvent = pHead;

 PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); 
 while(pEvent)
 {
     pEventPool->eventArray[ --pEventPool->freeIndex ] = pEvent;
     pEvent = pEvent->next;
 }
 PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags); 
}

/*******************************  Event Queue Test ***************************/
 


#ifdef  EVQ_DEBUG

/******************************************************************************/

/*! \brief 	Event qeueu show routine
	\param	pEvQueue  - pointer to the event queue
*/
/******************************************************************************/

void EventPoolShow( EVQ_POOL_P  pEventPool );
void EventListShow( EVQ_LIST_P pLists, UINT32 dataLen );

void EVQ_Show( const EVQ_QUEUE_P  pEvQueue )
{
 int         i;
 
 printf("Sem=%x numOfEvents=%u maxProcessNum=%u eventDataSize=%u numOfLists=%u\n",
         (UINT32) pEvQueue->waitSem, pEvQueue->numOfEvents, pEvQueue->maxProcessNum, 
         pEvQueue->eventDataSize, pEvQueue->numOfLists );
 printf("ListArrAdr=%x PoolStrAdr=%x\n", 
        (UINT32) pEvQueue->eventLists, (UINT32) pEvQueue->pEventPool );
 
 EventPoolShow(pEvQueue->pEventPool);
 
 for(i=0; i<pEvQueue->numOfLists; i++)
 {
     EventListShow(pEvQueue->eventLists[i], pEvQueue->eventDataSize);
 }

  
}

/******************************************************************************/

/*! \brief 	Event pool show routine
	\param	pEventPool  - pointer to the event pool
*/
/******************************************************************************/

void EventPoolShow( EVQ_POOL_P  pEventPool )
{
 int  i;

 printf("PoolAdr=%x eventArray=%x freeInd=%u dataSize=%u poolSize=%u\n",
        (UINT32) pEventPool->poolAddr, (UINT32) pEventPool->eventArray, 
        pEventPool->freeIndex, pEventPool->dataSize, pEventPool->poolSize );
 
 for(i=0; i<pEventPool->poolSize; i++) 
     printf("EvArray[%d]=%x\n", i, (UINT32)pEventPool->eventArray[i] ); 
}

/******************************************************************************/

/*! \brief 	Event list show routine
	\param	pList   - pointer to the list
	\param	dataLen - number of data bytes to show
*/
/******************************************************************************/

void EventListShow( EVQ_LIST_P pList, UINT32 dataLen )
{
 EVQ_EVENT_P pEvent = pList->head;
 int         i, j;

 printf("ListAdr=%x head=%x tail=%x count=%u maxCount=%u fun=%x\n", 
         (UINT32)pList, (UINT32)pList->head, (UINT32)pList->tail, 
         pList->count, pList->maxCount, (UINT32)pList->pActionFun );

 i=0;
 while (pEvent) 
 {
     printf("Ev[%d] Adr=%x type=%u len=%u data:",
            i++, (UINT32)pEvent, pEvent->type, pEvent->dataLen );
     for( j=0; j<pEvent->dataLen; j++) printf("%x ", ((UINT8*)(&pEvent->data))[j] );
     printf("\n");
     pEvent = pEvent->next;
 }
}


#endif /* EVQ_DEBUG */

#ifdef EVQ_TEST
#include "dda_EventQueueTest.c"
#endif

