/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/******************************************************************************/

/*! \file  dda_trace.c                                               
    \brief This module implements DDA trace utilities                                                                          
*/
/******************************************************************************/

/***********************************************************
	includes
*******************************************************/

#define _DDA_TRACE_C

#include "wlan_pal.h"
#include "dda.h"
#include "dda_trace.h"

/***********************************************************
	defs
***********************************************************/

typedef struct {
    UINT16  type;
    UINT16  id;
    UINT32  timeStamp;
    UINT32  par1;
    UINT32  par2;

} TRACE_RECORD_S;

typedef struct {
    TRACE_RECORD_S * pTraceBuf;
    UINT32           traceIndex;
    UINT32           maxNumOfRec;
    BOOL             traceEnable;
} TRACE_CONTROL_S;

#define SYS_GET_TIMESTAMP(tmp) __asm__ __volatile__ ( "mfc0 %0,$9"  : "=r" (tmp))

/***********************************************************
	macros
***********************************************************/


/***********************************************************
	gloabal variables
***********************************************************/

TRACE_CONTROL_S * pTracer = NULL;
UINT32 DDA_LogLevel;
UINT32 DTR_TraceBufAdr;
UINT32 DTR_TraceBufSize;

/***********************************************************
	static variables
***********************************************************/
 
char * TraceIDStr[20]= 
{
	"EMPTY     ",
	"TX START  ",
	"BSS_WLAN_R",
	"CHKSIZE_F ",
	"RX INTR   ",
	"RX EVENT  ",
	"TX_C_INTR ",
	"SND_Q_OK  ",
	"SND_Q_BUSY",
	"TX_C_EVENT",
	"TX_E_START",
	"TX_MASTR_E",
	"TX_TIMEOUT",
	"SKB_FAIL  ",
	"RX_EN_TO  ", 
	"WHAL_RX   ",
	"MGM_CHK   "
};


char * TraceEvStr[16]= 
{
	"Empty",
	"Mngmt ",
	"Rx    ",
	"Tx    ",
	"MainSM",
	"MailBox" 
};

char * TraceTypeStr[]= {"None",
                        "GetEv",
                        "SetEv",
                        "Aplic",
                        "Undef"
                        };

/***********************************************************
	Local Function Prototypes
***********************************************************/

static void ShowEvent( UINT16 type, UINT16 id, UINT32 par1, UINT32 par2 );

/*******************************************************************************
		PUBLIC FUNCTION IMPLEMENTATION
********************************************************************************/
 
/******************************************************************************/

/*! \brief 	Sets DDA debug level
	\param 	level  -  debug level to set
*/
/******************************************************************************/

inline void DTR_SetReportLevel( const UINT32 level )

{
    DDA_LogLevel = level;
}

/******************************************************************************/

/*! \brief 	Gets DDA debug level
	\param 	taskPriority - DDA task priority
	
	\return DDA debug/report level
*/
/******************************************************************************/

inline UINT32 DTR_GetReportLevel( void )

{
    return( DDA_LogLevel );
}

/******************************************************************************/

/*! \brief 	Creates and initializes trace buffer
	\param	maxNumOfRec - number of record in trace buffer
	\param 	traceBufAdr - address of trace buffer or NULL for allocation
	
	\return status of operation OK or ERROR
*/
/******************************************************************************/

STATUS DTR_TraceInit( UINT32 maxNumOfRec, UINT32 * traceBufAdr )
{
 pTracer = PAL_memAlloc( 0, sizeof(TRACE_CONTROL_S), 4, 0 );
 if (pTracer == NULL) 
 {
     DDA_LOG(DDA_LOG_ERR, "DTR_TraceInit: Failed to allocate memory\n");
     return ERROR;
 }
 if (traceBufAdr == NULL) 
 {
     pTracer->pTraceBuf = PAL_memAlloc( 0,sizeof(TRACE_RECORD_S)*maxNumOfRec, 4,0);
     if (pTracer->pTraceBuf == NULL) 
     {
         DDA_LOG(DDA_LOG_ERR, "DTR_TraceInit: Failed to get memory for trace buffer\n");
         PAL_memFree( 0, pTracer, 0 );
         return ERROR;
     }
 }
 else
 {
     pTracer->pTraceBuf = (TRACE_RECORD_S *) traceBufAdr;
 }
 pTracer->maxNumOfRec = maxNumOfRec;
 pTracer->traceIndex  = 0;
 pTracer->traceEnable = FALSE;
 PAL_memSet(pTracer->pTraceBuf, 0, sizeof(TRACE_RECORD_S)*maxNumOfRec ); 
 DTR_TraceBufAdr  =  (UINT32) pTracer->pTraceBuf;
 DTR_TraceBufSize =  sizeof(TRACE_RECORD_S)*maxNumOfRec;
 return OK;
}

/******************************************************************************/

/*! \brief 	Deletes tracer and trace buffer
	
	\return status of operation OK
*/
/******************************************************************************/


STATUS DTR_TraceDelete( void )
{
 TRACE_CONTROL_S * pTrc = pTracer;

 pTracer->traceEnable = FALSE;	/* Disable tracing */
 pTracer = NULL;
 
 
 if (pTrc->pTraceBuf)
 {
	 PAL_memFree(0,pTrc->pTraceBuf,0);  
 }
 if (pTrc)
 {
	 PAL_memFree(0,pTrc,0);  
 }
 DTR_TraceBufAdr  =  0;
 DTR_TraceBufSize =  0;
 return OK;
}

/******************************************************************************/

/*! \brief 	Sets trace record
	\param 	type       - type of operation
	\param 	id         - record id
	\param 	par1       - first  parameter
	\param 	par2       - second parameter
*/
/******************************************************************************/

void DTR_Trace( UINT16 type, UINT16 id, UINT32 par1, UINT32 par2 )
{
 TRACE_RECORD_S * pRec;
 unsigned int    flags;
 UINT32 tmp;

 if (!pTracer || !pTracer->traceEnable) 
 {
     return;
 }
// ShowEvent( type, id, par1, par2 );
 PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); 
 pRec = pTracer->pTraceBuf + pTracer->traceIndex;
 SYS_GET_TIMESTAMP(tmp);
 pRec->timeStamp = tmp;
 pRec->type = type;
 pRec->id   = id;
 pRec->par1 = par1;
 pRec->par2 = par2;
 if (++pTracer->traceIndex >= pTracer->maxNumOfRec)
 {
     pTracer->traceIndex = 0;
 }
 PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags); 
}

/******************************************************************************/

/*! \brief 	Sets trace status to enabled/disabled
	\param 	status TRUE - enabled, FALSE - disabled
*/
/******************************************************************************/


void DTR_TraceSet( BOOL status )
{

 if (pTracer == NULL) 
 {
     return;
 }
 pTracer->traceEnable = status;

}

/******************************************************************************/

/*! \brief 	Prints trace buffer contents to standard output
*/
/******************************************************************************/

#undef printf

void DTR_TraceShow( void )
{
 TRACE_RECORD_S * pRec;
 char           * idStr;
 int    i;

 printf("TraceBuf=%x Status=%x maxRecNum=%u index=%u\n",
         (UINT32) pTracer->pTraceBuf, pTracer->traceEnable,
         pTracer->maxNumOfRec, pTracer->traceIndex );
 
 printf("%10s %15s %15s %8s %8s\n", "Time","Type","ID","Par1","Par2");  /* title */

 for (i=0; i<pTracer->maxNumOfRec; i++) 
 {
     pRec = pTracer->pTraceBuf + i;
     if (pRec->type == DTR_TYPE_APLIC_A) 
     {
         idStr = TraceIDStr[ pRec->id ];
     }
     else
     {
         idStr = TraceEvStr[ pRec->id ];
     }
     printf("%10u %15s %15s %8x %8x\n",
            pRec->timeStamp, TraceTypeStr[ pRec->type ],
            idStr, pRec->par1, pRec->par2 ); 
 }
}


/******************************************************************************/

/*! \brief 	Shows event contents
*/
/******************************************************************************/

static void ShowEvent( UINT16 type, UINT16 id, UINT32 par1, UINT32 par2 )
{
 char * idStr;
 
 if (type == DTR_TYPE_APLIC_A) 
 {
    idStr = TraceIDStr[ id ];
 }
 else
 {
    idStr = TraceEvStr[ id ];
 }
 DDA_LOG_ARG( DDA_LOG_TRACE, "TRACE: type=%s id=%s par1=%u par2=%u\n", 
			  TraceTypeStr[ type ], idStr, par1, par2, 0,0 );

}



