/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/


/******************************************************************************/

/*! \file  wlan_pal.c                                               
    \brief This module is platform abstraction layer and implements interface 
		   to VxWorks OS dependent services
*/
/******************************************************************************/


#define  _PAL_C

/***********************************************************
	 includes.
*******************************************************/

#include "wlan_pal.h"
#include "stdarg.h"
#include "utilsTcb.h"
#include "dda.h"

/***********************************************************
	defs
************************************************************/

#define MAX_LOGS        256              /* must be power of 2, number of log buffers */
#define LOG_MASK        (MAX_LOGS-1)     /* musk to handle ring of buffers */
#define MAX_LOG_SIZE    256              /* size of logging buffer */
#define TIMER_TASK_PRI  40

/***********************************************************
	variables
************************************************************/

UINT32 PAL_reportLevel;
UINT8  PAL_LogBuf[MAX_LOGS][MAX_LOG_SIZE];
UINT32 PAL_LogIndexWr;
UINT32 PAL_LogIndexRd;
UINT32 PAL_MaxStrLen;
UINT32 PAL_PrintEnable = 1;				/* DBG */

#ifdef DDA_PRINT_TASK
int    PAL_printTaskId;
#endif

/***********************************************************
	local function prototypes
***********************************************************/

static void printTask( void );

/******************************************************************************/

/*! \brief 	Initializes PAL internal variables
	
	\return status of operation OK or ERROR
*/
/******************************************************************************/

STATUS PAL_Init(void)
{

	PAL_LogIndexWr = 0;
	PAL_LogIndexRd = 0;
	PAL_MaxStrLen  = 0;
    PAL_reportLevel= 0;
    PAL_memSet( PAL_LogBuf, 0, sizeof(PAL_LogBuf));

#ifdef DDA_PRINT_TASK
	PAL_printTaskId = taskSpawn("tPrint",   			/* name of DDA print task	  */
								254,      				/* priority of DDA print task */
								0,          			/* task option word           */
								5120,    				/* task stack size (bytes)    */
								(FUNCPTR) printTask,  	/* entry point of print task  */
								0,0,0,0,0,0,0,0,0,0   	/* arguments to pass          */
								);                       

	if (PAL_printTaskId == ERROR) 
	{
	   DDA_LOG( DDA_LOG_ERR, "PAL_Init: Failed to create Print task\n" );
	   return ERROR;
	}
#endif
	
	lbt_tcbInit(TIMER_TASK_PRI, PALSRV_msec2ticks(10));

	return OK;
}

/******************************************************************************/

/*! \brief 	Deletes PAL settings
	
	\return status of operation OK or ERROR
*/
/******************************************************************************/

STATUS PAL_Delete(void)
{

	lbt_tcbEnd();

#ifdef DDA_PRINT_TASK
	if (PAL_printTaskId)
	{
		if (taskDelete(PAL_printTaskId) != OK)
		{
			DDA_LOG( DDA_LOG_ERR, "PAL_Delete: Failed to delete Print task\n" );
			return ERROR;
		}
	}
#endif

	return OK;
}

/******************************************************************************/

/*! \brief 	Formats string and outputs it for logging
	
	\param 	format string
	\param 	optional parameters
	\return status of operation OK or ERROR
*/
/******************************************************************************/

void PALSRV_report(const char *format, ...) 
{
    int count;
	int index;
	int flags;
    va_list arg;
    
	if (!PAL_PrintEnable)
	{
		return;
	}
	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); 
	index=PAL_LogIndexWr;
	PAL_LogIndexWr = (PAL_LogIndexWr+1) & LOG_MASK;
	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags); 
    
	va_start(arg, format);
    count = vsprintf((char *) &PAL_LogBuf[index], format, arg );
    va_end(arg);
    if(count <= 0) 
        return;
    if(count >= MAX_LOG_SIZE)
	{
		logMsg("PALSRV_report: output msg is to long=%d!!!\n", 
			   count, 0,0,0,0,0 );
        PAL_LogBuf[index][MAX_LOG_SIZE - 1] = '\0';
	}
	if (count > PAL_MaxStrLen)
	{
		PAL_MaxStrLen = count;
	}
#ifndef DDA_PRINT_TASK
	logMsg("%u %s\n",(int) (tickGet()*1000/sysClkRateGet()),
			(int)&PAL_LogBuf[index], 0,0,0,0);
#endif

}

/******************************************************************************/

/*! \brief 	Creates new timer
	\param 	pointer to timer structure to return created timer
	
	\return status of operation OK or ERROR
*/
/******************************************************************************/

STATUS PALSRV_tcbNew( PALSRV_TcbObjTYPE * timer)
{
    * timer = lbt_tcbNew(0);
	if (*timer) 
	{
		return PAL_SOK;
	}
	else
	{
		return ERROR;
	}
}


/*******************************************************************************
		LOCAL FUNCTION IMPLEMENTATION
********************************************************************************/

#ifdef DDA_PRINT_TASK
/******************************************************************************/

/*! \brief 	Print task - outputs time in ms and messages from the log buffer
*/
/******************************************************************************/

#undef printf

static void printTask( void ) 
{
	int flags;
	int indexWr;
	
	for(;;)
	{
	  taskDelay(1);
	  
	  PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); 
	  indexWr=PAL_LogIndexWr;
	  PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags); 
	
	  while (PAL_LogIndexRd != indexWr)
	  {
		  printf("%u: %s\n",(int) (tickGet()*1000/sysClkRateGet()), 
				 (char *) &PAL_LogBuf[PAL_LogIndexRd]);
		  PAL_LogIndexRd = (PAL_LogIndexRd+1) & LOG_MASK;
	  }
	}
}
#endif /* DDA_PRINT_TASK */
