/*
 * WCFG         An implementation of the wcfglib library.
 *		See wcfg_lib companion document.
 *
 *
 * Version:     0.2.1
 *
 * Authors:     Felix Rubinstein, <felixr@ti.com>
 *
 */


#include <stdio.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/mman.h>

#include "wcfg_ipc.h"


/* Don't delete following two lines, very useful for lock debugging */
/*#define WCFG_IPC_LOCK_DEBUG*/
/*#define WCFG_MSYNC_DEBUG*/


#if WCFG_SYS_V_IPC
/* The following code is based on
   "Advanced Programming in the UNIX Environment" by W. Richard Stevens */

int ipc_create_shm(struct ipc *ipc)
{

	ipc->shared = NULL;
	
#warning "advanced error handling"
	/* TODO: advanced error handling,
	   i.e. what if shared memory segment still exists in memory, in case of a bug.
	   As of now, we assume that if EEXIST is returned, segment is already created. */
	if( (ipc->priv.shmid = shmget(WCFG_MAGIC, sizeof(struct ipc_shared),
					IPC_CREAT|IPC_EXCL | SHM_R|SHM_W)) == -1 )
	{
 		if( errno == EEXIST )
		{
			printf("\n\nwcfg ipc: referencing shared memory\n\n");
			if( (*shmid = shmget(key, size, SHM_R|SHM_W)) == -1 ) { /*we only reference it*/
				fprintf(stderr, "wcfg ipc: %s %d: %s\n", __func__, __LINE__, strerror(errno));
				goto out;
			}
		}
		else {
			fprintf(stderr, "wcfg ipc: %s %d: %s\n", __func__, __LINE__, strerror(errno));
			goto out;
		}
	}
	else printf("\n\nwcfg ipc: allocated shared memory\n\n");

	if( (ipc->shared = shmat(ipc->priv.shmid, NULL, 0)) == (char *)-1 ) {
		fprintf(stderr, "wcfg ipc: %s %d: %s\n", __func__, __LINE__, strerror(errno));
		goto err_shmctl;
	}

	if( wcfg_storage_read(ipc) )
		goto err_shmdt;

	return 0;

err_shmdt:	
	if( shmdt(ipc->shared) == -1 )
		fprintf(stderr, "wcfg ipc: %s %d: %s\n", __func__, __LINE__, strerror(errno));
err_shmctl:
	if( shmctl(ipc->priv.shmid, IPC_RMID, NULL) == -1 )
		fprintf(stderr, "wcfg ipc: %s %d: %s\n", __func__, __LINE__, strerror(errno));
out:
	return 1;
}

int ipc_remove_shm(struct ipc *ipc)
{
	/* update persistent storage */
	wcfg_storage_write(ipc);
	
	if( shmdt(ipc->shared) == -1 ) {
		fprintf(stderr, "wcfg ipc: %s %d: %s\n", __func__, __LINE__, strerror(errno));
		return 1;
	}

#warning "advanced error handling, the same as above"
	if( shmctl(ipc->priv.shmid, IPC_RMID, NULL) == -1 ) {
		fprintf(stderr, "wcfg ipc: %s %d: %s\n", __func__, __LINE__, strerror(errno));
		return 1;
	}
	else printf("\n\nwcfg ipc: destroyed shared memory\n\n");

	ipc->shared = NULL;	

	return 0;
}

#else

/* The following code is based on
   "UNIX Network Programming, Volume 2, Second Edition" by W. Richard Stevens */
int ipc_create_shm(struct ipc *ipc)
{
int fd = -1;

	ipc->shared = NULL;

	if( (fd = open(ipc->priv.storage_file, O_RDWR)) == -1 ) {
		fprintf(stderr, "Failed to open %s: %s\n", ipc->priv.storage_file,
				strerror(errno));
		return 1;
	}

	if( (ipc->shared = mmap(NULL, sizeof(struct ipc_shared),
					PROT_READ|PROT_WRITE, MAP_SHARED, fd, 0))
			== MAP_FAILED ) {
		fprintf(stderr, "wcfg ipc:%s %d: %s\n", __func__, __LINE__, strerror(errno));
		return 1;
	}

	if( close(fd) == -1 ) {
		if( munmap(ipc->shared, sizeof(int)) == -1 )
			fprintf(stderr, "wcfg ipc:%s %d: %s\n", __func__, __LINE__, strerror(errno));
		fprintf(stderr, "wcfg ipc:%s %d: %s\n", __func__, __LINE__, strerror(errno));
		return 1;
	}

	ipc->priv.fd = -1;
	
#ifdef WCFG_IPC_LOCK_DEBUG
	printf("%s %d: fd=%d\n", __func__, __LINE__, ipc->priv.fd);
#endif
			
	return 0;
}


int ipc_remove_shm(struct ipc *ipc)
{	
	if( munmap(ipc->shared, sizeof(struct ipc_shared)) == -1 ) {
		fprintf(stderr, "wcfg ipc:%s %d: %s\n", __func__, __LINE__, strerror(errno));
		return 1;
	}
	
	ipc->shared = NULL;	

	return 0;
}

int ipc_update_shm(struct ipc_shared *shared)
{
#ifdef WCFG_MSYNC_DEBUG
	printf("%s: %d\n", __func__, __LINE__);
#endif
	
	if( msync(shared, sizeof(struct ipc_shared),
				MS_SYNC|MS_INVALIDATE) == -1 ) {
		fprintf(stderr, "wcfg ipc: %s %d: %s\n", __func__, __LINE__, strerror(errno));
		return 1;
	}

#ifdef WCFG_MSYNC_DEBUG
	printf("%s: %d\n", __func__, __LINE__);
#endif
	
	return 0;
}

#endif



int ipc_mandatory_lock(struct ipc_private *priv, int lock)
{
struct stat statbuf;

#ifdef WCFG_IPC_LOCK_DEBUG
	//printf("%s %d: fd=%d\n", __func__, __LINE__, priv->fd);
#endif

	if( fstat(priv->fd, &statbuf) == -1 ) {
		fprintf(stderr, "wcfg ipc: %s %d: %s\n", __func__, __LINE__,
				strerror(errno));
		return 1;
	}

	if( lock == WCFG_MANDATORY_LOCK ) {
		lock = (statbuf.st_mode & ~S_IXGRP) | S_ISGID;
		priv->orig_mode = statbuf.st_mode;
	}
	else if( lock == WCFG_MANDATORY_UNLOCK )
		lock = priv->orig_mode;
	else {
		fprintf(stderr, "wcfg ipc: Invalid lock id\n");
		return 1;
	}

	if( fchmod(priv->fd, lock) == -1 ) {
		fprintf(stderr, "wcfg ipc: %s %d: %s\n", __func__, __LINE__,
				strerror(errno));
		return 1;
	}

	return 0;
}

static int lock_reg(int fd, int cmd, int type, off_t offset, int whence, off_t len)
{
	struct flock    lock;

	lock.l_type = type;     /* F_RDLCK, F_WRLCK, F_UNLCK */
	lock.l_start = offset;  /* byte offset, relative to l_whence */
	lock.l_whence = whence; /* SEEK_SET, SEEK_CUR, SEEK_END */
	lock.l_len = len;       /* #bytes (0 means to EOF) */

	return( fcntl(fd, cmd, &lock) );
}

/* in case of failure we must leave the lock in unlocked state, if we can,
 * so that there is no need to call unlock.
 */
int ipc_readw_lock(struct ipc_private *priv)
{
#ifdef WCFG_IPC_LOCK_DEBUG
	printf("%s %d: fd=%d\n", __func__, __LINE__, priv->fd);
#endif
	
	if( priv->fd != -1 ) {
		fprintf(stderr, "wcfg ipc: Persistent storage read descriptor is not closed.\n");
		return 1;
	}

	if( (priv->fd = open(priv->storage_file, O_RDONLY)) == -1 ) {
		fprintf(stderr, "wcfg ipc: Failed to open %s: %s\n",
				priv->storage_file, strerror(errno));
		return 1;
	}

	if( ipc_mandatory_lock(priv, WCFG_MANDATORY_LOCK) )
		return 1;
			
	if( lock_reg(priv->fd, F_SETLKW, F_RDLCK, 0, SEEK_SET, 0) == -1 ) {
		fprintf(stderr, "wcfg ipc: Failed to apply advisory read wait lock: %s\n",
				strerror(errno));
		ipc_mandatory_lock(priv, WCFG_MANDATORY_UNLOCK);
		return 1;
	}

	return 0;
}

/* in case of failure we must leave the lock in unlocked state, if we can,
 * so that there is no need to call unlock.
 */
int ipc_writew_lock(struct ipc_private *priv)
{
#ifdef WCFG_IPC_LOCK_DEBUG
	printf("%s %d: fd=%d\n", __func__, __LINE__, priv->fd);
#endif
	
	if( priv->fd != -1 ) {
		fprintf(stderr, "wcfg ipc: Persistent storage write descriptor is not closed.\n");
		return 1;
	}
	
	if( (priv->fd = open(priv->storage_file, O_WRONLY)) == -1 ) {
		fprintf(stderr, "wcfg ipc: Failed to open %s: %s\n",
				priv->storage_file, strerror(errno));
		return 1;
	}

	if( ipc_mandatory_lock(priv, WCFG_MANDATORY_LOCK) )
		return 1;
			
	if( lock_reg(priv->fd, F_SETLKW, F_WRLCK, 0, SEEK_SET, 0) == -1 ) {
		fprintf(stderr, "wcfg ipc: Failed to apply advisory write wait lock: %s\n",
				strerror(errno));
		ipc_mandatory_lock(priv, WCFG_MANDATORY_UNLOCK);
		return 1;
	}

	return 0;
}

int ipc_unlock(struct ipc_private *priv)
{	
	if( ipc_mandatory_lock(priv, WCFG_MANDATORY_UNLOCK) )
		return 1;

	if( lock_reg(priv->fd, F_SETLK, F_UNLCK, 0, SEEK_SET, 0) == -1) {
		ipc_mandatory_lock(priv, WCFG_MANDATORY_UNLOCK);
		return 1;
	}

	if( close(priv->fd) == -1 ) {
		fprintf(stderr, "wcfg ipc: Failed to close lock file descriptor: %s\n",
				strerror(errno));
		return 1;
	}

	priv->fd = -1;
	
#ifdef WCFG_IPC_LOCK_DEBUG
	printf("%s %d: fd=%d\n", __func__, __LINE__, priv->fd);
#endif

	return 0;
}


#if 0
int concurrent_restart_start(struct ipc *ipc)
{
	if( ipc_writew_lock(&ipc->priv) )
		return 1;

	while(1)
		if( !ipc->shared->restart_in_progress ) {
			ipc->shared->restart_in_progress = 1;
			if( ipc_unlock(&ipc->priv) )
				return 1;
			break;
		}
		else {
			printf("wcfg ipc: Concurrent restart encountered, "
				"waiting for previous restart to complete.\n");
			fflush(stdout);
			sleep(1);
		}

	return 0;
}


int concurrent_restart_stop(struct ipc *ipc)
{
	if( ipc_writew_lock(&ipc->priv) )
		return 1;

	ipc->shared->restart_in_progress = 0;

	if( ipc_unlock(&ipc->priv) )
		return 1;

	return 0;
}
#endif

