/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/


/*!
 *
 *   \file  wcfg_net.c
 *   \brief PURPOSE: wcfg library network utilities
 *
 */


#include <stdio.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <linux/route.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <linux/if_bridge.h>
#include <linux/autoconf.h>

#include "wcfglib.h"


/*!	\brief if_nametoindex
  	\param ifname - interface name

  	\return interface index  
  
	taken from bridge-utils
*/
unsigned int if_nametoindex(const char *ifname)
{
	struct ifreq ifr;
	int fd;

	if ((fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
		fprintf(stderr, "%s:%d %s\n",__FUNCTION__, __LINE__, strerror(errno));
		return 0;
	}

	strncpy (ifr.ifr_name, ifname, sizeof (ifr.ifr_name));
	if (ioctl (fd, SIOCGIFINDEX, &ifr) < 0) {
		fprintf(stderr, "%s:%d %s\n",__FUNCTION__, __LINE__, strerror(errno));
		close(fd);
		return 0;
	}

	close (fd);
	return ifr.ifr_ifindex;
}


/*!	\brief br_addif
  	\param br_name - bridge name
  	\param ifname - interface name

  	\return zero upon success, otherwise non-zero
  
	add ifname device to br_name bridge
*/
int br_addif(const char *br_name, const char *ifname)
{
	unsigned long args[4];
	unsigned long ifindex;
	struct ifreq ifr;
	int fd, err;

	if (!(ifindex = if_nametoindex(ifname))) {
		fprintf(stderr, "%s: interface %s does not exist!\n", __FUNCTION__, ifname);
		return 1;
	}

	args[0] = BRCTL_ADD_IF;
	args[1] = ifindex;
	args[2] = 0;
	args[3] = 0;

	if ((fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
		fprintf(stderr, "%s:%d %s\n",__FUNCTION__, __LINE__, strerror(errno));
		return 1;
	}

	memcpy(ifr.ifr_name, br_name, IFNAMSIZ);
	((unsigned long *)(&ifr.ifr_data))[0] = (unsigned long)args;

	if( (err = ioctl(fd, SIOCDEVPRIVATE, &ifr)) )
		switch ((err = ioctl(fd, SIOCDEVPRIVATE, &ifr))) {
			case EBUSY:
				fprintf(stderr, "%s: device %s is already a member of a bridge; "
						"can't enslave it to bridge %s.\n", __FUNCTION__, ifname,
						br_name);
				break;

			case ELOOP:
				fprintf(stderr, "%s: device %s is a bridge device itself; "
						"can't enslave a bridge device to a bridge device.\n",
						__FUNCTION__, ifname);
				break;

			default:
				fprintf(stderr, "%s:%d %s\n",__FUNCTION__, __LINE__, strerror(errno));
				break;
		}

	close(fd);
	return err;
}

/*!	\brief br_delif
  	\param br_name - bridge name
  	\param ifname - interface name

  	\return zero upon success, otherwise non-zero
  
	add ifname device to br_name bridge
*/
int br_delif(const char *br_name, const char *ifname)
{
	unsigned long args[4];
	unsigned long ifindex;
	struct ifreq ifr;
	int fd, err;

	if (!(ifindex = if_nametoindex(ifname))) {
		fprintf(stderr, "%s: interface %s does not exist!\n", __FUNCTION__, ifname);
		return 1;
	}

	args[0] = BRCTL_DEL_IF;
	args[1] = ifindex;
	args[2] = 0;
	args[3] = 0;

	if ((fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
		fprintf(stderr, "%s:%d %s\n",__FUNCTION__, __LINE__, strerror(errno));
		return 1;
	}

	memcpy(ifr.ifr_name, br_name, IFNAMSIZ);
	((unsigned long *)(&ifr.ifr_data))[0] = (unsigned long)args;

	if( (err = ioctl(fd, SIOCDEVPRIVATE, &ifr)) )
		switch ((err = ioctl(fd, SIOCDEVPRIVATE, &ifr))) {
			case EBUSY:
				fprintf(stderr, "%s: device %s is already a member of a bridge; "
						"can't enslave it to bridge %s.\n", __FUNCTION__, ifname,
						br_name);
				break;

			case ELOOP:
				fprintf(stderr, "%s: device %s is a bridge device itself; "
						"can't enslave a bridge device to a bridge device.\n",
						__FUNCTION__, ifname);
				break;

			default:
				fprintf(stderr, "%s:%d %s\n",__FUNCTION__, __LINE__, strerror(errno));
				break;
		}

	close(fd);
	return err;
}


/*!	\brief tiap_wcfg_set_ipaddr_or_netmask
  	\param devname - device name
  	\param id - request code, see ioctl(2)

  	\return zero upon success, otherwise non-zero
  
	configure IP or netmask address, depending on id.
*/
#ifndef MWCODE
int tiap_wcfg_set_ipaddr_or_netmask(const char *devname, int id, __u32 addr)
{
	struct ifreq ifr;
	struct sockaddr_in *sin = (void *) &ifr.ifr_ifru.ifru_addr;
	int fd;

    printf("%s: dev=%s id=%x addr=%x\n", __FUNCTION__, devname, id, addr );

	if (!devname) return 1;

	if ((fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
		fprintf(stderr, "%s:%d %s\n",__FUNCTION__, __LINE__, strerror(errno));
		return 1;
	}

	memset(&ifr, 0, sizeof(ifr));
	strncpy(ifr.ifr_name, devname, IFNAMSIZ);
	sin->sin_family = AF_INET;
	sin->sin_addr.s_addr = addr;

	if (ioctl(fd, id, &ifr) == -1) {
		fprintf(stderr, "%s: %s: %s (addr=%u id=%d)\n", __FUNCTION__, devname, strerror(errno), addr, id);
        close(fd);
		return 1;
	}

	close(fd);

	return 0;
}
#endif /* MWCODE */

/*!	\brief tiap_wcfg_dev_set_flags
  	\param devname - device name
  	\param flag - device flag
  	\param enable - enable or disable flag

  	\return zero upon success, otherwise non-zero
  
	enable or disable device flag.
*/
int tiap_wcfg_dev_set_flags(const char *devname, int flag, int enable)
{
	struct ifreq ifr;
	struct sockaddr_in *sin = (void *) &ifr.ifr_ifru.ifru_addr;
	int fd;

	if (!devname) return 1;

	if ((fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
		fprintf(stderr, "%d: %s\n", __LINE__, strerror(errno));
		return 1;
	}

	memset(&ifr, 0, sizeof(ifr));
	strncpy(ifr.ifr_name, devname, IFNAMSIZ);
	sin->sin_family = AF_INET;

	if (ioctl(fd, SIOCGIFFLAGS, &ifr) == -1) {
		fprintf(stderr, "%s: %s\n", devname, strerror(errno));
        close(fd);
		return 1;
	}

	if( !enable )
		ifr.ifr_flags &= flag;
	else ifr.ifr_flags |= flag;

	if (ioctl(fd, SIOCSIFFLAGS, &ifr) == -1) {
		fprintf(stderr, "%s: %s\n", devname, strerror(errno));
        close(fd);
		return 1;
	}

    close(fd);
	return 0;
}


/*!	\brief set_sockaddr
  	\param sin - socket address
  	\param addr - address
  	\param port - port

	set socket address.
*/
static void set_sockaddr(struct sockaddr_in *sin, __u32 addr, __u16 port)
{
	sin->sin_family = AF_INET;
	sin->sin_addr.s_addr = addr;
	sin->sin_port = port;
}


/*!	\brief tiap_wcfg_set_route
  	\param dev - device name
  	\param dst - destination address
  	\param genmask - netmask address
  	\param gateway - gateway address
  	\param id - add or delete an entry

  	\return zero upon success, otherwise non-zero
  
	manipulate the IP routing table, see route(8).
*/
int tiap_wcfg_set_route(const char *dev, __u32 dst, __u32 genmask, __u32 gateway, const int id)
{
	struct rtentry rm;
	int fd, err;

	if( id != SIOCADDRT && id != SIOCDELRT ) {
		fprintf(stderr, "Cannot add route, invalid id=%d\n", id); 
		return 1;
	}

	memset(&rm, 0, sizeof(rm));
	rm.rt_dev = (char *)dev;
	set_sockaddr((struct sockaddr_in *) &rm.rt_dst, dst, 0);
	set_sockaddr((struct sockaddr_in *) &rm.rt_genmask, genmask, 0);
	set_sockaddr((struct sockaddr_in *) &rm.rt_gateway, gateway, 0);
	rm.rt_flags = RTF_UP;
	if( gateway )
		rm.rt_flags |= RTF_GATEWAY;

	if ((fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
		perror("socket");
		return fd;
	}

	if ((err = ioctl(fd, id, &rm)) < 0) {
		fprintf(stderr, "Cannot add route %u (%d): %s.\n", gateway, err, strerror(errno));
		close(fd);
		return err;
	}

	close(fd);
	
	return 0;
}


/*!	\brief config_ip_settings
  	\param wd - wcfg descriptor
  	\param defaults - default configuration

  	\return zero upon success, otherwise non-zero
  
	configure system IP settings.
*/
int config_ip_settings(wcfgd *wd, int defaults)
{	
	int overwrite = 1;
	
	/*add wireless device to the bridge*/
	if( br_addif(wd->br_name, wd->dev_name) ) {
		fprintf(stderr, "wcfglib: Warning, failed to add %s device to the the %s bridge, "
				"configuration is not complete.\n", wd->dev_name, wd->br_name);
		fflush(stderr);
	}
	
	if( tiap_wcfg_dev_set_flags(wd->dev_name, IFF_UP, 1) )
		return 1;

/* If NFS support is configured into the kernel, don't overwrite system IP address
   unless configuration is being already stored. */
#ifdef CONFIG_NFS_FS
	if( defaults )
		overwrite = 0;
#endif

#ifndef MWCODE
	if( overwrite )
	{
		/*ip address*/
		if( tiap_wcfg_set_ipaddr_or_netmask(wd->br_name, SIOCSIFADDR, wd->storage.ldb.ipaddr) )
			return 1;

		/*ip address netmask*/
		if( tiap_wcfg_set_ipaddr_or_netmask(wd->br_name, SIOCSIFNETMASK, wd->storage.ldb.netmask) )
			return 1;

		if( tiap_wcfg_dev_set_flags(wd->br_name, IFF_UP, 1) )
			return 1;

		/*default gateway*/
		if( tiap_wcfg_set_route(wd->br_name, 0, 0, wd->storage.ldb.default_gw, SIOCADDRT) )
			return 1;
	}
#endif /* MWCODE */
	return 0;
}


/*!	\brief Configures bridge IP address and mask, sets interface UP
  	\param pointer to wcfg descriptor
  	\param default configuration

  	\return zero upon success, otherwise non-zero
*/
int tiap_wcfg_bridge_config(const char *devname, wcfgd *wd, int defaults)
{	
	int overwrite = 1;
	
/* If NFS support is configured into the kernel, don't overwrite system IP address
   unless configuration is being already stored. */
#ifdef CONFIG_NFS_FS
	if( defaults )
		overwrite = 0;
#endif

#if 0
	if( overwrite )
	{
		/*ip address*/
		if( tiap_wcfg_set_ipaddr_or_netmask(devname, SIOCSIFADDR, wd->storage.ldb.ipaddr) )
			return 1;

		/*ip address netmask*/
		if( tiap_wcfg_set_ipaddr_or_netmask(devname, SIOCSIFNETMASK, wd->storage.ldb.netmask) )
			return 1;

		if( tiap_wcfg_dev_set_flags(devname, IFF_UP, 1) )
			return 1;

		/*default gateway*/
		if( tiap_wcfg_set_route(devname, 0, 0, wd->storage.ldb.default_gw, SIOCADDRT) )
			return 1;
	}
#endif

	return 0;
}


/*!	\brief Adds wlan device to the bridge
  	\param pointer to wcfg descriptor

  	\return zero upon success, otherwise non-zero
*/
int tiap_wcfg_bridge_add_wlan(wcfgd *wd)
{		
	/*add wireless device to the bridge*/
	if( br_addif(wd->br_name, wd->dev_name) ) {
		fprintf(stderr, "wcfglib: Warning, failed to add %s device to the the %s bridge, "
				"configuration is not complete.\n", wd->dev_name, wd->br_name);
		fflush(stderr);
        return 1;
	}
	
	if( tiap_wcfg_dev_set_flags(wd->dev_name, IFF_UP, 1) )
		return 1;

	return 0;
}


/*!	\brief tiap_wcfg_get_default_gateway
  	\param gateway_addr - default gateway IP address

  	\return zero upon success, otherwise non-zero
  
	return default gateway IP address.
*/
int tiap_wcfg_get_default_gateway(__u32 *gateway_addr)
{
	char delim[]=" \t", *tok;
	FILE *proc_fd;
	char buf[1024];


	if (!(proc_fd = fopen("/proc/net/route","r"))) {
		perror("fopen");
		return 1;
	}
	/* each line in the file has the following format: 
	<Iface>   <Destination>   <Gateway> ....*/

	while (fgets(buf, sizeof(buf), proc_fd) != NULL)
	{
		if (! (tok = strtok(buf, delim))) /*interface*/ {
			fprintf(stderr, "Invalid file format\n");
            fclose(proc_fd);
			return 1;
		}

		tok = strtok(NULL, delim);/*dest address*/

		if (tok)
		{
			if (!strcmp(tok,"00000000")) 
			{
				tok = strtok(NULL, delim); /*gateway address*/
				sscanf(tok,"%X",gateway_addr);
                fclose(proc_fd);
				return 0;
			}
		}
	}
    fclose(proc_fd);
	return 1;
}



/*!	\brief tiap_wcfg_set_storage_ipaddr
  	\param wd - wcfg descriptor
	\param id - specifies which IP address to changes
	\param addr - address

	change on of the following:
	default gateway, device or netmask IP address,
	depending on the id and save the changes in the storage.
*/
void tiap_wcfg_set_storage_ipaddr(wcfgd *wd, const int id, const __u32 addr)
{
int update_storage = 0;

	switch( id )
	{
		case SIOCADDRT:
			wd->storage.ldb.default_gw = addr;
			update_storage = 1;
			break;

		case SIOCSIFADDR:
			wd->storage.ldb.ipaddr = addr;
			update_storage = 1;
			break;

		case SIOCSIFNETMASK:
			wd->storage.ldb.netmask = addr;
			update_storage = 1;
			break;
	}

	if( update_storage )
		wcfg_storage_write(&wd->storage, (const char *)wd->storage_file);
}


/*!	\brief tiap_wcfg_get_ipaddr_or_netmask
  	\param devname - device name
  	\param id - device or netmask
  	\param ip - IP address

  	\return zero upon success, otherwise non-zero
  
	get device or netmask IP address, depending on the id.
*/
int tiap_wcfg_get_ipaddr_or_netmask(const char *devname, int id, __u32 *ip)
{
	struct ifreq ifr;
	struct sockaddr_in *sin = (void *) &ifr.ifr_ifru.ifru_addr;
	int fd;

	if (!devname || !ip) return 1;

	if ((fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
		fprintf(stderr, "%d: %s\n", __LINE__, strerror(errno));
		return 1;
	}

	memset(&ifr, 0, sizeof(ifr));
	strncpy(ifr.ifr_name, devname, IFNAMSIZ);
	sin->sin_family = AF_INET;

	if (ioctl(fd, id, &ifr) == -1) {
		fprintf(stderr, "%s: %s\n", devname, strerror(errno));
        close(fd);
		return 1;
	}

	*ip = sin->sin_addr.s_addr;
    close(fd);
	return 0;
}

/*!
	\brief Retrieves network interface IP address, subnet mask and def gateway
	
	\param name of device including unit name e.g. "brgEnd32"
	\param pointer to the buffer to return IP address string in dot notation
	\param pointer to the buffer to return IP subnet mask string in dot notation
	\param pointer to the buffer to return def gateway IP address string in dot not.
	
	\return 0 if successful or 1 in the case of error
*/


int tiap_wcfg_get_ipConfig( const char * devname, 
                            char * dotIpAdr, 
                            char * dotIpMask,
                            char * dotGateway
                           )
{
	__u32 ipAdr;
	struct in_addr addr;
    char * ipStr;

    if( tiap_wcfg_get_ipaddr_or_netmask( devname, SIOCGIFADDR, &ipAdr) ) {
		printf("%s: Failed to get IP address.\n", __FUNCTION__);
		return 1;
	}
	addr.s_addr = ipAdr;
    ipStr = inet_ntoa( addr );
    strncpy( dotIpAdr, ipStr, INET_ADDRSTRLEN );  

    if(tiap_wcfg_get_ipaddr_or_netmask( devname, SIOCGIFNETMASK, &ipAdr)) {
		printf("%s: Failed to get IP subnet mask.\n", __FUNCTION__);
        return 1;
    }                                                
	addr.s_addr = ipAdr;
    ipStr = inet_ntoa( addr );
    strncpy( dotIpMask, ipStr, INET_ADDRSTRLEN );  

    if( tiap_wcfg_get_default_gateway( &ipAdr ) ) {
		printf("%s: Failed to get gateway IP address.\n", __FUNCTION__);
        return 1;
    }
	addr.s_addr = ipAdr;
    inet_ntoa( addr );
    ipStr = inet_ntoa( addr );
    strncpy( dotGateway, ipStr, INET_ADDRSTRLEN );  

    return 0;
}

