/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2004 Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |** 
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*																		   */
/*	 MODULE:	hiddenSecurityProfile.c									   */
/*   PURPOSE:															   */
/*				Manages authenticator or supplicant instance corresponding */
/*				to internal security profile not associated with any of    */
/*				multiple SSIDs used for STAs association                   */
/*																		   */					
/***************************************************************************/
/***************************************************************************/
#include <stdio.h>
#ifndef VXWORKS
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/un.h>
#include <net_utils.h>
#include <kos/knet.h>
#include <errno.h>
#include <ipc.h>
#include <linux_ipc_user.h>
#endif /* ifndef VXWORKS */
#include <802.1x/wep_port.h>
#include <wpa/wpa_port.h>

#include <802.1x/eap_radius.h>
#include <wpa/wpa_info_elem.h>
#include "dda_wpa.h"
#include "wpa_common.h"
#include "dda_wpaauth_ioctl.h"
#include "wpa_key_utils.h"
#include "wpa_supplicant.h"
#include "wpa_info_elem.h"
#include "securityProfile.h"
#include "hiddenSecurityProfileAPI.h"
#include "debug_report.h"
#include "wpa.h"

#define MAX_NUM_ASSOCS_PER_PROFILE  5

typedef struct
{
	UINT8 is_active;
	ether_mac_t mac;
}hiddenProfAssoc_T;

static UINT32 hiddenProfileEnabled = TRUE;    /*Indicates whether hidden profile is enabled */    
static UINT32 hiddenProfSecurity = FALSE;		/*Indicates whether hidden profile security is enabled*/
static initHiddenProfConf_T authStatusReportInfo;
static hiddenProfAssoc_T hiddenProfAssocs[MAX_NUM_ASSOCS_PER_PROFILE]; /* MACs of associations with the profile*/

/* Local function definition */
static UINT32 hiddenProfileReport(initHiddenProfConf_T *pProfileConfig, UINT8 *pMac, UINT32 success);
static void hidden_profileConfirmComplete(hiddenWpaInstMsgType type);
static UINT32 hidden_profileGetAssocNumber(void);

/******************************************************************************
*                      updateWpaPortConf()							  *
*******************************************************************************
DESCRIPTION:    update WPA port configuration parameters dynamically
                                                                         
INPUTS:			wpaPort - a pointer to wpa port
				pCfg - dynamically changed conf params

OUTPUT:			None

RETURNS:		None                                      
******************************************************************************/
static void updateWpaPortConf(wpa_port_t *wpaPort, UINT8 *pPwd, UINT32 pwdLength)
{
	UINT8 pskPwd[TIAP_WPA_MAX_PSK_PWD_LEN + 1];

	/* PSK password can be only in the ASCII mode*/
	memcpy(pskPwd, pPwd, pwdLength);
	/* Must be NULL-terminated !!! */
	pskPwd[pwdLength] = '\0';

	/* Convert the password to hex mode*/
	wpa_key_utils_calc_psk(&wpaPort->psk, pskPwd, wpaPort->params.ssid, wpaPort->params.ssid_len);
}


/******************************************************************************
*                      hidden_profile_ProcessInputMsg()							  *
*******************************************************************************
DESCRIPTION:    A callback called when WDS socket is ready for reading
				Processes the received message
                                                                         
INPUTS:			sock - WDS socket,
				data - ignored

OUTPUT:			None

RETURNS:		None                                             
******************************************************************************/
void hidden_profile_ProcessInputMsg(int sock, void *data)
{
	int ret;
	hiddenWpaInstanceMsg_T	 msg;
	ap_dev_cbs_t 		 	 *ap_dev_cbs;
	security_port_data_t 	 *hidden_port_data;

	ret = read(sock, (char *)&msg, sizeof(msg) );

	if (ret < 0) {
		printf("WPA HIDDEN INST.: error reading a message from socket\n");
		return;
	}

    hidden_port_data = wdrv_wpaGetHiddenSecPortData();
	
	switch(msg.type)
	{
	case HIDDEN_PROF_INIT:
		DEBUG_PRINT_m(SEV_INFO, MOD_WPA_HIDDEN, "HIDDEN PROFILE: INIT message received \n");
        hiddenProfSecurity = msg.u.initParams.secPortEnabled;
        authStatusReportInfo = msg.u.initParams;
        hidden_port_data->pWpaPortContext->params.ap_dev_ops->set_profile_privacy_opt
                (hidden_port_data->pWpaPortContext->ap_dev, 
                 hidden_port_data->pWpaPortContext->port_profile_index,
                 hiddenProfSecurity);
        memset(hiddenProfAssocs, 0, sizeof(hiddenProfAssocs));
        wpas_InitSupplicant();
        if(hiddenProfSecurity) {
            /* update the corresponding WPA port with auth configuration params*/
            updateWpaPortConf(hidden_port_data->pWpaPortContext, 
                              msg.u.initParams.pskPasswd, msg.u.initParams.pskPasswdLen);
        }

        hidden_profileConfirmComplete(HIDDEN_PROF_INIT_COMPLETE);
        break;
	case HIDDEN_PROF_AUTH_START:
		DEBUG_PRINT_m(SEV_INFO, MOD_WPA_HIDDEN, "HIDDEN PROFILE: AUTH_START message received \n");
		if (hiddenProfileEnabled && hiddenProfSecurity) 
		{
            if (hidden_profileGetAssocNumber() < MAX_NUM_ASSOCS_PER_PROFILE) 
			{
				int i;
                
				ap_dev_cbs = wdrv_wpaGetApDevCbs();
                for (i=0 ; i<MAX_NUM_ASSOCS_PER_PROFILE; i++) {
                    if (!memcmp(hiddenProfAssocs[i].mac, msg.u.confParams.secPartyMac, sizeof(ether_mac_t))) {
                        char *pMac = msg.u.confParams.secPartyMac;
                        /* Clean this entry */
                        if(hiddenProfAssocs[i].is_active == TRUE) {
                            printf("\n\n%s : MAC %02hhX:%02hhX:%02hhX:%02hhX:%02hhX:%02hhX\n\n Already in the table!\n\n",
                                   __FUNCTION__, pMac[0], pMac[1], pMac[2], pMac[3], pMac[4], pMac[5]);
                            ap_dev_cbs = wdrv_wpaGetApDevCbs();
                            /* close  authentication */
                            ap_dev_cbs->sta_disassoc(hidden_port_data->pWpaPortContext, pMac);
                        }
                        hiddenProfAssocs[i].is_active = FALSE;
                        memset(hiddenProfAssocs[i].mac, 0, sizeof(ether_mac_t));
                    }
                }

				for (i=0 ; i<MAX_NUM_ASSOCS_PER_PROFILE; i++) 
				{
					if (hiddenProfAssocs[i].is_active == FALSE) 
					{
						 /* copy the second party MAC to the assocs table */
						hiddenProfAssocs[i].is_active = TRUE;
						memcpy(hiddenProfAssocs[i].mac, msg.u.confParams.secPartyMac, sizeof(ether_mac_t));
						
						/* start new authentication */
						ap_dev_cbs->sta_assoc_done(hidden_port_data->pWpaPortContext, msg.u.confParams.secPartyMac,
									   (UINT8 *)hidden_port_data->pWpaPortContext->ap_rsn_info_elem,
									   wpa_info_elem_get_len(hidden_port_data->pWpaPortContext->ap_rsn_info_elem));
						break;
					}
				}
			}
			else
			{
				/* report failure to WDS */
				hidden_profile_ReportAuthStatus(msg.u.confParams.secPartyMac, SEC_AUTH_STATUS_FAILURE);
			}
					
		}
		break;
	case HIDDEN_PROF_SUPL_START:
		DEBUG_PRINT_m(SEV_INFO, MOD_WPA_HIDDEN, "HIDDEN PROFILE: SUPPL_START message received \n");
			/* if WPA supplicant instance is enabled */
			#ifdef HIDDEN_WPA_SUP_INSTANCE 
				if (hiddenProfileEnabled && hiddenProfSecurity) 
				{
                    /* update the corresponding WPA port with auth configuration params*/
					wpas_OpenNewAssoc(hidden_port_data->pWpaPortContext, &msg.u.confParams);
	
					/* Notify the supplicant state machine that configuration received and 
					that 4-way handshake should be (re)started */
					wpas_SupplGetEvent(WPAS_RESTART, NULL, hidden_port_data->pWpaPortContext);
				}
				else
				{
					hidden_profile_ReportAuthStatus(msg.u.confParams.secPartyMac, SEC_AUTH_STATUS_FAILURE);
				}
			#endif
         break;
	case HIDDEN_PROF_WPAS_ABORT:
		DEBUG_PRINT_m(SEV_INFO, MOD_WPA_HIDDEN, "HIDDEN PROFILE: WPAS_ABORT message received \n");
		#ifdef HIDDEN_WPA_SUP_INSTANCE
		{
			/* Abort current 4-way handshake process and transit to initial SM state */
			wpas_SupplGetEvent(WPAS_ABORT, NULL, hidden_port_data->pWpaPortContext);
		}
		#else
			/*TODO: support for authenticator instance abort */
		#endif
		break;
	
	case HIDDEN_PROF_AUTH_CLOSE:
		DEBUG_PRINT_m(SEV_INFO, MOD_WPA_HIDDEN, "HIDDEN PROFILE: AUTH_CLOSE message received \n");
        hidden_profile_CloseAuth(msg.u.confParams.secPartyMac);
        hidden_profileConfirmComplete(HIDDEN_PROF_LINK_CLOSE_COMPLETE);
		break;
		
	case HIDDEN_PROF_SUPL_CLOSE:
		DEBUG_PRINT_m(SEV_INFO, MOD_WPA_HIDDEN, "HIDDEN PROFILE: SUPL_CLOSE message received \n");
		#ifdef HIDDEN_WPA_SUP_INSTANCE
		{
			wpas_SupplGetEvent(WPAS_CLOSE_AUTH, NULL, hidden_port_data->pWpaPortContext);
		}
		#endif
        hidden_profileConfirmComplete(HIDDEN_PROF_LINK_CLOSE_COMPLETE);
        break;

	default:
		printf("WPA HIDDEN INST.: Wrong input message type\n");
		break;
	}

	return;
}

/******************************************************************************
*                      hidden_profile_PrepareRequestsSock()							  *
*******************************************************************************
DESCRIPTION:    Creates and binds to a name a linux domain socket for listening 
				to requests from other processes
				
                                                                         
INPUTS:			None

OUTPUT:			None

RETURNS:		Socket fd                                             
******************************************************************************/
STATUS hidden_profile_PrepareRequestsSock(UINT32 *pSock)
{
#ifdef VXWORKS
   return ipc_create_sock(TO_WPA_HIDDEN_PROFILE_SOCK_PORT, pSock); 
#else /* ifdef VXWORKS */
   return ipc_create_sock(TO_WPA_HIDDEN_PROFILE_SOCK, pSock); 	 
#endif /* ifdef VXWORKS*/  
}


/******************************************************************************
*                      hidden_profileConfirmComplete()							  *
*******************************************************************************
DESCRIPTION:    Response hidden profile init
                                                                         
INPUTS:			None 

OUTPUT:			None

RETURNS:		None                                             
******************************************************************************/
static void hidden_profileConfirmComplete(hiddenWpaInstMsgType type)
{
	UINT32 sock;	
    hiddenWpaInstanceMsg_T response;
	
#ifdef VXWORKS	
	struct sockaddr_in wpas_sockaddr;
	int sockAddrSize = sizeof(struct sockaddr_in);

    /* Create socket on which to send. */
	sock = socket(AF_INET, SOCK_DGRAM, 0);
	if (sock < 0) {
		perror("opening datagram socket");
		return;
	}
#else /* ifdef VXWORKS */
	struct sockaddr_un wpas_sockaddr;
	int sockAddrSize = sizeof(struct sockaddr_un);

	/* Create socket on which to send. */
	sock = socket(AF_UNIX, SOCK_DGRAM, 0);
	if (sock < 0) {
		perror("opening datagram socket");
		return;
	}
#endif  /* ifdef VXWORKS */

#ifdef VXWORKS
	wpas_sockaddr.sin_family = AF_INET;    
    bzero((char *)&wpas_sockaddr, sockAddrSize);
    wpas_sockaddr.sin_len = sockAddrSize;
    wpas_sockaddr.sin_family = AF_INET;
    wpas_sockaddr.sin_port = htons(TO_WPA_HIDDEN_PROFILE_SOCK_RESP_PORT);
    wpas_sockaddr.sin_addr.s_addr = inet_addr(IPC_LOCAL_INET_ADDR);
    if(wpas_sockaddr.sin_addr.s_addr == ERROR) {
	    printf("%s : failed in inet_addr error: %s\n",__FUNCTION__, strerror(errno));
        close(sock);
        return ;
    }
#else  /* ifdef VXWORKS */
	/* Construct name of socket to send to. */
	wpas_sockaddr.sun_family = AF_UNIX;
	strcpy(wpas_sockaddr.sun_path, TO_WPA_HIDDEN_PROFILE_SOCK_RESP);
#endif  /* ifdef VXWORKS */

    response.type = type;
	/* Send message. */
	if (sendto(sock, (void *)&response, sizeof(response), 0,
				(struct sockaddr *)&wpas_sockaddr, sockAddrSize) < 0) {
		perror("sending init response message ");
		return;
	}
	close(sock);

	return;
}


/******************************************************************************
*                      isAssocWithHiddenInstance()							  *
*******************************************************************************
DESCRIPTION:    
				
                                                                         
INPUTS:			None

OUTPUT:			None

RETURNS:		                                          
******************************************************************************/
UINT32 isAssocWithHiddenInstance(ether_mac_t staMac)
{
	int i;

	if ( !hiddenProfileEnabled ) 
		return FALSE;

	#ifdef HIDDEN_WPA_SUP_INSTANCE
		if (wpas_IsPktToSupplicant(staMac) == TRUE)
			 return TRUE;
	#endif /* HIDDEN_WPA_SUP_INSTANCE */
	
		
	for (i=0; i<MAX_NUM_ASSOCS_PER_PROFILE; i++) 
	{
        if(hiddenProfAssocs[i].is_active)
            if( !(memcmp(staMac, hiddenProfAssocs[i].mac, sizeof(staMac))) )
                return TRUE;
	}
	return FALSE;
	
	
	
}

/******************************************************************************
*                      hidden_profile_CreateReaderSock()							  *
*******************************************************************************
DESCRIPTION:    Creates and binds to a name a linux domain socket for listening to requests
				
                                                                         
INPUTS:			None

OUTPUT:			None

RETURNS:		Socket fd                                             
******************************************************************************/
UINT32 hidden_profile_IsEapolToSup(eapol_pkt_t *eapol)
{
	if (hiddenProfileEnabled && hiddenProfSecurity) {
		return wpas_IsPktToSupplicant(eapol->src);
	}
	else
		return FALSE;
	
}

/******************************************************************************
*                      hidden_profile_IsEnabled()							  *
*******************************************************************************
DESCRIPTION:    Detect if hidden profile is enabled
				
                                                                         
INPUTS:			None

OUTPUT:			None

RETURNS:		En/Dis                                             
******************************************************************************/
UINT32 hidden_profile_IsEnabled(void)
{
	return hiddenProfileEnabled;
}

/******************************************************************************
*                      hidden_profile_IsSecured()							  *
*******************************************************************************
DESCRIPTION:    Detect whether hidden profile security is enabled
				
                                                                         
INPUTS:			None

OUTPUT:			None

RETURNS:		En/Dis                                             
******************************************************************************/
UINT32 hidden_profile_IsSecured(void)
{
	return hiddenProfSecurity;
}

/******************************************************************************
*                      hidden_profile_Disable()							  *
*******************************************************************************
DESCRIPTION:    Disable hidden profile
				
                                                                         
INPUTS:			None

OUTPUT:			None

RETURNS:		Socket fd                                             
******************************************************************************/
void hidden_profile_Disable(void)
{
	hiddenProfileEnabled = FALSE;
}

/******************************************************************************
*                      hidden_profile_ReportAuthStatus()							  *
*******************************************************************************
DESCRIPTION:    Reports authentication status
				
                                                                         
INPUTS:			staMac - second party MAC
				status - status code

OUTPUT:			None

RETURNS:		None                                             
******************************************************************************/
void hidden_profile_ReportAuthStatus(ether_mac_t staMac, UINT32 status)
{
    if(status != SEC_AUTH_STATUS_OK) {
        hidden_profile_CloseAuth(staMac);
    }
    if(hiddenProfileReport(&authStatusReportInfo, staMac, status) != OK) {
        printf("%s : Cant notify security status!\n", __FUNCTION__);
    }
}

/******************************************************************************
*                      hiddenProfileReport()							  *
*******************************************************************************
DESCRIPTION:    Send  report to the socket with given params
				
                                                                         
INPUTS:			pProfileConfig - profile configuration
				pMac - authentication party MAC
				success - status

OUTPUT:			None

RETURNS:		OK/NOK                                            
******************************************************************************/
static UINT32 hiddenProfileReport(initHiddenProfConf_T *pProfileConfig, UINT8 *pMac, UINT32 success)
{
    int sock;
    hiddenProfSecurityResp_s resp;

    resp.msgType = pProfileConfig->reportMsgType;
    memcpy(resp.linkPartnerMac, pMac, MAC_ADDR_LEN);
    resp.result = success;
    resp.failReson = 0; /* Currently not used. */
    /* Create socket on which to send. */ 
	sock = socket(pProfileConfig->domain, pProfileConfig->type, pProfileConfig->protocol);
	if (sock < 0) {
        printf("%s : Opening datagram socket: %s\n", __FUNCTION__, strerror(errno));
		return (NOK);
	}
    /* Send message. */
    if (sendto(sock, (void *)&resp, sizeof(hiddenProfSecurityResp_s), 0,
                (struct sockaddr *)&pProfileConfig->addr, pProfileConfig->addrLen) < 0) {
		printf("%s : Sending IPC message: %s\n", __FUNCTION__, strerror(errno));
        close(sock);
        return (NOK);
    }
    close(sock);
    return (OK);
}

/******************************************************************************
*                      hidden_profile_CloseAuth()							  *
*******************************************************************************
DESCRIPTION:    Clean authentication parameters 
				
                                                                         
INPUTS:			pMac - authentication party MAC

OUTPUT:			None

RETURNS:		None                                            
******************************************************************************/
void hidden_profile_CloseAuth(UINT8 *pMac)
{
    int                      i;
	security_port_data_t 	 *hidden_port_data;
	ap_dev_cbs_t 		 	 *ap_dev_cbs;
    UINT32                   isFound = FALSE;

    hidden_port_data = wdrv_wpaGetHiddenSecPortData();

    for (i=0; i < MAX_NUM_ASSOCS_PER_PROFILE; i++ )
    {
        if (!memcmp(hiddenProfAssocs[i].mac, pMac, sizeof(ether_mac_t))) 
        {
            if(isFound) {
                printf("\n\n%s : MAC %02hhX:%02hhX:%02hhX:%02hhX:%02hhX:%02hhX\n\n is found twice!\n\n",
                       __FUNCTION__, pMac[0], pMac[1], pMac[2], pMac[3], pMac[4], pMac[5]);
            }
            if(hiddenProfAssocs[i].is_active == FALSE) {
                printf("\n\n%s already disabled.MAC %02hhX:%02hhX:%02hhX:%02hhX:%02hhX:%02hhX\n\n", 
                       __FUNCTION__, pMac[0], pMac[1], pMac[2], pMac[3], pMac[4], pMac[5]);
            }

            ap_dev_cbs = wdrv_wpaGetApDevCbs();
            /* close  authentication */
            ap_dev_cbs->sta_disassoc(hidden_port_data->pWpaPortContext, pMac);
            hiddenProfAssocs[i].is_active = FALSE;
            memset(hiddenProfAssocs[i].mac, 0, sizeof(ether_mac_t));
            isFound = TRUE;
        }
    }
}

/******************************************************************************
*                      hidden_profileGetAssocNumber()						  *
*******************************************************************************
DESCRIPTION:    Get number of stations currently associated with hidden profile 
				
                                                                         
INPUTS:			None

OUTPUT:			None

RETURNS:		Associations number                                            
******************************************************************************/
static UINT32 hidden_profileGetAssocNumber(void)
{
    int i;
    UINT32 assocNumber = 0;
    for (i=0 ; i<MAX_NUM_ASSOCS_PER_PROFILE; i++) {
        if(hiddenProfAssocs[i].is_active)
            assocNumber++;
    }
    return (assocNumber);
}
