/* LICENSE_CODE JGPL */

#define OS_INCLUDE_INET
#define OS_INCLUDE_SOCKET
#define OS_INCLUDE_IO
#include <os_includes.h>

/* OpenRG GPL includes */
#include <bio_socket.h>
#include <ipc.h>
#include "memory.h"
#include "openrg_gpl.h"
/* Note this code is GPL Licence, so do not add any new 
 * openrg incude files here. Talk to TL or SA before any change
 */

#define rg_error(level,str,args...) printf(str,##args)

#define SYNC_STR "sync"

/* Low level TCP/IPC */
int ipc_connect(u16 port)
{
    return ipc_connect_ip(port, inet_addr("127.0.0.1"));
}
    
#include "rg_error.h"
int ipc_connect_ip(u16 port, u32 addr)
{
    struct sockaddr_in sa;
    int fd = -1;

    while (1)
    {
		if ((fd = sock_socket(SOCK_STREAM, 0, 0)) < 0)
		{
			printf("error create sock.\n");			
			return -1;
		}
		
		MZERO(sa);
		sa.sin_family = AF_INET;
		sa.sin_addr.s_addr = addr;
		sa.sin_port = port;

		if (connect(fd, (struct sockaddr *)&sa, sizeof(sa)) < 0)
		{
			close(fd);
			printf("failed ipc connect.\n");
			return -1;
		}

		/* Linux 2.4 kernel has a bug with localhost connections.
		* After the server calls listen(), then the client calls
		* connect() and returns with success immediately, before
		* the server calls accept()!
		* Therefore, a SYNC_STR command is used to make sure the connect
		* succeeded.
		* This has not been tested on Linux 2.2 whether it too has
		* this bug. It probably does not exist on VxWorks, since VxWorks
		* TCP/IP stack is completely different (its BSD based).
		*/

		if (!ipc_client_sync(fd))
			break;

		close(fd);
//		rg_error(LWARN, "failed ipc connect sync - retrying");
		printf( "failed ipc connect sync - retrying\n");
		sys_sleep(1);
    }

    return fd;
}

int ipc_listen(u16 port)
{
    return ipc_listen_ip(port, inet_addr("127.0.0.1"));
}

int ipc_listen_ip(u16 port, u32 addr)
{
    int fd;
    
    if ((fd = sock_socket(SOCK_STREAM, addr, port)) < 0)
	return -1;
    /* 5 is a big enough queue length */
    if (listen(fd, 5) < 0)
    {
	rg_error(LERR, "failed ipc listen %m");
	goto Error;
    }
    return fd;

Error:
    close(fd);
    return -1;
}

int ipc_accept(int server_fd)
{
    int fd;

    if ((fd = bio_accept(server_fd)) < 0)
    {
	rg_error(LERR, "%d: failed ipc accept %m", server_fd);
	return -1;
    }
    if (ipc_server_sync(fd))
    {
	rg_error(LERR, "%d: failed ipc accept sync", server_fd);
	goto Error;
    }
    return fd;

Error:
    close(fd);
    return -1;
}

int ipc_client_close(int fd, int last_error)
{
    int ret = 0;
    
    if (!last_error)
	ret = ipc_client_sync(fd);
    close(fd);
    return ret;
}

int ipc_server_close(int fd, int last_error)
{
    int ret = 0;
    
    if (!last_error)
	ret = ipc_server_sync(fd);
    close(fd);
    return ret;
}

int ipc_client_sync(int fd)
{
#ifdef VXWORKS
    return 0;
#else
    char *sync = NULL;
    int rc = -1;

    if (ipc_string_write(fd, SYNC_STR))
	goto Exit;
    if (!(sync = ipc_string_read(fd)) || strcmp(sync, SYNC_STR))
	goto Exit;
    rc = 0;

Exit:
    if (rc)
	rg_error(LERR, "%d: failed ipc client sync", fd);
    nfree(sync);
    return rc;
#endif
}

int ipc_server_sync(int fd)
{
#ifdef VXWORKS
    return 0;
#else
    char *sync = NULL;
    int rc = -1;

    if (!(sync = ipc_string_read(fd)) || strcmp(sync, SYNC_STR))
	goto Exit;
    if (ipc_string_write(fd, SYNC_STR))
	goto Exit;
    rc = 0;

Exit:
    if (rc)
	rg_error(LERR, "%d: failed ipc server sync", fd);
    nfree(sync);
    return rc;
#endif
}

/* This function is blocking and it writes from the socket exactly 'count'
 * number of bytes
 */
int ipc_write(int fd, void *buf, size_t count)
{
    int rc;
 
    if ((rc = bio_write(fd, buf, count)))
	{
		printf("%d: failed write()", fd);

	}
    return rc;
}

int ipc_read(int fd, void *buf, size_t count)
{
    return bio_read(fd, buf, count);
}

int ipc_u32_read(int fd, u32 *n)
{
    if (ipc_read(fd, n, sizeof(*n)))
	return -1;
    *n = ntohl(*n);
    return 0;
}

int ipc_u32_write(int fd, u32 n)
{
    n = htonl(n);
    return ipc_write(fd, &n, sizeof(n));
}

static int ipc_varbuf_read(int fd, char **data, int *len)
{
    if (ipc_u32_read(fd, len))
	return -1;
    if (!(*data = malloc_l(*len)))
	goto Error;
    if (ipc_read(fd, *data, *len))
	goto Error;
    return 0;

Error:
    nfree(*data);
    return -1;
}

char *ipc_string_read(int fd)
{
    char *data;
    int len;
    
    if (ipc_varbuf_read(fd, &data, &len) || !len)
	return NULL;
    /* '\0' should be sent by the other side, we want to be on the safe side */
    data[len-1] = 0;
    return data;
}

int ipc_string_write(int fd, char *str)
{
    int len;

    len = strlen(str) + 1;
    if (ipc_u32_write(fd, len))
	return -1;
    if (ipc_write(fd, str, len))
	return -1;
    return 0;
}

/* High level simple API */

/* Open socket on a specific port for reading. */
int ipc_bind_listen_port(u16 port)
{
    return ipc_listen(port);
}

/* For reading data from a socket - use it in the module task. */
int ipc_accept_buf_read(int server_fd, void *data, int data_len)
{
    int fd, rc;
    
    if ((fd = ipc_accept(server_fd)) < 0)
	    return -1;
    rc = ipc_read(fd, data, data_len);
    ipc_server_close(fd, rc);
    return rc;
}

/* For writing data to a specific port via UDP socket - use it in the external process. */
int ipc_buf_write(u16 port, void *data, int data_len)
{
	int sock, rc;	
	struct sockaddr_in sockaddr;
	int sockAddrSize = sizeof(struct sockaddr_in);

    /* Create socket on which to send. */
	sock = socket(AF_INET, SOCK_DGRAM, 0);
	if (sock < 0) {
		perror("opening datagram socket");
		return -1;
	}
	bzero((char *)&sockaddr, sockAddrSize);
    sockaddr.sin_len = sockAddrSize;
    sockaddr.sin_family = AF_INET;
    sockaddr.sin_port = htons(port);
    sockaddr.sin_addr.s_addr = inet_addr(IPC_LOCAL_INET_ADDR);
    if(sockaddr.sin_addr.s_addr == ERROR) {
	    printf("%s : failed in inet_addr error: %s\n",__FUNCTION__, strerror(errno));
        close(sock);
        return -1;
    }
		/* Send message. */
	rc = sendto(sock, data, data_len, 0,
				(struct sockaddr *)&sockaddr, sockAddrSize) ;
	if (rc < 0) {
		perror("send failed");
		return -1;
	}
	else if (rc != data_len) {
		printf("%s: sendto failed: tried to send %d bytes, sent %d\n",__func__, data_len, rc);
		return -1;
	}
	close(sock);

	return 0;
}

/* For writing data to a specific port via TCP socket - use it in the external process. */
int ipc_connect_buf_write(u16 port, void *data, int data_len)
{

    int fd, rc;

    if ((fd = ipc_connect(port)) < 0)
	{
		printf("ipc_connect_buf_write :error connect.\n");
		return -1;
	}
    rc = ipc_write(fd, data, data_len);
	ipc_client_close(fd, rc);

    return rc;
}

#ifdef VXWORKS
/******************************************************************************
*                      ipc_create_sock()								  *
*******************************************************************************
DESCRIPTION:	Creates and binds a socket for interprocess communication in VxWorks 
                                                                         
INPUTS:			sock_port - port 

OUTPUT:			pSock 	  - an output buffer for socket

RETURNS:		OK/NOK                                             
******************************************************************************/
STATUS ipc_create_sock(UINT32 sock_port, UINT32 *pSock)
{

   UINT32  sock;
   struct sockaddr_in name;
   int sockAddrSize;
    

    /* Create socket from which to read. */
    sock = socket(AF_INET, SOCK_DGRAM, 0);
    if (sock < 0) {
        printf("WPA HIDDEN INST.:error opening datagram socket for WDS\n");
        return NOK;
    }
    /* Create name. */
	name.sin_family = AF_INET;
    sockAddrSize = sizeof(struct sockaddr_in);
    bzero((char *)&name, sockAddrSize);
    name.sin_len = sockAddrSize;
    name.sin_family = AF_INET;
    name.sin_port = htons(sock_port);
    name.sin_addr.s_addr = inet_addr(IPC_LOCAL_INET_ADDR);
    if(name.sin_addr.s_addr == ERROR) {
	    printf("%s : inet_addr returns error: %s\n",__FUNCTION__, strerror(errno));
        close(sock);
        return (NOK);
    }
	/* Bind the  domain address to the created socket */
    if (bind(sock, (struct sockaddr *) &name, sizeof(name)) < 0 )
	{
		  printf("WPA: error binding datagram socket\n");
          return NOK;
    }

	*pSock = sock;
	return OK;
}
#endif /* ifdef VXWORKS*/  
