/* LICENSE_CODE JUNGO */

#include <net_utils.h>
#include <event.h>
#include <util/random.h>
#include <rg_memory.h>
#include <rg_error.h>
#include <syscalls.h>
#include "wpa_port.h"
#include "wpa_info_elem.h"
#include "wpa_common.h"
#include "wpa_assoc.h"
#include "wpa_info_elem.h"
#include "wpa_key_utils.h"
#include "wpa_log.h"
#include "wpa_pmk_cache.h"
#include "hiddenSecurityProfileAPI.h"
#include "debug_report.h"

#define MIC_FAILURE_DELAY_MS 60000 /* 60 sec */
#define MIC_COUNTERMEASURES_PERIOD_MS 60000 /* 60 sec */

#define PORT_IN_WPA_PSK_MODE(t) (t->params.psk_param != WPA_PSK_PARAM_NONE)

void wpa_port_update_sta_counter(wpa_port_t *t, wpa_sta_type_t sta_type, 
								 int delta)
{
    switch(sta_type)
    {
	case WPA_STA_TYPE_WPA:
	case WPA_STA_TYPE_WPA2:
		t->num_of_wpa_stas += delta;
		break;
		
	case WPA_STA_TYPE_WEP_LEGACY:
		t->num_of_legacy_stas += delta;
		break;
		
	case WPA_STA_TYPE_WEP_8021X:
		t->num_of_8021x_stas += delta;
		break;
		
	default:
		break;
    }
}

static wpa_assoc_t *find_assoc(wpa_port_t *t, ether_mac_t mac)
{
    wpa_assoc_t *assoc;
	
    for (assoc = t->assoc_list; assoc; assoc = assoc->next)
    {
		if (!memcmp(assoc->sta_mac, mac, sizeof(ether_mac_t)))
			break;
    }
	
    return assoc;
}


wpa_cipher_t select_multicast_cipher(wpa_port_params_t *params)
{
    wpa_cipher_t multicast_cipher;
    
    /* If WEP-legacy stations are allowed, we must use the cipher of the 
	* preconfigured key as our multicast cipher
	*/
    if(params->rekeying_wep_cipher == WPA_CIPHER_TKIP)
		return (multicast_cipher = WPA_CIPHER_TKIP);
	
	if (params->allowed_sta_types & WPA_STA_TYPE_WEP_LEGACY)
		multicast_cipher = params->wep_keys[params->wep_key_index].cipher;
	
		/* If there are WEP-802.1X stations, we must use the WEP cipher configured
	* by the user for multicast */ 
    else if ((params->allowed_sta_types & WPA_STA_TYPE_WEP_8021X))
		multicast_cipher = params->rekeying_wep_cipher;
	
    /* Not configured for backward compatibility, we can use TKIP */
    else if (
		(params->allowed_sta_types & WPA_STA_TYPE_WPA2) &&
		!(params->allowed_sta_types & WPA_STA_TYPE_WPA)
		)
		multicast_cipher = WPA_CIPHER_CCMP;
	else
		multicast_cipher = WPA_CIPHER_TKIP;
	
    return multicast_cipher;
}
static int dup_params(wpa_port_params_t *dst_params, 
					  wpa_port_params_t *src_params)
{
    dst_params->allowed_sta_types = src_params->allowed_sta_types;
    dst_params->rekeying_wep_cipher = src_params->rekeying_wep_cipher;
    dst_params->gtk_update_interval = src_params->gtk_update_interval;
    memcpy(dst_params->wep_keys, src_params->wep_keys, 
		sizeof(dst_params->wep_keys));
    dst_params->wep_key_index = src_params->wep_key_index;
    dst_params->ssid_len = src_params->ssid_len;
    dst_params->dev_id = src_params->dev_id;
    dst_params->auth1x_port = src_params->auth1x_port;
	dst_params->pre_auth_on = src_params->pre_auth_on;

    memcpy(dst_params->mac, src_params->mac, sizeof(ether_mac_t));
	
    /* Copy PSK */
    dst_params->psk_param = src_params->psk_param;
	
    switch (src_params->psk_param)
    {
    case WPA_PSK_PARAM_ASCII:
		dst_params->psk.ascii = strdup_l(src_params->psk.ascii);
		
		if (!dst_params->psk.ascii)
			return -1;
		break;
		
    case WPA_PSK_PARAM_HEX:
		memcpy(&dst_params->psk.hex, &src_params->psk.hex, sizeof(wpa_psk_t));
		break;
		
    default:
		break;
    }
	
    /* Copy SSID */
    dst_params->ssid = malloc_l(src_params->ssid_len);
    
    if (!dst_params->ssid)
		goto Error;
	
    memcpy(dst_params->ssid, src_params->ssid, dst_params->ssid_len);
	
    /* Copy ap_dev_ops */
    dst_params->ap_dev_ops = malloc_l(sizeof(ap_dev_ops_t));
	
    if (!dst_params->ap_dev_ops)
		goto Error;
    
    memcpy(dst_params->ap_dev_ops, src_params->ap_dev_ops, 
		sizeof(ap_dev_ops_t)); 
	
    return 0;
	
Error:
    if (src_params->psk_param == WPA_PSK_PARAM_ASCII)
		nfree(dst_params->psk.ascii);
    nfree(dst_params->ssid);
    return -1;
}

static void free_params(wpa_port_t *t)
{
    if (t->params.psk_param == WPA_PSK_PARAM_ASCII)
		nfree(t->params.psk.ascii);
    
    nfree(t->params.ssid);
    nfree(t->params.ap_dev_ops);
}

static void generate_gtk(wpa_port_t *t)
{
    wpa_gmk_t gmk;
    
    DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Generating GTK\n");
	
    /* Generate a random master key */
    random_buf_get((unsigned char *)&gmk, sizeof(gmk));
	
    /* Generate GNonce */
    wpa_port_generate_nonce(t, &t->gnonce);
	
    /* Generate GTK */
    wpa_key_utils_calc_gtk(&t->gtk, &gmk, &t->gnonce, t->params.mac);

    DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Generated GTK: %s\n",
		wpa_log_format_buf((u8 *)&t->gtk, wpa_key_utils_get_key_len(t->multicast_cipher)));
}

static void gtk_update_timer_cb(void *t);

static void install_single_preconfigured_key(wpa_port_t *t, int index)
{
    DEBUG_PRINT_m(SEV_INFO, MOD_WPA_INIT,"Installing preconfigured WEP key in index %d. Key=\n%s", index, 
		wpa_log_format_buf((u8 *)t->params.wep_keys[index].data,
		wpa_key_utils_get_key_len(t->params.wep_keys[index].cipher)));
	
    t->params.ap_dev_ops->set_default_key(t->port_profile_index, index, 
		t->params.wep_keys[index].cipher, t->params.allowed_sta_types, 
		(wpa_temporal_key_t *)t->params.wep_keys[index].data); 
}

static void install_preconfigured_keys(wpa_port_t *t, 
									   int delay_current_gtk_installation)
{
    int i;
	
    /* Install all preconfigured keys. In some cases we don't want the current
	* group key to be replaced right away because this will cause a situation
	* where we're using the new key as our group key and our connected stations
	* still use the old key. To prevent this we'll first send the stations
	* another group key and only once all the stations have switched to it,
	* we'll replace the old group key
	*/
    for (i = 0; i < WPA_NUM_DEFAULT_KEYS; i++)
    {
		if (!delay_current_gtk_installation || i != t->current_gtk_index)
			install_single_preconfigured_key(t, i);
    }
	
    if (delay_current_gtk_installation)
    {
		DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Delaying installation of key in index %d", 
			t->current_gtk_index);
		
			/* Take note that we still have a preconfigured key to install when
			* the next GTK update is over
		*/
		t->delayed_preconfigured_key_install = t->current_gtk_index;
    }
}

static void schedule_next_gtk_update(wpa_port_t *t, int ms_to_wait)
{
    /* We must not update the GTK if a legacy station is connected */
    if (!t->num_of_legacy_stas)
		event_timer_set(ms_to_wait, gtk_update_timer_cb, t);
}

static void install_gtk(wpa_port_t *t)
{
    DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Installing a %s GTK in index %d:\n%s, profile = %d\n",wpa_log_cipher_to_str(t->multicast_cipher), t->current_gtk_index,
		wpa_log_format_buf((u8 *)&t->gtk, wpa_key_utils_get_key_len(t->multicast_cipher)),t->port_profile_index);

    /* Install GTK as a default key */
    t->params.ap_dev_ops->set_default_key(t->port_profile_index, t->current_gtk_index, 
		t->multicast_cipher, t->params.allowed_sta_types, &t->gtk);
	
#ifdef VXWORKS
	taskDelay(1);
#endif
		
    /* We may have a preconfigured key whose installation was delayed. Now is
	* the time to install it.
	*/
    if (t->delayed_preconfigured_key_install)
    {
		install_single_preconfigured_key(t, 
			t->delayed_preconfigured_key_install);
		
		t->delayed_preconfigured_key_install = 0;
    }
	
    /* We can now decrypt our mutlicast/broadcast traffic so we can stop
	* blocking it
	*/
    t->params.ap_dev_ops->block_multicast_traffic(t->ap_dev, 0);
    
    if (t->params.gtk_update_interval)
		schedule_next_gtk_update(t, t->params.gtk_update_interval);
}

static void update_gtk(wpa_port_t *t)
{
    wpa_assoc_t *assoc = t->assoc_list;
	
    DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Updating GTK\n");
	
    /* If we're still waiting for some stations to acknowledge the last GTK 
	* update, postpone the requested GTK update for later
	*/
    if (t->gtk_update_in_progress)
    {
		DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Postponing GTK update because a previous GTK update is "
			"still in progress\n");
		
		t->delayed_gtk_update = 1;
		
		return;
    }
	
    /* Remove GTK key update callback */
    event_timer_del(gtk_update_timer_cb, t);
	
    /* When there is no need to support legacy stations, the key can be
	* dynamically generated and the index can be swapped on every update.
	* When legacy stations should be supported we must always use the 
	* preconfigured key and index for our group key
	*/
    if (t->params.allowed_sta_types != WPA_STA_TYPE_WEP_LEGACY && 
		!t->num_of_legacy_stas)
    {
		/* Swap group key index */
		t->current_gtk_index = t->current_gtk_index == 1 ? 2 : 1;

		/* Generate a new GTK */
		generate_gtk(t);
    }
    else
    {
		/* Use preconfigured key and index for our next group key */
		t->current_gtk_index = t->params.wep_key_index;
		
		memcpy(&t->gtk, t->params.wep_keys[t->params.wep_key_index].data, 
			sizeof(t->params.wep_keys[t->params.wep_key_index].data));
    }
    
    /* Start group key update for all stations that support it */
    for (assoc = t->assoc_list; assoc; assoc = assoc->next)
    {
	/* Only send group key updates to station that already has the old group
	* key
		*/
		if (!assoc->has_gtk)
			continue;

		/* indicate that this assoc was in assoc list when we started GTK update 
		  (those that would connect during this GTK update won't have this flag on) */
        assoc->in_gtk_update = 1;
		
		/* Send the new GTK to this station */
		wpa_assoc_start_gtk_neg(assoc);
		
		t->gtk_update_in_progress = 1;
    }
	
    /* If we didn't start a GTK update with any station, we don't have to wait
	* for acknowledgments, we can install the new GTK right away
	*/
    if (!t->gtk_update_in_progress)
		install_gtk(t);
}

static void gtk_update_timer_cb(void *context)
{
    wpa_port_t *t = (wpa_port_t *)context;
	
    DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"GTK expired\n");
    
    /* Generate a new GTK and send new GTK to stations */
    update_gtk(t);
}

static void remove_assoc(wpa_port_t *t, ether_mac_t mac)
{
    wpa_assoc_t **assoc = &t->assoc_list;
    wpa_assoc_t *removed_assoc;
	
    /* Look for an association with the specified MAC */
    while ((*assoc) && memcmp(mac, (*assoc)->sta_mac, sizeof(ether_mac_t)))
		assoc = &((*assoc)->next);
	
    /* No such association */
    if (!(*assoc))
		return;
	
    /* Remove association from list */
    removed_assoc = *assoc;
    *assoc = (*assoc)->next;
	
	/* Update a pointer to staion assoc in PMK cache to NULL to indicate that station doesn't exist */
    if (removed_assoc->sta_type == WPA_STA_TYPE_WPA2) 
		wpa_pmk_cache_update_assoc_by_mac(removed_assoc->port, removed_assoc->sta_mac, NULL);
	
	/* Update station counter to reflect the fact that the station is leaving */
    wpa_port_update_sta_counter(t, removed_assoc->sta_type, -1);
    
    /* Close association */
    wpa_assoc_close(removed_assoc);
}

/* Called when it's time to stop the countermeasures and return to normal
* operation mode
*/
void countermeasures_timer_cb(void *context)
{
    wpa_port_t *t = (wpa_port_t *)context;
	
    t->countermeasures_invoked = 0;
}

static void test_for_end_of_gtk_update(wpa_port_t *t)
{
    wpa_assoc_t *assoc;
	
    /* If we already know that there's no GTK update in progress, there's no
	* need to go on
	*/
    if (!t->gtk_update_in_progress)
		return;
	
		/* Go over all stations and check if there are still stations that are in
		* the middle of a GTK update
	*/
    t->gtk_update_in_progress = 0;
	
    for (assoc = t->assoc_list; assoc; assoc = assoc->next)
    {
		if (assoc->in_gtk_update)
		{
			if ( wpa_assoc_is_in_gtk_handshake(assoc) )
			{
				DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Station %s still performing GTK update - don't install GTK\n",
							  mactoa(&assoc->sta_mac));
				
				t->gtk_update_in_progress = 1;
				break;
			}
			else
            {
				assoc->in_gtk_update = 0;
            }
		}

    }
	
    /* If we found out that the GTK update is over, install the new GTK */
    if (!t->gtk_update_in_progress)
    {
		install_gtk(t);
		
		/* In case that a GTK update was issued while the last GTK update was in
		* progress, the second GTK was delayed and should take place now
		*/
		if (t->delayed_gtk_update)
		{
			t->delayed_gtk_update = 0;
			
			DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Doing postponed GTK update\n");
			
			update_gtk(t);
		}
    }
}

static void handle_sta_disassoc(wpa_port_t *t, wpa_assoc_t *assoc)
{
    int sta_has_gtk;
    wpa_sta_type_t sta_type;
	
    /* Save some information about the station before removing it */
    sta_has_gtk = assoc->has_gtk;
    sta_type = assoc->sta_type;
	
    /* Remove station's assoication object from list */
    remove_assoc(t, assoc->sta_mac);

	/* Don't perform gtk update for hidden profile */
	if (t->port_profile_index == wdrv_wpaGetHiddenProfileIndex()) 
		return;
	
    /* If we were waiting for this station to acknowledge receiving the GTK,
	* because of it disassociating, we're never going to get this 
	* acknowledgement. In case it was the last station we're waiting to get 
	* acknowledgment from, we can install the new GTK now.
	*/
    test_for_end_of_gtk_update(t);
	
    /* Update group key in 2 cases:
	* 1. If the leaving station has the current GTK, so it will no longer be 
	*    able to decrypt multicast/broadcast traffic. This is unless WEP-legacy
	*    stations are connected, in which case GTK update is disabled.
	* 2. The leaving station was the last legacy station connected and 802.1X
	*    and/or WPA stations are allowed. This means that GTK update was 
	*    disabled to support WEP-legacy stations, and can now be enabled again.
	*/
    if (!t->num_of_legacy_stas && (sta_has_gtk || 
		(sta_type == WPA_STA_TYPE_WEP_LEGACY && 
		(t->params.allowed_sta_types & 
		(WPA_STA_TYPE_WPA | WPA_STA_TYPE_WEP_8021X | WPA_STA_TYPE_WPA2)))))
    {
	/* Don't do the GTK update immediately since we might be called from a
	* loop disassociating a number of stations and we don't want to start a
	* separate GTK update for each disassociating station
		*/ 
		schedule_next_gtk_update(t, 0);
    }
}

static void deauth_sta(wpa_port_t *t, wpa_assoc_t *assoc)
{
    wpa_assoc_t save_assoc = *assoc; /* Save assoc before it is freed */

    handle_sta_disassoc(t, assoc);

	/* Send deauth meassage to the driver*/
	wpa_assoc_deauth(&save_assoc);

}

void wpa_port_close(wpa_port_t *t, int profile_index)
{
/* Remove WPA information element from beacons to indicate that WPA is 
* disabled
	*/
    t->params.ap_dev_ops->set_ap_wpa_info_elem(t->ap_dev, NULL, 0);
    
    /* Deauthenticate all stations and free all association objects */
    while (t->assoc_list)
    {
	   wpa_assoc_t save_assoc;

		/* Save assoc before it is freed */
		save_assoc = *(t->assoc_list);

		remove_assoc(t, t->assoc_list->sta_mac);
		
		/* Save deauth message to the driver */
		wpa_assoc_deauth(&save_assoc);

    }
    
    /* Free information element */
	if (t->ap_wpa_info_elem)
    wpa_info_elem_close(t->ap_wpa_info_elem);

	if (t->ap_rsn_info_elem)
		wpa_info_elem_close(t->ap_rsn_info_elem);

	/* remove pmk cache */
	if ((t->params.allowed_sta_types & WPA_STA_TYPE_WPA2) || (t->params.allowed_sta_types & WPA_STA_TYPE_WPA))
		wpa_pmk_cache_kill(profile_index);
    
    /* Close AP device */
    t->params.ap_dev_ops->close(t->ap_dev);
	
    /* Free parameters */
    free_params(t);
		
    /* Remove callbacks */
    event_timer_del(gtk_update_timer_cb, t);
    event_timer_del(countermeasures_timer_cb, t);
    
    free(t);
}

/* Inspects station's IE and returns 0 (to allow station to associate) if
* station type is supported according to AP's configuration. Returns a reason 
* code as defined in the IEEE 802.11i otherwise
*/
static int sta_assoc_request_cb(void *context, u8 *info_elem_buf, 
								int info_elem_buf_len)
{
    wpa_cipher_t multicast_cipher, unicast_ciphers;
    wpa_auth_suite_t auth_suites;
    wpa_info_elem_t *info_elem = (wpa_info_elem_t *)info_elem_buf;
    wpa_port_t *t = (wpa_port_t *)context;
	int info_elem_type;

    DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Station is requesting to associate\n");
	
    /* If MIC countermeasures are invoked due to 2 MIC failure within one 
	* minute and if this is a WPA station, reject it
	*/
    if (t->countermeasures_invoked && info_elem)
    {
		DEBUG_PRINT_m(SEV_WARNING, MOD_WPA_AUTH, "wpa: Rejecting station since no WPA associations are "
			"currently allowed due to a recent MIC failure");
		goto Reject;
    }
	
    /* We can tell by the presence or absence of an information element if
	* this is a WPA station or not. Reject station if its type is not allowed
	*/
	
    if ((!info_elem &&
		!(t->params.allowed_sta_types & (WPA_STA_TYPE_WEP_LEGACY |
		WPA_STA_TYPE_WEP_8021X))) || (info_elem && !(t->params.allowed_sta_types
		& (WPA_STA_TYPE_WPA| WPA_STA_TYPE_WPA2))))
    {
		DEBUG_PRINT_m(SEV_WARNING, MOD_WPA_AUTH, "wpa: Rejecting station since AP is configured not "
			"allow %sWPA stations", info_elem ? "" : "non-");
		
		goto Reject;
    }
	
    /* If this is a WPA station, we still have to check its information element,
	* otherwise we're done here
	*/
    if (!info_elem)
		goto Accept;
	
    /* Parse station's information element */
    info_elem_type = wpa_info_elem_get_info_type(info_elem);
	
	if(info_elem_type == WPA_STA_TYPE_WPA)
	{
		if (wpa_info_elem_parse(info_elem, &multicast_cipher, &unicast_ciphers,
			&auth_suites, info_elem_buf_len))
		{
			DEBUG_PRINT_m(SEV_WARNING, MOD_WPA_AUTH, "wpa: Rejecting station since it has a "
				"malformed information element");
			goto Reject;
		}
	}
	else if(info_elem_type == WPA_STA_TYPE_WPA2)
	{
		int ret;
		u8 *pmk_id = NULL,pre_auth_flag = 0 ,num_pmk_id = 0;
	
		if ( (ret = wpa2_info_elem_parse(info_elem, &multicast_cipher, &unicast_ciphers,
			&auth_suites,&pre_auth_flag,&pmk_id,&num_pmk_id,info_elem_buf_len)) == -1)
		{
			DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"wpa: Rejecting station since it has a "
				"malformed information element(wpa2)\n");
			goto Reject;
		}
		
		if(num_pmk_id != 0)
			free(pmk_id);
	}
	else
	{
		DEBUG_PRINT_m(SEV_WARNING, MOD_WPA_AUTH, "wpa: Rejecting station since it has unknown "
				" information element type");
		goto Reject;
	}

    /* Reject station if it doesn't support our authentication suite */
    if ((PORT_IN_WPA_PSK_MODE(t) && !(auth_suites & WPA_AUTH_PSK)) || 
		(!PORT_IN_WPA_PSK_MODE(t) && !(auth_suites & WPA_AUTH_8021X)))
    {
		DEBUG_PRINT_m(SEV_WARNING, MOD_WPA_AUTH, "wpa: Rejecting station since it can't use AP's "
			"authentication suite");
		goto Reject;
    }
	
    /* Reject station if it can't use TKIP OR CCMP as a unicast cipher */
    if (!((unicast_ciphers & WPA_CIPHER_TKIP) || (unicast_ciphers & WPA_CIPHER_CCMP)) )
    {
		DEBUG_PRINT_m(SEV_WARNING, MOD_WPA_AUTH, "wpa: Rejecting station since it can't use TKIP Or ccmp as unicast cipher");
		goto Reject;
    }
	
    /* Reject station if it can't use our current multicast cipher */
    if (!(multicast_cipher && t->multicast_cipher))
    {
		DEBUG_PRINT_m(SEV_WARNING, MOD_WPA_AUTH, "wpa: Rejecting station since it can't use AP's "
			"multicast cipher");
		goto Reject;
    }
	
	/* station was in our pmk cache list and the pmk
	   is valid so we will use it and skip the authentication server connection.
	*/

Accept:
    DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Accepting station\n");
    return 0;
	
Reject:
    /* We currently always return 1 as a reason code for rejection */
    return 1;
}

/* Called when a station's association process is complete by the underlying
* layer
*/
static void sta_assoc_done_cb(void *context, ether_mac_t mac, u8 *info_elem_buf,
							  int info_elem_buf_len)
{
    wpa_assoc_t *assoc;
    wpa_port_t *t = (wpa_port_t *)context;
    wpa_info_elem_t *info_elem = (wpa_info_elem_t *)info_elem_buf;
	
    DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Station %s associated\n", mactoa((ether_mac_t *)mac));
    /* Start blocking data traffic going to this station, the blocking will be
	* removed later, once the station authenticates or once we detect that 
	* it's a legacy station incapable of authenticating
	*/
    t->params.ap_dev_ops->block_unicast_traffic(t->ap_dev, 1, mac);
	
    /* If association object for this station already exists, remove it before
	* we add it again
	*/
    remove_assoc(t, mac);
	
    /* Create a new association object */
    assoc = wpa_assoc_open(t, mac, info_elem,EAPOL_ETHER_TYPE);
    if (!assoc)
    {
		DEBUG_PRINT_m(SEV_WARNING, MOD_WPA_AUTH, "wpa: Can't allocate an association object for "
			"station");
		
		return;
    }
	
    /* Add association to list */
    assoc->next = t->assoc_list;
    t->assoc_list = assoc;
	
    /* In some cases we can know the station type at this stage, so we do
	* type-specific handling now, otherwise this handling will have to be done
	* later when we'll detect the station type
	*/
    if (assoc->sta_type)
		wpa_port_sta_type_detected(t, assoc);
    
    /* XXX Maybe this should move to wpa_assoc_open? */
    /* If this is a WPA station and we're in PSK mode,or this
	 station is in our pmk cache list or pre authentiacte, this is the time to start
	* 4-way-handshake ,
	*/
    if (
		(assoc->sta_type == WPA_STA_TYPE_WPA ||assoc->sta_type == WPA_STA_TYPE_WPA2)
		&& (PORT_IN_WPA_PSK_MODE(t) || assoc->pmk_valid == 1)
		)
	{
		wpa_assoc_start_ptk_neg(assoc);
	}
}

void wpa_port_8021x_auth_end(wpa_port_t *t,wpa_assoc_t *assoc, int success,double session_timeout)
{
    if (!success)
    {
		DEBUG_PRINT_m(SEV_WARNING, MOD_WPA_AUTH, "wpa: Failed to authenticate station %s",
			mactoa(&assoc->sta_mac));
		
		/* Deauthenticate station */
		deauth_sta(t, assoc);
		
		return;
    }
	
    DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Station %s authenticated\n", mactoa(&assoc->sta_mac));
	
	/* 
	check:case pre-authentication dont start 4 way handshake !!!*/
	if(assoc->sta_type == WPA_STA_TYPE_WPA2)
		wpa_pmk_cache_insert(assoc,session_timeout);
 	if(!assoc->pre_auth_conn)
	{
		wpa_assoc_start_ptk_neg(assoc);
	}
	else
	{
		DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"pre authentication end ,closing station assoc\n");
		remove_assoc(t,assoc->sta_mac);
	}
}

void wpa_port_ptk_neg_end(wpa_port_t *t, wpa_assoc_t *assoc, int success)
{
    if (!success)
    {
		if ((assoc->sta_type == WPA_STA_TYPE_WPA) || (assoc->sta_type == WPA_STA_TYPE_WPA2) )
		{
			DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_AUTH, "Failed doing 4-way handshake with station %s", 
				mactoa(&assoc->sta_mac));
		}
		else
		{
			DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_AUTH, "Failed sending pairwise key to station %s",
				mactoa(&assoc->sta_mac));
		}
		
		/* report that ptk negotiation failed - only for internal profile by now */
		authStatusReport(assoc->sta_mac, SEC_AUTH_STATUS_FAILURE);

		/* Deauthenticate station */
		deauth_sta(t, assoc);
		return;
    }
	
    if (assoc->sta_type == WPA_STA_TYPE_WPA || assoc->sta_type == WPA_STA_TYPE_WPA2)
    {
		DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"4-way handshake with station %s completed successfully", 
			mactoa(&assoc->sta_mac));
    }
    else
    {
		DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Pairwise key sent to station %s successfully", 
			mactoa(&assoc->sta_mac));
    }

	/* report that ptk negotiation finished successfully - only for internal profile by now */
	authStatusReport(assoc->sta_mac, SEC_AUTH_STATUS_OK); 
    
    /* Start group key update for the station */
	if(!assoc->has_gtk)
		wpa_assoc_start_gtk_neg(assoc);
	
		
}

void wpa_port_gtk_neg_end(wpa_port_t *t, wpa_assoc_t *assoc, int success)
{
    if (!success)
    {
		/* Deauthenticate station */
		deauth_sta(t, assoc);
		
		return;
    }
    
    /* This station have just acknowledged receiving the GTK. Check if it's the
	* last station we were waiting to acknowledge. In which case, we can
	* install the new GTK now.
	*/
    test_for_end_of_gtk_update(t);
}

/* Called when the underlying layer detects that a station disassociated */
static void sta_disassoc_cb(void *context, ether_mac_t mac)
{
    wpa_port_t *t = (wpa_port_t *)context;
    wpa_assoc_t *assoc;
	
    DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Station %s disassociated", mactoa((ether_mac_t *)mac));
    
    assoc = find_assoc(t, mac);
    if (!assoc)
		return;
    
    handle_sta_disassoc(t, assoc);
}

/* Called when a MIC failure is detected in a packet protected with a unicast
* key
*/ 
static void unicast_mic_failure_cb(void *context, ether_mac_t mac)
{
    wpa_port_t *t = (wpa_port_t *)context;
    wpa_assoc_t *assoc = find_assoc(t, mac);
	
    if (!assoc)
    {
		DEBUG_PRINT_m(SEV_WARNING, MOD_WPA_AUTH, "wpa: MIC failure detected in a packet received from "
			"an unknown station %s", mactoa((ether_mac_t *)&mac));
		return;
    }
	
    DEBUG_PRINT_m(SEV_WARNING, MOD_WPA_AUTH, "wpa: MIC failure detected in a packet received from "
		"station %s", mactoa(&assoc->sta_mac));
	
    wpa_port_mic_failure(t, assoc);
}

/* Called when a first data packet is received from a station, judging by the
* type of this packet we can tell whether a WEP station supports 802.1X
*/
static void first_data_packet_received_cb(void *context, int is_eapol, 
										  ether_mac_t mac)
{
    wpa_port_t *t = (wpa_port_t *)context;
    wpa_assoc_t *assoc = find_assoc(t, mac);
	
    DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"First packet received from station %s is %sEAPOL",
		mactoa((ether_mac_t *)mac), is_eapol ? "" : "non-");
    
    if (!assoc)
	{
	    DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_AUTH,"cant find station %s assoc obj.",
			mactoa((ether_mac_t *)mac));
		return;
	}
		/* Station type is unknown, try to figure it out according to the type of
		* the first packet
	*/
    if (!assoc->sta_type)
    {
	/* 802.1X stations send EAPOL packets as their first packets while 
	* legacy stations don't send EAPOL packets at all
		*/
		wpa_assoc_set_sta_type(assoc, is_eapol ? WPA_STA_TYPE_WEP_8021X :
	WPA_STA_TYPE_WEP_LEGACY);
    }
    /* Handle a special case where we assumed that the station was a WEP-legacy
	* station (to avoid to 2 sec delay) but it sends an EAPOL-Start message so
	* it turns out that it's actually a 802.1X station
	*/
#if 0
	/* Ths following code had been removed in order to support legacy station that sends EAPOL-Start 
	as their first data packet */
    else if ((assoc->sta_type == WPA_STA_TYPE_WEP_LEGACY) && is_eapol)
		wpa_assoc_set_sta_type(assoc, WPA_STA_TYPE_WEP_8021X);
		/* We already know the type of this station so we already did the 
		* type-specific handling for it, no need to go on
	*/
#endif
    else
	{
        switch(assoc->sta_type) {
            case WPA_STA_TYPE_WPA:
                DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"\nwpa: First packet received station type is WPA\n");
                break;
            case WPA_STA_TYPE_WPA2:
                DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"\nwpa: First packet received station type is WPA2\n");
                break;
            case WPA_STA_TYPE_WEP_LEGACY:
                DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"\nwpa: First packet received station type is WEP-Legacy\n");
                break;
            case WPA_STA_TYPE_WEP_8021X:
                DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"\nwpa: First packet received station type is WEP-802.1X\n");
                break;
            default:
                DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"\nwpa: First packet received station type is Unknown\n");
                return;
                break;
        }
	}
	
    /* Now that we know the station type, do type-specific handling for the 
	* station
	*/
    wpa_port_sta_type_detected(t, assoc); 
}

/* Callbacks to be called by the underlying layer */
static ap_dev_cbs_t ap_dev_cbs = {
    sta_assoc_request_cb,
		sta_assoc_done_cb,
		sta_disassoc_cb,
		unicast_mic_failure_cb,
		first_data_packet_received_cb,
};

/* Uses the key counter to generate a new Nonce */
void wpa_port_generate_nonce(wpa_port_t *t, wpa_nonce_t *nonce)
{
    memcpy(nonce, &t->key_counter, sizeof(wpa_nonce_t));
    wpa_key_utils_inc((u8 *)&t->key_counter, sizeof(t->key_counter));
}
 
wpa_port_t *wpa_port_open(wpa_port_params_t *params,int profile_index)
{
    wpa_port_t *t;
 /*   int info_elem_len = 0,wpa_rsn_elem_len = 0;
	wpa_info_elem_t  *ap_info_elem,*info_elem,*rsn_elem; 
	
*/	
    /* Create new wpa_port_t object */
    t = (wpa_port_t *)zalloc_l(sizeof(wpa_port_t));
    if (!t)
		return NULL;
	

    /* Save parameters */
    if (dup_params(&t->params, params))
		goto Error;
	
	/* save profile*/
	t->port_profile_index = profile_index;

    /* Initialize key counter */
    wpa_key_utils_init_key_counter(&t->key_counter, t->params.mac);
    
    switch (params->psk_param)
    {
		/* If PSK is provided as ASCII, convert it to hex */
    case WPA_PSK_PARAM_ASCII:
		wpa_key_utils_calc_psk(&t->psk, t->params.psk.ascii, t->params.ssid, 
			t->params.ssid_len);
		break;
		
		/* If PSK is provided as hex, simply copy it */	
    case WPA_PSK_PARAM_HEX:
		memcpy(&t->psk, &t->params.psk.hex, sizeof(wpa_psk_t));
		break;
		
    default:
		break;
    }
	
    /* Open AP device */
    t->ap_dev = t->params.ap_dev_ops->open(t->params.dev_id, &ap_dev_cbs, 
		(void *)t);
    if (!t->ap_dev)
		goto Error;
	
    t->current_gtk_index = 2;
    
    /* If only WEP-legacy stations are allowed, we can install preconfigured 
	* keys right now, if other types of stations are allowed we'll install the
	* preconfigured keys only when the first legacy station joins
	*/
    if (t->params.allowed_sta_types & WPA_STA_TYPE_WEP_LEGACY)
	{
	    /* Set the default key ID */
		t->params.ap_dev_ops->set_default_key_id(t->port_profile_index, t->params.wep_key_index);
		install_preconfigured_keys(t, 0);
	}
	
	
    t->multicast_cipher = select_multicast_cipher(params);
	
    DEBUG_PRINT_m(SEV_INFO, MOD_WPA_INIT,"Going to use %s as multicast cipher", 
		wpa_log_cipher_to_str(t->multicast_cipher));
	
		/* If WPA stations are supported, we're going to need an information element
	*/
    /* If WPA stations are supported, we're going to need an information element
     */

	t->mix_mode_flag = 0;
    if (params->allowed_sta_types & WPA_STA_TYPE_WPA)
    {
		DEBUG_PRINT_m(SEV_INFO, MOD_WPA_INIT,"init WPA info element\n");
		/* Build information element */
		t->ap_wpa_info_elem = wpa_info_elem_open(select_multicast_cipher(params), 
			WPA_CIPHER_TKIP, params->psk_param == WPA_PSK_PARAM_NONE ?
			WPA_AUTH_8021X : WPA_AUTH_PSK);

		if (!t->ap_wpa_info_elem)
		{
			DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_AUTH,"\nwpa_port_open:error creating ap wpa info element.");
			goto Error;
		}
    }

	if (params->allowed_sta_types & WPA_STA_TYPE_WPA2)
    {
		DEBUG_PRINT_m(SEV_INFO, MOD_WPA_INIT,"init RSN info element , unicast cipher  = %d\n",params->rekeying_wep_cipher);

		/* Build information element */
		t->ap_rsn_info_elem = wpa_rsn_elem_open(select_multicast_cipher(params), 
			WPA_CIPHER_CCMP, params->psk_param == WPA_PSK_PARAM_NONE ?
			WPA_AUTH_8021X : WPA_AUTH_PSK,params->pre_auth_on);

		if (!t->ap_rsn_info_elem)
		{
			DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_AUTH,"wpa_port_open:error creating RSN info element.\n");
			goto Error;
		}
		if(params->allowed_sta_types & WPA_STA_TYPE_WPA)
			t->mix_mode_flag = 1;
    }

	DEBUG_PRINT_m(SEV_INFO, MOD_WPA_INIT,"init pmk caching support\n");
	if ((params->allowed_sta_types & WPA_STA_TYPE_WPA2) || (params->allowed_sta_types & WPA_STA_TYPE_WPA))
		wpa_pmk_cache_init(t,params->mac,profile_index);			
	
    /* Initialize a group key and install it */
	if ((params->allowed_sta_types & WPA_STA_TYPE_WPA2) || (params->allowed_sta_types & WPA_STA_TYPE_WPA) || 
        (params->allowed_sta_types & WPA_STA_TYPE_WEP_8021X) )
	{
		DEBUG_PRINT_m(SEV_INFO, MOD_WPA_INIT,"update gtk !!!\n");
		update_gtk(t);		
	}
	
    return t;
	
Error:
    if (t->ap_wpa_info_elem)
		wpa_info_elem_close(t->ap_wpa_info_elem);
	
    if (t->ap_rsn_info_elem)
		wpa_info_elem_close(t->ap_rsn_info_elem);

    if (t->ap_dev)
		t->params.ap_dev_ops->close(t->ap_dev);
	
    free_params(t);
	
    free(t);
    return NULL;
}

/* Called when a station requests a GTK update */
void wpa_port_gtk_update_request(wpa_port_t *t, wpa_assoc_t *assoc)
{
    update_gtk(t);
}

/* Called when a MIC failure occurs */
void wpa_port_mic_failure(wpa_port_t *t, wpa_assoc_t *assoc)
{
    struct timeval current_time;
    wpa_assoc_t *next_assoc;
    
    sys_get_time(&current_time);
	
    /* If this is the second MIC failure within the last 60 seconds, disconnect 
	* all stations and invoke countermeasures
	*/
    if (tv2ms(&t->last_mic_failure_time) && ((tv2ms(&current_time) - 
		tv2ms(&t->last_mic_failure_time)) < MIC_FAILURE_DELAY_MS))
    {
		DEBUG_PRINT_m(SEV_WARNING, MOD_WPA_AUTH, "wpa: Second MIC failure within 60 seconds. "
			"Disassociating all WPA stations and invoking countermeasures");	
		
		/* Disassociate all WPA stations */
		for (assoc = t->assoc_list; assoc; assoc = next_assoc)
		{
		/* It's important to save assoc->next now, because when disassociate
		* the station, assoc is freed 
			*/	   
			next_assoc = assoc->next;
			
			/* its WiFi requirement to send deauthentication frame in case of consecutive MIC failures */
			if (assoc->sta_type == WPA_STA_TYPE_WPA || assoc->sta_type == WPA_STA_TYPE_WPA2)
				deauth_sta(t, assoc);
		}
		
		/* Invoke countermeasures */
		t->countermeasures_invoked = 1;
		
		/* The countermeasures are only invoked for a limited period of time, so
		* set a timer that will stop the countermeasures when it's due
		*/
		event_timer_set(MIC_COUNTERMEASURES_PERIOD_MS, countermeasures_timer_cb,
			t);
    }
	
    t->last_mic_failure_time = current_time;
}

/* Called whenever we manage to detect whether a specific station is 802.1X or
* WEP-legacy
*/
void wpa_port_sta_type_detected(wpa_port_t *t, wpa_assoc_t *assoc)
{
/* Now that we know the station type, we might have to disassociate it if 
* it's not allowed according to the configuration
	*/
    if (!(assoc->sta_type & t->params.allowed_sta_types))
    {
		DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Deauthenticating station %s because %s stations are not "
			"allowed", mactoa(&assoc->sta_mac), 
			wpa_log_sta_type_to_str(assoc->sta_type));
		
		goto Reject;
    } 
	
    /* Deauthenticate station if it's of a type that uses mapping keys and no
	* more mapping keys can be installed
	*/
    if ((assoc->sta_type != WPA_STA_TYPE_WEP_LEGACY) && (t->num_of_wpa_stas + 
		t->num_of_8021x_stas > 
		t->params.ap_dev_ops->get_max_num_of_mapping_keys(t->ap_dev)))
    {
		DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Deauthenticating station %s because all available mapping" 
			"keys are in use", mactoa(&assoc->sta_mac));
		
		goto Reject;
    }
    
    /* Let the association object do type-specific handling */
    wpa_assoc_sta_type_detected(assoc);
	
    if (assoc->sta_type != WPA_STA_TYPE_WEP_LEGACY)
		return;
	
		/* If there were no legacy stations associated before this one joined, 
		* then we're probably using a dynamically generated GTK, we have to 
		* start using the preconfigured key as our GTK because the legacy 
		* station doesn't support key distribution. 
		* If the only allowed station type is WEP-legacy there is no need to do
		* anything, we've already installed the preconfigured keys when the 
		* port was opened.
	*/
    if (t->params.allowed_sta_types == WPA_STA_TYPE_WEP_LEGACY ||
		t->num_of_legacy_stas > 1)
    {
		return;
    }
	
    install_preconfigured_keys(t, 1);
    
    DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Distributing preconfigured key");
	
    /* Start using the key preconfigured to be the TX key as our GTK and send it
	* to the connected stations
	*/
    update_gtk(t);
	
    return;
	
Reject:
    deauth_sta(t, assoc);
}

int wpa_port_reconf(wpa_port_t *t, wpa_port_params_t *params)
{
    int gtk_update_needed = 0;
	
    /* Changing most of the parameters requires reopnening the port, check if
	* any of those parameters has changed compared to our current configuration
	*/
    if ((t->params.psk_param != params->psk_param) ||
		((t->params.psk_param == WPA_PSK_PARAM_ASCII) && 
		strcmp(t->params.psk.ascii, params->psk.ascii)) ||
		((t->params.psk_param == WPA_PSK_PARAM_HEX) && 
		memcmp(&t->params.psk.hex, &params->psk.hex, 
		sizeof(t->params.psk.hex))))
    {
		goto ReopenNeeded;
    }
	
    if (t->params.rekeying_wep_cipher != params->rekeying_wep_cipher)
		goto ReopenNeeded;
	
    if (t->params.allowed_sta_types != params->allowed_sta_types)
		goto ReopenNeeded;
	
    if ((t->params.ssid_len != params->ssid_len) ||
		memcmp(t->params.ssid, params->ssid, t->params.ssid_len))
    {
		goto ReopenNeeded;
    }
	
    if (t->params.dev_id != params->dev_id)
		goto ReopenNeeded;
	
    if (memcmp(t->params.ap_dev_ops, params->ap_dev_ops, 
		sizeof(*t->params.ap_dev_ops)))
    {
		goto ReopenNeeded;
    }
	
    if (memcmp(t->params.mac, params->mac, sizeof(t->params.mac)))
		goto ReopenNeeded;
	
    /* Handle change in configuration of group key update interval */
    if (params->gtk_update_interval != t->params.gtk_update_interval)
    {
		DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Reconfiguring group key update interval");
		
		/* Save parameter */
		t->params.gtk_update_interval = params->gtk_update_interval;
		
		/* Cancel next scheduled GTK update */
		event_timer_del(gtk_update_timer_cb, t);
		
		/* Schedule a new GTK update according to the new configuration */
		if (t->params.gtk_update_interval)
			schedule_next_gtk_update(t, t->params.gtk_update_interval);
    }
	
    /* Handle change in WEP keys */
    if (memcmp(params->wep_keys, t->params.wep_keys, sizeof(params->wep_keys)))
    {
		DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Reconfiguring WEP keys");
		
		/* Save parameter */
		memcpy(t->params.wep_keys, params->wep_keys, 
			sizeof(t->params.wep_keys));
		
			/* This change has an immediate effect only if there are legacy stations
			* connected or if we're in legacy-only mode, otherwise the
			* preconfigured keys are currently not in use anyway
		*/
		if (t->num_of_legacy_stas || t->params.allowed_sta_types == 
			WPA_STA_TYPE_WEP_LEGACY)
		{
			install_preconfigured_keys(t, 1);
			
			gtk_update_needed = 1;
		}
    }
	
    /* Handle change in configuration of WEP key index */
    if (params->wep_key_index != t->params.wep_key_index)
    {
		DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Reconfiguring WEP key index");
		
		/* Save parameter */
		t->params.wep_key_index = params->wep_key_index;
		
		/* This change has an immediate effect only if we currently have legacy
		* stations connected or if we're in legacy-only mode. These are the
		* only cases in which we use the WEP key index.
		* When legacy stations are connected, the group key index is the
		* preconfigured key index, before we start using the new key index
		* we have to let all the stations know about this change
		*/
		gtk_update_needed = t->num_of_legacy_stas ||
			t->params.allowed_sta_types == WPA_STA_TYPE_WEP_LEGACY;
    }
	
    /* If one of the changes may have affected our group key, we need to notify
	* WPA and WEP-802.1X stations about the new group key
	*/
    if (gtk_update_needed)
    {
	/* One of the configuration changes might have changed the multicast
	* cipher we're supposed to use. Since we've already advertised our
	* existing multicast cipher in our information element and we have no
	* way of telling the connected station of such a change, this
	* reconfiguration requires reopening the port
		*/
		if ((t->params.allowed_sta_types & WPA_STA_TYPE_WPA) &&
			select_multicast_cipher(params) != t->multicast_cipher)
		{
			goto ReopenNeeded;
		}
		
		update_gtk(t);
    }
	
    return 0;
	
ReopenNeeded:
    DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"Reconfiguration requires reopening port");
    return 1;
}



/**************  pre authentication *******************/

/* receive pre authentication request and handle it */
void receivePreAuthRequest(wpa_port_t *port,void *eapolPacket)
{
    wpa_assoc_t **assoc_cur = &port->assoc_list,*assoc;
	ether_mac_t sta_mac;
	char        *tmp_mac = eapolPacket;

	/* Liorm bug fix */
	memcpy(sta_mac,&tmp_mac[6],6);
    /* Look for an association with the specified MAC */
    while ((*assoc_cur) && memcmp(sta_mac, (*assoc_cur)->sta_mac, sizeof(ether_mac_t)))
		assoc_cur = &((*assoc_cur)->next);

    /* we didnt find such association ,so we create new one*/
    if (!(*assoc_cur))
	{
		assoc = (wpa_assoc_t *)zalloc_l(sizeof(wpa_assoc_t));
		
		if (!assoc)
		{
			DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_AUTH,"receivePreAuthRequest:error alloc assoc obj\n.");
			return;
		}

	
		/* Fill new association context */
		assoc->port = port;
		memcpy(&assoc->sta_mac, &sta_mac , sizeof(ether_mac_t));
		
		/* this is not a new connection with
		pmk cache as a pmk source ,so we set from_pmk_cach to 0 */
		assoc->pmk_from_cache = 0;

		/* only wpa2 stations support pre-authentication*/
		wpa_assoc_set_sta_type(assoc, WPA_STA_TYPE_WPA2);
		assoc->ptk.temporal_key.dataSize = 16;

		/* open pre-authentication,  eth type = 0x88c7 (unic type for pre-authentication packets)*/
		assoc->auth1x_conn = openAuthxConn(assoc);
	
		if (!assoc->auth1x_conn)
		{
			DEBUG_PRINT_m(SEV_ERROR, MOD_WPA_AUTH,"receivePreAuthRequest: error create auth connection\n");
			return;
		}
		
		/* this flag will tell the AP to stop after authentication
		with server over ,without starting 4 way handshake*/
		assoc->pre_auth_conn = 1;
		
		/* finnally ,start authentication.*/
		auth1x_conn_auth_start(assoc->auth1x_conn);

		/* Add association to list */
		assoc->next = port->assoc_list;
		port->assoc_list = assoc;
	}
	else
		DEBUG_PRINT_m(SEV_INFO, MOD_WPA_AUTH,"found assoc for pre auth ,continue connection.\n");
		
}



/* is station exist */
BOOL wpa_port_is_station_exist(wpa_port_t *t, ether_mac_t mac)
{
	if(find_assoc(t,mac) != NULL)
		return TRUE;
	return FALSE;
}

/* Print port statistics */
void wpa_port_print_statistics(wpa_port_t *port)
{
	struct timeval current_time;

    if(port == NULL)
    {
        printf("The profile that was selected isn't a WPA profile - No statistics are available.\n");
        return;
    }

	printf("\n");
	printf("Number of legacy stations = %d\n", port->num_of_legacy_stas);
	printf("Number of 802.1X stations = %d\n", port->num_of_8021x_stas);
	printf("Number of WPA stations    = %d\n", port->num_of_wpa_stas);
    if (tv2ms(&port->last_mic_failure_time)) 
	{
		sys_get_time(&current_time);
		printf("Last MIC failure occured %f sec ago\n", tv2ms(&current_time) - tv2ms(&port->last_mic_failure_time));
	}
	
	printf("Counter measures invoked   : %s\n", (port->countermeasures_invoked ? "True" : "False"));	
}

