/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/****************************************************************************/
/*																			*/
/*		MODULE:		drvDbHash.c												*/
/*		PURPOSE:	Hash Implementation And station database. This file 	*/
/*					holds all the information regarding the stations		*/
/****************************************************************************/
#include "pform_types.h"
#include "wlan_pal.h"
#include "QoS/QoSClassifier.h"
#include "drvDbHash.h"
#include "wds.h"
#include "mgmtIf.h"

#define WDS_RESERVED_STATIONS_m(type) (((type) == DRV_DB_HASH_STA_TYPE_STATION) && \
                                        (wdrv_dbGetWdsEnable()) ? WDS_LINKS_NUMBER : 0)


/*-----------------------------------------------------------*/
/*         Local Variables                                   */
/*-----------------------------------------------------------*/
static stationParams_T stationParams[MAX_STATIONS];

/* The staTable holds the station. Each entry describes a station. The table is not sorted */
static staHashEntry_T	staTable[MAX_STATIONS];

/* The hash table is used to find fast a station index in the database. The station index is known by all 
	the modules of the software and its easier to use it than the station mac address 
	The hash table has less entries than the station table, but every entry is a linked list.
	Station that are equal in the last 6 bits of their mac addresses are mapped to the same hash entry. */
static staHashEntry_T	*hashTable[HASH_ENTRIES_NUM];


static UINT16	numStations;

static UINT16	numStationsNotPBCCCapabale;		/* implemented only if working in PBCC mode and the PBCC algorithm is static.*/
static BOOL		currentPBCCSupportMode;

static UINT16	numStationsNotShortPreambleCapabale;
static BOOL		currentShortPreambleSupport;

static UINT16	numStationsNotSlotTimeCapabale;
static BOOL		currentShortSlotTimeSupport;

static UINT16	numOfNonERPStations=0;
static UINT16   numOfERPstations=0;
static BOOL		currentERPProtectionMode =FALSE;
static BOOL     currentERPNonPresent=FALSE;

static UINT16 BroadcastAcSeqNum16=0;
static UINT32 BroadcastAcSeqNum32=0;

/*-----------------------------------------------------------*/
/*         Local Functions                                   */
/*-----------------------------------------------------------*/
static STATUS	resetParams(void); 
static void setStationAPRates(UINT16 index, dot11_supRates_T *rates, dot11_extSupRates_T *extRates,
                              UINT8 *apAllRates, UINT8 apAllRatesLen);

/*************************************************************************
 *                        wdrv_dbHashInit                                    *
 *************************************************************************
DESCRIPTION: This function initializes the hash module.                 
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      OK : Initialization succesfull.
             NOK: Initialization unsuccesfull.                                                                                                                                             
************************************************************************/
STATUS wdrv_dbHashInit(void)
{
	int rc;

	rc = resetParams();

	if (rc != OK)
		return NOK;

	return OK;
}

/*************************************************************************
 *                        resetParams                                    *
 *************************************************************************
DESCRIPTION: This function reset the hash module.                 
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      OK : reset succesfull.
             NOK: reset unsuccesfull.                                                                                                                                             
************************************************************************/
static STATUS resetParams(void)
{
	int i;
	UINT8 PbccAlgorithmMode;
	UINT8 ERPProtectionMode;
	UINT8 PreambleMode;
	UINT8 SlotTimeMode;

	/* reset station table */
	for (i = 0; i < MAX_STATIONS; i++)
	{
		PAL_memSet(staTable[i].macAddr, 0, MAC_ADDR_LEN);
		PAL_memSet(staTable[i].ssid, 0, WHAL_SSID_STRING_LEN);
		staTable[i].nextSta = NULL;
		staTable[i].prevSta = NULL;
		staTable[i].free = TRUE;
		staTable[i].index = i;

		/* reset the station parameters table */
		PAL_memSet(&stationParams[i], 0, sizeof(stationParams_T));
	}

	/* reset hash table */
	for (i = 0; i < HASH_ENTRIES_NUM; i++)
		hashTable[i] = NULL;

	numStations = 0;
	
	/* Init PBCC */
	numStationsNotPBCCCapabale = 0;
	wdrv_dbGetPBCCAlgorithmMode(0,&PbccAlgorithmMode);
	if (PbccAlgorithmMode != PBCC_ALGO_DISABLED)
	{
		currentPBCCSupportMode = TRUE;
    } else
	{
		currentPBCCSupportMode = FALSE;
	}

	/* Init Short Preamble */
	numStationsNotShortPreambleCapabale = 0;
	wdrv_dbGetPreambleMode(0,&PreambleMode);
	if (PreambleMode == PREAMBLE_MODE_LONG)
	{
		currentShortPreambleSupport = FALSE;
    } else
	{
		currentShortPreambleSupport = TRUE;
	}

	/* Init Short Slot Time */
	numStationsNotSlotTimeCapabale = 0;
	wdrv_dbGetSlotTimeMode(0,&SlotTimeMode);
	if (SlotTimeMode == SLOTTIME_MODE_DISABLED)
	{
		currentShortSlotTimeSupport = FALSE;
    } else
	{
		currentShortSlotTimeSupport = TRUE;
	}

	/* Init ERP Protection */
	numOfNonERPStations = 0;
	numOfERPstations = 0;
	wdrv_dbGetERPProtectionMode(0,&ERPProtectionMode);
	if (ERPProtectionMode == PROTECTION_MODE_ENABLED)
	{
		currentERPProtectionMode = TRUE;
    } else
	{
		currentERPProtectionMode = FALSE;
	}

	return OK;
}

/*************************************************************************
 *                        wdrv_dbGetStationsNumForERP                    *
 *************************************************************************
DESCRIPTION: This function returns number of nonERP and ERP stations
			 associated with AP.				  
                                                                                                   
INPUT:       None

OUTPUT:      nonERPStaNum, ERPStaNum

RETURN:      None                                                                                                                                             
************************************************************************/
void wdrv_dbGetStationsNumForERP(UINT16 *nonERPStaNum, UINT16 *ERPStaNum)
{
	*nonERPStaNum = numOfNonERPStations;
	*ERPStaNum    = numOfERPstations;
}

/*************************************************************************
 *                        wdrv_dbFindStaIndex                                    *
 *************************************************************************
DESCRIPTION: This function finds a station index in the hash table.                 
                                                                                                   
INPUT:       The station mac address

OUTPUT:      The station index

RETURN:      OK : station found.
             NOK: station not found.                                                                                                                                             
************************************************************************/
STATUS wdrv_dbFindStaIndex(char *mac, UINT16 *idx)
{
	staHashEntry_T	*hashPtr;
	UINT8	hashValue = HASH_FUNC(mac);
	unsigned int flags;


	hashPtr = hashTable[hashValue];	/* get the station hash value base on the mac */
	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags);  /* START OF CRITICAL SECTION */	

	while (hashPtr != NULL)
	{
        if (MACCMP( hashPtr->macAddr, mac))     /* compare the mac addresses */
		{
			*idx = hashPtr->index;
		   	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
            if (REPORT_LEVEL(RL_DB))
			{
				wdrv_utilsReport(RL_DB, PRI_INFORMATION, "WDRV_DB: Station found. Index = %d", *idx);
				PRINT_MAC(RL_DB, PRI_INFORMATION, mac);
			}
			return OK;
		}
		hashPtr = hashPtr->nextSta;
	}	
	
	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);   /* END OF CRITICAL SECTION */

    if (REPORT_LEVEL(RL_DB))
	{
		wdrv_utilsReport(RL_DB, PRI_INFORMATION, "WDRV_DB: Station not found.");
		PRINT_MAC(RL_DB, PRI_INFORMATION, mac);
	}
	return NOK;
}


/*************************************************************************
 *                        wdrv_dbIsTableIndexExists                      *
 *************************************************************************
DESCRIPTION: This function checks whether a station index is found in
             the hash table.                 
                                                                                                   
INPUT:       A station index

OUTPUT:      None.

RETURN:      TRUE : station free.
             FALSE: station not free (not found).
************************************************************************/
BOOL wdrv_dbIsTableIndexExists(UINT16 index)
{
    if (index >= MAX_STATIONS)
        return FALSE;
    return(!(staTable[index].free));
}


/*************************************************************************
 *                        wdrv_dbFindAndInsertStaIndex                   *
 *************************************************************************
DESCRIPTION: This function finds a station index in the hash table. If the station not found, 
				the function inserts the station into the hase table & station table.                  
                                                                                                   
INPUT:       The station mac address and type - station or WDS link

OUTPUT:      The station index

RETURN:      OK : station found or station inserted successfully.
             NOK: station not found and insertion failed.                                                                                                                                             
************************************************************************/
STATUS wdrv_dbFindAndInsertStaIndex(char *mac, UINT8 woneIndex, UINT16 *idx, drvDbHashStaType_e type)
{
	int i;
	staHashEntry_T	*hashPtr;
	UINT8	hashValue = HASH_FUNC(mac); 
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags);  /* START OF CRITICAL SECTION */
	/* first search for the station */
	hashPtr = hashTable[hashValue];

	while (hashPtr != NULL)
	{
        if (MACCMP( hashPtr->macAddr, mac))
		{
			*idx = hashPtr->index;
            stationParams[*idx].WoneIdentifierIndex = woneIndex;
            stationParams[*idx].staType = type;
            PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);   /* END OF CRITICAL SECTION */

            if (REPORT_LEVEL(RL_DB))
			{
				wdrv_utilsReport(RL_DB, PRI_INFORMATION, "WDRV_DB: Station found. Index = %d", *idx);
				PRINT_MAC(RL_DB, PRI_INFORMATION, mac);
			}

			return OK;
		}
		hashPtr = hashPtr->nextSta;
	}

	/* station not found, now try to insert the station */
    if(numStations == (MAX_STATIONS - WDS_RESERVED_STATIONS_m(type))) 	/* no place for the station, table is full */
	{
	   	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);   /* END OF CRITICAL SECTION */
        if (REPORT_LEVEL(RL_DB))
		{
			wdrv_utilsReport(RL_DB, PRI_INFORMATION, "WDRV_DB: Insertion failed1, table is full,");
			PRINT_MAC(RL_DB, PRI_INFORMATION, mac);
		}

		return NOK;
	}

	numStations++;
	
	/* first find a free index */
	for (i = 0; i < MAX_STATIONS; i++)
	{
		if (staTable[i].free == TRUE)
			break;
	}

    if (i == MAX_STATIONS)
	{
	   	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);   /* END OF CRITICAL SECTION */
		wdrv_utilsReport(RL_DB, PRI_INFORMATION, "WDRV_DB: Insertion failed2, table is full, numStations = %d",numStations);
		PRINT_MAC(RL_DB, PRI_INFORMATION, mac);
		return NOK;
	}

	hashPtr = &staTable[i];
	
	/* fill the entry with the station mac */
	hashPtr->free = FALSE;
	PAL_memCopy(hashPtr->macAddr, mac, MAC_ADDR_LEN);

	/* index to be returned to the caller */
	*idx = hashPtr->index;
    /*          set wone index */
    stationParams[*idx].WoneIdentifierIndex = woneIndex;
    stationParams[*idx].staType = type;
	/* now concatenate the station entry in the linked list */
	if (hashTable[hashValue] == NULL)	/* if linked list is empty*/
	{
		hashTable[hashValue] = hashPtr;

	   	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);   /* END OF CRITICAL SECTION */
        if (REPORT_LEVEL(RL_DB))
		{
			wdrv_utilsReport(RL_DB, PRI_INFORMATION, "WDRV_DB: Insertion station. Index = %d", *idx);
			PRINT_MAC(RL_DB, PRI_INFORMATION, mac);
		}

		return OK;
	}

	hashPtr->nextSta = hashTable[hashValue];	/* if linked list is not empty, insert as first entry */
	(hashPtr->nextSta)->prevSta = hashPtr;
	hashTable[hashValue] = hashPtr;
	
   	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);   /* END OF CRITICAL SECTION */
    if (REPORT_LEVEL(RL_DB))
	{
		wdrv_utilsReport(RL_DB, PRI_INFORMATION, "WDRV_DB: Insertion succeeded. Index = %d", *idx);
		PRINT_MAC(RL_DB, PRI_INFORMATION, mac);
	}

	return OK;
}

/*************************************************************************
 *                        wdrv_dbRemoveStaIndex				             *
 *************************************************************************
DESCRIPTION: This function removes a station from the hash table & station table.       
				It also resets the station parameters.
                                                                                                   
INPUT:       The station index

OUTPUT:      

RETURN:                                                                                                                                                   
************************************************************************/
void wdrv_dbRemoveStaIndex(UINT16 idx)
{
	UINT8	hashValue; 
	staHashEntry_T	*hashPtr;
	unsigned int flags;
	
	if (idx >= MAX_STATIONS)
	{
        if (REPORT_LEVEL(RL_DB))
			wdrv_utilsReport(RL_DB, PRI_INFORMATION, "WDRV_DB: Remove station failed, index is out of range.\n");
		return;
	}

	hashPtr = &staTable[idx];
	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags);  /* START OF CRITICAL SECTION */

	if (hashPtr->free == TRUE)		/* entry is not in use, station does not exist in the tables */
	{	
	   	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);   /* END OF CRITICAL SECTION */
        if (REPORT_LEVEL(RL_DB))
			wdrv_utilsReport(RL_DB, PRI_INFORMATION, "WDRV_DB: Remove station failed, index is free already.\n");
		return;
	}

	numStations--;

	/* now try to remove the station entry from the linked list */
	if (hashPtr->prevSta == NULL)		/* first entry in the linked list */
	{
		hashValue = HASH_FUNC(hashPtr->macAddr);

		if (hashPtr->nextSta == NULL)	/* entry is also the last one */	
			hashTable[hashValue] = NULL;

		else							/* entry is not the last one */	
		{
			(hashPtr->nextSta)->prevSta = NULL;
			hashTable[hashValue] = hashPtr->nextSta;
			hashPtr->nextSta = NULL;
		}
	}
	
	else								/* not the first entry in the linked list */
	{
		(hashPtr->prevSta)->nextSta = hashPtr->nextSta;
		
		if (hashPtr->nextSta != NULL)
		{
			(hashPtr->nextSta)->prevSta = hashPtr->prevSta;
			hashPtr->nextSta = NULL;
		}
		hashPtr->prevSta = NULL;
	}

	/* now update rest of the entry structure */
	PAL_memSet(hashPtr->macAddr, 0, MAC_ADDR_LEN);
	PAL_memSet(hashPtr->ssid, 0, WHAL_SSID_STRING_LEN);
        hashPtr->ssidLength = 0;
	hashPtr->free = TRUE;
   	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);   /* END OF CRITICAL SECTION */

    if (REPORT_LEVEL(RL_DB))
		wdrv_utilsReport(RL_DB, PRI_INFORMATION, "WDRV_DB: Remove station secceeded.\n");

	return;
}
/*************************************************************************
 *					wdrv_dbGetNumStationsByState					                                  *
*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       

OUTPUT:      

RETURN:      num stations
                                                                                                                                                          
************************************************************************/
int wdrv_dbGetNumStationsByState(void)
{
		int    i,n;
		UINT32 state;
		
		for (i=n=0; i<MAX_STATIONS; i++)
		{
				wdrv_mgmtGetState(i, &state);
				if (state != STATE_IDLE)
					n++;
		}

		return n;
}

/*************************************************************************
 *										                                  *
*************************************************************************
DESCRIPTION:  called by the console
                                                                                       
INPUT:       

OUTPUT:      buf

RETURN:      kength
                                                                                                                                                          
************************************************************************/
int wdrv_dbPrintNumOfStations(char *buf)
{
	return sprintf(buf,"Num of stations:%d\n", wdrv_dbGetNumStationsByState() /*numStations*/);
}

/* MW CODE BEGIN */
extern int wdrv_mgmtPrintBanList(char *buf);
/*************************************************************************
 *					wdrv_dbPrintStaTable					                                  *
*************************************************************************
DESCRIPTION: print the stations table, called by the console
                                                                                       
INPUT:       

OUTPUT:      buf

RETURN:     length 
                                                                                                                                                          
************************************************************************/
int  wdrv_dbPrintStaTable(char *buf)
{
	UINT32 i,   state , len;
	char stateString[MAX_PRINT_STRING];
	BOOL TIcap;
    UINT8 band = 0;
    UINT8 channel = 0;
    UINT8 wone_index =0;  
	UINT16 acxActiveRate;
	unsigned int flags;
    int authorizedSTAsCounter = 0;
    int numOfAuthorizedSTAs = wdrv_mgmtGetNumOfAuthorizedSta();
    char authorizedString[MAX_PRINT_STRING] = "  AUTHORIZED ";
	char tmp[128];

	len = 0;
#ifndef MWCODE
	len += sprintf(buf + len ,"Stations table\n");
	len += sprintf(buf + len ,"Number of stations not in the IDLE state:%d\n", wdrv_dbGetNumStationsByState() /*numStations*/);
        len += sprintf(buf + len ,"Total number of stations:%d\n", numStations);
        len += sprintf(buf + len ,"Number of Authorized Stations:%d\n", numOfAuthorizedSTAs);

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags);

	numOfAuthorizedSTAs = wdrv_mgmtGetNumOfAuthorizedSta();

	len += sprintf(buf + len ,"Number of Authorized Stations:%d\n", numOfAuthorizedSTAs);
	len += sprintf(buf + len ,"Ind        Mac          State       AcRate RateAcx PBCC Priv TIoui WME Chan Band Type\n");
	
	for (i = 0; i < MAX_STATIONS; i++)
	{
		wdrv_mgmtGetState(i, &state);
		if (state != STATE_IDLE)
		{
			len += sprintf(buf + len ,"%02d ", i);
			
			len += sprintf(buf + len ,"%02X-%02X-%02X-%02X-%02X-%02X ",
							(UINT8)staTable[i].macAddr[0], (UINT8)staTable[i].macAddr[1],
							(UINT8)staTable[i].macAddr[2], (UINT8)staTable[i].macAddr[3],
							(UINT8)staTable[i].macAddr[4], (UINT8)staTable[i].macAddr[5]);

			memset(stateString, 0, MAX_PRINT_STRING);
			wdrv_mgmtGetStateString(i, stateString);
			len += sprintf(buf + len ,"%14s ", stateString);
            if(!strcmp(authorizedString, stateString))
            {
                authorizedSTAsCounter++;
            }

			acxActiveRate = tx_getStaActiveRate(i);			
			len += sprintf(buf + len ,"%#6x ", convertAcxRateToDOT11Rate(acxActiveRate));			
			len += sprintf(buf + len ,"%#7x ", acxActiveRate);

			if ((stationParams[i].capabilities >> CAP_PBCC_SHIFT) & CAP_PBCC_MASK)
				len += sprintf(buf + len ,"%4d ", 1);
			else
				len += sprintf(buf + len ,"%4d ", 0);
			PAL_memSet(stateString, 0, MAX_PRINT_STRING);

			if ((stationParams[i].capabilities >> CAP_PRIVACY_SHIFT) & CAP_PRIVACY_MASK)
				len += sprintf(buf + len ,"%4d ", 1);
			else
				len += sprintf(buf + len ,"%4d ", 0);

			wdrv_dbGetStationTIcapEnable(i, &TIcap);
			len += sprintf(buf + len ,"%5d ", TIcap);

			len += sprintf(buf + len, "%3d ", stationParams[i].wme);
			
            wdrv_dbGetStationWoneIndex(i, &wone_index);
	        if (wone_index ==  0xff)
            {
                 len += sprintf(buf + len ,"ERROR   ERROR\n\n");
            } else
            {
                wdrv_dbGetChannel(wone_index,&channel);
                len += sprintf(buf + len ,"%4d ", channel);

                wdrv_dbGetBand(wone_index,&band);

                if (band == RADIO_BAND_5GHZ)
                    len += sprintf(buf + len ,"A");
                else
                    len += sprintf(buf + len ,"G");  
            }    
            len += sprintf(buf + len ,"   %s\n\n", (stationParams[i].staType == DRV_DB_HASH_STA_TYPE_STATION) ?
                           "STA" : "WDS");  
		}

	}
#else
	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags);

	if(wdrv_mgmtPrintBanList(tmp)==0) {
                len += sprintf(buf + len , "%s\n", tmp);
        } else {
                for (i = 0; i < MAX_STATIONS; i++) {
                        wdrv_mgmtGetState(i, &state);
                        if (state != STATE_IDLE) {
				len += sprintf(buf + len ,"%02X:%02X:%02X:%02X:%02X:%02X\n",
                                                        (UINT8)staTable[i].macAddr[0], (UINT8)staTable[i].macAddr[1],
                                                        (UINT8)staTable[i].macAddr[2], (UINT8)staTable[i].macAddr[3],
                                                        (UINT8)staTable[i].macAddr[4], (UINT8)staTable[i].macAddr[5]);
			        memset(stateString, 0, MAX_PRINT_STRING);
			        wdrv_mgmtGetStateString(i, stateString);
			        if(strcmp(authorizedString, stateString) == 0) {
				        authorizedSTAsCounter ++;
			        }
			}
                }
        }
#endif
    if(authorizedSTAsCounter != numOfAuthorizedSTAs)
    {
        len += sprintf(buf + len ,"\n----------------------------------------------------\n*** ERROR: Note that the number of authorized STAs that is reported is    ***\n*** differant than the number of Authorized STAs that appear in the table.***\n-----------------------------------------------------------------------------\n");
    }

	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);

	return len;
}

/*************************************************************************
 *					wdrv_dbPrintStaStatistics					                                  *
*************************************************************************
DESCRIPTION: print the stations statistics, called by the console
                                                                                       
INPUT:       

OUTPUT:      buf

RETURN:     length 
                                                                                                                                                          
************************************************************************/
int  wdrv_dbPrintStaStatistics(char *buf)
{
	UINT32 i,   state , len;
	char stateString[MAX_PRINT_STRING];
    UINT8 channel = 0;
    UINT8 wone_index = 0;
    UINT32  txCounter = 0;
    UINT32  rxCounter = 0;
	unsigned int flags;

	len = 0;
	len += sprintf(buf + len ,"Stations Statistics\n");

	len += sprintf(buf + len ,"Ind        Mac          State       Chan       Tx       Rx \n");
		
	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags);

	for (i = 0; i < MAX_STATIONS; i++)
	{
		wdrv_mgmtGetState(i, &state);
		if (state != STATE_IDLE)
		{
            /* Index */
			len += sprintf(buf + len ,"%02d ", i);
			
            /* MAC */
			len += sprintf(buf + len ,"%02X-%02X-%02X-%02X-%02X-%02X ",
							(UINT8)staTable[i].macAddr[0], (UINT8)staTable[i].macAddr[1],
							(UINT8)staTable[i].macAddr[2], (UINT8)staTable[i].macAddr[3],
							(UINT8)staTable[i].macAddr[4], (UINT8)staTable[i].macAddr[5]);

            /* State */
			memset(stateString, 0, MAX_PRINT_STRING);
			wdrv_mgmtGetStateString(i, stateString);
			len += sprintf(buf + len ,"%14s ", stateString);

			/* Channel */
			wdrv_dbGetStationWoneIndex(i, &wone_index);
	        if (wone_index ==  0xff)
            {
                 len += sprintf(buf + len ,"ERROR   ERROR\n\n");
            } 
            else
            {
                wdrv_dbGetChannel(wone_index,&channel);
                len += sprintf(buf + len ,"%4d ", channel);
            }

            /* Tx Counter */
            wdrv_dbGetStationTxPacketCounter(i, &txCounter);
            len += sprintf(buf + len ,"%8d ", txCounter);

            /* Rx Counter */
            wdrv_dbGetStationRxPacketCounter(i, &rxCounter);
            len += sprintf(buf + len ,"%8d \n", rxCounter);
		}

	}

	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags); 

	return len;
}

/*************************************************************************
 *			wdrv_dbPrintRatesTable							                                  *
*************************************************************************
DESCRIPTION: print the stations rates table
                                                                                       
INPUT:       

OUTPUT:     buf 

RETURN:    length  
                                                                                                                                                          
************************************************************************/
int  wdrv_dbPrintRatesTable(char *buf)
{
	int i, j, state, empty=TRUE;
	UINT16 rateWithFallback;
	int len = 0;
	
	len += sprintf(buf + len ,"\nStations Rates table\n");

	for (i = 0; i < MAX_STATIONS; i++)
	{
		wdrv_mgmtGetState(i, &state);
		if ((state == STATE_ASSOCIATED) || (state == STATE_AUTHORIZED))
		{
			empty = FALSE;

			len += sprintf(buf + len ,"Index		Mac		PBCC	ActiveRate    currentRateAcxWithFallback\n");

			len += sprintf(buf + len ," [%02d]   ", i);
			
			len += sprintf(buf + len ,"%02X-%02X-%02X-%02X-%02X-%02X  ", (UINT8)staTable[i].macAddr[0], (UINT8)staTable[i].macAddr[1], (UINT8)staTable[i].macAddr[2], (UINT8)staTable[i].macAddr[3], (UINT8)staTable[i].macAddr[4], (UINT8)staTable[i].macAddr[5]);

			if ((stationParams[i].capabilities >> CAP_PBCC_SHIFT) & CAP_PBCC_MASK)
				len += sprintf(buf + len ,"	%d	  ", 1);
			else
				len += sprintf(buf + len ,"	%d	  ", 0);

			len += wdrv_printRateInMbpsUnit(stationParams[i].activeRate,buf);

			wdrv_dbGetCurrentActiveRateWithFallback(i, &rateWithFallback);
			len += sprintf(buf + len ,"                    0x%x", rateWithFallback);
			len += sprintf(buf + len ,"\n"); 

			len += sprintf(buf + len ,"Rates =	"); 
			for (j = 0; j < stationParams[i].ratesLen; j++)
			{
				len += sprintf(buf + len ,"  "); 
				len += wdrv_printRateInMbpsUnit(stationParams[i].rates[j],buf);
			}
			for (j = 0; j < stationParams[i].extRatesLen; j++)
			{
				len += sprintf(buf + len ,"  "); 
				len += wdrv_printRateInMbpsUnit(stationParams[i].extRates[j],buf);
			}
			len += sprintf(buf + len ,"\n"); 

			/* DEBUG */
			len += sprintf(buf + len ,"APRates =	"); 
			for (j = 0; j < stationParams[i].apRatesLen; j++)
			{
				len += sprintf(buf + len ,"  "); 
				len +=wdrv_printRateInMbpsUnit(stationParams[i].apRates[j],buf+len);
			}
			len += sprintf(buf + len ,"\n"); 
			len += sprintf(buf + len ,"APExtRates =	"); 
			for (j = 0; j < stationParams[i].apExtRatesLen; j++)
			{
				len += sprintf(buf + len ,"  "); 
				len +=wdrv_printRateInMbpsUnit(stationParams[i].apExtRates[j],buf+len);
			}
			len += sprintf(buf + len ,"\n"); 
			/* DEBUG */
		}
	}

	if (empty)
	{
		len += sprintf(buf + len ,"The table is empty (no station is associated)\n");
	}
	return len;
}

/*************************************************************************
 *		wdrv_dbPrintHashTable								                                  *
*************************************************************************
DESCRIPTION: print the hash table, called by the console
                                                                                       
INPUT:       

OUTPUT:      buf

RETURN:      length
                                                                                                                                                          
************************************************************************/
int  wdrv_dbPrintHashTable(char *buf)
{
	int i;
	int len = 0;
	
	len += sprintf(buf + len ,"Hash table\n");

	for (i = 0; i < HASH_ENTRIES_NUM; i++)
	{
		
		len += sprintf(buf + len ,"[%d]", i);
		if (hashTable[i] != NULL)
			len += sprintf(buf + len ,"Entry Address:0x%x\n\n", (unsigned int)hashTable[i]);
		else
			len += sprintf(buf + len ,"Entry Address:NULL\n\n");
	}

	return len;
}

/*************************************************************************
 *					wdrv_dbGetStaMacAddress					                                  *
*************************************************************************
DESCRIPTION: receives an index, returns the station mac address
                                                                                       
INPUT:  index     

OUTPUT:      macAddressPtr

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetStaMacAddress(UINT16 index, char *macAddressPtr)
{
	PAL_memCopy(macAddressPtr, staTable[index].macAddr, MAC_ADDR_LEN);
}

/*************************************************************************
 *		wdrv_dbPrintFourXTable								                                  *
*************************************************************************
DESCRIPTION:  print the stations table, called by the console
                                                                                       
INPUT:       

OUTPUT:   buf   

RETURN:    length  
                                                                                                                                                          
************************************************************************/
int wdrv_dbPrintFourXTable(char *buf)
{
	int len = 0;
	int i, state;
	char stateString[MAX_PRINT_STRING];

	len += sprintf(buf + len,"4X table\n");
	len += sprintf(buf + len,"Number of Stations:%d\n", wdrv_dbGetNumStationsByState() /*numStations*/);

	len += sprintf(buf + len,"Index		Mac		State	  ActiveRate	ActiveRateAcx	PBCC  fourXConcatEnable fourXActive\n");
		
	for (i = 0; i < MAX_STATIONS; i++)
	{

		wdrv_mgmtGetState(i, &state);
		if (state != STATE_IDLE)
		{
			len += sprintf(buf + len," [%02d]   ", i);
			
			len += sprintf(buf + len,"%02X-%02X-%02X-%02X-%02X-%02X  ", (UINT8)staTable[i].macAddr[0], (UINT8)staTable[i].macAddr[1], (UINT8)staTable[i].macAddr[2], (UINT8)staTable[i].macAddr[3], (UINT8)staTable[i].macAddr[4], (UINT8)staTable[i].macAddr[5]);

			PAL_memSet(stateString, 0, MAX_PRINT_STRING);
			wdrv_mgmtGetStateString(i, stateString);
			len += sprintf(buf + len,"%s ", stateString);

			len += sprintf(buf + len,"	0x%X	", stationParams[i].activeRate);
			len += sprintf(buf + len,"	0x%X	", stationParams[i].activeRateAcx);

			if ((stationParams[i].capabilities >> CAP_PBCC_SHIFT) & CAP_PBCC_MASK)
				len += sprintf(buf + len,"%d	", 1);
			else
				len += sprintf(buf + len,"%d	", 0);

			if (stationParams[i].fourX_cap.concatEn == TRUE)
				len += sprintf(buf + len,"%d	", 1);
			else
				len += sprintf(buf + len,"%d	", 0);

			if (stationParams[i].fourXActive == TRUE)
				len += sprintf(buf + len,"%d	\n", 1);
			else
				len += sprintf(buf + len,"%d	\n", 0);

		}
	}
	return len;

}

/*************************************************************************
 *				wdrv_dbSetStaSsidByIndex						                                  *
*************************************************************************
DESCRIPTION: Attach the station's SSID as part of the station's details entry
                                                                                       
INPUT:     index  ssid SsidLen

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
STATUS wdrv_dbSetStaSsidByIndex(UINT16 index, char *ssid, int SsidLen)
{
    staHashEntry_T	*hashPtr;
	unsigned int flags;
    
    if (index >= MAX_STATIONS)
    {
        if (REPORT_LEVEL(RL_DB))
            wdrv_utilsReport(RL_DB, PRI_INFORMATION, "WDRV_DB: Setting SSID failed, index is out of range.\n");
        return ERROR;
    }

    if (SsidLen > WHAL_SSID_STRING_LEN) 
    {
        if (REPORT_LEVEL(RL_DB))
            wdrv_utilsReport(RL_DB, PRI_INFORMATION, "WDRV_DB: Setting SSID failed, too long!\n");
        return ERROR;
    }

    hashPtr = &staTable[index];
    PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */

    if (hashPtr->free == TRUE)      /* entry is not in used, station does not exist in the tables */
    {
        PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
        if (REPORT_LEVEL(RL_DB))
            wdrv_utilsReport(RL_DB, PRI_INFORMATION, "WDRV_DB: Setting SSID failed, index is free..!?\n");
        return ERROR;
    }

	PAL_memSet(hashPtr->ssid, 0, WHAL_SSID_STRING_LEN);
	PAL_memCopy(hashPtr->ssid, ssid, SsidLen);
    hashPtr->ssidLength = SsidLen;
   	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */

    if (REPORT_LEVEL(RL_DB))
		wdrv_utilsReport(RL_DB, PRI_INFORMATION, "WDRV_DB: Adding SSID to station table secceeded.\n");

	return OK;
}

/*************************************************************************
 *		wdrv_dbGetStaSsidByIndex								                                  *
*************************************************************************
DESCRIPTION: Get the station's associated SSID given its index
                                                                                       
INPUT:      index 

OUTPUT:      ssid SsidLen

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetStaSsidByIndex(UINT16 index, char *ssid, int *SsidLen)
{
	PAL_memCopy(ssid, staTable[index].ssid, staTable[index].ssidLength);
    *SsidLen = staTable[index].ssidLength;
}
 
/*************************************************************************
 *						wdrv_dbGetStaProfileByIndex				                                  *
*************************************************************************
DESCRIPTION:  Get the station's associated SSID given its index 
                                                                                       
INPUT:       index

OUTPUT:      p_profile_index

RETURN:      
                                                                                                                                                          
************************************************************************/
STATUS wdrv_dbGetStaProfileByIndex(UINT16 index, int *p_profile_index)
{
    if (index >= MAX_STATIONS)
    {
        if (REPORT_LEVEL(RL_DB))
            wdrv_utilsReport(RL_DB, PRI_INFORMATION, "WDRV_DB: Setting SSID failed, index is out of range.\n");
        return NOK;
    }
    *p_profile_index = staTable[index].profile_index;

    return OK;
}

/*************************************************************************
 *						wdrv_dbSetStaProfileIndex				                                  *
*************************************************************************
DESCRIPTION: Attach the station's SSID as part of the station's details entry
                                                                                       
INPUT:     index  profile_index)

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
STATUS wdrv_dbSetStaProfileIndex(UINT16 index, int profile_index)
{
    staHashEntry_T  *hashPtr;
	unsigned int flags;

    if (index >= MAX_STATIONS)
    {
        if (REPORT_LEVEL(RL_DB))
            wdrv_utilsReport(RL_DB, PRI_INFORMATION, "WDRV_DB: Setting SSID failed, index is out of range.\n");
        return ERROR;
    }


    hashPtr = &staTable[index];
    PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */

    if (hashPtr->free == TRUE)      /* entry is not in used, station does not exist in the tables */
    {
        PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
        if (REPORT_LEVEL(RL_DB))
            wdrv_utilsReport(RL_DB, PRI_INFORMATION, "WDRV_DB: Setting SSID failed, index is free..!?\n");
        return ERROR;
    }

    if (REPORT_LEVEL(RL_DB))
        wdrv_utilsReport(RL_DB, PRI_INFORMATION, "WDRV_DB: Adding SSID to station table secceeded.\n");

    staTable[index].profile_index = profile_index;
    PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */

    return OK;
}

/*************************************************************************
 *                        wdrv_dbIsRateSupported						 *
 *************************************************************************
DESCRIPTION: This function checks if rateAcx is supported
             by station[index]
                                                                                                   
INPUT:       index - the station index
			 rateAcx - the rate to check if station supports it

OUTPUT:      

RETURN:      TRUE  - rateAcx is supported in station[index]
             FALSE - rateAcx is not supported in station[index]
************************************************************************/
BOOL wdrv_dbIsRateSupported(UINT16 index, UINT16 rateAcx)
{
	if (stationParams[index].bitWiseSupRates & rateAcx)
	{
		return TRUE;
    } else
	{
		return FALSE;
	}
}


/* */
/*************************************************************************
 *					convertDOT11RatesToAcxBitWiseFormat					                                  *
*************************************************************************
DESCRIPTION:  convert a list of rates to a bitwise ACX format
                                                                                       
INPUT:  rates  ratesLen   

OUTPUT:      

RETURN:      result
                                                                                                                                                          
************************************************************************/
UINT16 convertDOT11RatesToAcxBitWiseFormat(UINT8 *rates, UINT8 ratesLen)
{
int i;
UINT16 bitWiseRates=0;

	for (i=0; i<ratesLen; i++)
	{
		switch (rates[i]&0x7F)
		{
			case DOT11_RATE_1:
				bitWiseRates |= ACX_RATE_1;
				break;

			case DOT11_RATE_2:
				bitWiseRates |= ACX_RATE_2;
				break;

			case DOT11_RATE_5_5:
				bitWiseRates |= ACX_RATE_5_5;
				break;

			case DOT11_RATE_6:
				bitWiseRates |= ACX_RATE_6;
				break;

			case DOT11_RATE_9:
				bitWiseRates |= ACX_RATE_9;
				break;

			case DOT11_RATE_11:
				bitWiseRates |= ACX_RATE_11;
				break;

			case DOT11_RATE_12:
				bitWiseRates |= ACX_RATE_12;
				break;

			case DOT11_RATE_18:
				bitWiseRates |= ACX_RATE_18;
				break;

			case DOT11_RATE_22:
				bitWiseRates |= ACX_RATE_22;
				break;

			case DOT11_RATE_24:
				bitWiseRates |= ACX_RATE_24;
				break;

			case DOT11_RATE_33:
				/* Do nothing as 33Mbps is not supported by the AP (1130) */
				break;

			case DOT11_RATE_36:
				bitWiseRates |= ACX_RATE_36;
				break;

			case DOT11_RATE_48:
				bitWiseRates |= ACX_RATE_48;
				break;

			case DOT11_RATE_54:
				bitWiseRates |= ACX_RATE_54;
				break;

			default:
				wdrv_utilsReport(RL_DB, PRI_INFORMATION, "WDRV_DB: Warning, Invalid station active rate.\n");
		}
	}

	return bitWiseRates;
}

/*************************************************************************
 *				convertAcxRateToDOT11Rate						                                  *
*************************************************************************
DESCRIPTION: convert an ACX rate to a DOT11 rate format
                                                                                       
INPUT:       rateAcx

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
UINT8 convertAcxRateToDOT11Rate(UINT16 rateAcx)
{
UINT8 rate=0;

	switch (rateAcx & ACX_RATE_MASK)
	{
		case ACX_RATE_1:
			rate = DOT11_RATE_1;
			break;

		case ACX_RATE_2:
			rate = DOT11_RATE_2;
			break;

		case ACX_RATE_5_5:
			rate = DOT11_RATE_5_5;
			break;

		case ACX_RATE_6:
			rate = DOT11_RATE_6;
			break;

		case ACX_RATE_9:
			rate = DOT11_RATE_9;
			break;

		case ACX_RATE_11:
			rate = DOT11_RATE_11;
			break;

		case ACX_RATE_12:
			rate = DOT11_RATE_12;
			break;

		case ACX_RATE_18:
			rate = DOT11_RATE_18;
			break;

		case ACX_RATE_22:
			rate = DOT11_RATE_22;
			break;

		case ACX_RATE_24:
			rate = DOT11_RATE_24;
			break;

		case ACX_RATE_36:
			rate = DOT11_RATE_36;
			break;

		case ACX_RATE_48:
			rate = DOT11_RATE_48;
			break;

		case ACX_RATE_54:
			rate = DOT11_RATE_54;
			break;

		default:
			wdrv_utilsReport(RL_DB, PRI_INFORMATION, "WDRV_DB: Warning, Invalid station active rate.\n");
	}

	return rate;
}

/*************************************************************************
 *				 setStationAPRates						                                  *
*************************************************************************
DESCRIPTION: Set station AP rates
                                                                                       
INPUT:     index  rates extRates apAllRates apAllRatesLen

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
static void setStationAPRates(UINT16 index, dot11_supRates_T *rates, dot11_extSupRates_T *extRates,
                              UINT8 *apAllRates, UINT8 apAllRatesLen)
{
int i, j, length;

	/* rates & extRates are the Station's supported rates as */
	/* was transmitted in the (re)Association request IEs    */
	/* check each one of them if supported by the AP and set */
	/* the station's database apRates & apExtRates arrays    */
	/* Save the BasicRate bit as is supported by the AP as   */
	/* these arrays are used in the (re)association response */

	length = 0;
	for (i=0; i<rates->hdr.eleLen; i++)
	{
		for (j=0; j<apAllRatesLen; j++)
		{
			if ((rates->rates[i] & 0x7F) == (apAllRates[j] & 0x7F))
			{
				stationParams[index].apRates[length++] = apAllRates[j];
			}
		}
	}
	stationParams[index].apRatesLen = length;

	/* If extended rates doesn't exist, set length to zero and exit */
	if (extRates == NULL)
	{
		stationParams[index].apExtRatesLen = 0;
		return;
	}

	length = 0;
	for (i=0; i<extRates->hdr.eleLen; i++)
	{
		for (j=0; j<apAllRatesLen; j++)
		{
			if ((extRates->rates[i] & 0x7F) == (apAllRates[j] & 0x7F))
			{
				stationParams[index].apExtRates[length++] = apAllRates[j];
			}
		}
	}
	stationParams[index].apExtRatesLen = length;
}


/*************************************************************************
 *                        wdrv_dbSetStationRates						 *
 *************************************************************************
DESCRIPTION: This function  sets the station transmit parameters in the station params table.
                                                                                                   
INPUT:       The station index
			 The station transmit parameters

OUTPUT:      

RETURN:                                                                                                                                                   
************************************************************************/
void wdrv_dbSetStationRates(UINT16 index, dot11_supRates_T *rates, dot11_extSupRates_T *extRates, BOOL fourXSupport)
{
	int     i;
	UINT8	apAllRates[DOT11_SUPPORTED_RATES_MAX+DOT11_EXT_SUPPORTED_RATES_MAX];
	UINT8	apAllRatesLen;
	UINT16  bitWiseRates;
	UINT8	activeRate;
	UINT16	activeRateAcx = 0;
	BOOL	Pbcc;
	unsigned int flags;

	/* Find all AP's supported rates */
	wdrv_dbGetStationConfigOperationalAllRateSet(0,apAllRates, &apAllRatesLen);
	bitWiseRates = convertDOT11RatesToAcxBitWiseFormat(apAllRates, apAllRatesLen);
	
	/* Set all station's supported rates in bit wise format */
	stationParams[index].bitWiseSupRates = convertDOT11RatesToAcxBitWiseFormat( \
		rates->rates, rates->hdr.eleLen);
	
	/* Add also the extended supported rates if exist */
	if (extRates != NULL)
	{
		stationParams[index].bitWiseSupRates |= convertDOT11RatesToAcxBitWiseFormat( \
			extRates->rates, extRates->hdr.eleLen);
	}

	/* Find the station active rate whice is the highest rate supported */
	/* by both the AP and the station */
	/* The left most bit in the bit wise rates format is the highest */
	bitWiseRates &= stationParams[index].bitWiseSupRates;

	for (i=0; i<ACX_RATE_NUM; i++)
	{
		if ((bitWiseRates >> i) & 0x1)
		{
			activeRateAcx = (1 << i);
		}
	}

	activeRate = convertAcxRateToDOT11Rate(activeRateAcx);

	/* Set the station database rates that are both supported by the AP and the station */
	setStationAPRates(index, rates, extRates, apAllRates, apAllRatesLen);

	wdrv_dbGetCurrentPBCCSupportMode(&Pbcc);

	/* Enable PBCC mode if all the following meets: */
	/* 1) AP supports PBCC      */
	/* 2) Station supports PBCC */
	/* 3) 5.5, 11 or 22 Mbps only   */
	if (((stationParams[index].capabilities >> CAP_PBCC_SHIFT) & CAP_PBCC_MASK) && (Pbcc))
	{
		if ((activeRate == DOT11_RATE_5_5) || (activeRate == DOT11_RATE_11) || (activeRate == DOT11_RATE_22))
		{
			activeRateAcx |= ACX_RATE_PBCC;
		}
    } else
    {
        if (activeRate == DOT11_RATE_22)
		{
			activeRateAcx = ACX_RATE_11;
			activeRate = DOT11_RATE_11;
		}
	}

	/* We set the four X enable feature only for rates greater than 11 */
    if (activeRate >= DOT11_RATE_11)
	{
		wdrv_dbSetStationConcatSupport(index, fourXSupport);
	}

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags);  /* START OF CRITICAL SECTION */

	stationParams[index].ratesLen = rates->hdr.eleLen;
	PAL_memCopy(stationParams[index].rates, rates->rates, rates->hdr.eleLen);
	
	if (extRates != NULL)
	{
		stationParams[index].extRatesLen = extRates->hdr.eleLen;
		PAL_memCopy(stationParams[index].extRates, extRates->rates, extRates->hdr.eleLen);
    } else
	{
		stationParams[index].extRatesLen = 0;
	}

	stationParams[index].activeRate = activeRate;
	stationParams[index].activeRateAcx = activeRateAcx;

	/* updated every transmit, init to zero */
	stationParams[index].currentActiveRateWithFallback = 0;

	/* Config the rate adaptation table in the Tx module. */
	wdrv_txRateAdapSetRate( index, activeRateAcx);

   	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);   /* END OF CRITICAL SECTION */
}

/*************************************************************************
 *                        wdrv_dbSetStationRsnIe						 *
 *************************************************************************
DESCRIPTION: This function  sets the station transmit parameters in the station params table.
                                                                                                   
INPUT:       The station index
			 The station transmit parameters

OUTPUT:      

RETURN:                                                                                                                                                   
************************************************************************/
void wdrv_dbSetStationRsnIe(UINT16 index, dot11_rsne_T *rsnInfoEle)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	PAL_memSet(&stationParams[index].rsnIe, 0, sizeof(dot11_rsne_T));
	if (rsnInfoEle != NULL)
	{
		PAL_memCopy(&stationParams[index].rsnIe, rsnInfoEle, rsnInfoEle->hdr.eleLen + 2);
	}
   	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}

/*************************************************************************
 *		wdrv_dbGetStationRsnIe								                                  *
*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:     index  

OUTPUT:      

RETURN:     rsnInfoEle 
                                                                                                                                                          
************************************************************************/

void wdrv_dbGetStationRsnIe(UINT16 index, dot11_rsne_T *rsnInfoEle)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	PAL_memSet(rsnInfoEle, 0, sizeof(dot11_rsne_T));
	PAL_memCopy(rsnInfoEle, &stationParams[index].rsnIe, stationParams[index].rsnIe.hdr.eleLen + 2);
   	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}

/*************************************************************************
 *					wdrv_dbGetActiveRateAcx					                                  *
*************************************************************************
DESCRIPTION: receives the station index, returns the station active rate 
                                                                                       
INPUT:    index   

OUTPUT: activeRate     

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetActiveRateAcx(UINT16 index, UINT8 *activeRate)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	*activeRate = stationParams[index].activeRateAcx;
   	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}

/* receives the station index, returns the station active rate */
/*************************************************************************
 *										                                  *
*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetActiveRate(UINT16 index, UINT8 *activeRate)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	*activeRate = stationParams[index].activeRate;
   	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}

/*************************************************************************
 *				wdrv_dbGetStationAPRates						                                  *
*************************************************************************
DESCRIPTION: receives the station index, returns the station rates used in (re)associate response
                                                                                       
INPUT:       index

OUTPUT:      value len

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetStationAPRates(UINT16 index, char *value, UINT8 *len)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	*len = stationParams[index].apRatesLen;
	PAL_memCopy(value, stationParams[index].apRates, stationParams[index].apRatesLen);
   	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}

/*************************************************************************
 *				wdrv_dbGetStationAPExtRates						                                  *
*************************************************************************
DESCRIPTION: receives the station index, returns the station extended rates used in (re)associate response
                                                                                       
INPUT:     index  

OUTPUT:     value len

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetStationAPExtRates(UINT16 index, char *value, UINT8 *len)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	*len = stationParams[index].apExtRatesLen;
	PAL_memCopy(value, stationParams[index].apExtRates, stationParams[index].apExtRatesLen);
   	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}


/*************************************************************************
 *						wdrv_dbGetStationAcxBitWiseRates				                                  *
*************************************************************************
DESCRIPTION: receives the station index, returns the station rates in ACX forma
                                                                                       
INPUT:       index

OUTPUT:    acxBitWiseRates  

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetStationAcxBitWiseRates(UINT16 index, UINT16 *acxBitWiseRates)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	*acxBitWiseRates = stationParams[index].bitWiseSupRates;
   	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}

/*************************************************************************
 *				wdrv_dbSetCurrentActiveRateWithFallback						                                  *
*************************************************************************
DESCRIPTION: update the station currentActiveRateWithFallback in the station params table
                                                                                       
INPUT:      index 

OUTPUT:      currentActiveRateWithFallback

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetCurrentActiveRateWithFallback(UINT16 index, UINT16 *currentActiveRateWithFallback)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	stationParams[index].currentActiveRateWithFallback = *currentActiveRateWithFallback;
   	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}

/*************************************************************************
 *					wdrv_dbGetCurrentActiveRateWithFallback					                                  *
*************************************************************************
DESCRIPTION:  receives the station index, returns the station currentActiveRateWithFallback
                                                                                       
INPUT:   index    

OUTPUT:      currentActiveRateWithFallback

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetCurrentActiveRateWithFallback(UINT16 index, UINT16 *currentActiveRateWithFallback)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	*currentActiveRateWithFallback = stationParams[index].currentActiveRateWithFallback;
   	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}

/*************************************************************************
 *								wdrv_dbSetListenInterval		                                  *
*************************************************************************
DESCRIPTION: update the station listen interval in the station params table
                                                                                       
INPUT:     index  listenInterval

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetListenInterval(UINT16 index, UINT16 *listenInterval)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	stationParams[index].listenInterval = *listenInterval;
   	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}

/*************************************************************************
 *				wdrv_dbGetListenInterva						                                  *
*************************************************************************
DESCRIPTION: receives the station index, returns the station listen interval
                                                                                       
INPUT:      index 

OUTPUT:      listenInterval

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetListenInterval(UINT16 index, UINT16 *listenInterval)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	*listenInterval = stationParams[index].listenInterval;
   	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}

/*************************************************************************
 *								wdrv_dbSetTIprotectionSupport		                                  *
*************************************************************************
DESCRIPTION: update the TI protection support in the station params table
                                                                                       
INPUT:    index   support

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetTIprotectionSupport(UINT16 index, BOOL *support)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	stationParams[index].TIprotectionSupported = *support;
   	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}

/*************************************************************************
 *					wdrv_dbGetTIprotectionSuppor					                                  *
*************************************************************************
DESCRIPTION:  receives the station index, returns the station TI protection support
                                                                                       
INPUT:       index

OUTPUT:      support

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetTIprotectionSupport(UINT16 index, BOOL *support)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	*support = stationParams[index].TIprotectionSupported;
   	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}

/*************************************************************************
 *						wdrv_dbSetCommonFeatureSet				                                  *
*************************************************************************
DESCRIPTION: update the FeatureSet in the station params table
                                                                                       
INPUT:    index , set

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetCommonFeatureSet(UINT16 index, UINT8 *set)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	stationParams[index].CommonFeatureSet = *set;
   	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}
 
/*************************************************************************
 *				wdrv_dbGetCommonFeatureSet						                                  *
*************************************************************************
DESCRIPTION: receives the station index, returns the Common FeatureSet for AP-STA
                                                                                       
INPUT:      index 

OUTPUT:   set   

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetCommonFeatureSet(UINT16 index, UINT8 *set)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	*set = stationParams[index].CommonFeatureSet;
   	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}

/*************************************************************************
 *						wdrv_dbSetStationWoneIndex				                                  *
*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:      index, woneIndex

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void    wdrv_dbSetStationWoneIndex(UINT16 index, UINT8 woneIndex)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	stationParams[index].WoneIdentifierIndex = woneIndex;
   	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}

/*************************************************************************
 *			wdrv_dbGetStationWoneIndex							                                  *
*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       index

OUTPUT:      woneIndex

RETURN:      
                                                                                                                                                          
************************************************************************/
void    wdrv_dbGetStationWoneIndex(UINT16 index, UINT8 *woneIndex)
{     
   unsigned int flags;

   *woneIndex = 0xff;

    if (index < MAX_STATIONS)
   {
      PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
      
      *woneIndex = stationParams[index].WoneIdentifierIndex;
        if (staTable[index].free == TRUE)
	  {
		  printf("wdrv_dbGetStationWoneIndex: This STA entry was released: %d !!!\n",index); 
	  }      
      PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
        if (*woneIndex >= MAX_WONE_NET)
	  {
		  printf("wdrv_dbGetStationWoneIndex: wrong woneIndex = %d\n",*woneIndex);
         *woneIndex = 0xff;
	  }
      
   }
}

/*************************************************************************
 *				 wdrv_dbGetStationRxPacketCounter						                                  *
*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:   index    

OUTPUT:    unicastCounter  

RETURN:      
                                                                                                                                                          
************************************************************************/
void    wdrv_dbGetStationRxPacketCounter(UINT16 index, UINT32 *unicastCounter)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */   
    *unicastCounter = stationParams[index].rxPacketCounter;
   	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}

/*************************************************************************
 *				wdrv_dbGetStationTxPacketCounter						                                  *
*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:     index  

OUTPUT:    unicastCounter  

RETURN:      
                                                                                                                                                          
************************************************************************/
void    wdrv_dbGetStationTxPacketCounter(UINT16 index, UINT32 *unicastCounter)
{
    *unicastCounter = stationParams[index].txPacketCounter;
}

/*************************************************************************
 *				wdrv_dbResetStationRxPacketCounter						                                  *
*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       index

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void    wdrv_dbResetStationRxPacketCounter(UINT16 index)
{
	unsigned int flags;

    PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
    stationParams[index].rxPacketCounter =0;
    PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}

/*************************************************************************
 *				wdrv_dbResetStationTxPacketCounter						                                  *
*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       index

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void    wdrv_dbResetStationTxPacketCounter(UINT16 index)
{
	unsigned int flags;

    PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
    stationParams[index].txPacketCounter =0;    
    PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}


/* update the station capabilities in the station params table */
/*************************************************************************
 *					wdrv_dbSetStationCapabilities					                                  *
*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:    index , capabilities

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetStationCapabilities(UINT16 index, UINT16 *capabilities)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	stationParams[index].capabilities = *capabilities;
   	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}

/* Returns the station capabilities */
/*************************************************************************
 *			wdrv_dbGetStationCapabilities							                                  *
*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:     index  

OUTPUT:    capabilities  

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetStationCapabilities(UINT16 index, UINT16 *capabilities)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	*capabilities = stationParams[index].capabilities;
   	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}

/* reset the station params */
/*************************************************************************
 *						wdrv_dbResetStationParams				                                  *
*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:      index 

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/

void wdrv_dbResetStationParams(UINT16 index)
{
	/* power save clean */
	wdrv_powerSaveCleanStaList(index, stationParams[index].WoneIdentifierIndex);
	
	PAL_memSet(&stationParams[index], 0, sizeof(stationParams_T));

    if (REPORT_LEVEL(RL_DB))
		wdrv_utilsReport(RL_DB, PRI_INFORMATION, "WDRV_DB: Reseting station params.\n");
	/* TODO: aging callbacks of other modules */
}
/*************************************************************************
 *			wdrv_dbGetNumOfStationNotPBCCCapable							                                  *
*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       

OUTPUT:      num

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetNumOfStationNotPBCCCapable(UINT16 *num)
{
	*num = numStationsNotPBCCCapabale;
}

/*************************************************************************
 *					wdrv_dbGetCurrentPBCCSupportMode					                                  *
*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       

OUTPUT:    support  

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetCurrentPBCCSupportMode(BOOL *support)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	*support = currentPBCCSupportMode;
    PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);
}
/*************************************************************************
 *					wdrv_dbSetCurrentPBCCSupportMode					                                  *
*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:     support  

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetCurrentPBCCSupportMode(BOOL support)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	currentPBCCSupportMode = support;
    PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);
}
/*************************************************************************
 *				wdrv_dbGetNumOfStationNotShortPreambleCapable						                                  *
*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       

OUTPUT:    num  

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetNumOfStationNotShortPreambleCapable(UINT16 *num)
{
	*num = numStationsNotShortPreambleCapabale;
}
/*************************************************************************
 *					wdrv_dbGetCurrentShortPreambleSupport					                                  *
*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       

OUTPUT:     support 

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetCurrentShortPreambleSupport(BOOL *support)
{
	*support = currentShortPreambleSupport;
}
/*************************************************************************
 *				wdrv_dbGetNumOfStationNotSlotTimeCapable						                                  *
*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       

OUTPUT:     num 

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetNumOfStationNotSlotTimeCapable(UINT16 *num)
{
	*num = numStationsNotSlotTimeCapabale;
}
/*************************************************************************
 *	wdrv_dbGetCurrentShortSlotTimeSupport									                                  *
*************************************************************************
DESCRIPTION: Get current short slot time support
                                                                                       
INPUT:       

OUTPUT:      support

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetCurrentShortSlotTimeSupport(BOOL *support)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	*support = currentShortSlotTimeSupport;
    PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);
}
/*************************************************************************
 *					wdrv_dbSetCurrentShortSlotTimeSupport					                                  *
*************************************************************************
DESCRIPTION: Sets current short slot time support
                                                                                       
INPUT:  support     

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetCurrentShortSlotTimeSupport(BOOL support)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	currentShortSlotTimeSupport = support;
    PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);
}
/*************************************************************************
 *				 wdrv_dbGetCurrentERPProtectionMode						                                  *
*************************************************************************
DESCRIPTION: Get current ERP protection mode
                                                                                       
INPUT:       

OUTPUT:  support    

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetCurrentERPProtectionMode(BOOL *support)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	*support = currentERPProtectionMode;
    PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);
}
/*************************************************************************
 *			wdrv_dbSetCurrentERPProtectionMode							                                  *
*************************************************************************
DESCRIPTION: Set current ERP protection mode
                                                                                       
INPUT:      support 

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetCurrentERPProtectionMode(BOOL support)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	currentERPProtectionMode = support;
    PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);
}
/*************************************************************************
 *		wdrv_dbGetCurrentERPNonPresent								                                  *
*************************************************************************
DESCRIPTION: Get current ERP Non Present
                                                                                       
INPUT:       

OUTPUT:      value

RETURN:      
                                                                                                                                                          
************************************************************************/
void  wdrv_dbGetCurrentERPNonPresent(BOOL *value)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	*value = currentERPNonPresent;
    PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);
}
/*************************************************************************
 *			wdrv_dbSetCurrentERPNonPresen							                                  *
*************************************************************************
DESCRIPTION: Set current ERP Non Presen
                                                                                       
INPUT:    value   

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void  wdrv_dbSetCurrentERPNonPresent(BOOL value)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */ 
    currentERPNonPresent = value;
    PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);
}
/*************************************************************************
 *			wdrv_dbSetStationConcatSuppor							                                  *
*************************************************************************
DESCRIPTION: Set station concatenation support
                                                                                       
INPUT:       index,support

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetStationConcatSupport(UINT16 index, BOOL support)
{
	BOOL  enableConcat;
	unsigned int flags;

	wdrv_dbGetConcatEnable(&enableConcat);

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */

	if ((enableConcat == TRUE) && (support == TRUE))
		stationParams[index].fourX_cap.concatEn = TRUE;
	else
		stationParams[index].fourX_cap.concatEn = FALSE;

   	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}
/*************************************************************************
 *			wdrv_dbGetStationConcatSupport							                                  *
*************************************************************************
DESCRIPTION: Get station concatenation support
                                                                                       
INPUT:     index  

OUTPUT:   mode   

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetStationConcatSupport(UINT16 index, BOOL *mode)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	*mode = stationParams[index].fourX_cap.concatEn;
   	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}
/*************************************************************************
 *				wdrv_dbSetStationConcatMaxSize						                                  *
*************************************************************************
DESCRIPTION: Set station concatenation max size
                                                                                       
INPUT:      index, size

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetStationConcatMaxSize(UINT16 index, UINT16 size)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	stationParams[index].ConcatMaxSize = size;
   	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}
/*************************************************************************
 *			wdrv_dbGetStationConcatMaxSize							                                  *
*************************************************************************
DESCRIPTION: Get station concatenation max size
                                                                                       
INPUT:   index    

OUTPUT:      size

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetStationConcatMaxSize(UINT16 index, UINT16 *size)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	*size = stationParams[index].ConcatMaxSize;
   	/* if the station cipher is TKIP, we need to keep place for the MIC and extended IV. */
	if (stationParams[index].cipherSuiteType == CIPHER_TKIP)
		*size = *size - MIC_LEN - 4;
	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}
/*************************************************************************
 *				wdrv_dbSetStationAckEmulSupport						                                  *
*************************************************************************
DESCRIPTION: Set station ack emulation
                                                                                       
INPUT:       index, support

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetStationAckEmulSupport(UINT16 index, BOOL support)
{
	BOOL  enableAckEmul;
	unsigned int flags;

	wdrv_dbGetAckEmulEnable(&enableAckEmul);

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */

	if ((enableAckEmul == TRUE) && (support == TRUE))
		stationParams[index].fourX_cap.ackEmulEn = TRUE;
	else
		stationParams[index].fourX_cap.ackEmulEn = FALSE;

   	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}
/*************************************************************************
 *	wdrv_dbGetStationAckEmul									                                  *
*************************************************************************
DESCRIPTION: Get station ack emulation
                                                                                       
INPUT:    index   

OUTPUT:      mode

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetStationAckEmul(UINT16 index, BOOL *mode)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	*mode = stationParams[index].fourX_cap.ackEmulEn;
   	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}

/*************************************************************************
 *		wdrv_dbSetStationTIcapEnable								                                  *
*************************************************************************
DESCRIPTION: update the station capabilities in the station params table
                                                                                       
INPUT:     index, support

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetStationTIcapEnable(UINT16 index, BOOL support)
{	
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */

	if (support == TRUE)
		stationParams[index].TIcapEn = TRUE;
	else
		stationParams[index].TIcapEn = FALSE;

   	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}

/*************************************************************************
 *			wdrv_dbSetStationFourXActive							                                  *
*************************************************************************
DESCRIPTION: update the station capabilities in the station params table
                                                                                       
INPUT:       index, fourXActive

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetStationFourXActive(UINT16 index, BOOL fourXActive)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */

    if (stationParams[index].fourX_cap.concatEn == FALSE)
        stationParams[index].fourXActive = FALSE;
    else
	    stationParams[index].fourXActive = fourXActive;

   	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}

/*************************************************************************
 *		wdrv_dbGetStationTIcapEnable								                                  *
*************************************************************************
DESCRIPTION: Returns the station capabilities
                                                                                       
INPUT:     index  

OUTPUT:      mode

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetStationTIcapEnable(UINT16 index, BOOL *mode)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	*mode = stationParams[index].TIcapEn;
   	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}

/*************************************************************************
 *		wdrv_dbGetStationFourXActive								                                  *
*************************************************************************
DESCRIPTION: Returns the station capabilities
                                                                                       
INPUT:      index 

OUTPUT:     mode 

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetStationFourXActive(UINT16 index, BOOL *mode)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
	*mode = stationParams[index].fourXActive;
   	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}

/*************************************************************************
 *					disableStationPBCCSupport					                                  *
*************************************************************************
DESCRIPTION: Disable PBCC support in station params table
                                                                                       
INPUT:       index

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
static void disableStationPBCCSupport(UINT16 index)
{
	if ((stationParams[index].activeRate == DOT11_RATE_5_5) ||
		(stationParams[index].activeRate == DOT11_RATE_11) ||
		(stationParams[index].activeRate == DOT11_RATE_22))
	{
		/* Disable PBCC mode for 5.5M, 11M & 22M rates */
		stationParams[index].activeRateAcx &= (~ACX_RATE_PBCC);

		/* Downgrade 22M to 11M as 22M can't work without PBCC */
		if (stationParams[index].activeRate == DOT11_RATE_22)
		{
			stationParams[index].activeRateAcx &= (~ACX_RATE_22);
			stationParams[index].activeRateAcx |= ACX_RATE_11;
		}

		wdrv_txRateAdapSetRate(index, stationParams[index].activeRateAcx);
	}

	#if 0
	if (stationParams[i].activeRate == 0x2C)
	{		/* if station active rate is 22, */
		stationParams[i].activeRateAcx = 110;		/* downgrade it to 11, no pbcc */
		wdrv_txRateAdapSetRate( i, 110);
    } else if ((stationParams[i].activeRate == 0x16) || (stationParams[i].activeRate == 0x0B))
	{		/* if the rate is pbcc encoded */
		stationParams[i].activeRateAcx &= 0x7F;		/* decode the rate as no pbcc */
		wdrv_txRateAdapSetRate( i,stationParams[i].activeRateAcx);
	}
	#endif
}

/*************************************************************************
 *					enableStationPBCCSupport					                                  *
*************************************************************************
DESCRIPTION: emable sta PBCC support
                                                                                       
INPUT:     index  

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
static void enableStationPBCCSupport(UINT16 index)
{
	if ((stationParams[index].activeRate == DOT11_RATE_5_5) ||
		(stationParams[index].activeRate == DOT11_RATE_11) ||
		(stationParams[index].activeRate == DOT11_RATE_22))
	{
		/* Enable PBCC mode for 5.5M, 11M & 22M rates */
		stationParams[index].activeRateAcx |= ACX_RATE_PBCC;

		/* Upgrade back to 22M, since PBCC enabled now */
		if (stationParams[index].activeRate == DOT11_RATE_22)
		{
			stationParams[index].activeRateAcx &= (~ACX_RATE_11);
			stationParams[index].activeRateAcx &= (~ACX_RATE_5_5);
			stationParams[index].activeRateAcx |= ACX_RATE_22;
		}

		wdrv_txRateAdapSetRate(index, stationParams[index].activeRateAcx);
	}

	#if 0
	if (stationParams[i].activeRate == 0x2C)
	{		/* if station active rate is 22, */
		stationParams[i].activeRateAcx = 220;		/* upgrade acx encoded rate to 22, pbcc */
		wdrv_txRateAdapSetRate( i,/*Rate*/220);
	}

	else if ((stationParams[i].activeRate == 0x0B) || (stationParams[i].activeRate == 0x16))
	{		/* if the rate should be pbcc encoded */
		stationParams[i].activeRateAcx |= ACX_RATE_PBCC;		/* encode the rate as pbcc */
		wdrv_txRateAdapSetRate( i,stationParams[i].activeRateAcx);
	}
	#endif
}

STATUS wdrv_dbGetStationType(UINT16 index, drvDbHashStaType_e *pStaType)
{
    if(wdrv_dbIsTableIndexExists(index)) {
        *pStaType = stationParams[index].staType;
        return (OK);
    }
    return (NOK);
}


/*************************************************************************
 *                        wdrv_dbPerformPBCCAlgorithmStatic					 *
 *************************************************************************
DESCRIPTION: This function  implement the PBCC static algorithm as following:
			1. Upon station association: 
				a. If the station supports pbcc and pbcc current support is TRUE:
					1. We do nothing
				b. If the station supports pbcc and pbcc current support is FALSE:
					1. We change its active rate (only the one which is configured to the ACX) to a non pbcc one.
				c. If the station does not support pbcc and it is the first non-pbcc station to associate:
					1. We set the rates of all pbcc stations to non pbcc rates,
					2. We set the beacon's pbcc bit to FALSE.
					3. We set the current pbcc suport mode to FALSE
				d. If the station does not support pbcc and it is not the first non-pbcc station to associate:
					1. We do nothing
			2. Upon station dis association: 
				a. If the station supports pbcc and pbcc current support is TRUE:
					1. We do nothing
				b. If the station supports pbcc and pbcc current support is FALSE:
					1. We do nothing
				c. If the station does not support pbcc and it is the last non-pbcc station to disassociate:
					1. We set the rates of all pbcc stations to pbcc rates,
					2. We set the beacon's pbcc bit to TRUE.
					3. We set the current pbcc suport mode to TRUE
				d. If the station does not support pbcc and it is not the last non-pbcc station to associate:
					1. We do nothing
                                                                                                   
INPUT:       The station index
			 The station action (associate, disassociate)

OUTPUT:      

RETURN:                                                                                                                                                   
************************************************************************/
void wdrv_dbPerformPBCCAlgorithmStatic(UINT16 index, algo_action_t action)
{
	BOOL   PBCCdynamicActive;
	
	if (action == ASSOCIATE)
	{
		if (!((stationParams[index].capabilities >> CAP_PBCC_SHIFT) & CAP_PBCC_MASK))  		/* station does not support PBCC */
		{
			numStationsNotPBCCCapabale++;
			
			/* if pbcc mode have been changed  */
			if ((numStationsNotPBCCCapabale) == 1)
			{
                if (REPORT_LEVEL(RL_DYNAMIC_ALG))
					printf("going to set PBCC3: %d\n",0);

				/* now, for stations that are PBCC, set active rate to non PBCC. */
				wdrv_dbSetPBCCStationsSupport(FALSE);
	
				whal_apiSetPbcc(0,0);
				currentPBCCSupportMode = FALSE;
			}
        } else    /* station supports PBCC */
		{
			if (currentPBCCSupportMode == FALSE)
			{	
				disableStationPBCCSupport(index);
			}
		}
	}

	else /* DISASSOCIATES */
	{
		if (!((stationParams[index].capabilities >> CAP_PBCC_SHIFT) & CAP_PBCC_MASK)) 		/* station does not support PBCC*/
		{

			/* This should always hold but in any case the number should be non-negative*/
			if (numStationsNotPBCCCapabale > 0) {
				numStationsNotPBCCCapabale--;
			}

			
			/* if there are no more notPBCCcap STAs and there is no external notPBCCcap BSSs
			   we can turn the PBCC on */
			if (numStationsNotPBCCCapabale == 0)
			{
				wdrv_mgmtIsPBCCexternalActive(&PBCCdynamicActive);
                if (PBCCdynamicActive == FALSE)
				{
                    if (REPORT_LEVEL(RL_DYNAMIC_ALG))
						printf("going to set PBCC4: %d\n",1);

					/* now, for stations that are PBCC, set active rate to PBCC. */
					wdrv_dbSetPBCCStationsSupport(TRUE);
	
					whal_apiSetPbcc(0,1);
					currentPBCCSupportMode = TRUE;
				}
			}
		}
	}
}


/*************************************************************************
 *                        wdrv_dbPerformPreambleAlgorithmStatic					 *
 *************************************************************************
DESCRIPTION: This function implement the Preamble static algorithm.
             It is called only if AP supports Short Preamble

                                                                                                   
INPUT:       index - the station index
			 action - the station action (associate or disassociate)

OUTPUT:      

RETURN:                                                                                                                                                   
************************************************************************/
void wdrv_dbPerformPreambleAlgorithmStatic(UINT16 index, algo_action_t action)
{
	int stationShortPreambleSupport;
	unsigned int flags;
	
	stationShortPreambleSupport = (stationParams[index].capabilities >> CAP_PREAMBLE_SHIFT) & CAP_PREAMBLE_MASK;

	if (action == ASSOCIATE)
	{
		if (!stationShortPreambleSupport)
		{
            PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
			numStationsNotShortPreambleCapabale++;
			
			/* if first station not capable */
			if (numStationsNotShortPreambleCapabale == 1)
			{
               currentShortPreambleSupport = FALSE;
               PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);
			   whal_apiSetERPLongPreambleOnly(0, 1);
            } else
               PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);
		}
	}

	else /* DISASSOCIATES */
	{
		if (!stationShortPreambleSupport)
		{
            PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */

			/* This should always hold but in any case the number should be non-negative*/
			if (numStationsNotShortPreambleCapabale > 0) 
			{
				numStationsNotShortPreambleCapabale--;
			}

			/* if no more stations that are not capable */
			if (numStationsNotShortPreambleCapabale == 0)
			{
                currentShortPreambleSupport = TRUE;
                PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);
				whal_apiSetERPLongPreambleOnly(0, 0);
            } else
                PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);
		}
	}
}


/*************************************************************************
 *                        wdrv_dbPerformSlotTimeAlgorithmStatic					 *
 *************************************************************************
DESCRIPTION: This function implement the Slot Time static algorithm.
             It is called only if AP supports short slot time

                                                                                                   
INPUT:       index - the station index
			 action - the station action (associate or disassociate)

OUTPUT:      

RETURN:                                                                                                                                                   
************************************************************************/
void wdrv_dbPerformSlotTimeAlgorithmStatic(UINT16 index, algo_action_t action)
{
	int stationShortSlotTimeSupport;
	
	stationShortSlotTimeSupport = (stationParams[index].capabilities >> CAP_SLOTTIME_SHIFT) & CAP_SLOTTIME_MASK;

	if (action == ASSOCIATE)
	{
		if (!stationShortSlotTimeSupport)
		{
			numStationsNotSlotTimeCapabale++;
			
			/* if first station not capable */
			if (numStationsNotSlotTimeCapabale == 1)
			{
				wdrv_mgmtShortSlotTimeSM(INPUT_SLOTTIME_B_STA_IN_BSS);
			}
		}
	}

	else /* DISASSOCIATES */
	{
		if (!stationShortSlotTimeSupport)
		{
			/* This should always hold but in any case the number should be non-negative*/
			if (numStationsNotSlotTimeCapabale > 0) 
			{
				numStationsNotSlotTimeCapabale--;
			}
			
			/* if no more stations that are not capable */
			if (numStationsNotSlotTimeCapabale == 0)
			{
				wdrv_mgmtShortSlotTimeSM(INPUT_SLOTTIME_G_STA_ONLY);
			}
		}
	}
}


/*************************************************************************
 *                        wdrv_dbPerformERPProtectionAlgorithmStatic					 *
 *************************************************************************
DESCRIPTION: This function implement the Slot Time static algorithm.
             It is called only if AP supports short slot time
			 (slot time capability is true in NVRAM)

                                                                                                   
INPUT:       index - the station index
			 action - the station action (associate or disassociate)

OUTPUT:      

RETURN:                                                                                                                                                   
************************************************************************/
void wdrv_dbPerformERPProtectionAlgorithmStatic(UINT16 index, algo_action_t action)
{
	BOOL nonERPStation;
	BOOL ERPdynamicActive;

	
	if (wdrv_dbIsRateSupported(index, ACX_RATE_ALL_OFDM_MASK))
	{
		/* Station supports OFDM so it is not a nonERP station */
		nonERPStation = FALSE;
    } else
	{
		nonERPStation = TRUE;
	}

	if (action == ASSOCIATE)
	{
		if (nonERPStation)
		{
			numOfNonERPStations++;
			
			/* if first nonERP station */
			if (numOfNonERPStations == 1)
			{
                if (REPORT_LEVEL(RL_DYNAMIC_ALG))
					wdrv_utilsReport(RL_DYNAMIC_ALG, PRI_INFORMATION, "WDRV_MGMT: going to set ERP1: 1,1\n");
				
				whal_apiSetERPProtection(0,1,1);
			}
        } else
			numOfERPstations++;
	}

	else /* DISASSOCIATES */
	{
		if (nonERPStation)
		{
			numOfNonERPStations--;
			
			/* if no more nonERP stations */
			if (numOfNonERPStations == 0)
			{				
				wdrv_mgmtIsERPexternalActive(&ERPdynamicActive);
                if (ERPdynamicActive)
				{				
                    if (REPORT_LEVEL(RL_DYNAMIC_ALG))
						wdrv_utilsReport(RL_DYNAMIC_ALG, PRI_INFORMATION, "WDRV_MGMT: going to set ERP2: 0,1\n");
					
					whal_apiSetERPProtection(0,0,1);
                } else
                {
                    if (REPORT_LEVEL(RL_DYNAMIC_ALG))
						wdrv_utilsReport(RL_DYNAMIC_ALG, PRI_INFORMATION, "WDRV_MGMT: going to set ERP3: 0,0\n");

					whal_apiSetERPProtection(0,0,0);
				}				
			}
        } else
			numOfERPstations--;
	}
}


/*************************************************************************
 *                        wdrv_dbSetPBCCStationsSupport			 		*
 *************************************************************************
DESCRIPTION: This function sets the pbcc support of station according to the input mode.
			Called by the dynamic or static pbcc algorithm function.
			1. If mode is TRUE, set the rates of all pbcc station to pbcc rates.
			2. If mode is FALSE, set the rates of all pbcc station to non-pbcc rates.
                                                                                                   
INPUT:       The required pbcc mode

OUTPUT:      

RETURN:                                                                                                                                                   
************************************************************************/
void wdrv_dbSetPBCCStationsSupport(BOOL mode)
{
	int i, state;

	if (mode == TRUE)
	{
		for (i = 0; i < MAX_STATIONS; i++)
		{
			wdrv_mgmtGetState(i, &state);

			if (((stationParams[i].capabilities >> CAP_PBCC_SHIFT) & CAP_PBCC_MASK) && ((state == STATE_ASSOCIATED) || (state == STATE_AUTHORIZED)))
			{	
				enableStationPBCCSupport(i);
			}
		}
	}

	else
	{
		for (i = 0; i < MAX_STATIONS; i++)
		{
			wdrv_mgmtGetState(i, &state);

			if (((stationParams[i].capabilities >> CAP_PBCC_SHIFT) & CAP_PBCC_MASK)	&& ((state == STATE_ASSOCIATED) || (state == STATE_AUTHORIZED))) 
			{	
				disableStationPBCCSupport(i);
			}
		}
	}
}


/*************************************************************************
 *                        wdrv_dbIsStationPBCCCapable			 		*
 *************************************************************************
DESCRIPTION: Returns whether the STA is PBCC capable or not
                                                                                                   
INPUT:       STA index

OUTPUT:      

RETURN:  PBCC cap/not cap (TRUE/FALSE)                                                                                                                                                 
************************************************************************/
BOOL wdrv_dbIsStationPBCCCapable(UINT16 index)
{
	if ((stationParams[index].capabilities >> CAP_PBCC_SHIFT) & CAP_PBCC_MASK)
		return TRUE;
	else
		return FALSE;
}

/*************************************************************************
 *                        wdrv_dbIsStationShortSlotTimeCapable	 		*
 *************************************************************************
DESCRIPTION: Returns whether the STA is short slot time capable or not
                                                                                                   
INPUT:       STA index

OUTPUT:      

RETURN:  Short Slot Time cap/not cap (TRUE/FALSE)                                                                                                                                                 
************************************************************************/
BOOL wdrv_dbIsStationShortSlotTimeCapable(UINT16 index)
{
    if ((stationParams[index].capabilities >> CAP_SLOTTIME_SHIFT) & CAP_SLOTTIME_MASK)
		return TRUE;
	else
		return FALSE;
}

/*************************************************************************
 *                        wdrv_dbIsStationERP	 		*
 *************************************************************************
DESCRIPTION: Returns whether the STA is an ERP or NonERP
                                                                                                   
INPUT:       STA index

OUTPUT:      

RETURN: ERP/NonERP (TRUE/FALSE)                                                                                                                                                  
************************************************************************/
BOOL wdrv_dbIsStationERP(UINT16 index)
{
    if (wdrv_dbIsRateSupported(index, ACX_RATE_ALL_OFDM_MASK))
		return TRUE;
	else
		return FALSE;
}

/*************************************************************************
 *                        wdrv_dbPrintStaRsnIeTable            	 		*
 *************************************************************************
DESCRIPTION: 
                                                                                                   
INPUT:      

OUTPUT:      

RETURN:                                                                                                                                             
************************************************************************/
void wdrv_dbPrintStaRsnIeTable(void)
{
   int i, state;
   char stateString[MAX_PRINT_STRING];

   printf("Stations RSN IE table\n");
   printf("Number of Stations:%d\n", wdrv_dbGetNumStationsByState() /*numStations*/);

   for (i = 0; i < MAX_STATIONS; i++)
   {

      wdrv_mgmtGetState(i, &state);
      if (state != STATE_IDLE)
      {
         printf(" [%02d]   ", i);

         printf("%02X-%02X-%02X-%02X-%02X-%02X  ", staTable[i].macAddr[0], staTable[i].macAddr[1], staTable[i].macAddr[2], staTable[i].macAddr[3], staTable[i].macAddr[4], staTable[i].macAddr[5]);

         PAL_memSet(stateString, 0, MAX_PRINT_STRING);
         wdrv_mgmtGetStateString(i, stateString);
         printf("%s ", stateString);

         if (stationParams[i].generateTxMicFailure == TRUE)
            printf("	generateTxMicFailure for this station is TRUE\n" );
         else
            printf("	generateTxMicFailure for this station is FALSE\n" );

         if (stationParams[i].cipherSuiteType == CIPHER_NONE)
            printf("	NONE\n" );
         else if (stationParams[i].cipherSuiteType == CIPHER_WEP)
            printf("	WEP\n" );
         else if (stationParams[i].cipherSuiteType == CIPHER_TKIP)
            printf("	TKIP\n" );
         else
            printf("	Invalid\n" );

         printf("RX MIC Key:	0x%X 0x%X 0x%X 0x%X 0x%X 0x%X 0x%X 0x%X\n", stationParams[i].key.data.tkip.rx_mic_key[0], stationParams[i].key.data.tkip.rx_mic_key[1], stationParams[i].key.data.tkip.rx_mic_key[2], stationParams[i].key.data.tkip.rx_mic_key[3], stationParams[i].key.data.tkip.rx_mic_key[4], stationParams[i].key.data.tkip.rx_mic_key[5], stationParams[i].key.data.tkip.rx_mic_key[6], stationParams[i].key.data.tkip.rx_mic_key[7]);
         printf("TX MIC Key:	0x%X 0x%X 0x%X 0x%X 0x%X 0x%X 0x%X 0x%X\n", stationParams[i].key.data.tkip.tx_mic_key[0], stationParams[i].key.data.tkip.tx_mic_key[1], stationParams[i].key.data.tkip.tx_mic_key[2], stationParams[i].key.data.tkip.tx_mic_key[3], stationParams[i].key.data.tkip.tx_mic_key[4], stationParams[i].key.data.tkip.tx_mic_key[5], stationParams[i].key.data.tkip.tx_mic_key[6], stationParams[i].key.data.tkip.tx_mic_key[7]);

         printf("\n\n");
      }
   }
}



/*************************************************************************
 * Util function to print rate in MBPS units 
 *************************************************************************
DESCRIPTION: 
                                                                                                   
INPUT:      

OUTPUT:      

RETURN:                                                                                                                                             
************************************************************************/
int wdrv_printRateInMbpsUnit(UINT8 rate ,char *buf)
{
	UINT32 rate1, rate2, rate3;
	BOOL basic;
	int len =0;

	if (rate)
	{
		rate1 = rate;
		basic = rate1 & 0x80;
		rate1 &= 0x7f;
		rate1 *= 500;
		rate2 = rate1/1000;
		rate3 = rate1%1000;

		if (rate3)
			len +=sprintf(buf +len,"%d.%dMbps", rate2, 5);
		else
			len +=sprintf(buf +len,"%dMbps", rate2);

		if (basic)
			len +=sprintf(buf +len,"B");
	}

	return len;
}

/*************************************************************************
 *                     wdrv_dbSetStationEncryptionKey            	 
 *************************************************************************
DESCRIPTION: 
                                                                                                   
INPUT:      

OUTPUT:      

RETURN:                                                                                                                                                 
*************************************************************************/
void wdrv_dbSetStationEncryptionKey(UINT16 index, UINT8 cipher, UINT16 keySize, UINT8 *keyValue)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */

    stationParams[index].cipherSuiteType = cipher;

    if(cipher == CIPHER_NONE)
        stationParams[index].keyInUse = FALSE;
    else
        stationParams[index].keyInUse = TRUE;

	if (keyValue == NULL)
	{
		PAL_memSet(&stationParams[index].key.data, 0, WPA_MAX_WEP_KEY_LEN);	
		stationParams[index].key.dataSize = 0;
    } else
	{
		PAL_memCopy(&stationParams[index].key.data, keyValue, keySize);	
		stationParams[index].key.dataSize = keySize;
	}

	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}


/*************************************************************************
 *                     wdrv_dbGetStationEncryptionKey            	 
 *************************************************************************
DESCRIPTION: 
                                                                                                   
INPUT:      

OUTPUT:      

RETURN:                                                                                                                                                 
*************************************************************************/
void wdrv_dbGetStationEncryptionKey(UINT16 index, UINT16 *keySize, UINT8 *keyValue, UINT8 *cipher)
{
	PAL_memCopy(keyValue, &stationParams[index].key.data ,stationParams[index].key.dataSize);
    *keySize = stationParams[index].key.dataSize;
    *cipher = stationParams[index].cipherSuiteType;
}

/*************************************************************************
 *                     wdrv_dbGetKeyInUse            	 
 *************************************************************************
DESCRIPTION: 
                                                                                                   
INPUT:      

OUTPUT:      

RETURN:                                                                                                                                                 
*************************************************************************/
void wdrv_dbGetKeyInUse(UINT16 index, UINT8 *value)
{
    *value = stationParams[index].keyInUse;
}

/*************************************************************************
 *                     wdrv_dbSetStationTxMicKey            	 
 *************************************************************************
DESCRIPTION: 
                                                                                                   
INPUT:      

OUTPUT:      

RETURN:                                                                                                                                                 
*************************************************************************/
void wdrv_dbSetStationTxMicKey(UINT16 index, UINT8 *keyValue)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */

	if (keyValue == NULL)
		PAL_memSet(stationParams[index].key.data.tkip.tx_mic_key, 0, MICHAEL_KEY_LEN);	
	else
		PAL_memCopy(stationParams[index].key.data.tkip.tx_mic_key, keyValue, MICHAEL_KEY_LEN);	

	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}


/*************************************************************************
 *                     wdrv_dbGetStationSuiteType            	 
 *************************************************************************
DESCRIPTION: 
                                                                                                   
INPUT:      

OUTPUT:      

RETURN:                                                                                                                                                 
*************************************************************************/
void wdrv_dbSetStationRxMicKey(UINT16 index, UINT8 *keyValue)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */

	if (keyValue == NULL)
		PAL_memSet(stationParams[index].key.data.tkip.rx_mic_key, 0, MICHAEL_KEY_LEN);	
	else
		PAL_memCopy(stationParams[index].key.data.tkip.rx_mic_key, keyValue, MICHAEL_KEY_LEN);	

	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}

/*************************************************************************
 *                                 	 
 *************************************************************************
DESCRIPTION: 
                                                                                                   
INPUT:      

OUTPUT:      

RETURN:                                                                                                                                                 
*************************************************************************/
void wdrv_dbGetStationTxMicKey(UINT16 index, UINT8 *keyValue)
{
	if (stationParams[index].generateTxMicFailure == TRUE)
	{
		PAL_memSet(keyValue, 1, MICHAEL_KEY_LEN);	
		stationParams[index].generateTxMicFailure = FALSE;
		printf("Generating a Tx MIC failure for station in index: %d\n", index);
    } else
		PAL_memCopy(keyValue, stationParams[index].key.data.tkip.tx_mic_key, MICHAEL_KEY_LEN);	
}

/*************************************************************************
 *       wdrv_dbGetStationRxMicKey                           	 
 *************************************************************************
DESCRIPTION: Sets station Rx Mic Key
                                                                                                   
INPUT:      index, keyValue

OUTPUT:      

RETURN:                                                                                                                                                 
*************************************************************************/
void wdrv_dbGetStationRxMicKey(UINT16 index, UINT8 *keyValue)
{
   unsigned int flags;

   PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */
   PAL_memCopy(keyValue, stationParams[index].key.data.tkip.rx_mic_key, MICHAEL_KEY_LEN);   
   PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
}


/*************************************************************************
 *       wdrv_dbIsDataFirstFromStation                          	 
 *************************************************************************
DESCRIPTION: Checks whether AP received packet from station
                                                                                                   
INPUT:   index   

OUTPUT:      

RETURN:   result                                                                                                                                        
*************************************************************************/
BOOL wdrv_dbIsDataFirstFromStation(UINT16 index)
{
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); /* START OF CRITICAL SECTION */

	if (stationParams[index].dataReceivedFromStation == FALSE)	
	{
		stationParams[index].dataReceivedFromStation = TRUE;
		PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
		return TRUE;
	}

	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  /* END OF CRITICAL SECTION */
	return FALSE;
}


/*************************************************************************
 *                       wdrv_dbGenerateTxMicFailure          	 
 *************************************************************************
DESCRIPTION: ReturnsGeneratefailure
                                                                                                   
INPUT:   mac   

OUTPUT:      

RETURN:                                                                                                                                                 
*************************************************************************/
void wdrv_dbGenerateTxMicFailure(char *mac)
{
   UINT16 index;
   int   rc;

   rc = wdrv_dbFindStaIndex(mac, &index);

   if (rc != OK)
      wdrv_utilsReport(RL_DB, PRI_INFORMATION,"wdrv_dbGenerateTxMicFailure: station not found\n");
   else
      stationParams[index].generateTxMicFailure = TRUE;
}
/*************************************************************************
 *		wdrv_dbIncreaseStationRxPacketCounter								                                  *
*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/

inline void wdrv_dbIncreaseStationRxPacketCounter(UINT16 index)
{
     stationParams[index].rxPacketCounter++;
}
/*************************************************************************
 *		wdrv_dbIncreaseStationTxPacketCounter								                                  *
*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
inline void wdrv_dbIncreaseStationTxPacketCounter(UINT16 index)
{
	if (index < MAX_STATIONS)
		stationParams[index].txPacketCounter++;
}
/*************************************************************************
 *		wdrv_dbGetStationSuiteType								                                  *
*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
inline void wdrv_dbGetStationSuiteType(UINT16 index, UINT8 *cipher)
{
	*cipher = stationParams[index].cipherSuiteType;
}
/*************************************************************************
 *			wdrv_dbSetWpaReq							                                  *
*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/

inline void wdrv_dbSetWpaReq(UINT16 index, UINT32 reason)
{
	stationParams[index].wpaReqReason = reason;
}
/*************************************************************************
 *		wdrv_dbGetWpaReq								                                  *
*************************************************************************
DESCRIPTION: REturns WPA Req
                                                                                       
INPUT:      index 

OUTPUT:     reason 

RETURN:      
                                                                                                                                                          
************************************************************************/

inline void wdrv_dbGetWpaReq(UINT16 index, UINT32 *reason)
{
	*reason = stationParams[index].wpaReqReason;
}

/*************************************************************************
 *		wdrv_dbGetNumStations								                                  *
*************************************************************************
DESCRIPTION: Returns number of stations
                                                                                       
INPUT:       

OUTPUT:      

RETURN:    result
                                                                                                                                                          
************************************************************************/
inline UINT16 wdrv_dbGetNumStations(void)
{
	return numStations;
}

/* Demiurg: WME */

/*************************************************************************
 *		wdrv_dbSetStaWME								                                  *
*************************************************************************
DESCRIPTION: Set station WME support status
                                                                                       
INPUT:   index,   val

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/

/*************************************************************************
 *		wdrv_dbSetStaWME								                                  *
*************************************************************************
DESCRIPTION: Set station WME support status
                                                                                       
INPUT:   index,   val

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetStaWME(UINT16 index, BOOL val) 
{
	stationParams[index].wme = val;
}
/*************************************************************************
 *		wdrv_dbGetStaWME								                                  *
*************************************************************************
DESCRIPTION: Get station WME support status
                                                                                       
INPUT:      index 

OUTPUT:      

RETURN:      result (TRUE/FALSE)
                                                                                                                                                          
************************************************************************/
BOOL  wdrv_dbGetStaWME(UINT16 index) 
{
    if (index < MAX_STATIONS)
		return stationParams[index].wme;
    return FALSE;
}
/************************************************************************
 *			wdrv_dbSetSecuritySeqNum							                          *
*************************************************************************
DESCRIPTION: Set security sequence number (used for recovery)
                                                                                       
INPUT:      index, lsb of sequence number 

OUTPUT:      

RETURN:      

NOTE: This function and DB itself use FW numbers for AC, which differ from these of the driver.
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetSecuritySeqNum(UINT16 index, UINT8 lsb, UINT8 ac, BOOL broadcast) {
	UINT16 * pAcSeqNum16;
	UINT32 * pAcSeqNum32;
	UINT8 old_lsb;
	unsigned int flags;
	
	/*
	printf("%s : staIndex : %d suite : %d AC : %d broadcast : %d\n", __FUNCTION__, index, 
				 stationParams[index].cipherSuiteType, ac, broadcast);
	*/
	if  (index > MAX_STATIONS && !broadcast)
		return;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags);

	if (broadcast) {
		pAcSeqNum16 = &BroadcastAcSeqNum16;
		pAcSeqNum32 = &BroadcastAcSeqNum32;
	} else {
		/* because we use only one sequence number for 
		  all AC's - use the BE sequence number */
		pAcSeqNum16 = &(stationParams[index].AcSeqNum16[0]); 
		pAcSeqNum32 = &(stationParams[index].AcSeqNum32[0]);
	}

	/* 16 lsb bits of seqNum are located in the AcSeqNum16 and 24 additional in the AcSeqNum32 */
	old_lsb = *pAcSeqNum16 & 0xFF;

	if ( (lsb > old_lsb) && ((lsb - old_lsb) < 127) ) 
	 {
		/* set new 8 lsbs to the AcSeqNum16 */
		*pAcSeqNum16 &= 0xFF00;
		*pAcSeqNum16 |= lsb;
	}

	/* check if 8 bit wrap around occurred */
	if ( (lsb < old_lsb) && ((old_lsb - lsb) > 127) ) 
	{
		/* progress the msb part of pAcSeqNum16 by 1 */		
		*pAcSeqNum16 += 0x100;
		/* check if 16 bit wrap around occurred */
		if(!(*pAcSeqNum16 & 0xFF00))
			*pAcSeqNum32 += 1;

		/* set new 8 lsbs to the AcSeqNum16 */
		*pAcSeqNum16 &= 0xFF00;
		*pAcSeqNum16 |= lsb;
	}

	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);

	
	//printf("%s : 32 upper : %d 16 lower : %d\n", __FUNCTION__, *pAcSeqNum32,  *pAcSeqNum16);
	
}
/************************************************************************
 *		wdrv_dbGetSecuritySeqNum  			                                  *
*************************************************************************
DESCRIPTION: Get station WME support status
                                                                                       
INPUT:      index, AC 

OUTPUT:     AcSeqNum16, AcSeqNum32 

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetSecuritySeqNum(UINT16 index,  UINT16 * AcSeqNum16, UINT32 * AcSeqNum32, BOOL broadcast) {
	
	unsigned int flags;

	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags);

	if (broadcast) {
		int i;
		
		for (i=0; i<WDRV_QOS_MAX_AC; ++i) {
			AcSeqNum16[i] = BroadcastAcSeqNum16;
			AcSeqNum32[i] = BroadcastAcSeqNum32;
		}
			
	} else {
		PAL_memCopy(AcSeqNum16, stationParams[index].AcSeqNum16, sizeof(stationParams[index].AcSeqNum16));
		PAL_memCopy(AcSeqNum32, stationParams[index].AcSeqNum32, sizeof(stationParams[index].AcSeqNum32));
	}

	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);
}
