/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*																		   */
/*		MODULE:	mpduCollec.c											   */
/*    	PURPOSE: Collection of the MSDUs for each MMPDU					   */
/*																		   */
/***************************************************************************/

#include "wlan_pal.h"
#include "rx.h"
#include "802_11Defs.h"
#include "memMngr.h"
#include "bssbridge.h"
#include "drvDbMain.h"

typedef struct {
	UINT32		seqNumber; 	/* The sequence number of the last MSDU received from the station  */
	UINT32		fragNumber;	/* The sequence number of the last fragment received from the station  */
	mem_BD_T	*lastPtr;  	/* Pointer to the last fragment in the collection.  */
    mem_MSDU_T  *msduPtr;    /* Pointer to the MSDU structure */
	BOOL		collect;  	/* A Boolean, set to TRUE if the station is in a middle of a fragment collection. */
	UINT32		timeStamp;  	/* A timestamp contains the receive time of the first fragment in the collection, if any. */
} CollectEntry_T;



CollectEntry_T* 	CollectTable;


UINT32  dupDetectTable[MAX_STATIONS][WDRV_QOS_MAX_AC];

static UINT32		mpduValidFrames = 0;
static UINT32		duplicateMsdu = 0;

/*************************************************************************
 *                        wdrv_rxDupMpduDetect                           *
 *************************************************************************
DESCRIPTION: This function performs detection of duplicated packets. In case
			 duch a packet detected, it is discarded. It gets the packet as
			 Mpdu and converts it to the Msdu.
                                                                                                   
INPUT:       pMpdu     - pointer to the packet.
			 staInd    - index of the station the packet received from.
			 mpduLen   - length of the packet.
                          
OUTPUT:		 msduPtr   - Pointer to the Msdu that will be distributed to the internal modules.


RETURN:      collect status, one of: MSDU_READY/MSDU_IN_PROGRESS/MPDU_DROP/MSDU_DROP
***********************************************************************/
collect_status_T wdrv_rxDupMpduDetect(mem_BD_T* pMpdu, UINT16 staInd,	UINT32 mpduLen, mem_MSDU_T **msduPtr) {
    dot11_header_T*	pHeader;
    UINT32          seqNumber;
    UINT32 prevSeqNumber;
	wdrv_QoSClassifierAC_t ac;
    *msduPtr = NULL;
    pHeader = (dot11_header_T*)(pMpdu->data);
	
    pHeader->seqCtrl = htons(pHeader->seqCtrl);

    seqNumber = ((((pHeader->seqCtrl & 0x00FF) << 4) & 0xFF0) | (((pHeader->seqCtrl & 0xF000) >> 12) & 0xF));

	ac = WDRV_QOS_CLASSIFIER_AC_BE;

    if (wdrv_dbGetWMEEnabled())
    {
        if ((((pHeader->fc & DOT11_FC_TYPESUBTYPE_MASK) == DOT11_FC_QOS_DATA) || 
             ((pHeader->fc & DOT11_FC_TYPESUBTYPE_MASK ) == DOT11_FC_QOS_NULL)))
        {
				dot11_QoSControlHeader_t * qosHeader;

            qosHeader = (dot11_QoSControlHeader_t *)((ULONG)pHeader + 
                         sizeof(dot11_AP_header_T) +
                         ((pHeader->fc & DOT11_FC_TO_DS) && (pHeader->fc & DOT11_FC_FROM_DS))*6);
            if (DOT11_QOS_UP(qosHeader) > WDRV_QOS_MAX_UP)
            {
                if (REPORT_LEVEL(RL_RX))
                {
						wdrv_utilsReport(RL_RX,PRI_EMERGENCY,
														 "%s: Error parsing QoS header !\n", __FUNCTION__);
					}
					return MPDU_DROP;
				}

				ac = wdrv_QoSAuxMapUPtoAC[DOT11_QOS_UP(qosHeader)];
					}
			}

		prevSeqNumber = dupDetectTable[staInd][ac];


   /*We should check retry bit */
   if ((seqNumber == prevSeqNumber) && (pHeader->fc & DOT11_FC_RETRY))
   {
      /* Duplicate MSDU - new MSDU with the same Sequence number as previous MSDU */
      wlan_memMngrFreeBD(pMpdu->handle);
      duplicateMsdu++;

      return MPDU_DROP;
   }
		
  /* New MSDU */
   mpduValidFrames++;
   wlan_memMngrAllocMSDUBufferOnly(msduPtr);
   if (*msduPtr == NULL)
   {
      if (REPORT_LEVEL(RL_RX))
      {
         wdrv_utilsReport(RL_RX,PRI_EMERGENCY,
                          "Rx: Can not allocate new MSDU buffer");
      }
      wlan_memMngrFreeBD(pMpdu->handle);
      return MPDU_DROP;
   }


   (*msduPtr)->headerLen = WLAN_HDR_LEN;
   (*msduPtr)->firstBDPtr = pMpdu;
   (*msduPtr)->dataLen = mpduLen;
   
   dupDetectTable[staInd][ac] = seqNumber;

   return MSDU_READY;
}

/*************************************************************************
 *                        fragmentCollectPrint                           *
 *************************************************************************
DESCRIPTION: This function prints mpdu collection statistics.

OUTPUT:

RETURN:  
************************************************************************/
void fragmentCollectPrint(void)
{
    printf("RX: Received Fragments - %d\n", mpduValidFrames);
    printf("RX: Duplicate MSDU - %d\n", duplicateMsdu);
}

/*************************************************************************
 *                        wdrv_rxGetCountersFrameDuplicateCount          *
 *************************************************************************
DESCRIPTION: This function returns number of frames that were found as
			 duplicated

OUTPUT:

RETURN:  
************************************************************************/
void wdrv_rxGetCountersFrameDuplicateCount(UINT32* count)
{
    *count = duplicateMsdu;
}

/*************************************************************************
 *                        wdrv_rxGetReceivedFragmentCount		         *
 *************************************************************************
DESCRIPTION: This function returns number of valid mpdus received by the module
OUTPUT:

RETURN:  
************************************************************************/
void wdrv_rxGetReceivedFragmentCount(UINT32* count)
{
    *count = mpduValidFrames;
}

