/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2003 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/****************************************************************************
 *
 *   MODULE:  whalAcxAccess.c
 *   PURPOSE: Support access to the ACX registers and memory
 *   
 ****************************************************************************/

#include "whalCommon.h"
#include "whalAcxAccess.h"
#include "whalAcxRegs.h"
#include "acx_hw.h"

/*
 * Registers access
 * ----------------
 * Implementation of the following functions for each type of WLAN bus and Endianess:
 *
 * whal_acxRegIsBitSet
 *
 * Note:
 * ACX100_REG_ADDRESS expands to function call that gets the ACX registers
 *  address, see whalCommon.h and whalApi.c
 */
 

/****************************************************************************
 *                         whal_acxRegIsBitSet ()                           *
 ****************************************************************************
 * DESCRIPTION: This function verifies the status of the specified bits in 
 *              ACX Device's register.
 * 
 * INPUTS:  
 *      UINT32  AcxReg          ACX Register's offset.
 *      UINT32  BitVal          Bitmask of bits to be verified.
 * 
 * OUTPUT:  None
 * 
 * RETURNS: 0 - If none of the specified bits are set.
 *          1 - If any of specified bits is set.
 ****************************************************************************/

int whal_acxRegIsBitSet(UINT32 AcxReg, UINT32 BitVal)
{
    register UINT32 RegVal;

    RegVal = whal_acxRegRead( AcxReg );
    if (RegVal & BitVal)
        return 1;
    return 0;
}

/****************************************************************************/








/*
 * Memory Access
 * -------------
 * Implementing the following functions for each type of WLAN bus and Endianess:
 * whal_acxMemCopy
 * whal_acxMemSetByteBits
 * whal_acxMemResetByteBits
 * whal_acxMemGetByte
 * whal_acxMemSetByte
 * 
 */






/***************** WLAN_BUS_TYPE_VLYNQ || WLAN_BUS_TYPE_PCI *************/
#if  ( (WLAN_BUS_TYPE == WLAN_BUS_TYPE_VLYNQ) \
    || (WLAN_BUS_TYPE == WLAN_BUS_TYPE_PCI))
    
/****************************************************************************
 *                           whal_acxMemCopy ()                             *
 ****************************************************************************
 * DESCRIPTION: Copy the memory block from ACX Device's memory
 *              to HOST and vice vesa.
 * 
 * INPUTS:  
 *      char *  Dest            Destination memory address.
 *      char *  Src             Source memory address.
 *      int     Len             Block length.
 * 
 * OUTPUT:  None
 * 
 * RETURNS: None
 ****************************************************************************/

void whal_acxMemCopy(char *Dest, char *Src, int Len)
{
    register volatile unsigned short *shortDest = (volatile unsigned short *)Dest;
    register volatile unsigned short *shortSrc = (unsigned short *)Src;
    register int Size = Len >> 1;
    unsigned short TempWord;

    while (Size--)
        *shortDest++ = *shortSrc++;

#if( USE_ENDIAN_SWAP == 1 )
    if (Len & 1)
    {
        TempWord = (*shortDest & 0x00ff) | (*shortSrc & 0xff00);
        *shortDest = TempWord;
    }
#else
    if (Len & 1)
    {
        TempWord = (*shortDest & 0xff00) | (*shortSrc & 0x00ff);
        *shortDest = TempWord;
    }
#endif
}

/****************************************************************************/



/****************************************************************************
 *                      whal_acxMemSetByteBits ()                           *
 ****************************************************************************
 * DESCRIPTION: Sets specified bits in byte
 * 
 * INPUTS:  
 *      UINT32  Addr            base address.
 *      int     ByteOffset      byte offset.
 *      UINT8   BitsVal         The bitmask to be set.
 * 
 * OUTPUT:  None
 * 
 * RETURNS: None
 ****************************************************************************/

void whal_acxMemSetByteBits(UINT32 Addr, int ByteOffset, UINT8 BitsVal)
{
   volatile UINT8 *ptr = (UINT8 *)(Addr + ByteOffset);
   *ptr |= BitsVal;
}

/****************************************************************************
 *                       whal_acxMemResetByteBits()                         *
 ****************************************************************************
 * DESCRIPTION: Clears specified bits in byte
 * 
 * INPUTS:  
 *      UINT32  Addr            base address.
 *      int     ByteOffset      byte offset.
 *      UINT8   BitsVal         The bitmask to be cleared.
 * 
 * OUTPUT:  None
 * 
 * RETURNS: None
 ****************************************************************************/

void whal_acxMemResetByteBits(UINT32 Addr, int ByteOffset, UINT8 BitsVal)
{
   volatile UINT8 *ptr = (UINT8 *)(Addr + ByteOffset);
   *ptr &= ~BitsVal;
}

/****************************************************************************
 *                          whal_acxMemGetByte()                            *
 ****************************************************************************
 * DESCRIPTION: The function returns the byte from specified address
 *              in ACX Device.
 * 
 * INPUTS:  
 *      UINT32  Addr            base address.
 *      int     ByteOffset      byte offset.
 * 
 * OUTPUT:  None
 * 
 * RETURNS:
 *      UINT8                   Requested byte content.
 ****************************************************************************/

UINT8 whal_acxMemGetByte(UINT32 Addr, int ByteOffset)
{
    volatile UINT8 *ptr = (UINT8 *)(Addr + ByteOffset);
    return (*ptr);
}

/****************************************************************************
 *                          whal_acxMemSetByte()                            *
 ****************************************************************************
 * DESCRIPTION: The function writes the byte to specified address
 *              in ACX Device.
 * 
 * INPUTS:  
 *      UINT32  Addr            base address.
 *      int     ByteOffset      byte offset.
 * 
 * OUTPUT:  None
 * 
 * RETURNS:
 *      UINT8                   Requested byte content.
 ****************************************************************************/

void whal_acxMemSetByte(UINT32 Addr, int ByteOffset, UINT8 Val)
{
    volatile UINT8 *ptr = (UINT8 *)(Addr + ByteOffset);
   *ptr = Val;
}


/***************** Other buses *************/
#else
#error *** WhalAcxAccess.c - WLAN Bus Memory access is not implemented for this bus type. ***
#endif /* WLAN_BUS_TYPE  */


