/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2001 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/*********************************************************************************
*                                                                                     
*   MODULE:  whalEEPromLess.c                                                                             
*   PURPOSE: EEProm Less operation 
*                                                                                                          
**********************************************************************************/
#include "wlan_pal.h"
#include "whalAcxRegs.h"
#include "whalAcxAccess.h"
#include "whalCommon.h"
#include "whalEEPromLess.h"
 

/*********************************/
/*			EEPROM LESS			 */
/*********************************/

static eepromImage_t eepromImage;
static BOOL eepromImageSet = FALSE;

STATUS whal_FwCtrl_Eepromless_SetImage(eepromImage_t *image) {
	if (image->len > MAX_EEPROM_LEN)
		return NOK;

	PAL_memCopy(eepromImage.image, image->image, image->len);
	eepromImage.len = image->len;

	eepromImageSet = TRUE;

	return OK;
}

/****************************************************************************
 *                      whal_FwCtrl_Eepromless_PrepareBoot()
 ****************************************************************************
 * DESCRIPTION:	prepare eepromless configuration before boot
 * 
 * INPUTS:  
 * 
 * OUTPUT:	
 * 
 * RETURNS:	
 ****************************************************************************/
UINT32 whal_FwCtrl_Eepromless_PrepareBoot( EepromlessCfg_T *pEepromlessCfg)
{
	pEepromlessCfg->SavedEEPROMlessInd = 0;
	pEepromlessCfg->NvsDataLen = 0;
	
	/*
	 * Do burst read from eeproless file and prepare the boot
	 */
	whal_hwCtrl_ProcessEepromBR( &pEepromlessCfg->tblsOffset);

	pEepromlessCfg->NvsDataLen = whal_hwCtrl_GetNvsDataSize(pEepromlessCfg->tblsOffset);

	/*  debug EEPROM parameters */
	printf("Get NVS file information: NvsDataLen = %#x tblsOffset %#x\n",
					   pEepromlessCfg->NvsDataLen, pEepromlessCfg->tblsOffset);


	/* save the index from SCR_PAD1 to restore after download */
	pEepromlessCfg->SavedEEPROMlessInd = whal_acxRegRead( ACX_EEPROMLESS_IND_REG); 

	/* SET EEPROMless mode to the FW (set SCR_PAD2 = 0) */
	whal_acxRegWrite( ACX_EEPROMLESS_IND_REG, HW_EEPROM_NOT_PRESENTED);
	/* set nvs data length to firmware */
	whal_acxRegWrite( ACX_EEPROMLESS_HAND_SHAKE_REG, pEepromlessCfg->NvsDataLen);
    
	
	/* Disable Watchdog and Rx/Tx */
	whal_acxRegWrite( WDOG_CTRL, 0x1);	/* disable WDOG */
	whal_acxRegWrite( ENABLE, 0x0);		/* disable TX,RX */
	
	/* Read the radio type and set in the data base */
	return whal_FwCtrl_Eepromless_GetRadioType();
}

/****************************************************************************
 *                      whal_FwCtrl_Eepromless_AfterBoot()
 ****************************************************************************
 * DESCRIPTION:	action after boot eepromless configuration before boot
 * 
 * INPUTS:  
 * 
 * OUTPUT:	 
 * 
 * RETURNS:	
 ****************************************************************************/
int whal_FwCtrl_Eepromless_AfterBoot( EepromlessCfg_T *pEepromlessCfg, UINT32 AcxMemAddr)
{
	UINT32 tblsOffset;

	/*
	 * Wait for CHANGE !!!!! (BUSY WAIT)
	 * stalling executiion !!!
	 */
	if (!whal_hwCtrl_WaitFwWhenEepromless((UINT32)pEepromlessCfg->NvsDataLen))
	{
		printf("whal_FwCtrl_Eepromless_AfterBoot: Timeout waiting for the Firmware in EEPROMless\n");

		return(NOK);
	}

	/*
	 * !!! DEBUG EEPROMLESS
	 */
	/* new eepromless support*/
	whal_hwCtrl_CopyNvsData(pEepromlessCfg->tblsOffset, pEepromlessCfg->NvsDataLen, AcxMemAddr);

/* sdio can take only 4 bytes at the time. by alcel*/
	tblsOffset = (UINT32)pEepromlessCfg->tblsOffset;

	/* Restore original value in SCR_PAD1 with tables offset*/
	whal_acxRegWrite(ACX_EEPROMLESS_HAND_SHAKE_REG, tblsOffset);

	/* Resume FW running */
	whal_acxRegWrite(ACX_EEPROMLESS_IND_REG, pEepromlessCfg->SavedEEPROMlessInd);

	return(OK);
}

/****************************************************************************
 *                      whal_FwCtrl_Eepromless_GetRadioType()
 ****************************************************************************
 * DESCRIPTION:	get the radio type from the external bin file
 * 
 * INPUTS:  
 * 
 * OUTPUT:	
 * 
 * RETURNS:	
 ****************************************************************************/
UINT32 whal_FwCtrl_Eepromless_GetRadioType(void)
{
	if (eepromImageSet)
		return (*(eepromImage.image + HW_EEPROM_CARD_TYPE_ADDR));
	else
		return 0xffffffff;
}


/****************************************************************************
 *                      whal_hwCtrl_ProcessEepromBR()
 ****************************************************************************
 * DESCRIPTION: Write EEPROM burst read section from map file to TNETW
 *				MAC registers.
 *
 * INPUTS:
 *
 * OUTPUT:	NONE
 *
 * RETURNS:	NONE
 *
 * NOTES:
 ****************************************************************************/
void whal_hwCtrl_ProcessEepromBR(UINT16 *Offset)
{
	UINT8* CurrEepromBufPtr   =   NULL;
	UINT16 CurrentAddressVal;
	UINT16 CisLocalPointer = 0;

	CurrEepromBufPtr = eepromImage.image;

	if (!eepromImageSet)
	{
		printf("whal_hwCtrl_ProcessEepromBR : Failed to get EEPROM image\n");

		return;
	}


	/*
	 * Run through BR section of eeprom till the last dummy entry
	 * with length field = 0
	*/
	do
	{

		CurrentAddressVal = whal_hwCtrl_ReadEepromU16value(CurrEepromBufPtr + EEPROM_BR_ENTRY_LEN);

#if( WLAN_ADAPTOR_VERSION == WLAN_ADAPTOR_VERSION_1150)
		/* 
		** the addresses in NVS file (Eepromless) for 1150 differs in the LSB bit between PHY
		** and MAC registers, for MAC the address LSB BIT is 1 (and for PHY its 0), hence in case
		** the LSB bit equal 1, we must decrement the address by 1 in order to get the correct
		** TNET1150 MAC address
		*/
		if (CurrentAddressVal & 0x1)
			CurrentAddressVal &= 0xFFFE ;
		else
		{
			
			/* in case write to Phy register - take example from Whal_Bus_sharedMemory !!!!!!!!!!!!!*/

			printf("whal_hwCtrl_ProcessEepromBR: write to Phy Registers !!!\n");
		}
#endif

		if (CurrentAddressVal == CIS_LADDR)
		{
			CisLocalPointer = whal_hwCtrl_ReadEepromU16value(CurrEepromBufPtr+
															 EEPROM_BR_ENTRY_DATA_START_OFFSET);
		}

		if (CurrentAddressVal != CisLocalPointer)
		{
			whal_hwCtrl_ProcessEepromBREntry(CurrEepromBufPtr);
		}

		CurrEepromBufPtr = CurrEepromBufPtr + EEPROM_BR_ENTRY_DATA_START_OFFSET
						   + ((*CurrEepromBufPtr) * EEPROM_BR_ENTRY_DATA_UNIT_LEN);


	}while (*CurrEepromBufPtr);


	/*
	 * At this point CurrEepromBufPtr points to dummy entry in EEPROM burst read.
	 * Then to get table pointers offset we need to bypass this entry.
	 *
	*/
	CurrEepromBufPtr = CurrEepromBufPtr + EEPROM_BR_ENTRY_DATA_START_OFFSET
					   + EEPROM_BR_ENTRY_DATA_UNIT_LEN;

	printf("whal_hwCtrl_ProcessEepromBR : CurrEepromBufPtr = %#x\n", *CurrEepromBufPtr);


	/* original pointers table offset */
	*Offset = CurrEepromBufPtr - eepromImage.image;



}

UINT32 whal_hwCtrl_Eepromless_ReadByte(UINT16 wAddr,UINT8 *pbVal)
{
	if (!eepromImageSet) 
	{
		printf("%s: failed to get EEPROM image\n",__FUNCTION__);
		return NOK;
	}
	if (wAddr > MAX_EEPROM_LEN - 1) 
	{
		printf("%s: invalid EEPROM address\n",__FUNCTION__);
		return NOK;
	}
		
	*pbVal = *(eepromImage.image + wAddr);
	return OK;
}

UINT16 whal_hwCtrl_ReadEepromU16value(UINT8  *pCurrLocation)
{
	UINT16 retValue;

	retValue = (*pCurrLocation) | (((*(pCurrLocation + 1)) << 8) & 0xFF00);
	return(retValue);
}


UINT32 whal_hwCtrl_ReadEepromU32value(UINT8  *pCurrLocation)
{
	UINT32 retValue;

	retValue =    (*pCurrLocation) |
				  (((*(pCurrLocation + 1)) << 8)  & 0x0000FF00) |
				  (((*(pCurrLocation + 2)) << 16) & 0x00FF0000) |
				  (((*(pCurrLocation + 3)) << 24) & 0xFF000000);

	return(retValue);
}


void whal_hwCtrl_ProcessEepromBREntry(UINT8 *pCurrEntry)
{
	UINT8     idx;
	UINT8     entryLength = *pCurrEntry;
	UINT16    CurrRegAddr;
	UINT32    CurrData;


	CurrRegAddr   = whal_hwCtrl_ReadEepromU16value(pCurrEntry + EEPROM_BR_ENTRY_LEN);

#if( WLAN_ADAPTOR_VERSION == WLAN_ADAPTOR_VERSION_1150)
	/* 
	** the addresses in NVS file (Eepromless) for 1150 differs in the LSB bit between PHY
	** and MAC registers, for MAC the address LSB BIT is 1 (and for PHY its 0), hence in case
	** the LSB bit equal 1, we must decrement the address by 1 in order to get the correct
	** TNET1150 MAC address
	*/
	if (CurrRegAddr & 0x1)
			CurrRegAddr &= 0xFFFE ;
	else
		{
			
			/* in case write to Phy register - take example from Whal_Bus_sharedMemory !!!!!!!!!!!!!*/

			printf("whal_hwCtrl_ProcessEepromBR: write to Phy Registers !!!\n");
		}
#endif

	for (idx=0;idx < entryLength; idx++)
	{

		CurrData  = whal_hwCtrl_ReadEepromU32value(pCurrEntry + EEPROM_BR_ENTRY_DATA_START_OFFSET
												   + (idx * EEPROM_BR_ENTRY_DATA_UNIT_LEN));


		whal_acxRegWrite(CurrRegAddr, CurrData);

		printf("whal_hwCtrl_ProcessEepromBREntry: Reg = %#x Data %#x\n",CurrRegAddr, CurrData);

		CurrRegAddr = CurrRegAddr + EEPROM_BR_ENTRY_DATA_UNIT_LEN;


	}


}

UINT16 whal_hwCtrl_GetNvsDataSize( UINT16 TblPtrOffset)
{
	UINT16  NvsDataLen;

	printf("whal_hwCtrl_GetNvsDataSize: eepromImage.len = 0x%x ",eepromImage.len);

	NvsDataLen = eepromImage.len - TblPtrOffset;

	/* nvs length must be aligned to upper 4-byte boundary */
	NvsDataLen = ((NvsDataLen / NVS_DATA_BUNDARY_ALIGNMENT) + 1) * NVS_DATA_BUNDARY_ALIGNMENT;

	return(NvsDataLen);

}


void whal_hwCtrl_CopyNvsData(UINT16 tblsOffset, UINT16 NvsDataLen, UINT32 AcxMemAddr)
{
	UINT8   *CurrEeepomPtr;
	UINT32  StartAddr;
	UINT32  NumWrittenChars = 0;
	INT32   byte_num        = 0;
	UINT32  temp            = 0;


	CurrEeepomPtr = eepromImage.image + tblsOffset;


	/*---------------------------------------------------*/
	/* read destination address in SRAM to copy nvs data */
	/*---------------------------------------------------*/
	StartAddr = whal_acxRegRead( ACX_EEPROMLESS_HAND_SHAKE_REG);
	printf ("whal_hwCtrl_CopyNvsData:  1 \n");

#ifdef DEBUG_EPROMLESS 

	printf ("------------------------------------------------------------------------\n");
	printf ("             EEPROM LESS:             BEFORE INIT\n");
	printf ("------------------------------------------------------------------------\n");

	HexDumpEep ((UINT8*)CurrEeepomPtr, 0xF4);
#endif

	/*
	  Download EEPROM data to ACX internal memory
	*/
	while ( NumWrittenChars < NvsDataLen )
	{
		temp |= ((*CurrEeepomPtr) << ( 8 * byte_num));
		CurrEeepomPtr++;
		NumWrittenChars++;
#if 0
		printf ("whal_hwCtrl_CopyNvsData NumWrittenChars=%d CurrEeepomPtr=0x%x\n",NumWrittenChars,temp);
#endif
		if ( ++byte_num > 3 )
		{
			temp = ENDIAN_HANDLE_LONG(temp);
#if 0
			printf ("whal_hwCtrl_CopyNvsData (copy to) NumWrittenChars=%d StartAddr=0x%x\n",NumWrittenChars,StartAddr);
#endif
			whal_acxMemCopy((char *)(StartAddr | AcxMemAddr), (char *)&temp, sizeof(temp));
			StartAddr += 4;
			temp = 0;
			byte_num = 0;
		}
	} 

#ifdef DEBUG_EPROMLESS 
	{
		UINT8 eepromBuf[EEPROM_LEN];

		StartAddr =   whal_acxRegRead( ACX_EEPROMLESS_HAND_SHAKE_REG);
	
		PAL_memSet(  eepromBuf, 0 ,sizeof(eepromBuf) );
	
		whal_acxMemCopy( &eepromBuf[0], (char *)(StartAddr | AcxMemAddr), sizeof(eepromBuf));
	
		printf ("------------------------------------------------------------------------\n");
		printf ("                           READ BACK FROM SRAM TO VERIFY\n");
		printf ("------------------------------------------------------------------------\n");
	
		printf ("*******************************************\n");
		printf ("SRAM ADDRESS for NVS DATA %#x\n",StartAddr);
	
		printf ("SCR_PAD1 = %#x  SCR_PAD2 = %#x\n",
						   whal_acxRegRead(SCR_PAD1),
						   whal_acxRegRead( SCR_PAD2));
		printf ("*******************************************\n");
	
		/* dump eeprom from SRAM */
		HexDumpEep ( (UINT8*)eepromBuf, sizeof(eepromBuf));
	}
#endif
}


BOOL whal_hwCtrl_WaitFwWhenEepromless( UINT32 WaitRegVal)
{
	int i;

	printf("whal_hwCtrl_WaitFwWhenEepromless: WaitRegVal = 0x%x\n",WaitRegVal);

	for (i=0; i < 2000; i++)
	{
		if (WaitRegVal == whal_acxRegRead(ACX_EEPROMLESS_HAND_SHAKE_REG))
		{
			PAL_sleepLong(50);
			/*printf("SCR_PAD7 = 0x%x (Add = 0x%x) \n",whal_acxRegRead(SCR_PAD7),SCR_PAD7);*/
		} else
		{
			printf("GOT HANDSHAKE FROM FIRMWARE !!!!	WaitRegVal = 0x%x \n",whal_acxRegRead(ACX_EEPROMLESS_HAND_SHAKE_REG));
			return(TRUE);
		}
	}

	return(FALSE);

}


#ifdef DEBUG_EPROMLESS 

#define DBG_BUF_LEN		256
char dbuf[DBG_BUF_LEN];

/* HEX DUMP Debug code only !!! */
void HexDumpEep ( UINT8 *data, int datalen)
{
	int j,dbuflen=0;
	static char hexdigits[16] = "0123456789ABCDEF";

	for (j=0; j < datalen;)
	{
		// Add a byte to the line
		dbuf[dbuflen] =  hexdigits[(data[j] >> 4)&0x0f];
		dbuf[dbuflen+1] = hexdigits[data[j] & 0x0f];
		dbuf[dbuflen+2] = ' ';
		dbuf[dbuflen+3] = '\0';
		dbuflen += 3;
		j++;
		if ((j % 16) == 0)
		{
			// Dump a line every 16 hex digits
			printf("%04.4x  %s\n", (j-16)+0x102, dbuf);

			dbuflen = 0;
		}
	}
	// Flush if something has left in the line
	if (dbuflen)
		printf("%04.4x  %s\n", j & 0xfff0, dbuf);
}
#endif




