/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2004 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/
/***************************************************************************
 *
 *   MODULE:  whalRxDesc.c
 *   PURPOSE: WLAN chip Rx descriptor object - functional implementation
 *   
 ***************************************************************************/

/****************************
 * HAL Include files
 ****************************/
#include "whalCommon.h"
#include "whal.h"
#include "AcxCommands.h"
#include "whalInterrupt.h"
#include "whalAcxAccess.h"
#include "whalRx.h"
#include "whalRxDesc.h"
#include "whalParams.h"

/***************************
 * Function Implementation
 ***************************/

/****************************************************************************
 *                      whal_rxDesc_Init()
 ****************************************************************************
 * DESCRIPTION:
 * Initializes WLAN chip Rx desctiptor. Actually. this function is empty
 * and provided for completeness only. Rx descriptors are initialized by
 * FW and no intervention is required by the driver.
 * 
 * INPUT :  None
 * OUTPUT:  RxDesc_T *pRxDesc = Pointer of descriptor to process
 *
 * RETURN:  None
 ****************************************************************************/
void whal_rxDesc_Init(RxDesc_T *pRxDesc)
{
}/* END whal_rxDesc_Init() */

/****************************************************************************
 *                      whal_rxDesc_GetAttr()
 ****************************************************************************
 * DESCRIPTION:
 * Copies requested attributes from WLAN chip Rx descriptor into output
 * pointers in the Host memory.
 * 
 * INPUT :  RxDesc_T *pRxDesc = Pointer to WLAN chip Rx descriptor
 * OUTPUT:
 *    UINT32   *MpduAddr   = Pointer to store address of host descriptor
 *    int      *MpduOffset = Pointer to store MPDU offset
 *    int      *MpduLen    = Pointer to store MPDU length
 *    RxAttr_T *pRxAttr    = Pointer to store Rx attributes
 * 
 * RETURNS: int: OK
 ****************************************************************************/
int whal_rxDesc_GetAttr(
   RxDesc_T *pRxDesc,
   UINT32   *MpduAddr,
   int      *MpduOffset,
   int      *MpduLen,
   RxAttr_T *pRxAttr)
{
   RxDesc_T localRxDesc;
   
   whal_rxDesc_Read( &localRxDesc, pRxDesc);
   
   /* MPDU is in SDRAM, returns pointer to host descriptor    */
   *MpduAddr   = (UINT32)localRxDesc.HostMemPtr + ACX100_MEM_ADDRESS;
   
   *MpduOffset     = whal_ParamsGetGenParams(0)->RxStatusHdrLen;
   *MpduLen        = localRxDesc.MPDULen;
   pRxAttr->Ctl    = localRxDesc.Ctl;
   pRxAttr->Rate   = localRxDesc.Rate;
   pRxAttr->Status = localRxDesc.Status;
   pRxAttr->SNR    = 0;
   pRxAttr->SIR    = 0;

   return OK;
}/* END whal_rxDesc_GetAttr() */

/****************************************************************************
 *                      whal_rxDesc_Read()
 ****************************************************************************
 * DESCRIPTION:
 * Copies Rx descriptor from WLAN chip into a buffer in Host's memory,
 * swapping relevant fields.
 * 
 * INPUT:   RxDesc_T *pSrcDesc  = Pointer of WLAN chip Rx descriptor
 * OUTPUT:  RxDesc_T *pDstDesc  = Pointer to destination buffer in the Host
 * 
 * RETURN:  int: OK
 ****************************************************************************/
int whal_rxDesc_Read(RxDesc_T *pDstDesc, RxDesc_T *pSrcDesc)
{
   whal_acxMemCopy( (char *)pDstDesc, (char *)pSrcDesc, sizeof(RxDesc_T));

   /* 
    * Convert only 16 and 32 bit fields 
    * (!!!) can copy only relevat fields
    */
   pDstDesc->next        = (UINT8 *)(ENDIAN_HANDLE_LONG((int)(pDstDesc->next)));
   pDstDesc->HostMemPtr  = (UINT8 *)(ENDIAN_HANDLE_LONG((int)(pDstDesc->HostMemPtr)));
   pDstDesc->ACXMemPtr   = (UINT32 *)(ENDIAN_HANDLE_LONG((int)(pDstDesc->ACXMemPtr)));
#if 0
   pDstDesc->time        = ENDIAN_HANDLE_LONG(pDstDesc->time);
#endif
   pDstDesc->MPDULen     = ENDIAN_HANDLE_WORD(pDstDesc->MPDULen);

   return OK;
}/* END whal_rxDesc_Read() */

/****************************************************************************
 *                      whal_rxDesc_GetCtl()
 ****************************************************************************
 * DESCRIPTION: Retrieves current value of "Ctl" field of the Rx descriptor.
 *
 * INPUT :  RxDesc_T *pRxDesc =  Pointer of WLAN chip Rx descriptor
 * OUTPUT:  None
 *
 * RETURN:  UINT8: "Ctl" field
 ****************************************************************************/
UINT8 whal_rxDesc_GetCtl(RxDesc_T *pRxDesc)
{
   return( whal_acxMemGetByte((UINT32)pRxDesc, RX_CTL_WORD_OFFSET));
}/* END whal_rxDesc_GetCtl() */

/****************************************************************************
 *                      whal_rxDesc_SetCtl()
 ****************************************************************************
 * DESCRIPTION:   Sets the "Ctl" field in a WLAN chip Rx descriptor.
 *
 * INPUT :  RxDesc_T *pRxDesc =  Pointer of WLAN chip Rx descriptor
 *          UINT8 Val         =  Value to set
 * OUTPUT:  None
 *
 * RETURN:  None
 ****************************************************************************/
void whal_rxDesc_SetCtl(RxDesc_T *pRxDesc, UINT8 Val)
{
   whal_acxMemSetByte( (UINT32)pRxDesc, RX_CTL_WORD_OFFSET, Val);
}/* END whal_rxDesc_SetCtl() */

/****************************************************************************
 *                      whal_rxDesc_GetHostNext()
 ****************************************************************************
 * DESCRIPTION:
 * Retrieves the pointer of the "Next" WLAN chip Rx descriptor in the ring
 * buffer maintained by the WLAN FW.
 *
 * INPUT :  RxDesc_T *pRxDesc =  Pointer of WLAN chip Rx descriptor
 * OUTPUT:  None
 *
 * RETURN:  RxDesc_T *: Pointer to next descriptor.
 ****************************************************************************/
RxDesc_T *whal_rxDesc_GetHostNext(RxDesc_T *pRxDesc)
{
   return( (RxDesc_T *)( whal_acxMemGetU32( &pRxDesc->next) + ACX100_MEM_ADDRESS));
}/* END whal_rxDesc_GetHostNext() */

/****************************************************************************
 *                      whal_rxDesc_SetHostNext()
 ****************************************************************************
 * DESCRIPTION:
 * Sets the host address in the "Next" WLAN chip Tx descriptor.
 *
 * INPUT :  RxDesc_T *pRxDesc = Pointer of current WLAN chip Rx descriptor
 *          UINT32 HostAddr   = Address of host descriptor.
 * OUTPUT:  None
 *
 * RETURN:  None
 ****************************************************************************/
void whal_rxDesc_SetHostNext(RxDesc_T *pRxDesc, UINT32 HostAddr)
{
   whal_acxMemSetU32( &pRxDesc->next, HostAddr - ACX100_MEM_ADDRESS);
}/* END whal_rxDesc_SetHostNext() */

/****************************************************************************
 *                      whal_rxDesc_GetAcxBufAddr()
 ****************************************************************************
 * DESCRIPTION: Returns the pointer of the frame data in the WLAN chip SRAM
 *
 * INPUT :  RxDesc_T pRxDesc = WLAN chip Rx descriptor
 * OUTPUT:  None
 *
 * RETURN:  UINT32 : Returned pointer value
 ****************************************************************************/
UINT32 whal_rxDesc_GetAcxBufAddr(RxDesc_T *pRxDesc)
{
   return(   whal_acxMemGetU32( &pRxDesc->ACXMemPtr)
           + whal_ParamsGetGenParams(0)->RxStatusHdrLen
           + ACX100_MEM_ADDRESS );
}/* END whal_rxDesc_GetAcxBufAddr() */

/****************************************************************************
 *                      whal_rxDesc_GetHostBufAddr()
 ****************************************************************************
 * DESCRIPTION: Returns the pointer to the frame data in the host memory
 *
 * INPUT :  RxDesc_T pRxDesc = WLAN chip Rx descriptor
 * OUTPUT:  None
 *
 * RETURN:  UINT32 : Pointer to frame data in the host memory.
 ****************************************************************************/
UINT32 whal_rxDesc_GetHostBufAddr(RxDesc_T *pRxDesc)
{
   return (   whal_acxMemGetU32( &pRxDesc->HostMemPtr)
            + whal_ParamsGetGenParams(0)->RxStatusHdrLen
            + ACX100_MEM_ADDRESS );
}/* END whal_rxDesc_GetHostBufAddr() */

/****************************************************************************
 *                      whal_rxDesc_Print()
 ****************************************************************************
 * DESCRIPTION:   Prints descriptor's main fields into a buffer.
 *
 * INPUT :  TxDesc_T pRxDesc = WLAN chip Rx descriptor
 * OUTPUT:  char *buf =  Pointer to output buffer, must be large enough.
 *
 * RETURN:  int : Total length of output string
 ****************************************************************************/
void whal_rxDesc_Print(RxDesc_T *pHwRxDesc)
{
   RxDesc_T localRxDesc;

   whal_rxDesc_Read( &localRxDesc, pHwRxDesc);

   printf("AcxMemPtr=0x%04x, Next=0x%04x, MpduLen=0x%04x, Ctl=0x%02x, Stt=0x%02x\n",
         (int)localRxDesc.ACXMemPtr, (int)localRxDesc.next, localRxDesc.MPDULen, localRxDesc.Ctl, localRxDesc.Status);
}/* END whal_rxDesc_Print() */

