/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2004 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/
/****************************************************************************
 *
 *   MODULE:  whalRxMaster.c
 *   PURPOSE: WLAN chip Rx Queue manager
 *
 ****************************************************************************/

/****************************
 * HAL Include files
 ****************************/
#include "acx_hw.h" 
#include "whalCommon.h"
#include "whal.h"
#include "whalRx.h"
#include "whalRxQueue.h"
#include "whalRxBufs.h"
#include "whalRxDesc.h"
#include "memMngr.h"
#include "whalInterrupt.h"
#include "whalAcxAccess.h"
#include "whalParams.h"
#include "whalTrace.h"
#include "whalHostQueue.h"
#include "whalRxMaster.h"
#include "dda.h"

/****************************
 * External Functions
 ****************************/
extern STATUS whal_HostQueue_watchNextToRead(HostQueue_Obj* this, HostDescAttr_T * pDscAttr);

/****************************
 * Local Data
 ****************************/
static HostQueue_Obj* pRxHostQueue = NULL;
/* halRestartSession counts each time HAL is restarted */
int halRestartSession = 0;
/* indicate the recovery  how many descriptors has read */
extern UINT32 rxQueueReadForRecovery;


/***************************
 * Function Implementation
 ***************************/

/****************************************************************************
 *                      whal_rxMasterInit()
 ****************************************************************************
 * DESCRIPTION:   Initialize Rx Queues in the WLAN chip and in the Host.
 *
 * INPUT :  None
 * OUTPUT:  None
 * 
 * RETURN:  int: OK or NOK
 ****************************************************************************/
STATUS whal_rxMasterInit()
{
	halRestartSession++;

#ifdef TIWLAN_DEBUG
	whal_trcInit();
#endif

   /* Init Rx queue ib ACX */
   pRxHostQueue = whal_HostQueue_init(
                     whal_ParamsGetDmaParams(0)->NumRxHostDesc, 
                     4096 /* whal_ParamsGetDmaParams()->RxBlockSize */
                     );
	if( pRxHostQueue == NULL)
		return NOK;

	return OK;
}/* END whal_rxMasterInit() */

 /****************************************************************************
 *                      whal_rxMasterUnload()
 ****************************************************************************
 * DESCRIPTION:   Unloads the Rx Host Queue.
 *
 * INPUT :  None  
 * OUTPUT:  None
 *
 * RETURN:  none
 ****************************************************************************/
void whal_rxMasterUnload()
{
	if (pRxHostQueue) {
	whal_HostQueue_unload(pRxHostQueue, 1);
		pRxHostQueue = NULL;
	}
}

/****************************************************************************
 *                      whal_rxMasterRecv()
 ****************************************************************************
 * DESCRIPTION:   Receives a buffer from the WLAN chip.
 * 
 * INPUT :  None  
 * OUTPUT:  INT32 *MpduBufLen = Returned MPDU length.
 *          RxAttr_T *pRxAttr = Storage of received attributes
 * 
 * RETURN:  MEM_HANDLE: Handle of the buffer descriptor (BD). NULL if fail.
 ****************************************************************************/
MEM_HANDLE whal_rxMasterRecv(INT32 *MpduBufLen, RxAttr_T *pRxAttr)
{
   register UINT32	 mpduLen, infoLen;
   mem_BD_T *pBd;
   HostDescAttr_T pDscAttr;
   WlanRxStatusInfo_T *rxStatusInfo;   
   UINT8 *newBuffer;
   UINT32 newBufToken;
   UINT32 offset;
  
   /*--------------------------------------------------------------------------
   Phase 1 : Watch Host Descriptor and read it's status.
   --------------------------------------------------------------------------*/
   if( whal_HostQueue_watchNextToRead(pRxHostQueue, &pDscAttr) == NOK )
       return NULL;
   
   /* 
   * Descriptor does not belong to the HOST, this can happen. 
   */
   if( !(pDscAttr.status & ACX_HOST_DESC_STAT_OWN ) )
   {
      return NULL;
   }

   /* Before getting the next descriptor, try to allocate the buffer
      for the next reception, store it in the blk_offset field */
	if(tiwlan_ddaRxAllocBufferIfNeeded(pRxHostQueue->BlockSize,
									&newBufToken, &newBuffer, TIWLAN_ALLOC_RUNTIME_PHASE) == OK)
	{
		if(!newBuffer)
		{
			if(REPORT_LEVEL(RL_HAL))
			   wdrv_utilsReport(RL_HAL, PRI_WARNING,"whal_rxMasterRecv: Failed to allocate the new buffer\n");     
			return NULL;
		}
	}
   else {
      /* for VxWorks, new buffer is the same as the previous */
		newBufToken = pDscAttr.bufToken;
   }
   /* get and advance the descriptor ptr */
   if( whal_HostQueue_getNext(pRxHostQueue, &pDscAttr) == NOK )
   {
	   tiwlan_ddaRxFreeBuffer(newBufToken);
       return NULL;
   }
   TIAP_TRACE_DRV_PKT_RX_ENTER(pDscAttr.bufToken);
   rxQueueReadForRecovery++;

   /* 
   * DMA error - save statistics and return after recycling the buffer.
   */
   if( pDscAttr.status & (ACX_HOST_DESC_STAT_ERR_MASK | ACX_HOST_DESC_STAT_DMA_ERR))
   {
      if( pDscAttr.status & ACX_HOST_DESC_STAT_TAR_ABORT )
         pRxHostQueue->statistics.targetAborts++;
      
      if( pDscAttr.status & ACX_HOST_DESC_STAT_MASTER_ABORT )
         pRxHostQueue->statistics.masterAborts++;
      
      if( pDscAttr.status & ACX_HOST_DESC_STAT_REQ_BEF_COMP )
         pRxHostQueue->statistics.reqBeforeComplete++;
      
      if( pDscAttr.status & ACX_HOST_DESC_STAT_UNEXP_HOST )
         pRxHostQueue->statistics.unexpectHostError++;
      
      if( pDscAttr.status & ACX_HOST_DESC_STAT_UNEXP_ACX )
         pRxHostQueue->statistics.unexpectAcxError++;
      
      if( pDscAttr.status & ACX_HOST_DESC_STAT_PCI_DISABLED )
         pRxHostQueue->statistics.pciNotAbled++;
      
      /*
       *  Status is not OK, reuse the buffer by clearing it to
       *  the next place availiable. 
      */
      pDscAttr.ctrl = 0;
      pDscAttr.status = 0;
	  tiwlan_ddaRxFreeBuffer(newBufToken);
      whal_HostQueue_insert( pRxHostQueue, &pDscAttr);
	  if(REPORT_LEVEL(RL_HAL))
         wdrv_utilsReport(RL_HAL, PRI_WARNING,"Error in the Host descriptor :: "
                          "Status = 0x%8x \n", pDscAttr.status );
      TIAP_TRACE_DRV_PKT_RX_EXIT(pDscAttr.bufToken,TIAP_TRACE_RX_RESULT_DMA_ERROR);
      return NULL;
   }
   
   if ( wlan_memMngrAllocBDs( 1, &pBd) != OK )
   {
	   tiwlan_ddaRxFreeBuffer(newBufToken);
	   pDscAttr.ctrl = 0;
	   pDscAttr.status = 0;
	   whal_HostQueue_insert( pRxHostQueue, &pDscAttr);
	   if(REPORT_LEVEL(RL_MEM))
		   wdrv_utilsReport(RL_HAL, PRI_WARNING,"whal_rxMasterRecv: wlan_memMngrAllocBDs failed");
       TIAP_TRACE_DRV_PKT_RX_EXIT(pDscAttr.bufToken,TIAP_TRACE_RX_RESULT_ALLOC_BD_FAIL);
       return NULL;
   }

   
   /*--------------------------------------------------------------------------
   Phase 3 :
   All descriptors are OK ! Get the Host descriptore prepare MPDU
   BD for the driver.
   --------------------------------------------------------------------------*/

    /*
      31----------------------------------------0
                  Control | MemBlkPtr             0x00
        ----------------------------------------                      
                     RxStatusInfo                 0x04
                     
        ----------------------------------------
                      PLCPHeader                  0x10 
                      
        ----------------------------------------
                         Body                     0x18                      
                      
                      
        ----------------------------------------  0xnn              
     */                                              
   
   /* offset calculation:
      - status & ACX_HOST_DESC_STAT_ALIGNMENT mask nullifies other bits.
	  - Shift right ACX_HOST_DESC_STAT_ALIGN_BIT_LOCATION shifts the bit to the 0 location
	  - Shift left 1 multiplies with 2 to get the offset, which possible values are 0 or 2. */
   offset = ((pDscAttr.status & ACX_HOST_DESC_STAT_ALIGNMENT) >> (ACX_HOST_DESC_STAT_ALIGN_BIT_LOCATION)) <<1;

   pDscAttr.blk_ptr += offset;
   
   rxStatusInfo = (WlanRxStatusInfo_T*) pDscAttr.blk_ptr; 
  
   PAL_cacheInvalidate(0, rxStatusInfo,/* virtual address */      
				       sizeof(WlanRxStatusInfo_T) + sizeof(RxPLCPHeader_T)  );    /* number of bytes to invalidate */


   /*newPlcp*/
   {
	   RxPLCPHeader_T *rxPlcpHeader;
	   rxPlcpHeader = (RxPLCPHeader_T *)(pDscAttr.blk_ptr + sizeof(WlanRxStatusInfo_T));
	   pRxAttr->Mic =  (rxPlcpHeader->status &  PLCP_MIC_FAILURE_STATUS  ) ? NOK : OK ;
   }

   /* our data length is only from mac header without plcp header */
   pRxAttr->woneIndex = (ENDIAN_HANDLE_WORD( rxStatusInfo->length ) & 0xC000)>>14;

   if(pRxAttr->woneIndex >= 2)
   {   
	   printf("whalRxMaster: woneIndex=%d !!!!!!!!!!!!!!!!!!!!!!!!\n",pRxAttr->woneIndex);
	   tiwlan_ddaRxFreeBuffer(newBufToken);
	   pDscAttr.ctrl = 0;
	   pDscAttr.status = 0;
	   pDscAttr.blk_ptr -= offset;
	   whal_HostQueue_insert( pRxHostQueue, &pDscAttr);
	   if (wlan_memMngrFreeBD(pBd->handle) != TRUE)
		   wdrv_utilsReport(RL_HAL, PRI_EMERGENCY,"whal_rxMasterRecv: wlan_memMngrFreeBD failed");
       TIAP_TRACE_DRV_PKT_RX_EXIT(pDscAttr.bufToken,TIAP_TRACE_RX_RESULT_NOK);
	   return NULL;
   }

   infoLen = (ENDIAN_HANDLE_WORD( rxStatusInfo->length ) & 0x0fff);
   /* W/A we can't pass any packet of zero length up to the wlanEnd */
   if(infoLen <= sizeof(RxPLCPHeader_T))
   {
       wdrv_utilsReport(RL_HAL, 
                        PRI_WARNING,
                        "whal_rxRecv: Error: %d bytes long packet received\n",
                        ENDIAN_HANDLE_WORD( rxStatusInfo->length ));
	  tiwlan_ddaRxFreeBuffer(newBufToken);
      pDscAttr.ctrl = 0;
      pDscAttr.status = 0;
	  pDscAttr.blk_ptr -= offset;
      whal_HostQueue_insert( pRxHostQueue, &pDscAttr);
      if (wlan_memMngrFreeBD(pBd->handle) != TRUE)
          wdrv_utilsReport(RL_HAL, PRI_EMERGENCY,"whal_rxMasterRecv: wlan_memMngrFreeBD failed");
      TIAP_TRACE_DRV_PKT_RX_EXIT(pDscAttr.bufToken,TIAP_TRACE_RX_RESULT_NOK);
      return NULL;
   }
   mpduLen = infoLen - sizeof(RxPLCPHeader_T);

   /*
    * Wrap the the BD for the Rx task :
    * Set the offset to the MAC header, after PLCP header and Status info.
    */
   /* newPlcp*/
   pBd->data       = (char*)((UINT32)pDscAttr.blk_ptr + sizeof(WlanRxStatusInfo_T) + sizeof(RxPLCPHeader_T) );
   pBd->dataOffset = 0;
   pBd->dataBuf    = NULL;  /* This indicates the memMngr to not return buffer to mem manager pool */

   pBd->length     = mpduLen;
   *MpduBufLen     = mpduLen;

   /*
    * Read the Rx info and keep it in the tracer.
    */
	pRxAttr->Status = rxStatusInfo->mod_mode;
    /* Save RX level in RX attributes */
    pRxAttr->rx_level = rxStatusInfo->rx_level;

   PAL_cacheInvalidate(0,
      pBd->data,    /* virtual address */
      mpduLen );    /* number of bytes to invalidate */

   

#ifdef TIWLAN_DEBUG
   if (whal_trcIsEnable())
       whal_trcAddRx(rxStatusInfo, "SUCC");
#endif


   /* 
    * swaping the frame control - Needed to be modified for little endian systems.
    */
   *((UINT16*)(pBd->data)) = ENDIAN_HANDLE_WORD((*((UINT16*)(pBd->data))));
   
   /* move the block pointer back */
   pDscAttr.blk_ptr -= offset;

   /* free function and args for host reclaiming the whole buffer */
   pBd->freeFunc =  (void*)whal_rxMasterFreeBuffer;

   pBd->freeArgs[0] =  (UINT32)pRxHostQueue;
   pBd->freeArgs[1] =  pDscAttr.blk_ptr;
   pBd->freeArgs[2] =  pDscAttr.blk_len;
   pBd->freeArgs[3] =  halRestartSession;
   pBd->freeArgs[4] =  newBufToken;
   pBd->bufToken	=  pDscAttr.bufToken;

   return (MEM_HANDLE)pBd; 
}/* END whal_rxMasterRecv() */


/****************************************************************************
 *                      whal_rxMasterFreeBuffer()
 ****************************************************************************
 * DESCRIPTION:   Frees a busy buffer to a Rx Queue. 
 * 
 * INPUT :  mem_BD_T *pBd = Pointer to memory BD
 * OUTPUT:  None
 * 
 * RETURNS: none.
 ****************************************************************************/
void whal_rxMasterFreeBuffer( mem_BD_T *pBd )
{
    HostQueue_Obj   *pRxHostQ;
    HostDescAttr_T  pDscAttr;
    unsigned int    flags;
	STATUS			status;


	if (halRestartSession == pBd->freeArgs[3])
	{
		/* this special case is for deconcatenated packets coming from rx
		   via Bss bridge to the tx - OS buffer should be freed, but no
		   new buffer should be attached to the RX ring;
		   freeArgs[4] = 0  indicates that the BD comes from rx, 
		   if it comes from tx(after BSS bridge), the DMA should not get another buffer instead */
		if(pBd->freeArgs[4])
		{
			pRxHostQ =  (HostQueue_Obj*) pBd->freeArgs[0];		
		
			pDscAttr.bufToken = pBd->freeArgs[4];
			pDscAttr.blk_ptr = (UINT32) TIWLAN_DDA_BUFFER_FROM_TOKEN(pDscAttr.bufToken);
			pDscAttr.blk_len =  (UINT16) pRxHostQ->BlockSize;
			pDscAttr.blk_offset = 0;
			pDscAttr.ctrl = 0;
			pDscAttr.status = 0;
		
			PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags);
			status = whal_HostQueue_insert(pRxHostQ, &pDscAttr);
			PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags); 
			
			if( status != OK ){
				puts("whal_rxMasterFreeBuffer() -- Host queue is full !");
			}
		
			if(pBd->bufToken != 0)
			{
				pBd->dataBuf = NULL;
				tiwlan_ddaRxFreeBuffer(pBd->bufToken);
			}
		}
		else if(pBd->bufToken)
		{
			 tiwlan_ddaRxFreeBuffer(pBd->bufToken);
			 return;
		}
	}
}/* END whal_rxMasterFreeBuffer() */


/****************************************************************************
 * DESCRIPTION:   Print Rx queue/bufs info into buffer. Buffer must be large
 *                enough to hold the exepected output.
 *
 * INPUT :  None
 * OUTPUT:  char *buf
 * 
 * RETURN:  Output length. 
 ****************************************************************************/
int whal_rxMasterInfoPrint(char *buf)
{
    return whal_HostQueue_show( pRxHostQueue ,buf);
}/* END whal_rxMasterInfoPrint() */


/****************************************************************************
 *                      whal_rxMaster_getHostQueueAddr()
 ****************************************************************************
 * DESCRIPTION: Gets the address of Host queue
 * 
 * INPUT :  None
 * OUTPUT:  None
 * 
 * RETURNS: UINT32: Host queue address
 ****************************************************************************/
UINT32 whal_rxMaster_getHostQueueAddr( void )
{
	return  whal_HostQueue_getFirstDescAddr( pRxHostQueue);
}/* END whal_rxMaster_getHostQueueAddr() */


/****************************************************************************
 *                      whal_rxMaster_getQueueInfo()
 ****************************************************************************
 * DESCRIPTION: Inquires Rx queue for up to date information
 * 
 * INPUT :  None
 * OUTPUT:
 *    UINT32 *RxDescAddr =  Rx Host descriptor address
 *    UINT32 *HostOwn    =  Descriptor status
 *    UINT32 *OwnerPercentage = Precentage of owned descriptors
 * 
 * RETURNS: UINT32 : OK or NOK.
 ****************************************************************************/
UINT32 whal_rxMaster_getQueueInfo( UINT32 *RxDescAddr, UINT32 *HostOwn, UINT32 *OwnerPercentage )
{
	HostDesc_T *pDesc;
	UINT32 NumOwnedDesc;
	UINT32 i;
    UINT32 maxCurr;

   /* Get the descriptor address */
   *RxDescAddr = (UINT32)pRxHostQueue->NextToRead;

   /* Get the Ownership */
   *HostOwn = ((ENDIAN_HANDLE_LONG( pRxHostQueue->NextToRead->status))
      & ACX_HOST_DESC_STAT_OWN);

   /* Get the percentage of entries with host ownership */
   maxCurr = pRxHostQueue->MaxCurrNumDescInQ;
   pDesc   = pRxHostQueue->FirstDesc;
   NumOwnedDesc = 0;
   for( i = 0; i < maxCurr; ++i)
   {
      if( ENDIAN_HANDLE_LONG( pDesc->status) & ACX_HOST_DESC_STAT_OWN) {
         ++NumOwnedDesc;
      }
      pDesc = whal_HostDesc_getNext( pDesc);
   }
   
   *OwnerPercentage = (NumOwnedDesc * 100)/(maxCurr ? maxCurr:1);

   return OK;
}/* END whal_rxMaster_getQueueInfo() */

