/**********************************************************************************************************************
  FILENAME:       tx_memory_calculation_driver.c

  DESCRIPTION:    Sets the correct fragmentation parameters

  Copyright (c) 2004 Texas Instruments Inc. 
  All Rights Reserved This program is the confidential and proprietary 
  product of Texas Instruments Inc.  Any Unauthorized use, reproduction or 
  transfer of this program is strictly prohibited.
***********************************************************************************************************************/
#include "whalTxMemCalc.h"
#include "public_rates.h"
#include "802_11Defs.h"

FragmentationData_t fragmentationData;  // C100 avishai should it be STATIC

extern UINT16 gMblkSize;
extern UINT32 dataFlowOptions;

const UINT8 TxMemCalcAckDurationTable[] =
{
    112,    // 1 Mbps 
    56,     // 2 Mbps
    20,     // 5.5 Mbps
    19,     // 6 Mbps
    19,     // 9 Mbps
    10,     // 11 Mbps
    12,     // 12 Mbps
    9,      // 18 Mbps
    10,     // 22 Mbps
    4,      // 24 Mbps
    4,      // 36 Mbps
    4,      // 48 Mbps, standard says ack at 36 Mbps
    4       // 54 Mbps, standard says ack at 36 Mbps
};

const UINT8 TxMemCalcRateValueTable[] =
{
    1,      // 1 Mbps 
    2,      // 2 Mbps
    55,     // 5.5 Mbps
    6,      // 6 Mbps
    9,      // 9 Mbps
    11,     // 11 Mbps
    12,     // 12 Mbps
    18,     // 18 Mbps
    22,     // 22 Mbps
    24,     // 24 Mbps
    36,     // 36 Mbps
    48,     // 48 Mbps
    54      // 54 Mbps
};

/**********************************************************************************************************************
   CalculateMargins

   DESCRIPTION:  Calculates the total header size needed for each fragment (after encryption if needed)

   PARAMETERS:   CipherSuite_e encryptionType - specifies the type of encryption for the frame
                                                valid values:
                                                      CIPHER_NONE = 0,
                                                      CIPHER_WEP  = 1,
                                                      CIPHER_TKIP = 2,
                                                      CIPHER_AES  = 3
                 UINT8 mpduHdrLength          - the header length of the original frame

   RETURNS:      Nothing (sets the header size and the aes overhead in the global structure)
***********************************************************************************************************************/
void CalculateMargins( UINT8 encryptionType )
{
    fragmentationData.wpa_MPDU_addition = 0;
    fragmentationData.wpa_MSDU_addition = 0;
    fragmentationData.blk_size_addition = 0;
    fragmentationData.driverPaddingSize = 0;

    if (encryptionType == CIPHER_WEP)
    {
        fragmentationData.wpa_MPDU_addition =   TX_MEM_CALC_WEP_TX_IV_SIZE +
                                                TX_MEM_CALC_WEP_TX_ICV_SIZE;
        fragmentationData.wpa_MSDU_addition = 0;
        fragmentationData.blk_size_addition = 0;
        fragmentationData.driverPaddingSize = 0;
    }
    else
    if (encryptionType == CIPHER_TKIP)
    {
    // If TKIP encryption, include the 4 byte  IV pad field set up by
    // the host as part of the header for each fragment and 
        fragmentationData.wpa_MPDU_addition =   TX_MEM_CALC_WEP_TX_IV_SIZE +
                                                TX_MEM_CALC_TKIP_TX_EIV_SIZE +
                                                TX_MEM_CALC_WEP_TX_ICV_SIZE;
        fragmentationData.wpa_MSDU_addition =   TX_MEM_CALC_WPA_MIC_LEN;
        fragmentationData.blk_size_addition =   TX_MEM_CALC_TKIP_TX_EIV_SIZE;
        fragmentationData.driverPaddingSize =   TX_MEM_CALC_TKIP_TX_EIV_SIZE;
    }
    else
    if (encryptionType == CIPHER_CCMP)
    {
    // If AES encryption, include the 8 bytes pad field set up by
    // the host as part of the header for each fragment (RSN_HEADER_LEN) and the 8 byte 
    // after the header before the data (AES_MIC_LEN)

        fragmentationData.wpa_MPDU_addition =   TX_MEM_CALC_RSN_HEADER_LEN +
                                                TX_MEM_CALC_WPA_MIC_LEN;
        fragmentationData.wpa_MSDU_addition =   0;
        fragmentationData.blk_size_addition =   TX_MEM_CALC_RSN_HEADER_LEN +
                                                TX_MEM_CALC_WPA_MIC_LEN;
        fragmentationData.driverPaddingSize =   TX_MEM_CALC_RSN_HEADER_LEN;
    }
}


/**********************************************************************************************************************
   CalculateFragmentationThreshold

   DESCRIPTION:  Calculates the fag threshold per frame according to the frag threshold defined by
                 the user and the TxOp fragmentation (if WME is used).

   PARAMETERS:   UINT16 originalFragmentThreshold - the user specified fragmentation threshold
                 UINT16 txOp                      - the txOpLimit for the specific AC ( will equal ZERO if not WME)
                 UINT32 txRate                    - the rate field as it is put in the TxDesc
                 UINT8  txNoAck                   - "1" if we are working in "NO_ACK" mode "0" otherwise
                 UINT8  txCts                     - "1" if the frame is sent with CTS protection "0" otherwise  
                 UINT8  txRts                     - "1" if RTS/CTS  is sent for the frame
                 UINT8  tx4X                      - "1" if the frame is sent with 4X on "0" otherwise  
   
   RETURNS:      Nothing (Sets the fragThresh in the global structure)
***********************************************************************************************************************/
void CalculateFragmentationThreshold(FragmentationDecisionData_t* fragDecisionData)
{
    UINT16 fragThreshExtra;
    UINT16 txOpFragThresh;

    fragThreshExtra = fragDecisionData->mpduHeaderLength + 
                      TX_MEM_CALC_FCS_LENGTH;

    if (fragDecisionData->tx4X == 0)
    {     
        if (fragDecisionData->txOpLimit == 0)
        {
            fragmentationData.fragThreshold = fragDecisionData->originalFragThreshold - fragThreshExtra;
        }
        else
        {
            txOpFragThresh = GetTxOpFragThreshold(fragDecisionData);

            if (txOpFragThresh >= TX_MEM_CALC_MIN_FRAG_THRESH)
            {
                fragmentationData.fragThreshold = txOpFragThresh - fragThreshExtra;
            }
            else
            {
                fragmentationData.fragThreshold = TX_MEM_CALC_MIN_FRAG_THRESH - fragThreshExtra;
            }
        }
    }
    else
    {
        fragmentationData.fragThreshold = TX_MEM_CALC_4X_FRAG_THRESHOLD - fragThreshExtra;
    }

    if (( fragDecisionData->unicastFrame == FALSE ) ||
        ( fragDecisionData->driverFragmentationOn == TRUE ))
    {
        fragmentationData.fragThreshold = TX_MEM_CALC_4X_FRAG_THRESHOLD - fragThreshExtra;
    }

}

/**********************************************************************************************************************
   GetTxOpFragThreshold

   DESCRIPTION:  Calculates the fragmentation threshold caused by the TxOpLimit.

   PARAMETERS:   UINT16 txOp                      - the txOpLimit for the specific AC ( will equal ZERO if not WME)
                 UINT32 txRate                    - the rate field as it is put in the TxDesc
                 UINT8  txNoAck                   - "1" if we are working in "NO_ACK" mode "0" otherwise  
                 UINT8  txCts                     - "1" if we are working with CTS protection "0" otherwise  
                 UINT8  txRts                     - "1" if RTS/CTS  is sent for the frame

   RETURNS:      The fragmentation threshold calculated
***********************************************************************************************************************/
UINT16 GetTxOpFragThreshold( FragmentationDecisionData_t* fragDecisionData )
{
    UINT8         divFactor, tmpAckDuration;
    UINT16        fragDataTime, txOpFragThresh, rate , txNoAckFragDataTimeChange = 0, txCtsFragDataTimeChange, txRtsFragDataTimeChange;
    TxMemCalcRateIndex_e rateIndex    = TX_MEM_CALC_RATE_INDEX_MAX;
    TxMemCalcRateIndex_e rateCtlIndex = TX_MEM_CALC_RATE_INDEX_11MBPS;
    
    
// endian is not a factor since the rate will be received as little endian (like the descriptor)   
    while (((1<<rateIndex)    & fragDecisionData->txRate    ) == 0) rateIndex--;
    while (((1<<rateCtlIndex) & fragDecisionData->txCtlRate ) == 0) rateCtlIndex--;
    
    rate = TxMemCalcRateValueTable[rateIndex];

    if (rate == 55) 
    {
        divFactor = 80;
    }
    else
    {
        divFactor = 8;
    }

    fragDataTime = fragDecisionData->txOpLimit;

    if (fragDecisionData->txNoAck == 1)
    {
        txNoAckFragDataTimeChange = 0;
    }
    else
    {
        tmpAckDuration = TxMemCalcAckDurationTable[rateIndex];
        if (fragDecisionData->txOpLimit > TX_MEM_CALC_SIFS_DURATION_FOR_FRAG + tmpAckDuration)
        {
            txNoAckFragDataTimeChange = TX_MEM_CALC_SIFS_DURATION_FOR_FRAG + tmpAckDuration;
        }
        else
        {
            // There is a problem, this should NOT happen (causes the frag threshold to be 256B)
            rate = 0;
        }
    }

    if (fragDecisionData->txCts == 0)
    {
        txCtsFragDataTimeChange = 0;
    }
    else
    {
        txCtsFragDataTimeChange = TxMemCalcGetCtsDuration(fragDecisionData->txRate,rateCtlIndex);
    }

    if (fragDecisionData->txRts == 0)
    {
        txRtsFragDataTimeChange = 0;
    }
    else
    {
        txRtsFragDataTimeChange = TX_MEM_CALC_RTS_FRAG_DATA_TIME;
    }

    if (fragDecisionData->txOpLimit > (txCtsFragDataTimeChange + txNoAckFragDataTimeChange + txRtsFragDataTimeChange))
    {
        fragDataTime = fragDataTime - txCtsFragDataTimeChange - txNoAckFragDataTimeChange - txRtsFragDataTimeChange;
    }
    else
    {
        // There is a problem, this should NOT happen (causes the frag threshold to be 256B)
        fragDataTime = 0;
    }
    txOpFragThresh = fragDataTime*rate/divFactor;
    
    return (txOpFragThresh);
}



/**********************************************************************************************************************
   NumMemBlocksTxAllocation

   DESCRIPTION: Utility function: finds the correct fragmentation threshold and the number of memory blocks to allocate   

   PARAMETERS: UINT16 originalFragmentThreshold - the user specified fragmentation threshold
               UINT16 txOp                      - the txOpLimit for the specific AC ( will equal ZERO if not WME)
               UINT32 txRate                    - the rate field as it is put in the TxDesc
               UINT8  txNoAck                   - "1" if we are working in "NO_ACK" mode "0" otherwise
               UINT8  unicastFrame              - "0" if the frame is Bcast / Multicast "1" if unicast
               UINT8  driverFragmentationOn        - bit_0 of the dataFlowOptions IE specifies whether driver encryption is used
               UINT8  txCts                     - bit_0 of the ctrl2 field "1" if there is CTS protection on the frame
               UINT8  txRts                     - "1" if RTS/CTS  is sent for the frame
               UINT16 dataLength                - the length of the data in the frame
               UINT16 memBlkSize                - the size of a memory blocks in bytes
               UINT8  mpduHeaderLength          - the header length of the original frame
               UINT8  tx4X                      - "1" if the frame is sent with 4X on "0" otherwise  
               CipherSuite_e encryptionType     - specifies the type of encryption for the frame
                                                  valid values:
                                                      CIPHER_NONE = 0,
                                                      CIPHER_WEP  = 1,
                                                      CIPHER_TKIP = 2,
                                                      CIPHER_AES  = 3

                UINT8* fragmentedFrame           - return value whether or not the frame will be fragmented
                UINT8* totalNumBlocksToAllocate  - return value number of blocks to allocate
                UINT16* finalFragThreshold       - return value the frag threshold used for the calculatio

 
    RETURNS:      NOTHING
***********************************************************************************************************************/
void NumMemBlocksTxAllocation(UINT8* fragmentedFrame, UINT8* totalNumBlocksToAllocate, UINT16* finalFragThreshold, FragmentationDecisionData_t* fragDecisionData)
{   
    UINT16 memBlocks;
    UINT16 numMemBlocksPerFullFrag;
    UINT16 payloadDataLen;

    CalculateMargins ( fragDecisionData->encryptionType );

    CalculateFragmentationThreshold( fragDecisionData );
    *finalFragThreshold = fragmentationData.fragThreshold;


    payloadDataLen  = fragDecisionData->dataLength - 
                      fragDecisionData->mpduHeaderLength -
                      fragmentationData.driverPaddingSize +
                      fragmentationData.wpa_MSDU_addition;
    
    numMemBlocksPerFullFrag = ((fragmentationData.fragThreshold +
                                fragDecisionData->mpduHeaderLength +
                                fragmentationData.blk_size_addition - 1 ) / 
                                fragDecisionData->memBlkSize ) + 1;

    fragmentationData.numMPDUs = 0;
    memBlocks                  = 0;

    if (payloadDataLen > fragmentationData.fragThreshold)
    {
        fragmentationData.numMPDUs += payloadDataLen/fragmentationData.fragThreshold;

        memBlocks  +=     fragmentationData.numMPDUs*numMemBlocksPerFullFrag;

        payloadDataLen -= fragmentationData.numMPDUs*fragmentationData.fragThreshold;
    }

    fragmentationData.numMPDUs++;

    memBlocks  +=  (( payloadDataLen +
                      fragDecisionData->mpduHeaderLength +
                      fragmentationData.blk_size_addition - 1 ) / 
                      fragDecisionData->memBlkSize ) + 1;
    

    if (fragmentationData.numMPDUs == 1)
    {
        *fragmentedFrame = 0;
    }
    else
    {
        *fragmentedFrame = 1;
        memBlocks += min(fragmentationData.numMPDUs,numMemBlocksPerFullFrag); // Add spare blocks for copy purposes
    }

    *totalNumBlocksToAllocate = (UINT8) memBlocks;


/*
    printf(" == Frag == hdr[%d] == thresh[%d] == len[%d] == blk[%d] == mpdus[%d] ==\n",
            fragDecisionData->mpduHeaderLength,
            fragmentationData.fragThreshold,
            fragDecisionData->dataLength,
            memBlocks,
            fragmentationData.numMPDUs );

*/
}
























#define ANY_OFDM_TXD_RATESET  (TXD_RATESET_6MBPS | TXD_RATESET_9MBPS  |    \
                              TXD_RATESET_12MBPS | TXD_RATESET_18MBPS |    \
                              TXD_RATESET_24MBPS | TXD_RATESET_36MBPS |    \
                              TXD_RATESET_48MBPS | TXD_RATESET_54MBPS)       




#define TX_PLCP_SHRT                    0x10
#define RATEVAL_RATECODE_MASK           0x00FF0000
#define RATEVAL_SERVICE_MASK            0x0000FFFF
#define RATEVAL_PREAMBLE_MASK          (TX_PLCP_SHRT << 24)

#define A_SIFS_MICROSECONDS             16
#define A_PIFS_MICROSECONDS             25
#define A_DIFS_MICROSECONDS             34
#define A_SLOT_TIME                     9

#define B_SIFS_MICROSECONDS             10
#define B_PIFS_MICROSECONDS             30
#define B_DIFS_MICROSECONDS             50
#define B_SLOT_TIME                     20

#define SHORT_PREAMBLE_MICROSECONDS     96
#define LONG_PREAMBLE_MICROSECONDS      192
#define OFDM_PREAMBLE_MICROSECONDS      12    // see 802.11g Fig. 19.3.2.4
#define OFDM_SIGNAL_EXT_MICROSECONDS    6
#define OFDM_PLCP_HDR_MICROSECONDS      24
//#define OFDM_SVC_ENDFRM_BYTES         2.75  // 16 bits service, 6 bits end frame
#define OFDM_DURATION_MICROSECONDS           (B_SIFS_MICROSECONDS + OFDM_PLCP_HDR_MICROSECONDS + OFDM_SIGNAL_EXT_MICROSECONDS)
#define NONOFDM_SHORT_DURATION_MICROSECONDS  (B_SIFS_MICROSECONDS + SHORT_PREAMBLE_MICROSECONDS)
#define NONOFDM_LONG_DURATION_MICROSECONDS   (B_SIFS_MICROSECONDS + LONG_PREAMBLE_MICROSECONDS)



#define CONVERT_RATE_INDEX_TO_RATESET_BIT(rateIndex) (1 << rateIndex)

#define  CEILING4(VALUE) (((VALUE+3)>>2)<<2)

typedef enum
{
    RATE_INDEX_1MBPS   =  0,
    RATE_INDEX_2MBPS   =  1,
    RATE_INDEX_5_5MBPS =  2,
    RATE_INDEX_6MBPS   =  3,
    RATE_INDEX_9MBPS   =  4,
    RATE_INDEX_11MBPS  =  5,
    RATE_INDEX_12MBPS  =  6,
    RATE_INDEX_18MBPS  =  7,
    RATE_INDEX_22MBPS  =  8,
    RATE_INDEX_24MBPS  =  9,
    RATE_INDEX_36MBPS  =  10,
    RATE_INDEX_48MBPS  =  11,
    RATE_INDEX_54MBPS  =  12,
    RATE_INDEX_MAX     =  RATE_INDEX_54MBPS,
    MAX_RATE_INDEX,
    INVALID_RATE_INDEX = MAX_RATE_INDEX
}RateIndex_e;



const uint16 gRateDurationFactors[MAX_RATE_INDEX] = 
{
    /*
        Use these pre-calculated multiples of us / Byte to figure duration times, etc.
        Each value represents the us/Byte multiplied by 4096, and rounded up to the next integer
        
        To figure duration in us:
        uint16 duration = ((numBytes * (uint32)value) >> 12)
        
        Notes:
        The remainder/error is less than one microsecond for all rates up to 4096 bytes
        Make sure the multiplication is cast to 32 bits before shifting out the result
    */

    32768,  // 1Mbs
    16384,  // 2Mbs
    5958,   // 5.5Mbs
    5462,   // 6Mbs
    3641,   // 9Mbs
    2979,   // 11MBPS
    2731,   // 12MBPS
    1821,   // 18MBPS
    1490,   // 22MBPS
    1366,   // 24MBPS
    911,    // 36MBPS
    683,    // 48MBPS
    607     // 54MBPS
};

#define RATE_MICROSECOND_SHIFT_FACTOR   12

#define RATE_TO_MICROSECONDS(numBytes, rateIndex) \
      CEILING4(((((uint32)(((numBytes<<3)+22)>>3) * (uint32)gRateDurationFactors[rateIndex])) >> RATE_MICROSECOND_SHIFT_FACTOR))



uint16 TxMemCalcGetCtsDuration(uint16 rateField,uint16 controlRateIndex) 
{
    uint16 ctsDuration = TxMemCalcAckDurationTable[controlRateIndex];//1 for 2MBPs
                                                            //5 for 11MBPs

    if(rateField & RATEVAL_PREAMBLE_MASK)
    {
        ctsDuration+=NONOFDM_SHORT_DURATION_MICROSECONDS;
    }
    else
    {
        ctsDuration+=NONOFDM_LONG_DURATION_MICROSECONDS;
    }
    return ctsDuration;
}


uint16 TxMemCalcFindHighestSupportedRate(uint16 rateField)
{
    uint16 mask;
    RateIndex_e rateIndex = RATE_INDEX_MAX;

        //Go find highest set Rate-bit
    mask = 1 << RATE_INDEX_MAX;
    while (mask)
    {
        if (rateField & mask)
        {
            break;
        }
        mask >>= 1;  //next lower rate.
        rateIndex--;
    }
    return rateIndex;
}


uint16 TxMemCalcComputeDuration( uint16 length, uint16 rateField, uint32 noAck )
{
	uint16 mask;
    uint16 preambleDuration;
    uint16 rateIndex = TxMemCalcFindHighestSupportedRate(rateField);
    uint16 duration;

	mask = 1<<rateIndex;
	
	// duration calculation is different for OFDM vs 802.11b rates
	if (!(mask & ANY_OFDM_TXD_RATESET))
	{
		// 802.11b rate
		if (rateField & RATEVAL_PREAMBLE_MASK) 
		{
			preambleDuration = NONOFDM_SHORT_DURATION_MICROSECONDS;
		}
		else
		{
			preambleDuration = NONOFDM_LONG_DURATION_MICROSECONDS;
		}
	}
    else
    {
		// OFDM rate
    	preambleDuration = OFDM_DURATION_MICROSECONDS;
    }

    duration = preambleDuration+RATE_TO_MICROSECONDS(length, rateIndex);
    
    if(FALSE==noAck)
    {
       duration+=preambleDuration+TxMemCalcAckDurationTable[rateIndex];
    }

    return duration;
}   
