/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/******************************************************************************/

/*! \file  dda_contexts.c                                               
    \brief This module implements driver context DDA Task                                                                          
*/
/******************************************************************************/

#define _DDA_CONTEXTS_C

/***********************************************************
	 includes.
*******************************************************/

#include "wlan_pal.h"
#include "dda.h"
#include "dda_EventQueue.h"
#include "dda_trace.h"
#include "mgmtSm.h"
#include "mainsm.h"
#include "rx.h"
#include "tx.h"
#include "whalAcxMbox.h"
#include "dda_contexts.h"
#include "dda_wpaauth_ioctl.h"
#include "wpaIfVxWorks.h"


/***********************************************************
	defs
************************************************************/

#define DDATASK_NAME        		"tDda"
#define DDATASK_STACK_SIZE  		20000
#define MAX_PROCESS_COUNT   		6
#define EVENT_POOL_SIZE     		300
#define DDAMBOX_TASK_NAME   		"tMbox"
#define DDAMBOX_TASK_STACK_SIZE 	5000
#define DDA_RESTART_TASK_NAME   	"tRestart"
#define DDA_RESTART_TASK_STACK_SIZE 5000


#undef DCX_TEST

typedef enum
{
    EVENT_FIRST_A,
    EVENT_MNGMT_A,
    EVENT_RX_A,
    EVENT_TX_A,
    EVENT_MAINSM_A,
    EVENT_MB_A,
    EVENT_LAST_A,

}   EVENT_TYPE_E;

/* 
   DDA Task queue lists in decreasing priority order:
   Managemnet is the highest and MailBox is the least priority
*/
typedef enum
{
    MNGMT_LIST_A,
    RX_LIST_A,
    TX_LIST_A,
    MAINSM_LIST_A,
    LAST_LIST_A

}   QUEUE_LIST_E;

#define QUEUE_LIST_NUMBER (LAST_LIST_A-MNGMT_LIST_A)

typedef struct  
{
    UINT32 type;

} MbMessage_T;

/* event data struct*/
typedef	struct 
{
	union
    {
		  mgmtMessage_T    mgmt;
		  mainsmMessage_T  mainSm;
		  RxMessage_T      rx;
		  TxMessage_T	   tx;
	} data;

} EVENT_DATA_S;



/***********************************************************
	macros
***********************************************************/

/***********************************************************
	static variables
************************************************************/

static EVQ_QUEUE_P  ddaTaskQueue; 
static int          ddaTaskId = 0;
static int          ddaMboxTaskId;
static SEM_ID		ddaMboxSem;
static int          ddaRestartTaskId;
static SEM_ID		ddaRestartSem;
static int 	 tx_sched_event_count=0;
static int   rx_sched_event_count=0;


EVQ_ACTION_FUN_P ActionFunTable[QUEUE_LIST_NUMBER] = 
{
    (EVQ_ACTION_FUN_P) wdrv_mgmt_getEvent,
    (EVQ_ACTION_FUN_P) wdrv_rx_getEvent,
    (EVQ_ACTION_FUN_P) wdrv_tx_getEvent,
    (EVQ_ACTION_FUN_P) wdrv_mainSm_getEvent,
};

/***********************************************************
	Local Function Prototypes
***********************************************************/

static void ddaTaskFun( void );
static void ddaMboxTaskFun( void );
static void ddaRestartTaskFun( void );

/*******************************************************************************
		PUBLIC FUNCTION IMPLEMENTATION
********************************************************************************/

/******************************************************************************/

/*! \brief 	Resets transmitting threshold counter
*/
/******************************************************************************/

void inline wdrv_DDA_init_tx_sched_count(void)  {tx_sched_event_count=0;}


/******************************************************************************/

/*! \brief 	Resets receiving threshold counter
*/
/******************************************************************************/

void inline wdrv_DDA_init_rx_sched_count(void)  {rx_sched_event_count=0;}


/******************************************************************************/

/*! \brief 	Creates DDA task and event queue
	\param 	taskPriority - DDA task priority
	\return status of operation OK or ERROR
*/
/******************************************************************************/

STATUS DCX_TaskInit( int taskPriority )
{

 ddaTaskQueue = EVQ_Create( QUEUE_LIST_NUMBER,    /* number of lists             */
                            EVENT_POOL_SIZE,      /* number of events in pool    */ 
                            MAX_PROCESS_COUNT,    /* max number of process itert.*/
                            sizeof(EVENT_DATA_S), /* event data field size       */
                            ActionFunTable        /* action function ptr array   */  
                          );
 if (ddaTaskQueue == NULL) 
 {
     DDA_LOG( DDA_LOG_ERR, "DCX_TaskInit: Failed to create event queue\n" );
     return ERROR;
 }
 wdrv_DDA_init_rx_sched_count();
 wdrv_DDA_init_tx_sched_count();

 ddaTaskId = taskSpawn( DDATASK_NAME,           /* name of DDA task         */
                        taskPriority,           /* priority of DDA task     */
                        0,                      /* task option word         */
                        DDATASK_STACK_SIZE,     /* task stack size (bytes)  */
                        (FUNCPTR) ddaTaskFun,   /* entry point of DDA task  */
                        0,0,0,0,0,0,0,0,0,0     /* arguments to pass        */
                      );                       
 
 if (ddaTaskId == ERROR) 
 {
     DDA_LOG( DDA_LOG_ERR, "DCX_TaskInit: Failed to create DDA task\n" );
     return ERROR;
 }

 ddaMboxSem = semBCreate( SEM_Q_FIFO, SEM_EMPTY );
 if (ddaMboxSem == NULL) 
 {
     DDA_LOG( DDA_LOG_ERR, "DCX_TaskInit: Failed to create Mbox semaphore\n" );
     return ERROR;
 }
 
 ddaMboxTaskId = taskSpawn( DDAMBOX_TASK_NAME,      /* name of DDA Mbox task    	*/
                        taskPriority - 2,      		/* priority of DDA Mbox task	*/
                        0,                      	/* task option word         	*/
                        DDAMBOX_TASK_STACK_SIZE,    /* task stack size (bytes)  	*/
                        (FUNCPTR) ddaMboxTaskFun,   /* entry point of DDA Mbox task */
                        0,0,0,0,0,0,0,0,0,0     	/* arguments to pass        	*/
                      );                       
 
 if (ddaMboxTaskId == ERROR) 
 {
     DDA_LOG( DDA_LOG_ERR, "DCX_TaskInit: Failed to create DDA Mbox task\n" );
     return ERROR;
 }

 ddaRestartSem = semBCreate( SEM_Q_FIFO, SEM_EMPTY );
 if (ddaRestartSem == NULL) 
 {
     DDA_LOG( DDA_LOG_ERR, "DCX_TaskInit: Failed to create Restart semaphore\n" );
     return ERROR;
 }

 ddaRestartTaskId = taskSpawn( DDA_RESTART_TASK_NAME,   /* name of DDA Restart task    	*/
                        taskPriority - 1,      			/* priority of DDA Restart task	*/
                        0,                      		/* task option word         	*/
                        DDA_RESTART_TASK_STACK_SIZE,    /* task stack size (bytes)  	*/
                        (FUNCPTR) ddaRestartTaskFun,   	/* entry point of Restart task  */
                        0,0,0,0,0,0,0,0,0,0     		/* arguments to pass        	*/
                      );                       
 if (ddaRestartTaskId == ERROR) 
 {
	DDA_LOG( DDA_LOG_ERR, "DCX_TaskInit: Failed to create Restart task\n" );
	return ERROR;
 }

 DDA_LOG_ARG( DDA_LOG_TASK, 
			  "DCX_TaskInit: DdaTID=%x Queue=%x MboxTID=%x MboxSem=%x RstTID=%x RstSem=%x\n",
              ddaTaskId, ddaTaskQueue, ddaMboxTaskId, ddaMboxSem, 
			  ddaRestartTaskId, ddaRestartSem );
 
 return OK;

}


/******************************************************************************/

/*! \brief 	Deletes DDA task and event queue
	\return status of operation OK or ERROR
*/
/******************************************************************************/

STATUS DCX_TaskDelete(void)
{
 STATUS status;

 status = taskDelete(ddaTaskId);
 if (status != OK) 
 {
     DDA_LOG( DDA_LOG_ERR, "DCX_TaskDelete: Failed to delete DDA task\n" );
     return ERROR;
 }
 ddaTaskId = 0;

 status = EVQ_Delete( ddaTaskQueue );
 if (status != OK) 
 {
     DDA_LOG( DDA_LOG_ERR, "DCX_TaskDelete: Failed to delete event queue\n" );
     return ERROR;
 }
 
 status = taskDelete(ddaMboxTaskId);
 if (status != OK) 
 {
     DDA_LOG( DDA_LOG_ERR, "DCX_TaskDelete: Failed to delete Mbox task\n" );
     return ERROR;
 }
 
 status = semDelete(ddaMboxSem);
 if (status != OK) 
 {
     DDA_LOG( DDA_LOG_ERR, "DCX_TaskDelete: Failed to delete Mbox sem\n" );
     return ERROR;
 }
 
 status = taskDelete(ddaRestartTaskId);
 if (status != OK) 
 {
     DDA_LOG( DDA_LOG_ERR, "DCX_TaskDelete: Failed to delete Restart task\n" );
     return ERROR;
 }
 
 status = semDelete(ddaRestartSem);
 if (status != OK) 
 {
     DDA_LOG( DDA_LOG_ERR, "DCX_TaskDelete: Failed to delete Restart sem\n" );
     return ERROR;
 }
 
 return OK;
}


/******************************************************************************/
/*! \brief Activates driver restart procedure
		
	\return status OK
*/
/******************************************************************************/

STATUS wdrv_DDA_restart_setEvent(void)
{
	STATUS res;

	DDA_LOG( DDA_LOG_EVENT_SET, "wdrv_DDA_restart_setEvent\n" );
	
	res = semGive( ddaRestartSem );

	if (res != OK)
	{
		DDA_LOG( DDA_LOG_ERR, "wdrv_DDA_restart_setEvent: Failed to give sem\n" );
		return ERROR;
	}
	return OK;
}

/******************************************************************************/

/*! \brief 	Wakes up process waiting on the semaphore
	\param 	wait_q - pointer to semaphore
	\return void
*/
/******************************************************************************/

void   wdrv_DDA_wake( void * wait_q )
{
 STATUS status;
 SEM_ID semId = * ((SEM_ID *) wait_q);

 taskLock();
 if (semId == NULL) 
 {
    semId = semBCreate( SEM_Q_FIFO, SEM_EMPTY );
    if (semId == NULL) 
    {
        DDA_LOG( DDA_LOG_ERR, "wdrv_DDA_sleep_timeout: Semaphore create error\n" );
        taskUnlock();
        return;
    }
    * ((SEM_ID *) wait_q) = semId;
 }
 taskUnlock();

 status = semGive( * ((SEM_ID *) wait_q) );
 if (status != OK) 
 {
     DDA_LOG( DDA_LOG_ERR, "wdrv_DDA_wake: Semaphore give error\n" );
 }
}

/******************************************************************************/

/*! \brief 	Waits on semaphore specified time (ms)
	\param 	wait_q  - pointer to wait mechanism (semaphore)
	\param 	timeout - time out in ms
	\return 1 if wake up before timeout and 0 if timed out
*/
/******************************************************************************/

long   wdrv_DDA_sleep_timeout(void *wait_q, long timeout)
{
 STATUS status;
 SEM_ID semId = * ((SEM_ID *) wait_q);
 int waitTicks = PALSRV_msec2ticks(timeout); /* convert ms to sys ticks */

 taskLock();
 if (semId == NULL) 
 {
    semId = semBCreate( SEM_Q_FIFO, SEM_EMPTY );
    if (semId == NULL) 
    {
        DDA_LOG( DDA_LOG_ERR, "wdrv_DDA_sleep_timeout: Semaphore create error\n" );
        taskUnlock();
        return 0;
    }
    * ((SEM_ID *) wait_q) = semId;
 }
 taskUnlock();

 status = semTake(semId, waitTicks);
 if (status != OK)
 {
   return 0;    /* timed out */
 }
 else
 {
   return 1;    /* wake up before time out */
 }

}

/******************************************************************************/

/*! \brief 	Suspends DDA task
*/
/******************************************************************************/

void DCX_suspend(void)
{
 
 if (ddaTaskId)
 {
	 if ( taskSuspend(ddaTaskId) != OK ) 
	 {
		 DDA_LOG( DDA_LOG_ERR, "DCX_suspend: Failed to suspend DDA task\n" );
	 }
 }

}


/******************************************************************************/

/*! \brief 	Resumes DDA task
*/
/******************************************************************************/

void DCX_resume(void)
{
 
 if (ddaTaskId)
 {
	 if ( taskResume(ddaTaskId) != OK ) 
	 {
		 DDA_LOG( DDA_LOG_ERR, "DCX_resume: Failed to resume DDA task\n" );
	 }
 }
}

/******************************************************************************
		LOCAL FUNCTION IMPLEMENTATION
*******************************************************************************/

/******************************************************************************/

/*! \brief 	DDA task main function
	\return void
*/
/******************************************************************************/

void ddaTaskFun( void )
{
 for(;;)
 {
     EVQ_WaitAndProcess( ddaTaskQueue );
 }
}

/******************************************************************************/

/*! \brief 	DDA Mbox task main function
	\return void
*/
/******************************************************************************/

void ddaMboxTaskFun( void )
{
 STATUS res;
 
 for(;;)
 {
     res = semTake( ddaMboxSem, WAIT_FOREVER );
	 if (res != OK)
	 {
		 DDA_LOG( DDA_LOG_ERR, "ddaMboxTaskFun: Semaphore take error\n" );
	 }
	 res = wdrv_mailbox_getEvent( 0 );
	 if (res != OK)
	 {
		 DDA_LOG( DDA_LOG_ERR, "ddaMboxTaskFun: wdrv_mailbox_getEvent error\n" );
	 }
 }
}

/******************************************************************************/

/*! \brief 	DDA Restart task main function
	\return void
*/
/******************************************************************************/

void ddaRestartTaskFun( void )
{
 STATUS res;
 
 for(;;)
 {
     res = semTake( ddaRestartSem, WAIT_FOREVER );
	 if (res != OK)
	 {
		 DDA_LOG( DDA_LOG_ERR, "ddaRestartTaskFun: Semaphore take error\n" );
	 }
	 wdrv_ActivateRestart();
 }
}

/******************************************************************************/

/*! \brief 	Puts management event into DDA event queue
	\param 	message - pointer to message
	\return status of operation OK or ERROR

Some type of events are processed in the caller context.
*/
/******************************************************************************/

STATUS wdrv_DDA_mgmt_setEvent( mgmtMessage_T *message)
{
 STATUS status;

 switch(message->type)
 {
	case MGMT_FRAME:
	case CLASS3:
	case BEACON_FRAME:
	
		wdrv_mgmt_getEvent(message);
		break;

	case AUTH_RESPONSE_TIMEOUT:
	case AGING:
	case DEAUTH_MNG:
	case DISASSOC_MNG:
	case BEACON_ERP_TIMEOUT:
	case BEACON_PBCC_TIMEOUT:
	case DYNAMIC_ALG_TIMEOUT:
	case SET_TEMPLATE_REQ:
	case RADIO_CALIBRATE_TIMEOUT:
	case RADIO_CALIBRATE_TIMEOUT_CRT:
	case MAC_AUTHORIZE:
	case MAC_DE_AUTHORIZE:
    case WPA_ASSOC_REQ_REP:
    case WDS_LINK_MNG:
    case DEAUTH_TX_COMPLETE:

        status = EVQ_PutEvent( ddaTaskQueue, EVENT_MNGMT_A, MNGMT_LIST_A,
                               (UINT8 *) message, sizeof(mgmtMessage_T) );
        if (status != OK) 
        {
            DDA_LOG( DDA_LOG_ERR, "wdrv_DDA_mgmt_setEvent: Failed to put event\n" );
            return NOK;
        }
		break;
	
    default: 
        DDA_LOG_ARG( DDA_LOG_ERR, "wdrv_DDA_mgmt_setEvent: Unknown mgmt event %u\n",
                     message->type, 0,0,0,0,0 );
		return NOK;
 }

 return OK;

}

/******************************************************************************/

/*! \brief 	Puts Main State Machine event into DDA event queue
	\param 	message - pointer to message
	\return status of operation OK or ERROR
*/
/******************************************************************************/

STATUS wdrv_DDA_mainSm_setEvent(mainsmMessage_T *message)
{
 STATUS status;


 status = EVQ_PutEvent( ddaTaskQueue, EVENT_MAINSM_A, MAINSM_LIST_A,
                        (UINT8 *) message, sizeof(mainsmMessage_T) );
 if (status != OK) 
 {
     DDA_LOG( DDA_LOG_ERR, "wdrv_DDA_mainSm_setEvent: Failed to put event\n" );
     return NOK;
 }
 return OK; 
}

/******************************************************************************/

/*! \brief 	Puts Receiver event into DDA event queue
	\param 	message - pointer to message
	\return status of operation OK or ERROR
*/
/******************************************************************************/

STATUS wdrv_DDA_rx_setEvent(RxMessage_T *message)
{
 STATUS status;
 
 DDA_LOG_ARG(DDA_LOG_TRACE, "%s: count=%u msg=%x\n", __func__,
			 rx_sched_event_count, message, 0,0,0 );
 
 if(rx_sched_event_count > 0)
   	return OK;
 
 rx_sched_event_count++;

 status = EVQ_PutEvent( ddaTaskQueue, EVENT_RX_A, RX_LIST_A,
                        (UINT8 *) message, sizeof(RxMessage_T) );
 if (status != OK) 
 {
    DDA_LOG( DDA_LOG_ERR, "wdrv_DDA_rx_setEvent: Failed to put event\n" );
    return NOK;
 }
 return OK; 
}

/******************************************************************************/

/*! \brief 	Puts Transmitter event into DDA event queue
	\param 	message - pointer to message
	\return status of operation OK or ERROR

Some type of events are processed in the caller context
*/
/******************************************************************************/

STATUS wdrv_DDA_tx_setEvent(TxMessage_T *message)
{
 STATUS status;
 
 switch(message->type)
 {
	case WDRV_TX_MSDU_SEND:
	case WDRV_TX_COMPLETE:
	case WDRV_TX_MSDU_PS_POLL:
	case WDRV_TX_PS_AGING:
	case WDRV_TX_MSDU_PS_STATUS_CHANGE:
	case WDRV_TX_MSDU_PS_CLEAN_STATION_LIST:

		wdrv_tx_getEvent(message);
		break;
	
	case WDRV_TX_MSDU_TIMEOUT:
		if(tx_sched_event_count > 0)
			break;
		tx_sched_event_count++;
		
        status = EVQ_PutEvent( ddaTaskQueue, EVENT_TX_A, TX_LIST_A,
                               (UINT8 *) message, sizeof(TxMessage_T) );
        if (status != OK) 
        {
            DDA_LOG( DDA_LOG_ERR, "wdrv_DDA_tx_setEvent: Failed to put event\n" );
            return NOK;
        }
		break;
	
	default: 
        DDA_LOG_ARG( DDA_LOG_ERR, "wdrv_DDA_tx_setEvent: Unknown tx event %u\n",
                     message->type, 0,0,0,0,0 );
		return NOK;

 }

 return OK;

}

/******************************************************************************/

/*! \brief 	Puts Mail Box event into DDA event queue
	\param 	event_type - type of Mail Box event by value
	\return status of operation OK or ERROR
*/
/******************************************************************************/

STATUS wdrv_DDA_mailbox_setEvent(UINT32  event_type)
{
 STATUS status;

 DDA_LOG_ARG(DDA_LOG_TRACE, "%s: type=%x\n", __func__, event_type, 0,0,0,0 );
 
 status = semGive( ddaMboxSem ); 
 if (status != OK) 
 {
     DDA_LOG( DDA_LOG_ERR, "wdrv_DDA_mailbox_setEvent: Failed to give sem\n" );
     return NOK;
 }
 return OK; 
}


/******************************  test func *********************************/

#ifdef DCX_TEST
#include "dda_contextsTest.c"
#endif /* DCX_TEST */

