/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2004 Texas Instruments Incorporated      	 |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/******************************************************************************/

/*! \file  utilsTcb.c                                               
    \brief This module implements timer call back services for VxWorks
*/
/******************************************************************************/


#define _UTILS_TCB_C

/***********************************************************
	 includes.
*******************************************************/

#include "wlan_pal.h"
#include "dda.h"
#include "msgQLib.h"
#include "dda_trace.h"

#include "utilsTcb.h"

/***********************************************************
	static variables
************************************************************/

static unsigned long _tcbTicks;

/*
   This Static Data Member Hold A Count Of The tick()s.
   It Is Initialized To 0  And Incremented Every tick()
*/
static unsigned long _clock = 0;

/*
   prevents the mechanism to be initialized more than once
*/
static int _initialized = 0;

static SEM_ID _semb;

/*
   This Is The Static Heap : It Is An Array Of Pointers To
   CallBack Objects. The Legal Entries Are Those Indexed
   From 1 To MAX_TIMED_CALLBACKS. The 0 Entry Is Used For Temporary Storage
*/
static lbt_tcbObjTYPE *_heap[MAX_TIMED_CALLBACKS+1];

/*
   The Index In The Heap Array Of The Last Waiting Object
   This Static Data Member Holds The Heap Index Of The Last Object In The Heap
   It Is Initialized To 0 : Meaning The Heap Is Empty
*/
static unsigned _inUse = 0;

static int _TaskID;


/* Move The Object Up The Heap To The     */
/* Correct Position                       */
static void lbt_tcbUpHeap(lbt_tcbObjTYPE *this);
/* Move The Object Down The Heap To The   */
/* Correct Position                       */
static void lbt_tcbDownHeap(lbt_tcbObjTYPE *this);

/***********************************************************
	Local Function Prototypes
***********************************************************/

void lbt_tcbTask(void);



/*******************************************************************************
		FUNCTION IMPLEMENTATION
********************************************************************************/


/******************************************************************************/

/*! \brief 	Mechanism Initializer
	\param	priority - priority of the TCB task
	\param	tcbTicks - wake-up interval
*/
/******************************************************************************/

long lbt_tcbInit(unsigned long priority, unsigned long tcbTicks) 
{

   if (_initialized) 
   {
	   DDA_LOG_ARG(DDA_LOG_ERR, 
				   "%s: TCB was allready init\n",__func__, 0,0,0,0,0 );
   }

   _initialized = 1;

   _tcbTicks = tcbTicks;
   
   _semb = semMCreate( SEM_Q_PRIORITY); /* mutex initial state is SEM_FULL */

   if (_semb == NULL)
   {
	   DDA_LOG_ARG(DDA_LOG_ERR, 
				   "%s: Error Creating sem\n",__func__, 0,0,0,0,0 );
	   return ERROR;

   }

   _TaskID = taskSpawn( "tTCB",           	 	/* name of Timer task         */
						 priority,           	/* priority of Timer task     */
						 0,                     /* task option word           */
						 16384,   				/* task stack size (bytes)    */
						 (FUNCPTR)lbt_tcbTask,  /* entry point of Timer task  */
						 0,0,0,0,0,0,0,0,0,0    /* arguments to pass          */
						);                       

   if (_TaskID == ERROR) 
   {
	   DDA_LOG_ARG(DDA_LOG_ERR, 
				   "%s: Error Creating task\n",__func__, 0,0,0,0,0 );
	   return ERROR;
   }
   DDA_LOG_ARG(DDA_LOG_TIMER, 
			   "%s: TCB initialization completed\n",__func__, 0,0,0,0,0 );

   return OK;
}

/******************************************************************************/

/*! \brief 	Mechanism Terminator
*/
/******************************************************************************/

void  lbt_tcbEnd(void) 
{
   	STATUS res;

	if (!_initialized) return;
    
	res = semTake(_semb, WAIT_FOREVER );
	if (res != OK)
	{
		DDA_LOG_ARG(DDA_LOG_ERR,"%s: Error taking sem\n",__func__, 0,0,0,0,0 );
	}
	res = taskDelete(_TaskID);
	if (res != OK)
	{
		DDA_LOG_ARG(DDA_LOG_ERR,"%s: Failed to delete task\n",__func__, 0,0,0,0,0 );
	}
	res = semDelete(_semb);
	if (res != OK)
	{
		DDA_LOG_ARG(DDA_LOG_ERR,"%s: Failed to delete sem\n",__func__, 0,0,0,0,0 );
	}
	_initialized = _inUse = 0;
}




/******************************************************************************/

/*! \brief 	Constructor of an object
	\param	msgQ - the message queue that receives the TCB messages
	\return pointer to created TCB object
*/
/******************************************************************************/

lbt_tcbObjTYPE *lbt_tcbNew(unsigned long  msgQ) {
   lbt_tcbObjTYPE *this;


   if ((this = (lbt_tcbObjTYPE *)malloc(sizeof(lbt_tcbObjTYPE)))==NULL)
   {
	   DDA_LOG_ARG(DDA_LOG_ERR,"%s: No memory for timer\n",__func__, 0,0,0,0,0 );
	   return NULL;
   }

   this->_msgQ = msgQ;
   this->_index=0;
   
   DDA_LOG_ARG(DDA_LOG_TIMER, "%s: Created timer=%x\n",__func__, 
    		   this, 0,0,0,0 );
   return this;
}

/******************************************************************************/

/*! \brief 	Destructor of an object
	\param	pointer to TCB object
*/
/******************************************************************************/
void lbt_tcbDelete(lbt_tcbObjTYPE *this)
{
   DDA_LOG_ARG(DDA_LOG_TIMER, "%s: Enter timer=%x\n",__func__, 
    		   this, 0,0,0,0 );
   
   lbt_tcbCancel(this);
   free(this);
}

/******************************************************************************/

/*! \brief 	Adds the TCB object to the static heap: message to be sent 
			after delta ticks.
	\param 	this - pointer to TCB object
	\param 	Delta - number of ticks to wait until wake up
	\param 	msginfo - message to be sent to queue
	\param 	periodic - flag if TCB is periodic or not
*/
/******************************************************************************/

void lbt_tcbQueue(lbt_tcbObjTYPE *this, unsigned long Delta,
				  unsigned long msginfo,int periodic) 
{
 unsigned long prevWup;
 STATUS   res;

 DDA_LOG_ARG(DDA_LOG_TIMER, "%s: Enter timer=%x delta=%u\n",__func__, 
			 this, Delta, 0,0,0 );
   
   if (!_initialized) {
      this->_index = 0;
      return;
   }

   if (Delta >= MAX_CLOCK)
      Delta = MAX_CLOCK-1;

   if (Delta == 0)  /* avoid very small touts to be 0 */
      Delta = 1;

   res = semTake(_semb, WAIT_FOREVER );
   if (res != OK)
   {
	   DDA_LOG_ARG(DDA_LOG_ERR, "%s: Error taking sem\n",__func__, 0,0,0,0,0 );
   }

   this->_isExtended=0;
   this->_msgInfo = msginfo;
   this->_act = NULL;

   this->_periodic = periodic;

   if (this->_periodic) this->_periodicDelta = Delta;

   if(0<this->_index) { /* element already in queue : */
      prevWup = this->_wakeUpTime;
         this->_wakeUpTime = _clock + Delta;
      if (prevWup < this->_wakeUpTime)
         lbt_tcbDownHeap(this);
      else
         lbt_tcbUpHeap(this);
   } else {
      this->_wakeUpTime = _clock + Delta;
      /* A New Element Is Added To The Heap ! ( If Possible ) */
      if (_inUse<MAX_TIMED_CALLBACKS) {
         this->_index=_inUse+=1;
         _heap[_inUse]=this;
         lbt_tcbUpHeap(this);
      }
   }

   res = semGive( _semb );
   if (res != OK)
   {
	   DDA_LOG_ARG(DDA_LOG_ERR, "%s: Error giving sem\n",__func__, 0,0,0,0,0 );
   }
   DDA_LOG_ARG(DDA_LOG_TIMER, "%s: Exit\n",__func__, 0,0,0,0,0 );
}



/******************************************************************************/

/*! \brief 	Adds the TCB object to the static heap: message to be sent 
			after delta ticks.
	\param 	this - pointer to TCB object
	\param 	Delta - number of ticks to wait until wake up
	\param 	msginfo - message to be sent to queue
	\param 	periodic - flag if TCB is periodic or not
*/
/******************************************************************************/
void lbt_tcbQueueExtended(lbt_tcbObjTYPE *this,unsigned long Delta,
						  unsigned long msginfo[4],int periodic) 
{
   unsigned long prevWup;
   STATUS   res;

   DDA_LOG_ARG(DDA_LOG_TIMER, "%s: Enter timer=%x\n",__func__, 
    		   this, 0,0,0,0 );

   if (!_initialized) {
      this->_index = 0;
      return;
   }

   if (Delta >= MAX_CLOCK)
      Delta = MAX_CLOCK-1;

   if (Delta == 0)  /* avoid very small touts to be 0 */
      Delta = 1;

   res = semTake(_semb, WAIT_FOREVER );
   if (res != OK)
   {
	   DDA_LOG_ARG(DDA_LOG_ERR, "%s: Error taking sem\n",__func__, 0,0,0,0,0 );
   }

   this->_isExtended=1;
   this->_msgInfo = 0;
   this->_msgInfoExtended[0] = msginfo[0];
   this->_msgInfoExtended[1] = msginfo[1];
   this->_msgInfoExtended[2] = msginfo[2];
   this->_msgInfoExtended[3] = msginfo[3];

   this->_act = NULL;

   this->_periodic = periodic;

   if (this->_periodic) this->_periodicDelta = Delta;

   if(0<this->_index) { /* element already in queue : */
      prevWup = this->_wakeUpTime;
         this->_wakeUpTime = _clock + Delta;
      if (prevWup < this->_wakeUpTime)
         lbt_tcbDownHeap(this);
      else
         lbt_tcbUpHeap(this);
   } else {
      this->_wakeUpTime = _clock + Delta;
      /* A New Element Is Added To The Heap ! ( If Possible ) */
      if (_inUse<MAX_TIMED_CALLBACKS) {
         this->_index=_inUse+=1;
         _heap[_inUse]=this;
         lbt_tcbUpHeap(this);
      }
   }

   res = semGive( _semb );
   if (res != OK)
   {
	   DDA_LOG_ARG(DDA_LOG_ERR, "%s: Error giving sem\n",__func__, 0,0,0,0,0 );
   }
   
   DDA_LOG_ARG(DDA_LOG_TIMER, "%s: Exit\n",__func__, 0,0,0,0,0 );
}

/******************************************************************************/

/*! \brief 	Adds the TCB object to the static heap: message to be sent 
			after delta ticks.
	\param 	this - pointer to TCB object
	\param 	Delta - number of ticks to wait until wake up
	\param 	act   - action to add
	\param 	periodic - flag if TCB is periodic or not
*/
/******************************************************************************/
void lbt_tcbQueueAct( lbt_tcbObjTYPE *this, unsigned long Delta,
					  lbt_tcbActTYPE act, UINT32 data, int periodic) 
{
   unsigned long prevWup;
   STATUS   res;

   DDA_LOG_ARG(DDA_LOG_TIMER, "%s: Enter timer=%x delta=%u\n",__func__, 
			   this, Delta, 0,0,0 );

   if (!_initialized) {
      this->_index = 0;
      return;
   }

   if (Delta >= MAX_CLOCK)
      Delta = MAX_CLOCK-1;

   if (Delta == 0)  /* avoid very small touts to be 0 */
      Delta = 1;

   res = semTake(_semb, WAIT_FOREVER );
   if (res != OK)
   {
	   DDA_LOG_ARG(DDA_LOG_ERR, "%s: Error taking sem\n",__func__, 0,0,0,0,0 );
   }

   this->_msgInfo = 0;
   this->_isExtended=0;

   this->_act  = act;
   this->_data = data;

   this->_periodic = periodic;

   if (this->_periodic) this->_periodicDelta = Delta;

   if(0<this->_index) { /* element already in queue */
      prevWup = this->_wakeUpTime;
       this->_wakeUpTime = _clock + Delta;
      if (prevWup < this->_wakeUpTime)
         lbt_tcbDownHeap(this);
      else
         lbt_tcbUpHeap(this);
   } else {
      this->_wakeUpTime = _clock + Delta;
      /* A New Element Is Added To The Heap ! ( If Possible ) */
      if (_inUse<MAX_TIMED_CALLBACKS) {
         this->_index=_inUse+=1;
         _heap[_inUse]=this;
         lbt_tcbUpHeap(this);
      }
   }

   res = semGive( _semb );
   if (res != OK)
   {
	   DDA_LOG_ARG(DDA_LOG_ERR, "%s: Error giving sem\n",__func__, 0,0,0,0,0 );
   }
   
   DDA_LOG_ARG(DDA_LOG_TIMER, "%s: Exit\n",__func__, 0,0,0,0,0 );

}


/******************************************************************************/

/*! \brief 	Delete from queue the object: interchange with last and diffuse down 
	\param 	this - pointer to TCB object
*/
/******************************************************************************/
void lbt_tcbCancel(lbt_tcbObjTYPE *this) 
{
   unsigned Index;
   STATUS   res;

   DDA_LOG_ARG(DDA_LOG_TIMER, "%s: Enter timer=%x\n",__func__, 
			   this, 0,0,0,0 );
   
   if (!_initialized) {
      this->_index = 0;
      return;
   }

   res = semTake(_semb, WAIT_FOREVER );
   if (res != OK)
   {
	   DDA_LOG_ARG(DDA_LOG_ERR, "%s: Error taking sem\n",__func__, 0,0,0,0,0 );
   }
   
   Index=this->_index;
   if (Index>0) {
      this->_index=0;
      _inUse-=1;
      if (Index<=_inUse) {
         _heap[Index]=_heap[_inUse+1];
         _heap[Index]->_index=Index;
         lbt_tcbDownHeap(_heap[Index]);
         lbt_tcbUpHeap(_heap[Index]);       
      }
   }
   res = semGive( _semb );
   if (res != OK)
   {
	   DDA_LOG_ARG(DDA_LOG_ERR, "%s: Error giving sem\n",__func__, 0,0,0,0,0 );
   }
   DDA_LOG_ARG(DDA_LOG_TIMER, "%s: Exit\n",__func__, 0,0,0,0,0 );
}


/******************************************************************************/

/*! \brief 	Suspends all timers 
*/
/******************************************************************************/

void lbt_tcbSuspend(void)
{
	STATUS res;

	DDA_LOG_ARG(DDA_LOG_TIMER, "%s: Enter\n",__func__, 0,0,0,0,0 );
	if (!_initialized) 
	{
	   return;
	}
	res = taskSuspend(_TaskID);
	if (res != OK)
	{
		DDA_LOG_ARG(DDA_LOG_ERR, "%s: Failed to supend timers\n",
					__func__, 0,0,0,0,0 );
	}
}

/******************************************************************************/

/*! \brief 	Resumes all timers 
*/
/******************************************************************************/

void lbt_tcbResume(void)
{
	STATUS res;

	DDA_LOG_ARG(DDA_LOG_TIMER, "%s: Enter\n",__func__, 0,0,0,0,0 );
	
	if (!_initialized) 
	{
	   return;
	}
	res = taskResume(_TaskID);
	if (res != OK)
	{
		DDA_LOG_ARG(DDA_LOG_ERR, "%s: Failed to resume timers\n",
					__func__, 0,0,0,0,0 );
	}
}

/******************************************************************************/

/*! \brief 	TCB task main function 
*/
/******************************************************************************/
void lbt_tcbTask(void) 
{
   STATUS   res;
   unsigned int  i;
   lbt_tcbObjTYPE **First, tempFirst;

   for(;;) {

	  taskDelay( _tcbTicks );

      _clock += _tcbTicks;
      
	  /* it's time to advance clock and to go over the heap */
	  res = semTake(_semb, WAIT_FOREVER );
	  if (res != OK)
	  {
		  DDA_LOG_ARG(DDA_LOG_ERR, "%s: Error taking sem\n",__func__, 0,0,0,0,0 );
	  }

      /* First Points To The First Element In The Heap Array */
      First=_heap+1;

      while ((_inUse>0) && (*First)->_wakeUpTime<=_clock) {

          /* store action function and data before removing the timer from the heap */
          tempFirst._act = (*First)->_act;
          tempFirst._data = (*First)->_data;

          if ((*First)->_periodic!=0) {
             /* ReInstall In The Queue ! */
             (*First)->_wakeUpTime = _clock + (*First)->_periodicDelta;
          } else { /* take out from heap */
             _heap[_inUse]->_index = 1;
             (*First)->_index      = 0;
             *First                = _heap[_inUse];
             _inUse               -= 1;
          }

          if (0 < _inUse) {
             lbt_tcbDownHeap((*First));
          }

         if (tempFirst._act != NULL)
		 {
			 /* call act func */
			 (tempFirst._act)( tempFirst._data );
		 }
		 else
		 {
			 DDA_LOG_ARG(DDA_LOG_ERR,"%s: NULL action for timer=%x\n", __func__, 
						 *First,0,0,0,0 );
		 }
      }

      if (_clock>=MAX_CLOCK) {
         /* zero _clock, allign down all counters */
         for(i=1;i<_inUse;i++) {
            _heap[i]->_wakeUpTime-=_clock;
         }
         _clock = 0;
      }

	  res = semGive( _semb );
	  if (res != OK)
	  {
		  DDA_LOG_ARG(DDA_LOG_ERR, "%s: Error giving sem\n",__func__, 0,0,0,0,0 );
	  }
   }

}




/* Heap Management : */

/******************************************************************************/

/*! \brief 	Move the element up the heap 
	\param 	this - pointer to TCB object

	The clock value is the comparison key. Move the object up the heap to the 		  *
	correct position
*/
/******************************************************************************/


static void lbt_tcbUpHeap(lbt_tcbObjTYPE *this) {
   unsigned K = this->_index;

   /* This Ensures That the While Loop Ends */
   _heap[0]=this;

   /* While Current Clock Is Less Then "Father"s Clock */
   while (this->_wakeUpTime < _heap[K>>1]->_wakeUpTime) {
      /* Swap With "Father" */
      _heap[K] = _heap[K>>1];
       _heap[K]->_index = K;
      K>>=1;
   }
   _heap[K]=this;
   this->_index=K;
}

/******************************************************************************/

/*! \brief 	Move the element down the heap 
	\param 	this - pointer to TCB object

	The clock value is the comparison key. Move the object down the heap to the 		  *
	correct position
*/
/******************************************************************************/


static void lbt_tcbDownHeap(lbt_tcbObjTYPE *this) {
   unsigned J,K=this->_index,Kmax=_inUse>>1;

   while (K<=Kmax) {
      J=K<<1;
       if (J<_inUse) {
         /* Choose The J With The Least Clock */
         /* Among Two Child Nodes !           */
            if (_heap[J|1]->_wakeUpTime < _heap[J]->_wakeUpTime) {
            J|=1;
         }
      }
      /* J Has The Least Clock Among Childs Of Current Node */
      /* If It Has A Clock Value Larger Than Ours : STOP    */
      if (this->_wakeUpTime <= _heap[J]->_wakeUpTime) {
            break;
      }

      /* We Must Interchange With That Node : */
      _heap[K]=_heap[J];
      _heap[K]->_index=K;
      K=J;
   }

   _heap[K]=this;
   this->_index=K;
}


