/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/******************************************************************************/

/*! \file  wlan_pal.h                                               
    \brief This module is platform abstraction layer and implements interface 
		   to VxWorks OS dependent services 
*/
/******************************************************************************/



#ifndef __PAL_H
#define __PAL_H

/****************************
    includes
*****************************/


#include "vxWorks.h"
#include "taskLib.h"
#include "stdlib.h"
#include "stdio.h"
#include "string.h"
#include "logLib.h"
#include "semLib.h"
#include "intLib.h"
#include "netLib.h"
#include "inetLib.h"
#include "memLib.h"
#include "sysLib.h"
#include "sysApi.h"
#include "iosLib.h"
#include "sockLib.h"
#include "socket.h"
#include "tickLib.h"
#include "muxLib.h"
#include "end.h"
#include "errno.h"
#include "errnoLib.h"
#include "in.h"
#include "pform_types.h"
#include "utilsTcb.h"
#include "pal_defs.h"
#include "pal_osProtect.h"

#ifdef _PAL_C
#   define  PUBLIC
#else
#   define  PUBLIC  extern
#endif

extern BOOL recoveryIsInProcess;

/* macros definitions */

/* key definition */
#define RL_ROOT		    1
#define RL_TEST		    2
#define RL_TCB			3
#define RL_IDLE		    4
#define RL_DB			5
#define RL_MAINSM	    6
#define RL_MGMT	        7
#define RL_HAL	        8
#define RL_TX           9
#define RL_SM	        10
#define RL_RX	        11
#define RL_RX_SNIFF	    12
#define RL_TX_SNIFF	    13
#define RL_BRIDGE	    14
#define RL_WPA			15
#define RL_HTTP	        16
#define RL_4X		    17
#define RL_MEM		    18
#define RL_DYNAMIC_ALG	19

/* level definition */
typedef enum {
   PRI_ALWAYS_PRINT = 1,
   PRI_EMERGENCY,
   PRI_ALERT,
   PRI_CRITICAL,
   PRI_ERROR,
   PRI_WARNING,
   PRI_NOTICE,
   PRI_INFORMATION,
   PRI_DEBUG
} Priority_T;

#ifdef TIWLAN_DEBUG
	#define REPORT_LEVEL(key) (PAL_reportLevel & (1<<(key)))
#else
	#define REPORT_LEVEL(key) 0
#endif

/* PALSRV_report_set - set key to the modules bitmask
   key: the key value to set
 */
#define PALSRV_report_set(key, level) \
	do { PAL_reportLevel |= 1 << (key); } while(0)
	
/* PALSRV_report_unset - unset key from the modules bitmask
   key: the key value to unset
 */
#define PALSRV_report_unset(key, level) \
	do { PAL_reportLevel &= ~(1 << (key)); } while(0)

#define PALSRV_report_disable(level) \
	do { PAL_reportLevel = 0; } while(0)

#define PALSRV_report_get() PAL_reportLevel

#define PALSRV_report_setAll() PAL_reportLevel = 0xFFFFFFFF


#define printf(format,args...)	PALSRV_report(format,##args)
#define printk(format,args...)	PALSRV_report(format,##args)
#define puts(s)					PALSRV_report("%s",s)
#define wdrv_utilsReport(key,level,format,args...) \
            PALSRV_report(format,##args)


/*! \def PAL_protectEntry(level, cookie)
	\brief Level - lock level required, cuurently supports only interrupt locking
	\brief cookie - flags for interrupt locking
	\brief Saves current interrupt state in flags and disables the interrupts
	\param flags: current interrupt state is saved in flags
 */
#define PAL_protectEntry(level, cookie) PAL_osProtectEntry(level,cookie)

/*! \def PAL_protectExit(level, cookie) 
	\brief Level - lock level required, cuurently supports only interrupt locking
	\brief cookie - flags for interrupt locking
	\brief Enables the interrupts and restores interrupt state saved in flags
	\param flags: interrupt state was saved in flags
 */
#define PAL_protectExit(level, cookie) PAL_osProtectExit(level,cookie)


#define PAL_memAlloc(segId, size, alignment, memAddr) malloc(size)
		
#define PAL_memFree(segId, addr, numBytes) free(addr)

#define PAL_memMove(dest, src, count) memmove(dest, src, count)

/* PAL_memCopy - Copy one area of memory to another
   dest: Where to copy to
   src: Where to copy from
   count: The size of the area
 */
#define PAL_memCopy(dest, src, count) memcpy(dest, src, count)

/* PAL_memSet - Fill a region of memory with the given value
   s: Pointer to the start of the area
   c: The byte to fill the area with
   count: The size of the area
 */
#define PAL_memSet(s, c, count) memset(s, c, count)


/* PAL_memCmp - Compare two areas of memory
   cs: One area of memory
   ct: Another area of memory
   count: The size of the area
 */
#define PAL_memCmp(cs, ct, count) memcmp(cs, ct, count)

#define PAL_atomic_set(pVar,val) \
do { \
	unsigned int flags; \
	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); \
	*pVar = val;  	   \
	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  \
} while(0)

#define PAL_atomic_read(pVar,val) \
do { \
	unsigned int flags; \
	PAL_protectEntry(PAL_OSPROTECT_INTERRUPT, &flags); \
	val = *pVar;  	   \
	PAL_protectExit(PAL_OSPROTECT_INTERRUPT, flags);  \
} while(0)


#define PAL_sleepLong(msecs) \
do { \
    if (!intContext()) {                                    \
      taskDelay(((UINT32)(msecs)*sysClkRateGet())/1000); }  \
    else {                                                  \
      logMsg("PAL_sleepLong(%u) is called in interrupt!!!\n",msecs,0,0,0,0,0);} \
} while(0)

typedef lbt_tcbObjTYPE * PALSRV_TcbObjTYPE;

#define PALSRV_tcbDelete(timer)  \
do { \
	if (*(timer)) {	lbt_tcbDelete( (lbt_tcbObjTYPE *) *(timer)); 	\
					*(timer)=NULL;} 							  	\
} while(0)

#define PALSRV_tcbCancel(timer)  \
do { \
	if (*(timer)) lbt_tcbCancel( (lbt_tcbObjTYPE *) *(timer)); \
} while(0)

#define PALSRV_tcbSuspend()      lbt_tcbSuspend()

#define PALSRV_tcbResume()       lbt_tcbResume()

/* Palsrv_tcbQueueAct - start a timer
   timer: the timer to be added
   timeout: when the timer expires in milliseonds
   func: the function to be executed when the timer expires
   dat: data to be passed to the function
   periodic: ignored

   Note: don't change dat to data !!!
 */
#define PALSRV_tcbQueueAct(timer, timeout, func, dat, periodic) \
						   lbt_tcbQueueAct(*timer, PALSRV_msec2ticks(timeout), \
						   func, dat, periodic)
		
#define PALSRV_msec2ticks(msecs)  ((msecs)*sysClkRateGet()/1000)

#define PALSRV_getTick()	tickGet()

#define PALSRV_ticks2usec(ticks) (ticks/sysClkRateGet()*1000000)

	/* Array Dimension */
#define PAL_DIM(array)  (sizeof(array)/sizeof(array[0]))


#define PAL_cacheFlush(addrSpace,addr,size) \
			sysFlushDataCache((UINT32)addr,size)

#define PAL_cacheInvalidate(addrSpace,addr,size) \
			sysInvalidateDataCache((UINT32)addr,size)
            
#define PAL_GET_TIMESTAMP()                         \
({ int __res;                                       \
        __asm__ __volatile__(                       \
        ".set\tpush\n\t"                            \
        ".set\treorder\n\t"                         \
        "mfc0\t%0,$9\n\t"                           \
        ".set\tpop"                                 \
        : "=r" (__res));                            \
        __res;})

/* Endianness */


/* Using inlines to avoid accessing the device more than once when the parameter 
   is a memory mapped HW. */
static __inline__ UINT16 u16bswap( UINT16 u_in)
{
   register UINT16 u = u_in;
   return (u << 8) | (u >> 8);
}

static __inline__ UINT32 u32bswap( UINT32 u_in)
{
   register UINT32 u = u_in;
   return (u << 24) | (u >> 24) | ((u >> 8) & 0xff00) | ((u & 0xff00) << 8);
}

#define ENDIAN_HANDLE_WORD(x)	(u16bswap((UINT16)(x)))
#define ENDIAN_HANDLE_LONG(x) 	(u32bswap((UINT32)(x)))

#define PAL_LE2UINT16(x) 		(u16bswap((UINT16)(x)))
#define PAL_BE2UINT16(x) 		(x)
#define	PAL_LE2UINT32(x) 		(u32bswap((UINT32)(x)))
#define	PAL_BE2UINT32(x) 		(x)

typedef int PAL_atomic_t;

PUBLIC UINT32 PAL_reportLevel;


PUBLIC STATUS 	PAL_Init(void);
PUBLIC STATUS 	PAL_Delete(void);
PUBLIC void 	PALSRV_report(const char *format, ...); 
PUBLIC STATUS 	PALSRV_tcbNew( PALSRV_TcbObjTYPE * timer);

#define _min(a,b) ( (a) < (b) ? (a) : (b))
#define _max(a,b) ( (a) > (b) ? (a) : (b))

#undef PUBLIC
#endif /* __PAL_H */
