/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2003 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/


/*!
 *
 *   \file  cli.c
 *   \brief PURPOSE: command line interface (cli) implementation
 *
 */

/***************************************************************************/
/* The Implementation of the command console is divided to two modules,    */
/* The first module, implemented in the file utilsConsole.c,is parsing the */
/* user input according to a table of commands (CommandArray[]), which     */
/* includes the command name, parameters and function. If the user's input */
/* is found in the table, the command function is called, and the parsed   */
/* parameters are passed to it. The command's functions are implemented    */
/* in command.c module.                                                    */
/***************************************************************************/

#ifndef VXWORKS
#	include <unistd.h>
#	include <string.h>
#	include <stdlib.h>
#	include <stdio.h>
#	include <sys/stat.h>
#	include <sys/socket.h>
#	include <sys/mman.h>
#	include <netinet/in.h>
#	include <arpa/inet.h>
#	include <linux/if_ether.h>
#	include <sys/klog.h>
#   include "pform_types_user.h"
#   include "wds.h"
#	include "dda_wext_ioctl.h"
#	include "utilsConsole.h"
#	include "wcfglib.h"
#	include "wcfg_ioctl.h"
#	include "wcfg_net.h"
#	include "cfg_proc_if.h"
#	include "dda_proc_fs_defs.h"
#	include "drvDbRegDomainDefs.h"
#	include "whalDefs.h"
#   include "802_11Defs.h"
#else
#	include "wlan_pal.h"
#	include "dda.h"
#	include "ctype.h"
#	include "dda_trace.h"
#	include "dda_ioctl.h"
#	include "dda_wpaauth_ioctl.h"
#	include "dda_proc.h"
#	include "utilsConsole.h"
#	include "wcfglib.h"
#	include "wcfg_ioctl.h"
#	include "wcfg_net.h"
#	include "dda_proc_fs_defs.h"
#	include "drvDbRegDomainDefs.h"
#	include "whalDefs.h"
#   include "802_11Defs.h"
#endif  /* VXWORKS */

 
static	char driver_load_failed = FALSE;
static int tiap_wcfg_start(void);

#define LAST_COMMAND  "!" /* the string for 'last command' command */

/* wcfg descriptor */
wcfgd *wd;
static const char fmtMac[] = "%02x-%02x-%02x-%02x-%02x-%02x";   /* MAC address */

typedef struct {    /* This is a storage for a list of Channel Range elements */
	tiap_param_cur_reg_domain_t cur_regdomain;
	UINT8                       ranges_nr;
	tiap_param_channel_rang_elem_t *rang_elem;
} cli_channel_range_tbl_elem_t;

/* 802.11d Allowed Channel Tables - hardcoded values */
static cli_channel_range_tbl_elem_t ch_rang_a[NUM_OF_REGULATORY_DOMAINS_A];
static cli_channel_range_tbl_elem_t ch_rang_b[NUM_OF_REGULATORY_DOMAINS_B];


__u8 reportMode = IMMEDIATE;

#define DRIVER_LOADED_ASSERT if (driver_load_failed) { \
  puts("Info is not available when WLAN driver is not loaded!"); \
  return; \
}

#ifdef VXWORKS
#	undef printf
#	undef puts
	int klogctl( int mode,  void * dummy, int idummy);
#endif

/*!	\brief 

	initialize channel range tables.
*/
void wdrv_dbInitRangeTables(void)
{
	ch_rang_a[0].cur_regdomain.type = TIAP_PARAM_RD_US;
	ch_rang_a[0].ranges_nr = NUM_OF_RANGE_ELEMENTS_US;
	if( !(ch_rang_a[0].rang_elem = 
			(tiap_param_channel_rang_elem_t *)malloc(sizeof(tiap_param_channel_rang_elem_t)*2)) ) {
		fprintf(stderr, "Allocation error.\n\n");
		exit(1);
	}
	
	ch_rang_a[0].rang_elem[0].start = REG_DOMAIN_US_START_CH_0;
	ch_rang_a[0].rang_elem[0].stop = REG_DOMAIN_US_STOP_CH_0;
	ch_rang_a[0].rang_elem[0].step = REG_DOMAIN_US_STEP_OPT_0;
	ch_rang_a[0].rang_elem[0].power_limit = REG_DOMAIN_US_POWER_LIMIT_CH_0;
	ch_rang_a[0].rang_elem[1].start = REG_DOMAIN_US_START_CH_1;
	ch_rang_a[0].rang_elem[1].stop = REG_DOMAIN_US_STOP_CH_1;
	ch_rang_a[0].rang_elem[1].step = REG_DOMAIN_US_STEP_OPT_1;
	ch_rang_a[0].rang_elem[1].power_limit = REG_DOMAIN_US_POWER_LIMIT_CH_1;

	ch_rang_a[1].cur_regdomain.type = TIAP_PARAM_RD_WORLD;
	ch_rang_a[1].ranges_nr = NUM_OF_RANGE_ELEMENTS_WORLD;
	if( !(ch_rang_a[1].rang_elem =
			(tiap_param_channel_rang_elem_t *)malloc(sizeof(tiap_param_channel_rang_elem_t)*3)) ) {
		fprintf(stderr, "Allocation error.\n\n");
		exit(1);
	}
	ch_rang_a[1].rang_elem[0].start = REG_DOMAIN_WORLD_START_CH_0;
	ch_rang_a[1].rang_elem[0].stop  = REG_DOMAIN_WORLD_STOP_CH_0;
	ch_rang_a[1].rang_elem[0].step = REG_DOMAIN_WORLD_STEP_OPT_0;
	ch_rang_a[1].rang_elem[0].power_limit = REG_DOMAIN_WORLD_POWER_LIMIT_CH_0;
	ch_rang_a[1].rang_elem[1].start = REG_DOMAIN_WORLD_START_CH_1;
	ch_rang_a[1].rang_elem[1].stop  = REG_DOMAIN_WORLD_STOP_CH_1;
	ch_rang_a[1].rang_elem[1].step = REG_DOMAIN_WORLD_STEP_OPT_1;
	ch_rang_a[1].rang_elem[1].power_limit = REG_DOMAIN_WORLD_POWER_LIMIT_CH_1;
	ch_rang_a[1].rang_elem[2].start = REG_DOMAIN_WORLD_START_CH_2;
	ch_rang_a[1].rang_elem[2].stop = REG_DOMAIN_WORLD_STOP_CH_2;
	ch_rang_a[1].rang_elem[2].step = REG_DOMAIN_WORLD_STEP_OPT_2;
	ch_rang_a[1].rang_elem[2].power_limit = REG_DOMAIN_WORLD_POWER_LIMIT_CH_2;

	/*              - G - band              */
	ch_rang_b[0].cur_regdomain.type = TIAP_PARAM_RD_FCC;
	ch_rang_b[0].ranges_nr = 1;
	if( !(ch_rang_b[0].rang_elem =
		(tiap_param_channel_rang_elem_t *)malloc(sizeof(tiap_param_channel_rang_elem_t))) ) {
		fprintf(stderr, "Allocation error.\n\n");
		exit(1);
	}
	ch_rang_b[0].rang_elem[0].start = REG_DOMAIN_FCC_START_CH_0;
	ch_rang_b[0].rang_elem[0].stop  = REG_DOMAIN_FCC_STOP_CH_0;
	ch_rang_b[0].rang_elem[0].step = REG_DOMAIN_FCC_STEP_OPT_0;
	ch_rang_b[0].rang_elem[0].power_limit = REG_DOMAIN_FCC_POWER_LIMIT_CH_0;

	ch_rang_b[1].cur_regdomain.type = TIAP_PARAM_RD_IC;
	ch_rang_b[1].ranges_nr = 1;
	if( !(ch_rang_b[1].rang_elem =
		(tiap_param_channel_rang_elem_t *)malloc(sizeof(tiap_param_channel_rang_elem_t))) ) {
		fprintf(stderr, "Allocation error.\n\n");
		exit(1);
	}
	ch_rang_b[1].rang_elem[0].start = REG_DOMAIN_IC_START_CH_0;
	ch_rang_b[1].rang_elem[0].stop  = REG_DOMAIN_IC_STOP_CH_0;
	ch_rang_b[1].rang_elem[0].step = REG_DOMAIN_IC_STEP_OPT_0;
	ch_rang_b[1].rang_elem[0].power_limit = REG_DOMAIN_IC_POWER_LIMIT_CH_0;

	ch_rang_b[2].cur_regdomain.type = TIAP_PARAM_RD_ETSI;
	ch_rang_b[2].ranges_nr = 1;
	if( !(ch_rang_b[2].rang_elem =
		(tiap_param_channel_rang_elem_t *)malloc(sizeof(tiap_param_channel_rang_elem_t))) ) {
		fprintf(stderr, "Allocation error.\n\n");
		exit(1);
	}
	ch_rang_b[2].rang_elem[0].start = REG_DOMAIN_ETSI_START_CH_0;
	ch_rang_b[2].rang_elem[0].stop  = REG_DOMAIN_ETSI_STOP_CH_0;
	ch_rang_b[2].rang_elem[0].step = REG_DOMAIN_ETSI_STEP_OPT_0;
	ch_rang_b[2].rang_elem[0].power_limit = REG_DOMAIN_ETSI_POWER_LIMIT_CH_0;

	ch_rang_b[3].cur_regdomain.type = TIAP_PARAM_RD_SPAIN;
	ch_rang_b[3].ranges_nr = 1;
	if( !(ch_rang_b[3].rang_elem =
		(tiap_param_channel_rang_elem_t *)malloc(sizeof(tiap_param_channel_rang_elem_t))) ) {
		fprintf(stderr, "Allocation error.\n\n");
		exit(1);
	}
	ch_rang_b[3].rang_elem[0].start = REG_DOMAIN_SPAIN_START_CH_0;
	ch_rang_b[3].rang_elem[0].stop  = REG_DOMAIN_SPAIN_STOP_CH_0;
	ch_rang_b[3].rang_elem[0].step = REG_DOMAIN_SPAIN_STEP_OPT_0;
	ch_rang_b[3].rang_elem[0].power_limit = REG_DOMAIN_SPAIN_POWER_LIMIT_CH_0;

	ch_rang_b[4].cur_regdomain.type = TIAP_PARAM_RD_FRANCE;
	ch_rang_b[4].ranges_nr = 1;
	if( !(ch_rang_b[4].rang_elem =
		(tiap_param_channel_rang_elem_t *)malloc(sizeof(tiap_param_channel_rang_elem_t))) ) {
		fprintf(stderr, "Allocation error.\n\n");
		exit(1);
	}
	ch_rang_b[4].rang_elem[0].start = REG_DOMAIN_FRANCE_START_CH_0;
	ch_rang_b[4].rang_elem[0].stop  = REG_DOMAIN_FRANCE_STOP_CH_0;
	ch_rang_b[4].rang_elem[0].step = REG_DOMAIN_FRANCE_STEP_OPT_0;
	ch_rang_b[4].rang_elem[0].power_limit = REG_DOMAIN_FRANCE_POWER_LIMIT_CH_0;

	ch_rang_b[5].cur_regdomain.type = TIAP_PARAM_RD_MKK;
	ch_rang_b[5].ranges_nr = 1;
	if( !(ch_rang_b[5].rang_elem =
		(tiap_param_channel_rang_elem_t *)malloc(sizeof(tiap_param_channel_rang_elem_t))) ) {
		fprintf(stderr, "Allocation error.\n\n");
		exit(1);
	}
	ch_rang_b[5].rang_elem[0].start = REG_DOMAIN_MKK_START_CH_0;
	ch_rang_b[5].rang_elem[0].stop = REG_DOMAIN_MKK_STOP_CH_0;
	ch_rang_b[5].rang_elem[0].step = REG_DOMAIN_MKK_STEP_OPT_0 | NO_OFDM_OPT;
	ch_rang_b[5].rang_elem[0].power_limit = REG_DOMAIN_MKK_POWER_LIMIT_CH_0;

	ch_rang_b[6].cur_regdomain.type = TIAP_PARAM_RD_MKK1;
	ch_rang_b[6].ranges_nr = 2;
	if( !(ch_rang_b[6].rang_elem =
		(tiap_param_channel_rang_elem_t *)malloc(sizeof(tiap_param_channel_rang_elem_t)*2)) ) {
		fprintf(stderr, "Allocation error.\n\n");
		exit(1);
	}
	ch_rang_b[6].rang_elem[0].start = REG_DOMAIN_MKK1_START_CH_0;
	ch_rang_b[6].rang_elem[0].stop  = REG_DOMAIN_MKK1_STOP_CH_0;
	ch_rang_b[6].rang_elem[0].step = REG_DOMAIN_MKK1_STEP_OPT_0;
	ch_rang_b[6].rang_elem[0].power_limit = REG_DOMAIN_MKK1_POWER_LIMIT_CH_0;
	ch_rang_b[6].rang_elem[1].start = REG_DOMAIN_MKK1_START_CH_1;
	ch_rang_b[6].rang_elem[1].stop  = REG_DOMAIN_MKK1_STOP_CH_1;
	ch_rang_b[6].rang_elem[1].step = REG_DOMAIN_MKK1_STEP_OPT_1 | NO_OFDM_OPT;
	ch_rang_b[6].rang_elem[1].power_limit = REG_DOMAIN_MKK1_POWER_LIMIT_CH_1;

    ch_rang_b[7].cur_regdomain.type = TIAP_PARAM_RD_ISRAEL;
	ch_rang_b[7].ranges_nr = 1;
	if( !(ch_rang_b[7].rang_elem =
		(tiap_param_channel_rang_elem_t *)malloc(sizeof(tiap_param_channel_rang_elem_t))) ) {
		fprintf(stderr, "Allocation error.\n\n");
		exit(1);
	}
	ch_rang_b[7].rang_elem[0].start = REG_DOMAIN_ISRAEL_START_CH_0;
	ch_rang_b[7].rang_elem[0].stop  = REG_DOMAIN_ISRAEL_STOP_CH_0;
	ch_rang_b[7].rang_elem[0].step = REG_DOMAIN_ISRAEL_STEP_OPT_0;
	ch_rang_b[7].rang_elem[0].power_limit = REG_DOMAIN_ISRAEL_POWER_LIMIT_CH_0;
}

/*!	\def - convert_rates(tv, rates_dest, rates_src, len)
	\brief convert_rates
  	\param tv - table values
  	\param rates_dest - destination table
  	\param rates_src - source table
  	\param len - rates_dest size

  	\return 
  
*/
#define convert_rates(tv, rates_dest, rates_src, len)    		 \
do {                                            			 \
	int i, k;							 \
	for(i=0,k=0; i<len; i++)        				 \
		if(rates_src[i] == TIAP_PARAM_RATE_SUPPORTED)		 \
			rates_dest[k++] = tv[i];               		 \
		else if(rates_src[i] == TIAP_PARAM_RATE_BASIC_SUPPORTED) \
			rates_dest[k++] = tv[i] | 0x80;			 \
} while(0)


/*!	\brief get_domain_elems_num
  	\param band - band type
  	\param domain - domain type

  	\return number of range elements 
  
*/
static int get_domain_elems_num(int band, __u8 domain)
{
	switch (band)
	{
		case BAND_A:
			switch (domain)
			{
				case TIAP_PARAM_RD_US:
					return NUM_OF_RANGE_ELEMENTS_US;

				case TIAP_PARAM_RD_WORLD:
					return NUM_OF_RANGE_ELEMENTS_WORLD;

				default:
					break;
			}
			break;

		case BAND_B:
			switch (domain)
			{
				case TIAP_PARAM_RD_FCC:
					return 1;

				case TIAP_PARAM_RD_IC:
					return 1;

				case TIAP_PARAM_RD_ETSI:
					return 1;

				case TIAP_PARAM_RD_SPAIN:
					return 1;

				case TIAP_PARAM_RD_FRANCE:
					return 1;

				case TIAP_PARAM_RD_MKK:
					return 1;

				case TIAP_PARAM_RD_MKK1:
					return 2;

                case TIAP_PARAM_RD_ISRAEL:
                    return 1;

				default:
					break;
			}
			break;

		default:
			break;
	}
	return 0;
}


/*!	\brief 
  	\param 

  	\return 
  
*/
int is_std_reg_domain(__u8 domain)
{
	switch (domain)
	{
		case TIAP_PARAM_RD_FCC:
		case TIAP_PARAM_RD_IC:
		case TIAP_PARAM_RD_ETSI:
		case TIAP_PARAM_RD_SPAIN:
		case TIAP_PARAM_RD_FRANCE:
		case TIAP_PARAM_RD_MKK:
		case TIAP_PARAM_RD_MKK1:
        case TIAP_PARAM_RD_ISRAEL:
		case TIAP_PARAM_RD_US:
		case TIAP_PARAM_RD_WORLD:
			return 1;
	}
	return 0;
}

static int get_rangeElement(int band, tiap_param_cur_reg_domain_t cur_reg_domain,
			   UINT8 RangeIndex,tiap_param_channel_rang_elem_t *rang_elem);

extern int wapp_utilsConsoleCurrentContext;
void consoleTask(void);
//extern L2ADDRESS MAC_address;

static UINT8 currentConfigWoneInex;

void BITCB(ConsoleParameter_T parameters[]);
void MgmtCB(ConsoleParameter_T parameters[]);
void MgmtTstCB(ConsoleParameter_T parameters[]);
void WpaCB(ConsoleParameter_T parameters[]);
void WpaTstCB(ConsoleParameter_T parameters[]);
void WpaRepCB(ConsoleParameter_T parameters[]);
void ServCB(ConsoleParameter_T parameters[]);
void HalCB(ConsoleParameter_T parameters[]);
void HalDiagCB(ConsoleParameter_T parameters[]);
void HwInterfCB(ConsoleParameter_T parameters[]);
void ProdCB(ConsoleParameter_T parameters[]);
void SecurityCB(ConsoleParameter_T parameters[]);
void BringUpCB(ConsoleParameter_T parameters[]);
void BringUpTxCB(ConsoleParameter_T parameters[]);
void WdsCB(ConsoleParameter_T parameters[]);

/************************************************************************/
/*						Root menu functions								*/
/************************************************************************/
void consoleMac(ConsoleParameter_T parameters[]);
void consoleReportLevel(ConsoleParameter_T parameters[]);
void consoleReportMode(ConsoleParameter_T parameters[]);
void consoleLastCommand(ConsoleParameter_T parameters[]);
void consoleReboot(ConsoleParameter_T parameters[]);
void consoleVersion(ConsoleParameter_T parameters[]);

/************************************************************************/
/*						BIT sub-menu functions							*/
/************************************************************************/
void consoleBitFcc(ConsoleParameter_T parameters[]);
void consoleBitTelec(ConsoleParameter_T parameters[]);

/************************************************************************/
/*						Mgmt sub-menu functions	 DONE						*/
/************************************************************************/
void printMgmtAccess(ConsoleParameter_T parameters[]);
void setAccessMAC(ConsoleParameter_T parameters[]);
void remAccessMAC(ConsoleParameter_T parameters[]);
void enableAccess(ConsoleParameter_T parameters[]);
void disableAccess(ConsoleParameter_T parameters[]);
void printHashTable(ConsoleParameter_T parameters[]);
void printStaTable(ConsoleParameter_T parameters[]);
void printStaStatistics(ConsoleParameter_T parameters[]);
void printRatesTable(ConsoleParameter_T parameters[]);
void printMgmtCounters(ConsoleParameter_T parameters[]);

void deAuthMng(ConsoleParameter_T parameters[]);
void disAssocMng(ConsoleParameter_T parameters[]);
void mgmtLog(ConsoleParameter_T parameters[]);
void mgmtAging(ConsoleParameter_T parameters[]);
void mgmtShow(ConsoleParameter_T parameters[]);

/************************************************************************/
/*						Mgmt Test sub-menu functions DONE							*/
/************************************************************************/
void findSta(ConsoleParameter_T parameters[]);
void insertSta(ConsoleParameter_T parameters[]);
void removeSta(ConsoleParameter_T parameters[]);
void null(ConsoleParameter_T parameters[]);
void auth1(ConsoleParameter_T parameters[]);
void auth3(ConsoleParameter_T parameters[]);
void assoc(ConsoleParameter_T parameters[]);
void reAssoc(ConsoleParameter_T parameters[]);	 
void deAuth(ConsoleParameter_T parameters[]);
void disAssoc(ConsoleParameter_T parameters[]);
void class3(ConsoleParameter_T parameters[]);
void showChallenge(ConsoleParameter_T parameters[]);
void mgmtReset(ConsoleParameter_T parameters[]);

/************************************************************************/
/*						privacy sub-menu functions	   WORKS						*/
/************************************************************************/
void wpaWorkingMode(ConsoleParameter_T parameters[]);
void wpaPassPhraseMode(ConsoleParameter_T parameters[]);
void wpaPskHex(ConsoleParameter_T parameters[]);
void wpaShow(ConsoleParameter_T parameters[]);
void wpaDefaultCipher(ConsoleParameter_T parameters[]);
void wpaGTKInterval(ConsoleParameter_T parameters[]);
void wpaPskPwd(ConsoleParameter_T parameters[]);
void wpaPrintPSKinHexFormat(ConsoleParameter_T parameters[]);
void wpaRadiusSecret(ConsoleParameter_T parameters[]);
void wpaRadiusPort(ConsoleParameter_T parameters[]);
void wpaRadiusAddr(ConsoleParameter_T parameters[]);
void wpaDefaultGateway(ConsoleParameter_T parameters[]);
void wpaPreAuthflag(ConsoleParameter_T parameters[]);
void privacySetPrivacyWEPDefaultKeyId(ConsoleParameter_T parameters[]);
void privacySetDefaultKey5(ConsoleParameter_T parameters[]);
void privacySetDefaultKey13(ConsoleParameter_T parameters[]);
void privacySetDefaultKey29(ConsoleParameter_T parameters[]);

/* wep */
void privacySetPrivacyWEPDefaultKeyId(ConsoleParameter_T parameters[]);
void privacySetDefaultKey5(ConsoleParameter_T parameters[]);
void privacySetDefaultKey13(ConsoleParameter_T parameters[]);
void privacySetDefaultKey29(ConsoleParameter_T parameters[]);
void privacySetAuthAlgo(ConsoleParameter_T parameters[]);

/************************************************************************/
/*						WPA Test sub-menu functions							*/
/************************************************************************/
void assocStation(ConsoleParameter_T parameters[]);
void reAssocStation(ConsoleParameter_T parameters[]);
void stationsRsnIe(ConsoleParameter_T parameters[]);
void macAuthorize(ConsoleParameter_T parameters[]);
void macDeAuthorize(ConsoleParameter_T parameters[]);
void showWpaConfig(ConsoleParameter_T parameters[]);
void isMacAuthorized(ConsoleParameter_T parameters[]);
void setStationSuite(ConsoleParameter_T parameters[]);
void setBcastSuite(ConsoleParameter_T parameters[]);
void testVectors(ConsoleParameter_T parameters[]);
void setRxMicKey(ConsoleParameter_T parameters[]);
void setTxMicKey(ConsoleParameter_T parameters[]);
void setMicReportInRx(ConsoleParameter_T parameters[]);
void setMicReportFromStation(ConsoleParameter_T parameters[]);
void generateTxMicFailure(ConsoleParameter_T parameters[]);

/************************************************************************/
/*						WPA Reports sub-menu functions					*/
/************************************************************************/
void wpa_repConfSeverity(ConsoleParameter_T parameters[]);
void wpa_repConfReportModule(ConsoleParameter_T parameters[]);
void wpa_reportShow(ConsoleParameter_T parameters[]);
void wpa_reportPrintStaTbl(ConsoleParameter_T parameters[]);
void wpa_reportPrintStatistics(ConsoleParameter_T parameters[]);
void wpa_reportPrintProfCfg(ConsoleParameter_T parameters[]);
void wpa_reportPrintPmkCacheList(ConsoleParameter_T parameters[]);

/************************************************************************/
/* Service Application sub-menu functions				*/
/************************************************************************/
void servEnable(ConsoleParameter_T parameters[]);
void servDisable(ConsoleParameter_T parameters[]);
void wlanSetDualAp(ConsoleParameter_T parameters[]);
void wlanSetWoneConfIndex(ConsoleParameter_T parameters[]);
void wlanSetUsageTime(ConsoleParameter_T parameters[]);
void wlanRadioParams(ConsoleParameter_T parameters[]);
void wlanSetUsageTimeMode(ConsoleParameter_T parameters[]);
void wlanSetMacAddr(ConsoleParameter_T parameters[]);
void wlanSetRates(ConsoleParameter_T parameters[]);
void wlanSetGRates(ConsoleParameter_T parameters[]);
void wlanSetAdditionalRates(ConsoleParameter_T parameters[]);
void wlanSetRespRateMode(ConsoleParameter_T parameters[]);
void wlanSetChannel(ConsoleParameter_T parameters[]);
void wlanSetBeacon(ConsoleParameter_T parameters[]);
void wlanSetDtim(ConsoleParameter_T parameters[]);
void wlanSetEepromLess(ConsoleParameter_T parameters[]);
void wlanSetRtsThreshold(ConsoleParameter_T parameters[]);
void wlanSetShoreRetryLimit(ConsoleParameter_T parameters[]);
void wlanSetLongRetryLimit(ConsoleParameter_T parameters[]);
void wlanSetRateFallback(ConsoleParameter_T parameters[]);
void wlanSetRateAdaptation(ConsoleParameter_T parameters[]);
void wlanSetFragmentationTreshold(ConsoleParameter_T parameters[]);
void wlanSetMaxTransmitMSDULifetime(ConsoleParameter_T parameters[]);
void wlanSetMaxReceiveLifetime(ConsoleParameter_T parameters[]);
void wlanPrintParams(ConsoleParameter_T parameters[]);
void wlanPrintRatesParams(ConsoleParameter_T parameters[]);
void wlanPrintCounters(ConsoleParameter_T parameters[]);
void wlanSetNvramDefaults(ConsoleParameter_T parameters[]);
void wlanSetAgile(ConsoleParameter_T parameters[]);
void wlanSetPreambleMode(ConsoleParameter_T parameters[]);
void wlanSetPBCCAlgorithmMode(ConsoleParameter_T parameters[]);
void wlanSetSlotTimeMode(ConsoleParameter_T parameters[]);
void wlanSetSlotTimeThreshold(ConsoleParameter_T parameters[]);
void wlanSetERPProtectionMode(ConsoleParameter_T parameters[]);
void wlanSetERPProtectionType(ConsoleParameter_T parameters[]);
void wlanSetDot11Mode(ConsoleParameter_T parameters[]);
void wlanSetCWmin(ConsoleParameter_T parameters[]);
void wlanSetDynAlgoTimeout(ConsoleParameter_T parameters[]);
void wlanSetRadioCalibrateOption(ConsoleParameter_T parameters[]);
void wlanSetRadioCalibrateInterval(ConsoleParameter_T parameters[]);
void wlanSetBssIsolation(ConsoleParameter_T parameters[]);
void wlanPrintBssIsolationState(ConsoleParameter_T parameters[]);
void wlanSetRadarDetect(ConsoleParameter_T parameters[]);
void wlanPrintRadarDetect(ConsoleParameter_T parameters[]);
void wlanSetApBridgeMode(ConsoleParameter_T parameters[]);

/************************************************************************/
/* SSID Features sub-menu functions                                     */
/************************************************************************/
void wlanSsidPrintParams(ConsoleParameter_T parameters[]);
void wlanSSID(ConsoleParameter_T parameters[]);
void wlanSetHiddenSSID(ConsoleParameter_T parameters[]);
void wlanSetMultipleSSID(ConsoleParameter_T parameters[]);
void wlanAddMultipleSsidElement(ConsoleParameter_T parameters[]);
void wlanRemoveMultipleSsidElement(ConsoleParameter_T parameters[]);

/************************************************************************/
/* QoS Features sub-menu functions                                      */
/************************************************************************/

void wlanQoSEnableWME(ConsoleParameter_T parameters[]);
void wlanQoSPrintWMEConf(ConsoleParameter_T parameters[]);
void wlanQoSSetAC(ConsoleParameter_T parameters[]);

/************************************************************************/
/* 802.11 D & H sub-menu functions                                      */
/************************************************************************/
void wlan_dh_PrintParams(ConsoleParameter_T parameters[]);
void wlan_dh_PrintRegDomain(ConsoleParameter_T parameters[]);
void wlanSetMultiDomain(ConsoleParameter_T parameters[]);
void wlanSetSpectrumMgmt(ConsoleParameter_T parameters[]);
void wlanSetLegacyStation(ConsoleParameter_T parameters[]);
void wlanSetPowerConstraint(ConsoleParameter_T parameters[]);
void wlanSetRegulatoryDomain(ConsoleParameter_T parameters[]);
void wlanSetTxPowerLevel(ConsoleParameter_T parameters[]);
void wlanSetPowerLevel(ConsoleParameter_T parameters[]);
void wlanSetCountryString(ConsoleParameter_T parameters[]);
void wlanAddChnRangToCustomRegDomain(ConsoleParameter_T parameters[]);

/************************************************************************/
/* Show sub-menu functions 		                                        */
/************************************************************************/
void showBssBridge(ConsoleParameter_T parameters[]);
void showPowerSave(ConsoleParameter_T parameters[]);
void showPrintTxStat(ConsoleParameter_T parameters[]);
void showMibWlan(ConsoleParameter_T parameters[]);
void showLanStat(ConsoleParameter_T parameters[]);
void showLanPool(ConsoleParameter_T parameters[]);
void showMemMngrStat(ConsoleParameter_T parameters[]);
void showRateAdaptation(ConsoleParameter_T parameters[]);

/************************************************************************/
/* Hal sub-menu functions												*/
/************************************************************************/

void whalMenu_printRxBufs(ConsoleParameter_T parameters[]);
void whalMenu_eed(ConsoleParameter_T parameters[]);
void whalMenu_eer(ConsoleParameter_T parameters[]);
void whalMenu_eew(ConsoleParameter_T parameters[]);
void whalMenu_printFwVersion(ConsoleParameter_T parameters[]);
void whalMenu_printAcxParams(ConsoleParameter_T parameters[]);
void whalMenu_printTxInfo(ConsoleParameter_T parameters[]);
void whalMenu_printRxInfo(ConsoleParameter_T parameters[]);
void whalMenu_printTxQueue(ConsoleParameter_T parameters[]);
void whalMenu_printRxQueue(ConsoleParameter_T parameters[]);
void whalMenu_printIntr(ConsoleParameter_T parameters[]);
void whalMenu_printMbox(ConsoleParameter_T parameters[]);
void whalMenu_printAcxStat(ConsoleParameter_T parameters[]);
void whalMenu_printRestartStat(ConsoleParameter_T parameters[]);
void whalMenu_EnableEnergyDetect(ConsoleParameter_T parameters[]);
void whalMenu_EnergyDetectShow(ConsoleParameter_T parameters[]);
void whalMenu_setWAoptions(ConsoleParameter_T parameters[]);
void whalMenu_setAciRateThresh(ConsoleParameter_T parameters[]);
void whalMenu_setAciEventThresh(ConsoleParameter_T parameters[]);
void whalMenu_setAciTimeout(ConsoleParameter_T parameters[]);
void whalMenu_setNbTimeout(ConsoleParameter_T parameters[]);
void whalMenu_setPdThresh(ConsoleParameter_T parameters[]);
void whalMenu_setBusyMediumThresh(ConsoleParameter_T parameters[]);
void whalMenu_showACIparams(ConsoleParameter_T parameters[]);
void whalMenu_EnableRestartWA(ConsoleParameter_T parameters[]);
void whalMenu_EnableInterfWA(ConsoleParameter_T parameters[]);
void whalMenu_gpio_owner(ConsoleParameter_T []);
void whalMenu_printMboxErrors(ConsoleParameter_T parameters[]);
void whalMenu_mediumUsage(ConsoleParameter_T parameters[]);

/************************************************************************/
/* Hal Diag sub-menu functions											*/
/************************************************************************/
void whalMenu_SetBssPs(ConsoleParameter_T parameters[]);
void whalMenu_SetTxCompleteTrsh(ConsoleParameter_T parameters[]);
void whalMenu_PrintRegs(ConsoleParameter_T parameters[]);
void whalMenu_PhyRegWrite(ConsoleParameter_T parameters[]);
void whalMenu_PhyRegRead(ConsoleParameter_T parameters[]);
void whalMenu_MacRegWrite(ConsoleParameter_T parameters[]);
void whalMenu_MacRegRead(ConsoleParameter_T parameters[]);
void whalMenu_RxPromiscuous(ConsoleParameter_T parameters[]);
void whalMenu_ModeSniffer(ConsoleParameter_T parameters[]);
void whalMenu_SetNvByte(ConsoleParameter_T parameters[]);
void whalMenu_TraceEnable(ConsoleParameter_T parameters[]);
void whalMenu_TracePrint(ConsoleParameter_T parameters[]);
void whalMenu_TracePrintRxStat(ConsoleParameter_T parameters[]);
void whalMenu_SelectFwNumber(ConsoleParameter_T parameters[]);
void whalMenu_SetQueueSizeType(ConsoleParameter_T parameters[]);
void whalMenu_SelectRadioNumber(ConsoleParameter_T parameters[]);
void whalMenu_DownloadFW(ConsoleParameter_T parameters[]);
void whalMenu_DM(ConsoleParameter_T parameters[]);
void whalMenu_MM(ConsoleParameter_T parameters[]);
void whalMenu_DebugServerIP(ConsoleParameter_T parameters[]);

/************************************************************************/
/* Hal HW interference configuration sub-menu functions											*/
/************************************************************************/
void whalMenu_inputCCA(ConsoleParameter_T parameters[]);
void whalMenu_txCCA(ConsoleParameter_T parameters[]);
void whalMenu_qualifCCA(ConsoleParameter_T parameters[]);
void whalMenu_stompTx(ConsoleParameter_T parameters[]);
void whalMenu_stompRx(ConsoleParameter_T parameters[]);
void whalMenu_hwShow(ConsoleParameter_T parameters[]);

/************************************************************************/
/* production sub-menu functions                                        */
/************************************************************************/
#ifdef VXWORKS
void consoleDir(ConsoleParameter_T parameters[]);
void consoleErase(ConsoleParameter_T parameters[]);
void consoleDl(ConsoleParameter_T parameters[]);
void consoleSetDef(ConsoleParameter_T parameters[]);
void consoleProdShow(ConsoleParameter_T parameters[]);
void consoleProdSet(ConsoleParameter_T parameters[]);
void consoleProdDef(ConsoleParameter_T parameters[]);
void consolePassword(ConsoleParameter_T parameters[]);
#endif /* VXWORKS */


/************************************************************************/
/* 4X sub-menu functions  ----------------- DONE ----------------------												*/
/************************************************************************/

void fourXTable(ConsoleParameter_T parameters[]);
void show4xParams(ConsoleParameter_T parameters[]);
void set4xMaxConcatSize(ConsoleParameter_T parameters[]);
void set4x(ConsoleParameter_T parameters[]);
void setPacketBursting(ConsoleParameter_T parameters[]);
void setConcatDebug(ConsoleParameter_T parameters[]);
void wlanSet4xMixMode(ConsoleParameter_T parameters[]);
void set4xFeatureSet(ConsoleParameter_T parameters[]);
void setBurstLimit(ConsoleParameter_T parameters[]);

/************************************************************************/
/* WDS sub-menu functions                                               */
/************************************************************************/
void applyWdsParams(ConsoleParameter_T parameters[]);  
void enableWds(ConsoleParameter_T parameters[]);  
void showWdsParams(ConsoleParameter_T parameters[]);
void showWdsReport(ConsoleParameter_T parameters[]);    
void showWdsStat(ConsoleParameter_T parameters[]);    
void showWdsInfoDb(ConsoleParameter_T parameters[]);    
void setWdsHop(ConsoleParameter_T parameters[]);        
void setWdsChSel(ConsoleParameter_T parameters[]);      
void setWdsAutoCfg(ConsoleParameter_T parameters[]);    
void setWdsPrivacy(ConsoleParameter_T parameters[]);    
void setWdsSharedSecret(ConsoleParameter_T parameters[]);    
void setWdsName(ConsoleParameter_T parameters[]);       
void setWdsBssMode(ConsoleParameter_T parameters[]);    
void setWdsCrudeMode(ConsoleParameter_T parameters[]);    
void setWdsUplink(ConsoleParameter_T parameters[]);     
void setWdsDownlink0(ConsoleParameter_T parameters[]);  
void setWdsDownlink1(ConsoleParameter_T parameters[]);  
void setWdsDownlink2(ConsoleParameter_T parameters[]);  
void setWdsDownlink3(ConsoleParameter_T parameters[]);  
void setWdsPrintSeverity(ConsoleParameter_T parameters[]);
void setWdsRestart(ConsoleParameter_T parameters[]);
void setWdsDefaults(ConsoleParameter_T parameters[]);

/************************************************************************/
/* IP sub-menu functions												*/
/************************************************************************/
void setIpAddr(ConsoleParameter_T parameters[]);
void setIpMask(ConsoleParameter_T parameters[]);
void setDefGateway(ConsoleParameter_T parameters[]);
void applyIpSettings( ConsoleParameter_T parameters[] );
void showIpSettings(ConsoleParameter_T parameters[]); 

/* auxilary functions */
int wpaGetPSKinHexFormat(int profile_index, char *psk);
/*void wpaCalcPSKinHexFormat(int profile_index, char* pskStr);*/

static void access_list_to_str(char *buff, int buff_size,
			       tiap_param_access_list_t access_list, 
			       int *num_entries);
static void set_enable_wa_restart(int val);

/************************************************************************/
/*						The console root menu							*/
/************************************************************************/
/*!	\brief 
  	\param 

  	\return 
  
*/
ConsoleCommand_T wapp_utilsConsoleRootCommandArray[] = {
   { "replevel",    1, consoleReportLevel,		{INTEGER_PRM,  NO_PARAM,      NO_PARAM,     NO_PARAM,     NO_PARAM}, "Update [set|reset] Report level: repl [-|+]<level_id>" } ,
   { "RepMode",    1, consoleReportMode,		{INTEGER_PRM,  NO_PARAM,      NO_PARAM,     NO_PARAM,     NO_PARAM}, "1 - IMMEDIATE, 0 - DEFERRED" }, 
   { "reboot",      0, consoleReboot,        {NO_PARAM,      NO_PARAM,     NO_PARAM,     NO_PARAM}, "reboot the AP" },
   { "version",     0, consoleVersion,        {NO_PARAM,      NO_PARAM,     NO_PARAM,     NO_PARAM}, "Display version information" },
   { LAST_COMMAND,  0, consoleLastCommand,		{NO_PARAM,     NO_PARAM,      NO_PARAM,     NO_PARAM,     NO_PARAM}, "Execute the last command" },
   WAPP_UTILS_CONSOLE_MAIN_ARRAY_POSTFIX,
   { "nullcommand",0, NULL,						{NO_PARAM,     NO_PARAM,      NO_PARAM,     NO_PARAM,     NO_PARAM}, "Undefined" },
};

/**********************************************************/
/*						BIT sub-menu									 */
/**********************************************************/
/*!	\brief 
  	\param 

  	\return 
  
*/
ConsoleCommand_T ConsoleBITCommandArray[] = {
	{ "fcc",          0, consoleBitFcc		,	{NO_PARAM,      NO_PARAM,      NO_PARAM,      NO_PARAM,      NO_PARAM    }, "fcc - interactive selection of band, channel, rate and mode" },
	{ "telec",        0, consoleBitTelec	,	{NO_PARAM,      NO_PARAM,      NO_PARAM,      NO_PARAM,      NO_PARAM    }, "telec - interactive selection of channel per current band" },
   WAPP_UTILS_CONSOLE_SUB_MENU_ARRAY_POSTFIX,
   { "nullcommand",	0, NULL,                {NO_PARAM,      NO_PARAM,     NO_PARAM,       NO_PARAM,      NO_PARAM    }, "Undefined" },
};


/************************************************************************/
/*						Mgmt sub-menu									*/
/************************************************************************/
/*!	\brief 
  	\param 

  	\return 
  
*/
ConsoleCommand_T ConsoleMgmtCommandArray[] = {
	{ "accessPrint", 0, printMgmtAccess, {NO_PARAM, NO_PARAM, NO_PARAM, NO_PARAM, NO_PARAM}, "Print Access List"},
	{ "accessSetMAC", 1, setAccessMAC, {MAC_ADDR_PRM, NO_PARAM, NO_PARAM, NO_PARAM, NO_PARAM}, "Add MAC address to access list <mac>"},
	    { "accessRemMAC", 0, remAccessMAC, {NO_PARAM, NO_PARAM, NO_PARAM, NO_PARAM, NO_PARAM}, "Remove MAC address from access list"},
	{ "accessEnable", 1, enableAccess, {SHORT_PRM, NO_PARAM, NO_PARAM, NO_PARAM, NO_PARAM}, "Enable access list <type(1-black, 0-white)>"},
	{ "accessDisable", 0, disableAccess, {NO_PARAM, NO_PARAM, NO_PARAM, NO_PARAM, NO_PARAM}, "Disable access list"},
	{ "deauthmng",			2, deAuthMng,			{MAC_ADDR_PRM,   SHORT_PRM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Send a DeAuthentication message to a station <mac> <reason>" },
	{ "disassocmng",		2, disAssocMng,			{MAC_ADDR_PRM,   SHORT_PRM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Send a DisAssociation message to a station <mac> <reason>" },
	{ "staTable",			0, printStaTable,		{NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Print the station table" },
    { "staStatistics",		0, printStaStatistics,	{NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Print the stations statistics" },
	{ "hashTable",			0, printHashTable,		{NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Print the hash table" },
	{ "ratesTable",			0, printRatesTable,		{NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Print the station rates table" },
	{ "counters",			0, printMgmtCounters,			{NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Print the mgmt counters" },
	{ "log",				1, mgmtLog		,			{INTEGER_PRM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Enable/Disable Mgmt log <mode(1-On, 0-Off)>" },
	{ "aging",				1, mgmtAging		,			{INTEGER_PRM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Enable/Disable Mgmt aging mechanism <mode(1-On, 0-Off)>" },
	{ "show",				0, mgmtShow,			{NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Show parameters" },
   WAPP_UTILS_CONSOLE_SUB_MENU_ARRAY_POSTFIX,
   { "nullcommand",			0, NULL,                {NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Undefined" },
};

/************************************************************************/
/*						Mgmt Test sub-menu									*/
/************************************************************************/
/*!	\brief 
  	\param 

  	\return 
  
*/
ConsoleCommand_T ConsoleMgmtTstCommandArray[] = {
    { "ratesTable",			0, printRatesTable,		{NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Print the station rates table" },
    { "null",				4, null,				{INTEGER_PRM,   INTEGER_PRM,   INTEGER_PRM,   INTEGER_PRM,   NO_PARAM}, "Send a null mgmt frame <FC> <SA> <DA> <BSSID>" },
    { "a1",					2, auth1,				{INTEGER_PRM,   SHORT_PRM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Send an authentication#1 frame <SA> <AlgoNum>" },
    { "a3",					3, auth3,				{INTEGER_PRM,   SHORT_PRM,   INTEGER_PRM,   NO_PARAM,   NO_PARAM}, "Send an authentication#3 frame <SA> <AlgoNum> <challenge>" },
{ "assoc",				4, assoc,				{INTEGER_PRM,   INTEGER_PRM,   INTEGER_PRM,   INTEGER_PRM}, "Send an association frame <SA> <PBCC (1-On, 0-Off)> <privacy (1-On, 0-Off)> <four x (1-On, 0-Off)>" },
    { "reassoc",			4, reAssoc,				{INTEGER_PRM,   INTEGER_PRM,   INTEGER_PRM,   INTEGER_PRM}, "Send a ReAssociation frame <SA> <PBCC (1-On, 0-Off)> <privacy (1-On, 0-Off)> <four x (1-On, 0-Off)>" },
    { "deauth",				2, deAuth,				{INTEGER_PRM,   SHORT_PRM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Send a DeAuthentication frame <SA> <reason>" },
    { "disassoc",			2, disAssoc,			{INTEGER_PRM,   SHORT_PRM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Send a DisAssociation frame <SA> <reason>" },
    { "class3",				2, class3,				{INTEGER_PRM,   SHORT_PRM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Send class3 message to Mgmt task <SA> <index>" },
    { "challenge",			1, showChallenge,		{SHORT_PRM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Show the challenge text that was sent to a station <index>" },
    { "staTable",			0, printStaTable,		{NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Print the station table" },
    { "hashTable",			0, printHashTable,		{NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Print the hash table" },
    { "findSta",			1, findSta,				{INTEGER_PRM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Find a station index <mac>" },
    { "insertSta",			1, insertSta,			{INTEGER_PRM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Find and insert a station index <mac>" },
    { "removeSta",			1, removeSta,			{INTEGER_PRM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Remove a station index <index>" },
    { "log",				1, mgmtLog		,			{INTEGER_PRM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Enable/Disable Mgmt log <mode>" },
    { "reset",				0, mgmtReset		,			{NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Reset Mgmt table" },
    { "showc",				0, mgmtShow,			{NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Show parameters" },
    { "shows",			   0, wlanPrintParams,  {NO_PARAM,      NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Print the WLAN configuration params" },
    { "reboot",      0, consoleReboot,        {NO_PARAM,      NO_PARAM,     NO_PARAM,     NO_PARAM}, "reboot the AP" },
   WAPP_UTILS_CONSOLE_SUB_MENU_ARRAY_POSTFIX,
   { "nullcommand",         0, NULL,                {NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Undefined" },
};

/************************************************************************/
/*						Privacy sub-menu								*/
/************************************************************************/
/*!	\brief 
  	\param 

  	\return 
  
*/
ConsoleCommand_T ConsoleWpaCommandArray[] = {
	{ "show", 0, wpaShow,			{NO_PARAM}, "Show WPA configuration" },
	{ "staTable",			0, printStaTable,		{NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Print the station table" },
	{ "mode", 5, wpaWorkingMode,			{INTEGER_PRM,INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, NO_PARAM}, "Set allowed station types for <profile index>, 1 - On, 0 - Off <WEP Stations> <Dynamic WEP - EAP-TLS Stations> <WPA Stations> <WPA-2 Stations>" },
	{ "pass", 2, wpaPassPhraseMode,			{INTEGER_PRM,INTEGER_PRM, NO_PARAM}, "Set the pass phrase mode for <profile index> < pass mode>(0 - None, 1 - HEX, 2 - ASCII)" },
	{ "hex", 1, wpaPskHex,			{INTEGER_PRM, NO_PARAM}, "Set PSK in HEX format for <profile index>, enter 32 ASCII characters. They will be converted to 64 Hex chars" },
	{ "pwd",            1, wpaPskPwd,               {INTEGER_PRM,NO_PARAM},    "Set PSK in ASCII format for <profile index>, maximum 63 characters"},  
    { "printPSK",       1, wpaPrintPSKinHexFormat,  {INTEGER_PRM,NO_PARAM},    "Print PSK in Hex format for <profile index>"},
	{ "cipher", 2, wpaDefaultCipher,			{INTEGER_PRM,INTEGER_PRM}, "Set WEP cipher for  <profile index>. ( 0 - None  1 - WEP 40, 2 - WEP 128, 3 - WEP 256, ) " },
	{ "gtk", 2, wpaGTKInterval,			{INTEGER_PRM,INTEGER_PRM}, "Set Group key update interval for <profile index>, in seconds" },
	{ "pre_auth", 2, wpaPreAuthflag,			{INTEGER_PRM,INTEGER_PRM}, "Enable pre authentication for <profile index> (1-on ,0-off)" },
	{ "secret",            1, wpaRadiusSecret,               {INTEGER_PRM,NO_PARAM},    "Set Radius Secret for <profile index>, maximum 64 characters"},  
	{ "port",            2, wpaRadiusPort,               {INTEGER_PRM,INTEGER_PRM},    "Set Radius Port for <profile index> <port>"},  
	{ "addr",            5, wpaRadiusAddr,               {INTEGER_PRM,INTEGER_PRM,INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM},    "Set Radius IP Address for <profile index> <addr 1> <addr 2> <addr 3> <addr 4>"},  
	{ "gateway",            4, wpaDefaultGateway,               {INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM},    "Set Default Gateway <addr 1> <addr 2> <addr 3> <addr 4>"},  
	{ "id",						2, privacySetPrivacyWEPDefaultKeyId,	{INTEGER_PRM, INTEGER_PRM,  NO_PARAM,   NO_PARAM}, "Set WEP default key for <profile index>, ID <id (0..3)>" },
	{ "def5",				7, privacySetDefaultKey5,	{INTEGER_PRM, INTEGER_PRM,INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM}, "Set default key for <profile > <index (0..3)> <key[1]> ..... <key[5]>" },
	{ "def13",				15, privacySetDefaultKey13,	{INTEGER_PRM, INTEGER_PRM,INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM
															, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM}, "Set default key for <profile > <index (0..3)> <key[1]> ..... <key[13]>" },
	{ "def29",				31, privacySetDefaultKey29,	{INTEGER_PRM, INTEGER_PRM,INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM
															, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM
															, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM
												, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM}, "Not supported, since there is no support for the commands long than one line. Use WEB GUI for this parameter configuration" },
    { "authAlgo", 1, privacySetAuthAlgo,			{INTEGER_PRM}, "Set Authentication Algorithm.\n                     (0 - Open System,  1 - Shared Key(only for WEP), 2 - Both)" },
	{ "reboot",      0, consoleReboot,        {NO_PARAM,      NO_PARAM,     NO_PARAM,     NO_PARAM}, "reboot the AP" },
   WAPP_UTILS_CONSOLE_SUB_MENU_ARRAY_POSTFIX,
   { "nullcommand",			0, NULL,                {NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Undefined" },
};

/************************************************************************/
/*						WPA Test sub-menu									*/
/************************************************************************/
/*!	\brief 
  	\param 

  	\return 
  
*/
ConsoleCommand_T ConsoleWpaTstCommandArray[] = {
	{ "staTable",			0, printStaTable,		{NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Print the station table" },
	{ "rsnTable",	0, stationsRsnIe,		{NO_PARAM}, "Print the station's RSN IE" },
	{ "insertSta",			1, insertSta,			{INTEGER_PRM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Find and insert a station index <mac>" },
	{ "a1",					2, auth1,				{INTEGER_PRM,   SHORT_PRM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Send an authentication#1 frame <SA> <AlgoNum>" },
	{ "assocStation",	4, assocStation,		{INTEGER_PRM,   INTEGER_PRM,   INTEGER_PRM,   INTEGER_PRM,   NO_PARAM}, "Send assoc from station <SA> <RNS IE (0, 1)> <RNS Ie Len> <RSN Ie Value>" },
	{ "reAssocStation",	4, reAssocStation,		{INTEGER_PRM,   INTEGER_PRM,   INTEGER_PRM,   INTEGER_PRM,   NO_PARAM}, "Send reAssoc from station <SA> <RNS IE (0, 1)> <RNS Ie Len> <RSN Ie Value>" },
	{ "auth",	1, macAuthorize,			{MAC_ADDR_PRM, NO_PARAM,   NO_PARAM,   NO_PARAM}, "Authorize a station <mac>" },
	{ "deauth",	1, macDeAuthorize,			{MAC_ADDR_PRM, NO_PARAM,   NO_PARAM,   NO_PARAM}, "De Authorize a station <mac>" },
	{ "show", 0, showWpaConfig,			{NO_PARAM}, "Show WPA configuration" },
	{ "mac", 1, isMacAuthorized,			{MAC_ADDR_PRM,   NO_PARAM}, "Is mac authorized in TX direction" },
	{ "suite", 2, setStationSuite,			{MAC_ADDR_PRM,   INTEGER_PRM, NO_PARAM}, "Set station suite <MAC> <0 - NONE, 1 - WEP, 2 - TKIP>" },
	{ "bcast", 1, setBcastSuite,			{INTEGER_PRM, NO_PARAM}, "Set broadcast suite <0 - NONE, 1 - WEP, 2 - TKIP>" },
	{ "vector", 0, testVectors,			{NO_PARAM}, "Run MICHAEL test vectors" },
	{ "setRxKey", 9, setRxMicKey,			{MAC_ADDR_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM}, "Set Rx Mic key, <MAC> <key[1]> ....<key[8]>" },
	{ "setTxKey", 9, setTxMicKey,			{MAC_ADDR_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM}, "Set Tx Mic key, <MAC> <key[1]> ....<key[8]>" },
	{ "micReportRx", 1, setMicReportInRx,			{INTEGER_PRM}, "Set Mic Failure report in the RX path, <1 - report, 0 - do not report>" },
	{ "micReportStation", 1, setMicReportFromStation,			{INTEGER_PRM}, "Set Mic Failure report from station, <1 - report, 0 - do not report>" },
	{ "genTxMicFailure",			1, generateTxMicFailure,			{MAC_ADDR_PRM}, "Generate a TX Mic failure for a station <mac>" },
	
	{ "reboot",      0, consoleReboot,        {NO_PARAM,      NO_PARAM,     NO_PARAM,     NO_PARAM}, "reboot the AP" },
   WAPP_UTILS_CONSOLE_SUB_MENU_ARRAY_POSTFIX,
   { "nullcommand",			0, NULL,                {NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Undefined" },
};

/************************************************************************/
/*						WPA Reports sub-menu									*/
/************************************************************************/
/*!	\brief 
  	\param 

  	\return 
  
*/
ConsoleCommand_T ConsoleWpaReportCommandArray[] = {
   { "severity", 1, wpa_repConfSeverity, {INTEGER_PRM}, "Configure reports severity: 0 - None,  1- Error, 2 - Warning, 3 - Info"},	
   { "reportModule", 1, wpa_repConfReportModule, {INTEGER_PRM}, "Configure reports module: 0 - None,  1-interfaces , 2 - authentications,\n"
					"                   3 - WDS security profile, 4 - WPA initialization"},
   { "show", 0, wpa_reportShow, {NO_PARAM}, "Show wpa debug report parameters"},
   { "staTbl", 1, wpa_reportPrintStaTbl, {INTEGER_PRM}, "Print internal WPA station tables: <profile index>"},
   { "printStat", 1, wpa_reportPrintStatistics, {INTEGER_PRM}, "Show profile statistics <profile index>"},
   { "profileCfg", 1, wpa_reportPrintProfCfg, {INTEGER_PRM}, "Show profile configuration in WPA <profile index>"},
   { "PMKCacheList", 0, wpa_reportPrintPmkCacheList, {NO_PARAM}, "Show PMK Cache List"},
   WAPP_UTILS_CONSOLE_SUB_MENU_ARRAY_POSTFIX,
   { "nullcommand",			0, NULL,                {NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Undefined" },
};
/************************************************************************/
/*						Service Appilication sub-menu									*/
/************************************************************************/
/*!	\brief 
  	\param 

  	\return 
  
*/
ConsoleCommand_T ConsoleServiceAppCommandArray[] = {
	{ "Enable",			0, servEnable,				{NO_PARAM,	   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Service Enable" },
	{ "Disable",		0, servDisable,				{NO_PARAM,	   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Service Disable" },
    { "dualAP",	        1, wlanSetDualAp,           {INTEGER_PRM,  NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Set Dual Ap status < 0= regular, Other = Dual>" },
    { "woneIndex",      1, wlanSetWoneConfIndex,    {INTEGER_PRM,  NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Set the Wone configuration Index" },
    { "macaddr",        1, wlanSetMacAddr,          {MAC_ADDR_PRM, NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Set the WLAN port MAC address <mac>" },
    { "rates",          2, wlanSetRates,            {INTEGER_PRM,  INTEGER_PRM}, "Set the 802.11b AP rates,  <Max basic rate> <Max supported rate>.\n The max supported rate must be higher or equal to the max basic rate.\n The available rates are: 1, 2, 5 (for 5.5), 11 & 22.\n" },
    { "grates",         0, wlanSetGRates,           {NO_PARAM,     NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Set the 802.11g AP rates used in Extended IE. For each rate enter :\n                 'Y'=Supported, 'B'=Basic Supported,\n                 'N'=Not supported, 'Enter'=unchanged" },
    { "addrates",       0, wlanSetAdditionalRates,  {NO_PARAM,     NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Set the additional 802.11g AP rates used in Legacy IE. For each rate enter :\n                 'Y'=Supported, 'B'=Basic Supported,\n                 'N'=Not supported, 'Enter'=unchanged" },
    { "respratemode",   1, wlanSetRespRateMode,     {INTEGER_PRM}, "Set the (re)Associate Response supported rates IE mode :\n                 0=All AP rates in D6.1 format, 1=Only AP rates supported by STA in STA format" },
    { "channel",        1, wlanSetChannel,          {INTEGER_PRM}, "Set the channel" },
    { "beacon",	        1, wlanSetBeacon,           {SHORT_PRM,    NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Set the WLAN beacon period <1..65535>" },
    { "ssid",           0, wlanSSID,                {NO_PARAM},  "Set SSID"},  
    { "dtim",           1, wlanSetDtim,             {SHORT_PRM,    NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Set the WLAN DTIM period <1..255>" },
    { "rts",            1, wlanSetRtsThreshold,     {SHORT_PRM,    NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Set RTS threhsold <0..2347>" },
    { "short",          1, wlanSetShoreRetryLimit,  {SHORT_PRM,    NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Set the short retry limit <1..255>" },
    { "eepLess",        1, wlanSetEepromLess, 		{SHORT_PRM,    NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Set EEPROM LESS Mode <0-Disable, 1-Enable>" },  
    { "long",           1, wlanSetLongRetryLimit,   {SHORT_PRM,    NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Set the long retry limit <1..255>" },
    { "fallback",       1, wlanSetRateFallback,     {SHORT_PRM,    NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Set rate fallback value" },
    { "adapt",          1, wlanSetRateAdaptation,   {SHORT_PRM,    NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Set rate adaptation <0-Disable, 1-Enable>" },
    { "frag",           1, wlanSetFragmentationTreshold,    {SHORT_PRM,        NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Set the fragmentation threshold <256..2346>" },
    { "transmit",       1, wlanSetMaxTransmitMSDULifetime,  {INTEGER_PRM,      NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Set the Max Transmit MSDU Lifetime <1..4294967295>" },
    { "receive",        1, wlanSetMaxReceiveLifetime,       {INTEGER_PRM,      NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Set the Max Receive Lifetime <1..4294967295>" },
    { "show",           0, wlanPrintParams,         {NO_PARAM,     NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Print the WLAN configuration params" },
    { "printrates",	    0, wlanPrintRatesParams,    {NO_PARAM,     NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Print the Rates configuration params" },
    { "counters",       0, wlanPrintCounters,       {NO_PARAM,     NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Print the WLAN counters " },
    { "defaults",       0, wlanSetNvramDefaults,    {NO_PARAM,     NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "WARNING: FOR DEBUG USE ONLY!!! Restore nvram defaults" },
    { "reboot",         0, consoleReboot,           {NO_PARAM,     NO_PARAM,   NO_PARAM,   NO_PARAM}, "reboot the AP" },
    { "agile",          1, wlanSetAgile,            {INTEGER_PRM},  "Set agile attribute <1-On,0-Off>" },
    { "preamblemode",   1, wlanSetPreambleMode,     {SHORT_PRM},    "Set preamble mode :\n                 <0-Always Long, 1-Always Short, 2-Local STAs>" },
    { "dynalgotimeout", 1, wlanSetDynAlgoTimeout,   {INTEGER_PRM},  "Set Enhanced Dynamic Algorithms Timeout in [ms]" },
    { "pbccalgo",       1, wlanSetPBCCAlgorithmMode,{SHORT_PRM},    "Set PBCC algorithm Mode :\n		<0-Always off, 1-Always on, 2-Local STAs, 3-Dynamic, 4-Enhanced Dynamic>" },
    { "slotmode",       1, wlanSetSlotTimeMode,     {SHORT_PRM},    "Set Short Slot Time mode :\n                 <0-Always off, 1-Always on, 2-Local STAs, 3-Enhanced Dynamic>" },
    { "slotthreshold",  1, wlanSetSlotTimeThreshold,{SHORT_PRM},    "Set Short Slot Time threshold (# of B rate packets)" },
    { "protmode",       1, wlanSetERPProtectionMode,{SHORT_PRM},    "Set Protection Mode :\n                <0-Always off, 1-Always on, 2-Local STAs, 3-Dynamic, 4-Enhanced Dynamic>" },
    { "prottype",       1, wlanSetERPProtectionType,{SHORT_PRM},    "Set Protection Type <0-RTS/CTS, 1-CTS To Self, 2-TI proprietary>" },
    { "dot11mode",      1, wlanSetDot11Mode,        {SHORT_PRM},    "Set 802.11 Mode <0-Mixed mode, 1-802.11b only, 2-802.11b+, 3-802.11g only, 4-802.11a >" },
    { "CwMinMode",      1, wlanSetCWmin,            {SHORT_PRM},    "Set CwMin mode :\n  		 <0-(CwMin=31), 1-(CwMin=15), 2-(CwMin dot11mode dependent)> \n 	 	 Note that CwMin=15 in any mode when 4x is enabled" },
    { "caloption",      1, wlanSetRadioCalibrateOption,     {INTEGER_PRM,      NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Set the radio calibration option" },
    { "calinterval",    1, wlanSetRadioCalibrateInterval,   {SHORT_PRM,        NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Set the radio calibration interval in [sec], 0-Disable" },
    { "usageTimeMode",  1, wlanSetUsageTimeMode,        {SHORT_PRM, NO_PARAM,    NO_PARAM,   NO_PARAM,   NO_PARAM}, "Set the band Usage Time type: 0 = short 1 = long" },
    { "setUsageTime",   1, wlanSetUsageTime,        {INTEGER_PRM, NO_PARAM,    NO_PARAM,   NO_PARAM,   NO_PARAM}, "Set the band Usage Time" },
    { "radioParams",    0, wlanRadioParams,         {NO_PARAM, NO_PARAM,    NO_PARAM,   NO_PARAM,   NO_PARAM}, "Edit Radio parameter table" },
    { "bssIsolation",   1, wlanSetBssIsolation,     {INTEGER_PRM}, "Set BSS Bridge isolation state <0-Disable, 1-Enable>" },	
    { "printBssIsolation",    0, wlanPrintBssIsolationState,         {NO_PARAM, NO_PARAM,    NO_PARAM,   NO_PARAM,   NO_PARAM}, "Print BSS isolation state" },
    { "radarDetect",   1, wlanSetRadarDetect,     {SHORT_PRM}, "Set radar detect mode <0-Disable, 1-Enable>" },	
    { "printRadarDetect",    0, wlanPrintRadarDetect, {NO_PARAM, NO_PARAM,    NO_PARAM,   NO_PARAM,   NO_PARAM}, "Print radar detect mode" },	
    { "apBridgeMode",   1, wlanSetApBridgeMode, {INTEGER_PRM}, "Set AP Bridge Mode <0-Disable, 1-Enable>" },  
   WAPP_UTILS_CONSOLE_SUB_MENU_ARRAY_POSTFIX,
   { "nullcommand",     0, NULL,                    {NO_PARAM,   NO_PARAM,     NO_PARAM,   NO_PARAM,   NO_PARAM}, "Undefined" },
};

/************************************************************************/
/*					SSID SubAppilication sub-menu						*/
/************************************************************************/
/*!	\brief 
  	\param 

  	\return 
  
*/
ConsoleCommand_T ConsoleSsidAppCommandArray[] = {
    { "show",              0, wlanSsidPrintParams,           {NO_PARAM},  "Print the SSID params" },
    { "ssid",              0, wlanSSID,                      {NO_PARAM},  "Set SSID"},  
    { "hiddenssid",        1, wlanSetHiddenSSID,             {SHORT_PRM}, "Enable / Disable Hidden SSID mode" },
    { "multiplessid",      1, wlanSetMultipleSSID,           {SHORT_PRM}, "Enable / Disable Multiple SSID mode ,case enable, <vlan_tag> for primary ssid" },
    { "addssid",           1, wlanAddMultipleSsidElement,    {INTEGER_PRM},  "Add Multiple SSID element for <vlan>"},  
    { "removessid",        0, wlanRemoveMultipleSsidElement, {NO_PARAM},  "Remove Multiple SSID element from a list"}, 
    { "reboot",            0, consoleReboot,                 {NO_PARAM,     NO_PARAM,   NO_PARAM,   NO_PARAM}, "reboot the AP" },
   WAPP_UTILS_CONSOLE_SUB_MENU_ARRAY_POSTFIX,
   { "nullcommand",     0, NULL,                    {NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Undefined" },
};

/************************************************************************/
/*					QoS SubAppilication sub-menu			                    			*/
/************************************************************************/

/*!	\brief 
  	\param 

  	\return 
  
*/
ConsoleCommand_T ConsoleQoSCommandArray[] = {
    { "show",              0, wlanQoSPrintWMEConf,           {NO_PARAM},  "Print QoS configuration" },
    { "enableWME",         2, wlanQoSEnableWME,              {SHORT_PRM, SHORT_PRM},  "Set WME parameters <0-Disable, 1-Enable> <Classifier Mode 0-VLAN, 1-DiffServ, 2-UDP Port for Debug only)>"},  
		{ "setAC",             1, wlanQoSSetAC,                  {SHORT_PRM},  "Set AC parameters <0-AP, 1-STA>"},
   WAPP_UTILS_CONSOLE_SUB_MENU_ARRAY_POSTFIX,
   { "nullcommand",     0, NULL,                    {NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Undefined" },
};



/************************************************************************/
/*					802.11 d & h SubAppilication sub-menu				*/
/************************************************************************/
/*!	\brief 
  	\param 

  	\return 
  
*/
ConsoleCommand_T Console_DH_AppCommandArray[] = {
    { "show",           0, wlan_dh_PrintParams,             {NO_PARAM},   "Show 802.11 d & h config params" },
    { "printreg",       0, wlan_dh_PrintRegDomain,          {NO_PARAM},   "Show Regulatory Domain information" },
    { "channel",        1, wlanSetChannel,                  {INTEGER_PRM},"Set the channel" },
    { "multidomain",    1, wlanSetMultiDomain,              {SHORT_PRM},  "Enable - 1 and Disable - 0 multidomain capability (802.11d)" },  
#ifdef WLAN_801_11_H_IMPLEMENTED
    { "spectrummgmt",   1, wlanSetSpectrumMgmt,             {SHORT_PRM},  "Enable - 1 and Disable - 0 Apectrum Management Requirement (802.11h)" },
    { "legacystation",  1, wlanSetLegacyStation,            {SHORT_PRM},  "Enable - 1 and Disable - 0 Legacy equipment (802.11h)" },
#endif /* WLAN_801_11_H_IMPLEMENTED */
    { "powerconst",     1, wlanSetPowerConstraint,          {SHORT_PRM},  "Set power constraint (default = 0 dBm)" },  
    { "SetRegDomain",   0, wlanSetRegulatoryDomain,         {NO_PARAM},   "Set Regulatory Domain from a list (default = FCC)" },  
    { "txpowerlevel",   1, wlanSetTxPowerLevel,             {SHORT_PRM},  "Set Max Tx Power Level <1..5>" }, 
    /*{ "setpowerlevels", 6, wlanSetPowerLevel,               {INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM},  "Set the 5 Tx power levels (dBm) per band (1st param, 0-G, 1-A)" }, */
    { "countrystring",  1, wlanSetCountryString,            {STRING_PRM}, "Set dot11CountryString (2 bytes)" }, 
    { "AddChannelRange",3, wlanAddChnRangToCustomRegDomain, {INTEGER_PRM, INTEGER_PRM, INTEGER_PRM}, "Add Channel range to customize Reg Doamin. <First Channel> <Number of channels> <Tx Power Level (dBm)>" }, 
    { "reboot",         0, consoleReboot,                   {NO_PARAM,     NO_PARAM,   NO_PARAM,   NO_PARAM}, "reboot the AP" },
   WAPP_UTILS_CONSOLE_SUB_MENU_ARRAY_POSTFIX,
   { "nullcommand",     0, NULL,                            {NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Undefined" },
};

 
/************************************************************************/
/*						Show sub-menu									*/
/************************************************************************/
/*!	\brief 
  	\param 

  	\return 
  
*/
ConsoleCommand_T ConsoleShowCommandArray[] = {
    { "bssbridge",          0, showBssBridge,	{NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Bss Bridge Statistics" },
    { "powersave",          0, showPowerSave,	{NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Power Save Status" },
    { "txstat"   ,          0, showPrintTxStat,	    {NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Print Tx Statistics" },
    { "mibwlan"  ,          0, showMibWlan,	        {NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Print WLAN MIB2 counters" },
    { "lanstat"  ,          0, showLanStat,	        {NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Print EMAC MIB2 counters" },
    { "lanpool"  ,          0, showLanPool,	        {NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Print EMAC buffer pool" },
    { "memmngrstat"  ,      0, showMemMngrStat,	    {NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Print Memory Manager statistics" },
    { "rateadapt"  ,        0, showRateAdaptation,	{NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Print Rate Adaptation status" },  
    WAPP_UTILS_CONSOLE_SUB_MENU_ARRAY_POSTFIX,
   { "nullcommand",     0, NULL,                    {NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Undefined" },
};

/************************************************************************/
/*					     4x sub-menu									*/                                                   
/************************************************************************/
/*!	\brief 
  	\param 

  	\return 
  
*/
ConsoleCommand_T Console4xCommandArray[] = {
	{ "show",				0, show4xParams,		{NO_PARAM,      NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Show the 4x params" },
	{ "concatsize",			1, set4xMaxConcatSize,  {INTEGER_PRM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Set the max concatenation size (1522-4095)" },    
	{ "set",			2, set4x		,			{INTEGER_PRM,   INTEGER_PRM,NO_PARAM,NO_PARAM}, "Enable/Disable 4x Feature, \n <4x Enable(1-On, 0-Off)> <Concatenation(1,0)> \n" },
	{ "mixmode",	    1, wlanSet4xMixMode ,		{SHORT_PRM,     NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Enable/Disable 4x with cck" },
    { "bursting",			1, setPacketBursting	,	{INTEGER_PRM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Enable/Disable Packet bursting <mode(1-On, 0-Off)>" },
    { "dbg",			1, setConcatDebug		,	{INTEGER_PRM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Enable/Disable concat debug <mode(1-On, 0-Off)>" },
	{ "table",			0, fourXTable,				{INTEGER_PRM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Displays four X table" },
	{ "featureset",     1, set4xFeatureSet,     {SHORT_PRM}, "4x FeatureSet (0/1)" }, 
	{ "burstLimit",     1, setBurstLimit,           {SHORT_PRM}, "Set Packet burst time limit in MicroSec" },   
	{ "reboot",         0, consoleReboot,        {NO_PARAM,      NO_PARAM,     NO_PARAM,     NO_PARAM}, "reboot the AP" },
    WAPP_UTILS_CONSOLE_SUB_MENU_ARRAY_POSTFIX,
   { "nullcommand",     0, NULL,                    {NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Undefined" },
};


/************************************************************************/
/*						Hal sub-menu									*/
/************************************************************************/
/*!	\brief 
  	\param 

  	\return 
  
*/
ConsoleCommand_T ConsoleHalCommandArray[] = {
	{ "fwVersion",			0, whalMenu_printFwVersion,	{NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Print the firmware version" },
	{ "acxParams",			0, whalMenu_printAcxParams,	{NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Print Acx100 nvram parameters" },
	{ "txinfo",				2, whalMenu_printTxInfo,		{INTEGER_PRM,INTEGER_PRM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Print Tx Info <Qid=0,1 EnableQprint = 0-FALSE/1-TRUE> (queues and buffers)" },
	{ "rxinfo",				0, whalMenu_printRxInfo,		{NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Print Rx Info (queues and buffers)" },
	{ "txqueue",			1, whalMenu_printTxQueue,		{INTEGER_PRM,NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Print Tx queue <Qid=0,1> status" },
	{ "rxqueue",			1, whalMenu_printRxQueue,		{INTEGER_PRM,NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Print Rx queue <Qid=0> status" },
	{ "intr",				0, whalMenu_printIntr,			{NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Print Interrupt object status" },
	{ "mbox",				0, whalMenu_printMbox,			{NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Print mailbox object status" },
	{ "mboxerrors",			0, whalMenu_printMboxErrors,	{NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Print mailbox errors trace" },
	{ "acxstat",			0, whalMenu_printAcxStat,		{NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Print Acx100 statistics" },
	{ "restartstat",		0, whalMenu_printRestartStat,	{NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Print restart statistics" },
	{ "restartenable",		1, whalMenu_EnableRestartWA,	{SHORT_PRM}, "Enable FW Watchdog WA <1-enable 0-disable>" },
	{ "interfWA",		    1, whalMenu_EnableInterfWA,	    {SHORT_PRM}, "ACI/NB Interference hanling: < 0-disable, 1- handling by HW (only 1150 and higher)," 
							   "2 - Interference Workaround in the SW>" },
    { "waoptions",			6, whalMenu_setWAoptions,		{SHORT_PRM, SHORT_PRM, SHORT_PRM, SHORT_PRM,SHORT_PRM, SHORT_PRM}, "Set WA options\n  Rx Free Mem (0-disable,1-enable)"
        " Beacon stuck(0-disable, N-trigger time[sec])\n    Tx Acx free (0-disable, N-trigger time[sec])\n  Rx Cons FCS (0-disable, N-trigger time[sec])\n"
		"    ACI (0-disable, 1-enable), NB (0-disable, 1-enable) "},
	{ "aciRateThreshold" ,  1, whalMenu_setAciRateThresh,  		{INTEGER_PRM}, "Set ACI Rate threshold"}, 	
	{ "aciEventThreshold" , 1, whalMenu_setAciEventThresh,  		{INTEGER_PRM}, "Set number of ACI events which triggers ACI state"}, 	
	{ "aciStateTimeout",	1, whalMenu_setAciTimeout,      {INTEGER_PRM}, "Set ACI Work Around timeout (in seconds)"},
	{ "nbStateTimeout",		1, whalMenu_setNbTimeout,       {INTEGER_PRM}, "Set NB Work Around timeout (in seconds)"},
	{ "pdThresh",			2, whalMenu_setPdThresh,        {INTEGER_PRM, INTEGER_PRM }, "Set packet detection thresholds: <low thresh> <high thresh>"},
	{ "mediumBusyThresh" ,    1, whalMenu_setBusyMediumThresh, 	{INTEGER_PRM}, "Set busy medium threshold in percents "}, 	 
	{ "showACIparams",		0, whalMenu_showACIparams,		{NO_PARAM}, "Show ACI and NB work around configuration"},
	{ "energydetectEn", 	1, whalMenu_EnableEnergyDetect, {SHORT_PRM}, "Enable Energy Detect <1-enable 0-disable>" },
	{ "energydetectSh", 	0, whalMenu_EnergyDetectShow, {SHORT_PRM}, "Displays energy detect status " },
	{ "gpio_owner",		1, whalMenu_gpio_owner,	{SHORT_PRM}, "set gpio ownership <1-driver 0-fw 2-show>" },
	{ "mediumUsage",		0, whalMenu_mediumUsage,		{NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Read medium usage" },
    WAPP_UTILS_CONSOLE_SUB_MENU_ARRAY_POSTFIX,
   { "nullcommand",			0, NULL,							{NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Undefined" },
};

/************************************************************************/
/*						Hal Diag sub-menu									*/
/************************************************************************/
/*!	\brief 
  	\param 

  	\return 
  
*/
ConsoleCommand_T ConsoleHalDiagCommandArray[] = {
	{ "eed",			      0, whalMenu_eed,        	   {NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Serial EEPROM dump" },
	{ "eer",			      1, whalMenu_eer,	            {INTEGER_PRM,NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Serial EEPROM read <addr>" },
	{ "eew",			      2, whalMenu_eew,	            {INTEGER_PRM,INTEGER_PRM,NO_PARAM,   NO_PARAM,   NO_PARAM}, "Serial EEPROM write <addr> <value>" },
	{ "SetTxCompleteTrsh",   1, whalMenu_SetTxCompleteTrsh, {INTEGER_PRM,	NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Set Tx Complete Threshold (Val)" },
	{ "SetPsMode",			1, whalMenu_SetBssPs ,		{INTEGER_PRM,	NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "SetPsMode(Val) - Set Ps mode (0, 1)" },
	{ "PrintRegs",			0, whalMenu_PrintRegs ,		{NO_PARAM,		NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Print low level registers" },
	{ "PhyRegWrite",		2, whalMenu_PhyRegWrite ,	{INTEGER_PRM,	INTEGER_PRM,NO_PARAM,   NO_PARAM,   NO_PARAM}, "PhyRegWrite(Addr, Val) - Write Phy register" },
	{ "PhyRegRead",			1, whalMenu_PhyRegRead ,	{INTEGER_PRM,	NO_PARAM,	NO_PARAM,   NO_PARAM,   NO_PARAM}, "PhyRegRead(Addr)       - Read  Phy register" },
	{ "MacRegWrite",		2, whalMenu_MacRegWrite ,	{INTEGER_PRM,	INTEGER_PRM,NO_PARAM,   NO_PARAM,   NO_PARAM}, "MacRegWrite(Addr, Val) - Write Mac register" },
	{ "MacRegRead",			1, whalMenu_MacRegRead ,	{INTEGER_PRM,	NO_PARAM,	NO_PARAM,   NO_PARAM,   NO_PARAM}, "MacRegRead(Addr)       - Read  Mac register" },
	{ "SetRxPromiscuous",	1, whalMenu_RxPromiscuous ,	{INTEGER_PRM,	NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "SetRxPromiscuous(Val)  - Set Promiscuous mode (0, 1)" },
	{ "ModeSniffer",		1, whalMenu_ModeSniffer ,	{INTEGER_PRM,	NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "ModeSniffer(Val)	   - Set Mode Sniffer (0, 1)" },
	{ "SelectFwNumber",		1, whalMenu_SelectFwNumber,	{INTEGER_PRM,	NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "SelectFwNumber(Val)   - 0 or 1 (local), 2 (remote), 3 (enable JTAG), 4 (disable JTAG) 5 (FW ready to run)" },
	{ "SetQueueSizeType",	1, whalMenu_SetQueueSizeType,{INTEGER_PRM,	NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "SetQueueSizeType(Val)  - Type = 0(more queues) or 1" },
	{ "SelectRadioNum",		1, whalMenu_SelectRadioNumber,{INTEGER_PRM,	NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "SelectRadioNumber(Val)   - 0 or 1 (local), 2 (remote)" },
	{ "SetNvByte",			2, whalMenu_SetNvByte ,		{INTEGER_PRM,	INTEGER_PRM,NO_PARAM,   NO_PARAM,   NO_PARAM}, "SetNvByte(Addr, Val)   - Set hal nvram byte" },
	{ "TraceEnable",		1, whalMenu_TraceEnable ,	{INTEGER_PRM,	NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "TraceEnable(Val)       - Enable tracer (0, 1)" },
	{ "TracePrint",			2, whalMenu_TracePrint ,	{INTEGER_PRM,	INTEGER_PRM,NO_PARAM,   NO_PARAM,   NO_PARAM}, "TracePrint(Num, From)  - Print trace info" },
	{ "TraceRxStat",        0, whalMenu_TracePrintRxStat, {NO_PARAM,	NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Print the tracer Rx statistics" }, 
	{ "DownloadFW",			0,  whalMenu_DownloadFW ,	{NO_PARAM, NO_PARAM, NO_PARAM, NO_PARAM, NO_PARAM}, "DownloadFW  - Download FW through TFTP (assumes file name is FW.bin, uses server addr from FWDebugServerIP)" },
	{ "dm",            4,  whalMenu_DM, {STRING_PRM, INTEGER_PRM, INTEGER_PRM,   INTEGER_PRM,   NO_PARAM}, "dm <base addr> <offset> <size in bytes> <step: 1 - bytes, 2 - words> - ACX memory dump "},
	{ "mm",            5,  whalMenu_MM, {STRING_PRM, INTEGER_PRM, INTEGER_PRM,   INTEGER_PRM,   INTEGER_PRM}, "mm <base addr> <offset> <width: 1 - byte, 2 - word, 4 - int> <new value> <count> - ACX memory dump "},
	{ "FWDebugServerIP",     4,  whalMenu_DebugServerIP, {INTEGER_PRM, INTEGER_PRM, INTEGER_PRM,   INTEGER_PRM,   NO_PARAM}, "FWTraceIP <ip1> <ip2> <ip3> <ip4> - Set Debug host IP address "},
    WAPP_UTILS_CONSOLE_SUB_MENU_ARRAY_POSTFIX,
   { "nullcommand",			0, NULL,                {NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Undefined" },
};

/************************************************************************/
/*						Hal HW interference sub-menu									*/
/************************************************************************/
/*!	\brief 
  	\param 

  	\return 
  
*/
ConsoleCommand_T ConsoleHalHwInterfCommandArray[] = {
	{ "inputCCA",			      1, whalMenu_inputCCA,        {SHORT_PRM}, "Input CCA: 0 -  AGC | 11A-PD | 11B-PD, 1 -  (11A-PD & IB_CCA) | 11B-PD" },
	{ "txCCA",			      1, whalMenu_txCCA,        	   {SHORT_PRM}, "Tx CCA: 0 -  AGC | 11A-PD | 11B-PD, 1 -  11B-PD | IB_CCA" },
	{ "qualifCCA",			      1, whalMenu_qualifCCA,       {SHORT_PRM}, "Qualified CCA: 0 -  Input_CCA, 1 -  PD_Chk_CCA "
	"2 -  NB_Chk_CCA, 3 -  ACI_Chk_CCA, 4 -  PLCP_Chk_CCA, 5 -  IB_Chk_CCA" },
	{ "stompRx",			      1, whalMenu_stompRx,        	   {SHORT_PRM}, "Stomp for RX" },
	{ "stompTx",			      1, whalMenu_stompTx,        	   {SHORT_PRM}, "Stomp for TX" },
	{ "show",			      	  0, whalMenu_hwShow,        	   {SHORT_PRM}, "Show HW interference handling parameters" },
	WAPP_UTILS_CONSOLE_SUB_MENU_ARRAY_POSTFIX,
   { "nullcommand",			0, NULL,                {NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Undefined" },

};
/************************************************************************/
/*                   Production sub-menu                                */
/************************************************************************/

#ifdef VXWORKS

/*!	\brief 
  	\param 

  	\return 
  
*/
ConsoleCommand_T consoleProductionCommandArray[] =
{
   { "reboot",   0, consoleReboot,        {NO_PARAM,     NO_PARAM,      NO_PARAM,      NO_PARAM,NO_PARAM},"reboot the AP"}, 
   { "dir",      0, consoleDir,           {NO_PARAM,     NO_PARAM,      NO_PARAM,      NO_PARAM,     NO_PARAM}, "dir            - List available modem software versions"         },
   { "erase",    1, consoleErase,         {INTEGER_PRM,  NO_PARAM,      NO_PARAM,      NO_PARAM,     NO_PARAM}, "erase <sec #>  - Erase sector <sec#> from Flash"                 },
   { "dl",       3, consoleDl,            {INTEGER_PRM,  IP_ADDR_PRM,   STRING_PRM,    NO_PARAM,     NO_PARAM}, "dl <sec#> <srvIP> <name> - Downld <name> from <srvIP> to <sec>"  },
   { "setdef",   1, consoleSetDef,        {INTEGER_PRM,  NO_PARAM,      NO_PARAM,      NO_PARAM,     NO_PARAM}, "setdef <sec#>  - Set <sec#> as default modem software"           },
   { "prodshow", 0, consoleProdShow,      {NO_PARAM,     NO_PARAM,      NO_PARAM,      NO_PARAM,     NO_PARAM}, "prodshow       - Show production params"                         },
   { "prodset",  0, consoleProdSet,       {NO_PARAM,     NO_PARAM,      NO_PARAM,      NO_PARAM,     NO_PARAM}, "prodset        - Set production params"                          },
   { "proddef",  0, consoleProdDef,       {NO_PARAM,     NO_PARAM,      NO_PARAM,      NO_PARAM,     NO_PARAM}, "proddef        - Set production default params"                  },
   { "password", 0, consolePassword,      {NO_PARAM,     NO_PARAM,      NO_PARAM,      NO_PARAM,     NO_PARAM}, "password <pwrd>- Set production menu password"                   },
   WAPP_UTILS_CONSOLE_SUB_MENU_ARRAY_POSTFIX,
   { "nullcommand",0, NULL,               {NO_PARAM,     NO_PARAM,      NO_PARAM,      NO_PARAM,     NO_PARAM}, "Undefined"                                                       }
};

#endif /* VXWORKS */

/************************************************************************/
/*          Build-in test sub-menu                                      */
/************************************************************************/
/*!	\brief 
  	\param 

  	\return 
  
*/
#ifdef INCLUDE_TESTAPP
ConsoleCommand_T consoleTestAppCommandArray[] =
{
   { "reboot",   0, consoleReboot,        {NO_PARAM,     NO_PARAM,      NO_PARAM,   NO_PARAM,   NO_PARAM},"reboot the AP"                          },
   { "macr",     1, tstReadMacReg,        {ARGS_PRM,     NO_PARAM,      NO_PARAM,   NO_PARAM,   NO_PARAM},"Read  TNETW MAC register"              },
   { "macw",     1, tstWriteMacReg,       {ARGS_PRM,     NO_PARAM,      NO_PARAM,   NO_PARAM,   NO_PARAM},"Update  TNETW MAC register"            },
   { "pr",       1, tstReadPhyReg,        {ARGS_PRM,     NO_PARAM,      NO_PARAM,   NO_PARAM,   NO_PARAM},"Read  TNETW PHY register"              },
   { "pw",       1, tstWritePhyReg,       {ARGS_PRM,     NO_PARAM,      NO_PARAM,   NO_PARAM,   NO_PARAM},"Update  TNETW PHY register"            },
   { "phyregs",  0, tstDumpPHYRegs,       {NO_PARAM,     NO_PARAM,      NO_PARAM,   NO_PARAM,NO_PARAM},   "Dump PHY registers"                     },  
   { "mr",       1, tstReadMem,           {ARGS_PRM,     NO_PARAM,      NO_PARAM,   NO_PARAM,   NO_PARAM},"Read  TNETW memory"                    },
   { "mw",       1, tstWriteMem,          {ARGS_PRM,     NO_PARAM,      NO_PARAM,   NO_PARAM,   NO_PARAM},"Update  TNETW memory location"         },
   { "eer",      1, tstReadEEPROM,        {INTEGER_PRM,  NO_PARAM,      NO_PARAM,   NO_PARAM,   NO_PARAM},"Read from TNETW EEPROM location"       },
   { "eew",      2, tstWriteEEPROM,       {INTEGER_PRM,  INTEGER_PRM,   NO_PARAM,   NO_PARAM,   NO_PARAM},"Update TNETW EEPROM location"          },
   { "agc",      1, tstReadWriteAGC,      {ARGS_PRM,     NO_PARAM,      NO_PARAM,   NO_PARAM,   NO_PARAM},"Read AGC table location"                },
   { "init",     1, tstInit,              {INTEGER_PRM,  NO_PARAM,      NO_PARAM,   NO_PARAM,   NO_PARAM},"0=reboot_all 1=init_TNETW 2=init_radio"   },
   { "status",   0, tstStatus,            {NO_PARAM,     NO_PARAM,      NO_PARAM,   NO_PARAM,NO_PARAM},"Show current settings"                     },  
   WAPP_UTILS_CONSOLE_SUB_MENU_ARRAY_POSTFIX,
   { "nullcommand",0, NULL,               {NO_PARAM,     NO_PARAM,      NO_PARAM,      NO_PARAM,     NO_PARAM}, "Undefined"                        }
};

/*!	\brief 
  	\param 

  	\return 
  
*/
ConsoleCommand_T consoleBringUpCommandArray[] =
{
   { "macr",     1, tstReadMacReg,        {ARGS_PRM,     NO_PARAM,      NO_PARAM,   NO_PARAM,   NO_PARAM},"Read  TNETW MAC register"              },
   { "macw",     1, tstWriteMacReg,       {ARGS_PRM,     NO_PARAM,      NO_PARAM,   NO_PARAM,   NO_PARAM},"Update  TNETW MAC register"            },
   { "pr",       1, tstReadPhyReg,        {ARGS_PRM,     NO_PARAM,      NO_PARAM,   NO_PARAM,   NO_PARAM},"Read  TNETW PHY register"              },
   { "pw",       1, tstWritePhyReg,       {ARGS_PRM,     NO_PARAM,      NO_PARAM,   NO_PARAM,   NO_PARAM},"Update  TNETW PHY register"            },
   { "phyregs",  0, tstDumpPHYRegs,       {NO_PARAM,     NO_PARAM,      NO_PARAM,   NO_PARAM,   NO_PARAM},   "Dump PHY registers"                  },  
   { "mr",       1, tstReadMem,           {ARGS_PRM,     NO_PARAM,      NO_PARAM,   NO_PARAM,   NO_PARAM},"Read  TNETW memory"                    },
   { "mw",       1, tstWriteMem,          {ARGS_PRM,     NO_PARAM,      NO_PARAM,   NO_PARAM,   NO_PARAM},"Update  TNETW memory location"         },
   { "eedump",   0, tstDumpEEPROM,        {NO_PARAM,     NO_PARAM,      NO_PARAM,   NO_PARAM,   NO_PARAM},"Dump TNETW EEPROM"                     },
   { "eer",      1, tstReadEEPROM,        {INTEGER_PRM,  NO_PARAM,      NO_PARAM,   NO_PARAM,   NO_PARAM},"Read from TNETW EEPROM location"       },
   { "eew",      2, tstWriteEEPROM,       {INTEGER_PRM,  INTEGER_PRM,   NO_PARAM,   NO_PARAM,   NO_PARAM},"Update TNETW EEPROM location"          },
   { "eeprog",   1, tstProgEEPROM,        {ARGS_PRM,     NO_PARAM,      NO_PARAM,	NO_PARAM,   NO_PARAM},"eeprog <srvIP> <filename>"              },
   { "agc",      1, tstReadWriteAGC,      {ARGS_PRM,     NO_PARAM,      NO_PARAM,   NO_PARAM,   NO_PARAM},"Read AGC table location"                },
   { "init",     1, tstInit,              {INTEGER_PRM,  NO_PARAM,      NO_PARAM,   NO_PARAM,   NO_PARAM},"0=reboot_all 1=init_TNETW 2=init_radio"   },
   { "status",   0, tstStatus,            {NO_PARAM,     NO_PARAM,      NO_PARAM,   NO_PARAM,NO_PARAM},"Show current settings"                     },  
   { "reboot",      0, consoleReboot,        {NO_PARAM,      NO_PARAM,     NO_PARAM,     NO_PARAM}, "reboot the AP" },
   WAPP_UTILS_CONSOLE_SUB_MENU_ARRAY_POSTFIX,
   { "nullcommand",0, NULL,               {NO_PARAM,     NO_PARAM,      NO_PARAM,      NO_PARAM,     NO_PARAM}, "Undefined"                                                       }
};

/*!	\brief 
  	\param 

  	\return 
  
*/
ConsoleCommand_T consoleBringUpTxCommandArray[] = 
{
   { "macr",     1, tstReadMacReg,        {ARGS_PRM,     NO_PARAM,      NO_PARAM,   NO_PARAM,   NO_PARAM},"Read  TNETW MAC register"              },
   { "macw",     1, tstWriteMacReg,       {ARGS_PRM,     NO_PARAM,      NO_PARAM,   NO_PARAM,   NO_PARAM},"Update  TNETW MAC register"            },
   { "pr",       1, tstReadPhyReg,        {ARGS_PRM,     NO_PARAM,      NO_PARAM,   NO_PARAM,   NO_PARAM},"Read  TNETW PHY register"              },
   { "pw",       1, tstWritePhyReg,       {ARGS_PRM,     NO_PARAM,      NO_PARAM,   NO_PARAM,   NO_PARAM},"Update  TNETW PHY register"            },
   { "phyregs",  0, tstDumpPHYRegs,       {NO_PARAM,     NO_PARAM,      NO_PARAM,   NO_PARAM,NO_PARAM},   "Dump PHY registers"                     },  
   { "channel",  1, tstChannel,           {ARGS_PRM,     NO_PARAM,      NO_PARAM,   NO_PARAM,NO_PARAM},"Select radio channel (1..14)"},
   { "status",   0, tstStatus,            {NO_PARAM,     NO_PARAM,      NO_PARAM,   NO_PARAM,NO_PARAM},"Show current settings"                           },  
   { "enable",   1, tstTxEnable,          {ARGS_PRM,     NO_PARAM,      NO_PARAM,   NO_PARAM,NO_PARAM},"Enable TX path 1=enable 0=disable"         },   
   { "antenna",  1, tstSelectTxAntenna,   {ARGS_PRM,     NO_PARAM,      NO_PARAM,   NO_PARAM,NO_PARAM},"Set Transmit antenna 1=antenna1 2=antenna2"},
   { "rate",     1, tstSelectTxRate,      {ARGS_PRM,     NO_PARAM,      NO_PARAM,   NO_PARAM,NO_PARAM},"Set TX data rate in Mbps (1,2,5.5,11,22)"  }, 
   { "mod",      1, tstSelectTxModulation,{ARGS_PRM,     NO_PARAM,      NO_PARAM,   NO_PARAM,NO_PARAM},"Set TX modulation 0=CCK 1=PBCC"},
   { "carrier",  1, tstCarrierTx,         {ARGS_PRM,     NO_PARAM,      NO_PARAM,   NO_PARAM,NO_PARAM},"Enable TX carrier only mode 1=enable 0=disable"},
   { "pmble",    1, tstPreamble,          {ARGS_PRM,     NO_PARAM,      NO_PARAM,   NO_PARAM,NO_PARAM},"Set preamble type 0=short 1=long"},
   { "length",   1, tstMPDULen,           {ARGS_PRM,     NO_PARAM,      NO_PARAM,   NO_PARAM,NO_PARAM},"Set MPDU length"                           },
   { "power",    1, tstPower,             {ARGS_PRM,     NO_PARAM,      NO_PARAM,   NO_PARAM,NO_PARAM},"Set TX power 0..63"                        },   
   { "pa"   ,    1, tstPAEnable,          {ARGS_PRM,     NO_PARAM,      NO_PARAM,   NO_PARAM,NO_PARAM},"Set PA mode: 0=disable 1=enable 2=auto"    },   
   { "osc",      1, tstSetOscCalValue,    {ARGS_PRM,     NO_PARAM,      NO_PARAM,   NO_PARAM,NO_PARAM},"Set/Get oscillator calibration (0-15)"     },     
   { "bias",     1, tstSetBias,           {ARGS_PRM,     NO_PARAM,      NO_PARAM,   NO_PARAM,NO_PARAM},"Set/Get PA bias (0..15)"                   },     
   { "dpattern", 1, tstDataPattern,       {ARGS_PRM,     NO_PARAM,      NO_PARAM,   NO_PARAM,NO_PARAM},"Data pattern: 0 = PRBS, 1 = 010101"        },     
   { "scramb",   1, tstScramb,            {ARGS_PRM,     NO_PARAM,      NO_PARAM,   NO_PARAM,NO_PARAM},"Enable TX scrambler 0=disable 1=enable"    },  
   { "tx",       1, tstTX,                {ARGS_PRM,     NO_PARAM,      NO_PARAM,   NO_PARAM,NO_PARAM},"Start TX bursts c=cont.mode; 0=stop"       },   
	{ "reboot",      0, consoleReboot,        {NO_PARAM,      NO_PARAM,     NO_PARAM,     NO_PARAM}, "reboot the AP" },
   WAPP_UTILS_CONSOLE_SUB_MENU_ARRAY_POSTFIX,
   { "nullcommand",0, NULL,               {NO_PARAM,     NO_PARAM,      NO_PARAM,      NO_PARAM,     NO_PARAM}, "Undefined"                                                       }
};

/*!	\brief 
  	\param 

  	\return 
  
*/
ConsoleCommand_T consoleBringUpRxCommandArray[] = 
{
   { "macr",     1, tstReadMacReg,        {ARGS_PRM,     NO_PARAM,      NO_PARAM,   NO_PARAM,   NO_PARAM},"Read  TNETW MAC register"              },
   { "macw",     1, tstWriteMacReg,       {ARGS_PRM,     NO_PARAM,      NO_PARAM,   NO_PARAM,   NO_PARAM},"Update  TNETW MAC register"            },
   { "pr",       1, tstReadPhyReg,        {ARGS_PRM,     NO_PARAM,      NO_PARAM,   NO_PARAM,   NO_PARAM},"Read  TNETW PHY register"              },
   { "pw",       1, tstWritePhyReg,       {ARGS_PRM,     NO_PARAM,      NO_PARAM,   NO_PARAM,   NO_PARAM},"Update  TNETW PHY register"            },
   { "phyregs",  0, tstDumpPHYRegs,       {NO_PARAM,     NO_PARAM,      NO_PARAM,   NO_PARAM,NO_PARAM},   "Dump PHY registers"                     },  
   { "channel",  1, tstChannel,           {ARGS_PRM,     NO_PARAM,      NO_PARAM,   NO_PARAM,NO_PARAM},"Select radio channel (1..14)"              },
   { "status",   0, tstStatus,            {NO_PARAM,     NO_PARAM,      NO_PARAM,   NO_PARAM,NO_PARAM},"Show current settings"                           },  
   { "enable" ,  1, tstRxEnable,          {ARGS_PRM,     NO_PARAM,      NO_PARAM,   NO_PARAM,NO_PARAM},"Enable RX path 1=enable 0=disable"         },   
   { "per",      1, tstPER,               {ARGS_PRM,     NO_PARAM,      NO_PARAM,   NO_PARAM,NO_PARAM},"Collect Packet Error Rate Statistics for <sec>"  },   
   { "diversity",1, tstSetAntDiversity,   {ARGS_PRM,     NO_PARAM,      NO_PARAM,   NO_PARAM,NO_PARAM},"Set Diversity 1=ant1 2=ant2 3=full 4=partial"    },
	{ "reboot",      0, consoleReboot,        {NO_PARAM,      NO_PARAM,     NO_PARAM,     NO_PARAM}, "reboot the AP" },
   WAPP_UTILS_CONSOLE_SUB_MENU_ARRAY_POSTFIX,
   { "nullcommand",0, NULL,               {NO_PARAM,     NO_PARAM,      NO_PARAM,      NO_PARAM,     NO_PARAM}, "Undefined"                        }
};

#endif /* INCLUDE_TESTAPP */

ConsoleCommand_T ConsoleWdsCommandArray[] = {
	{ "enable",			    1, enableWds,           {INTEGER_PRM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Enable WDS service: 1 - enable, 0 - disable." },    
    { "apply",				0, applyWdsParams,		{NO_PARAM,      NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Save params and restart the WDS manager" },
	{ "show",				0, showWdsParams,		{NO_PARAM,      NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Show the WDS params" },
	{ "report",				0, showWdsReport,		{NO_PARAM,      NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Show the WDS report" },
    { "stat",				0, showWdsStat,		    {NO_PARAM,      NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Show the WDS statistics" },
    { "infodb",				0, showWdsInfoDb,	    {NO_PARAM,      NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Show information database" },
    { "hop",			    1, setWdsHop,           {INTEGER_PRM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Set the WDS hop number. 0 - root, 1 - not root." },    
    { "name",		        1, setWdsName,          {STRING_PRM,    NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "WDS name (max 8 symbols)." },    
    { "chSelect",		    1, setWdsChSel,         {INTEGER_PRM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Channel select mode 0 - user, 1 - auto." },    
    { "autoconfig",		    1, setWdsAutoCfg,       {INTEGER_PRM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Auto config mode 0 - disable, 1 - enable." },    
    { "privacy",		    1, setWdsPrivacy,       {INTEGER_PRM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "WDS privacy 0 - disable, 1 - enable." },    
    { "secret",		        1, setWdsSharedSecret,  {STRING_PRM,    NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Shared secret (max 32 symbols)." },    
    { "bssmode",		    1, setWdsBssMode,       {INTEGER_PRM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "BSS mode 0 - disable, 1 - ebnable." },    
    { "crudemode",	    1, setWdsCrudeMode,     {INTEGER_PRM,   NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Crude mode 0 - disable, 1 - ebnable." },    
    { "uplink",		        1, setWdsUplink,        {MAC_ADDR_PRM,  NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Uplink MAC address" },    
    { "downlink0",	        1, setWdsDownlink0,     {MAC_ADDR_PRM,  NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Downlink #0 MAC address" },    
    { "downlink1",	        1, setWdsDownlink1,     {MAC_ADDR_PRM,  NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Downlink #1 MAC address" },    
    { "downlink2",	        1, setWdsDownlink2,     {MAC_ADDR_PRM,  NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Downlink #2 MAC address" },    
    { "downlink3",	        1, setWdsDownlink3,     {MAC_ADDR_PRM,  NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Downlink #3 MAC address" },    
    { "severity",	        3, setWdsPrintSeverity, {INTEGER_PRM,   INTEGER_PRM,INTEGER_PRM,NO_PARAM,   NO_PARAM}, "Print severity - error warning info" },    
    { "defaults",			0, setWdsDefaults,		{NO_PARAM,      NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Reset WDS configuration to defaults" },
    { "restart",			0, setWdsRestart,		{NO_PARAM,      NO_PARAM,   NO_PARAM,   NO_PARAM,   NO_PARAM}, "Restart WDS manager with current config" },
    { "reboot",      0, consoleReboot,        {NO_PARAM,      NO_PARAM,     NO_PARAM,     NO_PARAM}, "reboot the AP" },
   WAPP_UTILS_CONSOLE_SUB_MENU_ARRAY_POSTFIX,
   { "nullcommand",0, NULL,               {NO_PARAM,     NO_PARAM,      NO_PARAM,      NO_PARAM,     NO_PARAM}, "Undefined"                        }
};

ConsoleCommand_T ConsoleIpSetCommandArray[] = {
	{ "address",	        4, setIpAddr,           {INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, NO_PARAM}, "Set IP address e.g. 192 168 100 1" },    
    { "mask",		        4, setIpMask,		    {INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, NO_PARAM}, "Set IP subnet mask e.g 255 255 255 0" },
    { "gateway",	        4, setDefGateway,	    {INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, INTEGER_PRM, NO_PARAM}, "Set default gateway e.g 192 168 100 10" },
	{ "apply",				0, applyIpSettings,		{NO_PARAM,    NO_PARAM,    NO_PARAM,    NO_PARAM,    NO_PARAM}, "Apply IP settings" },
	{ "show",				0, showIpSettings,		{NO_PARAM,    NO_PARAM,    NO_PARAM,    NO_PARAM,    NO_PARAM}, "Show IP settings" },
    { "reboot",             0, consoleReboot,       {NO_PARAM,    NO_PARAM,    NO_PARAM,    NO_PARAM,    NO_PARAM}, "reboot the AP" },
   WAPP_UTILS_CONSOLE_SUB_MENU_ARRAY_POSTFIX,
   { "nullcommand",0, NULL,               {NO_PARAM,     NO_PARAM,      NO_PARAM,      NO_PARAM,     NO_PARAM}, "Undefined"                        }
};

/**************************************************************************/
/* BIT sub-menu command definition                                        */
/**************************************************************************/
/*!	\brief 
  	\param 

  	\return 
  
*/
void BITCB(ConsoleParameter_T parameters[]){
   puts("BIT submenu");
}

/**************************************************************************/
/* Mgmt sub-menu command definition                                       */
/**************************************************************************/
/*!	\brief 
  	\param 

  	\return 
  
*/
void MgmtCB(ConsoleParameter_T parameters[]){
   printf("Mgmt submenu\n");
}

/**************************************************************************/
/* Mgmt Test sub-menu command definition                                  */
/**************************************************************************/
/*!	\brief 
  	\param 

  	\return 
  
*/
void MgmtTstCB(ConsoleParameter_T parameters[]){
   printf("Mgmt Test submenu\n");
}

/**************************************************************************/
/* WPA sub-menu command definition                                       */
/**************************************************************************/
/*!	\brief 
  	\param 

  	\return 
  
*/
void WpaCB(ConsoleParameter_T parameters[]){
   printf("Privacy submenu\n");
}

/**************************************************************************/
/* Wpa Test sub-menu command definition                                  */
/**************************************************************************/
/*!	\brief 
  	\param 

  	\return 
  
*/
void WpaTstCB(ConsoleParameter_T parameters[]){
   printf("WPA Test submenu\n");
}

/**************************************************************************/
/* Wpa Reports sub-menu command definition                                  */
/**************************************************************************/
/*!	\brief 
  	\param 

  	\return 
  
*/
void WpaRepCB(ConsoleParameter_T parameters[]){
   printf("WPA Reports submenu\n");
}


/**************************************************************************/
/* Privacy sub-menu command definition                                    */
/**************************************************************************/
/**************************************************************************/
/* Service App sub-menu command definition                                */
/**************************************************************************/
/*!	\brief 
  	\param 

  	\return 
  
*/
void ServCB(ConsoleParameter_T parameters[]){
   printf("Service Application submenu\n");
}

/**************************************************************************/
/* Service App sub-menu command definition                                */
/**************************************************************************/
/*!	\brief 
  	\param 

  	\return 
  
*/
void ShowCB(ConsoleParameter_T parameters[]){
   printf("Show submenu\n");
}

/**************************************************************************/
/* Service App sub-menu command definition                                */
/**************************************************************************/
/*!	\brief 
  	\param 

  	\return 
  
*/
void x4CB(ConsoleParameter_T parameters[]){
   printf("4x submenu\n");
}


/**************************************************************************/
/* Hal sub-menu command definition                                        */
/**************************************************************************/
/*!	\brief 
  	\param 

  	\return 
  
*/
void HalCB(ConsoleParameter_T parameters[]){
   printf("Hal submenu\n");
}

/**************************************************************************/
/* Hal Diagnosticsub-menu command definition                              */
/**************************************************************************/
/*!	\brief 
  	\param 

  	\return 
  
*/
void HalDiagCB(ConsoleParameter_T parameters[]){
   printf("Hal Diagnostic submenu\n");
}

/**************************************************************************/
/* HW Interference handling command definition                              */
/**************************************************************************/
/*!	\brief 
  	\param 

  	\return 
  
*/
void HwInterfCB(ConsoleParameter_T parameters[]){
   printf("HW Interference handling configuration\n");
}

/**************************************************************************/
/* Production sub-menu command definition                                 */
/**************************************************************************/
/*!	\brief 
  	\param 

  	\return 
  
*/
void ProdCB(ConsoleParameter_T parameters[])
{/*
   UINT8 pbPassword[HW_PROD_PASSWORD_LEN];
   whal_nvram_getConsolePassword(pbPassword);
   if(strncmp(pbPassword,parameters[0].STRING,HW_PROD_PASSWORD_LEN) != 0)
   {
      puts("Invalid password");
      consoleQuit(parameters);
      return;
   }
   consoleHelp(parameters);*/
}
/**************************************************************************/
/* Security sub-menu command definition										  */
/**************************************************************************/
/*!	\brief 
  	\param 

  	\return 
  
*/
void SecurityCB(ConsoleParameter_T parameters[]){
   printf("Security submenu\n");
}

/**************************************************************************/
/* Security Test sub-menu command definition                                       */
/**************************************************************************/
/*!	\brief 
  	\param 

  	\return 
  
*/
void SecTstCB(ConsoleParameter_T parameters[]){
   printf("Security Test submenu\n");
}
/**************************************************************************/
/* Wds sub-menu command definition                                        */
/**************************************************************************/
void WdsCB(ConsoleParameter_T parameters[]){
   printf("Wds submenu\n");
}

/**************************************************************************/
/* IP sub-menu command definition                                         */
/**************************************************************************/
void IpSetCB(ConsoleParameter_T parameters[]){
   printf("IP Setting\n");
}

/**************************************************************************/
/* Build-in test sub-menues command definition                              */
/**************************************************************************/
/*!	\brief 
  	\param 

  	\return 
  
*/
#ifdef INCLUDE_TESTAPP
void TestAppCB(ConsoleParameter_T parameters[])
{
   printf("Test application submenu\n");
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void BringUpCB(ConsoleParameter_T parameters[])
{
   /*UINT8 pbPassword[HW_PROD_PASSWORD_LEN];
   
   whal_nvram_getConsolePassword(pbPassword);

   if(strncmp(pbPassword,parameters[0].STRING,HW_PROD_PASSWORD_LEN) != 0)
   {
      puts("Invalid password");
      consoleQuit(parameters);
      return;
   }
   // Stop WLAN tasks to enable bring-up tests 
   tstInitBringUpMode();
   printf("Bring up submenu\n");*/
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void BringUpTxCB(ConsoleParameter_T parameters[])
{
   puts("TA/bringup/tx submenu");
}

#endif   /* INCLUDE_TESTAPP */

/*************************************************************************/
/*                     commmand functions                                */
/*************************************************************************/

/************************************************************************/
/*						Root menu functions								            */
/************************************************************************/
/***************************************************************************/
/* prints the MAC address of the device                                    */
/***************************************************************************/
/*!	\brief 
  	\param 

  	\return 
  
*/
/*!	\brief 
  	\param 

  	\return 
  
*/
void consoleMac(ConsoleParameter_T parameters[]) {
   /* number of parameters = 0  */
   /*unsigned char *mac;
   unsigned char prnBuffer[WAPP_UTILS_MAX_LINE_LENGTH];

   mac = (unsigned char*)&MAC_address;
   sprintf(prnBuffer,"CM MAC address is %02hhX-%02hhX-%02hhX-%02hhX-%02hhX-%02hhX",
            mac[0],mac[1],mac[2],mac[3],mac[4],mac[5]);
   printf("%s\n",prnBuffer);
   wapp_utilsSetDevConsoleStatus(prnBuffer);
   */
}

/***************************************************************************/
/* sets the report level of the device                                     */
/***************************************************************************/
/*!	\brief 
  	\param 

  	\return 
  
*/
void consoleReportLevel(ConsoleParameter_T parameters[]) {
   /* number of parameters = 1    */
   /* display resulting value     */
   
/*
   WDRV_UTILS_REPORT_LEVEL value;
   int level,l;
   unsigned char prnBuffer[WAPP_UTILS_MAX_LINE_LENGTH], prn_parm[10];

   strcpy(prnBuffer,"");
   level = parameters[0].INTEGER;
   if (level > 0) 
	{ // set level 
		if (level == 27) 
		{ // set all 
			value = wdrv_utilsGetReportValue();
         wdrv_utilsSetReportValue(value | 0x07ffffff);
      } 
		else 
         wdrv_utilsSetReportLevel((WDRV_UTILS_REPORT_LEVEL)(level));

   } 
	
	else if (level < 0) 
	{
      if (level == -27) 
		{ // clear all 
         value = wdrv_utilsGetReportValue();
         wdrv_utilsSetReportValue(value & 0xf0000000);
      } 
		else 
		{
         level = -level;
         wdrv_utilsClearReportLevel((WDRV_UTILS_REPORT_LEVEL)(level));
      }
   }

   value = wdrv_utilsGetReportValue();


   value &= 0x0fffffff;

   strcat(prnBuffer,"Report levels ON: ");
   for (l=0;value;l++) 
	{
      if (value & 1) 
		{
         sprintf(prn_parm," %d",l);
         strcat(prnBuffer,prn_parm);
      }
      value >>= 1;
   }
   strcat(prnBuffer,"\n");

   if (level == 0) 
	{
      // display detailed list 
       strcat(prnBuffer," 1 ROOT\n");
       strcat(prnBuffer," 2 TEST\n");
       strcat(prnBuffer," 3 TCB\n");
       strcat(prnBuffer," 4 IDLE\n");
       strcat(prnBuffer," 5 DB\n");
       strcat(prnBuffer," 6 MAINSM\n");
       strcat(prnBuffer," 7 MGMT\n");
       strcat(prnBuffer," 8 HAL\n");
       strcat(prnBuffer," 9 TX\n");
       strcat(prnBuffer," 10 SM\n");
       strcat(prnBuffer," 11 RX\n");
       strcat(prnBuffer," 12 RX Sniffer\n");
       strcat(prnBuffer," 13 TX Sniffer\n");
       strcat(prnBuffer," 14 Bridge\n");
       strcat(prnBuffer," 15 WPA\n");
       strcat(prnBuffer," 16 HTTP\n");
       strcat(prnBuffer," 17 4X\n");
   }

   printf("%s\n",prnBuffer);
   wapp_utilsSetDevConsoleStatus(prnBuffer);
   */

	   unsigned char prnBuffer[MAX_LINE_LEN], mode_str[MAX_LINE_LEN], prn_parm[10];
	   int level,l, value;


	   strcpy(prnBuffer,"");
	   level = parameters[0].INTEGER;
	   if (level > 0) 
		{ /* set level */ 
		 
			if (level == MAX_REPLEV) { /* set all */
				
				for(l=1; l<MAX_REPLEV - 1; l++) 
				{
					/* the storage is not updated because of performance reasons*/
					if(tiap_wcfg_set_basic(wd, TIAP_PARAM_REPLEV, &l, !UPDATE_STORAGE)) {
						PRINT_CONFIG_ERR_MSG("Failed to set report level.\n");
						return;
					}
				}

				/* update the storage only in the last ioctl call */
				if(tiap_wcfg_set_basic(wd, TIAP_PARAM_REPLEV, &l, UPDATE_STORAGE)) {
						PRINT_CONFIG_ERR_MSG("Failed to set report level.\n");
						return;
				}

			} 
			else {
				 if(tiap_wcfg_set_basic(wd, TIAP_PARAM_REPLEV, &level, UPDATE_STORAGE)) {
					PRINT_CONFIG_ERR_MSG("Failed to set report level.\n");
					return;
				 }
			}

	   } 
		
	   else if (level < 0) 
		{
			if (level == MIN_REPLEV) { /* clear all */ 
				l = 0;
				if(tiap_wcfg_set_basic(wd, TIAP_PARAM_REPLEV, &l, UPDATE_STORAGE)) {
					PRINT_CONFIG_ERR_MSG("Failed to set report level.\n");
					return;
				 }
			} 
			else {
				if(tiap_wcfg_set_basic(wd, TIAP_PARAM_REPLEV, &level, UPDATE_STORAGE)) {
					PRINT_CONFIG_ERR_MSG("Failed to set report level.\n");
					return;
				}
			}
		}

	   if(tiap_wcfg_get_basic(wd, TIAP_PARAM_REPLEV, &value, !FROM_STORAGE || driver_load_failed)) {
		PRINT_CONFIG_ERR_MSG("Failed to get report level.\n");
		return;
	   }

	   value &= 0x0fffffff;

	   sprintf(mode_str, "Report mode: %s  ", (reportMode == IMMEDIATE ? "IMMEDIATE": "DEFERRED"));
	   strcat(prnBuffer, mode_str);
	   strcat(prnBuffer,"Report levels ON: ");
	   for (l=0;value;l++) 
	   {
	      if (value & 1) 
	      {
		 sprintf(prn_parm," %d",l);
		 strcat(prnBuffer,prn_parm);
	      }
	      value >>= 1;
	   }
	   strcat(prnBuffer,"\n");

	   if (level == 0) 
	{
	      // display detailed list 
	       strcat(prnBuffer," 1 ROOT\n");
	       strcat(prnBuffer," 2 TEST\n");
	       strcat(prnBuffer," 3 TCB\n");
	       strcat(prnBuffer," 4 IDLE\n");
	       strcat(prnBuffer," 5 DB\n");
	       strcat(prnBuffer," 6 MAINSM\n");
	       strcat(prnBuffer," 7 MGMT\n");
	       strcat(prnBuffer," 8 HAL\n");
	       strcat(prnBuffer," 9 TX\n");
	       strcat(prnBuffer," 10 SM\n");
	       strcat(prnBuffer," 11 RX\n");
	       strcat(prnBuffer," 12 RX Sniffer\n");
	       strcat(prnBuffer," 13 TX Sniffer\n");
	       strcat(prnBuffer," 14 Bridge\n");
	       strcat(prnBuffer," 15 WPA\n");
	       strcat(prnBuffer," 16 HTTP\n");
	       strcat(prnBuffer," 17 4X\n");
	       strcat(prnBuffer," 18 MEM\n");
	       strcat(prnBuffer," 19 DYNAMIC_ALG\n");
	       
	   }

	   printf("%s\n",prnBuffer);
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void consoleReportMode(ConsoleParameter_T parameters[])
{
	int mode, idummy = 0, not_read;
	char dummy, buf[LOG_BUF_LEN];

	mode = parameters[0].INTEGER;

	if (mode != 0 && mode != 1) {
		printf("Wrong value\n");
		return;
	}

	reportMode = mode;
	
	if (reportMode == DEFERRED) {
		printf("Report mode was set to Deferred\n");

	    if (klogctl(6,  &dummy, idummy) < 0 ) /* disable printk's to console */
				   perror("Failed to disable printk to console\n");
			   
			   /* clear the kernel log buffer before starting the console */
			
			   if ( (not_read = klogctl(9, 0, 0)) ){ /* poll the log buffer */ 

					if( klogctl(2, buf, not_read) < 0 ) /* read and clear the buffer*/
						perror("Failed to clear kernel message buffer\n");
			   }
	}
	else {
		printf("Report mode was set to Immediate\n");
		if (klogctl(7,  &dummy, idummy) < 0 ) /* enable printk's to console */
				perror("Failed to enable printk to console\n");
	}
	return;
	
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void consoleReboot(ConsoleParameter_T parameters[]) {
/*
	whal_apiAcxReset();
#ifdef VXWORKS
	cm_hwReset(); // nothing else matters... 
#else
	//	puts("Console can not reboot the system, reboot or reload the driver using other utilities"); 
	tiwlan_user_cleanup(0);
	
#endif
	  */
#ifndef VXWORKS	
#if 1
	if (driver_load_failed) {
		puts("CLI will exit now. In order to reload the WLAN driver, rerun the init process or reboot the board.");
		exit(0);
	} else {
		if ( !(wd = tiap_wcfg_restart(wd, &driver_load_failed)) ) {
			if (driver_load_failed) {
				puts("\n\nDriver Init failed, probably due to misconfiguration. \nRunning CLI to fix the problem\n");
				if( !(wd = malloc(sizeof(wcfgd))) )
					exit(1);
				memset(wd, 0, sizeof(wcfgd));
				/*store storage file*/
				strncpy(wd->storage_file,  WCFG_MTDBLOCK, _POSIX_NAME_MAX+_POSIX_PATH_MAX);
				wd->storage_file[_POSIX_NAME_MAX+_POSIX_PATH_MAX] = '\0';
			} else {
				exit(1);
			}
		}
	}
		
#else
	tiap_wcfg_system_reboot(wd);
#endif
#else
	tiap_wcfg_system_reboot(wd);
#endif
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void consoleVersion(ConsoleParameter_T parameters[])
{ /*
   unsigned char prn_buffer[81], *buf;

   buf = prn_buffer;
   cm_hwFormatBootInfo1(buf, 80);

   //print the version information of application, boot loader and FW. 
   printf("CPU type = %s\nBoard type = %s\n", GetCpuName(), GetBoardName());
   
 
#ifdef DEBUG_OPTION
   printf("APDK version %d.%d.%d.%d - Debug, Date: %s\n",
      SW_VERSION_MAJOR,SW_VERSION_MINOR,SW_RELEASE_MAJOR,SW_RELEASE_MINOR,__DATE__);
#else     
   printf("APDK version %d.%d.%d.%d, Date: %s\n",
      SW_VERSION_MAJOR,SW_VERSION_MINOR,SW_RELEASE_MAJOR,SW_RELEASE_MINOR,__DATE__);
#endif   
   whal_apiPrintFwVersion();
   puts(buf);*/


	int len;
	char buf[4096];

	DRIVER_LOADED_ASSERT;

	if( (len = dda_read_proc_entry(WLAN_PROC_PROD_DIR_NAME,
                                        WLAN_PROC_PROD_VERSION, buf, sizeof(buf))) == -1 ) {
	
                PRINT_PROC_READ_ERR(WLAN_PROC_PROD_DIR_NAME, WLAN_PROC_PROD_VERSION);
				return;
	}
	
	buf[len] = '\0';
	printf("APDK version: %s\n", buf); 
#ifndef VXWORKS
	fflush(NULL);
#endif
	whalMenu_printFwVersion(parameters);


}


/************************************************************/
/*                  BIT sub-menu  functions                 */
/************************************************************/

/* Following are some auxiliary routines to serve the BIT command routines */

/*!	\brief 
  	\param 

  	\return 
  
*/
static BOOL IsChannelInsideBandRange(int band, int chID)
{
	/*
	BOOL found = FALSE;
	int ch;

	if (band == RADIO_BAND_2_4GHZ)
	{
		if ((chID < 1) || (chID > 14))
			return FALSE;
		return TRUE;
	}
	else
	{
		if ((chID < 36) || (chID > 161))
			return FALSE;
		// channel is inside the range, let's verify its value 
		for (ch=36; ch<=64;ch++)
			if (ch == chID)
			{
				found=TRUE;
				break;
			}
		if (!found)
			for (ch=100; ch<=140;ch++)
				if (ch == chID)
				{
					found=TRUE;
					break;
				}
		if (!found)
			for (ch=149; ch<=161;ch++)
				if (ch == chID)
				{
					found=TRUE;
					break;
				}
	}
	return found;
	*/
	return 1;
}

/*!	\brief 
  	\param 

  	\return 
  
*/
/*!	\brief 
  	\param 

  	\return 
  
*/
int ShowAndReturnBand(void)
{
	
	UINT8 boardType = 2;
	int band, len;
	char tmpBuffer[100];
	char buf[4096];

	if( (len = dda_read_proc_entry(WLAN_PROC_SERV_DIR_NAME,
                                        WLAN_PROC_SERV_RADIO_FREQ, buf, sizeof(buf))) == -1 )
                PRINT_PROC_READ_ERR(WLAN_PROC_SERV_DIR_NAME, WLAN_PROC_SERV_RADIO_FREQ);

	sscanf(buf, "%hhu", &boardType);

	printf("Available Bands:\n");
	printf("----------------\n");
	printf("1. Band 'G' (2.4GHz)\n");
	if (boardType == TIAP_PARAM_RADIO_BAND_2_4_AND_5GHZ)
	{
		printf("2. Band 'A' (5GHz)\n");
	}
	printf("\nPlease enter the selected band:");

	fgets(tmpBuffer, 100, stdin);
	tmpBuffer[strlen(tmpBuffer) - 1] = 0; // remove newline 
    band = atoi(tmpBuffer);
    if(band <= 0)
        return (-1);
	if (band > 2)
	{
		printf("Band out of range!\n");
        return (-1);
	}
	if (band == 1)
		band = TIAP_PARAM_RADIO_BAND_2_4GHZ;
	else
	{
		if (boardType != TIAP_PARAM_RADIO_BAND_2_4_AND_5GHZ)
		{
			printf("Band out of range!\n");
			return (-1);
		}
		band = TIAP_PARAM_RADIO_BAND_5GHZ;
	}
	return band; 
}


/*!	\brief 
  	\param 

  	\return 
  
*/
int ShowAndReturnChannel(int band)
{
	int chID;
    char tmpBuffer[100];

	printf("valid channels for this band:\n\t");
	if (band == TIAP_PARAM_RADIO_BAND_2_4GHZ)
		printf("1..14\n");
	else
		printf("[36..64] or [100..140] or [149-161] with step of 4 (e.g. 36,40,44,..)\n\n");
	printf("Please select a channel (or hit <Enter> to quit)");

    fgets(tmpBuffer, 100, stdin);
    tmpBuffer[strlen(tmpBuffer) - 1] = 0; // remove newline 
    chID = atoi(tmpBuffer);
    if(chID <= 0)
        return (-1);
	return chID;
	
}


/*!	\brief 
  	\param 

  	\return 
  
*/
int ShowAndReturnRate(void)
{
    char tmpBuffer[100];
	int rate;

	printf("\nAvailable rates:\n");
	printf("----------------\n");
	printf("\t1, 2, 5, 11, 22, 6, 9, 12, 18, 24, 36, 48, 54\n");
	printf("Please select a rate:");
    fgets(tmpBuffer, 100, stdin);
    tmpBuffer[strlen(tmpBuffer) - 1] = 0; // remove newline 
    rate = atoi(tmpBuffer);
	switch(rate)
	{
		case 1:
		case 2:
		case 5:
		case 6:
		case 9:
		case 11:
		case 12:
		case 18:
		case 22:
		case 24:
		case 36:
		case 48:
		case 54:
			break;
		default:
			printf("Rate out of range!\n");
			return (-1);
	}
	return rate;
}

/*!	\brief 
  	\param 

  	\return 
  
*/
int ShowAndReturnModes(void)
{

    char tmpBuffer[100];
	int mode;

	printf("\nAvailable modes:\n");
	printf("----------------\n");
	printf("1 - QPSK\n");
	printf("2 - CCK\n");
	printf("3 - PBCC\n");
	printf("4 - OFDM\n");
	printf("\nPlease select a mode:");
    fgets(tmpBuffer, 100, stdin);
    tmpBuffer[strlen(tmpBuffer) - 1] = 0; // remove newline 
    mode = atoi(tmpBuffer);
	switch(mode)
	{
		case 1:
		case 2:
		case 3:
		case 4:
			mode--;
			break;
		default:
			printf("Mode out of range!\n");
			return (-1);
	}
	return mode;
	
}


/*!	\brief 
  	\param 

  	\return 
  
*/
void consoleBitFcc(ConsoleParameter_T parameters[])
{
/*
    int chID, rate, mode, band;
    char tmpBuffer[100];
	BOOL found = FALSE;

	band = ShowAndReturnBand();
	if (band < 0)
		return;
    chID = ShowAndReturnChannel(band);
    if(chID <= 0)
        return;
    if (IsChannelInsideBandRange(band,chID) == FALSE)
	{
		printf("Channel out of range!\n");
		return;
	}

	// We have the channel, progressing to having the rate.. 
	rate = ShowAndReturnRate();
	if (rate <=0 )
		return;

	// last step - get the correct mode 
	mode = ShowAndReturnModes();
	if (mode < 0)
		return;

	// OK.. let's do it
	wdrv_dbSetEnRestart(0, FALSE);	// Disable Restart WA w/o writing to NV 

	whal_apiAcxBIT_Fcc(band, chID, rate, mode);
*/
	int chID, rate, mode, band, ret;
	tiap_param_bit_args_t   bitArgs;


	band = ShowAndReturnBand();
	if (band < 0)
		return;
	chID = ShowAndReturnChannel(band);
	if(chID <= 0)
		return;
	if (IsChannelInsideBandRange(band,chID) == FALSE)
	{
		printf("Channel out of range!\n");
		return;
	}

	/* We have the channel, progressing to having the rate.. */
	rate = ShowAndReturnRate();
	if (rate <=0 )
		return;

	/* last step - get the correct mode */
	mode = ShowAndReturnModes();
	if (mode < 0)
		return;

	/* OK.. let's do it */

    bitArgs.cmdType = BIT_FCC;
    bitArgs.band = band;
    bitArgs.channel = chID;
    bitArgs.rate = rate;
    bitArgs.mode = mode;

    ret = tiap_wcfg_set_bitArgs(wd, &bitArgs, 0);
    if(ret)
    {
		printf("Failed to set FCC parameters.\n");
    }	
}

/*!	\brief 
  	\param 

  	\return 
  
*/
static void set_enable_wa_restart(int val)
{
	tiap_param_wd_t wa;

	wa.cmd = TIAP_PARAM_WD_ENABLE;
	wa.value.val8 = val;
	
	if(tiap_wcfg_set_wd(wd, &wa, UPDATE_STORAGE))
		PRINT_CONFIG_ERR_MSG("Failed to set WA restart.");
}

/*!	\brief Disable enable ACI/NB iterference workaround in the driver
  	\param 

  	\return 
  
*/
static void set_interf_wa(int val)
{
	tiap_param_wd_t wa;

	wa.cmd = TIAP_PARAM_WD_INTERF;
	wa.value.val8 = val;
	
	if(tiap_wcfg_set_wd(wd, &wa, UPDATE_STORAGE))
		PRINT_CONFIG_ERR_MSG("Failed to set interference WA .");
}


/*!	\brief 
  	\param 

  	\return 
  
*/
void consoleBitTelec(ConsoleParameter_T parameters[])
{

	/*
   int chID;
   UINT8 band;
   char tmpBuffer[100];


   band = ShowAndReturnBand();
   if (band < 0)
	   return;
   chID = ShowAndReturnChannel(band);
   if(chID <= 0)
	   return;
   if (IsChannelInsideBandRange(band,chID) == FALSE)
   {
	   printf("Channel out of range!\n");
	   return;
   }

   wdrv_dbSetEnRestart(0, FALSE);	// Disable Restart WA w/o writing to NV 

   whal_apiAcxBIT_Telec(band, chID);
   printf("Done.\n");
*/
   int chID, band;
   tiap_param_bit_args_t   bitArgs;
   int ret;


   band = ShowAndReturnBand();
   if (band < 0)
	   return;
   chID = ShowAndReturnChannel(band);
   if(chID <= 0)
	   return;
   if (IsChannelInsideBandRange(band,chID) == FALSE)
   {
	   printf("Channel out of range!\n");
	   return;
   }

   bitArgs.cmdType = BIT_TELEC;
    bitArgs.band = band;
    bitArgs.channel = chID;

    ret = tiap_wcfg_set_bitArgs(wd, &bitArgs, 0);
    if(ret)
    {
		printf("Failed to set TELEC parameters.\n");
        return;
    }
    
   printf("Done.\n");
}


/************************************************************************/
/*						Mgmt sub-menu functions							*/
/************************************************************************/

/*!	\brief 
  	\param 

  	\return 
  
*/
void printMgmtAccess(ConsoleParameter_T parameters[])
{
	/*
    char tmp_buf[80*25];
    UINT8 enable, type;
    int size = 80*25;
    int numOfElements;

    whal_nvram_getAccessListEnable(&enable);
	mgmtAccessListStatus((char*)tmp_buf, size, &numOfElements);
    if (enable == 0)
        printf("Access Control status is: Disable\n\n");
    else
    {
        whal_nvram_getAccessListType(&type);
        printf("Access Control status is: Enabled, Type : %s\n\n", (type==1) ? "Black" : "White");
    }
    if (numOfElements == 0 )
        printf("\tThe Access Control table is empty.\n");
	else
    {
        puts("Access Table content:");
        puts(tmp_buf);
    }
    */

    char buff[MAX_SCREEN_BUFF];
    int num_entries = 0;

    tiap_param_access_list_t	access_list;

    access_list.cmd = TIAP_PARAM_ACCESS_LST;
    if (tiap_wcfg_get_access_list(wd, &access_list, !FROM_STORAGE || driver_load_failed))
    {
	    PRINT_CONFIG_ERR_MSG("Reading configuration failed");
	    return;
    }
    
    //for (i=0; i<TIAP_PARAM_ACCESS_LIST_SIZE; i++)
   //	print_mac(access_list.list[i]);
    
    access_list_to_str(buff, MAX_SCREEN_BUFF, access_list, &num_entries);
    if (access_list.enabled == 0)
        printf("Access Control status is: Disable\n\n");
    else
    {
        printf("Access Control status is: Enabled, Type : %s\n\n", (access_list.type==1) ? "Black" : "White");
    }
    if (! num_entries)
        printf("\tThe Access Control table is empty.\n");
	else
    {
        puts("Access Table content:");
        puts(buff);
    }
    
    
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wpa_repConfSeverity(ConsoleParameter_T parameters[])
{
	if (tiap_wcfg_wpaSetSeverity(wd, parameters[0].INTEGER))
			printf("%s: failed to set WPA reports severity\n",__func__);
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wpa_repConfReportModule(ConsoleParameter_T parameters[])
{
	if (tiap_wcfg_wpaSetRepModule(wd, parameters[0].INTEGER))
			printf("%s: failed to set WPA reports module\n",__func__);

}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wpa_reportShow(ConsoleParameter_T parameters[])
{
	UINT32 sev, mod;
	tiap_wcfg_wpaGetSeverity(wd, &sev);
	tiap_wcfg_wpaGetRepModule(wd, &mod);

	printf("\nReport severity:\n");
	if (sev & (1 << 1)) 
		printf("     Error      -     On\n");
	else
		printf("     Error      -     Off\n");
	if (sev & (1 << 2)) 
		printf("     Warning    -     On\n");
	else
		printf("     Warning    -     Off\n");
	if (sev & (1 << 3)) 
		printf("     Info       -     On\n");
	else
		printf("     Info       -     Off\n");
	printf("\n");
	printf("\nReport module:\n");
	if (mod & (1 << 1)) 
		printf("     Interfaces info       -    On\n");
	else
		printf("     Interfaces info       -    Off\n");
	if (mod & (1 << 2)) 
		printf("     Authentication info   -    On\n");
	else
		printf("     Authentication info   -    Off\n");
	if (mod & (1 << 3)) 
		printf("     WDS security profile  -    On\n");
	else
		printf("     WDS security profile  -    Off\n");
	if (mod & (1 << 4)) 
		printf("     WPA initialization	   -    On\n");
	else
		printf("     WPA initialization	   -    Off\n");
}

/*!	\brief  
  	\param 

  	\return 
  
*/
void wpa_reportPrintStatistics(ConsoleParameter_T parameters[])
{
	if (parameters[0].INTEGER < 0 || parameters[0].INTEGER > TIAP_MAX_NUM_OF_SUPPORTED_PROFILES-1) 
	{
		printf("Invalid profile index\n");
		return;
	}

	if (tiap_wcfg_wpaPrintProfStatistics(parameters[0].INTEGER))
		printf("Failed to print WPA internal profile statistics\n");

}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wpa_reportPrintProfCfg(ConsoleParameter_T parameters[])
{
	if (parameters[0].INTEGER < 0 || parameters[0].INTEGER > TIAP_MAX_NUM_OF_SUPPORTED_PROFILES-1) 
	{
		printf("Invalid profile index\n");
		return;
	}

	if (tiap_wcfg_wpaPrintProfCfg(parameters[0].INTEGER))
		printf("Failed to print security profile configuration\n");

}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wpa_reportPrintStaTbl(ConsoleParameter_T parameters[])
{
	if (parameters[0].INTEGER < 0 || parameters[0].INTEGER > TIAP_MAX_NUM_OF_SUPPORTED_PROFILES-1) 
	{
		printf("Invalid profile index\n");
		return;
	}

	if (tiap_wcfg_wpaPrintStaTbl(parameters[0].INTEGER))
		printf("Failed to print WPA internal STA table\n");

}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wpa_reportPrintPmkCacheList(ConsoleParameter_T parameters[])
{
	if (tiap_wcfg_wpaPrintPmkCacheList())
    {
        printf("Failed to print WPA PMK Cache list\n");
    }
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void printStaTable(ConsoleParameter_T parameters[]){
	//wdrv_dbPrintStaTable();
	DRIVER_LOADED_ASSERT;
	if(dda_print_proc_entry(WLAN_PROC_MGMT_DIR_NAME, WLAN_PROC_MGMT_STATABLE) == -1)
		PRINT_PROC_READ_ERR(WLAN_PROC_MGMT_DIR_NAME, WLAN_PROC_MGMT_STATABLE);
		
}



/*!	\brief 
  	\param 

  	\return 
  
*/
void printStaStatistics(ConsoleParameter_T parameters[]){
	DRIVER_LOADED_ASSERT;
	if(dda_print_proc_entry(WLAN_PROC_MGMT_DIR_NAME, WLAN_PROC_MGMT_STASTATISTICS) == -1)
		PRINT_PROC_READ_ERR(WLAN_PROC_MGMT_DIR_NAME, WLAN_PROC_MGMT_STASTATISTICS);
		
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void printRatesTable(ConsoleParameter_T parameters[]){
	//wdrv_dbPrintRatesTable();
	DRIVER_LOADED_ASSERT;
	if (dda_print_proc_entry(WLAN_PROC_MGMT_DIR_NAME, WLAN_PROC_MGMT_STARATES) == -1)
		PRINT_PROC_READ_ERR(WLAN_PROC_MGMT_DIR_NAME, WLAN_PROC_MGMT_STARATES);
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void printHashTable(ConsoleParameter_T parameters[]){
	//wdrv_dbPrintHashTable();
	DRIVER_LOADED_ASSERT;
	if (dda_print_proc_entry(WLAN_PROC_MGMT_DIR_NAME, WLAN_PROC_MGMT_STAHASH) == -1)
		PRINT_PROC_READ_ERR(WLAN_PROC_MGMT_DIR_NAME, WLAN_PROC_MGMT_STAHASH);
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void mgmtShow(ConsoleParameter_T parameters[]){
	
	/*BOOL	enable4x;
	dot11_TIcap_T	TIcap;

	wdrv_dbGet4xEnable(&enable4x);
	wdrv_dbGetTIInfoElement(TIcap.cap, &(TIcap.hdr.eleLen));		
	
	wdrv_dbPrintNumOfStations();

	if (wdrv_mgmtLogEnable())
		printf("Management Log:		Enable\n");
	else
		printf("Management Log:		Disable\n");

	if (wdrv_mgmtAgingEnable())
		printf("Management Aging:	Enable\n");
	else
		printf("Management Aging:	Disable\n");

	if (enable4x == TRUE)
	{
		int i;

		printf("4x:	Enable\n");
		printf("0x%x ", TIcap.hdr.eleId);
		printf("0x%x ", TIcap.hdr.eleLen);
		for(i=0;i<TIcap.hdr.eleLen;i++)
			printf("0x%x ", TIcap.cap[i]);
		printf("\n");
	}
	else
		printf("4x:	Disable\n");
	*/
	DRIVER_LOADED_ASSERT;
	if (dda_print_proc_entry(WLAN_PROC_MGMT_DIR_NAME, WLAN_PROC_MGMT_SHOW) == -1)
		PRINT_PROC_READ_ERR(WLAN_PROC_MGMT_DIR_NAME, WLAN_PROC_MGMT_SHOW);
	
}



/*!	\brief 
  	\param 

  	\return 
  
*/
void servEnable(ConsoleParameter_T parameters[])
{
	//wdrv_WlanServiceEnable();
	__u8 state = 1;
	
	if( tiap_wcfg_set_basic(wd, TIAP_PARAM_OPER_STATE, &state, UPDATE_STORAGE) ) {
		fprintf(stderr, "Failed to set AP's operating state.\n\n");
		fflush(stderr);
	}
	else
		printf("\n\nAP operating state was set to Enabled\n\n");
}


/*!	\brief 
  	\param 

  	\return 
  
*/
void servDisable(ConsoleParameter_T parameters[])
{
	//wdrv_WlanServiceDisable();
	__u8 state = 0;
	
	if( tiap_wcfg_set_basic(wd, TIAP_PARAM_OPER_STATE, &state, UPDATE_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to set AP's operating state.");
		return;
	}
	else
		printf("\n\nAP operating state was set to Disabled\n\n");
}



/*!	\brief 
  	\param 

  	\return 
  
*/
void deAuthMng(ConsoleParameter_T parameters[]){
	//wdrv_mgmtDeAuthMng(parameters[0].MAC_ADDR, parameters[1].SHORT);
	proc_params proc_pars;
	int ret;

	DRIVER_LOADED_ASSERT;
	
	proc_pars.s.magic = PROC_PARAMS_MAGIC;
	proc_pars.s.Param1 = parameters[1].SHORT;
	memcpy(proc_pars.s.mac, &parameters[0].MAC_ADDR, sizeof(proc_pars.s.mac));

	ret = dda_write_proc_entry(WLAN_PROC_MGMT_DIR_NAME, WLAN_PROC_MGMT_DEAUTH, 
		(char *)&proc_pars, sizeof(proc_pars));

	if (ret != sizeof(proc_pars))
		PRINT_PROC_WRITE_ERR(WLAN_PROC_MGMT_DIR_NAME, WLAN_PROC_MGMT_DEAUTH);


}

/********************************************************************/
/*                                                                  */
/*              Management Access Control Commands                  */
/*                                                                  */
/********************************************************************/

/*!	\brief 
  	\param 

  	\return 
  
*/
void setAccessMAC(ConsoleParameter_T parameters[])
{
/*	char tmpTable[MGMT_ACCESS_LIST_SIZE*MAC_ADDR_LEN];
    int length = MGMT_ACCESS_LIST_SIZE*MAC_ADDR_LEN;
    STATUS status;

	if (mgmtAccessSetMAC(parameters[0].MAC_ADDR) != OK)
    {
        printf("Error - Unable to set this address to the table!\n");
        return;
    }
    // Update table in NVRAM 
	status = mgmtAccessGetList(tmpTable, &length);
    if (status != OK) 
    {
        printf("setAccessMAC: Fatal Error from mgmtAccessGetList (file %s)\n",__FILE__);
        return;
    }
	whal_nvram_setAccessList(tmpTable, IMMEDIATELY);
*/
    
    tiap_param_access_list_t access_lst;

	access_lst.cmd = TIAP_PARAM_ACCESS_LST;

	/*get access list type*/
	if (tiap_wcfg_get_access_list(wd, &access_lst, FROM_STORAGE))
	{
		PRINT_CONFIG_ERR_MSG("Reading configuration failed");
		return;
	}

    access_lst.cmd = TIAP_PARAM_ACCESS_ADD;

    memcpy(access_lst.list[0], parameters[0].MAC_ADDR, ETH_ALEN);
    
	if (tiap_wcfg_set_access_list(wd, &access_lst, UPDATE_STORAGE))
	{
		PRINT_CONFIG_ERR_MSG("Configuration failed");
		return;
	}
    
    return;
    	
}


/*!	\brief 
  	\param 

  	\return 
  
*/
void remAccessMAC(ConsoleParameter_T parameters[])
{
/*
    char tmp_buf[80*25];    // attention: multiple use of this buffer ahead.. 
	char tmpTable[MGMT_ACCESS_LIST_SIZE*MAC_ADDR_LEN];
    int length = MGMT_ACCESS_LIST_SIZE*MAC_ADDR_LEN;
    int numOfElements;
    STATUS status;
    UINT32 index, size=80*25;

    // print the table 
	mgmtAccessListStatus((char*)tmp_buf, size, &numOfElements);
    puts("Access Table content:");
	puts(tmp_buf);

    // get the index to remove 
    printf("Type MAC index to remove: ");
    fgets(tmp_buf, 100, stdin);
    tmp_buf[strlen(tmp_buf) - 1] = 0; // remove newline 
    index = atoi(tmp_buf);
    if ((index == 0) || (index > numOfElements))
    {
        printf("Bad value.\n");
        return;
    }

    // remove the index 
	if (mgmtAccessRemMAC((UINT16)index) != OK)
    {
        printf("Error - index not found!\n");
        return;
    }
    // Update table in NVRAM 
	status = mgmtAccessGetList(tmp_buf, &length);
    if (status != OK) 
    {
        printf("remAccessMAC: Fatal Error from mgmtAccessGetList (file %s)\n",__FILE__);
        return;
    }
	whal_nvram_setAccessList(tmp_buf, IMMEDIATELY);
*/

    char tmp_buf[MAX_SCREEN_BUFF];    /* attention: multiple use of this buffer ahead.. */
    int numOfElements, index; 

    tiap_param_access_list_t access_lst;

    access_lst.cmd = TIAP_PARAM_ACCESS_LST;
    if (tiap_wcfg_get_access_list(wd, &access_lst, FROM_STORAGE))
    {
	    PRINT_CONFIG_ERR_MSG("Reading configuration failed.");
	    return;
    }

    /* print the table */
    access_list_to_str(tmp_buf, MAX_SCREEN_BUFF, access_lst, &numOfElements);
    puts("Access Table content:");
    puts(tmp_buf);

    /* get the index to remove */
    printf("Type MAC index to remove: ");
    fgets(tmp_buf, MAX_PRINT_STRING, stdin);
    tmp_buf[strlen(tmp_buf) - 1] = 0; /* remove newline */
    index = atoi(tmp_buf);
    if ((index == 0) || (index > numOfElements))
    {
        printf("Bad value.\n");
        return;
    }

    /* remove the index */
  
    access_lst.cmd = TIAP_PARAM_ACCESS_DEL_BY_INDEX;
    access_lst.index = index;
    
    if (tiap_wcfg_set_access_list(wd, &access_lst, UPDATE_STORAGE)) {
	PRINT_CONFIG_ERR_MSG("Configuration failed.");
	return;
    }
    
    return;
}


/*!	\brief 
  	\param 

  	\return 
  
*/
void enableAccess(ConsoleParameter_T parameters[])
{
	/*
	whal_nvram_setAccessListEnable(1, IMMEDIATELY);
    whal_nvram_setAccessListType(parameters[0].SHORT, IMMEDIATELY);
    printf("Access Control is enabled. Type is: %s\n",(parameters[0].SHORT==0) ? "White" : "Black");
	printf("to take effect RESTART AP.\n");
	*/

	tiap_param_access_list_t access_lst;


	access_lst.cmd = TIAP_PARAM_ACCESS_LST;
	
	if (tiap_wcfg_get_access_list(wd, &access_lst, FROM_STORAGE))
	{
	    PRINT_CONFIG_ERR_MSG("Reading configuration failed");
	    return;
	}


	access_lst.enabled = 1;
	access_lst.type = parameters[0].SHORT;
	access_lst.cmd = TIAP_PARAM_ACCESS_LST;
    
	if (tiap_wcfg_set_access_list(wd, &access_lst, UPDATE_STORAGE))
	{
		PRINT_CONFIG_ERR_MSG("Configuration failed");
		return;
	}
	printf("Access Control is enabled. Type is: %s\n",(parameters[0].SHORT==0) ? "White" : "Black");
	printf("to take effect RESTART AP.\n");
	
}


/*!	\brief 
  	\param 

  	\return 
  
*/
void disableAccess(ConsoleParameter_T parameters[])
{
	/*
	whal_nvram_setAccessListEnable(0, IMMEDIATELY);
	printf("Access Control is disabled\n");
	printf("to take effect RESTART AP.\n");
	*/
	tiap_param_access_list_t access_lst;

	access_lst.cmd = TIAP_PARAM_ACCESS_LST;
	
	if (tiap_wcfg_get_access_list(wd, &access_lst, FROM_STORAGE))
	{
	    PRINT_CONFIG_ERR_MSG("Reading configuration failed");
	    return;
	}
	
	access_lst.enabled = 0;
    
	if (tiap_wcfg_set_access_list(wd, &access_lst, UPDATE_STORAGE))
	{
		PRINT_CONFIG_ERR_MSG("Configuration failed");
		return;
	}
	printf("Access Control is disabled\n");
	printf("to take effect RESTART AP.\n");

}

/*      End -  Management Access Control Commands - End             */


/*!	\brief 
  	\param 

  	\return 
  
*/
void disAssocMng(ConsoleParameter_T parameters[]){
	//wdrv_mgmtDisAssocMng(parameters[0].MAC_ADDR, parameters[1].SHORT);

	proc_params proc_pars;
	int ret;

	DRIVER_LOADED_ASSERT;

	proc_pars.s.magic = PROC_PARAMS_MAGIC;
	proc_pars.s.Param1 = parameters[1].SHORT;
	memcpy(proc_pars.s.mac, &parameters[0].MAC_ADDR, sizeof(proc_pars.s.mac));

	ret = dda_write_proc_entry(WLAN_PROC_MGMT_DIR_NAME, WLAN_PROC_MGMT_DISASS, 
		(char *)&proc_pars, sizeof(proc_pars));

	if (ret != sizeof(proc_pars))
		PRINT_PROC_WRITE_ERR(WLAN_PROC_MGMT_DIR_NAME, WLAN_PROC_MGMT_DISASS);
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void mgmtLog(ConsoleParameter_T parameters[]){
	/*
	if ((parameters[0].INTEGER != 0) && (parameters[0].INTEGER != 1))
	{
		printf("Invalid Log mode: %d\n", parameters[0].INTEGER);
		return;
	}
	wdrv_mgmtSetLog((BOOL)parameters[0].INTEGER);
	*/

	int ret;

	DRIVER_LOADED_ASSERT;

	if ((parameters[0].INTEGER != 0) && (parameters[0].INTEGER != 1))
	{
		printf("Invalid Log mode: %ld\n", parameters[0].INTEGER);
		return;
	}

	ret = dda_write_proc_entry(WLAN_PROC_MGMT_DIR_NAME, WLAN_PROC_MGMT_SETLOG, 
		(char *)&(parameters[0].INTEGER), sizeof(parameters[0].INTEGER));
	if (ret != sizeof(parameters[0].INTEGER))  
		PRINT_PROC_WRITE_ERR(WLAN_PROC_MGMT_DIR_NAME, WLAN_PROC_MGMT_SETLOG);

}

/*!	\brief 
  	\param 

  	\return 
  
*/
void printMgmtCounters(ConsoleParameter_T parameters[]){
	//wdrv_mgmtPrintCounters();
	if (dda_print_proc_entry(WLAN_PROC_MGMT_DIR_NAME, WLAN_PROC_MGMT_COUNTERS) == -1)
		PRINT_PROC_READ_ERR(WLAN_PROC_MGMT_DIR_NAME, WLAN_PROC_MGMT_COUNTERS);
		
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void mgmtAging(ConsoleParameter_T parameters[]){
	/*
	if ((parameters[0].INTEGER != 0) && (parameters[0].INTEGER != 1))
	{
		printf("Invalid Aging mode: %d\n", parameters[0].INTEGER);
		return;
	}
	wdrv_mgmtSetAging((BOOL)parameters[0].INTEGER);
	*/
	int ret;

	DRIVER_LOADED_ASSERT;

	if ((parameters[0].INTEGER != 0) && (parameters[0].INTEGER != 1))
	{
		printf("Invalid Aging mode: %ld\n", parameters[0].INTEGER);
		return;
	}

	ret = dda_write_proc_entry(WLAN_PROC_MGMT_DIR_NAME, WLAN_PROC_MGMT_AGING, 
		(char *)&(parameters[0].INTEGER), sizeof(parameters[0].INTEGER));

	if (ret != sizeof(parameters[0].INTEGER))
		PRINT_PROC_WRITE_ERR(WLAN_PROC_MGMT_DIR_NAME, WLAN_PROC_MGMT_AGING);

}

/************************************************************************/
/*						Mgmt-test sub-menu functions							*/
/************************************************************************/
/*!	\brief 
  	\param 

  	\return 
  
*/
void findSta(ConsoleParameter_T parameters[]){

	/*UINT16 idx;
	char mac[MAC_ADDR_LEN];
	memset(mac, parameters[0].INTEGER, MAC_ADDR_LEN);
	wdrv_dbFindStaIndex(mac, &idx);*/

	char buf[GENERIC_IOCTL_BUF_SIZE];

	DRIVER_LOADED_ASSERT;
	 
	if (ioctl_get_find_sta((char *)&parameters[0].INTEGER, buf, GENERIC_IOCTL_BUF_SIZE, wd->dev_name)) 
		puts("ioctl_get_find_sta failed !");
	else
		puts(buf);
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void insertSta(ConsoleParameter_T parameters[]){

	char buf[GENERIC_IOCTL_BUF_SIZE];

	DRIVER_LOADED_ASSERT;

	if (ioctl_get_insert_sta((char *)&parameters[0].INTEGER, parameters[1].INTEGER,  buf, GENERIC_IOCTL_BUF_SIZE, wd->dev_name)) 
		puts("ioctl_get_insert_sta failed !");
	else
		puts(buf);
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void removeSta(ConsoleParameter_T parameters[])
{
/*
    char macAddr[MAC_ADDR_LEN];

    wdrv_dbGetStaMacAddress(parameters[0].INTEGER,macAddr);
    wdrv_mgmtDeAuthMng(macAddr,12*//*STATUS_ASSOC_DENIED_OUT_OF_SCOPE*//*);*/
	/*wdrv_dbRemoveStaIndex(parameters[0].INTEGER);*/
     deAuthMng(parameters);
    
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void null(ConsoleParameter_T parameters[]){
	//wdrv_mgmtTstNull(parameters[0].INTEGER, parameters[1].INTEGER, parameters[2].INTEGER, parameters[3].INTEGER);

	proc_params proc_pars;
	int ret;

	DRIVER_LOADED_ASSERT;

	proc_pars.s.magic = PROC_PARAMS_MAGIC;
	proc_pars.s.Param1 = parameters[0].INTEGER;
	proc_pars.s.Param2 = parameters[1].INTEGER;
	proc_pars.s.Param3 = parameters[2].INTEGER;
	proc_pars.s.Param4 = parameters[3].INTEGER;

	ret = dda_write_proc_entry(WLAN_PROC_MGMT_DIR_NAME, WLAN_PROC_MGMT_TST_NULL, 
		(char *)&proc_pars, sizeof(proc_pars));

	if (ret != sizeof(proc_pars))
		PRINT_PROC_WRITE_ERR(WLAN_PROC_MGMT_DIR_NAME, WLAN_PROC_MGMT_TST_NULL);



}

/*!	\brief 
  	\param 

  	\return 
  
*/
void auth1(ConsoleParameter_T parameters[]){
	//wdrv_mgmtTstAuth1(parameters[0].INTEGER, parameters[1].SHORT);
	proc_params proc_pars;
	int ret;

	DRIVER_LOADED_ASSERT;
	
	proc_pars.s.magic = PROC_PARAMS_MAGIC;
	proc_pars.s.Param1 = parameters[0].INTEGER;
	proc_pars.s.Param2 = parameters[1].SHORT;

	ret = dda_write_proc_entry(WLAN_PROC_MGMT_DIR_NAME, WLAN_PROC_MGMT_TST_AUTH1, 
		(char *)&proc_pars, sizeof(proc_pars));

	if (ret != sizeof(proc_pars))
		PRINT_PROC_WRITE_ERR(WLAN_PROC_MGMT_DIR_NAME, WLAN_PROC_MGMT_TST_AUTH1);

}

/*!	\brief 
  	\param 

  	\return 
  
*/
void auth3(ConsoleParameter_T parameters[]){
	//wdrv_mgmtTstAuth3(parameters[0].INTEGER, parameters[1].SHORT, parameters[2].INTEGER);
	proc_params proc_pars;
	int ret;

	DRIVER_LOADED_ASSERT;

	proc_pars.s.magic = PROC_PARAMS_MAGIC;
	proc_pars.s.Param1 = parameters[0].INTEGER;
	proc_pars.s.Param2 = parameters[1].SHORT;
	proc_pars.s.Param3 = parameters[2].INTEGER;

	ret = dda_write_proc_entry(WLAN_PROC_MGMT_DIR_NAME, WLAN_PROC_MGMT_TST_AUTH3, 
		(char *)&proc_pars, sizeof(proc_pars));

	if (ret != sizeof(proc_pars))
		PRINT_PROC_WRITE_ERR(WLAN_PROC_MGMT_DIR_NAME, WLAN_PROC_MGMT_TST_AUTH3);


}

/*!	\brief 
  	\param 

  	\return 
  
*/
void assoc(ConsoleParameter_T parameters[]){
	//wdrv_mgmtTstAssoc(parameters[0].INTEGER, (BOOL)parameters[1].INTEGER, parameters[2].INTEGER, parameters[3].INTEGER);
	proc_params proc_pars;
	int ret;

	DRIVER_LOADED_ASSERT;

	proc_pars.s.magic = PROC_PARAMS_MAGIC;
	proc_pars.s.Param1 = parameters[0].INTEGER;
	proc_pars.s.Param2 = parameters[1].INTEGER;
	proc_pars.s.Param3 = parameters[2].INTEGER;
	proc_pars.s.Param4 = parameters[3].INTEGER;

	ret = dda_write_proc_entry(WLAN_PROC_MGMT_DIR_NAME, WLAN_PROC_MGMT_TST_ASS, 
		(char *)&proc_pars, sizeof(proc_pars));

	if (ret != sizeof(proc_pars))
		PRINT_PROC_WRITE_ERR(WLAN_PROC_MGMT_DIR_NAME, WLAN_PROC_MGMT_TST_ASS);

}

/*!	\brief 
  	\param 

  	\return 
  
*/
void reAssoc(ConsoleParameter_T parameters[]){
	//wdrv_mgmtTstReassoc(parameters[0].INTEGER, (BOOL)parameters[1].INTEGER, parameters[2].INTEGER, parameters[3].INTEGER);
	proc_params proc_pars;
	int ret;

	DRIVER_LOADED_ASSERT;

	proc_pars.s.magic = PROC_PARAMS_MAGIC;
	proc_pars.s.Param1 = parameters[0].INTEGER;
	proc_pars.s.Param2 = parameters[1].INTEGER;
	proc_pars.s.Param3 = parameters[2].INTEGER;
	proc_pars.s.Param4 = parameters[3].INTEGER;

	ret = dda_write_proc_entry(WLAN_PROC_MGMT_DIR_NAME, WLAN_PROC_MGMT_TST_REASS, 
		(char *)&proc_pars, sizeof(proc_pars));

	if (ret != sizeof(proc_pars))
		PRINT_PROC_WRITE_ERR(WLAN_PROC_MGMT_DIR_NAME, WLAN_PROC_MGMT_TST_REASS);

}

/*!	\brief 
  	\param 

  	\return 
  
*/
void deAuth(ConsoleParameter_T parameters[]){
	//wdrv_mgmtTstDeAuth(parameters[0].INTEGER, parameters[1].SHORT);
	proc_params proc_pars;
	int ret;

	DRIVER_LOADED_ASSERT;

	proc_pars.s.magic = PROC_PARAMS_MAGIC;
	proc_pars.s.Param1 = parameters[0].INTEGER;
	proc_pars.s.Param2 = parameters[1].SHORT;
	

	ret = dda_write_proc_entry(WLAN_PROC_MGMT_DIR_NAME, WLAN_PROC_MGMT_TST_DEAUTH, 
		(char *)&proc_pars, sizeof(proc_pars));

	if (ret != sizeof(proc_pars))
		PRINT_PROC_WRITE_ERR(WLAN_PROC_MGMT_DIR_NAME, WLAN_PROC_MGMT_TST_DEAUTH);
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void disAssoc(ConsoleParameter_T parameters[]){
	//wdrv_mgmtTstDisAssoc(parameters[0].INTEGER, parameters[1].SHORT);
	proc_params proc_pars;
	int ret;

	DRIVER_LOADED_ASSERT;

	proc_pars.s.magic = PROC_PARAMS_MAGIC;
	proc_pars.s.Param1 = parameters[0].INTEGER;
	proc_pars.s.Param2 = parameters[1].SHORT;
	

	ret = dda_write_proc_entry(WLAN_PROC_MGMT_DIR_NAME, WLAN_PROC_MGMT_TS_DEASS, 
		(char *)&proc_pars, sizeof(proc_pars));

	if (ret != sizeof(proc_pars))
		PRINT_PROC_WRITE_ERR(WLAN_PROC_MGMT_DIR_NAME, WLAN_PROC_MGMT_TS_DEASS);


}

/*!	\brief 
  	\param 

  	\return 
  
*/
void class3(ConsoleParameter_T parameters[]){
	//wdrv_mgmtTstClass3(parameters[0].INTEGER, parameters[1].SHORT);
	proc_params proc_pars;
	int ret;

	DRIVER_LOADED_ASSERT;

	proc_pars.s.magic = PROC_PARAMS_MAGIC;
	proc_pars.s.Param1 = parameters[0].INTEGER;
	proc_pars.s.Param2 = parameters[1].SHORT;
	

	ret = dda_write_proc_entry(WLAN_PROC_MGMT_DIR_NAME, WLAN_PROC_MGMT_TST_CLASS3, 
		(char *)&proc_pars, sizeof(proc_pars));

	if (ret != sizeof(proc_pars))
		PRINT_PROC_WRITE_ERR(WLAN_PROC_MGMT_DIR_NAME, WLAN_PROC_MGMT_TST_CLASS3);

}

/*!	\brief 
  	\param 

  	\return 
  
*/
void showChallenge(ConsoleParameter_T parameters[]){
	//wdrv_mgmtPrintChallenge(parameters[0].SHORT);
#if 0
	proc_params proc_pars;
	int ret;

	proc_pars.s.magic = PROC_PARAMS_MAGIC;
	proc_pars.s.Param1 = parameters[0].SHORT;
	

	ret = dda_write_proc_entry(WLAN_PROC_MGMT_DIR_NAME, WLAN_PROC_MGMT_TST_CHALLENGE, 
		(char *)&proc_pars, sizeof(proc_pars));

	if (ret != sizeof(proc_pars))
		PRINT_PROC_WRITE_ERR(WLAN_PROC_MGMT_DIR_NAME, WLAN_PROC_MGMT_TST_CHALLENGE);
#endif
	char buf[GENERIC_IOCTL_BUF_SIZE];

	DRIVER_LOADED_ASSERT;

	if (ioctl_get_challenge(parameters[0].SHORT, buf, GENERIC_IOCTL_BUF_SIZE, wd->dev_name)) 
		puts("ioctl_get_challenge failed !");
	else
		puts(buf);
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void mgmtReset(ConsoleParameter_T parameters[]){
	//wdrv_mgmtReset();
	DRIVER_LOADED_ASSERT;
	dda_write_proc_entry(WLAN_PROC_MGMT_DIR_NAME, WLAN_PROC_MGMT_TST_RESET, NULL, 0);
}

/************************************************************************/
/*						Privacy sub-menu functions							*/
/************************************************************************/

/*!	\brief 
  	\param 

  	\return 
  
*/
void wpaShow(ConsoleParameter_T parameters[])
{
	
	__u32	value = 0;
    __u16   val16 = 0;
	int  i, n, wepIndex, keyLen;
	__u8 len,flag;
	__s8 passwd[TIAP_WPA_MAX_PSK_PWD_LEN+1];
    __s8 tempChar;
	__s8 secret[TIAP_WPA_MAX_RADIUS_SECRET_LEN+1];
	struct in_addr addr;
	int ret;
	tiap_param_multi_ssid_t ssid_table;
	tiap_param_security_type_t mode;
    tiap_wpa_wep_key_t wep_key;
    __u8 def_key_id = 0;
    tiap_param_auth_algo_t tbl[TIAP_PARAM_MAX_AUTH_ALG_SUPPORTED];
    BOOL openSystemEnable = FALSE;
    BOOL sharedKeyEnable = FALSE;

	if(tiap_wcfg_get_multi_ssid(wd,&ssid_table,FROM_STORAGE))
	{
		PRINT_CONFIG_ERR_MSG("error getting multi ssid table.");
		return;
	}
    
    tiap_wcfg_get_auth_type(wd, tbl, FROM_STORAGE);
    for (i = 0; i < TIAP_PARAM_MAX_AUTH_ALG_SUPPORTED; i++)
	{
		if ((tbl[i].algo == MIB_OPEN_SYSTEM) && (tbl[i].enable == TRUE))
			openSystemEnable = TRUE;
		else if ((tbl[i].algo == MIB_SHARED_KEY) && (tbl[i].enable == TRUE))
			sharedKeyEnable = TRUE;
	}

	n = 0;
	for( i =0; i < TIAP_PARAM_NUM_MULTIPLE_SSID; i++)
	{
		if ( ssid_table.tbl[i].ssid_type == TIAP_PARAM_SSID_NON_ACTIVE ||
			 ssid_table.tbl[i].ssid_type == TIAP_PARAM_SSID_INTERNAL) 
			 continue;

		if(tiap_wcfg_get_security_type(wd, &mode, i)) {
			PRINT_CONFIG_ERR_MSG("Failed to get wpa mode.\n");
			return;
		}
	    
        tiap_wcfg_get_default_cipher(wd, &value,i);
        
		/* Show to user sequence numbers of profiles and not actual */
		printf("ssid %s ,profile index %d:\n",ssid_table.tbl[i].ssid, n);
		if (mode & TIAP_PARAM_SECURITY_WEP)
        {
            if (value == TIAP_WPA_CIPHER_NON)
                printf("	WEP Legacy                      - Off\n");
            else
                printf("	WEP Legacy		                - On\n");
        }
		else
			printf("	WEP Legacy                      - Off\n");
		
		if (mode & TIAP_PARAM_SECURITY_802_1X)
			printf("	EAP-TLS (Dynamic WEP) Stations  - On\n");
		else
			printf("	EAP-TLS (Dynamic WEP) Stations  - Off\n");
		
		if (mode & TIAP_PARAM_SECURITY_WPA)
			printf("	WPA Stations                    - On\n");
		else
			printf("	WPA Stations                    - Off\n");
		
		if (mode & TIAP_PARAM_SECURITY_WPA2)
			printf("	WPA2 Stations                   - On\n");
		else
			printf("	WPA2 Stations                   - Off\n");
		
		
		tiap_wcfg_get_psk_passphrase_mode(wd, &value,i);
		
		
		if (value == TIAP_WPA_PSK_PARAM_NONE)
			printf("	Pass Phrase Mode                - None\n");
		else if (value == TIAP_WPA_PSK_PARAM_HEX)
			printf("	Pass Phrase Mode                - HEX\n");
		else if (value == TIAP_WPA_PSK_PARAM_ASCII)
			printf("	Pass Phrase Mode                - ASCII\n");
		else 
			printf("	Pass Phrase Mode                - Invalid\n");
		
		tiap_wcfg_get_default_cipher(wd, &value,i);
		
		if (value == TIAP_WPA_CIPHER_NON)
			printf("	Default Cipher                  - None\n");
		else if (value == TIAP_WPA_CIPHER_WEP_40)
			printf("	Default Cipher                  - WEP - 40\n");
		else if (value == TIAP_WPA_CIPHER_WEP_104)
			printf("	Default Cipher                  - WEP - 128\n");
		else if (value == TIAP_WPA_CIPHER_WEP_232)
			printf("	Default Cipher                  - WEP - 256\n");
		else if (value == TIAP_WPA_CIPHER_TKIP)
			printf("	Default Cipher                  - TKIP\n");
		else if (value == TIAP_WPA_CIPHER_CCMP)
			printf("	Default Cipher                  - CCMP\n");
		else
			printf("	Default Cipher                  Invalid\n");

        if ((openSystemEnable == TRUE) && (sharedKeyEnable == FALSE))
            printf("	Auth Algorithm                  - Open System\n");
        
        else if ((openSystemEnable == FALSE) && (sharedKeyEnable == TRUE))
            printf("	Auth Algorithm                  - Shared Key\n");
        
        else if ((openSystemEnable == TRUE) && (sharedKeyEnable == TRUE))
            printf("	Auth Algorithm                  - Both Open & Shared\n");
		        
		tiap_wcfg_get_psk_passwd(wd, passwd, &len,i);
		passwd[len] = '\0';
        if(len <= 40)
        {
            printf("	PSK Password:			%s\n", passwd);
        }
        else
        {
            tempChar = passwd[40];
            passwd[40] = '\0';
            printf("	PSK Password:			%s\n", passwd);
            passwd[40] = tempChar;
            printf("	             			%s\n", &passwd[40]);
        }
		
        /* Geeting Default WEP key ID */
        tiap_wcfg_get_wep_default_key_id(wd, &def_key_id, i);

        /* Print WEP keys */
        printf("	WEP Keys: ");
        for(wepIndex = 0; wepIndex < 4; wepIndex++)
        {
            if(wepIndex != 0)
                printf("                  ");
            tiap_wcfg_get_def_wep_key(wd, wepIndex, &wep_key, i);
            printf("Index %d:	        ", wepIndex);

            for(keyLen = 0; keyLen < wep_key.len; keyLen++)
            {
                if(keyLen == 20)
                {
                    printf("\n                          	        ");
                }
                printf("%x", wep_key.key[keyLen]);
            }

            if(def_key_id == wepIndex)
                printf("(Default)\n");
            else
                printf("\n");
        }
        
		tiap_wcfg_get_gtk_interval(wd, &value,i);
		printf("	Group Key update Interval:	%d	seconds\n", value / 1000);
		
		tiap_wcfg_get_pre_auth_flag(wd, &flag,i);
		if(flag)
			printf("	pre authentication support:	on\n" );
		else
			printf("	pre authentication support:	off\n" );
		
        tiap_wcfg_get_radius_secret(wd, secret, &len,i);
        secret[len] = '\0';
        if(len <= 40)
        {
            printf("	Radius Secret:			%s\n", secret);
        }
        else
        {
            tempChar = secret[40];
            secret[40] = '\0';
            printf("	Radius Secret:			%s\n", secret);
            secret[40] = tempChar;
            printf("	              			%s\n", &secret[40]);
        }
        
		
		tiap_wcfg_get_radius_port(wd, &val16, i);
		printf("	Radius Port:			%hu\n", (UINT32)val16);
		
		tiap_wcfg_get_radius_ipaddr(wd, &value,i);
		addr.s_addr = value;
		
		printf("	Radius IP Address:		%s\n", inet_ntoa(addr));
		
		ret = tiap_wcfg_get_default_gateway(&value);	
		if (ret == NOK)
			return;
		else 
		{
			addr.s_addr = value;
			printf("	Default Gateway:		%s\n\n", inet_ntoa(addr));
		}

		n++;
	}

/*	if (dda_print_proc_entry(WLAN_PROC_PRIVACY_DIR_NAME, WLAN_PROC_PRIVACY_CFG) == -1)
		PRINT_PROC_READ_ERR(WLAN_PROC_PRIVACY_DIR_NAME, WLAN_PROC_PRIVACY_CFG);*/
	

}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wpaWorkingMode(ConsoleParameter_T parameters[])
{	
	int     profile_index = parameters[0].INTEGER;
    int     tempProfileIndex;
	int     mode = 0;
    __u32   wepCipher = 0;
    BOOL    isCipherExists;

	if(profile_index > tiap_wcfg_get_num_of_multi_ssid(wd))
	{
		printf("Invalid profile index.\n");
		return;
	}

	if ((parameters[1].INTEGER == FALSE) && (parameters[2].INTEGER == FALSE) &&
		(parameters[3].INTEGER == FALSE) && (parameters[4].INTEGER == FALSE))
	{
        mode |= DOT11_WPA_STA_TYPE_WEP_LEGACY;

        printf("NO SECURITY , profile %d:setting default cipher to NONE\n",profile_index);
        if (tiap_wcfg_set_default_cipher(wd, TIAP_WPA_CIPHER_NON,profile_index, !UPDATE_STORAGE))
        {
            PRINT_CONFIG_ERR_MSG("Failed to set default cipher");
            return;
        }
	}
    else
    {
        
        if (parameters[1].INTEGER == TRUE)
        {
            mode |= DOT11_WPA_STA_TYPE_WEP_LEGACY;
        }
        if (parameters[2].INTEGER == TRUE)
        {
            mode |= DOT11_WPA_STA_TYPE_WEP_8021X;
        }
        if (parameters[3].INTEGER == TRUE)
        {
            mode |= DOT11_WPA_STA_TYPE_WPA;
        }
        
        if (parameters[4].INTEGER == TRUE)
        {
            mode |= DOT11_WPA_STA_TYPE_WPA2;
        }
        
        /* setting default cipher */
        if(mode & DOT11_WPA_STA_TYPE_WPA)
        {
            printf("WPA cipher supported in profile %d:setting default cipher to TKIP\n",profile_index);
            if (tiap_wcfg_set_default_cipher(wd, TIAP_WPA_CIPHER_TKIP,profile_index, !UPDATE_STORAGE))
            {
                PRINT_CONFIG_ERR_MSG("Failed to set default cipher");
                return;
            }
        }
        
        else if(mode & DOT11_WPA_STA_TYPE_WPA2)
        {
            printf("WPA2 only , profile %d:setting default cipher to CCMP\n",profile_index);
            if (tiap_wcfg_set_default_cipher(wd, TIAP_WPA_CIPHER_CCMP,profile_index, !UPDATE_STORAGE))
            {
                PRINT_CONFIG_ERR_MSG("Failed to set default cipher");
                return;
            }
        }
        
        else if(mode & DOT11_WPA_STA_TYPE_WEP_8021X) 
        {
            printf("WEP 802.1X , profile %d:setting default cipher to WEP40\n",profile_index);
            if (tiap_wcfg_set_default_cipher(wd, TIAP_WPA_CIPHER_WEP_40,profile_index, !UPDATE_STORAGE))
            {
                PRINT_CONFIG_ERR_MSG("Failed to set default cipher");
                return;
            }
        }
        
        else if(mode & DOT11_WPA_STA_TYPE_WEP_LEGACY)
        {
            /* In Multi-SSID: Only one pofile can be configured to work with     *
             * static WEP. Before setting the default cihper to WEP-40, need to  *
             * check that no other profile is configured to static WEP           */

            wepCipher |= TIAP_WPA_CIPHER_WEP_40;
            wepCipher |= TIAP_WPA_CIPHER_WEP_104;
            wepCipher |= TIAP_WPA_CIPHER_WEP_232;

            if(tiap_wcfg_isCipherExistsInMultiSsid(wd, wepCipher, &isCipherExists, 
                                                   &tempProfileIndex) == OK)
            { /* function succeeded */

                /* If no other profile is configured with WEP 
                   --> Set Cihper to WEP in profile_index      */
                if( (!isCipherExists) || 
                    ((isCipherExists) && (tempProfileIndex == profile_index)) )
                {
                    printf("WEP LEGACY , profile %d:setting default cipher to WEP40\n",profile_index);
                    if (tiap_wcfg_set_default_cipher(wd, TIAP_WPA_CIPHER_WEP_40,profile_index, !UPDATE_STORAGE))
                    {
                        PRINT_CONFIG_ERR_MSG("Failed to set default cipher");
                        return;
                    }
                }
                else /* There is already a profile with WEP */
                {
                    printf("There is already a profile with the Cipher configured to WEP.\nConfiguration failed!\n");
                    return;
                }
            }
            else /* function 'tiap_wcfg_isCipherExistsInMultiSsid()' failed */ 
            {
                PRINT_CONFIG_ERR_MSG("Failed to set default cipher");
                return;
            }
        }
    }

	if(tiap_wcfg_set_security_type(wd, mode, profile_index, !UPDATE_STORAGE)) 
	{
		PRINT_CONFIG_ERR_MSG("Failed to set wpa mode.\n");
		return;
	}
    if(wcfg_storage_write(&wd->storage, (const char *)wd->storage_file)) {
        PRINT_CONFIG_ERR_MSG("Failed to save settings");
        return;

    }
	printf("to take effect RESTART AP.\n");

}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wpaPassPhraseMode(ConsoleParameter_T parameters[])
{

	int profile_index = parameters[0].INTEGER;

	if(profile_index >= tiap_wcfg_get_num_of_multi_ssid(wd))
	{
		printf("Invalid profile index.\n");
		return;
	}

	if ((parameters[1].INTEGER != TIAP_WPA_PSK_PARAM_NONE) && 
		(parameters[1].INTEGER != TIAP_WPA_PSK_PARAM_HEX) && 
		(parameters[1].INTEGER != TIAP_WPA_PSK_PARAM_ASCII))
	{
		printf("Invalid pass phrase mode\n");
		return;
	}


	/*if (parameters[1].INTEGER == TIAP_WPA_PSK_PARAM_HEX)
	{
		PRINT_NOT_SUPPORTED_MSG("Passphrase in Hex mode");
		return;
	}*/
	if (tiap_wcfg_set_psk_passphrase_mode(wd, parameters[1].INTEGER, profile_index, UPDATE_STORAGE))
		PRINT_CONFIG_ERR_MSG("Failed to set psk passphrase mode");

	printf("to take effect RESTART AP.\n");

}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wpaPskHex(ConsoleParameter_T parameters[])
{
    __u8 pwd[32 + 1];
    __u8 len = 32;
    int profile_index = parameters[0].INTEGER;
    
    if(profile_index >= tiap_wcfg_get_num_of_multi_ssid(wd))
    {
        printf("Invalid profile index.\n");
        return;
    }
    
    /* Get password from local storage and terminate it with NULL */
	tiap_wcfg_get_psk_passwd(wd, pwd, &len,profile_index);
    pwd[len] = '\0'; 
    printf("profile %d\nPassword <%s>\nNew Password: ",profile_index,pwd);
    /* Input new password*/
    fgets(pwd, TIAP_WPA_MAX_PSK_PWD_LEN+1, stdin);
    len = min(strlen(pwd),32);
    /* eat the \n char */
    if (pwd[len-1] == '\n')
    {
        pwd[len-1] = '\0';
        len -= 1;
    }
    if(len == 32)
    {
        /* Legal string 32 printable characters (64 hex characters) */
        pwd[len] = 0;
        printf("setting Password <%s> in hex format\n",pwd);
        if (tiap_wcfg_set_psk_passwd(wd, pwd, len,profile_index, UPDATE_STORAGE))
        {
            PRINT_CONFIG_ERR_MSG("Failed to set psk passphrase");
            return;
        }
        puts("to take effect RESTART AP.");
    }
    else 
        if( len > 0)
        {
            /* Input is too short or too long, reject it */
            printf("Password length must be exactly 64 hex characters. Rejected.\n");
        }
        /* Empty string, do nothing */
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wpaPskPwd(ConsoleParameter_T parameters[])
{
 
	 __u8 pwd[TIAP_WPA_MAX_PSK_PWD_LEN + 1];
	 __u8 len = TIAP_WPA_MAX_PSK_PWD_LEN;

	 int profile_index = parameters[0].INTEGER;

       if(profile_index >= tiap_wcfg_get_num_of_multi_ssid(wd))
	   {
		  printf("Invalid profile index.\n");
		  return;
	   }

	   /* Get password from local storage and terminate it with NULL */
	   tiap_wcfg_get_psk_passwd(wd, pwd, &len,profile_index);
	   pwd[len] = '\0'; 
	   printf("profile %d\nPassword <%s>\nNew Password: ",profile_index,pwd);
	   /* Input new password*/
	   fgets(pwd, TIAP_WPA_MAX_PSK_PWD_LEN+1, stdin);
	   len = min(strlen(pwd),TIAP_WPA_MAX_PSK_PWD_LEN);
	   /* eat the \n char */
	   if (pwd[len-1] == '\n')
	   {
		   pwd[len-1] = '\0';
		   len -= 1;
	   }
	   if( (len >= TIAP_WPA_MIN_PSK_PWD_LEN) && (len <= TIAP_WPA_MAX_PSK_PWD_LEN))
	   {
		  /* Legal string 8-63 printable characters */
		  pwd[len] = 0;
		  printf("setting Password <%s>\n",pwd);
		  if (tiap_wcfg_set_psk_passwd(wd, pwd, len,profile_index, UPDATE_STORAGE))
		  {
			  PRINT_CONFIG_ERR_MSG("Failed to set psk passphrase");
			  return;
		  }
		  puts("to take effect RESTART AP.");
	   }
	   else if( len > 0)
	   {
		  /* Input is too short or too long, reject it */
		  printf("According to WPA RFC2898 password length is [%u..%u]. Rejected.\n",
				 TIAP_WPA_MIN_PSK_PWD_LEN, TIAP_WPA_MAX_PSK_PWD_LEN);
	   }
   /* Empty string, do nothing */
}/* END wpaPskPwd() */

/*!	\brief 
  	\param 

  	\return 
  
*/
static char *printLogFormatBuf(u8 *buf, int len)
{
    static char out[1024];
    char *pos = out;
    int i;

    for (i = 0; i < len; i++)
    {
	sprintf(pos, "%02X ", buf[i]);
	pos += 3;

	/* Insert a new line every 8 bytes except in case we just printed the 
	 * last byte
	 */
	if (!((i + 1) % 8) && i < (len - 2))
	{
	    sprintf(pos, "\n");
	    pos++;
	}
    }
    
    return out;
}

 /*!	\brief 
 \param 
 
  	\return 
    
*/
void wpaPrintPSKinHexFormat(ConsoleParameter_T parameters[])
{
    char pskStr[32];
    int profile_index = parameters[0].INTEGER;

    if(!wpaGetPSKinHexFormat(profile_index, pskStr))
    {
        printf("PSK in Hex format:\n%s\n", printLogFormatBuf(pskStr, 32));
}
}

 /*!	\brief 
 \param 
 
  	\return 
    
*/
int wpaGetPSKinHexFormat(int profile_index, char *psk)
{
    int passphrasemode;
    
    if(profile_index >= tiap_wcfg_get_num_of_multi_ssid(wd))
    {
        printf("Invalid profile index.\n");
        return 1;
    }

    tiap_wcfg_get_psk_passphrase_mode(wd, &passphrasemode,profile_index);
    if(passphrasemode == TIAP_WPA_PSK_PARAM_NONE)
    {
        printf("PSK is disabled.\n");
        return 1;
    }

    if (tiap_wcfg_wpaGetPSKinHexFormat(profile_index, psk))
    {
        printf("Failed to get PSK.\n");
        return 1;
    }

    return 0;
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wpaDefaultCipher(ConsoleParameter_T parameters[])
{
	int     profile_index = parameters[0].INTEGER;
    int     tempProfileIndex;
    __u32   wepCipher = 0;
    BOOL    isCipherExists;
    tiap_param_security_type_t  sec_type;

	if(profile_index > tiap_wcfg_get_num_of_multi_ssid(wd))
	{
		printf("Invalid profile index.\n");
		return;
	}
    
    /* In Multi-SSID: Only one pofile can be configured to work with     *
     * static WEP. Before setting the default cihper to WEP-40, need to  *
     * check that no other profile is configured to static WEP           */    
    wepCipher |= TIAP_WPA_CIPHER_WEP_40;
    wepCipher |= TIAP_WPA_CIPHER_WEP_104;
    wepCipher |= TIAP_WPA_CIPHER_WEP_232;
    
    if(tiap_wcfg_isCipherExistsInMultiSsid(wd, wepCipher, &isCipherExists, 
                                           &tempProfileIndex) != OK)
    { /* Function Failed */
        PRINT_CONFIG_ERR_MSG("Failed to set default cipher");
        return;
    }

    if(tiap_wcfg_get_security_type(wd, &sec_type , profile_index) != OK)
    {
        /* Function Failed */
        PRINT_CONFIG_ERR_MSG("Failed to set default cipher");
        return;
    }

	switch(parameters[1].INTEGER)
	{
	case 0: /* Mone */

        /* Can't configure cipher to NONE if security type is EAP-TLS */
        if(sec_type != TIAP_PARAM_SECURITY_802_1X)
        {
            if (tiap_wcfg_set_default_cipher(wd, TIAP_WPA_CIPHER_NON,profile_index, UPDATE_STORAGE))
            {
                PRINT_CONFIG_ERR_MSG("Failed to set default cipher");
                return;
            }
        }
        else
        {
            printf("Can't configure EAP-TLS with No encryption.\nConfiguration failed!\n");
            return;
        }

		break;
	case 1:/* WEP 40 */

        /* If no other profile is configured with static WEP 
           --> Set Cihper to WEP 40 in profile_index         */
        if( (!isCipherExists) || 
            ((isCipherExists) && (tempProfileIndex == profile_index)) ||
            ((isCipherExists) && (sec_type == TIAP_PARAM_SECURITY_802_1X)) )
        {
            if (tiap_wcfg_set_default_cipher(wd, TIAP_WPA_CIPHER_WEP_40,profile_index, UPDATE_STORAGE))
            {
                PRINT_CONFIG_ERR_MSG("Failed to set default cipher");
                return;
            }
        }
        else /* There is already another profile with WEP */
        {
            printf("There is already a profile with the Cipher configured to static WEP.\nConfiguration failed!\n");
            return;
        }
		break;
        
	case 2:/* WEP 128 */

        /* If no other profile is configured with static WEP 
           --> Set Cihper to WEP 40 in profile_index         */
        if( (!isCipherExists) || 
            ((isCipherExists) && (tempProfileIndex == profile_index)) ||
            ((isCipherExists) && (sec_type == TIAP_PARAM_SECURITY_802_1X)) )
        {
            if (tiap_wcfg_set_default_cipher(wd, TIAP_WPA_CIPHER_WEP_104,profile_index, UPDATE_STORAGE))
            {
                PRINT_CONFIG_ERR_MSG("Failed to set default cipher");
                return;
            }
        }
        else /* There is already another profile with WEP */
        {
            printf("There is already a profile with the Cipher configured to static WEP.\nConfiguration failed!\n");
            return;
        }
        break;
	
	case 3:/* WEP 256 */

        /* If no other profile is configured with static WEP 
           --> Set Cihper to WEP 40 in profile_index         */
        if( (!isCipherExists) || 
            ((isCipherExists) && (tempProfileIndex == profile_index)) ||
            ((isCipherExists) && (sec_type == TIAP_PARAM_SECURITY_802_1X)) )
        {
            if (tiap_wcfg_set_default_cipher(wd, TIAP_WPA_CIPHER_WEP_232,profile_index, UPDATE_STORAGE))
            {
                PRINT_CONFIG_ERR_MSG("Failed to set default cipher");
                return;
            }
        }
        else /* There is already another profile with WEP */
        {
            printf("There is already a profile with the Cipher configured to static WEP.\nConfiguration failed!\n");
            return;
        }
        break;

	default:
		printf("Invalid default cipher\n");
		
	}
	
	printf("to take effect RESTART AP.\n");
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wpaGTKInterval(ConsoleParameter_T parameters[])
{
	int profile_index = parameters[0].INTEGER;

	if(profile_index > tiap_wcfg_get_num_of_multi_ssid(wd))
	{
		printf("Invalid profile index.\n");
		return;
	}

	if ((parameters[1].INTEGER > 0) && (parameters[1].INTEGER < TIAP_WPA_MIN_GTK_PERIOD))
		parameters[1].INTEGER = TIAP_WPA_MIN_GTK_PERIOD;

	if (tiap_wcfg_set_gtk_interval(wd, parameters[1].INTEGER * 1000,profile_index, UPDATE_STORAGE))
	{
		PRINT_CONFIG_ERR_MSG("Failed to set gtk interval");
		return;
	}
}


/*!	\brief 
  	\param 

  	\return 
  
*/
void wpaPreAuthflag(ConsoleParameter_T parameters[])
{
	int profile_index = parameters[0].INTEGER;
    BOOL isPreAuthOn; 
    int temp_profile_index;
    tiap_param_security_type_t  sec_type;

	if(profile_index > tiap_wcfg_get_num_of_multi_ssid(wd))
	{
		printf("Invalid profile index.\n");
		return;
	}

	if ((parameters[1].INTEGER != 0) && (parameters[1].INTEGER != 1))
	{
		PRINT_CONFIG_ERR_MSG("error pre auth flag value (use:0->off ,1->on)");
		return;
	}

    if(parameters[1].INTEGER == 1)
    {
        if(!tiap_wcfg_get_security_type(wd, &sec_type ,profile_index))
        {
            /* Seeting Pre-Auth ON is allowed only in WPA2 mode or mixed mode */
            if( !(sec_type & TIAP_PARAM_SECURITY_WPA2) )
            {
                printf("Pre-Authentication is supported only in WPA2 mode.\nConfiguration failed!\n");
                return;
            }
            else /* mode is WPA2 or Mixed */
            {
                /* In Multi-SSID: Only one profile can be configured with Pre-Auth ON */
                if(tiap_wcfg_isPreAuthOnInMultiSsid(wd, &isPreAuthOn, &temp_profile_index) 
                    == OK)/* if function succeeded */
                {
                    /* If no other profile is configured with Pre-Auth --> 
                       Set Pre-Auth ON to profile_index */
                    if( (!isPreAuthOn) || 
                        ((isPreAuthOn) && (temp_profile_index == profile_index)) )
                    {
                        if (tiap_wcfg_set_pre_auth_flag(wd, parameters[1].INTEGER,profile_index, UPDATE_STORAGE))
                        {
                            PRINT_CONFIG_ERR_MSG("Failed to set pre authentication flag");
                            return;
                        }
                    }
                    else /* Pre-Auth is already ON in another profile */
                    {
                        printf("There is already a profile with Pre-Authentication Enabled.\nConfiguration failed!\n");
                        return;
                        
                    }
                }
                else /* The function 'tiap_wcfg_isPreAuthOnInMultiSsid()' failed */
                {
                    PRINT_CONFIG_ERR_MSG("Failed to set pre authentication flag");
                    return;
                }
            }
        }
        else /* Failed to get security type */
        {
            printf("Failed to get security type.\nConfiguration failed!\n");
            return;

        }
    }
    else /* parameters[1].INTEGER = 0 */
    {
        if (tiap_wcfg_set_pre_auth_flag(wd, parameters[1].INTEGER,profile_index, UPDATE_STORAGE))
        {
            PRINT_CONFIG_ERR_MSG("Failed to set pre authentication flag");
            return;
        }
    }
    printf("to take effect RESTART AP.\n");
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wpaRadiusSecret(ConsoleParameter_T parameters[])
{
	__s8 secret[TIAP_WPA_MAX_RADIUS_SECRET_LEN + 1];
	__u8 len = TIAP_WPA_MAX_RADIUS_SECRET_LEN;

	int profile_index = parameters[0].INTEGER;

	if(profile_index > tiap_wcfg_get_num_of_multi_ssid(wd))
	{
		printf("Invalid profile index.\n");
		return;
	}

	tiap_wcfg_get_radius_secret(wd, secret, &len,profile_index);
		
	if( len > TIAP_WPA_MAX_RADIUS_SECRET_LEN ) {
		PRINT_CONFIG_ERR_MSG("Invalid len.");
		return;
	}
	secret[len] = 0;

	printf("profie %d\nSecret <%s>\nNew Secret: ",profile_index, secret);
	fgets(secret, TIAP_WPA_MAX_RADIUS_SECRET_LEN+1, stdin);
	len = min(strlen(secret), TIAP_WPA_MAX_RADIUS_SECRET_LEN);
	if (secret[len-1] == '\n')
	{
		secret[len-1] = '\0';
		len -= 1;
	}

	if(len > 0)
	{
		secret[len] = 0;
		printf("setting Radius Secret <%s> for profile %d\n", secret,profile_index);
		if( tiap_wcfg_set_radius_secret(wd, secret, len,profile_index, UPDATE_STORAGE) ) {
			PRINT_CONFIG_ERR_MSG("Failed to set radius secret.");
			return;
		}
		puts("to take effect RESTART AP.");
	}
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wpaRadiusPort(ConsoleParameter_T parameters[])
{
/*
	wdrv_dbSetRadiusPort(parameters[0].INTEGER);
	printf("to take effect RESTART AP.\n");
*/

	int profile_index = parameters[0].INTEGER;

	if(profile_index > tiap_wcfg_get_num_of_multi_ssid(wd))
	{
		printf("Invalid profile index.\n");
		return;
	}

	
	if(tiap_wcfg_set_radius_port(wd, (__u16)parameters[1].INTEGER,profile_index, UPDATE_STORAGE)) 
	{
		PRINT_CONFIG_ERR_MSG("Failed to set radius port.");
		return;
	}

	
	printf("to take effect RESTART AP.\n");
}



/*!	\brief 
  	\param 

  	\return 
  
*/
void wpaRadiusAddr(ConsoleParameter_T parameters[])
{

	char str_addr[20];
	struct in_addr in_addr;
	int profile_index = parameters[0].INTEGER;

	if(profile_index > tiap_wcfg_get_num_of_multi_ssid(wd))
	{
		printf("Invalid profile index.\n");
		return;
	}



	if ((parameters[1].INTEGER > 255) || (parameters[2].INTEGER > 255) ||
	   (parameters[3].INTEGER > 255) || (parameters[4].INTEGER > 255)) {
		printf("Invalid IP address\n");
		return;
	}

	sprintf(str_addr,"%u.%u.%u.%u",(UINT8)parameters[1].INTEGER,(UINT8)parameters[2].INTEGER,
					(UINT8)parameters[3].INTEGER, (UINT8)parameters[4].INTEGER);

	if (tiap_inet_aton(str_addr, &in_addr) != OK)
	{
		perror("inet_aton");
		return;
	}

	if( tiap_wcfg_set_radius_ipaddr(wd, in_addr.s_addr,profile_index, UPDATE_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to set radius IP address.");
		return;
	}
	printf("to take effect RESTART AP.\n");

}

/*!	\brief 
  	\param 

  	\return 
  
*/
void whalMenu_DM(ConsoleParameter_T parameters[])
{
#ifndef VXWORKS
	int fd, addr, size, offset, i, step;
	char * mem = NULL;

	fd = open("/dev/kmem", O_RDONLY);
	if (fd < 0) {
		perror("Can't open /dev/kmem");
		return;
	}

	sscanf(parameters[0].STRING, "0x%x", &addr);
	offset  = parameters[1].INTEGER;
	size = parameters[2].INTEGER;
	step = parameters[3].INTEGER;

	if ( (step <= 0) && (step>16) ) {
		printf("Wrong argument: step can be only 1 for bytes or 2 for words\n");
		return;
	}
	printf("Base addess : %#x, Offset : %#x, Size : %#x, Format: %s\n", addr, offset, size,
		    ( step == 1 ? "bytes" : "words") );

	mem = mmap(NULL, size+offset, PROT_READ, MAP_SHARED, fd, addr - 0xA0000000);
	if (mem == MAP_FAILED) {
		perror("mmap() failed!");
		return;
	}

	 for (i=0; i<size; i+=step)
     {
		if (! (i % 16))
			printf("\n%#x : ", addr+offset+i);

		if (step == 1)      /* print byte */
			printf("%02hhx ", mem[offset+i]);
		else if (step == 2) /* print word */
			printf("%02hhx%02hhx ", mem[offset+i+1], mem[offset+i]);
        else                /* print long */
            printf("%02hhx%02hhx%02hhx%02hhx ", mem[offset+i+3], mem[offset+i+2],mem[offset+i+1],mem[offset+i] );

    }
	puts("");
	close(fd);
#endif

}

/*!	\brief 
  	\param 

  	\return 
  
*/
void whalMenu_MM(ConsoleParameter_T parameters[])
{
#ifndef VXWORKS
	int fd, addr, new_val, offset, width, count, i;
	char * mem = NULL;

	fd = open("/dev/mem", O_RDWR);
	if (fd < 0) {
		perror("Can't open /dev/mem");
		return;
	}

	sscanf(parameters[0].STRING, "0x%x", &addr);
	offset  = parameters[1].INTEGER;
	width = parameters[2].INTEGER;
	new_val = parameters[3].INTEGER;
	count = parameters[4].INTEGER;
	
	if ( (width != 1) && (width != 2) && (width != 4)) {
		puts("Wrong width value");
		return;
	}
 
	mem = mmap(NULL, 4+offset, PROT_READ|PROT_WRITE, MAP_SHARED, fd, addr - 0xA0000000);
	if (mem == MAP_FAILED) {
		perror("mmap() failed!");
		return;
	}

	printf("Base addess : %#x, Offset : %#x, New value : %#x\n", addr, offset, new_val);

	for (i=0; i<count;) {
		switch (width) {
		case 1:
			*((__u8 *)&mem[offset+i]) = (__u8)new_val;
			++i;
			break;
		case 2:
			*((__u16 *)&mem[offset+i]) = (__u16)new_val;
			i+=2;
			break;
		case 4:
			*((__u32 *)&mem[offset+i]) = (__u32)new_val;
			i+=4;
			break;
		default:
			return;
		}
	}

	puts("");
	close(fd);
#endif

}

/*!	\brief 
  	\param 

  	\return 
  
*/
void whalMenu_DebugServerIP(ConsoleParameter_T parameters[]) 
{
	char str_addr[20];
	struct in_addr in_addr;

	if ((parameters[0].INTEGER > 255) || (parameters[1].INTEGER > 255) ||
	   (parameters[2].INTEGER > 255) || (parameters[3].INTEGER > 255)) {
		printf("Invalid IP address\n");
		return;
	}

	sprintf(str_addr,"%u.%u.%u.%u",(UINT8)parameters[0].INTEGER,(UINT8)parameters[1].INTEGER,
					(UINT8)parameters[2].INTEGER, (UINT8)parameters[3].INTEGER);

	if (tiap_inet_aton(str_addr, &in_addr) != OK) {
		perror("inet_aton");
		return;
	}

	if (tiap_wcfg_debug_set_server_ip(wd, in_addr.s_addr) ) {
		PRINT_CONFIG_ERR_MSG("Failed to set FW Trace host IP address.");
		return;
	}
	printf("to take effect RESTART AP.\n");
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void whalMenu_DownloadFW(ConsoleParameter_T parameters[]) 
{
	//	char str_addr[20];
	struct in_addr addr;
	/*
	if ((parameters[0].INTEGER > 255) || (parameters[1].INTEGER > 255) ||
			(parameters[2].INTEGER > 255) || (parameters[3].INTEGER > 255)) {
		printf("Invalid IP address\n");
		return;
	}

	sprintf(str_addr,"%hhu.%hhu.%hhu.%hhu",parameters[0].SHORT,parameters[1].SHORT,
					parameters[2].SHORT, parameters[3].SHORT);
	*/
	addr.s_addr = wd->storage.debug.debug_server_ip_addr;

	
	if (tftp_get_file("FW.bin", inet_ntoa(addr), WCFG_TMP_DIR))
		return;

	snprintf(wd->fw_filename, sizeof(wd->fw_filename), "%s/FW.bin", WCFG_TMP_DIR);

	if ( !(wd = tiap_wcfg_restart(wd, &driver_load_failed)))
		puts("Restart with new FW failed !");

#ifndef VXWORKS	
	wd->fw_filename[0] = 0;
#endif
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wpaDefaultGateway(ConsoleParameter_T parameters[])
{/*
	if ((parameters[0].INTEGER > 255) ||
		(parameters[1].INTEGER > 255) ||
		(parameters[2].INTEGER > 255) ||
		(parameters[3].INTEGER > 255))
	{
		printf("Invalid Default Gateway \n");
		return;
	}
	
	wdrv_dbSetDefaultGateway(parameters[0].INTEGER, parameters[1].INTEGER, parameters[2].INTEGER, parameters[3].INTEGER);
	printf("to take effect RESTART AP.\n");
*/

	__u32 i, gateway;
	__u8 *p = (__u8 *)&gateway;

	if ((parameters[0].INTEGER > 255) || (parameters[1].INTEGER > 255) ||
	    (parameters[2].INTEGER > 255) || (parameters[3].INTEGER > 255)) {
		printf("Invalid Default Gateway \n");
		return;
	}

	for(i=0; i<4; i++)
		p[i] = (__u8)parameters[i].INTEGER;

	if( tiap_wcfg_set_route(wd->dev_name, 0, 0, gateway, SIOCADDRT) ) {
		PRINT_CONFIG_ERR_MSG("Failed to set default route.");
		return;	
	}
}

/************************************************************************/
/*						WPA-test sub-menu functions							*/
/************************************************************************/
/*!	\brief 
  	\param 

  	\return 
  
*/
void assocStation(ConsoleParameter_T parameters[])
{/*
	if ((parameters[1].INTEGER == TRUE) && (parameters[2].INTEGER > DOT11_RSNE_MAX)) 
		printf("RSN IE len is too big\n");
	if ((parameters[1].INTEGER == TRUE) && (parameters[2].INTEGER < 3)) 
		printf("RSN IE len is too small\n");
	else
		wdrv_wpaTstAssocStation(parameters[0].INTEGER, parameters[1].INTEGER, parameters[2].INTEGER, parameters[3].INTEGER);
*/
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void reAssocStation(ConsoleParameter_T parameters[])
{/*
	if ((parameters[1].INTEGER == TRUE) && (parameters[2].INTEGER > DOT11_RSNE_MAX)) 
		printf("RSN IE len is too big\n");
	if ((parameters[1].INTEGER == TRUE) && (parameters[2].INTEGER < 3)) 
		printf("RSN IE len is too small\n");
	else
		wdrv_wpaTstReAssocStation(parameters[0].INTEGER, parameters[1].INTEGER, parameters[2].INTEGER, parameters[3].INTEGER);
*/
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void stationsRsnIe(ConsoleParameter_T parameters[])
{
	//wdrv_wpaPrintStationRsnIe();
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void macAuthorize(ConsoleParameter_T parameters[])
{/*
	char bcastMac[MAC_ADDR_LEN];

	memset(bcastMac, 0xFF, MAC_ADDR_LEN);

	if (memcmp(bcastMac, parameters[0].MAC_ADDR, MAC_ADDR_LEN) == 0)
		setBcastState(TRUE);
	else
		wdrv_mgmtMacAuthorize(parameters[0].MAC_ADDR);
*/
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void macDeAuthorize(ConsoleParameter_T parameters[])
{/*
	char bcastMac[MAC_ADDR_LEN];

	memset(bcastMac, 0xFF, MAC_ADDR_LEN);

	if (memcmp(bcastMac, parameters[0].MAC_ADDR, MAC_ADDR_LEN) == 0)
		setBcastState(FALSE);
	else
		wdrv_mgmtMacDeAuthorize(parameters[0].MAC_ADDR);
*/
}


/*!	\brief 
  	\param 

  	\return 
  
*/
void showWpaConfig(ConsoleParameter_T parameters[])
{/*
	char mac[MAC_ADDR_LEN];
	UINT8 suite;
	

	if (wdrv_mgmtIsBroadcastAuthorized())
		printf("Broadcast MAC:	Authorized\n");
	else
		printf("Broadcast MAC:	Not Authorized\n");

	wdrv_dbGetBroadcastSuiteType(&suite);

	if (suite == CIPHER_NONE)
		printf("Broadcast cipher:	None\n");
	else if (suite == CIPHER_WEP)
		printf("Broadcast cipher:	WEP\n");
	else if (suite == CIPHER_TKIP)
		printf("Broadcast cipher:	TKIP\n");
	else
		printf("Broadcast cipher:	Invalid %d\n", suite);

	if (wdrv_wpaGetMicFailureReportInRx() == TRUE)
		printf("MIC Failure Report In the Rx path:	TRUE\n");
	else
		printf("MIC Failure Report In the Rx path:	FALSE\n");

	if (wdrv_wpaGetMicFailureReportFromStation() == TRUE)
		printf("MIC Failure Report from station:	TRUE\n");
	else
		printf("MIC Failure Report from station:	FALSE\n");
*/
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void isMacAuthorized(ConsoleParameter_T parameters[])
{/*
	UINT16 staIndex;

    if( wdrv_dbFindStaIndex(parameters[0].MAC_ADDR, &staIndex) != OK )
	{
		printf("Sattion has no index\n");
		return;
	}

	if (wdrv_mgmtIsStationAuthorized(staIndex) == TRUE)
		printf("			Yes\n");
	else
		printf("			No\n");*/
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void setStationSuite(ConsoleParameter_T parameters[])
{/*
	UINT16 staIndex;

	if ((parameters[1].INTEGER != CIPHER_NONE) && (parameters[1].INTEGER != CIPHER_WEP) && (parameters[1].INTEGER != CIPHER_TKIP))
	{
		printf("Invalid suite\n");
		return;
	}

	wdrv_dbFindStaIndex(parameters[0].MAC_ADDR, &staIndex);

	wdrv_dbSetStationSuiteType(staIndex, parameters[1].INTEGER);*/
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void setBcastSuite(ConsoleParameter_T parameters[])
{/*
	if ((parameters[0].INTEGER != CIPHER_NONE) && (parameters[0].INTEGER != CIPHER_WEP) && (parameters[0].INTEGER != CIPHER_TKIP))
	{
		printf("Invalid suite\n");
		return;
	}

	wdrv_dbSetBroadcastSuiteType(parameters[0].INTEGER);*/
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void testVectors(ConsoleParameter_T parameters[])
{
	//wdrv_wpaTstVector();
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void setRxMicKey(ConsoleParameter_T parameters[])
{/*
	UINT16	index;
	int status, i;
	UINT8	micKey[8];

	status = wdrv_dbFindStaIndex(parameters[0].MAC_ADDR, &index);

	if (status != OK)
	{
		printf("Station not found \n");
		return;
	}
	
	for (i = 0; i < 8; i++)
		micKey[i] = parameters[i + 1].INTEGER;

	wdrv_dbSetStationRxMicKey(index, micKey);*/
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void setTxMicKey(ConsoleParameter_T parameters[])
{/*
	UINT16	index;
	int status, i;
	UINT8	micKey[8];

	status = wdrv_dbFindStaIndex(parameters[0].MAC_ADDR, &index);

	if (status != OK)
	{
		printf("Station not found \n");
		return;
	}
	
	for (i = 0; i < 8; i++)
		micKey[i] = parameters[i + 1].INTEGER;

	wdrv_dbSetStationTxMicKey(index, micKey);*/
}

void setMicReportInRx(ConsoleParameter_T parameters[])
{
	//wdrv_wpaSetMicFailureReportInRx(parameters[0].INTEGER);
}

void setMicReportFromStation(ConsoleParameter_T parameters[])
{
	//wdrv_wpaSetMicFailureReportFromStation(parameters[0].INTEGER);
}

void generateTxMicFailure(ConsoleParameter_T parameters[]){
	//wdrv_dbGenerateTxMicFailure(parameters[0].MAC_ADDR);
}



/*!	\brief 
  	\param 

  	\return 
  
*/
void privacySetPrivacyWEPDefaultKeyId(ConsoleParameter_T parameters[]){
	/*
	if ((parameters[0].INTEGER < MIN_KEY_ID) || (parameters[0].INTEGER > MAX_KEY_ID))
	{
		printf("Invalid default key id: %d\n", parameters[0].INTEGER);
		return;
	}
	wdrv_dbSetPrivacyWEPDefaultKeyId((UINT8)parameters[0].INTEGER);*/
	int profile_index = parameters[0].INTEGER;

	if(profile_index >= tiap_wcfg_get_num_of_multi_ssid(wd))
	{
		printf("Invalid profile index.\n");
		return;
	}

	if ((parameters[1].INTEGER < TIAP_PARAM_MIN_KEY_ID) || (parameters[1].INTEGER > TIAP_PARAM_MAX_KEY_ID))
	{
		printf("Invalid default key id: %ld\n", parameters[1].INTEGER);
		return;
	}
	if (tiap_wcfg_set_wep_default_key_id(wd, (UINT8)parameters[1].INTEGER,profile_index, UPDATE_STORAGE))
		PRINT_CONFIG_ERR_MSG("failed to set wep default key id");
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void privacySetPrivacyExclude(ConsoleParameter_T parameters[]){
	/*
	if ((parameters[0].INTEGER != 0) && (parameters[0].INTEGER != 1))
	{
		printf("Invalid exclude unencrypted mode: %d\n", parameters[0].INTEGER);
		return;
	}
	wdrv_dbSetPrivacyExcludeUnencrypted((BOOL)parameters[0].INTEGER);*/

	proc_params proc_pars;
	int ret; 

	DRIVER_LOADED_ASSERT;

	if ((parameters[0].INTEGER != 0) && (parameters[0].INTEGER != 1))
	{
		printf("Invalid exclude unencrypted mode: %ld\n", parameters[0].INTEGER);
		return;
	}
	

	printf("Warning: this parameter is not used by driver and is not saved in non-volatile storage\n");

	proc_pars.s.magic = PROC_PARAMS_MAGIC;
	proc_pars.s.Param1 = parameters[0].INTEGER;

	ret = dda_write_proc_entry(WLAN_PROC_PRIVACY_DIR_NAME, WLAN_PROC_PRIVACY_EXCLUDE, 
		(char *)&proc_pars, sizeof(proc_pars));

	if (ret != sizeof(proc_pars))
		PRINT_PROC_WRITE_ERR(WLAN_PROC_PRIVACY_DIR_NAME, WLAN_PROC_PRIVACY_EXCLUDE);
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void privacySetDefaultKey5(ConsoleParameter_T parameters[])
{/*
	char key[5];
	int i;
	UINT32	value = 0;
	
	wdrv_dbGetWpaMode(&value);

	// Configuring the default Key ID & default Keys in a working mode that does not include legacy stations 
	//	is forbidden 
	if (!(value & DOT11_WPA_STA_TYPE_WEP_LEGACY))
	{
		printf("Can not configure default key ID, the WPA working mode does not include legacy stations \n");
		return;
	}

	if ((parameters[0].INTEGER < MIN_KEY_ID) || (parameters[0].INTEGER > MAX_KEY_ID))
	{
		printf("Invalid default key index: %d\n", parameters[0].INTEGER);
		return;
	}
	for (i = 0; i < 5; i++)
		key[i] = (UINT8)parameters[i + 1].INTEGER;
	
	wdrv_dbSetWEPDefaultKeys(parameters[0].INTEGER, 5, key);
	*/
	
	tiap_wpa_wep_key_t wep_key;
	int i;
	int profile_index = parameters[0].INTEGER;

	if(profile_index >= tiap_wcfg_get_num_of_multi_ssid(wd))
	{
		printf("Invalid profile index.\n");
		return;
	}

	/* Configuring the default Key ID & default Keys in a working mode that does not include legacy stations 
		is forbidden */



	if ((parameters[1].INTEGER < TIAP_PARAM_MIN_KEY_ID) || (parameters[1].INTEGER > TIAP_PARAM_MAX_KEY_ID))
	{
		printf("Invalid default key index: %ld\n", parameters[1].INTEGER);
		return;
	}

	for (i = 0; i < 5; i++)
		wep_key.key[i] = parameters[i + 2].INTEGER;
	wep_key.len = 5;

	if(tiap_wcfg_set_def_wep_key(wd, parameters[1].INTEGER, &wep_key,profile_index, UPDATE_STORAGE))
		PRINT_CONFIG_ERR_MSG("failed to set wep default key 5");
		

}

/*!	\brief 
  	\param 

  	\return 
  
*/
void privacySetDefaultKey13(ConsoleParameter_T parameters[])
{/*
	char key[13];
	int i;

	UINT32	value = 0;
	wdrv_dbGetWpaMode(&value);

	// Configuring the default Key ID & default Keys in a working mode that does not include legacy stations 
	//is forbidden 
	if (!(value & DOT11_WPA_STA_TYPE_WEP_LEGACY))
	{
		printf("Can not configure default key ID, the WPA working mode does not include legacy stations \n");
		return;
	}

	if ((parameters[0].INTEGER < MIN_KEY_ID) || (parameters[0].INTEGER > MAX_KEY_ID))
	{
		printf("Invalid default key index: %d\n", parameters[0].INTEGER);
		return;
	}
	for (i = 0; i < 13; i++)
		key[i] = (UINT8)parameters[i + 1].INTEGER;
	
	wdrv_dbSetWEPDefaultKeys(parameters[0].INTEGER, 13, key);
*/
	tiap_wpa_wep_key_t wep_key;
	int i;
	int profile_index = parameters[0].INTEGER;

	if(profile_index >= tiap_wcfg_get_num_of_multi_ssid(wd))
	{
		printf("Invalid profile index.\n");
		return;
	}


	if ((parameters[1].INTEGER < TIAP_PARAM_MIN_KEY_ID) || (parameters[1].INTEGER > TIAP_PARAM_MAX_KEY_ID))
	{
		printf("Invalid default key index: %ld\n", parameters[1].INTEGER);
		return;
	}

	for (i = 0; i < 13; i++)
		wep_key.key[i] = parameters[i + 2].INTEGER;
	wep_key.len = 13;

	if(tiap_wcfg_set_def_wep_key(wd, parameters[1].INTEGER, &wep_key,profile_index, UPDATE_STORAGE))
		PRINT_CONFIG_ERR_MSG("failed to set wep default key 13");
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void privacySetDefaultKey29(ConsoleParameter_T parameters[])
{/*
	char key[29];
	int i;

	UINT32	value = 0;
	wdrv_dbGetWpaMode(&value);

	// Configuring the default Key ID & default Keys in a working mode that does not include legacy stations 
	//	is forbidden 
	if (!(value & DOT11_WPA_STA_TYPE_WEP_LEGACY))
	{
		printf("Can not configure default key ID, the WPA working mode does not include legacy stations \n");
		return;
	}

	if ((parameters[0].INTEGER < MIN_KEY_ID) || (parameters[0].INTEGER > MAX_KEY_ID))
	{
		printf("Invalid default key index: %d\n", parameters[0].INTEGER);
		return;
	}
	for (i = 0; i < 29; i++)
		key[i] = (UINT8)parameters[i + 1].INTEGER;
	
	wdrv_dbSetWEPDefaultKeys(parameters[0].INTEGER, 29, key);
	*/

	tiap_wpa_wep_key_t wep_key;
	int i;
	int profile_index = parameters[0].INTEGER;

	if(profile_index >= tiap_wcfg_get_num_of_multi_ssid(wd))
	{
		printf("Invalid profile index.\n");
		return;
	}

	if ((parameters[1].INTEGER < TIAP_PARAM_MIN_KEY_ID) || (parameters[1].INTEGER > TIAP_PARAM_MAX_KEY_ID))
	{
		printf("Invalid default key index: %ld\n", parameters[1].INTEGER);
		return;
	}

	for (i = 0; i < 29; i++)
		wep_key.key[i] = parameters[i + 2].INTEGER;
	wep_key.len = 29;

	if(tiap_wcfg_set_def_wep_key(wd, parameters[1].INTEGER, &wep_key,profile_index, UPDATE_STORAGE))
		PRINT_CONFIG_ERR_MSG("failed to set wep default key 29");

}

/*!	\brief 
  	\param 

  	\return 
  
*/
void privacySetPrivacyBitFlag(ConsoleParameter_T parameters[]){
/*
	if ((parameters[0].INTEGER != 0) && (parameters[0].INTEGER != 1))
	{
		printf("Invalid privacy bit mode: %d\n", parameters[0].INTEGER);
		return;
	}
	wdrv_dbSetPrivacyBitFlag((BOOL)parameters[0].INTEGER);*/
	PRINT_NOT_SUPPORTED_MSG("Privacy bit flag");
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void privacySetAuthAlgo(ConsoleParameter_T parameters[])
{
    int                     algo            = parameters[0].INTEGER;
    tiap_param_auth_algo_t  tbl[TIAP_PARAM_MAX_AUTH_ALG_SUPPORTED];
    tiap_param_multi_ssid_t multi_ssid;
    tiap_param_security_type_t  sec_type;
    int profile_index;
    __u32 cipher;
    

    if(tiap_wcfg_get_multi_ssid(wd, &multi_ssid, FROM_STORAGE))
    {
        printf("Failed to set Authentication Algorithm!\n");
        return;
    }

    if(multi_ssid.secondary_enabled == 1)
    {
        printf("Failed to set Authentication Algorithm!\nCan't set Auth Algo when Multiple SSIDs is Enabled.\n");
        return;
    }

    if ((algo < 0) || (algo > 2))
	{
		printf("Invalid Authentication Algorithm index: %d\n", algo);
		return;
	}

    if(tiap_wcfg_get_primary_profileIndex(wd, &profile_index, FROM_STORAGE))
    {
        printf("Failed to set Authentication Algorithm!\n");
        return;
    }
    
    if(tiap_wcfg_get_security_type(wd, &sec_type ,profile_index))
    {
        printf("Failed to set Authentication Algorithm!\n");
		return;
    }

    tiap_wcfg_get_default_cipher(wd, &cipher ,profile_index);

    if( (algo != 0) && ((sec_type != TIAP_PARAM_SECURITY_WEP) || (cipher == TIAP_WPA_CIPHER_NON)) )
    {
        printf("Failed to set Authentication Algorithm!\nCan't work with Shared Key in non-WEP profile.\n");
		return;
    }

    tbl[0].algo = TIAP_PARAM_MIB_OPEN_SYSTEM;
    tbl[1].algo = TIAP_PARAM_MIB_SHARED_KEY;
    switch (algo)
    {
        case 0:
             tbl[0].enable = TRUE;
             tbl[1].enable = FALSE;
             break;
                    
        case 1:
             tbl[0].enable = FALSE;
             tbl[1].enable = TRUE;
             break;
                    
        case 2:
             tbl[0].enable = TRUE;
             tbl[1].enable = TRUE;
             break;
                    
        default:
    }
    
    
    if(tiap_wcfg_set_auth_type(wd, tbl, UPDATE_STORAGE))
        PRINT_CONFIG_ERR_MSG("failed to set Authentication Algorithm\n");
}

/************************************************************************/
/*						Service Application sub-menu functions 					*/
/************************************************************************/


/* Util function to print rate in MBPS units */
/*!	\brief 
  	\param 

  	\return 
  
*/
void printOFDMRateInMbpsUnit(UINT8 *rate)
{
	UINT8 rates[TIAP_PARAM_OFDM_RATES_MAX] = {6,9,12,18,24,36,48,54};
	int   i;

	for(i=0; i<TIAP_PARAM_OFDM_RATES_MAX; i++)
	{
		if(rate[i] == TIAP_PARAM_RATE_SUPPORTED)
			printf("%dMbps ", rates[i]);
		else if(rate[i] == TIAP_PARAM_RATE_BASIC_SUPPORTED)
			printf("%dMbpsB ", rates[i]);
	}
}

/* Util function to print rate in MBPS units */
/*!	\brief 
  	\param 

  	\return 
  
*/
void printRateInMbpsUnit(UINT8 *rate)
{
	UINT8 rates[TIAP_PARAM_CCK_PBCC_RATES_MAX] = {1,2,5,11,22};
	int   i;

	for(i=0; i<TIAP_PARAM_CCK_PBCC_RATES_MAX; i++)
	{
		if(rates[i] != 5)
		{
			if(rate[i] == TIAP_PARAM_RATE_SUPPORTED)
				printf("%dMbps ", rates[i]);
			else if(rate[i] == TIAP_PARAM_RATE_BASIC_SUPPORTED)
				printf("%dMbpsB ", rates[i]);
		}
		else
		{
			if(rate[i] == TIAP_PARAM_RATE_SUPPORTED)
				printf("%d.5Mbps", 5);
			else if(rate[i] == TIAP_PARAM_RATE_BASIC_SUPPORTED)
				printf("%d.5MbpsB", 5);
		}
	}
}


/* util function that check if rate is included in ratesList */
/* output in str string as Y(exist), B(exist & basic) or N(not exist) */
/*!	\brief 
  	\param 

  	\return 
  
*/
static void isRateInList(UINT8 *ratesList, UINT8 ratesLen, UINT8 rate, char *str)
{
	int i;
	BOOL found=FALSE;

	for (i=0; i<ratesLen; i++)
	{
		if (rate == (ratesList[i]&0x7F))
		{
			if (ratesList[i]&0x80)
			{
				strcpy(str, "B");
			}
			else
			{
				strcpy(str, "Y");
			}
			found= TRUE;
			break;
		}
	}

	if (!found)
	{
		strcpy(str, "N");
	}

}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSetDualAp(ConsoleParameter_T parameters[])
{/*
    UINT8 RadioFrequency;
    wdrv_dbGetRadioFrequency(&RadioFrequency);
    if (parameters[0].INTEGER == 0)
    {                    
        // Reguler AP 
        wdrv_dbSetNumSupWone(1,TRUE);
        currentConfigWoneInex = 0;
        printf("\nSet Dual AP Off\n");
        printf("\nWone configuration Index set to 0\n");
    }
    else if ( !wdrv_dbIsVhcfEnabled() )  // Allow dual AP only if vHCF is disabled 
    {
        if(RadioFrequency != RADIO_BAND_2_4_AND_5GHZ)
        {
            printf("\n Error: This board support only 2.4 Ghz \n");
            return;
        }
        // Dual AP 
        wdrv_dbSetNumSupWone(2,TRUE);
        printf("\nSet Dual AP On\n");
    }
	else
        printf("\nRejected!! Can't enable Dual AP while vHCF is enabled!!\n")*/


    UINT8 RadioFrequency;
    tiap_param_vhcf_t vhcf;
    __u8 wone_nr;
    char buf[4096];
    int len;

    if( tiap_wcfg_get_vhcf(wd, &vhcf, FROM_STORAGE) ) {
	    PRINT_CONFIG_ERR_MSG("Failed to get vhcf parameters.");
	    return;
    }
       

    if( (len = dda_read_proc_entry(WLAN_PROC_SERV_DIR_NAME,
                                        WLAN_PROC_SERV_RADIO_FREQ, buf, sizeof(buf))) == -1 )
                PRINT_PROC_READ_ERR(WLAN_PROC_SERV_DIR_NAME, WLAN_PROC_SERV_RADIO_FREQ);

    sscanf(buf, "%hhu", &RadioFrequency);
    printf("radio freq: %hhu\n", RadioFrequency); 
#ifndef VXWORKS
	fflush(NULL);
#endif
    if (parameters[0].INTEGER == 0)
    {                    
        /* Regular AP */
        
		wone_nr = TIAP_PARAM_WONE_SINGLE_BAND;
		if( tiap_wcfg_set_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, UPDATE_STORAGE) ) {
			PRINT_CONFIG_ERR_MSG("Failed to set wone number.");
			return;
		}
			currentConfigWoneInex = 0;
			printf("\nSet Dual AP Off\n");
			printf("\nWone configuration Index set to 0\n");
	}
    else if ( !vhcf.enable )  /* Allow dual AP only if vHCF is disabled */
    {
        if(RadioFrequency != TIAP_PARAM_RADIO_BAND_2_4_AND_5GHZ)
        {
            printf("\n Error: This board support only 2.4 Ghz \n");
            return;
        }
        /* Dual AP */
		wone_nr = TIAP_PARAM_WONE_DUAL_BAND;
		if(tiap_wcfg_set_dual_ap(wd, wone_nr, UPDATE_STORAGE)) {
			PRINT_CONFIG_ERR_MSG("Failed to set wone number.");
			return;
	}
        printf("\nSet Dual AP On\n");
	printf("to take effect RESTART AP.\n");
    }
    else
	    printf("\nRejected!! Can't enable Dual AP while vHCF is enabled!!\n");
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSetWoneConfIndex(ConsoleParameter_T parameters[])
{/*
    UINT8  num_sup_wone;

    wdrv_dbGetNumSupWone(&num_sup_wone);

   if((parameters[0].INTEGER >= num_sup_wone)||(parameters[0].INTEGER < 0 ))
    {
        printf("\nError:  %d > num_sup_wone (%d)   \n",parameters[0].INTEGER,num_sup_wone); 
    }
    else
    {
        currentConfigWoneInex = parameters[0].INTEGER;
        printf("\n setting Current Config Wone Index To %d \n",currentConfigWoneInex); 
    } 
    */


	__u8 num_sup_wone;

	if( tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_NUM, &num_sup_wone, FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get wone number.");
		return;
	}

	if((parameters[0].INTEGER >= num_sup_wone)||(parameters[0].INTEGER < 0 ))
	{
		printf("\nError:  %ld > num_sup_wone (%hhu)   \n",parameters[0].INTEGER,num_sup_wone); 
	}
	else
	{
		currentConfigWoneInex = parameters[0].INTEGER;
		printf("\n setting Current Config Wone Index To %d \n",currentConfigWoneInex); 
	} 

}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSetUsageTime(ConsoleParameter_T parameters[])
{/*
    wdrv_dbSetUsageTime(currentConfigWoneInex, parameters[0].INTEGER);
    printf("Setting Wone %d UsageTime %d\n", currentConfigWoneInex, parameters[0].INTEGER); */

	tiap_param_wone_t wone;

	wone.index = currentConfigWoneInex;
	wone.type = wone_usage_time;
	wone.elem.usage_time = parameters[0].INTEGER;

	if( tiap_wcfg_set_wone(wd, &wone, UPDATE_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to set usage time.");
		return;
	}
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanRadioParams( ConsoleParameter_T parameters[])
{/*
   int i, selRate, selLevel, idx, val, ovd, modified=0;
   char buf[180];
   const char *sMsg1[] =
   {
      "100%", 
      "50%",
      "25%",
      "12.5%",
      "lowest"
   };
   RadioParam_t rdp = {0};

   wdrv_dbGetRadioParams( &rdp);

   // Display the table 
   puts( "\n\tRates\t:\t54-48\t36-18\t 12-6\t 11b");
   puts( "-------------------------------------------------------------------");
   for( i =0 ; i < MAX_RADIO_PARAM_POWER_LEVELS; i++) {
      int j = i * MAX_RADIO_PARAM_POWER_RATES;
      printf( "LVL %d (%s)\t:\t", i+1, sMsg1[i]);
      printf( "%4d\t%4d\t%4d\t%4d\n",
         sign_extend8(rdp.powerTable_param[j  ]),
         sign_extend8(rdp.powerTable_param[j+1]),
         sign_extend8(rdp.powerTable_param[j+2]),
         sign_extend8(rdp.powerTable_param[j+3])
         );
   }
   if( ovd = (rdp.overrideBitmask & RADIO_PARAM_OVERRIDE_POWERTABLE)) {
      puts("Radio Power table overrides HW settings");
   } else {
      puts("Radio Power table uses HW settings");
   }

   // Ask if user wants to edit this setting 
   printf( "Edit this setting [y/n] ? ");
   buf[0] = '\0';
   fgets( buf, sizeof(buf), stdin);
   if( tolower(buf[0]) == 'y' )
   {
      // ask what to edit. 
      printf( "\nSelect Rate:\n"
         "[1] 54-48\n"
         "[2] 36-18\n"
         "[3] 12-6\n"
         "[4] 11b\n"
         "Enter selection [1-%d]: ", MAX_RADIO_PARAM_POWER_RATES);
      buf[0] = '\0';
      fgets( buf, sizeof(buf), stdin);
      if( 1 != sscanf( buf, "%d", &selRate))
         return;
      if( (selRate >= 1) && (selRate <= MAX_RADIO_PARAM_POWER_RATES))
         --selRate;
      else{
         puts("Invalid selection");
         return;
      }

      puts( "\nSelect level:");
      for( i = 0; i<MAX_RADIO_PARAM_POWER_LEVELS; ++i) {
         printf( "[%d] %s\n", i+1, sMsg1[i]);
      }
      printf( "Enter selection [1-%d]: ", i);
      buf[0] = '\0';
      fgets( buf, sizeof(buf), stdin);
      if( 1 != sscanf( buf, "%d", &selLevel))
         return;
      if( (selLevel>= 1) && (selLevel<= i))
         --selLevel;
      else{
         puts("Invalid selection");
         return;
      }
   
      idx = selLevel * MAX_RADIO_PARAM_POWER_RATES + selRate;
      printf( "Enter new value [%d]: ", sign_extend8(rdp.powerTable_param[idx]));
      buf[0] = '\0';
      fgets( buf, sizeof(buf), stdin);
      if( (buf[0] == '-') || (isdigit(buf[0])))
      {
         if( 1 != sscanf( buf, "%d", &val)) {
            val = rdp.powerTable_param[idx];
         } else {
            val &= 0xff;
            rdp.powerTable_param[idx] = val;
         }
         wdrv_dbSetRadioParams( &rdp, RADIO_PARAM_POWER_TABLE, FALSE);
         modified = 1;
      }
   }

   // Ask if user wants to override HW power table 
   printf( "Override HW radio power table [y/n] <%c> ? ", ovd ? 'y':'n');
   buf[0] = '\0';
   fgets( buf, sizeof(buf), stdin);
   if( buf[0] != '\0') {
      if( tolower(buf[0]) == 'y' ) {
         if( !ovd)  modified = 1;
         ovd = 1;
         rdp.overrideBitmask |= 1;
      } else if( tolower(buf[0]) == 'n') {
         if( ovd ) modified = 1;
         ovd = 0;
         rdp.overrideBitmask &= ~1;
      };
   }
   wdrv_dbSetRadioParams( &rdp, RADIO_PARAM_OVERRIDE_BMASK, modified);

   // set the radio params according to the new configuration 
   whal_apiSetRadioParams();
   */



	   int i, selRate, selLevel, idx, val, ovd, modified=0;
   char buf[180];
   int len;
   const char *sMsg1[] =
   {
	   "100%", 
	   "50%",
	   "25%",
	   "12.5%",
	   "lowest"
   };
   tiap_param_radio_t radio = {0};
   
   if( (len = dda_read_proc_entry(WLAN_PROC_SERV_DIR_NAME, WLAN_PROC_SERV_RADIO_PARAMS,
				   (char *)&radio, sizeof(radio))) == -1) {
#ifndef VXWORKS
	   printf("%s: Failed to read %s %s\n", __func__,
			   WLAN_PROC_SERV_DIR_NAME, WLAN_PROC_SERV_RADIO_PARAMS);
#else
	   printf("%s: Failed to read %d %d\n", __func__,
			   WLAN_PROC_SERV_DIR_NAME, WLAN_PROC_SERV_RADIO_PARAMS);
#endif
	   return;
   }

   // Display the table 
   puts( "\n\tRates\t:\t54-48\t36-18\t 12-6\t 11b");
   puts( "-------------------------------------------------------------------");
   for( i =0 ; i < TIAP_PARAM_MAX_RADIO_PARAM_POW_LEVELS; i++) {
      int j = i * TIAP_PARAM_MAX_RADIO_PARAM_POW_RATES;
      printf( "LVL %d (%s)\t:\t", i+1, sMsg1[i]);
      printf( "%4d\t%4d\t%4d\t%4d\n",
         radio.tbl.param[j  ],
         radio.tbl.param[j+1],
         radio.tbl.param[j+2],
         radio.tbl.param[j+3]
         );
   }
   if( (ovd = (radio.mask & RADIO_PARAM_OVERRIDE_POWERTABLE))) {
      puts("The above Radio Power table is used - overrides HW settings");
   } else {
      puts("HW settings are used for the Radio Power params, and not the above table");
   }

   

   // Ask if user wants to edit this setting 
   printf( "Edit this setting [y/n] ? ");
   buf[0] = '\0';
   fgets( buf, sizeof(buf), stdin);
   if( tolower(buf[0]) == 'y' )
   {
	   // ask what to edit. 
	   printf( "\nSelect Rate:\n"
			   "[1] 54-48\n"
			   "[2] 36-18\n"
			   "[3] 12-6\n"
			   "[4] 11b\n"
			   "Enter selection [1-%d]: ", TIAP_PARAM_MAX_RADIO_PARAM_POW_RATES);
	   buf[0] = '\0';
	   fgets( buf, sizeof(buf), stdin);
	   if( 1 != sscanf( buf, "%d", &selRate))
		   return;
	   if( (selRate >= 1) && (selRate <= TIAP_PARAM_MAX_RADIO_PARAM_POW_RATES))
		   --selRate;
	   else{
		   puts("Invalid selection");
		   return;
	   }

	   puts( "\nSelect level:");
	   for( i = 0; i<TIAP_PARAM_MAX_RADIO_PARAM_POW_LEVELS; ++i) {
		   printf( "[%d] %s\n", i+1, sMsg1[i]);
	   }
	   printf( "Enter selection [1-%d]: ", i);
	   buf[0] = '\0';
	   fgets( buf, sizeof(buf), stdin);
	   if( 1 != sscanf( buf, "%d", &selLevel))
		   return;
	   if( (selLevel>= 1) && (selLevel<= i))
		   --selLevel;
	   else{
		   puts("Invalid selection");
		   return;
	   }

	   idx = selLevel * TIAP_PARAM_MAX_RADIO_PARAM_POW_RATES + selRate;
	   printf( "Enter new value [%hhd]: ", radio.tbl.param[idx]);
	   buf[0] = '\0';
	   fgets( buf, sizeof(buf), stdin);
	   if( (buf[0] == '-') || (isdigit(buf[0])))
	   {
		   if( 1 != sscanf( buf, "%d", &val)) {
			   val = radio.tbl.param[idx];
		   } else {
			   val &= 0xff;
			   radio.tbl.param[idx] = val;
		   }
		   radio.mask &= ~1;
		   if( tiap_wcfg_set_cfg_pow_tbl(wd, &radio, !UPDATE_STORAGE) ) {
			   PRINT_CONFIG_ERR_MSG("Failed to set radio power table.");
			   return;
		   }

		   modified = 1;
	   }
   }

   // Ask if user wants to override HW power table 
   printf( "Override HW radio power table and use the above table instead[y/n] <%c> ? ", ovd ? 'y':'n');
   buf[0] = '\0';
   fgets( buf, sizeof(buf), stdin);

   if( buf[0] == '\n') /* user pressed Enter*/
	   buf[0] = (ovd ? 'y':'n');
   
	if( tolower(buf[0]) == 'y' ) {
		   if( !ovd)  modified = 1;
		   ovd = 1;
		   radio.mask |= 1;
	
	} else if( tolower(buf[0]) == 'n') {
		   if( ovd ) modified = 1;
		   ovd = 0;
		   radio.mask &= ~1;
	}
	else
	{
		printf("\nThe valid answer is only 'y' or 'n'\n");
		return;
	}
   
   if( tiap_wcfg_set_cfg_pow_tbl(wd, &radio, modified) ) {
		   PRINT_CONFIG_ERR_MSG("Failed to set radio power table.");
		   return;
	}
   
	   
}/* END wlanRadioParams() */

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSetUsageTimeMode(ConsoleParameter_T parameters[])
{/*
   UINT8  num_sup_wone;

   // If Dual AP is disable, reject usagetimeMode command and exit. 
   wdrv_dbGetNumSupWone(&num_sup_wone);
   if (num_sup_wone == WONE_SINGLE_BAND)
   {
	   printf("\nRejected!! Can't change Usage Time Mode while Dual AP is disable !!\n");
	   return;
   }
   wdrv_dbSetUsageTimeMode(parameters[0].SHORT, TRUE);

   printf("\n parameters[0].INTEGER = %x \n",parameters[0].SHORT );
   
   if (parameters[0].INTEGER == FALSE)
       printf("Set Short Usage Time\n" ); 
   else
       printf("Set Long Usage Time\n" ); 
       */

	   UINT8  num_sup_wone, usage_timemod;

	   /* If Dual AP is disable, reject usagetimeMode command and exit. */

	   if( tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_NUM, &num_sup_wone, FROM_STORAGE) ) {
			PRINT_CONFIG_ERR_MSG("Failed to get wone number.");
			return;
	   }
	   if (num_sup_wone == TIAP_PARAM_WONE_SINGLE_BAND)
	   {
		   printf("\nRejected!! Can't change Usage Time Mode while Dual AP is disable !!\n");
		   return;
	   }


	   usage_timemod = (UINT8)parameters[0].SHORT;

	   if( tiap_wcfg_set_basic(wd, TIAP_PARAM_WONE_TIME_MODE, &usage_timemod,UPDATE_STORAGE) ) 
	   {
			PRINT_CONFIG_ERR_MSG("Failed to set wone usage time mode.");
			return;
	   }

	   /* printf("\n parameters[0].INTEGER = %x \n",parameters[0].SHORT );*/
   
	   if (parameters[0].INTEGER == FALSE)
	       printf("Set Short Usage Time\n" ); 
	   else
	       printf("Set Long Usage Time\n" ); 

}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSetMacAddr(ConsoleParameter_T parameters[])
{
/*
   wdrv_dbSetOperationMacAddress(parameters[0].MAC_ADDR);
   printf("setting WLAN MAC = <%02hhX-%02hhX-%02hhX-%02hhX-%02hhX-%02hhX>\n",
          parameters[0].MAC_ADDR[0],parameters[0].MAC_ADDR[1],parameters[0].MAC_ADDR[2],parameters[0].MAC_ADDR[3],
          parameters[0].MAC_ADDR[4],parameters[0].MAC_ADDR[5] );
   printf("to take effect RESTART AP.\n");*/


	if( tiap_wcfg_set_hwaddr(wd, parameters[0].MAC_ADDR) ) {
		PRINT_CONFIG_ERR_MSG("Failed to set hardware address.");
		return;
	}
	
	printf("setting WLAN MAC = <%02hhX-%02hhX-%02hhX-%02hhX-%02hhX-%02hhX>\n",
			parameters[0].MAC_ADDR[0],parameters[0].MAC_ADDR[1],
			parameters[0].MAC_ADDR[2],parameters[0].MAC_ADDR[3],
			parameters[0].MAC_ADDR[4],parameters[0].MAC_ADDR[5] );
	printf("to take effect RESTART AP.\n");
}

/************************************************/
/*                  WME                         */
/************************************************/

/* Demiurg: WME */

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanQoSSetAC(ConsoleParameter_T parameters[]) {
	tiap_param_qos_t params;
	short num;
	int i;
	int val;
	char strVal[10];


	num = parameters[0].SHORT;

	printf("Setting AC params for %s\n", num == 0 ? "AP" : "STA");

	if ( tiap_wcfg_get_qos_params(wd, &params, FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get QoS params.");
		return;
	}
 
	for (i=0; i<4; ++i) 
	{
		switch (i) {
		case 0:
			puts("AC BE"); break;
		case 1:
			puts("AC BK"); break;
		case 2:
			puts("AC VI"); break;
		case 3:
			puts("AC VO"); break;
		default:
			puts("What the heck !");
		}

		printf("cwMin <%hhd>: ", params.edcf_info[num][i].cwMin);
		fgets(strVal, 10, stdin);
		strVal[strlen(strVal) - 1] = 0; /* remove newline */ 
		val = atoi(strVal);
		if(val > 0 || strlen(strVal) > 0)
        {
			params.edcf_info[num][i].cwMin = val;
            printf("---> Set cwMin = %hhd\n", params.edcf_info[num][i].cwMin);
		}
		printf("cwMax <%hd>: ", params.edcf_info[num][i].cwMax);
		fgets(strVal, 10, stdin);
		strVal[strlen(strVal) - 1] = 0; /* remove newline */ 
		val = atoi(strVal);
		if(val > 0 || strlen(strVal) > 0)
        {
			params.edcf_info[num][i].cwMax = val;
            printf("---> Set cwMax = %hd\n", params.edcf_info[num][i].cwMax);
		}

		printf("AISFN <%hhd>: ", params.edcf_info[num][i].aifsn);
		fgets(strVal, 10, stdin);
		strVal[strlen(strVal) - 1] = 0; /* remove newline */ 
		val = atoi(strVal);
		if(val > 0 || strlen(strVal) > 0)
        {
			params.edcf_info[num][i].aifsn = val;
            printf("---> Set AISFN = %hhd\n", params.edcf_info[num][i].aifsn);
		}	

		printf("short Retry Limit <%hhd>: ", params.edcf_info[num][i].shortRetryLimit);
		fgets(strVal, 10, stdin);
		strVal[strlen(strVal) - 1] = 0; /* remove newline */ 
		val = atoi(strVal);
		if(val > 0 || strlen(strVal) > 0)
        {
			params.edcf_info[num][i].shortRetryLimit = val;
            printf("---> Set short Retry Limit = %hhd\n", params.edcf_info[num][i].shortRetryLimit);
		}

		printf("Long Retry Limit <%hhd>: ", params.edcf_info[num][i].longRetryLimit);
		fgets(strVal, 10, stdin);
		strVal[strlen(strVal) - 1] = 0; /* remove newline */ 
		val = atoi(strVal);
		if(val > 0 || strlen(strVal) > 0)
        {
			params.edcf_info[num][i].longRetryLimit = val;
            printf("---> Set Long Retry Limit = %hhd\n", params.edcf_info[num][i].longRetryLimit);
		}

		printf("TXOP Limit 11b <%hd>: ", params.edcf_info[num][i].txopLimit11b);
		fgets(strVal, 10, stdin);
		strVal[strlen(strVal) - 1] = 0; /* remove newline */ 
		val = atoi(strVal);
		if(val > 0 || strlen(strVal) > 0)
        {
			params.edcf_info[num][i].txopLimit11b = val;
            printf("---> Set TXOP Limit 11b = %hd\n", params.edcf_info[num][i].txopLimit11b);
		}

		printf("TXOP Limit 11g <%hd>: ", params.edcf_info[num][i].txopLimit11g);
		fgets(strVal, 10, stdin);
		strVal[strlen(strVal) - 1] = 0; /* remove newline */ 
		val = atoi(strVal);
		if(val > 0 || strlen(strVal) > 0)
        {
			params.edcf_info[num][i].txopLimit11g = val;
            printf("---> Set TXOP Limit 11g = %hd\n", params.edcf_info[num][i].txopLimit11g);
		}

		printf("TX MAx Lifetime <%d>: ", params.edcf_info[num][i].txMaxLifetime);
		fgets(strVal, 10, stdin);
		strVal[strlen(strVal) - 1] = 0; /* remove newline */ 
		val = atoi(strVal);
		if(val > 0 || strlen(strVal) > 0)
        {
			params.edcf_info[num][i].txMaxLifetime = val;
            printf("---> Set TX MAx Lifetime = %d\n", params.edcf_info[num][i].txMaxLifetime);
		}

		printf("TX OP Continuation <%hd>: ", params.edcf_info[num][i].txOpContinuation);
		fgets(strVal, 10, stdin);
		strVal[strlen(strVal) - 1] = 0; /* remove newline */ 
		val = atoi(strVal);
		if(val > 0 || strlen(strVal) > 0)
        {
			params.edcf_info[num][i].txOpContinuation = val;
            printf("---> Set TX OP Continuation = %hd\n", params.edcf_info[num][i].txOpContinuation);
		}

		printf("ACK Policy (0-Ack,1-No Ack) <%hd>: ", params.edcf_info[num][i].ackPolicy);
		fgets(strVal, 10, stdin);
		strVal[strlen(strVal) - 1] = 0; /* remove newline */ 
		val = atoi(strVal);
		if(val > 0 || strlen(strVal) > 0)
        {
			params.edcf_info[num][i].ackPolicy = val;
            printf("---> Set ACK Policy (0-Ack,1-No Ack) = %hd\n", params.edcf_info[num][i].ackPolicy);
		}

	}

	if ( tiap_wcfg_set_qos_ac_params(wd, (const tiap_param_qos_t *)&params, num, UPDATE_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to set QoS params.");
		return;
	}

}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanQoSEnableWME(ConsoleParameter_T parameters[]) {
	tiap_param_qos_t params;
	char buf[4];

	if ( tiap_wcfg_get_qos_params(wd, &params, FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get QoS params.");
		return;
	}		

	params.enabled = 	parameters[0].SHORT;
	params.classifierMode = parameters[1].SHORT;

	printf("%s: Setting WME to : %d, Classifier mode to : %d \n", __FUNCTION__, params.enabled, params.classifierMode);

	if (params.classifierMode == TIAP_PARAM_QOS_CLASSIFIER_MODE_DIFFSERV) {
		int i, codepoint, vlan_priority;
		puts("Setting Diffserv mapping");
		printf("Use default DiffServ Codepoint to VLAN priority mapping [y/n] :");
		buf[0] = '\0';
		fgets( buf, sizeof(buf), stdin);
		if ( tolower(buf[0]) == 'n' ) {
			printf("Number of CodePoints <%hhd>: ", params.diffserv.numOfCP);
			if (scanf("%hhd", &params.diffserv.numOfCP) != 1) {
				PRINT_CONFIG_ERR_MSG("Wrong value");
				return;
			}
			
			if (params.diffserv.numOfCP > TIAP_PARAM_QOS_MAX_CP) {
				PRINT_CONFIG_ERR_MSG("Invalid number of Diffserv Code Points");
				return;
			}
			memset(params.diffserv.map, TIAP_PARAM_QOS_CLASSIFIER_AC_BE, TIAP_PARAM_QOS_MAX_CP);
			
			for (i=0; i<params.diffserv.numOfCP; ++i) {
				printf("CodePoint: ");
				if (scanf("%d", &codepoint) != 1) {
					PRINT_CONFIG_ERR_MSG("Wrong value");
					return;
				}
				
				printf("VLAN Priority: ");
				if (scanf("%d", &vlan_priority) != 1) {
					PRINT_CONFIG_ERR_MSG("Wrong value");
					return;
				}
				
				params.diffserv.map[codepoint] = vlan_priority;
			}
		}		
	} else {
		if ( params.enabled )
			puts("\nWARNING: Remember to enable Multiple SSID support !");
	}

	if ( tiap_wcfg_set_qos_params(wd, &params, UPDATE_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to set QoS params.");
		return;
	}

	puts("New WME settings will not take effect until reboot!");
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanQoSPrintWMEConf(ConsoleParameter_T parameters[]) {
	tiap_param_qos_t params;
	int i, j;
	char prefix[32], prefix_which[2][4] = {" AP", "STA"}, 
		prefix_ac[4][3] = {"BE", "BK", "VI", "VO"};

	if ( tiap_wcfg_get_qos_params(wd, &params, !FROM_STORAGE || driver_load_failed) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get QoS params.");
		return;
	}

	printf("QoS Enabled : %d\n", params.enabled);
	if (params.enabled) {
		switch(params.classifierMode)
		{
		case 	TIAP_PARAM_QOS_CLASSIFIER_MODE_VLAN:
			printf("Classifier Mode : VLAN\n");
			break;
		case 	TIAP_PARAM_QOS_CLASSIFIER_MODE_DIFFSERV:
			printf("Classifier Mode : DiffServ\n");
			break;
		case 	TIAP_PARAM_QOS_CLASSIFIER_MODE_UDP_PORT:
			printf("Classifier Mode : UDP Port\n");
			break;
		}
		puts("_______________________________________________________________________");
		for (i=0; i<2; ++i) {
			//			printf("AC for %s\n", i == 0 ? "AP" : "STA");
			for (j=0; j<4; ++j) {
				snprintf(prefix, 32, "[%s][%s]", prefix_which[i], prefix_ac[j]);
				printf("%s cwMin: %hhd, cwMax: %hd\n" 
							 "%s txop Limit 11b: %hd, txop Limit 11g: %hd\n"
							 "%s AIFSN : %hhd ACK Policy: %hhd\n"
							 "%s Long Retry Limit : %hhd, Short Retry Limit : %hhd\n"
							 "%s TX Max Lifetime : %d TX op Continuation : %d\n",
							 prefix, params.edcf_info[i][j].cwMin, params.edcf_info[i][j].cwMax, 
							 prefix, params.edcf_info[i][j].txopLimit11b, params.edcf_info[i][j].txopLimit11g, 
							 prefix, params.edcf_info[i][j].aifsn, params.edcf_info[i][j].ackPolicy, 
							 prefix, params.edcf_info[i][j].longRetryLimit, params.edcf_info[i][j].shortRetryLimit, 
							 prefix, params.edcf_info[i][j].txMaxLifetime, params.edcf_info[i][j].txOpContinuation);
				puts("_______________________________________________________________________");
			}
		}
		if (params.classifierMode == TIAP_PARAM_QOS_CLASSIFIER_MODE_DIFFSERV) {
			printf("Diffserv mapping, number of CodePoints = %d\n", params.diffserv.numOfCP);
			for (i=0; i<TIAP_PARAM_QOS_MAX_CP /*params.diffserv.numOfCP*/; ++i) {
				printf("Diffserv CodePoint : %d <--> VLAN priority : %d\n", i, params.diffserv.map[i]);
			}
		}
		else if (params.classifierMode == TIAP_PARAM_QOS_CLASSIFIER_MODE_UDP_PORT) 
		{
			printf("UDP Port mapping\n");
			printf("----------------\n");
			printf(" 5000 - BE\n 5001 - BK\n 5002 - VI\n 5003 - VO\n 5008 - RoundRobin\n\n");
			
		}

	}
}

/************************************************/
/*                  S S I D                     */
/************************************************/

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSsidPrintParams(ConsoleParameter_T parameters[])
{/*
    char  macaddr[6], ssid[50];
    int i;
    UINT16 HiddenSSID, MultipleSsidFlag;
    char MultipleSsidTable[NUM_MULTIPLE_SSID][WHAL_SSID_STRING_LEN];
    UINT32 length = NUM_MULTIPLE_SSID*WHAL_SSID_STRING_LEN;

    wdrv_dbGetSsid(ssid);
    wdrv_dbGetHiddenSsidStatus(&HiddenSSID);
    wdrv_dbGetMultipleSsidStatus(&MultipleSsidFlag);
    whal_nvram_getMultipleSsidTable(MultipleSsidTable[0],&length);

    printf("SSID = %s\n", ssid); 

    printf("Hidden SSID mode is %s\n", (HiddenSSID==1) ? "Enabled" : "Disabled");
    if (wdrv_dbIsPrivacyEnabled())
    {
        printf("Multiple SSID features is DISABLED, due to Privacy use.\n\n");
    }
    else
        printf("Multiple SSID mode is %s\n", (MultipleSsidFlag==1) ? "Enabled" : "Disabled");
    
    if (MultipleSsidTable[0][0] == '\0')
    {
        printf("Multiple SSID table is empty.\n");
    }
    else
    {
        printf("\nList of Multiple SSID:\n");
        for (i=0; i<NUM_MULTIPLE_SSID; i++) 
        {
            if (MultipleSsidTable[i][0] == '\0')
                break;
            printf("\t%d. %s\n",i+1,MultipleSsidTable[i]);
        }
        if (i==0)
            printf("\tNone.\n");
    }*/


	char  ssid[IW_ESSID_MAX_SIZE+1];
	int i, n;
	__u16 hidden_ssid;
	tiap_param_multi_ssid_t multi_ssid;

	if( tiap_wcfg_get_basic(wd, TIAP_PARAM_HIDDEN_SSID, &hidden_ssid, !FROM_STORAGE || driver_load_failed) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get hidden ssid.");
		return;
	}

	if( tiap_wcfg_get_multi_ssid(wd, &multi_ssid, FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get multiple ssid status.");
		return;
	}

	

	if(multi_ssid.secondary_enabled)
	{
		printf("\nList of AP supported SSID's:\n");
		printf("\nProfile    SSID        Vlan\nindex\n\n");

		n =0;
		for (i=0; i < TIAP_PARAM_NUM_MULTIPLE_SSID; i++) 
		{
			if (multi_ssid.tbl[i].ssid_type == TIAP_PARAM_SSID_PRIMARY)
				printf("* \n");

			if (multi_ssid.tbl[i].ssid_type == TIAP_PARAM_SSID_PRIMARY ||
				multi_ssid.tbl[i].ssid_type == TIAP_PARAM_SSID_SECONDARY )
			{
				printf("  %d      %-16s %d\n", n,multi_ssid.tbl[i].ssid,multi_ssid.tbl[i].vlan);
				n++;
			}
		}
		printf("\n* - primary ssid.\n"); 
	}

	tiap_wcfg_get_primary_ssid(wd, ssid, !FROM_STORAGE || driver_load_failed);
		printf("\nAP SSID:      %-16s", ssid); 
	
	printf("\nssid flags status:");

	printf("\n  Hidden SSID mode is %s\n", (hidden_ssid==1) ? "Enabled" : "Disabled");

	printf("  Multiple SSID mode is %s\n\n\n", (multi_ssid.secondary_enabled==1) ? "Enabled" : "Disabled");

}


/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSSID(ConsoleParameter_T parameters[])
{/*
    UINT8 pbSSID[WHAL_SSID_STRING_LEN+1];
    UINT8 bLen = WHAL_SSID_STRING_LEN;
    whal_nvram_getSsid(pbSSID, &bLen);
    pbSSID[bLen] = 0;
   
    printf("SSID <%s>: ",pbSSID);
    fgets(pbSSID, WHAL_SSID_STRING_LEN, stdin);
    pbSSID[strlen(pbSSID) - 1] = 0; // remove newline 
    bLen = min(strlen(pbSSID),WHAL_SSID_STRING_LEN);
    if(bLen > 0)
    {
        pbSSID[bLen] = 0;
        printf("setting SSID <%s>\n",pbSSID);
        whal_nvram_setSsid(pbSSID,bLen,IMMEDIATELY);
        puts("to take effect RESTART AP.");
    }*/

	char ssid[IW_ESSID_MAX_SIZE+2];
	__u8 len;

	if( tiap_wcfg_get_primary_ssid(wd, ssid, !FROM_STORAGE || driver_load_failed) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get ssid.");
		return;
	}

	printf("SSID <%s>: ", ssid);
	fgets(ssid, IW_ESSID_MAX_SIZE+2, stdin);
	len = strlen(ssid);
	ssid[len-1] = '\0'; /* remove new line character */

	if( strlen(ssid) )
	{
		tiap_param_multi_ssid_t multi_ssid;
		tiap_param_multi_ssid_elem_t dummy_elem;
		int dummy_index;

		if (tiap_wcfg_get_multi_ssid(wd, &multi_ssid, FROM_STORAGE))
		{
			printf("Failed to get multiple ssid structure\n");
			return;
		}

		if (tiap_wcfg_lookUpMultiSsidBySsid(&multi_ssid, ssid, &dummy_elem, &dummy_index) == OK) 
		{
			printf("The given SSID already exists in the Multiple SSID table. Setting aborted!\n");
			return;
		}

		if( tiap_wcfg_set_primary_ssid(wd, ssid, UPDATE_STORAGE) ) {
			PRINT_CONFIG_ERR_MSG("Failed to set ssid.");
			return;
		}
		printf("setting SSID <%s>\n", ssid);
		puts("to take effect RESTART AP.");
	}
}


/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSetHiddenSSID(ConsoleParameter_T parameters[])
{/*
	if ((parameters[0].SHORT != 0) &&
		(parameters[0].SHORT != 1))
	{
		printf("Invalid mode: %d. Only 0 and 1 are legal values.\n", parameters[0].SHORT);
		return;
	}
	wdrv_dbSetHiddenSsidStatus(parameters[0].SHORT);
	printf("Setting Hidden SSID mode to: %s\n",(parameters[0].SHORT==1) ? "Enable":"Disable");
	printf("to take effect RESTART AP.\n");*/

	if ((parameters[0].SHORT != 0) && (parameters[0].SHORT != 1))
	{
		printf("Invalid mode: %d. Only 0 and 1 are legal values.\n", parameters[0].SHORT);
		return;
	}

	if( tiap_wcfg_set_basic(wd, TIAP_PARAM_HIDDEN_SSID, &parameters[0].SHORT, UPDATE_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to set hidden ssid.");
		return;
	}

	printf("Setting Hidden SSID mode to: %s\n",(parameters[0].SHORT==1) ? "Enable":"Disable");
	printf("to take effect RESTART AP.\n");
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSetMultipleSSID(ConsoleParameter_T parameters[])
{
	int i;  
	tiap_param_multi_ssid_t multi_ssid;
    UINT16 chosen;

	if ((parameters[0].SHORT != 0) && (parameters[0].SHORT != 1))
	{
		printf("Invalid mode: %d. Only 0 and 1 are legal values.\n", parameters[0].SHORT);
		return;
	}

	if(parameters[0].SHORT==1)
	{
		char pbSSID[64];
		
		printf("please type vlan tag for primary ssid: ");
		fgets(pbSSID, IW_ESSID_MAX_SIZE, stdin);
		pbSSID[strlen(pbSSID) - 1] = 0; // remove newline 
		chosen = atoi(pbSSID);
		if( (chosen > 4095) || (chosen < 1))
		{
			PRINT_CONFIG_ERR_MSG("error vlan recevived: must be in 1-4095 range");
			return;
		}

	}
    else {
        chosen = 0;
    }

	if( tiap_wcfg_get_multi_ssid(wd, &multi_ssid, FROM_STORAGE) ) 
	{
		PRINT_CONFIG_ERR_MSG("Failed to get multiple ssid status.");
		return;
	 }

	multi_ssid.secondary_enabled = parameters[0].SHORT;  	
	
	if (parameters[0].SHORT == 0) /* disable multiple ssid feature */
	{
		for (i=0; i < TIAP_PARAM_NUM_MULTIPLE_SSID; i++) 
		{
			if( multi_ssid.tbl[i].ssid_type == TIAP_PARAM_SSID_SECONDARY )
			{
				__u32 sec_profile_index;

				multi_ssid.tbl[i].ssid_type = TIAP_PARAM_SSID_NON_ACTIVE;
				/* free the corresponding security profile*/
				sec_profile_index = multi_ssid.tbl[i].sec_profile_index;
				wcfg_storage_setdefaults_sec_profile(&wd->storage.wpa.profile[sec_profile_index]);

			}
		}
	}	

	printf("Setting Multiple SSID mode to: %s\n",(parameters[0].SHORT==1) ? "Enable":"Disable");
	if( tiap_wcfg_set_multi_ssid(wd, &multi_ssid, !UPDATE_STORAGE) )
	{
		PRINT_CONFIG_ERR_MSG("Failed to set multiple ssid status.");
		return;
	}
    if(tiap_wcfg_set_primary_vlan(wd,chosen, UPDATE_STORAGE))
    {
        PRINT_CONFIG_ERR_MSG("Failed to set primary vlan id.");
        return;
    }
    
	printf("to take effect RESTART AP.\n");
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanAddMultipleSsidElement(ConsoleParameter_T parameters[]) 
{
	tiap_param_multi_ssid_t multi_ssid;
	tiap_param_multi_ssid_elem_t elem;
	UINT8 pbSSID[IW_ESSID_MAX_SIZE+4];
	UINT8 bLen = IW_ESSID_MAX_SIZE;
	UINT16 vlan_tag;
	
	int dummy_index;

	if( tiap_wcfg_get_multi_ssid(wd, &multi_ssid, FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get multiple ssid status.");
		return;
	}

	if (!multi_ssid.secondary_enabled) 
	{
		printf("The Multiple SSID mode is disabled. Enable it first.\n");
		return;
	}

	vlan_tag = parameters[0].INTEGER;

	if (vlan_tag < 1 || vlan_tag > 4095) 
	{
		printf("Wrong vlan tag value %d: must be in 1-4095 range.\n", vlan_tag);
		return;
	}
	
	if (tiap_wcfg_lookUpMultiSsidByVlan(&multi_ssid, vlan_tag, &elem, &dummy_index) == OK) 
	{
		printf("vlan already in use for ssid %s.\n",elem.ssid);
		return;
	}

	printf("Type SSID: ");
	memset(pbSSID,0,IW_ESSID_MAX_SIZE+1);
	fgets(pbSSID, IW_ESSID_MAX_SIZE+3, stdin);
	pbSSID[strlen(pbSSID) - 1] = 0; /* remove newline */
	if (strlen(pbSSID) > IW_ESSID_MAX_SIZE)
	{
		printf("String too long! must be lower than %d.\n",IW_ESSID_MAX_SIZE+1);
		return;
	}
	bLen = min(strlen(pbSSID),IW_ESSID_MAX_SIZE);
	if(bLen > 0)
	{
		/* Check that the new SSID doesn't exist (in the STORAGE)  */
		if (tiap_wcfg_lookUpMultiSsidBySsid(&multi_ssid, pbSSID, &elem, &dummy_index) == OK) 
		{
			printf("The given SSID already exists in the Multiple SSID table. Setting aborted!\n");
			return;
		}
		pbSSID[bLen] = 0;
		
		elem.ssid_type = TIAP_PARAM_SSID_SECONDARY;
		elem.vlan = vlan_tag;
		strcpy(elem.ssid, pbSSID);

		printf("setting new SSID element to Multiple SSID table <%s> ,vlan = %d\n",elem.ssid,elem.vlan);

		if (tiap_wcfg_addMultiSsid(wd, &elem, UPDATE_STORAGE))
		{
			PRINT_CONFIG_ERR_MSG("Failed to add ssid to multiple ssid table.");
			return;
		}

		puts("to take effect RESTART AP.");
		
	}
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanRemoveMultipleSsidElement(ConsoleParameter_T parameters[])
{
	UINT8 pbSSID[IW_ESSID_MAX_SIZE+1];
	int i, chosen, n, actual_ind;
	tiap_param_multi_ssid_t multi_ssid;

	
	if( tiap_wcfg_get_multi_ssid(wd, &multi_ssid, FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get multiple ssid status.");
		return;
	}
	
	if (!multi_ssid.secondary_enabled) 
	{
		printf("The Multiple SSID mode is disable. Enable it first.\n");
		return;
	}

	n = 0; /* counter of secondary ssids*/
    printf("List of Multiple SSID:\n");
	for (i =0; i < TIAP_PARAM_NUM_MULTIPLE_SSID; i++) 
	{
		if (multi_ssid.tbl[i].ssid_type == TIAP_PARAM_SSID_PRIMARY)
				printf("* \n");

		/* Only secondary SSIDs can be removed from command line */
		if (multi_ssid.tbl[i].ssid_type == TIAP_PARAM_SSID_SECONDARY ||
			multi_ssid.tbl[i].ssid_type == TIAP_PARAM_SSID_PRIMARY) 
		{
			printf("%d. %s\n", n, multi_ssid.tbl[i].ssid);
			n++;
		}
	}
	printf("\n* - primary ssid.\n"); 

	if (n == 0) 
	{
		printf("The Multiple SSID table is empty.\n");
		return;
	}

	printf("Type SSID index: ");
	fgets(pbSSID, IW_ESSID_MAX_SIZE, stdin);
	pbSSID[strlen(pbSSID) - 1] = 0; // remove newline 
	chosen = atoi(pbSSID);
	
	if( (chosen >= 0) && (chosen < n) )
	{
		printf("removing SSID element #%d from the Multiple SSID table\n",chosen);

		/* find secondary SSID index with sequence number chosen */
		tiap_wcfg_get_actual_ssid_ind(&wd->storage.ldb.multi_ssid, chosen, &actual_ind);
		
		if (multi_ssid.tbl[actual_ind].ssid_type == TIAP_PARAM_SSID_PRIMARY) 
		{
			printf("Primary SSID couldn't be removed\n");
			return;
		}

		if (tiap_wcfg_delMultiSsid(wd, actual_ind, UPDATE_STORAGE))
		{
			PRINT_CONFIG_ERR_MSG("Failed to delete from multiple ssid table.");
			return;
		}

		puts("to take effect RESTART AP.");
	}
	else
		printf("index out of range\n");
}

/****************************************************/
/*                  802.11 D & H                    */
/****************************************************/

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlan_dh_PrintParams(ConsoleParameter_T parameters[])
{/*
    UINT8 value, band, regDomain, uint8value;
    INT8  int8value;
    UINT32   level, i;
    wlan_bandType bandType;
    IntegerRegulatoryDomains_e RegDomain;
    tChannelRangeElementStruct RangeElementStruct;
    char countryString[3];


    whal_nvram_getBand(currentConfigWoneInex, &band);

#ifdef WLAN_801_11_H_IMPLEMENTED
	printf("802.11D and 802.11H parameters:\n");
    printf("===============================\n");
#else
	printf("802.11D parameters:\n");
    printf("===================\n");
#endif // WLAN_801_11_H_IMPLEMENTED 
    wdrv_dbGetdot11MultiDomainCapabilityStatus(&value);
    printf("Multi Domain Capability status is: %s\n",(value == 0) ? "Disable" : "Enable");
#ifdef WLAN_801_11_H_IMPLEMENTED
    wdrv_dbGetdot11SpectrumManagementRequiredStatus(&value);
    printf("Spectrum Management Requirement status is: %s\n",(value == 0) ? "Disable" : "Enable");
    wdrv_dbGetLegacyStatusEnableStatus(&value);
    printf("Legacy Station establishment status is : %s\n",(value == 0) ? "Disable" : "Enable");
#endif // WLAN_801_11_H_IMPLEMENTED 
    wdrv_dbGetdot11LocalPowerConstraint(&uint8value);
    printf("Power Constraint is %d (dB)\n",uint8value);
    printf("Band is: %s\n",(band == RADIO_BAND_2_4GHZ) ? "RADIO_BAND_2_4GHZ (G)" : "RADIO_BAND_5GHZ (A)");
    wdrv_dbGetdot11CurrentRegDomain(band,&regDomain);
    printf("Regulatory Domain is: ");

    // compute regularity domain 
    switch (band) 
    {
        case RADIO_BAND_2_4GHZ: // g 
            switch (regDomain)
            {
                case REG_DOMAIN_FCC:
                    printf("REG_DOMAIN_FCC (0x10)\n");
                    break;
                case REG_DOMAIN_IC:
                    printf("REG_DOMAIN_IC (0x20)\n");
                    break;
                case REG_DOMAIN_ETSI:
                    printf("REG_DOMAIN_ETSI (0x30)\n");
                    break;
                case REG_DOMAIN_SPAIN:
                    printf("REG_DOMAIN_SPAIN (0x31)\n");
                    break;
                case REG_DOMAIN_FRANCE:
                    printf("REG_DOMAIN_FRANCE (0x32)\n");
                    break;
                case REG_DOMAIN_MKK:
                    printf("REG_DOMAIN_MKK (0x40)\n");
                    break;
                case REG_DOMAIN_MKK1:
                    printf("REG_DOMAIN_MKK1 (0x41)\n");
                    break;
                default :
                    printf("CUSTOM_REG_DOMAIN (0x%X)\n",regDomain);
                    break;
            }
            break;
    
        case RADIO_BAND_5GHZ:   // a 
            switch (regDomain)
            {
                case REG_DOMAIN_US:
                    printf("REG_DOMAIN_US (0x50)\n");
                    break;
                case REG_DOMAIN_WORLD:
                    printf("REG_DOMAIN_WORLD (0x51)\n");
                    break;
                default:
                    printf("CUSTOM_REG_DOMAIN (0x%X)\n",regDomain);
                    break;
            }
            break;
    }

    wdrv_dbGetMaxTxPowerLevel(&int8value);
    printf("Max Tx Power is %d (dBm)\n",int8value);
    printf("Tx Power levels for band 5GHz: (dBm)\n");
    for (level=1; level<=5; level++)
    {
        wdrv_dbGetTxPowerLevels(RADIO_BAND_5GHZ, level,&int8value);
        printf("\t%d.\t%d\n",level,(int)int8value);
    }
    printf("Tx Power levels for band 2.4GHz: (dBm)\n");
    for (level=1; level<=5; level++)
    {
        wdrv_dbGetTxPowerLevels(RADIO_BAND_2_4GHZ, level,&int8value);
        printf("\t%d.\t%d\n",level,(int)int8value);
    }
    wdrv_dbGetdot11CountryString(countryString);
    printf("Country String is: %c%c, type: %c ",
           countryString[0],
           countryString[1],
           countryString[2]);
    if (countryString[2] == 'I') 
        printf("(Indoor)\n");
    else if (countryString[2] == 'O') 
            printf("(Outdoor)\n");
    else
        printf("(Other)\n");*/


UINT8 value, uint8value;
    __u8  int8value;
    UINT32   level;
    char countryString[3];
    tiap_param_wone_t wone;
    tiap_param_cur_reg_domain_t cur_regdomain;
    tiap_param_txpower_t txpow;


    wone.index = currentConfigWoneInex;
    wone.type = wone_band;
    if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE || driver_load_failed) ) {
	    PRINT_CONFIG_ERR_MSG("Failed to get band.");
	    return;
    }

#ifdef WLAN_801_11_H_IMPLEMENTED
	printf("802.11D and 802.11H parameters:\n");
    printf("===============================\n");
#else
	printf("802.11D parameters:\n");
    printf("===================\n");
#endif // WLAN_801_11_H_IMPLEMENTED 

    if( tiap_wcfg_get_basic(wd, TIAP_PARAM_MULTI_DOMAIN_CAPAB, &value, !FROM_STORAGE || driver_load_failed) ) {
	    PRINT_CONFIG_ERR_MSG("Failed to get multiple domain capability.");
	    return;
    }
    printf("Multi Domain Capability status is: %s\n",(value == 0) ? "Disable" : "Enable");
#ifdef WLAN_801_11_H_IMPLEMENTED
    if( tiap_wcfg_get_basic(wd, TIAP_PARAM_SPECTRUM_MGMT, &value, !FROM_STORAGE || driver_load_failed) ) {
	    PRINT_CONFIG_ERR_MSG("Failed to get spectrum management required.");
	    return;
    }
    printf("Spectrum Management Requirement status is: %s\n",(value == 0) ? "Disable" : "Enable");
    if( tiap_wcfg_get_basic(wd, TIAP_PARAM_SPECTRUM_MGMT, &value, !FROM_STORAGE || driver_load_failed) ) {
	    PRINT_CONFIG_ERR_MSG("Failed to get spectrum management required.");
	    return;
    }
    if( tiap_wcfg_get_basic(wd, TIAP_PARAM_LEGACY_STA, &value, !FROM_STORAGE || driver_load_failed) ) {
	    PRINT_CONFIG_ERR_MSG("Failed to get legacy station establishment status.");
	    return;
    }
    printf("Legacy Station establishment status is : %s\n",(value == 0) ? "Disable" : "Enable");
#endif // WLAN_801_11_H_IMPLEMENTED 
    if( tiap_wcfg_get_basic(wd, TIAP_PARAM_LOCAL_POWER_CONS, &uint8value, !FROM_STORAGE || driver_load_failed) ) {
	    PRINT_CONFIG_ERR_MSG("Failed to get power constraint.");
	    return;
    }
    printf("Power Constraint is %d (dB)\n",uint8value);
    printf("Band is: %s\n",(wone.elem.band == TIAP_PARAM_RADIO_BAND_2_4GHZ) ?
		    "RADIO_BAND_2_4GHZ (G)" : "RADIO_BAND_5GHZ (A)");
    cur_regdomain.band = wone.elem.band;
    if( tiap_wcfg_get_cur_regdomain(wd, &cur_regdomain, !FROM_STORAGE || driver_load_failed) ) {
	    PRINT_CONFIG_ERR_MSG("Failed to get power constraint.");
	    return;
    }
    printf("Regulatory Domain is: ");

    // compute regularity domain 
    switch (wone.elem.band) 
    {
        case TIAP_PARAM_RADIO_BAND_2_4GHZ: // g 
            switch ((__u8)cur_regdomain.type)
            {
                case TIAP_PARAM_RD_FCC:
                    printf("REG_DOMAIN_FCC (0x10)\n");
                    break;
                case TIAP_PARAM_RD_IC:
                    printf("REG_DOMAIN_IC (0x20)\n");
                    break;
                case TIAP_PARAM_RD_ETSI:
                    printf("TIAP_PARAM_RD_ETSI (0x30)\n");
                    break;
                case TIAP_PARAM_RD_SPAIN:
                    printf("TIAP_PARAM_RD_SPAIN (0x31)\n");
                    break;
                case TIAP_PARAM_RD_FRANCE:
                    printf("TIAP_PARAM_RD_FRANCE (0x32)\n");
                    break;
                case TIAP_PARAM_RD_MKK:
                    printf("TIAP_PARAM_RD_MKK (0x40)\n");
                    break;
                case TIAP_PARAM_RD_MKK1:
                    printf("TIAP_PARAM_RD_MKK1 (0x41)\n");
                    break;
                case TIAP_PARAM_RD_ISRAEL:
                    printf("REG_DOMAIN_ISRAEL (0x43)\n");
                    break;
                default :
                    printf("CUSTOM_REG_DOMAIN (0x%X)\n", (__u8)cur_regdomain.type);
                    break;
            }
            break;
    
        case TIAP_PARAM_RADIO_BAND_5GHZ:   // a 
            switch ((__u8)cur_regdomain.type)
            {
                case TIAP_PARAM_RD_US:
                    printf("REG_DOMAIN_US (0x50)\n");
                    break;
                case TIAP_PARAM_RD_WORLD:
                    printf("REG_DOMAIN_WORLD (0x51)\n");
                    break;
                default:
                    printf("CUSTOM_REG_DOMAIN (0x%X)\n", (__u8)cur_regdomain.type);
                    break;
            }
            break;
    }

    if( tiap_wcfg_get_basic(wd, TIAP_PARAM_MAX_TX_POWER_LEVEL, &int8value, !FROM_STORAGE || driver_load_failed) ) {
	    PRINT_CONFIG_ERR_MSG("Failed to get max tx power.");
	    return;
    }
    printf("Max Tx Power index is %d\n",int8value);
    printf("Tx Power levels for band 5GHz: (dBm)\n");

    txpow.hr = txpower_a;
    if( tiap_wcfg_giwtxpow(wd, &txpow, !FROM_STORAGE || driver_load_failed) ) {
	    PRINT_CONFIG_ERR_MSG("Failed to get tx powers.");
	    return;
    }
    for (level=1; level<=5; level++)
        printf("\t%d.\t%d\n", level, (int)txpow.txpower[level-1]);


    txpow.hr = txpower_g;
    if( tiap_wcfg_giwtxpow(wd, &txpow, !FROM_STORAGE || driver_load_failed) ) {
	    PRINT_CONFIG_ERR_MSG("Failed to get tx powers.");
	    return;
    }
    printf("Tx Power levels for band 2.4GHz: (dBm)\n");
    for (level=1; level<=5; level++)
        printf("\t%d.\t%d\n", level, (int)txpow.txpower[level-1]);

    if( tiap_wcfg_get_basic(wd, TIAP_PARAM_COUNTRY_STR, &countryString, !FROM_STORAGE || driver_load_failed) ) {
	    PRINT_CONFIG_ERR_MSG("Failed to get power constraint.");
	    return;
    }
    printf("Country String is: %c%c, type: %c ", countryString[0], countryString[1], countryString[2]);
    if (countryString[2] == 'I') 
        printf("(Indoor)\n");
    else if (countryString[2] == 'O') 
            printf("(Outdoor)\n");
    else
        printf("(Other)\n");
}


/*!	\brief 
  	\param 

  	\return 
  
*/
void wlan_dh_PrintRegDomain(ConsoleParameter_T parameters[])
{/*
    UINT8 value, band, regDomainA, regDomainB, regDomain, uiRanges;
    INT8  int8value;
    UINT32   level, numOfRanges, index, chosen;
    wlan_bandType bandType;
    IntegerRegulatoryDomains_e RegDomain;
    tChannelRangeElementStruct RangeElementStruct;
    char countryString[3], tmpBuffer[100];

    whal_nvram_getBand(currentConfigWoneInex, &band);
    printf("Band is: %s\n",(band == RADIO_BAND_2_4GHZ) ? "RADIO_BAND_2_4GHZ (G)" : "RADIO_BAND_5GHZ (A)");
    wdrv_dbGetdot11CurrentRegDomain(band, &regDomain);
    printf("Current Regularity Domain is: ");

    // compute regularity domain 
    switch (band) 
    {
        case RADIO_BAND_2_4GHZ: // g 
            switch (regDomain)
            {
                case REG_DOMAIN_FCC:
                    printf("REG_DOMAIN_FCC (0x10)\n");
                    break;
                case REG_DOMAIN_IC:
                    printf("REG_DOMAIN_IC (0x20)\n");
                    break;
                case REG_DOMAIN_ETSI:
                    printf("REG_DOMAIN_ETSI (0x30)\n");
                    break;
                case REG_DOMAIN_SPAIN:
                    printf("REG_DOMAIN_SPAIN (0x31)\n");
                    break;
                case REG_DOMAIN_FRANCE:
                    printf("REG_DOMAIN_FRANCE (0x32)\n");
                    break;
                case REG_DOMAIN_MKK:
                    printf("REG_DOMAIN_MKK (0x40)\n");
                    break;
                case REG_DOMAIN_MKK1:
                    printf("REG_DOMAIN_MKK1 (0x41)\n");
                    break;
                default :
                    printf("CUSTOM_REG_DOMAIN (0x%X)\n",regDomain);
                    break;
            }
            break;
    
        case RADIO_BAND_5GHZ:   // a 
            switch (regDomain)
            {
                case REG_DOMAIN_US:
                    printf("REG_DOMAIN_US (0x50)\n");
                    break;
                case REG_DOMAIN_WORLD:
                    printf("REG_DOMAIN_WORLD (0x51)\n");
                    break;
                default:
                    printf("CUSTOM_REG_DOMAIN (0x%X)\n",regDomain);
                    break;
            }
            break;
    }

    printf("Type one of the following:\n");
    printf("Regulatory Domains for 802.11g:\n");
    printf("-------------------------------\n");
    printf("1. REG_DOMAIN_FCC    (0x10)\n");
    printf("2. REG_DOMAIN_IC     (0x20)\n");
    printf("3. REG_DOMAIN_ETSI   (0x30)\n");
    printf("4. REG_DOMAIN_SPAIN  (0x31)\n");
    printf("5. REG_DOMAIN_FRANCE (0x32)\n");
    printf("6. REG_DOMAIN_MKK    (0x40)\n");
    printf("7. REG_DOMAIN_MKK1   (0x41)\n");
    whal_nvram_getPrivateRegDomain(&regDomainB);
    if (regDomain != 0)
        printf("8. CUSTOM_REG_DOMAIN (0x%X)\n",regDomainB);
    else
        printf("Note: CUSTOM_REG_DOMAIN undefined\n",regDomainB);
    printf("\nRegulatory Domains for 802.11a:\n");
    printf("-------------------------------\n");
    printf("9. REG_DOMAIN_US     (0x50)\n");
    printf("10. REG_DOMAIN_WORLD  (0x51)\n");
    whal_nvram_getPrivateRegDomain(&regDomainA);
    if (regDomain != 0)
        printf("11. CUSTOM_REG_DOMAIN (0x%X)\n",regDomainA);
    else
        printf("Note: CUSTOM_REG_DOMAIN undefined\n",regDomainA);


    printf("\nType index number:");
    fgets(tmpBuffer, 100, stdin);
    tmpBuffer[strlen(tmpBuffer) - 1] = 0; // remove newline 
    chosen = atoi(tmpBuffer);
    if(chosen <= 0)
        return;
    
    if (chosen > 12) 
    {
        printf("index not exists!\n");
        return;
    }

    switch (chosen)
    {
        case 1:
            regDomain = REG_DOMAIN_FCC;    
            bandType = WLAN_BAND_B;
            break;
            
        case 2:
            regDomain = REG_DOMAIN_IC;    
            bandType = WLAN_BAND_B;
            break;
            
        case 3:
            regDomain = REG_DOMAIN_ETSI;    
            bandType = WLAN_BAND_B;
            break;

        case 4:
            regDomain = REG_DOMAIN_SPAIN;    
            bandType = WLAN_BAND_B;
            break;

        case 5:
            regDomain = REG_DOMAIN_FRANCE;    
            bandType = WLAN_BAND_B;
            break;

        case 6:
            regDomain = REG_DOMAIN_MKK;    
            bandType = WLAN_BAND_B;
            break;

        case 7:
            regDomain = REG_DOMAIN_MKK1;    
            bandType = WLAN_BAND_B;
            break;

        case 8:
            if (regDomainB==0)
            {
                printf("Custom Regulatory Domain for Band G is undefined.\n");
                return;
            }
            regDomain = regDomainB;
            bandType = WLAN_BAND_B;
            break;

        case 9:
            regDomain = REG_DOMAIN_US;    
            bandType = WLAN_BAND_A;
            break;
            
        case 10:
            regDomain = REG_DOMAIN_WORLD;    
            bandType = WLAN_BAND_A;
            break;
            
        case 11:
            if (regDomainA==0)
            {
                printf("Custom Regulatory Domain for Band A is undefined.\n");
                return;
            }
            regDomain = regDomainA;
            bandType = WLAN_BAND_A;
            break;
    }
    switch (chosen) 
    {
        case 1:
        case 2:
        case 3:
        case 4:
        case 5:
        case 6:
        case 7:
        case 9:
        case 10:
            numOfRanges = wdrv_dbGetNumberOfDomainElements(bandType,regDomain);
            printf("Start Ch.    Stop Ch.    Step    OFDM    Power Limit\n");
            printf("---------    --------    ----    ----    -----------\n");
            for (index = 1; index <= numOfRanges; index++) 
            {
                wdrv_dbGetRangeElement(bandType,regDomain,index,&RangeElementStruct);
                printf("%d \t     %d \t %d \t %s \t %d\n",
                       RangeElementStruct.startCh,
                       RangeElementStruct.stopCh,
                       RangeElementStruct.stepOpt & 0xF,
                       (RangeElementStruct.stepOpt & 0x10) ? "No" : "Yes",
                       RangeElementStruct.powerLimit);
            }
            printf("\n");
            break;

        case 8:     // private "G" 
        case 11:    // private "A" 
            wdrv_dbGetNumOfPrivateRanges(bandType,&uiRanges);
            if (uiRanges == 0) 
            {
                printf("Not configured yet.\n");
                return;
            }
            printf("Start Ch.    Stop Ch.    Step    OFDM    Power Limit\n");
            printf("---------    --------    ----    ----    -----------\n");
            for (index = 1; index <= uiRanges; index++) 
            {
                wdrv_dbGetPrivateRangeElement(bandType,index,&RangeElementStruct);
                printf("%d \t     %d \t %d \t %s \t %d\n",
                       RangeElementStruct.startCh,
                       RangeElementStruct.stopCh,
                       RangeElementStruct.stepOpt & 0xF,
                       (RangeElementStruct.stepOpt & 0x10) ? "No" : "Yes",
                       RangeElementStruct.powerLimit);
            }
            printf("\n");
            break;
#if 0
        case 11:    // private "A" 
            wdrv_dbGetNumOfPrivateRanges(bandType,&uiRanges);
            if (uiRanges == 0) 
            {
                printf("Not configured yet.\n");
                return;
            }
            printf("Start Ch.    Stop Ch.    Step    OFDM    Power Limit\n");
            printf("---------    --------    ----    ----    -----------\n");
            for (index = 1; index <= uiRanges; index++) 
            {
                wdrv_dbGetPrivateRangeElement(WLAN_BAND_A,index,&RangeElementStruct);
                printf("%d \t     %d \t    %d \t%s \t%d\n",
                       RangeElementStruct.startCh,
                       RangeElementStruct.stopCh,
                       RangeElementStruct.stepOpt & 0xF,
                       (RangeElementStruct.stepOpt & 0x10) ? "No" : "Yes",
                       RangeElementStruct.powerLimit);
            }
            printf("\n");
            break;
#endif    
    }*/


    __u32 index, chosen, num_ranges;
    int band_type = BAND_B;
    char tmpBuffer[100];
    tiap_param_wone_t wone;
    tiap_param_cur_reg_domain_t cur_regdomain;
    tiap_param_priv_reg_domain_t priv_regdomain, priv_regdomain_a, priv_regdomain_b;
    tiap_param_channel_rang ch_rang;
    tiap_param_channel_rang_elem_t *ch_rang_a_g, range_elem;

    wone.index = currentConfigWoneInex;
    wone.type = wone_band;
    if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE || driver_load_failed) ) {
	    PRINT_CONFIG_ERR_MSG("Failed to get band.");
	    return;
    }

    printf("Band is: %s\n",(wone.elem.band == TIAP_PARAM_RADIO_BAND_2_4GHZ) ? 
		    "RADIO_BAND_2_4GHZ (G)" : "RADIO_BAND_5GHZ (A)");

    cur_regdomain.band = wone.elem.band;
    if( tiap_wcfg_get_cur_regdomain(wd, &cur_regdomain, !FROM_STORAGE || driver_load_failed) ) {
	    PRINT_CONFIG_ERR_MSG("Failed to get current regulatory domain.");
	    return;
    }

    printf("Current Regularity Domain is: ");

    // compute regularity domain 
    switch (wone.elem.band) 
    {
	    case TIAP_PARAM_RADIO_BAND_2_4GHZ: // g 
		    switch (cur_regdomain.type)
		    {
			    case TIAP_PARAM_RD_FCC:
				    printf("REG_DOMAIN_FCC (0x10)\n");
				    break;
			    case TIAP_PARAM_RD_IC:
				    printf("REG_DOMAIN_IC (0x20)\n");
				    break;
			    case TIAP_PARAM_RD_ETSI:
				    printf("REG_DOMAIN_ETSI (0x30)\n");
				    break;
			    case TIAP_PARAM_RD_SPAIN:
				    printf("TIAP_PARAM_RD_SPAIN (0x31)\n");
				    break;
			    case TIAP_PARAM_RD_FRANCE:
				    printf("TIAP_PARAM_RD_FRANCE (0x32)\n");
				    break;
			    case TIAP_PARAM_RD_MKK:
				    printf("TIAP_PARAM_RD_MKK (0x40)\n");
				    break;
			    case TIAP_PARAM_RD_MKK1:
				    printf("TIAP_PARAM_RD_MKK1 (0x41)\n");
				    break;
                case TIAP_PARAM_RD_ISRAEL:
                    printf("REG_DOMAIN_ISRAEL (0x43)\n");
                    break;
			    default :
				    printf("CUSTOM_REG_DOMAIN (0x%X)\n", cur_regdomain.type);
				    break;
		    }
		    break;

	    case TIAP_PARAM_RADIO_BAND_5GHZ:   // a 
		    switch (cur_regdomain.type)
		    {
			    case TIAP_PARAM_RD_US:
				    printf("TIAP_PARAM_RD_US (0x50)\n");
				    break;
			    case TIAP_PARAM_RD_WORLD:
				    printf("TIAP_PARAM_RD_WORLD (0x51)\n");
				    break;
			    default:
				    printf("CUSTOM_REG_DOMAIN (0x%X)\n", cur_regdomain.type);
				    break;
		    }
		    break;
    }

    printf("Type one of the following:\n");
    printf("Regulatory Domains for 802.11g:\n");
    printf("-------------------------------\n");
    printf("1. REG_DOMAIN_FCC    (0x10)\n");
    printf("2. REG_DOMAIN_IC     (0x20)\n");
    printf("3. REG_DOMAIN_ETSI   (0x30)\n");
    printf("4. REG_DOMAIN_SPAIN  (0x31)\n");
    printf("5. REG_DOMAIN_FRANCE (0x32)\n");
    printf("6. REG_DOMAIN_MKK    (0x40)\n");
    printf("7. REG_DOMAIN_MKK1   (0x41)\n");
    printf("8. REG_DOMAIN_ISRAEL (0x43)\n");

    priv_regdomain_b.band = BAND_B;
    if( tiap_wcfg_get_priv_regdomain(wd, &priv_regdomain_b, !FROM_STORAGE || driver_load_failed) ) {
	    PRINT_CONFIG_ERR_MSG("Failed to get current regulatory domain.");
	    return;
    }

    if (priv_regdomain_b.domain != 0)
	    printf("9. CUSTOM_REG_DOMAIN (0x%X)\n", priv_regdomain_b.domain);
    else
	    printf("Note: CUSTOM_REG_DOMAIN undefined (0x%X)\n", priv_regdomain_b.domain);
    printf("\nRegulatory Domains for 802.11a:\n");
    printf("-------------------------------\n");
    printf("10. REG_DOMAIN_US     (0x50)\n");
    printf("11. REG_DOMAIN_WORLD  (0x51)\n");

    priv_regdomain_a.band = BAND_A;
    if( tiap_wcfg_get_priv_regdomain(wd, &priv_regdomain_a, !FROM_STORAGE || driver_load_failed) ) {
	    PRINT_CONFIG_ERR_MSG("Failed to get current regulatory domain.");
	    return;
    }

    if (priv_regdomain_a.domain != 0)
	    printf("12. CUSTOM_REG_DOMAIN (0x%X)\n", priv_regdomain_a.domain);
    else
	    printf("Note: CUSTOM_REG_DOMAIN undefined (0x%X)\n", priv_regdomain_a.domain);


    printf("\nType index number:");
    fgets(tmpBuffer, 100, stdin);
    tmpBuffer[strlen(tmpBuffer) - 1] = 0; // remove newline 
    chosen = atoi(tmpBuffer);
    if(chosen <= 0)
	    return;

    if (chosen > 12) 
    {
	    printf("index not exists!\n");
	    return;
    }

    switch (chosen)
    {
	    case 1:
		    cur_regdomain.type = TIAP_PARAM_RD_FCC;    
		    band_type = BAND_B;
		    break;

	    case 2:
		    cur_regdomain.type = TIAP_PARAM_RD_IC;    
		    band_type = BAND_B;
		    break;

	    case 3:
		    cur_regdomain.type = TIAP_PARAM_RD_ETSI;    
		    band_type = BAND_B;
		    break;

	    case 4:
		    cur_regdomain.type = TIAP_PARAM_RD_SPAIN;    
		    band_type = BAND_B;
		    break;

	    case 5:
		    cur_regdomain.type = TIAP_PARAM_RD_FRANCE;    
		    band_type = BAND_B;
		    break;

	    case 6:
		    cur_regdomain.type = TIAP_PARAM_RD_MKK;    
		    band_type = BAND_B;
		    break;

	    case 7:
		    cur_regdomain.type = TIAP_PARAM_RD_MKK1;    
		    band_type = BAND_B;
		    break;

        case 8:
            cur_regdomain.type = TIAP_PARAM_RD_ISRAEL;    
            band_type = BAND_B;
            break;

	    case 9:
		    if (priv_regdomain_b.domain==0)
		    {
			    printf("Custom Regulatory Domain for Band G is undefined.\n");
			    return;
		    }
		    cur_regdomain.type = priv_regdomain_b.domain;
		    band_type = BAND_B;
		    break;

	    case 10:
		    cur_regdomain.type = TIAP_PARAM_RD_US;    
		    band_type = BAND_A;
		    break;

	    case 11:
		    cur_regdomain.type = TIAP_PARAM_RD_WORLD;    
		    band_type = BAND_A;
		    break;

	    case 12:
		    if (priv_regdomain_a.domain==0)
		    {
			    printf("Custom Regulatory Domain for Band A is undefined.\n");
			    return;
		    }
		    cur_regdomain.type = priv_regdomain_a.domain;
		    band_type = BAND_A;
		    break;
    }
    switch (chosen) 
    {
	    case 1:
	    case 2:
	    case 3:
	    case 4:
	    case 5:
	    case 6:
	    case 7:
        case 8:
	    case 10:
	    case 11:
		    num_ranges = get_domain_elems_num(band_type, cur_regdomain.type);
		    printf("Start Ch.    Stop Ch.    Step    OFDM    Power Limit\n");
		    printf("---------    --------    ----    ----    -----------\n");
		    for (index = 1; index <= num_ranges; index++) 
		    {
				get_rangeElement(band_type,cur_regdomain,index,&range_elem);
				printf("%d \t     %d \t %d \t %s \t %d\n",
					   range_elem.start,
					   range_elem.stop,
					   range_elem.step & 0xF,
					   (range_elem.step & 0x10) ? "No" : "Yes",
					   range_elem.power_limit);
			}
		    printf("\n");
		    break;

	    case 9:     // private "G" 
	    case 12:    // private "A" 
		    ch_rang.type = band_type;
		    if( tiap_wcfg_get_ch_range(wd, &ch_rang, !FROM_STORAGE || driver_load_failed) ) {
			    PRINT_CONFIG_ERR_MSG("Failed to get channel range.");
			    return;
		    }
		    
		    ch_rang_a_g = (ch_rang.type == TIAP_PARAM_RANG_A ?
				    ch_rang.rng.a : (ch_rang.type == TIAP_PARAM_RANG_G ? 
				    ch_rang.rng.g : NULL));
		    if( !ch_rang_a_g ) {
			    fprintf(stderr, "%s: invalid channel\n", __func__);
			    return;
		    }

		    if (!ch_rang.ranges_nr) 
		    {
			    printf("Not configured yet.\n");
			    return;
		    }
		    printf("Start Ch.    Stop Ch.    Step    OFDM    Power Limit\n");
		    printf("---------    --------    ----    ----    -----------\n");
		    for (index = 1; index <= ch_rang.ranges_nr; index++) 
		    {
			    printf("%d \t     %d \t %d \t %s \t %d\n",
					    ch_rang_a_g[index-1].start,
					    ch_rang_a_g[index-1].stop,
					    ch_rang_a_g[index-1].step & 0xF,
					    (ch_rang_a_g[index-1].step & 0x10) ? "No" : "Yes",
					    ch_rang_a_g[index-1].power_limit);
		    }
		    printf("\n");
		    break;
    }
}


/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSetMultiDomain(ConsoleParameter_T parameters[])
{
	__u8 multiDomainCap = (__u8)parameters[0].SHORT; 
	/*
	if ((parameters[0].SHORT != 0) &&
		(parameters[0].SHORT != 1))
	{
		printf("Invalid mode: %d. Only 0 and 1 are legal values.\n", parameters[0].SHORT);
		return;
	}
	wdrv_dbSetdot11MultiDomainCapabilityStatus(parameters[0].SHORT);
	printf("Setting dot11MultiDomainCapabilityEnable to: %s\n",(parameters[0].SHORT==1) ? "Enable":"Disable");
	printf("to take effect RESTART AP.\n");*/

	if ((parameters[0].SHORT != 0) && (parameters[0].SHORT != 1))
	{
		printf("Invalid mode: %d. Only 0 and 1 are legal values.\n", parameters[0].SHORT);
		return;
	}
	
	if( tiap_wcfg_set_basic(wd, TIAP_PARAM_MULTI_DOMAIN_CAPAB, &multiDomainCap, UPDATE_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to set mullti-domain capability.\n");
		return;
	}
		
	printf("Setting dot11MultiDomainCapabilityEnable to: %s\n",(parameters[0].SHORT==1) ? "Enable":"Disable");
	printf("to take effect RESTART AP.\n");
}


/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSetSpectrumMgmt(ConsoleParameter_T parameters[])
{/*
	if ((parameters[0].SHORT != 0) &&
		(parameters[0].SHORT != 1))
	{
		printf("Invalid mode: %d. Only 0 and 1 are legal values.\n", parameters[0].SHORT);
		return;
	}
	wdrv_dbSetdot11SpectrumManagementRequiredStatus(parameters[0].SHORT);
	printf("Setting dot11SpectrumManagementRequired to: %s\n",(parameters[0].SHORT==1) ? "Enable":"Disable");
	printf("to take effect RESTART AP.\n"); */
}


/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSetLegacyStation(ConsoleParameter_T parameters[])
{/*
	if ((parameters[0].SHORT != 0) &&
		(parameters[0].SHORT != 1))
	{
		printf("Invalid mode: %d. Only 0 and 1 are legal values.\n", parameters[0].SHORT);
		return;
	}
	wdrv_dbSetLegacyStatusEnableStatus(parameters[0].SHORT);
	printf("Setting whalEnableLegacyStations to: %s\n",(parameters[0].SHORT==1) ? "Enable":"Disable");
	printf("to take effect RESTART AP.\n"); */
}


/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSetPowerConstraint(ConsoleParameter_T parameters[])
{
	__s8 localPowerCons = (__s8)parameters[0].SHORT; 
	/*
	wdrv_dbSetdot11LocalPowerConstraint(parameters[0].SHORT);
	printf("Setting dot11LocalPowerConstraint to: %d\n",parameters[0].SHORT);
	printf("to take effect RESTART AP.\n"); */

	if( tiap_wcfg_set_basic(wd, TIAP_PARAM_LOCAL_POWER_CONS, &localPowerCons, UPDATE_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to set tx power constraint.\n");
		return;
	}
	
	printf("Setting dot11LocalPowerConstraint to: %d\n",parameters[0].SHORT);
	printf("to take effect RESTART AP.\n");
}


/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSetRegulatoryDomain(ConsoleParameter_T parameters[])
{/*
    char tmpBuffer[100];
    int chosen, maxChosen;
    UINT8 regDomain=0, band;
    wlan_bandType bandType;

    // read current band 
    whal_nvram_getBand(currentConfigWoneInex, &band);
    switch (band) 
    {
        case RADIO_BAND_2_4GHZ: // g 
            printf("Type one of the following:\n");
            printf("1. REG_DOMAIN_FCC    (0x10)\n");
            printf("2. REG_DOMAIN_IC     (0x20)\n");
            printf("3. REG_DOMAIN_ETSI   (0x30)\n");
            printf("4. REG_DOMAIN_SPAIN  (0x31)\n");
            printf("5. REG_DOMAIN_FRANCE (0x32)\n");
            printf("6. REG_DOMAIN_MKK    (0x40)\n");
            printf("7. REG_DOMAIN_MKK1   (0x41)\n");
            whal_nvram_getPrivateRegDomain(&regDomain);
            if (regDomain != 0)
            {
                printf("8. CUSTOM_REG_DOMAIN (0x%X)\n",regDomain);
                maxChosen = 8;
            }
            else
            {
                printf("Note: CUSTOM_REG_DOMAIN undefined\n",regDomain);
                maxChosen = 7;
            }
            break;
    
        case RADIO_BAND_5GHZ:   // a 
            printf("1. REG_DOMAIN_US     (0x50)\n");
            printf("2. REG_DOMAIN_WORLD  (0x51)\n");
            whal_nvram_getPrivateRegDomain(&regDomain);
            if (regDomain != 0)
            {
                printf("3. PRIVATE_REG_DOMAIN (0x%X)\n",regDomain);
                maxChosen = 3;
            }
            else
            {
                printf("Note: CUSTOM_REG_DOMAIN undefined\n",regDomain);
                maxChosen = 2;
            }
            break;
    }

    printf("\nType index number:");
    fgets(tmpBuffer, 100, stdin);
    tmpBuffer[strlen(tmpBuffer) - 1] = 0; // remove newline 
    chosen = atoi(tmpBuffer);
    if(chosen <= 0)
        return;
    
    if (chosen > maxChosen) 
    {
        printf("index not exists!\n");
        return;
    }

    switch (band) 
    {
        case RADIO_BAND_2_4GHZ: // g 
            bandType = WLAN_BAND_B;
            switch (chosen)
            {
                case 1:
                    regDomain = REG_DOMAIN_FCC;    
                    break;

                case 2:
                    regDomain = REG_DOMAIN_IC;    
                    break;

                case 3:
                    regDomain = REG_DOMAIN_ETSI;    
                    break;

                case 4:
                    regDomain = REG_DOMAIN_SPAIN;    
                    break;

                case 5:
                    regDomain = REG_DOMAIN_FRANCE;    
                    break;

                case 6:
                    regDomain = REG_DOMAIN_MKK;    
                    break;

                case 7:
                    regDomain = REG_DOMAIN_MKK1;    
                    break;
                
                default: // 8  - regDomain already set 
                    break;
            }
            break;

        case RADIO_BAND_5GHZ:   // a
            bandType = WLAN_BAND_A;
            switch (chosen)
            {
                case 1:
                    regDomain = REG_DOMAIN_US;    
                    break;

                case 2:
                    regDomain = REG_DOMAIN_WORLD;    
                    break;

                default: // 3  - regDomain already set 
                    break;
            }
            break;
    }

	if (wdrv_dbSetdot11CurrentRegDomain(regDomain) != OK)
	{
		printf("Error setting new regulatory domain into wdrv_dbSetdot11CurrentRegDomain.\n");
		return;
	}
	printf("to take effect RESTART AP.\n"); */


	char tmpBuffer[100];
	int chosen, maxChosen=0;
	tiap_param_wone_t wone;
	tiap_param_priv_reg_domain_t priv_regdomain;
	tiap_param_cur_reg_domain_t cur_regdomain;

	wone.index = currentConfigWoneInex;
	wone.type = wone_band;
	// read current band 
	if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE || driver_load_failed) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get band.");
		return;
	}

	switch (wone.elem.band) 
	{
		case TIAP_PARAM_RADIO_BAND_2_4GHZ: // g 
			printf("Type one of the following:\n");
			printf("1. REG_DOMAIN_FCC    (0x10)\n");
			printf("2. REG_DOMAIN_IC     (0x20)\n");
			printf("3. REG_DOMAIN_ETSI   (0x30)\n");
			printf("4. REG_DOMAIN_SPAIN  (0x31)\n");
			printf("5. REG_DOMAIN_FRANCE (0x32)\n");
			printf("6. REG_DOMAIN_MKK    (0x40)\n");
			printf("7. REG_DOMAIN_MKK1   (0x41)\n");
            printf("8. REG_DOMAIN_ISRAEL (0x43)\n");
			
			priv_regdomain.band = BAND_B;
			if( tiap_wcfg_get_priv_regdomain(wd, &priv_regdomain, !FROM_STORAGE || driver_load_failed) ) {
				PRINT_CONFIG_ERR_MSG("Failed to get private regulatory domain.");
				return;
			}
			
			if (priv_regdomain.domain != 0)
			{
				printf("9. CUSTOM_REG_DOMAIN (0x%X)\n", priv_regdomain.domain);
				maxChosen = 9;
			}
			else
			{
				printf("Note: CUSTOM_REG_DOMAIN undefined (0x%X)\n", priv_regdomain.domain);
				maxChosen = 8;
			}
			break;

		case TIAP_PARAM_RADIO_BAND_5GHZ:   // a 
			printf("1. REG_DOMAIN_US     (0x50)\n");
			printf("2. REG_DOMAIN_WORLD  (0x51)\n");

			priv_regdomain.band = BAND_A;
			if( tiap_wcfg_get_priv_regdomain(wd, &priv_regdomain, !FROM_STORAGE || driver_load_failed) ) {
				PRINT_CONFIG_ERR_MSG("Failed to get private regulatory domain.");
				return;
			}

			if (priv_regdomain.domain != 0)
			{
				printf("3. PRIVATE_REG_DOMAIN (0x%X)\n", priv_regdomain.domain);
				maxChosen = 3;
			}
			else
			{
				printf("Note: CUSTOM_REG_DOMAIN undefined (0x%X)\n", priv_regdomain.domain);
				maxChosen = 2;
			}
			break;
	}

	printf("\nType index number:");
	fgets(tmpBuffer, 100, stdin);
	tmpBuffer[strlen(tmpBuffer) - 1] = 0; // remove newline 
	chosen = atoi(tmpBuffer);
	if(chosen <= 0)
		return;

	if (chosen > maxChosen) 
	{
		printf("index not exists!\n");
		return;
	}

	/* Felix R.: exactly as in original code, 
	   though I think we should have informed user in case of invalid input. */
	switch (wone.elem.band) 
	{
		case TIAP_PARAM_RADIO_BAND_2_4GHZ: // g 
			priv_regdomain.band = BAND_B;
			switch (chosen)
			{
				case 1:
					priv_regdomain.domain = TIAP_PARAM_RD_FCC;
					break;

				case 2:
					priv_regdomain.domain = TIAP_PARAM_RD_IC;
					break;

				case 3:
					priv_regdomain.domain = TIAP_PARAM_RD_ETSI;
					break;

				case 4:
					priv_regdomain.domain = TIAP_PARAM_RD_SPAIN;
					break;

				case 5:
					priv_regdomain.domain = TIAP_PARAM_RD_FRANCE;
					break;

				case 6:
					priv_regdomain.domain = TIAP_PARAM_RD_MKK;
					break;

				case 7:
					priv_regdomain.domain = TIAP_PARAM_RD_MKK1;
					break;

                case 8:
                    priv_regdomain.domain = TIAP_PARAM_RD_ISRAEL;
                    break;

				default: // 9  - regDomain already set 
					break;
			}
			break;

		case TIAP_PARAM_RADIO_BAND_5GHZ:   // a
			priv_regdomain.band = BAND_A;
			switch (chosen)
			{
				case 1:
					priv_regdomain.domain = TIAP_PARAM_RD_US;
					break;

			case 2:
					priv_regdomain.domain = TIAP_PARAM_RD_WORLD;
					break;

				default: // 3  - regDomain already set 
					break;
			}
			break;
	}

	cur_regdomain.type = priv_regdomain.domain;
	cur_regdomain.band = priv_regdomain.band;  
	
	//if( tiap_wcfg_set_priv_regdomain(wd, &priv_regdomain) ) {
	if( tiap_wcfg_set_cur_regdomain(wd, &cur_regdomain, UPDATE_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to set current regulatory domain.");
		return;
	}
	
	printf("to take effect RESTART AP.\n"); 
}


/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSetTxPowerLevel(ConsoleParameter_T parameters[])
{/*
    if ((parameters[0].SHORT < 1) || (parameters[0].SHORT > 5))
    {
        printf("Out of range.\n");
        return;
    }
	wdrv_dbSetMaxTxPowerLevel((UINT8)parameters[0].SHORT);
	printf("to take effect RESTART AP.\n"); */

	__u8 maxTxPower = (__u8)parameters[0].SHORT;

	if ((parameters[0].SHORT < 1) || (parameters[0].SHORT > 5))
	{
		printf("Out of range.\n");
		return;
	}

	if( tiap_wcfg_set_basic(wd, TIAP_PARAM_MAX_TX_POWER_LEVEL, &maxTxPower, UPDATE_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to set tx power level.\n");
		return;
	}
	
	printf("to take effect RESTART AP.\n"); 
}


/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSetPowerLevel(ConsoleParameter_T parameters[])
{/*
    // Check band 
    if ((parameters[0].INTEGER < 0) || (parameters[0].INTEGER > 1)) 
    {
        printf("Band is out of range. Use: 1 for band A (5 Ghz) and 0 for G (2.4 Ghz).\n");
        return;
    }
    // Check range 
    if ((parameters[5].INTEGER < -127) || (parameters[1].INTEGER > 128)) 
    {
        printf("Parameters are out of range.\n");
        return;
    }
    // Check that values are given in incrementally order 
    if (parameters[2].INTEGER >= parameters[1].INTEGER) 
    {
        printf("Invalid value! level 1 (%d) Must be greater than level 2 (%d)\n",
               parameters[0].SHORT,
               parameters[1].SHORT);
        return;
    }
    if (parameters[3].INTEGER >= parameters[2].INTEGER) 
    {
        printf("Invalid value! level 2 (%d) Must be greater than level 3 (%d)\n",
               parameters[1].SHORT,
               parameters[2].SHORT);
        return;
    }
    if (parameters[4].INTEGER >= parameters[3].INTEGER) 
    {
        printf("Invalid value! level 3 (%d) Must be greater than level 4 (%d)\n",
               parameters[2].SHORT,
               parameters[3].SHORT);
        return;
    }
    if (parameters[5].INTEGER >= parameters[4].INTEGER) 
    {
        printf("Invalid value! level 4 (%d) Must be greater than level 5 (%d)\n",
               parameters[3].INTEGER,
               parameters[4].INTEGER);
        return;
    }
	if (wdrv_dbSetTxPowerLevels((INT8)parameters[0].INTEGER,
                                (INT8)parameters[1].INTEGER,
                                (INT8)parameters[2].INTEGER,
                                (INT8)parameters[3].INTEGER,
                                (INT8)parameters[4].INTEGER,
                                (INT8)parameters[5].INTEGER) == ERROR)
    {
        printf("Error setting new power levels!\n");
        return;
    }
	printf("to take effect RESTART AP.\n"); */

	tiap_param_txpower_t txpow;
	
	// Check band 
	if ((parameters[0].INTEGER < 0) || (parameters[0].INTEGER > 1)) 
	{
		printf("Band is out of range. Use: 1 for band A (5 Ghz) and 0 for G (2.4 Ghz).\n");
		return;
	}
	// Check range 
	if ((parameters[5].INTEGER < -127) || (parameters[1].INTEGER > 128)) 
	{
		printf("Parameters are out of range.\n");
		return;
	}
	// Check that values are given in incrementally order 
	if (parameters[2].INTEGER >= parameters[1].INTEGER) 
	{
		printf("Invalid value! level 1 (%ld) Must be greater than level 2 (%ld)\n",
				parameters[0].INTEGER,
				parameters[1].INTEGER);
		return;
	}
	if (parameters[3].INTEGER >= parameters[2].INTEGER) 
	{
		printf("Invalid value! level 2 (%ld) Must be greater than level 3 (%ld)\n",
				parameters[1].INTEGER,
				parameters[2].INTEGER);
		return;
	}
	if (parameters[4].INTEGER >= parameters[3].INTEGER) 
	{
		printf("Invalid value! level 3 (%ld) Must be greater than level 4 (%ld)\n",
				parameters[2].INTEGER,
				parameters[3].INTEGER);
		return;
	}
	if (parameters[5].INTEGER >= parameters[4].INTEGER) 
	{
		printf("Invalid value! level 4 (%ld) Must be greater than level 5 (%ld)\n",
				parameters[3].INTEGER,
				parameters[4].INTEGER);
		return;
	}

	txpow.hr = parameters[0].INTEGER;
	txpow.txpower[0] = parameters[1].INTEGER;
	txpow.txpower[1] = parameters[2].INTEGER;
	txpow.txpower[2] = parameters[3].INTEGER;
	txpow.txpower[3] = parameters[4].INTEGER;
	txpow.txpower[4] = parameters[5].INTEGER;

	if( tiap_wcfg_siwtxpow(wd, &txpow, UPDATE_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to set tx powers.");
		return;
	}
    
	printf("to take effect RESTART AP.\n"); 
}


/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSetCountryString(ConsoleParameter_T parameters[])
{/*
    char tmpBuffer[100], countryString[3];
    int chosen;

	if (strlen(parameters[0].STRING) != 2)
	{
		printf("Invalid length: %d. Must be 2.\n", strlen(parameters[0].STRING));
		return;
	}

    printf("Current Type is: ");
    wdrv_dbGetdot11CountryString(countryString);
    if (countryString[2] == 'I') 
        printf("'I' (Indoor)\n");
    else if (countryString[2] == 'O') 
            printf("'O' (Outdoor)\n");
    else
        printf("(Other - not Indoor nor Outdoor)\n\n");

    printf("Type new type no. or hit <Enter> to keep the same type:\n");
    printf("1. Indoor\n");
    printf("2. Outdoor\n");
    printf("3. Other\n");
    fgets(tmpBuffer, 100, stdin);
    tmpBuffer[strlen(tmpBuffer) - 1] = 0; // remove newline 
    chosen = atoi(tmpBuffer);
    switch (chosen) 
    {
        case 0:
            break;
            
        case 1:
            countryString[2] = 'I';
            break;
            
        case 2:
            countryString[2] = 'O';
            break;
            
        case 3:
            countryString[2] = ' ';
            break;

        default:
            printf("Invalid value. Command aborted\n");
            return;
    }
    countryString[0] = parameters[0].STRING[0];
    countryString[1] = parameters[0].STRING[1];
	wdrv_dbSetdot11CountryString(countryString);
    printf("Setting Country String to: \"%c%c%c\"\n",countryString[0],countryString[1],countryString[2]);
	printf("to take effect RESTART AP.\n"); */


	char tmpBuffer[100];
	int chosen;
	__s8 country_str[TIAP_PARAM_COUNTRY_STRING_LEN];

	if (strlen(parameters[0].STRING) != 2)
	{
		printf("Invalid length: %d. Must be 2.\n", strlen(parameters[0].STRING));
		return;
	}

	printf("Current Type is: ");
	
	if( tiap_wcfg_get_basic(wd, TIAP_PARAM_COUNTRY_STR, country_str, !FROM_STORAGE || driver_load_failed) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get country string.");
		return;
	}
	
	if (country_str[2] == 'I') 
		printf("'I' (Indoor)\n");
	else if (country_str[2] == 'O') 
		printf("'O' (Outdoor)\n");
	else
		printf("(Other - not Indoor nor Outdoor)\n\n");

	printf("Type new type no. or hit <Enter> to keep the same type:\n");
	printf("1. Indoor\n");
	printf("2. Outdoor\n");
	printf("3. Other\n");
	fgets(tmpBuffer, 100, stdin);
	tmpBuffer[strlen(tmpBuffer) - 1] = 0; // remove newline 
	chosen = atoi(tmpBuffer);
	switch (chosen) 
	{
		case 0:
			break;

		case 1:
			country_str[2] = 'I';
			break;

		case 2:
			country_str[2] = 'O';
			break;

		case 3:
			country_str[2] = ' ';
			break;

		default:
			printf("Invalid value. Command aborted\n");
			return;
	}
	country_str[0] = parameters[0].STRING[0];
	country_str[1] = parameters[0].STRING[1];
	
	if( tiap_wcfg_set_basic(wd, TIAP_PARAM_COUNTRY_STR, country_str, UPDATE_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to set country string.");
		return;
	}

	printf("Setting Country String to: \"%c%c%c\"\n", country_str[0], country_str[1], country_str[2]);
	printf("to take effect RESTART AP.\n"); 
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanAddChnRangToCustomRegDomain(ConsoleParameter_T parameters[])
{
    tiap_param_priv_reg_domain_t priv_regdomain;
    tiap_param_channel_rang      ch_rang;
    UINT8                        start, numberOfChannels;
    int                          txPowerLevel;

    start               = (UINT8)parameters[0].INTEGER;
    numberOfChannels    = (UINT8)parameters[1].INTEGER;
    txPowerLevel        = parameters[2].INTEGER;

    if( (start > 14) || ((start + numberOfChannels - 1) > 14) )
    {
        PRINT_CONFIG_ERR_MSG("Failed to Set Channel Range. channel are out of range.");
        return;
    }

    priv_regdomain.band = BAND_B;
    if( tiap_wcfg_get_priv_regdomain(wd, &priv_regdomain, FROM_STORAGE) ) 
    {
        PRINT_CONFIG_ERR_MSG("Failed to get private regulatory domain.");
        return;
    }

    if(priv_regdomain.domain != TIAP_PARAM_RD_CUSTOM)
    {
        printf("Setting Private Reg Doamin\n");
        priv_regdomain.band = BAND_B;
        priv_regdomain.domain = TIAP_PARAM_RD_CUSTOM;
        if( tiap_wcfg_set_priv_regdomain(wd, &priv_regdomain, UPDATE_STORAGE) )
        {
            PRINT_CONFIG_ERR_MSG("Failed to add a customize Reg Domain.");
            return;
        }
    }

    ch_rang.type = TIAP_PARAM_RANG_G;
    if( tiap_wcfg_get_ch_range(wd, &ch_rang, FROM_STORAGE) ) 
    {
        PRINT_CONFIG_ERR_MSG("Failed to get channel range.");
        return;
    }
    
    ch_rang.rng.g[ch_rang.ranges_nr].start = start;
    ch_rang.rng.g[ch_rang.ranges_nr].step = 1;
    ch_rang.rng.g[ch_rang.ranges_nr].stop = start + numberOfChannels - 1;
    ch_rang.rng.g[ch_rang.ranges_nr].power_limit = txPowerLevel;
    ch_rang.ranges_nr += 1;
    if( tiap_wcfg_set_ch_range(wd, &ch_rang, UPDATE_STORAGE) )
    {
        PRINT_CONFIG_ERR_MSG("Failed to add a channel Range to the customize Reg Domain.");
        return;
    }
    
    printf("to take effect RESTART AP.\n"); 
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanMakeRange(ConsoleParameter_T parameters[])
{/*
    wlan_bandType bandType;
    UINT8 numOfRanges;
    char tmpBuffer[100], hexBuffer[20];
    int chosen, RegDomain;

	if (strlen(parameters[0].STRING) != 1)  // only 'a' and 'g' are valid bands 
	{
		printf("Invalid value: %s. Must be either 'a' or 'g'.\n", strlen(parameters[0].STRING));
		return;
	}
    switch (parameters[0].STRING[0]) 
    {
        case 'a':
        case 'A':
            bandType = WLAN_BAND_A;
            break;
        case 'g':
        case 'G':
            bandType = WLAN_BAND_B;
            break;
        default:
            printf("Invalid band. Must be either 'a' or 'g'.\n");
            return;
    }
    // analyzes RegDomain 
	if (strlen(parameters[1].STRING) != 2)
	{
		printf("Invalid value: %s. Must be an Hexa value (e.g. for 0xA0 type A0.\n", parameters[1].STRING);
		return;
	}
    if ((isxdigit(parameters[1].STRING[0]) == 0) || (isxdigit(parameters[1].STRING[1]) == 0)) 
    {
        printf("Bad Hex value. (Example: for 0xA0 tyoe A0)\n");
        return;
    }
    sprintf(hexBuffer,"0x%c%c",parameters[1].STRING[0],parameters[1].STRING[1]);
    sscanf(hexBuffer,"%x",&RegDomain);
    printf("RegDomain is 0x%x\n",RegDomain);
    if (wdrv_dbIsStdRegDomain((UINT8)RegDomain)) 
    {
        printf("Error - This Regulatory Domain is already defined as standard.\n");
        return;
    }

    printf("Type number of ranges (1-");
    if (bandType==WLAN_BAND_A)
        printf("%d) <default=%d> ",8,1);
    else
        printf("%d) <default=%d> ",4,1);

    fgets(tmpBuffer, 100, stdin);
    tmpBuffer[strlen(tmpBuffer) - 1] = 0; // remove newline 
    chosen = atoi(tmpBuffer);
    if(chosen < 0)
        return;
    if (chosen == 0)
    {
        numOfRanges = 1;
    }
    else
        numOfRanges = (UINT8)chosen;
    if (bandType==WLAN_BAND_A) 
    {
        if (numOfRanges > 8)
        {
            printf("Invalid range. Must be up to 8.\n");
            return;
        }
    }
    else    // G BAND 
        if (numOfRanges > 4)
        {
            printf("Invalid range. Must be up to 4.\n");
            return;
        }
    wdrv_dbSetPrivateRegulatoryDomain(RegDomain);
    if (wdrv_dbSetNumOfPrivateRanges(bandType, numOfRanges) != OK)
    {
        printf("Error setting new range to the given band.\n");
        return;
    }
	printf("to take effect RESTART AP.\n"); */


}


/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanChangeRange(ConsoleParameter_T parameters[])
{/*
    wlan_bandType bandType;
    UINT8 numOfRanges, index;
    char tmpBuffer[100];
    int chosen, startCh, stopCh;
    tChannelRangeElementStruct rangeElement;

	if (strlen(parameters[0].STRING) != 1)
	{
		printf("Invalid value: %s. Must be either 'a' or 'g'.\n", strlen(parameters[0].STRING));
		return;
	}
    switch (parameters[0].STRING[0]) 
    {
        case 'a':
        case 'A':
            bandType = WLAN_BAND_A;
            wdrv_dbGetNumOfPrivateRanges(bandType, &numOfRanges);
            break;
        case 'g':
        case 'G':
            bandType = WLAN_BAND_B;
            wdrv_dbGetNumOfPrivateRanges(bandType, &numOfRanges);
            break;
        default:
            printf("Invalid band. Must be either 'a' or 'g'.\n");
            return;
    }
    index = parameters[1].SHORT;
    if (index > numOfRanges) 
    {
        printf("Invalid index. Must be no greater than %d.\n",numOfRanges);
        return;
    }
    startCh = parameters[2].SHORT;
    stopCh  = parameters[3].SHORT;
    if (stopCh < startCh) 
    {
        printf("Invalid values! startCh is greater than stopCh.\n");
        return;
    }
    //verify that earlier range is lower than the current one 
    if (index>1) 
    {
        whal_nvram_getRangeElement4RegDomain(bandType, index, &rangeElement);
        if (startCh <= rangeElement.stopCh) 
        {
            printf("Invalid values! earlier range is greater.\n");
            return;
        }
    }
    rangeElement.startCh = startCh;
    rangeElement.stopCh  = stopCh;
    // check optional values 
    printf("Enter step between channels <default=%d>\n",(bandType==WLAN_BAND_A) ? 4 : 1);
    fgets(tmpBuffer, 100, stdin);
    tmpBuffer[strlen(tmpBuffer) - 1] = 0; // remove newline 
    chosen = atoi(tmpBuffer);
    if(chosen < 0)
        return;
    if (chosen == 0)
        rangeElement.stepOpt = (bandType==WLAN_BAND_A) ? 4 : 1;
    else if (chosen > 15) 
    {
        printf("Invalid step. Must be lower than 16.\n");
        return;
    }
    else
        rangeElement.stepOpt = chosen;

    printf("Enter Power limit <default=20>\n");
    fgets(tmpBuffer, 100, stdin);
    tmpBuffer[strlen(tmpBuffer) - 1] = 0; // remove newline 
    chosen = atoi(tmpBuffer);
    if(chosen < 0)
        return;
    if (chosen == 0)
        rangeElement.powerLimit = 20;
    else
        rangeElement.powerLimit = chosen;


    if (whal_nvram_setRangeElement4RegDomain(bandType, index, rangeElement, IMMEDIATELY) != OK)
    {
        printf("Error setting values to the given range.\n");
        return;
    }
	printf("to take effect RESTART AP.\n"); */

}


/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanChangeRangeOption(ConsoleParameter_T parameters[])
{ /*
    wlan_bandType bandType;
    UINT8 numOfRanges, index, option;
    char tmpBuffer[100];
    tChannelRangeElementStruct rangeElement;

	if (strlen(parameters[0].STRING) != 1)
	{
		printf("Invalid value: %s. Must be either 'a' or 'g'.\n", strlen(parameters[0].STRING));
		return;
	}
    switch (parameters[0].STRING[0]) 
    {
        case 'a':
        case 'A':
            bandType = WLAN_BAND_A;
            wdrv_dbGetNumOfPrivateRanges(bandType, &numOfRanges);
            break;
        case 'g':
        case 'G':
            bandType = WLAN_BAND_B;
            wdrv_dbGetNumOfPrivateRanges(bandType, &numOfRanges);
            break;
        default:
            printf("Invalid band. Must be either 'a' or 'g'.\n");
            return;
    }
    index = parameters[1].SHORT;
    if (index > numOfRanges) 
    {
        printf("Invalid index. Must be no greater than %d.\n",numOfRanges);
        return;
    }
    option = parameters[2].SHORT;
    if (option > 1) 
    {
        printf("Invalid values! <opt> should be either 0 (OFDM) or 1 (NO_OFDM).\n");
        return;
    }
    option <<= 4; // set bit 4 if option is 1 
    whal_nvram_getRangeElement4RegDomain(bandType, index, &rangeElement);
    rangeElement.stepOpt = (rangeElement.stepOpt & 0xF) | option;
    if (whal_nvram_setRangeElement4RegDomain(bandType, index, rangeElement, IMMEDIATELY) != OK)
    {
        printf("Error setting values to the given range.\n");
        return;
    }
	printf("to take effect RESTART AP.\n"); */

}

/****************************************************/

/*!	\brief 
  	\param 

  	\return 
  
*/
static UINT8 setOFDMrates(UINT8 *curRates, UINT8 curRatesLen, UINT8 *newRates)
{/*
	UINT8 i, newRatesLen=0;
	BOOL quit;
	char str[4];
	char inbuf[255];
	char rateString[OFDM_RATES_MAX][3] =
				{ " 6", " 9", "12", "18", "24", "36", "48", "54" };
	UINT8 rateValue[OFDM_RATES_MAX] =
				{ DOT11_RATE_6,  DOT11_RATE_9,
				  DOT11_RATE_12, DOT11_RATE_18,
				  DOT11_RATE_24, DOT11_RATE_36,
				  DOT11_RATE_48, DOT11_RATE_54 };

	printf ("\nSet Rate List\n");
	printf ("[ Y ] = Supported\n[ N ] = Not Supported\n[ B ] = Basic Supported\n\n");

	for (i=0; i<OFDM_RATES_MAX; i++)
	{
		isRateInList(curRates, curRatesLen, rateValue[i], str);
		
		quit = FALSE;
		while (!quit)
		{
			printf("Rate %s Mbps - [%s] > ", rateString[i], str);
			gets(inbuf);

			// if enter was pressed, use previous value 
			if (strlen(inbuf) == 0)
			{
				strcpy(inbuf, str);
			}
		
			if ((inbuf[0] == 'Y') || (inbuf[0] == 'y'))
			{
				newRates[newRatesLen++] = rateValue[i];
				quit = TRUE;
			}
			else if ((inbuf[0] == 'B') || (inbuf[0] == 'b'))
			{
				newRates[newRatesLen++] = (rateValue[i] | 0x80);
				quit = TRUE;
			}
			else if ((inbuf[0] == 'N') || (inbuf[0] == 'n'))
			{
				// do not add rate ! 
				quit = TRUE;
			}
			// else illegal input was pressed, try again 
		}
	}

	if (newRatesLen == 0)
	{
		printf("Setting Rates = None"); 
	}
	else
	{
		printf("Setting Rates = ");
		for (i=0; i<newRatesLen; i++)
		{
			printf("  "); 
			printRateInMbpsUnit(newRates[i]);
		}
	}
	printf("\nWait ... ");

	return newRatesLen; */



	UINT8 i, newRatesLen=0;
	BOOL quit;
	char str[4];
	char inbuf[255];
	char rateString[TIAP_PARAM_OFDM_RATES_MAX][3] =
				{ " 6", " 9", "12", "18", "24", "36", "48", "54" };
	UINT8 rateValue[TIAP_PARAM_OFDM_RATES_MAX] =
				{ DOT11_RATE_6,  DOT11_RATE_9,
				  DOT11_RATE_12, DOT11_RATE_18,
				  DOT11_RATE_24, DOT11_RATE_36,
				  DOT11_RATE_48, DOT11_RATE_54 };

	printf ("\nSet Rate List\n");
	printf ("[ Y ] = Supported\n[ N ] = Not Supported\n[ B ] = Basic Supported\n\n");

	for (i=0; i<TIAP_PARAM_OFDM_RATES_MAX; i++)
	{
		isRateInList(curRates, curRatesLen, rateValue[i], str);
		
		quit = FALSE;
		while (!quit)
		{
			printf("Rate %s Mbps - [%s] > ", rateString[i], str);
			fflush(stdout);
			read(0, inbuf, sizeof(inbuf)-1);
					
			// if enter was pressed, use previous value 
			if (strlen(inbuf) == 0)
			{
				strcpy(inbuf, str);
			}
		
			if ((inbuf[0] == 'Y') || (inbuf[0] == 'y'))
			{
				newRates[newRatesLen++] = TIAP_PARAM_RATE_SUPPORTED;
				quit = TRUE;
			}
			else if ((inbuf[0] == 'B') || (inbuf[0] == 'b'))
			{
				newRates[newRatesLen++] = TIAP_PARAM_RATE_BASIC_SUPPORTED;
				quit = TRUE;
			}
			else if ((inbuf[0] == 'N') || (inbuf[0] == 'n'))
			{
				// do not add rate !
				newRates[newRatesLen++] = TIAP_PARAM_RATE_NOT_SUPPORTED;
				quit = TRUE;
			}
			// else illegal input was pressed, try again 
		}
	}

	if (newRatesLen == 0)
	{
		printf("Setting Rates = None"); 
	}
	else
	{
		printf("Setting Rates = ");
/*		for (i=0; i<newRatesLen; i++)
		{
			printf("  "); */
		printOFDMRateInMbpsUnit(newRates);
/*
		}
*/
	}
	printf("\nWait ... ");

	return newRatesLen; 
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSetGRates(ConsoleParameter_T parameters[])
{/*
	UINT8 curRates[OFDM_RATES_MAX];
	UINT8 curRatesLen=0;
	UINT8 newRates[OFDM_RATES_MAX];
	UINT8 newRatesLen;

	UINT8 band;
	wdrv_dbGetBand(currentConfigWoneInex,&band);
    if (band == RADIO_BAND_5GHZ)
        {
            printf("\nError Active Set Wone Index is %d\n",currentConfigWoneInex);  
            printf("802.11 A Not support CCK Rates\n");  
            return;
        }

	// Read current rates setting 
	whal_nvram_getOperationalOFDMRateSetLength(currentConfigWoneInex,&curRatesLen);
	whal_nvram_getOperationalOFDMRateSet(currentConfigWoneInex,curRates, curRatesLen);

    printf("\n---------------------\n");
    printf("-  Wone Index is %d  -\n",currentConfigWoneInex);
    printf("----------------------\n");
	newRatesLen = setOFDMrates(curRates, curRatesLen, newRates);

	wdrv_dbSetStationConfigOperationalOFDMRateSet(currentConfigWoneInex,newRates, newRatesLen);

	printf("done, to take effect RESTART AP.\n"); */


	UINT8 curRates[TIAP_PARAM_OFDM_RATES_MAX];
	UINT8 newRatesLen;
	__u8 ofdmRateValue[TIAP_PARAM_OFDM_RATES_MAX] =
		{DOT11_RATE_6,DOT11_RATE_9,DOT11_RATE_12,DOT11_RATE_18,DOT11_RATE_24,
		DOT11_RATE_36,DOT11_RATE_48,DOT11_RATE_54 };
	tiap_param_wone_t wone;

	wone.index = currentConfigWoneInex;
	wone.type = wone_band;
	if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE || driver_load_failed) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get band.");
		return;
	}


	if (wone.elem.band == TIAP_PARAM_RADIO_BAND_5GHZ)
	{
            printf("\nError Active Set Wone Index is %d\n",currentConfigWoneInex);  
            printf("802.11 A Not support CCK Rates\n");  
            return;
        }

	// Read current rates setting 
	wone.index = currentConfigWoneInex;
	wone.type = wone_oper_ofdm;
	if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE || driver_load_failed) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get band.");
		return;
	}
	
	printf("\n---------------------\n");
	printf("-  Wone Index is %d  -\n",currentConfigWoneInex);
	printf("----------------------\n");
	
	convert_rates(ofdmRateValue, curRates, wone.elem.ofdm_extend.rates, TIAP_PARAM_OFDM_RATES_MAX);
	newRatesLen = setOFDMrates(curRates, wone.elem.ofdm_extend.len, wone.elem.ofdm_extend.rates);
	wone.elem.ofdm_extend.len = newRatesLen;
	
	tiap_wcfg_set_wone(wd, &wone, UPDATE_STORAGE);

	printf("done, to take effect RESTART AP.\n"); 
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSetAdditionalRates(ConsoleParameter_T parameters[])
{/*
	UINT8 curRates[OFDM_RATES_MAX];
	UINT8 curRatesLen=0;
	UINT8 newRates[OFDM_RATES_MAX];
	UINT8 newRatesLen;
	UINT8 band;


	// Read current rates setting 
	whal_nvram_getOperationalAdditionalRateSetLength(currentConfigWoneInex,&curRatesLen);
	whal_nvram_getOperationalAdditionalRateSet(currentConfigWoneInex,curRates, curRatesLen);

    printf("\n---------------------\n");
    printf("-  Wone Index is %d  -\n",currentConfigWoneInex);
    printf("----------------------\n");

	newRatesLen = setOFDMrates(curRates, curRatesLen, newRates);

	wdrv_dbSetStationConfigOperationalAdditionalRateSet(currentConfigWoneInex,newRates, newRatesLen);

	printf("done, to take effect RESTART AP.\n"); */


	UINT8 curRates[TIAP_PARAM_OFDM_RATES_MAX];
	UINT8 newRatesLen;
	__u8 ofdmRateValue[TIAP_PARAM_OFDM_RATES_MAX] =
		{DOT11_RATE_6,DOT11_RATE_9,DOT11_RATE_12,DOT11_RATE_18,DOT11_RATE_24,
		DOT11_RATE_36,DOT11_RATE_48,DOT11_RATE_54 };
	tiap_param_wone_t wone;

	// Read current rates setting 
	wone.index = currentConfigWoneInex;
	wone.type = wone_oper_add;
	if( tiap_wcfg_get_wone(wd, &wone, FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get band.");
		return;
	}

	printf("\n---------------------\n");
	printf("-  Wone Index is %d  -\n",currentConfigWoneInex);
	printf("----------------------\n");

	convert_rates(ofdmRateValue, curRates, wone.elem.ofdm_legacy.rates, TIAP_PARAM_OFDM_RATES_MAX);
	newRatesLen = setOFDMrates(curRates, wone.elem.ofdm_legacy.len, wone.elem.ofdm_legacy.rates);
	wone.elem.ofdm_legacy.len = newRatesLen;

	tiap_wcfg_set_wone(wd, &wone, UPDATE_STORAGE);

	printf("done, to take effect RESTART AP.\n"); 
	
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSetRates(ConsoleParameter_T parameters[])
{ /*
	char rates[CCK_PBCC_RATES_MAX];
	UINT8 ratesLen = 0, i;
	UINT8 band;
	wdrv_dbGetBand(currentConfigWoneInex,&band);
    if (band == RADIO_BAND_5GHZ)
        {
            printf("\nError Active Set Wone Index is %d\n",currentConfigWoneInex);  
            printf("802.11 A Not support CCK Rates\n");  
            return;
        }
	
	memset(rates, 0, sizeof(rates));
	
	if (parameters[0].INTEGER > parameters[1].INTEGER)
	{
		printf("Invalid Rates configuration, basic rate is higher than supported. basic rate: %d, supported rate: %d\n", parameters[0].INTEGER, parameters[1].INTEGER); 
		return;
	}

	switch(parameters[1].INTEGER)
	{
	case 22:
		rates[4] = DOT11_RATE_22;
		ratesLen++;

	case 11:
		rates[3] = DOT11_RATE_11;
		ratesLen++;

	case 5:
		rates[2] = DOT11_RATE_5_5;
		ratesLen++;

	case 2:
		rates[1] = DOT11_RATE_2;
		ratesLen++;

	case 1:
		rates[0] = DOT11_RATE_1;
		ratesLen++;
		break;

	default:
		printf("Invalid rate = %d", parameters[1].INTEGER); 
		return;
	}

	
	switch(parameters[0].INTEGER)
	{
	case 11:
		rates[3] |= 0x80;

	case 5:
		rates[2] |= 0x80;

	case 2:
		rates[1] |= 0x80;

	case 1:
		rates[0] |= 0x80;
		break;

	default:
		printf("Invalid basic rate = %d", parameters[0].INTEGER);
		return;
	}

	wdrv_dbSetStationConfigOperationalRateSet(currentConfigWoneInex,rates, ratesLen);

	printf("Wone index %d Setting Rates = ",currentConfigWoneInex); 
	for (i = 0; i < ratesLen; i++)
	{
		printf("  "); 
		printRateInMbpsUnit(rates[i]);
	}
	printf("\n"); 
	printf("to take effect RESTART AP.\n"); */




	char rates[TIAP_PARAM_CCK_PBCC_RATES_MAX];
	__u8 cckRateValue[TIAP_PARAM_CCK_PBCC_RATES_MAX] =
		{DOT11_RATE_1,DOT11_RATE_2,DOT11_RATE_5_5,DOT11_RATE_11,DOT11_RATE_22};
	UINT8 ratesLen = 0;
	tiap_param_wone_t wone;

	wone.index = currentConfigWoneInex;
	wone.type = wone_band;
	if( tiap_wcfg_get_wone(wd, &wone, FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get band.");
		return;
	}

	if (wone.elem.band == TIAP_PARAM_RADIO_BAND_5GHZ)
        {
            printf("\nError Active Set Wone Index is %d\n",currentConfigWoneInex);  
            printf("802.11 A Not support CCK Rates\n");  
            return;
        }
	
	memset(rates, 0, sizeof(rates));
	
	if (parameters[0].INTEGER > parameters[1].INTEGER)
	{
		printf("Invalid Rates configuration, basic rate is higher than supported.\n"
		"Basic rate: %ld, supported rate: %ld\n", parameters[0].INTEGER, parameters[1].INTEGER); 
		return;
	}

	switch(parameters[1].INTEGER)
	{
		case 22:
			rates[4] = TIAP_PARAM_RATE_SUPPORTED;
			ratesLen++;

		case 11:
			rates[3] = TIAP_PARAM_RATE_SUPPORTED;
			ratesLen++;

		case 5:
			rates[2] = TIAP_PARAM_RATE_SUPPORTED;
			ratesLen++;

		case 2:
			rates[1] = TIAP_PARAM_RATE_SUPPORTED;
			ratesLen++;

		case 1:
			rates[0] = TIAP_PARAM_RATE_SUPPORTED;
			ratesLen++;
			break;

		default:
			printf("Invalid rate = %ld", parameters[1].INTEGER); 
			return;
	}


	switch(parameters[0].INTEGER)
	{
		case 11:
			rates[3] = TIAP_PARAM_RATE_BASIC_SUPPORTED;

		case 5:
			rates[2] = TIAP_PARAM_RATE_BASIC_SUPPORTED;

		case 2:
			rates[1] = TIAP_PARAM_RATE_BASIC_SUPPORTED;

		case 1:
			rates[0] = TIAP_PARAM_RATE_BASIC_SUPPORTED;
			break;

		default:
			printf("Invalid basic rate = %ld", parameters[0].INTEGER);
			return;
	}




	wone.index = currentConfigWoneInex;
	wone.type = wone_oper_rate;
	memcpy((void *)&wone.elem.b_legacy.rates, (void *)&rates,
			sizeof(wone.elem.b_legacy.rates));
	wone.elem.b_legacy.len = TIAP_PARAM_CCK_PBCC_RATES_MAX;
	if( tiap_wcfg_set_wone(wd, &wone, UPDATE_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to set operational rates.");
		return;
	}
		
	

	convert_rates(cckRateValue, rates, wone.elem.b_legacy.rates, TIAP_PARAM_CCK_PBCC_RATES_MAX);
	printf("Wone index %d Setting Rates = ",currentConfigWoneInex); 
/*	for (i = 0; i < ratesLen; i++)
	{
*/
		printf("  "); 
		printRateInMbpsUnit(rates);
/*
	}
*/
	printf("\n"); 
	printf("to take effect RESTART AP.\n");
	
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSetRespRateMode(ConsoleParameter_T parameters[])
{ /*
	if ((parameters[0].INTEGER != ASSOCRESP_RATE_ALL_AP) &&
		(parameters[0].INTEGER != ASSOCRESP_RATE_AP_AND_STA))
	{
		printf("Invalid mode: %d\n", parameters[0].INTEGER);
		return;
	}
	wdrv_dbSetRespRateMode((UINT8)(parameters[0].INTEGER));
	printf("Setting Associate Response supported rate IE mode = %d\n", parameters[0].INTEGER); 
	printf("to take effect RESTART AP.\n"); */

	PRINT_NOT_SUPPORTED_MSG("");
}


/*!	\brief 
  	\param 

  	\return 
  
*/
static int get_rangeElement(int band,
		tiap_param_cur_reg_domain_t cur_reg_domain,
		UINT8         RangeIndex,
		tiap_param_channel_rang_elem_t *rang_elem)
{
	if (RangeIndex == 0)    /* index is a positive number */
		return 1;

	switch (band)
	{
		case BAND_A:
			switch((__u8)cur_reg_domain.type)
			{
				case TIAP_PARAM_RD_US:
					if (RangeIndex > NUM_OF_RANGE_ELEMENTS_US)
						return 1;
					memcpy(rang_elem, &ch_rang_a[0].rang_elem[RangeIndex-1],
						sizeof(*rang_elem));
					break;

				case TIAP_PARAM_RD_WORLD:
					if (RangeIndex > NUM_OF_RANGE_ELEMENTS_WORLD)
						return 1;
					memcpy(rang_elem, &ch_rang_a[1].rang_elem[RangeIndex-1],
						sizeof(*rang_elem));
					break;

				default: return 1;
			}
			break;
		case BAND_B:
			switch((__u8)cur_reg_domain.type)
			{
				case TIAP_PARAM_RD_FCC:
					if (RangeIndex > 1)
						return 1;
					memcpy(rang_elem, &ch_rang_b[0].rang_elem[RangeIndex-1],
						sizeof(*rang_elem));
					break;

				case TIAP_PARAM_RD_IC:
					if (RangeIndex > 1)
						return 1;
					memcpy(rang_elem, &ch_rang_b[1].rang_elem[RangeIndex-1],
						sizeof(*rang_elem));
					break;

				case TIAP_PARAM_RD_ETSI:
					if (RangeIndex > 1)
						return 1;
					memcpy(rang_elem, &ch_rang_b[2].rang_elem[RangeIndex-1],
						sizeof(*rang_elem));
					break;

				case TIAP_PARAM_RD_SPAIN:
					if (RangeIndex > 1)
						return 1;
					memcpy(rang_elem, &ch_rang_b[3].rang_elem[RangeIndex-1],
						sizeof(*rang_elem));
					break;

				case TIAP_PARAM_RD_FRANCE:
					if (RangeIndex > 1)
						return 1;
					memcpy(rang_elem, &ch_rang_b[4].rang_elem[RangeIndex-1],
						sizeof(*rang_elem));
					break;

				case TIAP_PARAM_RD_MKK:
					if (RangeIndex > 1)
						return 1;
					memcpy(rang_elem, &ch_rang_b[5].rang_elem[RangeIndex-1],
						sizeof(*rang_elem));
					break;

				case TIAP_PARAM_RD_MKK1:
					if (RangeIndex > 2)
						return 1;
					memcpy(rang_elem, &ch_rang_b[6].rang_elem[RangeIndex-1],
						sizeof(*rang_elem));
					break;

                case TIAP_PARAM_RD_ISRAEL:
                    if (RangeIndex > 1)
                        return 1;
                    memcpy(rang_elem, &ch_rang_b[7].rang_elem[RangeIndex-1],
                        sizeof(*rang_elem));
                    break;

				default:
					return 1;
			}

			break;

		default:
			return 1;
	}
	return OK;
}


/* This routine tests if the channel is allowed according to the band and the current regulatory domain */
/*!	\brief 
  	\param 

  	\return 
  
*/
static BOOL IsChannelAllowed(UINT8 woneIndex, UINT8 channel)
{
	UINT8 isPrivateDomain, ranges_nr = 0, i;
	int band;
	UINT8 step, ch;
	tiap_param_wone_t wone;
	tiap_param_cur_reg_domain_t cur_regdomain;
	tiap_param_channel_rang ch_rang;
	tiap_param_channel_rang_elem_t rang_elem;

	wone.index = woneIndex;
	wone.type = wone_band;

	if( tiap_wcfg_get_wone(wd, &wone, FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get band.");
		return FALSE;
	}

	band = (wone.elem.band == TIAP_PARAM_RADIO_BAND_2_4GHZ) ? BAND_B : BAND_A;

	cur_regdomain.band = band;
	if( tiap_wcfg_get_cur_regdomain(wd, &cur_regdomain, FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get current regulatory domain.");
		return FALSE;
	}

    ch_rang.type = band;
	if( tiap_wcfg_get_ch_range(wd, &ch_rang, FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get channel range.");
		return FALSE;
	}

	isPrivateDomain = FALSE;
	if( band == BAND_A )
	{
	    
		if (channel > 64) { /* currently we assume that such channels are not supported */
			return FALSE;
		}
		/*Alexa: TODO: check the type of the board and if it supports
		 channels higher than 64 to allow them */

		switch( (__u8)cur_regdomain.type )
		{
			case TIAP_PARAM_RD_FCC:
			case TIAP_PARAM_RD_US:
				ranges_nr = NUM_OF_RANGE_ELEMENTS_US;
				break;
				
			case TIAP_PARAM_RD_IC:
			case TIAP_PARAM_RD_ETSI:
			case TIAP_PARAM_RD_SPAIN:
			case TIAP_PARAM_RD_FRANCE:
			case TIAP_PARAM_RD_MKK:
			case TIAP_PARAM_RD_MKK1:
            case TIAP_PARAM_RD_ISRAEL:
			case TIAP_PARAM_RD_WORLD:
				ranges_nr = NUM_OF_RANGE_ELEMENTS_WORLD;
				break;
			default:
				ranges_nr = ch_rang.ranges_nr;
				isPrivateDomain = TRUE;
		}
	}
	else if( band == BAND_B )
	{
		switch( (__u8)cur_regdomain.type )
		{
			case TIAP_PARAM_RD_FCC:
			case TIAP_PARAM_RD_IC:
			case TIAP_PARAM_RD_ETSI:
			case TIAP_PARAM_RD_SPAIN:
			case TIAP_PARAM_RD_FRANCE:
			case TIAP_PARAM_RD_MKK:
            case TIAP_PARAM_RD_ISRAEL:
				ranges_nr = 1;
				break;
			case TIAP_PARAM_RD_MKK1:
				ranges_nr = 2;
				break;
			default:
				ranges_nr = ch_rang.ranges_nr;
				isPrivateDomain = TRUE;
		}
	}
	else return FALSE;

	for (i = 1; i <= ranges_nr; i++)
	{

		if (!isPrivateDomain)
			get_rangeElement(band, cur_regdomain, i, &rang_elem);
		
		else rang_elem = (band == BAND_A ? ch_rang.rng.a[i-1] : ch_rang.rng.g[i-1]);
		
		step = rang_elem.step & 0xF;
		for (ch =  rang_elem.start; ch <= rang_elem.stop; ch += step) {
			if (ch == channel)
				return TRUE;
		}
	}

	return FALSE;
}



/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSetChannel(ConsoleParameter_T parameters[])
{/*
    UINT8 channel = (UINT8)parameters[0].INTEGER; 

    if (IsChannelAllowed(currentConfigWoneInex,channel) == FALSE) 
    {
        printf("This channel is not in the scope of the Current Regulatory Domain.\n");
        printf("(Use the \"printreg\" command under the 'features' directory for channels list.)\n");
        return;
    }
	wdrv_dbSetChannel(currentConfigWoneInex,channel);
	printf("Setting Wone %d Radio Channel = %d\n", currentConfigWoneInex, parameters[0].INTEGER); 
	printf("to take effect RESTART AP.\n"); */

	UINT8 channel = (UINT8)parameters[0].INTEGER; 
	tiap_param_wone_t wone;

	if (IsChannelAllowed(currentConfigWoneInex, channel) == FALSE) 
	{
		printf("This channel is not in the scope of the Current Regulatory Domain.\n");
		printf("(Use the \"printreg\" command under the 'features' directory for channels list.)\n");
		return;
	}
	
	wone.index = currentConfigWoneInex;
	wone.type = wone_ch;
	wone.elem.ch = parameters[0].INTEGER;
	if( tiap_wcfg_set_wone(wd, &wone, UPDATE_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to set channel.");
		return;
	}

	printf("Setting Wone %d Radio Channel = %ld\n", currentConfigWoneInex, parameters[0].INTEGER); 
	printf("to take effect RESTART AP.\n"); 
}


/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSetBeacon(ConsoleParameter_T parameters[])
{/*

   if ((parameters[0].SHORT < MIN_BEACON_PERIOD) || (parameters[0].SHORT > MAX_BEACON_PERIOD))
	{
		printf("Invalid beacon period: %d\n", parameters[0].SHORT);
		return;
	}
		
	wdrv_dbSetStationConfigBeaconPeriod(parameters[0].SHORT);
	printf("Setting beacon period = %d msecs\n", parameters[0].SHORT); 
	printf("to take effect RESTART AP.\n"); */


	if ((parameters[0].SHORT < MIN_BEACON_PERIOD) || (parameters[0].SHORT > MAX_BEACON_PERIOD))
	{
		printf("Invalid beacon period: %d\n", parameters[0].SHORT);
		return;
	}
	if( tiap_wcfg_set_basic(wd, TIAP_PARAM_BEACON_PERIOD, &parameters[0].SHORT,UPDATE_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to set beacon period.\n");
		return;
	}
	printf("Setting beacon period = %d msecs\n", parameters[0].SHORT); 
	printf("Setting AP BestEffort and Background msduMaxLifeTime to %d mSec[ 5 * Beacon Period*DTIM Period] \n",
                5*(wd->storage.ldb.dtim_period*wd->storage.ldb.beacon_period)); 
	printf("to take effect RESTART AP.\n");
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSetDtim(ConsoleParameter_T parameters[])
{
	__u8  dtimPeriod = (__u8) parameters[0].SHORT;

	/*
	if ((parameters[0].SHORT < MIN_DTIM_PERIOD) || (parameters[0].SHORT > MAX_DTIM_PERIOD))
	{
		printf("Invalid DTIM period: %d\n", parameters[0].SHORT);
		return;
	}
	wdrv_dbSetStationConfigDtimPeriod(parameters[0].SHORT);
	printf("Setting DTIM period = %d beacons\n", parameters[0].SHORT); 
	printf("to take effect RESTART AP.\n");*/
	
	if ((parameters[0].SHORT < MIN_DTIM_PERIOD) || (parameters[0].SHORT > MAX_DTIM_PERIOD))
	{
		printf("Invalid DTIM period: %d\n", parameters[0].SHORT);
		return;
	}
	if( tiap_wcfg_set_basic(wd, TIAP_PARAM_DTIM_PERIOD, &dtimPeriod, UPDATE_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to set DTIM period.\n");
		return;
	}
	printf("Setting DTIM period = %d beacons\n", parameters[0].SHORT); 
	printf("Setting AP BestEffort and Background msduMaxLifeTime to %d mSec [ 5 * Beacon Period*DTIM Period] \n", 
                5*(wd->storage.ldb.dtim_period*wd->storage.ldb.beacon_period)); 
	printf("to take effect RESTART AP.\n");
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSetEepromLess(ConsoleParameter_T parameters[])
{
	if ((parameters[0].SHORT < 0) || (parameters[0].SHORT > 1))
	{
		printf("Invalid Eeprom Less : %d\n", parameters[0].SHORT);
		return;
	}
	if( tiap_wcfg_set_basic(wd, TIAP_PARAM_EEPROM_LESS_ENABLE, &parameters[0].SHORT,UPDATE_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to set eeprom less.\n");
		return;
	}
	printf("Setting eeprom less = %d \n", parameters[0].SHORT); 
	printf("to take effect RESTART AP.\n");
}

void wlanSetRtsThreshold(ConsoleParameter_T parameters[])
{/*
	// It is enough to compare the max range only, since the it is unsigned. 
	if (parameters[0].SHORT > MAX_RTS_THRESHOLD)
	{
		printf("Invalid Rts Threshold: %d\n", parameters[0].SHORT);
		return;
	}
	wdrv_dbSetOperationRTSThreshold(parameters[0].SHORT);
	printf("Setting RTS threshold = %d\n", parameters[0].SHORT); 
	printf("to take effect RESTART AP.\n"); */

	if (parameters[0].SHORT > MAX_RTS_THRESHOLD)
	{
		printf("Invalid Rts Threshold: %d\n", parameters[0].SHORT);
		return;
	}
	if( tiap_wcfg_siwrts(wd, parameters[0].SHORT, UPDATE_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to set rts period.\n");
		return;
	}
	printf("Setting RTS threshold = %d\n", parameters[0].SHORT); 
	printf("to take effect RESTART AP.\n");
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSetShoreRetryLimit(ConsoleParameter_T parameters[])
{/*
	if ((parameters[0].SHORT < MIN_SHORT_RETRY_LIMIT) || (parameters[0].SHORT > MAX_SHORT_RETRY_LIMIT))
	{
		printf("Invalid short retry limit: %d\n", parameters[0].SHORT);
		return;
	}
	wdrv_dbSetOperationShortRetryLimit((UINT8)parameters[0].SHORT);
	printf("Setting short retry limit = %d\n", parameters[0].SHORT); 
	printf("to take effect RESTART AP.\n"); */
	
	if ((parameters[0].SHORT < MIN_SHORT_RETRY_LIMIT) || (parameters[0].SHORT > MAX_SHORT_RETRY_LIMIT))
	{
		printf("Invalid short retry limit: %d\n", parameters[0].SHORT);
		return;
	}
	if( tiap_wcfg_siwretry(wd, (UINT8)parameters[0].SHORT, TIAP_PARAM_RETRY_SHORT, UPDATE_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to set short retry limit.");
		return;
	}
		
	printf("Setting short retry limit = %d\n", parameters[0].SHORT); 
	printf("to take effect RESTART AP.\n");
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSetLongRetryLimit(ConsoleParameter_T parameters[])
{/*
	if ((parameters[0].SHORT < MIN_LONG_RETRY_LIMIT) || (parameters[0].SHORT > MAX_LONG_RETRY_LIMIT))
	{
		printf("Invalid long retry limit: %d\n", parameters[0].SHORT);
		return;
	}
	wdrv_dbSetOperationLongRetryLimit((UINT8)parameters[0].SHORT);
	printf("Setting long retry limit = %d\n", parameters[0].SHORT); 
	printf("to take effect RESTART AP.\n"); */

	if ((parameters[0].SHORT < MIN_LONG_RETRY_LIMIT) || (parameters[0].SHORT > MAX_LONG_RETRY_LIMIT))
	{
		printf("Invalid long retry limit: %d\n", parameters[0].SHORT);
		return;
	}
	if( tiap_wcfg_siwretry(wd, (UINT8)parameters[0].SHORT, TIAP_PARAM_RETRY_LONG, UPDATE_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to set short retry limit.");
		return;
	}
	printf("Setting long retry limit = %d\n", parameters[0].SHORT); 
	printf("to take effect RESTART AP.\n");
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSetRateFallback(ConsoleParameter_T parameters[])
{
	__u8  rateFallback = parameters[0].SHORT;
	/*
	wdrv_dbSetRateFallback((UINT8)parameters[0].SHORT);
	printf("Setting Rate fallback = %d\n", parameters[0].SHORT); 
	printf("to take effect RESTART AP.\n"); */

	if( tiap_wcfg_set_basic(wd, TIAP_PARAM_RATE_FALLBACK, &rateFallback, UPDATE_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to set rate fallback.");
		return;
	}
	printf("Setting Rate fallback = %d\n", parameters[0].SHORT); 
	printf("to take effect RESTART AP.\n");
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSetRateAdaptation(ConsoleParameter_T parameters[])
{/*

	wdrv_dbSetRateAdaptation(currentConfigWoneInex,(UINT8)parameters[0].SHORT);
	printf("Wone index %d Setting Rate adaptation = %d\n",currentConfigWoneInex, parameters[0].SHORT); 
	printf("to take effect RESTART AP.\n"); */
	
	tiap_param_wone_t wone;

	wone.index = currentConfigWoneInex;
	wone.type = wone_rate_adapt;
	wone.elem.rate_adapt = (UINT8)parameters[0].SHORT;
	
	if( tiap_wcfg_set_wone(wd, &wone, UPDATE_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to set rate adaptation.");
		return;
	}
	printf("Wone index %d Setting Rate adaptation = %d\n",currentConfigWoneInex, parameters[0].SHORT); 
	printf("to take effect RESTART AP.\n");
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSetFragmentationTreshold(ConsoleParameter_T parameters[])
{/*
	BOOL Status4x;
  
	if ((parameters[0].SHORT < MIN_FRAG_THRESHOLD) || (parameters[0].SHORT > MAX_FRAG_THRESHOLD))
	{
		printf("Invalid fragmentation threshold: %d\n", parameters[0].SHORT);
		return;
	}
	if ((parameters[0].SHORT % 2) !=0 )
	{
		printf("Invalid fragmentation threshold, must be an even number.\n");
		return;
	}


   wdrv_dbGet4xEnable(&Status4x);

   if(( parameters[0].SHORT != MAX_FRAG_THRESHOLD )&&( Status4x == TRUE ))
   {
      printf("Fragmentation Treshold must be %d when 4X is enabled.\n", MAX_FRAG_THRESHOLD);
      return;
   }

	wdrv_dbSetOperationFragmentationThreshold(parameters[0].SHORT);
   printf("Setting fragmentation threshold = %d\n", parameters[0].SHORT); 
	printf("to take effect RESTART AP.\n"); */

	tiap_param_4x_t _4x;

	if ((parameters[0].SHORT < MIN_FRAG_THRESHOLD) || (parameters[0].SHORT > MAX_FRAG_THRESHOLD))
	{
		printf("Invalid fragmentation threshold: %d\n", parameters[0].SHORT);
		return;
	}
	if ((parameters[0].SHORT % 2) !=0 )
	{
		printf("Invalid fragmentation threshold, must be an even number.\n");
		return;
	}

	
	_4x.cmd = TIAP_PARAM_4X_4X;
	if( tiap_wcfg_get_4x(wd, &_4x, FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get 4x enable.");
		return;
	}
	if(( parameters[0].SHORT != MAX_FRAG_THRESHOLD )&&( _4x.value == TRUE ))
	{
		printf("Fragmentation Treshold must be %d when 4X is enabled.\n", MAX_FRAG_THRESHOLD);
		return;
	}

	if( tiap_wcfg_siwfrag(wd, parameters[0].SHORT, UPDATE_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to set fragmentation threshold.");
		return;
	}
			
	printf("Setting fragmentation threshold = %d\n", parameters[0].SHORT); 
	printf("to take effect RESTART AP.\n");
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSetMaxTransmitMSDULifetime(ConsoleParameter_T parameters[])
{/*
	if ((parameters[0].INTEGER < MIN_TRANSMIT_MSDU_LIFETIME) || (parameters[0].INTEGER > MAX_TRANSMIT_MSDU_LIFETIME))
	{
		printf("Invalid Max transmit MSDU lifetime: %d\n", parameters[0].INTEGER);
		return;
	}
	wdrv_dbSetOperationMaxTransmitMSDULifetime(parameters[0].INTEGER);
	printf("Setting max transmit MSDU lifetime = %d\n", parameters[0].INTEGER); 
	printf("to take effect RESTART AP.\n"); */

	if ((parameters[0].INTEGER < MIN_TRANSMIT_MSDU_LIFETIME) || (parameters[0].INTEGER > MAX_TRANSMIT_MSDU_LIFETIME))
	{
		printf("Invalid Max transmit MSDU lifetime: %ld\n", parameters[0].INTEGER);
		return;
	}

	if( tiap_wcfg_siwretry(wd, parameters[0].INTEGER, TIAP_PARAM_LIFETIME_TX, UPDATE_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to set max transmit MSDU lifetime.");
		return;
	}
	printf("Setting max transmit MSDU lifetime = %ld\n", parameters[0].INTEGER); 
	printf("to take effect RESTART AP.\n"); 
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSetMaxReceiveLifetime(ConsoleParameter_T parameters[])
{/*
	if ((parameters[0].INTEGER < MIN_RECEIVE_MSDU_LIFETIME) || (parameters[0].INTEGER > MAX_RECEIVE_MSDU_LIFETIME))
	{
		printf("Invalid Max receive lifetime: %d\n", parameters[0].INTEGER);
		return;
	}
	wdrv_dbSetOperationMaxReceiveLifetime(parameters[0].INTEGER);
	printf("Setting max receive MSDU lifetime = %d\n", parameters[0].INTEGER); 
	printf("to take effect RESTART AP.\n"); */

	if ((parameters[0].INTEGER < MIN_RECEIVE_MSDU_LIFETIME) || (parameters[0].INTEGER > MAX_RECEIVE_MSDU_LIFETIME))
	{
		printf("Invalid Max receive lifetime: %ld\n", parameters[0].INTEGER);
		return;
	}

	if( tiap_wcfg_siwretry(wd, parameters[0].INTEGER, TIAP_PARAM_LIFETIME_RX, UPDATE_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to set max receive MSDU lifetime.");
		return;
	}
	printf("Setting max receive MSDU lifetime = %ld\n", parameters[0].INTEGER); 
	printf("to take effect RESTART AP.\n"); 
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanPrintRatesParams(ConsoleParameter_T parameters[])
{/*
	int i;

	char rates[CCK_PBCC_RATES_MAX+OFDM_RATES_MAX];
	UINT8 ratesLen;
	char extRates[OFDM_RATES_MAX];
	UINT8 extRatesLen;
	char allRates[CCK_PBCC_RATES_MAX+OFDM_RATES_MAX];
	UINT8 allRatesLen;

    UINT8 wone_index;
    UINT8  num_sup_wone;
    wdrv_dbGetNumSupWone(&num_sup_wone);
    for (wone_index = 0 ; wone_index < num_sup_wone; wone_index ++)
    {
        wdrv_dbGetStationConfigOperationalRateSet(wone_index, rates, &ratesLen);
        wdrv_dbGetStationConfigOperationalOFDMRateSet(wone_index, extRates, &extRatesLen);
        wdrv_dbGetStationConfigOperationalAllRateSet(wone_index, allRates, &allRatesLen);

	printf("\n"); 
        printf("Wone Index %d\n",wone_index); 

	printf("Rates in Legacy IE =    "); 
	for (i = 0; i < ratesLen; i++)
	{
		printf("  "); 
		printRateInMbpsUnit(rates[i]);
	}
	printf("\n\n"); 
	printf("Rates in Extended IE =    "); 
	for (i = 0; i < extRatesLen; i++)
	{
		printf("  "); 
		printRateInMbpsUnit(extRates[i]);
	}
	printf("\n\n"); 
	printf("All Rates =    "); 
	for (i = 0; i < allRatesLen; i++)
	{
		printf("  "); 
		printRateInMbpsUnit(allRates[i]);
	}
        printf("\n"); 
    } */


	dda_print_proc_entry(WLAN_PROC_SERV_DIR_NAME, WLAN_PROC_SERV_RATES);
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanPrintParams(ConsoleParameter_T parameters[])
{/*
	char  macaddr[6], ssid[50], bssid[6];
	UINT8 shortRetryLimit, longRetryLimit, dtim, channel, defaultActiveRate;
	UINT8 RateFallback, RateAdaptation;
	int i;
	UINT16 capabilities, beacon, rtsThreshold, fragThreshold, stationsNotPBCCCapabale;
	UINT32 transmitLifetime, receiveLifetime;
	BOOL currentPBCCSupport;
	UINT8 PBCCAlgorithmMode; 
	UINT32 swMinor, swMajor;
	UINT8 SlotTimeMode, PreambleMode;
	UINT16 SlotTimeThreshold;
	UINT16 stationsNotSlotTimeCapabale, stationsNotShortPreambleCapabale;
	BOOL currentShortSlotTimeSupport, currentShortPreambleSupport;
	UINT8 ERPProtectionMode, ERPProtectionType;
	UINT8 ERPProtectionTI;
	BOOL currentERPProtectionMode;
	char allRates[CCK_PBCC_RATES_MAX+OFDM_RATES_MAX];
	UINT8 allRatesLen;
	UINT8 respRateMode;
	UINT8 dot11Mode,CWminMode;
	UINT16 CWminWindowSize;
	UINT32	DynAlgoTimeout;
	UINT32 RadioCalibrateOption;
	UINT16 RadioCalibrateInterval;
   UINT8 band;
   UINT8 num_sup_wone;
   UINT16 HiddenSSID;

   UINT8 usageTimeMode;
   UINT8 usageTime;

        UINT16 MultipleSsidFlag;
        char MultipleSsidTable[NUM_MULTIPLE_SSID][WHAL_SSID_STRING_LEN];
   UINT8  idx;
   tVHCF_PARAMS  vhcfParams;

	if (wdrv_dbIsNvramIdenticalToDatabase() == FALSE)
	{
		printf("NOTICE:	The following parameters values are taken from the SDRAM,\n");
		printf("		some of them may be different from the values stored in the NVRAM!!!\n");
		printf("\n");
	}
    wdrv_dbGetNumSupWone(&num_sup_wone);
    if (num_sup_wone == WONE_DUAL_BAND )
    {
        printf("\n---------  Dual AP ---------\n");

        wdrv_dbGetUsageTimeMode(&usageTimeMode);
        if(usageTimeMode)
           printf("\n--------- Long Usage Time ---------\n");
        else  

           printf("\n---------  Short Usage Time ---------\n");
    }
    printf("\n--------- Active Set Wone Index is %d ---------\n",currentConfigWoneInex);  

    wdrv_dbGetBand(currentConfigWoneInex,&band);


	swMajor = GET_MAJOR_SW_VER;
	swMinor = GET_MINOR_SW_VER;

	wdrv_dbGetOperationMacAddress(macaddr);
	wdrv_dbGetBssid(bssid);	
	wdrv_dbGetSsid(ssid);
    wdrv_dbGetHiddenSsidStatus(&HiddenSSID);
    wdrv_dbGetMultipleSsidStatus(&MultipleSsidFlag);
    if (MultipleSsidFlag)
    {
        UINT32 length = NUM_MULTIPLE_SSID*WHAL_SSID_STRING_LEN;
        whal_nvram_getMultipleSsidTable(MultipleSsidTable[0],&length);
    }
    wdrv_dbGetStationConfigOperationalAllRateSet(currentConfigWoneInex,allRates, &allRatesLen);
	wdrv_dbGetDefaultActiveRate(currentConfigWoneInex,&defaultActiveRate);
	wdrv_dbGetApCapabilities(currentConfigWoneInex,&capabilities);
    wdrv_dbGetChannel(currentConfigWoneInex,&channel);
	wdrv_dbGetStationConfigBeaconPeriod(&beacon);
	wdrv_dbGetStationConfigDtimPeriod(&dtim);

	wdrv_dbGetOperationRTSThreshold(&rtsThreshold);
	wdrv_dbGetOperationShortRetryLimit(&shortRetryLimit);
	wdrv_dbGetOperationLongRetryLimit(&longRetryLimit);
	wdrv_dbGetRateFallback(&RateFallback);
	wdrv_dbGetRateAdaptation(currentConfigWoneInex,&RateAdaptation);
	wdrv_dbGetOperationFragmentationThreshold(&fragThreshold);
	wdrv_dbGetOperationMaxTransmitMSDULifetime(&transmitLifetime);
	wdrv_dbGetOperationMaxReceiveLifetime(&receiveLifetime);
	wdrv_dbGetPBCCAlgorithmMode(currentConfigWoneInex,&PBCCAlgorithmMode);
	wdrv_dbGetNumOfStationNotPBCCCapable(&stationsNotPBCCCapabale);
	wdrv_dbGetCurrentPBCCSupportMode(&currentPBCCSupport);
	wdrv_dbGetDot11Mode(currentConfigWoneInex,&dot11Mode);
	wdrv_dbGetRadioCalibrateOption(&RadioCalibrateOption);
	wdrv_dbGetRadioCalibrateInterval(&RadioCalibrateInterval);

	wdrv_dbGetPreambleMode(currentConfigWoneInex,&PreambleMode);
	wdrv_dbGetNumOfStationNotShortPreambleCapable(&stationsNotShortPreambleCapabale);
	wdrv_dbGetCurrentShortPreambleSupport(&currentShortPreambleSupport);

	wdrv_dbGetSlotTimeMode(currentConfigWoneInex,&SlotTimeMode);
	wdrv_dbGetSlotTimeThreshold(&SlotTimeThreshold);
        wdrv_dbGetNumOfStationNotSlotTimeCapable(&stationsNotSlotTimeCapabale);
	wdrv_dbGetCurrentShortSlotTimeSupport(&currentShortSlotTimeSupport);

	wdrv_dbGetERPProtectionMode(currentConfigWoneInex,&ERPProtectionMode);
	wdrv_dbGetCurrentERPProtectionMode(&currentERPProtectionMode);
	wdrv_dbGetERPProtectionType(currentConfigWoneInex,&ERPProtectionType);
	wdrv_dbGetERPProtectionTI(currentConfigWoneInex,&ERPProtectionTI);

	wdrv_dbGetDynamicAlgorithmsTimeout(&DynAlgoTimeout);

	wdrv_dbGetRespRateMode(&respRateMode);

	wdrv_dbGetCwMinMode(currentConfigWoneInex,&CWminMode);
	wdrv_dbGetCwMinWinSize(&CWminWindowSize);
   wdrv_dbGetUsageTime(currentConfigWoneInex,&usageTime);
   wdrv_dbGetVhcfParams(&vhcfParams);


	printf("Software Version:	%d.%d\n",swMajor, swMinor);
#ifdef VXWORKS
   {
      unsigned char prn_buffer[81], *buf;
      buf = prn_buffer;
      cm_hwFormatBootInfo1(buf, 80);
      puts(buf);
   }
#endif // VXWORKS 

	printf("WLAN MAC = <%02hhX-%02hhX-%02hhX-%02hhX-%02hhX-%02hhX>\n",		
		  macaddr[0],macaddr[1],macaddr[2],macaddr[3],
		  macaddr[4],macaddr[5] );

	printf("WLAN BSSID = <%02hhX-%02hhX-%02hhX-%02hhX-%02hhX-%02hhX>\n",		
		  bssid[0],bssid[1],bssid[2],bssid[3],
		  bssid[4],bssid[5] );

	printf("SSID = %s\n", ssid); 

    if (band == RADIO_BAND_5GHZ)    
        printf("Band = 5 MHZ\n");  
    else
        printf("Band = 2.4 MHZ\n"); 

    printf("Hidden SSID mode is %s\n", (HiddenSSID==1) ? "Enabled" : "Disabled");
    printf("Multiple SSID mode is %s\n", (MultipleSsidFlag==1) ? "Enabled" : "Disabled");
    if (MultipleSsidFlag==1) 
    {
        printf("List of Multiple SSID:\n");
        for (i=0; i<NUM_MULTIPLE_SSID; i++) 
        {
            if (MultipleSsidTable[i][0] == '\0')
                break;
            printf("\t%d. %s\n",i+1,MultipleSsidTable[i]);
        }
        if (i==0)
            printf("\tNone.\n");
    }

	printf("Capabilities = AGILE  PBCC  ShortPreamble  Privacy  CFPollReq  CFPoll  IBSS  ESS\n"); 

	printf("                 %d    ", ((capabilities >> CAP_AGILE_SHIFT) & CAP_AGILE_MASK));
	printf("  %d   ", ((capabilities >> CAP_PBCC_SHIFT) & CAP_PBCC_MASK));
	printf("      %d       ", ((capabilities >> CAP_PREAMBLE_SHIFT) & CAP_PREAMBLE_MASK));
	printf("    %d     ", ((capabilities >> CAP_PRIVACY_SHIFT) & CAP_PRIVACY_MASK));
	printf("    %d      ", ((capabilities >> CAP_CF_REQ_SHIFT) & CAP_CF_REQ_MASK));
	printf("   %d    ", ((capabilities >> CAP_CF_POLL_SHIFT) & CAP_CF_POLL_MASK));
	printf("  %d   ", ((capabilities >> CAP_IBSS_SHIFT) & CAP_IBSS_MASK));
	printf("  %d\n", ((capabilities >> CAP_ESS_SHIFT) & CAP_ESS_MASK));
	
	printf("Capabilities = RSN  ShortSlotTime\n"); 
	printf("                %d         ", ((capabilities >> CAP_RSN_SHIFT) & CAP_RSN_MASK));
	printf("%d\n", ((capabilities >> CAP_SLOTTIME_SHIFT) & CAP_SLOTTIME_MASK));

	printf("Rates =	"); 
	for (i = 0; i < allRatesLen; i++)
	{
		printf("  "); 
		printRateInMbpsUnit(allRates[i]);
	}
	printf("\n"); 

	printf("Default active rate = "); 
	printf("  "); 
	printRateInMbpsUnit(defaultActiveRate);
	printf("\n"); 

	printf("Channel = %d\n", channel); 

	if (channel == 14)
		printf("Domain = MKK\n"); 
	else
		printf("Domain = ETSI\n"); 

	printf("Beacon period = %d msecs\n", beacon); 
	
	printf("DTIM period = %d beacons\n", dtim); 

	printf("rtsThreshold = %d bytes\n", rtsThreshold); 
	printf("shortRetryLimit = %d\n", shortRetryLimit); 
	printf("longRetryLimit = %d\n", longRetryLimit); 
	printf("rate fallback = %d\n", RateFallback); 
	if (RateAdaptation)
	{
		printf("rate adaptation = Enabled\n");
	}
	else
	{
		printf("rate adaptation = Disabled\n");
	}
	printf("fragThreshold = %d bytes\n", fragThreshold); 
	printf("transmitLifetime = %d msecs\n", transmitLifetime); 
	printf("receiveLifetime = %d msecs\n", receiveLifetime); 
	printf("Radio Calibrate option=%d interval=%d\n", RadioCalibrateOption, RadioCalibrateInterval); 

	if (dot11Mode == DOT11_MODE_MIXED)
	{
		printf("Dot11 mode = Mixed mode\n");
	}
	else if (dot11Mode == DOT11_MODE_B_ONLY)
	{
		printf("Dot11 mode = B only mode\n");
	}
	else if(dot11Mode == DOT11_MODE_OFDM_ONLY)
	{
		printf("Dot11 mode = G only mode\n");
	}
	else if(dot11Mode == DOT11_MODE_B_ONLY_PLUS)
	{
		printf("Dot11 mode = B+ only mode\n");
	}
	else
		printf("Dot11 mode = A only mode\n");

	
	if (respRateMode == ASSOCRESP_RATE_ALL_AP)
	{
		printf("AssocResp rate mode = %d (All AP rates)\n", ASSOCRESP_RATE_ALL_AP);
	}
	else if (ASSOCRESP_RATE_AP_AND_STA)
	{
		printf("AssocResp rate mode = %d (Only AP rates supported by STA)\n", ASSOCRESP_RATE_AP_AND_STA);
	}

	printf("Enhanced Dynamic Algorithms Timeout = %d ms\n", DynAlgoTimeout);

	printf("CWmin mode = %d, CWminWindowSize = %d\n", CWminMode,CWminWindowSize);


	// PBCC support section
	printf("--------------------------------------------\n"); 
	if (PBCCAlgorithmMode == PBCC_ALGO_DISABLED)
		printf("PBCC Algorithm			Always off\n");
	else if (PBCCAlgorithmMode == PBCC_ALGO_LOCAL_STATIONS)
		printf("PBCC Algorithm			Local Stations\n");
	else if (PBCCAlgorithmMode == PBCC_ALGO_DYNAMIC)
		printf("PBCC Algorithm			Dynamic\n");
	else if (PBCCAlgorithmMode == PBCC_ALGO_ENHANCED_DYNAMIC)
		printf("PBCC Algorithm			Enhanced Dynamic\n");	
	else if (PBCCAlgorithmMode == PBCC_ALGO_ENABLED)
		printf("PBCC Algorithm			Always on\n");
	else
		printf("PBCC Algorithm			Unrecognized\n");

	if ((PBCCAlgorithmMode == PBCC_ALGO_LOCAL_STATIONS) || (PBCCAlgorithmMode == PBCC_ALGO_DYNAMIC) || (PBCCAlgorithmMode == PBCC_ALGO_ENHANCED_DYNAMIC))
	{
		printf("Number of station that are not PBCC capable:	%d\n", stationsNotPBCCCapabale);
		if ((capabilities >> CAP_PBCC_SHIFT) & CAP_PBCC_MASK)
		{	
			if (currentPBCCSupport)
				printf("Current PBCC support		On\n");
			else
				printf("Current PBCC support		Off\n");
		}
	}

	// Preamble support section
	printf("--------------------------------------------\n"); 
	if (PreambleMode == PREAMBLE_MODE_LONG)
		printf("Preamble Mode                   Always Long\n");
	else if (PreambleMode == PREAMBLE_MODE_SHORT)
		printf("Preamble Mode                   Always Short\n");
	else if (PreambleMode == PREAMBLE_MODE_LOCAL_STATIONS)
	{
		printf("Preamble Mode                   Local Stations\n");
		if (currentShortPreambleSupport)
			printf("Current Short Preamble support  On\n");
		else
			printf("Current Short Preamble support  Off\n");
		printf("Num of stations not capable     %d\n", stationsNotShortPreambleCapabale);
	}

	// Slot Time support section
	printf("--------------------------------------------\n"); 
	if (SlotTimeMode == SLOTTIME_MODE_DISABLED)
		printf("Slot Time Mode                  Always off\n");
	else if (SlotTimeMode == SLOTTIME_MODE_ENABLED)
		printf("Slot Time Mode                  Always on\n");
	else if (SlotTimeMode == SLOTTIME_MODE_LOCAL_STATIONS)
	{
		printf("Slot Time Mode                  Local Stations\n");
		printf("Slot Time Threshold             %d\n", SlotTimeThreshold);
		if (currentShortSlotTimeSupport)
			printf("Current Short Slot Time support On\n");
		else
			printf("Current Short Slot Time support Off\n");
		printf("Num of stations not capable     %d\n", stationsNotSlotTimeCapabale);
	}
	else if (SlotTimeMode == SLOTTIME_MODE_ENHANCED_DYNAMIC)
	{
		printf("Slot Time Mode                  Enhanced Dynamic\n");
		printf("Slot Time Threshold             %d\n", SlotTimeThreshold);
		if (currentShortSlotTimeSupport)
			printf("Current Short Slot Time support On\n");
		else
			printf("Current Short Slot Time support Off\n");
		printf("Num of stations not capable     %d\n", stationsNotSlotTimeCapabale);
	}

	

	// ERP Protection support section
	printf("--------------------------------------------\n"); 
	if (ERPProtectionMode == PROTECTION_MODE_DISABLED)
	{
		printf("ERP Protection Mode             Always off\n");
	}
	else
	{
		if(ERPProtectionMode == PROTECTION_MODE_LOCAL_STATIONS)		
			printf("ERP Protection Mode             Local Stations\n");
		else if(ERPProtectionMode == PROTECTION_MODE_DYNAMIC)
			printf("ERP Protection Mode             Dynamic\n");
		else if(ERPProtectionMode == PROTECTION_MODE_ENHANCED_DYNAMIC)
			printf("ERP Protection Mode             Enhanced Dynamic\n");
		else if (ERPProtectionMode == PROTECTION_MODE_ENABLED)
			printf("ERP Protection Mode             Always on\n");
		else
			printf(" Undefined ERP Protection Mode\n");


		if (currentERPProtectionMode)
		{
			printf("Current ERP Protection Mode     On\n");
		}
		else
		{
			printf("Current ERP Protection Mode     Off\n");
		}

		if(ERPProtectionTI)
			printf("ERP Protection Type             TI-proprietary\n");
		else
		{
			if (ERPProtectionType == PROTECTION_TYPE_RTS_CTS)
				printf("ERP Protection Type             RTS/CTS\n");
			else
				printf("ERP Protection Type             CTS TO SELF\n");
		}
	}
   if(num_sup_wone == 2)
   printf("Wone Usage Time is %d\n",usageTime);

   // Show vHCF configuration parameters 
   printf("--------------------------------------------\n"); 
   if(vhcfParams.vhcfEnable)   
   {	
	  printf("vHCF Enabled:\n");
	  for(idx = 0; idx < MAX_VHCF_CLASSIFIERS ; idx++)
	  {
		  if (vhcfParams.vhcfClassifier[idx].Protocol != VHCF_PROT_NONE)
		  {
			  printf("Classifier #%u: Prot = %s, Dest_IP = %u.%u.%u.%u, Dest_Port = %u\n",
				  idx, 
				  (vhcfParams.vhcfClassifier[idx].Protocol == VHCF_PROT_TCP) ? "TCP" : "UDP",
				  (vhcfParams.vhcfClassifier[idx].DestIp >> 24) & 0xFF,
				  (vhcfParams.vhcfClassifier[idx].DestIp >> 16) & 0xFF,
				  (vhcfParams.vhcfClassifier[idx].DestIp >>  8) & 0xFF,
				  (vhcfParams.vhcfClassifier[idx].DestIp      ) & 0xFF,
				   vhcfParams.vhcfClassifier[idx].DestPort);
		  }
		  else
			  printf("Classifier #%u: Not Used.\n", idx);
	  }
	}
	else
		printf("vHCF Disabled!\n"); */

	char buf[GENERIC_IOCTL_BUF_SIZE];

	DRIVER_LOADED_ASSERT;

	if (ioctl_get_wlan_params(currentConfigWoneInex, buf, GENERIC_IOCTL_BUF_SIZE, wd->dev_name)) 
		puts("ioctl_get_wlan_params failed !");
	else
		puts(buf);
	
	
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanPrintCounters(ConsoleParameter_T parameters[])
{
	/*
	UINT32 count1, count2, count3, count4, count5, count6, count7;
	UINT32 count8, count9, count10, count11, count12, count13, count14;
	UINT32 count15, count16;

	wdrv_dbGetCountersTransmittedFragmentCount(&count1);
	wdrv_dbGetCountersMulticastTransmittedFrameCount(&count2);
	wdrv_dbGetCountersFailedCount(&count3);
	wdrv_dbGetCountersRetryCount(&count4);
	wdrv_dbGetCountersMultipleRetryCount(&count5);
	wdrv_dbGetCountersFrameDuplicateCount(&count6);
	wdrv_dbGetCountersRTSSuccessCount(&count7);
	wdrv_dbGetCountersRTSFailureCount(&count8);
	wdrv_dbGetCountersACKFailureCount(&count9);
	wdrv_dbGetCountersReceivedFragmentCount(&count10);
	wdrv_dbGetCountersMulticastReceivedFrameCount(&count11);
	wdrv_dbGetCountersFCSErrorCount(&count12);
	wdrv_dbGetCountersTransmittedFrameCount(&count13);
	wdrv_dbGetCountersWEPUndecryptableCount(&count14);
	wdrv_dbGetPrivacyWEPICVErrorCount(&count15);
	wdrv_dbGetPrivacyWEPExcludedCount(&count16);

	printf("***Counters***\n"); 
	printf("Transmitted Fragment Count			= %d\n", count1); 
	printf("Multicast Transmitted Frame Count		= %d\n", count2); 
	printf("Failed Count					= %d\n", count3); 
	printf("Retry Count					= %d\n", count4); 
	printf("Multiple Retry Count				= %d\n", count5); 
	printf("Frame Duplicate Count				= %d\n", count6); 
	printf("RTS Success Count				= %d\n", count7); 
	printf("RTS Failure Count				= %d\n", count8); 
	printf("ACK Failure Count				= %d\n", count9); 
	printf("Received Fragment Count				= %d\n", count10); 
	printf("Multicast Received Frame Count			= %d\n", count11); 
	printf("FCS Error Count					= %d\n", count12); 
	printf("Transmitted Frame Count				= %d\n", count13); 
	printf("WEP Undecryptable Count				= %d\n", count14); 
	printf("WEP ICV Error Count				= %d\n", count15); 
	printf("WEP Excluded Count				= %d\n", count16); 
*/


	dda_print_proc_entry(WLAN_PROC_SERV_DIR_NAME, WLAN_PROC_SERV_COUNTERS);
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSetNvramDefaults(ConsoleParameter_T parameters[])
{/*
	wdrv_dbRestoreNvramDefaults();
	printf("Restoring nvram defaults\n"); 
	printf("to take effect RESTART AP.\n"); */

	if (tiap_wcfg_set_defaults(wd)){
		PRINT_CONFIG_ERR_MSG("Failed to set defaults.");
		return;
	}
	printf("Restoring nvram defaults\n"); 
	printf("to take effect RESTART AP.\n");
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSetAgile(ConsoleParameter_T parameters[])
{/*
	if ((parameters[0].INTEGER != 0) && (parameters[0].INTEGER != 1))
	{
		printf("Invalid agile mode: %d\n", parameters[0].INTEGER);
		return;
	}
	wdrv_dbSetAgile((BOOL)(parameters[0].INTEGER));
	printf("Setting Agile = %d\n", parameters[0].INTEGER); 
	printf("to take effect RESTART AP.\n"); */

	PRINT_NOT_SUPPORTED_MSG("Setting Agile");
}

void wlanSetPreambleMode(ConsoleParameter_T parameters[])
{/*
	if ((parameters[0].SHORT != PREAMBLE_MODE_LONG) &&
		(parameters[0].SHORT != PREAMBLE_MODE_SHORT) &&
		(parameters[0].SHORT != PREAMBLE_MODE_LOCAL_STATIONS))
	{
		printf("Invalid preamble mode: %d\n", parameters[0].SHORT);
		return;
	}

	wdrv_dbSetPreambleMode(currentConfigWoneInex,parameters[0].SHORT);
	printf("Wone Index %d Setting Preamble mode = %d\n", currentConfigWoneInex, parameters[0].SHORT);
	printf("to take effect RESTART AP.\n"); */

	tiap_param_wone_t wone;

	if ((parameters[0].SHORT != TIAP_PARAM_PREAMBLE_MODE_LONG) &&
			(parameters[0].SHORT != TIAP_PARAM_PREAMBLE_MODE_SHORT) &&
			(parameters[0].SHORT != TIAP_PARAM_PREAMBLE_MODE_LOCAL_STATIONS))
	{
		printf("Invalid preamble mode: %d\n", parameters[0].SHORT);
		return;
	}

	wone.index = currentConfigWoneInex;
	wone.type = wone_pream_mode;
	wone.elem.pream_mode = parameters[0].SHORT;

	if( tiap_wcfg_set_wone(wd, &wone, UPDATE_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to set preamble mode.");
		return;
	}


	printf("Wone Index %d Setting Preamble mode = %d\n", currentConfigWoneInex, parameters[0].SHORT);
	printf("to take effect RESTART AP.\n");
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSetSlotTimeMode(ConsoleParameter_T parameters[])
{/*
	if ((parameters[0].SHORT != SLOTTIME_MODE_DISABLED) &&
		(parameters[0].SHORT != SLOTTIME_MODE_ENABLED) &&
		(parameters[0].SHORT != SLOTTIME_MODE_LOCAL_STATIONS) &&
		(parameters[0].SHORT != SLOTTIME_MODE_ENHANCED_DYNAMIC))
	{
		printf("Invalid Slot Time mode: %d\n", parameters[0].SHORT);
		return;
	}
	wdrv_dbSetSlotTimeMode(currentConfigWoneInex, parameters[0].SHORT);
	printf("Wone Index %d Setting Slot Time mode = %d\n",currentConfigWoneInex, parameters[0].SHORT); 
	printf("to take effect RESTART AP.\n"); */

	tiap_param_wone_t wone;
	
	if ((parameters[0].SHORT != TIAP_PARAM_SLOTTIME_MODE_DISABLED) &&
			(parameters[0].SHORT != TIAP_PARAM_SLOTTIME_MODE_ENABLED) &&
		(parameters[0].SHORT != TIAP_PARAM_SLOTTIME_MODE_LOCAL_STATIONS) &&
		(parameters[0].SHORT != TIAP_PARAM_SLOTTIME_MODE_ENHANCED_DYNAMIC))
	{
		printf("Invalid Slot Time mode: %d\n", parameters[0].SHORT);
		return;
	}


	wone.index = currentConfigWoneInex;
	wone.type = wone_slot_mode;
	wone.elem.slot_mode = parameters[0].SHORT;
	if( tiap_wcfg_set_wone(wd, &wone, UPDATE_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to set preamble mode.");
		return;
	}

	printf("Wone Index %d Setting Slot Time mode = %d\n",currentConfigWoneInex, parameters[0].SHORT); 
	printf("to take effect RESTART AP.\n");
}

void wlanSetSlotTimeThreshold(ConsoleParameter_T parameters[])
{/*
	wdrv_dbSetSlotTimeThreshold(parameters[0].SHORT);
	printf("Setting Slot Time threshold = %d\n", parameters[0].SHORT); 
	printf("to take effect RESTART AP.\n"); */

	PRINT_NOT_SUPPORTED_MSG("Slot Time threshold");
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSetERPProtectionMode(ConsoleParameter_T parameters[])
{/*
	if ((parameters[0].SHORT != PROTECTION_MODE_DISABLED) &&
		(parameters[0].SHORT != PROTECTION_MODE_ENABLED) &&
		(parameters[0].SHORT != PROTECTION_MODE_LOCAL_STATIONS) &&
		(parameters[0].SHORT != PROTECTION_MODE_DYNAMIC) &&
		(parameters[0].SHORT != PROTECTION_MODE_ENHANCED_DYNAMIC))
	{
		printf("Invalid Protection mode: %d\n", parameters[0].SHORT);
		return;
	}
	wdrv_dbSetERPProtectionMode(currentConfigWoneInex, parameters[0].SHORT);
	printf("Wone Index %d Setting Protection mode = %d\n",currentConfigWoneInex, parameters[0].SHORT); 
	printf("to take effect RESTART AP.\n"); */

	tiap_param_wone_t wone;
	
	if ((parameters[0].SHORT != TIAP_PARAM_PROTECTION_MODE_DISABLED) &&
		(parameters[0].SHORT != TIAP_PARAM_PROTECTION_MODE_ENABLED) &&
		(parameters[0].SHORT != TIAP_PARAM_PROTECTION_MODE_LOCAL_STATIONS) &&
		(parameters[0].SHORT != TIAP_PARAM_PROTECTION_MODE_DYNAMIC) &&
		(parameters[0].SHORT != TIAP_PARAM_PROTECTION_MODE_ENHANCED_DYNAMIC))
	{
		printf("Invalid Protection mode: %d\n", parameters[0].SHORT);
		return;
	}

	wone.index = currentConfigWoneInex;
	wone.type = wone_erp_mode;
	wone.elem.erp_mode = parameters[0].SHORT;
	if( tiap_wcfg_set_wone(wd, &wone, UPDATE_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to set preamble mode.");
		return;
	}

	printf("Wone Index %d Setting Protection mode = %d\n",currentConfigWoneInex, parameters[0].SHORT); 
	printf("to take effect RESTART AP.\n");
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSetERPProtectionType(ConsoleParameter_T parameters[])
{/*
	if ((parameters[0].SHORT != PROTECTION_TYPE_RTS_CTS) &&
		(parameters[0].SHORT != PROTECTION_TYPE_CTS_TOSELF) &&
		(parameters[0].SHORT != PROTECTION_TYPE_TI))
	{
		printf("Invalid Protection type: %d\n", parameters[0].SHORT);
		return;
	}
	// if protection type is TIprotection, set it internally to CTS_TOSELF,
	//   because there might be STAs that don't support TIprotection 
	if(parameters[0].SHORT != PROTECTION_TYPE_TI)
	{
		wdrv_dbSetERPProtectionTI(currentConfigWoneInex,PROTECTION_TI_DISABLED);
		wdrv_dbSetERPProtectionType(currentConfigWoneInex,parameters[0].SHORT);
	}
	else
	{
		wdrv_dbSetERPProtectionTI(currentConfigWoneInex,PROTECTION_TI_ENABLED);
		wdrv_dbSetERPProtectionType(currentConfigWoneInex,PROTECTION_TYPE_CTS_TOSELF);
	}
		
	printf("Wone Index %d Setting Protection type = %d\n",currentConfigWoneInex, parameters[0].SHORT); 
	printf("to take effect RESTART AP.\n"); */

	tiap_param_wone_t wone;

	if ((parameters[0].SHORT != TIAP_PARAM_PROTECTION_TYPE_RTS_CTS) &&
		(parameters[0].SHORT != TIAP_PARAM_PROTECTION_TYPE_CTS_TOSELF) &&
		(parameters[0].SHORT != TIAP_PARAM_PROTECTION_TYPE_TI))
	{
		printf("Invalid Protection type: %d\n", parameters[0].SHORT);
		return;
	}

	if(parameters[0].SHORT != TIAP_PARAM_PROTECTION_TYPE_TI)
	{
		wone.index = currentConfigWoneInex;
		wone.type = wone_erp_ti;
		wone.elem.erp_ti = TIAP_PARAM_PROTECTION_TI_DISABLED;
		if( tiap_wcfg_set_wone(wd, &wone, UPDATE_STORAGE) ) {
			PRINT_CONFIG_ERR_MSG("Failed to set preamble mode.");
			return;
		}

		wone.index = currentConfigWoneInex;
		wone.type = wone_erp_type;
		wone.elem.erp_type = parameters[0].SHORT;
		if( tiap_wcfg_set_wone(wd, &wone, UPDATE_STORAGE) ) {
			PRINT_CONFIG_ERR_MSG("Failed to set preamble mode.");
			return;
		}
	}
	else
	{
		wone.index = currentConfigWoneInex;
		wone.type = wone_erp_ti;
		wone.elem.erp_ti = TIAP_PARAM_PROTECTION_TI_ENABLED;
		if( tiap_wcfg_set_wone(wd, &wone, UPDATE_STORAGE) ) {
			PRINT_CONFIG_ERR_MSG("Failed to set preamble mode.");
			return;
		}

		wone.index = currentConfigWoneInex;
		wone.type = wone_erp_type;
		wone.elem.erp_type = TIAP_PARAM_PROTECTION_TYPE_CTS_TOSELF;
		if( tiap_wcfg_set_wone(wd, &wone, UPDATE_STORAGE) ) {
			PRINT_CONFIG_ERR_MSG("Failed to set preamble mode.");
			return;
		}
	}

	printf("Wone Index %d Setting Protection type = %d\n",currentConfigWoneInex, parameters[0].SHORT); 
	printf("to take effect RESTART AP.\n");
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSetDot11Mode(ConsoleParameter_T parameters[])
{/*
	char  rates[CCK_PBCC_RATES_MAX];
	UINT8 grates[OFDM_RATES_MAX];
	UINT8 ratesLen = 0;
	UINT8 pbccMode;	
    UINT8 RadioFrequency;
	

	if ((parameters[0].SHORT != DOT11_MODE_MIXED) &&
		(parameters[0].SHORT != DOT11_MODE_B_ONLY) &&
		(parameters[0].SHORT != DOT11_MODE_OFDM_ONLY) &&
		(parameters[0].SHORT != DOT11_MODE_B_ONLY_PLUS)&&
        (parameters[0].SHORT != DOT11_MODE_A_ONLY))
	{
		printf("Invalid Dot11 mode: %d\n", parameters[0].SHORT);
		return;
	}

    wdrv_dbGetRadioFrequency(&RadioFrequency);

    switch (RadioFrequency){
    case RADIO_BAND_2_4_AND_5GHZ:
        // nothing to check 
        break;

    case RADIO_BAND_2_4GHZ:
        // check for modes 0,1,2,3 
        if (parameters[0].SHORT == DOT11_MODE_A_ONLY) {
            printf("  Invalid Dot11 mode: %d RadioFrequency is 2.4 GHZ only \n", parameters[0].SHORT);
            return;
        }

        break;
    case RADIO_BAND_5GHZ:
        if(parameters[0].SHORT != DOT11_MODE_A_ONLY){
            printf("  Invalid Dot11 mode: %d RadioFrequency is 5 GHZ only \n", parameters[0].SHORT);
            return;
        }
        break;
    default:
        printf("Invalid RadioFrequency parameter \n");
        return;
        break;
        

    }

	wdrv_dbSetDot11Mode(currentConfigWoneInex,parameters[0].SHORT);

	whal_nvram_setDefaultsPartial(currentConfigWoneInex, (UINT8)parameters[0].SHORT);


	printf("Wone Index %d Dot11 mode = %d\n",currentConfigWoneInex, parameters[0].SHORT); 
	printf("to take effect AP is restarting now...\n");
    rt_osTimerWakeAfter(200);
	consoleReboot(0); */


	UINT8 RadioFrequency;
	char buf[4096];
	int len;

	if ((parameters[0].SHORT != TIAP_PARAM_D11MODE_MIXED) &&
			(parameters[0].SHORT != TIAP_PARAM_D11MODE_B) &&
			(parameters[0].SHORT != TIAP_PARAM_D11MODE_OFDM) &&
			(parameters[0].SHORT != TIAP_PARAM_D11MODE_B_PLUS)&&
			(parameters[0].SHORT != TIAP_PARAM_D11MODE_A))
	{
		printf("Invalid Dot11 mode: %d\n", parameters[0].SHORT);
		return;
	}

	if( (len = dda_read_proc_entry(WLAN_PROC_SERV_DIR_NAME,
					WLAN_PROC_SERV_RADIO_FREQ, buf, sizeof(buf))) == -1 )
		PRINT_PROC_READ_ERR(WLAN_PROC_SERV_DIR_NAME, WLAN_PROC_SERV_RADIO_FREQ);

	sscanf(buf, "%hhu", &RadioFrequency);

	switch (RadioFrequency){
		case TIAP_PARAM_RADIO_BAND_2_4_AND_5GHZ:
			// nothing to check 
			break;

		case TIAP_PARAM_RADIO_BAND_2_4GHZ:
			// check for modes 0,1,2,3 
			if (parameters[0].SHORT == TIAP_PARAM_D11MODE_A) {
				printf("  Invalid Dot11 mode: %d RadioFrequency is 2.4 GHZ only \n", parameters[0].SHORT);
				return;
			}

			break;
		case TIAP_PARAM_RADIO_BAND_5GHZ:
			if(parameters[0].SHORT != TIAP_PARAM_D11MODE_A){
				printf("  Invalid Dot11 mode: %d RadioFrequency is 5 GHZ only \n", parameters[0].SHORT);
				return;
			}
			break;
		default:
			printf("Invalid RadioFrequency parameter \n");
			return;
			break;
	}

	if( tiap_wcfg_set_dot11mode(wd, parameters[0].SHORT, currentConfigWoneInex, UPDATE_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to set .11 mode.");
		return;
	}

	printf("Wone Index %d Dot11 mode = %d\n",currentConfigWoneInex, parameters[0].SHORT); 
	printf("to take effect AP is restarting now...\n");
	tiap_wcfg_restart(wd, &driver_load_failed);
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSetCWmin(ConsoleParameter_T parameters[])
{/*
	if ((parameters[0].SHORT != DOT11_CWMIN_DISABLED) &&
		(parameters[0].SHORT != DOT11_CWMIN_ENABLED) &&
		(parameters[0].SHORT != DOT11_CWMIN_DOT11_MODE))
	{
		printf("Invalid CWmin mode: %d\n", parameters[0].SHORT);
		return;
	}
	wdrv_dbSetCwMinMode(currentConfigWoneInex,parameters[0].SHORT);
	printf("Wone Index %d CWmin mode = %d\n", currentConfigWoneInex, parameters[0].SHORT); 
	printf("to take effect RESTART AP.\n"); */

	tiap_param_wone_t wone;

	if ((parameters[0].SHORT != TIAP_PARAM_DOT11_CWMIN_DISABLED) &&
		(parameters[0].SHORT != TIAP_PARAM_DOT11_CWMIN_ENABLED) &&
		(parameters[0].SHORT != TIAP_PARAM_DOT11_CWMIN_DOT11_MODE))
	{
		printf("Invalid CWmin mode: %d\n", parameters[0].SHORT);
		return;
	}

	wone.index = currentConfigWoneInex;
	wone.type = wone_cwmin_mode;
	wone.elem.cwmin_mode = parameters[0].SHORT;
	if( tiap_wcfg_set_wone(wd, &wone, UPDATE_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to set CWmin mode.");
		return;
	}

	printf("Wone Index %d CWmin mode = %d\n", currentConfigWoneInex, parameters[0].SHORT); 
	printf("to take effect RESTART AP.\n");
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSetRadioCalibrateOption(ConsoleParameter_T parameters[])
{/*
	wdrv_dbSetRadioCalibrateOption(parameters[0].INTEGER);
	printf("Radio Calibrate Option = %d\n", parameters[0].INTEGER); 
	printf("to take effect RESTART AP.\n"); */

	if( tiap_wcfg_set_basic(wd, TIAP_PARAM_RADIO_CAL, (__u32 *)&parameters[0].INTEGER, UPDATE_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to set radio calibrate option.");
		return;
	}
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSetRadioCalibrateInterval(ConsoleParameter_T parameters[])
{/*
	wdrv_dbSetRadioCalibrateInterval(parameters[0].SHORT);
	printf("Radio Calibrate Interval = %d\n", parameters[0].SHORT); 
	printf("to take effect RESTART AP.\n");*/
	
	if( tiap_wcfg_set_basic(wd, TIAP_PARAM_RADIO_CAL_INT, (__u16 *)&parameters[0].SHORT, UPDATE_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to set radio calibrate interval.");
		return;
	}
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSetBssIsolation(ConsoleParameter_T parameters[])
{
	tiap_param_bss_br_state_t bss_bridge_state;
	int bss_isolation;

	bss_isolation = parameters[0].INTEGER;

	switch (bss_isolation)
	{
		case DISABLE:
			bss_bridge_state = TIAP_PARAM_BSS_BR_ENABLE;
			break;
		case ENABLE:
			bss_bridge_state = TIAP_PARAM_BSS_BR_BLOCK;
			break;
		default:
			printf("\nInvalid Bss Isolation State: can be 1 or 0\n");
			return;

	}
	if (tiap_wcfg_set_basic(wd, TIAP_PARAM_BSS_BRIDGE_STATE, &bss_bridge_state, UPDATE_STORAGE))
	{
		PRINT_CONFIG_ERR_MSG("Failed to set BSS Bridge isolation state.");
		return;
	}
	printf("\n\nSetting Bss Isolation state to %s\n\n", (bss_isolation == DISABLE) ? "Disabled" : "Enabled");
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanPrintBssIsolationState(ConsoleParameter_T parameters[])
{
	__u32 bss_bridge_state;

	if (tiap_wcfg_get_basic(wd, TIAP_PARAM_BSS_BRIDGE_STATE, &bss_bridge_state, !FROM_STORAGE || driver_load_failed)){
		PRINT_CONFIG_ERR_MSG("Failed to get BSS isolation state.");
		return;
	}
	printf("\n\nBss Isolation state is %s\n\n", (bss_bridge_state == TIAP_PARAM_BSS_BR_BLOCK) ? "Enabled" : "Disabled" );
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSetRadarDetect(ConsoleParameter_T parameters[])
{
	__u8  radarDetect = parameters[0].SHORT;

	if (tiap_wcfg_set_basic(wd, TIAP_PARAM_RADAR_DETECT_MODE,
							&radarDetect, UPDATE_STORAGE)) {
		PRINT_CONFIG_ERR_MSG("Failed to set radar detect mode.");
		return;
	}
	printf("\n\nSetting radar detect mode to %s\n\n", (parameters[0].SHORT) ? "Enabled" : "Disabled");
	printf("to take effect RESTART AP.\n");

}


/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanPrintRadarDetect(ConsoleParameter_T parameters[])
{
	__u8 radar_detect;

	if (tiap_wcfg_get_basic(wd, TIAP_PARAM_RADAR_DETECT_MODE, &radar_detect, !FROM_STORAGE || driver_load_failed)){
		PRINT_CONFIG_ERR_MSG("Failed to get radar detect mode.");
		return;
	}
	printf("\n\nRadar detect mode is %s\n\n", (radar_detect) ?  "Enabled" : "Disabled");
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSetApBridgeMode(ConsoleParameter_T parameters[])
{
	__u8	bridgeMode = (__u8) parameters[0].INTEGER;

	if (tiap_wcfg_set_basic(wd, TIAP_PARAM_AP_BRIDGE_MODE, &bridgeMode, 
							UPDATE_STORAGE)) {
		PRINT_CONFIG_ERR_MSG("Failed to set ap bridge mode.");
		return;
	}
	printf("\n\nSetting ap bridge mode to %s\n\n", (parameters[0].INTEGER) ? "Enabled" : "Disabled");
    if(parameters[0].INTEGER)
    {
        printf("If QoS is Enabled:\n1. Max rate will be set to 36Mbps.\n2. Max Lifetime parametes in all AC will be set to 9,000,000.\n\n");
    }
	printf("to take effect RESTART AP.\n");
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSetPBCCAlgorithmMode(ConsoleParameter_T parameters[])
{/*

	UINT8 band;
	wdrv_dbGetBand(currentConfigWoneInex,&band);
    if (band == RADIO_BAND_5GHZ)
        {
            printf("\nError Active Set Wone Index is %d\n",currentConfigWoneInex);  
            printf("802.11 A Not support PBCC Algorithm\n");  
            return;
        }

	if ((parameters[0].SHORT != PBCC_ALGO_ENABLED) && (parameters[0].SHORT != PBCC_ALGO_DISABLED) &&
		(parameters[0].SHORT != PBCC_ALGO_LOCAL_STATIONS) && (parameters[0].SHORT != PBCC_ALGO_DYNAMIC) &&
		(parameters[0].SHORT != PBCC_ALGO_ENHANCED_DYNAMIC))
	{
		printf("Invalid PBCC algorithm mode: %d\n", parameters[0].SHORT);
		return;
	}
	wdrv_dbSetPBCCAlgorithmMode(currentConfigWoneInex, parameters[0].SHORT);
	printf("Setting PBCC algorithm mode = %d\n", parameters[0].SHORT); 
	printf("to take effect RESTART AP.\n"); */

	tiap_param_wone_t wone;
	
	wone.index = currentConfigWoneInex;
	wone.type = wone_band;

	if( tiap_wcfg_get_wone(wd, &wone, FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get band.");
		return;
	}
	
	if (wone.elem.band == TIAP_PARAM_RADIO_BAND_5GHZ)
	{
		printf("\nError Active Set Wone Index is %d\n",currentConfigWoneInex);  
		printf("802.11 A Not support PBCC Algorithm\n");  
		return;
	}

	if ((parameters[0].SHORT != TIAP_PARAM_PBCC_ALGO_ENABLED) &&
		(parameters[0].SHORT != TIAP_PARAM_PBCC_ALGO_DISABLED) &&
		(parameters[0].SHORT != TIAP_PARAM_PBCC_ALGO_LOCAL_STATIONS) && 
		(parameters[0].SHORT != TIAP_PARAM_PBCC_ALGO_DYNAMIC) &&
		(parameters[0].SHORT != TIAP_PARAM_PBCC_ALGO_ENHANCED_DYNAMIC))
	{
		printf("Invalid PBCC algorithm mode: %d\n", parameters[0].SHORT);
		return;
	}

	wone.index = currentConfigWoneInex;
	wone.type = wone_pbcc;
	wone.elem.pbcc_mode = parameters[0].SHORT;
	
	if( tiap_wcfg_set_wone(wd, &wone, UPDATE_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to set pbcc algorithm mode.");
		return;
	}
	
	printf("Setting PBCC algorithm mode = %d\n", parameters[0].SHORT); 
	printf("to take effect RESTART AP.\n"); 
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSetDynAlgoTimeout(ConsoleParameter_T parameters[])
{/*
	wdrv_dbSetDynamicAlgorithmsTimeout(parameters[0].INTEGER);
	printf("Setting Enhanced Dynamic Algorithm timeout = %d\n", parameters[0].INTEGER); 
	printf("to take effect RESTART AP.\n");*/

	PRINT_NOT_SUPPORTED_MSG("Enhanced Dynamic Algorithm timeout");
}

/************************************************************************/
/*						Hal sub-menu functions							*/
/************************************************************************/

/*!	\brief 
  	\param 

  	\return 
  
*/
void whalMenu_printFwVersion(ConsoleParameter_T parameters[]){
	//whal_apiPrintFwVersion();
	if(dda_print_proc_entry(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_FWVER) == -1)
		PRINT_PROC_READ_ERR(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_FWVER);
	
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void whalMenu_printAcxParams(ConsoleParameter_T parameters[]){
	//whal_apiPrintAcxParams();
	if(dda_print_proc_entry(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_ACX_PARAM) == -1)
		PRINT_PROC_READ_ERR(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_ACX_PARAM);

}

/*!	\brief 
  	\param 

  	\return 
  
*/
void whalMenu_printTxInfo(ConsoleParameter_T parameters[]){
	char buf[GENERIC_IOCTL_BUF_SIZE];

	DRIVER_LOADED_ASSERT;
	
	if (ioctl_get_tx_info(parameters[0].INTEGER, parameters[1].INTEGER,  buf, GENERIC_IOCTL_BUF_SIZE, wd->dev_name)) 
		puts("ioctl_get_tx_info failed !");
	else
		puts(buf);
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void whalMenu_printRxInfo(ConsoleParameter_T parameters[]){
	//whal_apiPrintRxInfo();

	if(dda_print_proc_entry(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_RX_INFO) == -1)
		PRINT_PROC_READ_ERR(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_RX_INFO);
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void whalMenu_printTxQueue(ConsoleParameter_T parameters[]){
	//whal_apiPrintTxQueue(parameters[0].INTEGER);
	
	char buf[GENERIC_IOCTL_BUF_SIZE];

	DRIVER_LOADED_ASSERT;

	if (ioctl_get_tx_queue(parameters[0].INTEGER, buf, GENERIC_IOCTL_BUF_SIZE, wd->dev_name)) 
		puts("ioctl_get_tx_queue failed !");
	else
		puts(buf);

}

/*!	\brief 
  	\param 

  	\return 
  
*/
void whalMenu_printRxQueue(ConsoleParameter_T parameters[]){
	//whal_apiPrintRxQueue(parameters[0].INTEGER);
	if(dda_print_proc_entry(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_RX_QUEUE) == -1)
		PRINT_PROC_READ_ERR(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_RX_QUEUE);
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void whalMenu_printRxBufs(ConsoleParameter_T parameters[]){
	//whal_apiPrintRxBufs();
	if(dda_print_proc_entry(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_RX_BUFFS) == -1)
		PRINT_PROC_READ_ERR(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_RX_BUFFS);
	
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void whalMenu_printIntr(ConsoleParameter_T parameters[]){
	//whal_apiPrintIntr();
	if(dda_print_proc_entry(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_INTR) == -1)
		PRINT_PROC_READ_ERR(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_INTR);
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void whalMenu_printMbox(ConsoleParameter_T parameters[]){
	//whal_apiPrintMbox();
	if(dda_print_proc_entry(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_MBOX) == -1)
		PRINT_PROC_READ_ERR(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_MBOX);

}

/*!	\brief 
  	\param 

  	\return 
  
*/
void whalMenu_printMboxErrors(ConsoleParameter_T parameters[]){
	//whal_apiPrintMbox();
	if(dda_print_proc_entry(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_MBOX_ERR) == -1)
		PRINT_PROC_READ_ERR(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_MBOX_ERR);

}


/*!	\brief 
  	\param 

  	\return 
  
*/
void whalMenu_printAcxStat(ConsoleParameter_T parameters[]){
	//whal_apiPrintAcxStatistics();
	if(dda_print_proc_entry(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_ACXSTAT) == -1)
		PRINT_PROC_READ_ERR(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_ACXSTAT);
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void whalMenu_printRestartStat(ConsoleParameter_T parameters[]){
	//whal_apiPrintRestartStatistics();
	if(dda_print_proc_entry(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_RESTART_STAT) == -1)
		PRINT_PROC_READ_ERR(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_RESTART_STAT);
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void whalMenu_EnableRestartWA(ConsoleParameter_T parameters[])
{/*
	if ((parameters[0].SHORT != 0) && (parameters[0].SHORT != 1))
	{
		printf("Invalid enable restart WA value: %d\n", parameters[0].SHORT);
		return;
	}
	wdrv_dbSetEnRestart(parameters[0].SHORT, TRUE);	
	printf("Enable restart WA = %d\n", parameters[0].SHORT); 
	printf("to take effect RESTART AP.\n"); */
	
	if ((parameters[0].SHORT != 0) && (parameters[0].SHORT != 1))
	{
		printf("Invalid enable restart WA value: %d\n", parameters[0].SHORT);
		return;
	}

	set_enable_wa_restart(parameters[0].SHORT);
	printf("Enable restart WA = %d\n", parameters[0].SHORT); 
	printf("to take effect RESTART AP.\n");
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void whalMenu_EnableInterfWA(ConsoleParameter_T parameters[])
{
	tiap_param_wd_t wa;
	UINT8 val = (UINT8)parameters[0].INTEGER;

	if ((val != 0) && (val != 1) && (val != 2))
	{
		printf("Invalid enable interference WA value: %d\n", parameters[0].SHORT);
		return;
	}

	wa.cmd = TIAP_PARAM_WD_INTERF;
	wa.value.val8 = val;
	
	if(tiap_wcfg_set_wd(wd, &wa, UPDATE_STORAGE))
		PRINT_CONFIG_ERR_MSG("Failed to set interference WA .");
	else
	{
		printf("Enable intereference WA = %d\n", parameters[0].SHORT); 
		printf("to take effect RESTART AP.\n");
	}

}


/*!	\brief 
  	\param 

  	\return 
  
*/
void whalMenu_gpio_owner(ConsoleParameter_T parameters[])
{
	if (parameters[0].SHORT == 2) {
		char buf[50];
		dda_read_proc_entry(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_LED, buf, sizeof(buf));
		printf("%s\n", buf); fflush(stdout);
		return;		
	}
	
	if ((parameters[0].SHORT != 0) && (parameters[0].SHORT != 1))
	{
		printf("Invalid gpio ownership value: %d\n", parameters[0].SHORT);
		return;
	}

	tiap_wcfg_set_gpio_owner(wd, parameters[0].SHORT, UPDATE_STORAGE);
	printf("Enable restart WA = %d\n", parameters[0].SHORT); 
	printf("to take effect RESTART AP.\n");
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void whalMenu_mediumUsage(ConsoleParameter_T parameters[]){
	//whal_apiPrintIntr();
	if(dda_print_proc_entry(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_MEDIUM_USAGE) == -1)
		PRINT_PROC_READ_ERR(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_MEDIUM_USAGE);
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void whalMenu_setWAoptions(ConsoleParameter_T parameters[])
{/*
	if ((parameters[0].SHORT != 0) && (parameters[0].SHORT != 1))
	{
		printf("Invalid Rx Free Mem option: %d\n", parameters[0].SHORT);
		return;
	}
	if (parameters[1].SHORT > 30)
	{
		printf("Invalid Beacon not sent option: %d\n", parameters[1].SHORT);
		return;
	}
	if (parameters[2].SHORT > 30)
	{
		printf("Invalid Tx Acx free option: %d\n", parameters[2].SHORT);
		return;
	}
	if (parameters[3].SHORT > 30)
	{
		printf("Invalid Rx Cons FCS option: %d\n", parameters[3].SHORT);
		return;
	}


	wdrv_dbSetWAoptions(parameters[0].SHORT, parameters[1].SHORT, parameters[2].SHORT, parameters[3].SHORT, IMMEDIATELY);	
	printf("Rx Free Mem option     = %d\n", parameters[0].SHORT); 
	printf("Beacon not sent option = %d\n", parameters[1].SHORT); 
	printf("Tx Acx free option     = %d\n", parameters[2].SHORT); 
	printf("Rx Cons FCS threshold  = %d\n", parameters[3].SHORT); 
	printf("to take effect RESTART AP.\n"); */

	tiap_param_wd_t wa;

	if ((parameters[0].SHORT != 0) && (parameters[0].SHORT != 1))
	{
		printf("Invalid Rx Free Mem option: %d\n", parameters[0].SHORT);
		return;
	}
	if (parameters[1].SHORT > 30)
	{
		printf("Invalid Beacon not sent option: %d\n", parameters[1].SHORT);
		return;
	}
	if (parameters[2].SHORT > 30)
	{
		printf("Invalid Tx Acx free option: %d\n", parameters[2].SHORT);
		return;
	}
	if (parameters[3].SHORT > 30)
	{
		printf("Invalid Rx Cons FCS option: %d\n", parameters[3].SHORT);
		return;
	}
    if ((parameters[4].SHORT != 0) && (parameters[4].SHORT != 1))
	{
		printf("Invalid ACI WA option: %d\n", parameters[4].SHORT);
		return;
	}
	
	if ((parameters[5].SHORT != 0) && (parameters[5].SHORT != 1))
	{
		printf("Invalid NB WA option: %d\n", parameters[5].SHORT);
		return;
	}


	wa.cmd = TIAP_PARAM_WD_RXFREE;
	wa.value.val8 = parameters[0].SHORT;

	if(tiap_wcfg_set_wd(wd, &wa, !UPDATE_STORAGE))
	{
		PRINT_CONFIG_ERR_MSG("Failed to set Rx Free Mem WA.");
		return;
	}

	wa.cmd = TIAP_PARAM_WD_BEACONS;
	wa.value.val8 = parameters[1].SHORT;

	if(tiap_wcfg_set_wd(wd, &wa, !UPDATE_STORAGE))
	{
		PRINT_CONFIG_ERR_MSG("Failed to set Beacon not sent WA.");
		return;
	}

	wa.cmd = TIAP_PARAM_WD_TXSTUCK;
	wa.value.val8 = parameters[2].SHORT;

	if(tiap_wcfg_set_wd(wd, &wa, !UPDATE_STORAGE))
	{
		PRINT_CONFIG_ERR_MSG("Failed to set Tx Acx free WA.");
		return;
	}

	wa.cmd = TIAP_PARAM_WD_FCSERR;
	wa.value.val16 = parameters[3].SHORT;

	if(tiap_wcfg_set_wd(wd, &wa, !UPDATE_STORAGE))
	{
		PRINT_CONFIG_ERR_MSG("Failed to set Rx Cons FCS WA.");
		return;
	}
	wa.cmd = TIAP_PARAM_WD_ACI;
	wa.value.val8 = parameters[4].SHORT;

	if(tiap_wcfg_set_wd(wd, &wa, !UPDATE_STORAGE))
	{
		PRINT_CONFIG_ERR_MSG("Failed to set ACI WA.");
		return;
	}

	wa.cmd = TIAP_PARAM_WD_NB;
	wa.value.val8 = parameters[5].SHORT;

	/* write to the storage only in the last call */
	if(tiap_wcfg_set_wd(wd, &wa, UPDATE_STORAGE))
	{
		PRINT_CONFIG_ERR_MSG("Failed to set NB WA.");
		return;
	}

	printf("Rx Free Mem option     = %d\n", parameters[0].SHORT); 
	printf("Beacon not sent option = %d\n", parameters[1].SHORT); 
	printf("Tx Acx free option     = %d\n", parameters[2].SHORT); 
	printf("Rx Cons FCS threshold  = %d\n", parameters[3].SHORT); 
    printf("ACI WA option          = %d\n", parameters[4].SHORT); 
	printf("NB WA option           = %d\n", parameters[5].SHORT); 

	printf("to take effect RESTART AP.\n"); 

}

void whalMenu_setAciRateThresh(ConsoleParameter_T parameters[])
{
	tiap_param_interference_wd_t aci_rate_thr;

	aci_rate_thr.cmd = TIAP_PARAM_ACI_RATE_THR;
	aci_rate_thr.value.val32 = parameters[0].INTEGER;

	if (tiap_wcfg_set_interf_wd_config(wd, &aci_rate_thr, UPDATE_STORAGE))
	{
		PRINT_CONFIG_ERR_MSG("Failed to set ACI rate threshold.");
		return;
	}
	printf("to take effect RESTART AP.\n");
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void whalMenu_setAciEventThresh(ConsoleParameter_T parameters[])
{
	tiap_param_interference_wd_t aci_event_thr;

	aci_event_thr.cmd = TIAP_PARAM_ACI_EVENT_THR;
	aci_event_thr.value.val32 = parameters[0].INTEGER;

	if (tiap_wcfg_set_interf_wd_config(wd, &aci_event_thr, UPDATE_STORAGE))
	{
		PRINT_CONFIG_ERR_MSG("Failed to set ACI rate threshold.");
		return;
	}
	printf("to take effect RESTART AP.\n");
}

void whalMenu_setAciTimeout(ConsoleParameter_T parameters[])
{
	tiap_param_interference_wd_t interf_param;

	interf_param.cmd = TIAP_PARAM_ACI_TIMEOUT;
	interf_param.value.val32 = parameters[0].INTEGER;

	if (tiap_wcfg_set_interf_wd_config(wd, &interf_param, UPDATE_STORAGE))
	{
		PRINT_CONFIG_ERR_MSG("Failed to set ACI state timeout.");
		return;
	}
	printf("to take effect RESTART AP.\n");
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void whalMenu_setNbTimeout(ConsoleParameter_T parameters[])
{
	tiap_param_interference_wd_t interf_param;

	interf_param.cmd = TIAP_PARAM_NB_TIMEOUT;
	interf_param.value.val32 = parameters[0].INTEGER;

	if (tiap_wcfg_set_interf_wd_config(wd, &interf_param, UPDATE_STORAGE))
	{
		PRINT_CONFIG_ERR_MSG("Failed to set NB state timeout.");
		return;
	}
	printf("to take effect RESTART AP.\n");
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void whalMenu_setPdThresh(ConsoleParameter_T parameters[])
{
	tiap_param_interference_wd_t interf_param;

	interf_param.cmd = TIAP_PARAM_PD_LOW;
	interf_param.value.val16 = parameters[0].INTEGER;

	if (tiap_wcfg_set_interf_wd_config(wd, &interf_param, !UPDATE_STORAGE))
	{
		PRINT_CONFIG_ERR_MSG("Failed to set low PD threshold.");
		return;
	}

	interf_param.cmd = TIAP_PARAM_PD_HIGH;
	interf_param.value.val16 = parameters[1].INTEGER;

	/*update storage only with the last parameter*/
	if (tiap_wcfg_set_interf_wd_config(wd, &interf_param, UPDATE_STORAGE))
	{
		PRINT_CONFIG_ERR_MSG("Failed to set high PD threshold.");
		return;
	}

	printf("to take effect RESTART AP.\n");
}


/*!	\brief 
  	\param 

  	\return 
  
*/
void whalMenu_setBusyMediumThresh(ConsoleParameter_T parameters[])
{
	tiap_param_interference_wd_t interf_param;

	interf_param.cmd = TIAP_PARAM_MEDIUM_BUSY_THR;
	interf_param.value.val32 = parameters[0].INTEGER;

	if (tiap_wcfg_set_interf_wd_config(wd, &interf_param, UPDATE_STORAGE))
	{
		PRINT_CONFIG_ERR_MSG("Failed to set busy medium threshold.");
		return;
	}
	printf("to take effect RESTART AP.\n");
}

/* Prints interference Workaround configuration*/
/*!	\brief 
  	\param 

  	\return 
  
*/
void whalMenu_showACIparams(ConsoleParameter_T parameters[])
{
	if(dda_print_proc_entry(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_INTERF_WD_PARAMS) == -1)
		PRINT_PROC_READ_ERR(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_INTERF_WD_PARAMS);
}


/*!	\brief 
  	\param 

  	\return 
  
*/
void whalMenu_EnergyDetectShow(ConsoleParameter_T parameters[])
{/*
	UINT8 energyDetect;

	wdrv_dbGetEnergyDetect(&energyDetect);
	if(energyDetect)
		printf("Energy Detect Enabled\n");
	else
		printf("Energy Detect Disabled\n"); */

	UINT8 energy_detect;

	if(tiap_wcfg_get_basic(wd, TIAP_PARAM_ENERGY_DETECT, &energy_detect, !FROM_STORAGE || driver_load_failed))
	{
		PRINT_CONFIG_ERR_MSG("Failed to get energy detect configuration.");
		return;
	}

	if(energy_detect)
		printf("Energy Detect Enabled\n");
	else
		printf("Energy Detect Disabled\n");

}

/*!	\brief 
  	\param 

  	\return 
  
*/
void whalMenu_EnableEnergyDetect(ConsoleParameter_T parameters[])
{/*
	if ((parameters[0].SHORT != 0) && (parameters[0].SHORT != 1))
	{
		printf("Invalid energy enable value: %d\n", parameters[0].SHORT);
		return;
	}
	wdrv_dbSetEnergyDetect(parameters[0].SHORT, IMMEDIATELY);
	printf("Enable Energy Detect = %d\n", parameters[0].SHORT); 
	printf("to take effect RSTART AP.\n"); */

	if ((parameters[0].SHORT != 0) && (parameters[0].SHORT != 1))
	{
		printf("Invalid energy enable value: %d\n", parameters[0].SHORT);
		return;
	}
	
	if(tiap_wcfg_set_basic(wd, TIAP_PARAM_ENERGY_DETECT, &(parameters[0].SHORT), UPDATE_STORAGE))
	{
		PRINT_CONFIG_ERR_MSG("Failed to set energy detect.");
		return;
	}

	printf("Enable Energy Detect = %d\n", parameters[0].SHORT); 
	printf("to take effect RSTART AP.\n");

}

/************************************************************************/
/*						Hal Diag sub-menu functions						*/
/************************************************************************/

/*!	\brief 
  	\param 

  	\return 
  
*/
void whalMenu_eed(ConsoleParameter_T parameters[]){
   //whal_apiDumpEEPROM();

	if (dda_print_proc_entry(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_EED) == -1) {
		PRINT_PROC_READ_ERR(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_EED);
	}
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void whalMenu_eer(ConsoleParameter_T parameters[]){

   //whal_apiReadEEPROM(parameters[0].INTEGER);

	char buf[GENERIC_IOCTL_BUF_SIZE];

	DRIVER_LOADED_ASSERT;

	if (ioctl_get_eer(parameters[0].INTEGER, buf, GENERIC_IOCTL_BUF_SIZE, wd->dev_name)) 
		puts("ioctl_get_eer failed !");
	else
		puts(buf);

	
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void whalMenu_eew(ConsoleParameter_T parameters[]){
   //whal_apiWriteEEPROM(parameters[0].INTEGER, parameters[1].INTEGER);
	proc_params proc_pars;
	int ret;

	DRIVER_LOADED_ASSERT;

	proc_pars.s.magic = PROC_PARAMS_MAGIC;
	proc_pars.s.Param1 = parameters[0].INTEGER;
	proc_pars.s.Param2 = parameters[1].INTEGER;

	ret = dda_write_proc_entry(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_EEW, 
		(char *)&proc_pars, sizeof(proc_pars));

	if (ret != sizeof(proc_pars))
		PRINT_PROC_WRITE_ERR(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_EEW);
	
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void whalMenu_SetBssPs(ConsoleParameter_T parameters[]){
	//whal_apiSetBssPs(parameters[0].INTEGER);
	proc_params proc_pars;
	int ret;

	DRIVER_LOADED_ASSERT;

	proc_pars.s.magic = PROC_PARAMS_MAGIC;
	proc_pars.s.Param1 = parameters[0].INTEGER;

	ret = dda_write_proc_entry(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_SETPS_MODE, 
		(char *)&proc_pars, sizeof(proc_pars));

	if (ret != sizeof(proc_pars))
		PRINT_PROC_WRITE_ERR(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_SETPS_MODE);

}

/*!	\brief 
  	\param 

  	\return 
  
*/
void whalMenu_SetTxCompleteTrsh(ConsoleParameter_T parameters[]){
	/*int sts;

	sts = whal_apiSetTxCompleteThreshold(parameters[0].INTEGER);
	if(sts == OK)
	{
		printf("The Tx Complete Interrupt Threshold set to %d\n",parameters[0].INTEGER);
	} */

	proc_params proc_pars;
	int ret;

	DRIVER_LOADED_ASSERT;

	proc_pars.s.magic = PROC_PARAMS_MAGIC;
	proc_pars.s.Param1 = parameters[0].INTEGER;

	ret = dda_write_proc_entry(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_SET_TX_COMPTRSH, 
		(char *)&proc_pars, sizeof(proc_pars));

	if (ret != sizeof(proc_pars))
		PRINT_PROC_WRITE_ERR(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_SET_TX_COMPTRSH);
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void whalMenu_PrintRegs(ConsoleParameter_T parameters[]){
	//whal_apiPrintRegs();
	if(dda_print_proc_entry(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_PRINTREGS) == -1)
		PRINT_PROC_READ_ERR(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_PRINTREGS);

}

/*!	\brief 
  	\param 

  	\return 
  
*/
void whalMenu_PhyRegWrite(ConsoleParameter_T parameters[]){
	//whal_apiPhyWrite(parameters[0].INTEGER, parameters[1].INTEGER);
	proc_params proc_pars;
	int ret;

	DRIVER_LOADED_ASSERT;

	proc_pars.s.magic = PROC_PARAMS_MAGIC;
	proc_pars.s.Param1 = parameters[0].INTEGER;
	proc_pars.s.Param2 = parameters[1].INTEGER;

	ret = dda_write_proc_entry(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_PHY_REG_WRITE, 
		(char *)&proc_pars, sizeof(proc_pars));

	if (ret != sizeof(proc_pars))
		PRINT_PROC_WRITE_ERR(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_PHY_REG_WRITE);

}

/*!	\brief 
  	\param 

  	\return 
  
*/
void whalMenu_PhyRegRead(ConsoleParameter_T parameters[]){
	//whal_apiPhyRead(parameters[0].INTEGER);
	char buf[GENERIC_IOCTL_BUF_SIZE];

	DRIVER_LOADED_ASSERT;

	if (ioctl_get_phy_reg_read(parameters[0].INTEGER, buf, GENERIC_IOCTL_BUF_SIZE, wd->dev_name)) 
		puts("ioctl_get_phy_reg_read failed !");
	else
		puts(buf);

}

void whalMenu_MacRegWrite(ConsoleParameter_T parameters[]){
	//whal_apiMacWrite(parameters[0].INTEGER, parameters[1].INTEGER);

	proc_params proc_pars;
	int ret;

	DRIVER_LOADED_ASSERT;

	proc_pars.s.magic = PROC_PARAMS_MAGIC;
	proc_pars.s.Param1 = parameters[0].INTEGER;
	proc_pars.s.Param2 = parameters[1].INTEGER;

	ret = dda_write_proc_entry(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_MAC_REG_WRITE, 
		(char *)&proc_pars, sizeof(proc_pars));

	if (ret != sizeof(proc_pars))
		PRINT_PROC_WRITE_ERR(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_MAC_REG_WRITE);
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void whalMenu_MacRegRead(ConsoleParameter_T parameters[]){
	//whal_apiMacRead(parameters[0].INTEGER);
	char buf[GENERIC_IOCTL_BUF_SIZE];

	DRIVER_LOADED_ASSERT;

	if (ioctl_get_mac_reg_read(parameters[0].INTEGER, buf, GENERIC_IOCTL_BUF_SIZE, wd->dev_name)) 
		puts("ioctl_get_mac_reg_read failed !");
	else
		puts(buf);

}

void whalMenu_RxPromiscuous(ConsoleParameter_T parameters[]){
	//whal_apiRxPromiscuous(parameters[0].INTEGER);
	proc_params proc_pars;
	int ret;

	DRIVER_LOADED_ASSERT;
	
	proc_pars.s.magic = PROC_PARAMS_MAGIC;
	proc_pars.s.Param1 = parameters[0].INTEGER;

	ret = dda_write_proc_entry(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_SET_RX_PROMIS, 
		(char *)&proc_pars, sizeof(proc_pars));

	if (ret != sizeof(proc_pars))
		PRINT_PROC_WRITE_ERR(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_SET_RX_PROMIS);

}

/*!	\brief 
  	\param 

  	\return 
  
*/
void whalMenu_ModeSniffer(ConsoleParameter_T parameters[]){
	//whal_apiModeSniffer(parameters[0].INTEGER);
	proc_params proc_pars;
	int ret;

	DRIVER_LOADED_ASSERT;

	proc_pars.s.magic = PROC_PARAMS_MAGIC;
	proc_pars.s.Param1 = parameters[0].INTEGER;

	ret = dda_write_proc_entry(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_MODE_SNIFFER, 
		(char *)&proc_pars, sizeof(proc_pars));

	if (ret != sizeof(proc_pars))
		PRINT_PROC_WRITE_ERR(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_MODE_SNIFFER);

}

/*!	\brief 
  	\param 

  	\return 
  
*/
void whalMenu_SetNvByte(ConsoleParameter_T parameters[]){
	//whal_apiSetNvByte(parameters[0].INTEGER, parameters[1].INTEGER);

	proc_params proc_pars;
	int ret;

	DRIVER_LOADED_ASSERT;

	proc_pars.s.magic = PROC_PARAMS_MAGIC;
	proc_pars.s.Param1 = parameters[0].INTEGER;
	proc_pars.s.Param2 = parameters[1].INTEGER;

	ret = dda_write_proc_entry(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_SET_NVBYTE, 
		(char *)&proc_pars, sizeof(proc_pars));

	if (ret != sizeof(proc_pars))
		PRINT_PROC_WRITE_ERR(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_SET_NVBYTE);


}

void whalMenu_TraceEnable(ConsoleParameter_T parameters[]){
	//whal_apiTraceEnable(parameters[0].INTEGER);
	proc_params proc_pars;
	int ret;

	DRIVER_LOADED_ASSERT;

	proc_pars.s.magic = PROC_PARAMS_MAGIC;
	proc_pars.s.Param1 = parameters[0].INTEGER;

	ret = dda_write_proc_entry(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_TRACE_ENABLE, 
		(char *)&proc_pars, sizeof(proc_pars));

	if (ret != sizeof(proc_pars))
		PRINT_PROC_WRITE_ERR(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_TRACE_ENABLE);

}


/*!	\brief 
  	\param 

  	\return 
  
*/
void whalMenu_TracePrint(ConsoleParameter_T parameters[]){

	char buf[GENERIC_IOCTL_BUF_SIZE];

	DRIVER_LOADED_ASSERT;

	if (ioctl_get_trace_print(parameters[0].INTEGER, parameters[1].INTEGER, buf, GENERIC_IOCTL_BUF_SIZE, wd->dev_name)) 
		puts("ioctl_get_trace_print failed !");
	else
		puts(buf);

}

/*!	\brief 
  	\param 

  	\return 
  
*/
void whalMenu_TracePrintRxStat(ConsoleParameter_T parameters[]){

	if(dda_print_proc_entry(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_TRACE_RX_STAT) == -1)
		PRINT_PROC_READ_ERR(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_TRACE_RX_STAT);

}

/*!	\brief 
  	\param 

  	\return 
  
*/
void whalMenu_SelectFwNumber(ConsoleParameter_T parameters[]){
	int ret;
	char enabled;

	switch (parameters[0].INTEGER) {
	case 0:
	case 1:
	case 2:
		enabled = parameters[0].INTEGER == 2;

		printf("%s : Setting automatic FW load option to : %d\n", __FUNCTION__,  enabled);
		if (tiap_wcfg_debug_automatic_fw_load(wd, enabled) ) {
			PRINT_CONFIG_ERR_MSG("Failed to set automatic FW load option.");
		}

		break;
	case 3:
	case 4:
		enabled = parameters[0].INTEGER == 3;

		printf("%s : Setting FW JTAG debug option to : %d\n", __FUNCTION__, enabled);
		if (tiap_wcfg_debug_jtag(wd, enabled) ) {
			PRINT_CONFIG_ERR_MSG("Failed to set automatic FW load option.");
		}

		if (enabled)
			puts("After reboot, load FW using JTAG and signal the driver to continue using CLI hal>DIAG>SelectFwNumber 5");

		break;
	case 5:
		if ((ret = ioctl_set_go(wd, 1)))
			printf("%s : Sending second stage go command failed !!!\n", __FUNCTION__);

		break;
	default:
		puts("Wrong parameter");
		return;
	}



}

/*!	\brief 
  	\param 

  	\return 
  
*/
void whalMenu_SetQueueSizeType(ConsoleParameter_T parameters[]){
	//whal_apiSetQueueSizeType(parameters[0].INTEGER);
	proc_params proc_pars;
	int ret;

	DRIVER_LOADED_ASSERT;

	proc_pars.s.magic = PROC_PARAMS_MAGIC;
	proc_pars.s.Param1 = parameters[0].INTEGER;

	ret = dda_write_proc_entry(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_SETY_Q_SIZE_TYPE, 
		(char *)&proc_pars, sizeof(proc_pars));

	if (ret != sizeof(proc_pars))
		PRINT_PROC_WRITE_ERR(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_SETY_Q_SIZE_TYPE);

}

/*!	\brief 
  	\param 

  	\return 
  
*/
void whalMenu_SelectRadioNumber(ConsoleParameter_T parameters[]){
	//whal_apiSelectRadioNumber(parameters[0].INTEGER);
	int ret;
	proc_params proc_pars;

	DRIVER_LOADED_ASSERT;

	proc_pars.s.magic = PROC_PARAMS_MAGIC;
	proc_pars.s.Param1 = parameters[0].INTEGER;

	ret = dda_write_proc_entry(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_SELEC_RADIO_NUM, 
		(char *)&proc_pars, sizeof(proc_pars));

	if (ret != sizeof(proc_pars))
		PRINT_PROC_WRITE_ERR(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_SELEC_RADIO_NUM);
}

/***************************************************************************/
/* HW Interference handling configuration submenu subroutines                                                */
/***************************************************************************/
void whalMenu_inputCCA(ConsoleParameter_T parameters[])
{
	tiap_param_hw_interf_t param;
	UINT8 val = (UINT8)parameters[0].INTEGER;

	if ((val != 0) && (val != 1))
	{
		printf("Invalid input CCA value: %d\n", val);
		return;
	}

	param.cmd = TIAP_PARAM_INPUT_CCA;
	param.value = val;
	
	if(tiap_wcfg_set_hw_interf_param(wd, &param, UPDATE_STORAGE))
		PRINT_CONFIG_ERR_MSG("Failed to set HW parameter: input CCA\n");
	else
	{
		printf("Input CCA = %d\n", val); 
		printf("to take effect RESTART AP.\n");
	}
	
}

void whalMenu_txCCA(ConsoleParameter_T parameters[])
{
	tiap_param_hw_interf_t param;
	UINT8 val = (UINT8)parameters[0].INTEGER;

	if ((val != 0) && (val != 1))
	{
		printf("Invalid TX CCA value: %d\n", val);
		return;
	}

	param.cmd = TIAP_PARAM_TX_CCA;
	param.value = val;
	
	if(tiap_wcfg_set_hw_interf_param(wd, &param, UPDATE_STORAGE))
		PRINT_CONFIG_ERR_MSG("Failed to set HW parameter: TX CCA\n");
	else
	{
		printf("TX CCA = %d\n", val); 
		printf("to take effect RESTART AP.\n");
	}
	
}

void whalMenu_qualifCCA(ConsoleParameter_T parameters[])
{
	tiap_param_hw_interf_t param;
	UINT8 val = (UINT8)parameters[0].INTEGER;

	if (val > 5)
	{
		printf("Invalid qualified CCA value: %d\n", val);
		return;
	}

	param.cmd = TIAP_PARAM_QUALIF_CCA;
	param.value = val;
	
	if(tiap_wcfg_set_hw_interf_param(wd, &param, UPDATE_STORAGE))
		PRINT_CONFIG_ERR_MSG("Failed to set HW parameter: qualified CCA\n");
	else
	{
		printf("Qualified CCA = %d\n", val); 
		printf("to take effect RESTART AP.\n");
	}
	
}

void whalMenu_stompTx(ConsoleParameter_T parameters[])
{
	tiap_param_hw_interf_t param;
	UINT8 val = (UINT8)parameters[0].INTEGER;

	if (val != 0)
	{
		printf("Invalid stomp for TX value: %d\n", val);
		return;
	}

	param.cmd = TIAP_PARAM_STOMP_TX;
	param.value = val;
	
	if(tiap_wcfg_set_hw_interf_param(wd, &param, UPDATE_STORAGE))
		PRINT_CONFIG_ERR_MSG("Failed to set HW parameter: stomp for TX\n");
	else
	{
		printf("Stomp for TX = %d\n", val); 
		printf("to take effect RESTART AP.\n");
	}
	
}

void whalMenu_hwShow(ConsoleParameter_T parameters[])
{
	tiap_param_hw_interf_t param;

	printf("\nHW interference handling parameters:\n");
	param.cmd = TIAP_PARAM_INPUT_CCA;
	if(tiap_wcfg_get_hw_interf_param(wd, &param, !FROM_STORAGE))
		PRINT_CONFIG_ERR_MSG("Failed to get HW parameter: input CCA\n");
	else
		printf("Input CCA = %d\n", param.value);

	param.cmd = TIAP_PARAM_TX_CCA;
	if(tiap_wcfg_get_hw_interf_param(wd, &param, !FROM_STORAGE))
		PRINT_CONFIG_ERR_MSG("Failed to get HW parameter: TX CCA\n");
	else
		printf("TX CCA = %d\n", param.value);

	param.cmd = TIAP_PARAM_QUALIF_CCA;
	if(tiap_wcfg_get_hw_interf_param(wd, &param, !FROM_STORAGE))
		PRINT_CONFIG_ERR_MSG("Failed to get HW parameter: qualified CCA\n");
	else
		printf("Qualified CCA = %d\n", param.value);

	param.cmd = TIAP_PARAM_STOMP_TX;
	if(tiap_wcfg_get_hw_interf_param(wd, &param, !FROM_STORAGE))
		PRINT_CONFIG_ERR_MSG("Failed to get HW parameter: stomp for TX\n");
	else
		printf("Stomp for TX = %d\n", param.value);

	param.cmd = TIAP_PARAM_STOMP_RX;
	if(tiap_wcfg_get_hw_interf_param(wd, &param, !FROM_STORAGE))
		PRINT_CONFIG_ERR_MSG("Failed to get HW parameter: stomp for RX\n");
	else
		printf("Stomp for RX = %d\n", param.value);

	#if( WLAN_ADAPTOR_VERSION != 1150)
		printf("Warning: the above configuration won't take effect -"" the chip doesn't support interference handling");
	#endif
}

void whalMenu_stompRx(ConsoleParameter_T parameters[])
{
	tiap_param_hw_interf_t param;
	UINT8 val = (UINT8)parameters[0].INTEGER;

	if (val > 6)
	{
		printf("Invalid stomp for RX value: %d\n", val);
		return;
	}

	param.cmd = TIAP_PARAM_STOMP_RX;
	param.value = val;
	
	if(tiap_wcfg_set_hw_interf_param(wd, &param, UPDATE_STORAGE))
		PRINT_CONFIG_ERR_MSG("Failed to set HW parameter: stomp for RX\n");
	else
	{
		printf("Stomp for RX = %d\n", val); 
		printf("to take effect RESTART AP.\n");
	}
	
}

/***************************************************************************/
/* Show submenu subroutines                                                */
/***************************************************************************/
/*!	\brief 
  	\param 

  	\return 
  
*/
void showBssBridge(ConsoleParameter_T parameters[]) 
{
	//wdrv_bssBridgeShowStatistics();
	DRIVER_LOADED_ASSERT;

	if(dda_print_proc_entry(WLAN_PROC_SHOW_DIR_NAME, WLAN_PROC_SHOW_BSS) == -1)
		PRINT_PROC_READ_ERR(WLAN_PROC_SHOW_DIR_NAME, WLAN_PROC_SHOW_BSS);


}


/*!	\brief 
  	\param 

  	\return 
  
*/
void showPowerSave(ConsoleParameter_T parameters[])
{
	//wdrv_powerSavePrintDb();
	DRIVER_LOADED_ASSERT;
	if(dda_print_proc_entry(WLAN_PROC_SHOW_DIR_NAME, WLAN_PROC_SHOW_PS) == -1)
		PRINT_PROC_READ_ERR(WLAN_PROC_SHOW_DIR_NAME, WLAN_PROC_SHOW_PS);

}

/*!	\brief 
  	\param 

  	\return 
  
*/
void showPrintTxStat(ConsoleParameter_T parameters[]){
	//wdrv_txPrintStatistics();
	DRIVER_LOADED_ASSERT;
	if(dda_print_proc_entry(WLAN_PROC_SHOW_DIR_NAME, WLAN_PROC_SHOW_TX_STAT) == -1)
		PRINT_PROC_READ_ERR(WLAN_PROC_SHOW_DIR_NAME, WLAN_PROC_SHOW_TX_STAT);
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void showMibWlan(ConsoleParameter_T parameters[])
{/*
#ifdef VXWORKS
   wlanMib2StatShow(0);
#else
	 puts("What do you want from me ?");
#endif // VXWORKS */
	int ret;

	DRIVER_LOADED_ASSERT;

	ret = dda_print_proc_entry(WLAN_PROC_SHOW_DIR_NAME, WLAN_PROC_SHOW_LAN_MIB);
	printf("ret : %d\n",ret);
	if( ret == -1)
		PRINT_PROC_READ_ERR(WLAN_PROC_SHOW_DIR_NAME, WLAN_PROC_SHOW_LAN_MIB);	

}

/*!	\brief 
  	\param 

  	\return 
  
*/
void showLanStat(ConsoleParameter_T parameters[])
{/*
#ifdef VXWORKS
   extern void sysEmacStatShow( int  unit);
   sysEmacStatShow(0);
#else
	 puts("What do you want from me ?");
#endif // VXWORKS */
	DRIVER_LOADED_ASSERT;
	if(dda_print_proc_entry(WLAN_PROC_SHOW_DIR_NAME, WLAN_PROC_SHOW_LAN_STAT) == -1)
		PRINT_PROC_READ_ERR(WLAN_PROC_SHOW_DIR_NAME, WLAN_PROC_SHOW_LAN_STAT);	
}

void showLanPool(ConsoleParameter_T parameters[])
{/*
#ifdef VXWORKS
   extern void sysEmacNetPoolShow( int  unit);
   sysEmacNetPoolShow(0);
#else
	 puts("What do you want from me ?");
#endif // VXWORKS */
	if(dda_print_proc_entry(WLAN_PROC_SHOW_DIR_NAME, WLAN_PROC_SHOW_LAN_POOL) == -1)
		PRINT_PROC_READ_ERR(WLAN_PROC_SHOW_DIR_NAME, WLAN_PROC_SHOW_LAN_POOL);	

}

/*!	\brief 
  	\param 

  	\return 
  
*/
void showMemMngrStat(ConsoleParameter_T parameters[])
{/*
   memMngr_Statistics_T s;

   puts("------------ MemMngr Stat ------------");
   if( OK == wlan_memMngrPrintStatistics(&s))
   {
      printf("msduPool BDs location   @ %p\n", s.msduPool);
      printf("bdPool data BDs         @ %p\n", s.bdPool);
      printf("List of data buffers    @ %p\n", s.dataBufPool);

      printf("firstFreeMSDU           @ %p\n", s.firstFreeMSDU);
      printf("firstFreeBD             @ %p\n", s.firstFreeBD);
      printf("firstFreeDataBuf        @ %p\n", s.firstFreeDataBuf);

      printf("numFreeMSDU             = %d\n", s.numFreeMSDU     );
      printf("numFreeBD               = %d\n", s.numFreeBD       );
      printf("numFreeDataBuf          = %d\n", s.numFreeDataBuf  );
   } */
	DRIVER_LOADED_ASSERT;
	if(dda_print_proc_entry(WLAN_PROC_SHOW_DIR_NAME, WLAN_PROC_SHOW_MEM_STAT) == -1)
		PRINT_PROC_READ_ERR(WLAN_PROC_SHOW_DIR_NAME, WLAN_PROC_SHOW_MEM_STAT);	

}

/*!	\brief 
  	\param 

  	\return 
  
*/
void showRateAdaptation(ConsoleParameter_T parameters[])
{
	//wdrv_txPrintRateAdaptation();
	DRIVER_LOADED_ASSERT;
	if(dda_print_proc_entry(WLAN_PROC_SHOW_DIR_NAME, WLAN_PROC_SHOW_RATE_ADAP) == -1)
		PRINT_PROC_READ_ERR(WLAN_PROC_SHOW_DIR_NAME, WLAN_PROC_SHOW_RATE_ADAP);	
}


/***************************************************************************/
/* 4x submenu subroutines                                                  */
/***************************************************************************/
/*!	\brief 
  	\param 

  	\return 
  
*/
void show4xParams(ConsoleParameter_T parameters[])
{
	tiap_param_4x_t _4x;

	DRIVER_LOADED_ASSERT;
	/*if(dda_print_proc_entry(WLAN_PROC_4X_DIRNAME, WLAN_PROC_SHOW_4X_MAX_CONCATSIZE) == -1)
		PRINT_PROC_READ_ERR(WLAN_PROC_4X_DIRNAME, WLAN_PROC_SHOW_4X_MAX_CONCATSIZE);	*/
	if(dda_print_proc_entry(WLAN_PROC_4X_DIRNAME, WLAN_PROC_SHOW_4X_MAX_CONCATSIZE) == -1)
		PRINT_PROC_READ_ERR(WLAN_PROC_4X_DIRNAME, WLAN_PROC_SHOW_4X_MAX_CONCATSIZE);
	
	_4x.cmd = TIAP_PARAM_4X_4X;
	if (tiap_wcfg_get_4x(wd, &_4x, !FROM_STORAGE || driver_load_failed))
		PRINT_CONFIG_ERR_MSG("Reading configuration failed.");
	else
	{

		if (_4x.value == TRUE)
		{
			printf("4x:		Enable\n");
		}
		else
			printf("4x:		Disable\n");
	}

	_4x.cmd = TIAP_PARAM_4X_CAT;

	if (tiap_wcfg_get_4x(wd, &_4x, !FROM_STORAGE || driver_load_failed))
		PRINT_CONFIG_ERR_MSG("Reading configuration failed.");
	else
	{
		if (_4x.value == TRUE)
		{
			printf("Concatenation:	Enable\n");
		}
		else
			printf("Concatenation:	Disable\n");
	}


	/* Ack emulation is not supported*/
	/*
	wdrv_dbGetAckEmulEnable(&enable);
	
	if (enable == TRUE)
	{
		printf("Ack Emulation:	Enable\n");
	}
	else
		printf("Ack Emulation:	Disable\n");
	*/


	_4x.cmd = TIAP_PARAM_4X_MIXMOD;

	if (tiap_wcfg_get_4x(wd, &_4x, !FROM_STORAGE || driver_load_failed))
		PRINT_CONFIG_ERR_MSG("Reading configuration failed.");
	else
	{
	    if (_4x.value == TRUE)
		printf("Mix Mode:   Enable\n");
	    else
		printf("Mix Mode:   Disable\n");
	}


	_4x.cmd = TIAP_PARAM_4X_FEATSET;

	if (tiap_wcfg_get_4x(wd, &_4x, !FROM_STORAGE || driver_load_failed))
		PRINT_CONFIG_ERR_MSG("Reading configuration failed.");
	else
		printf("FeatureSet:   %d\n", _4x.value);

	
	//whal_txGetPacketBursting(&burst);

	/* Felix R.: actually it should have been TIAP_PARAM_4X_PKTBURST,
	   but for now we based on TIAP_PARAM_4X_4X */
	_4x.cmd = TIAP_PARAM_4X_4X;

	if (tiap_wcfg_get_4x(wd, &_4x, !FROM_STORAGE || driver_load_failed))
		PRINT_CONFIG_ERR_MSG("Reading configuration failed.");
	else
	{
	    if(_4x.value)
	    {
	       printf("Packet Bursting:		Enable\n");
	    }
	    else
	    {
	       printf("Packet Bursting:		Disable\n");
	    }
	}

	#ifdef TIWLAN_DEBUG
	{
		__u32 burst_limit;

		if (tiap_wcfg_get_basic(wd, TIAP_PARAM_BURST_LIMIT, &burst_limit, !FROM_STORAGE || driver_load_failed)) 
			PRINT_CONFIG_ERR_MSG("Reading configuration failed.");
		else
			printf("Packet burst time limit is %d MicroSec\n", burst_limit);	
	}
	#endif
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void set4xMaxConcatSize(ConsoleParameter_T parameters[])
{
	//wdrv_txSetMaxConcatSize(parameters[0].INTEGER);
	
	int ret;
	proc_params proc_pars;
	
	DRIVER_LOADED_ASSERT;

	proc_pars.s.Param1 = parameters[0].INTEGER;
	proc_pars.s.magic = PROC_PARAMS_MAGIC;

	ret = dda_write_proc_entry(WLAN_PROC_4X_DIRNAME, WLAN_PROC_4X_MAX_CONCATSIZE, 
		(char *)&proc_pars, sizeof(proc_pars));

	if (ret != sizeof(proc_pars))
		PRINT_PROC_WRITE_ERR(WLAN_PROC_4X_DIRNAME, WLAN_PROC_4X_MAX_CONCATSIZE);
	
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void set4x(ConsoleParameter_T parameters[])
{
	tiap_param_4x_t _4x;

	if ((parameters[0].INTEGER != 0) && (parameters[0].INTEGER != 1))
	{
		printf("Invalid 4x mode: %ld\n", parameters[0].INTEGER);
		return;
	}

	if ((parameters[1].INTEGER != 0) && (parameters[1].INTEGER != 1))
	{
		printf("Invalid concatenation mode: %ld\n", parameters[1].INTEGER);
		return;
	}

	if ((BOOL)parameters[0].INTEGER == FALSE)
	{
		printf("Setting all 4X features to FALSE \n");

		_4x.cmd = TIAP_PARAM_4X_4X;
		_4x.value = FALSE;

		if (tiap_wcfg_set_4x(wd, &_4x, 0, UPDATE_STORAGE)) 
		{
			PRINT_CONFIG_ERR_MSG("Configuration failed.");
			return;
		}

		_4x.cmd = TIAP_PARAM_4X_CAT;
		_4x.value = FALSE;

		if (tiap_wcfg_set_4x(wd, &_4x, 0, UPDATE_STORAGE)) 
		{
			PRINT_CONFIG_ERR_MSG("Configuration failed.");
			return;
		}		
	}

	else
	{
		tiap_param_vhcf_t vhcf;

		printf("Setting 4X enable to TRUE \n");
		
		if( tiap_wcfg_get_vhcf(wd, &vhcf, FROM_STORAGE) ) {
			PRINT_CONFIG_ERR_MSG("Failed to get vhcf parameters.");
			return;
		}

		if (vhcf.enable) {
			printf("\nRejected!! Can't enable 4x while vHCF is enabled!!\n");
			return;
		}
		_4x.cmd = TIAP_PARAM_4X_4X;
		_4x.value = TRUE;

		if (tiap_wcfg_set_4x(wd, &_4x, 0, UPDATE_STORAGE)) 
		{
			PRINT_CONFIG_ERR_MSG("Configuration failed.");
			return;
		}

		/* Concatenation  */
		if ((BOOL)parameters[1].INTEGER == TRUE)
		{
			printf("Setting concatenation to TRUE \n");

			_4x.cmd = TIAP_PARAM_4X_CAT;
			_4x.value = TRUE;
			
			if (tiap_wcfg_set_4x(wd, &_4x, 0, UPDATE_STORAGE)) 
			{
				PRINT_CONFIG_ERR_MSG("Configuration failed.");
				return;
			}

		}
		else
		{
			printf("Setting concatenation to FALSE \n");
			
			_4x.cmd = TIAP_PARAM_4X_CAT;
			_4x.value = FALSE;
			
			if (tiap_wcfg_set_4x(wd, &_4x, 0, UPDATE_STORAGE)) 
			{
				PRINT_CONFIG_ERR_MSG("Configuration failed.");
				return;
			}
			
		}
	}
}


/*!	\brief 
  	\param 

  	\return 
  
*/
void setPacketBursting(ConsoleParameter_T parameters[])
{
        tiap_param_4x_t _4x;

	   if ((parameters[0].INTEGER != 0) && (parameters[0].INTEGER != 1))
	   {
	       printf("Invalid mode: %ld\n", parameters[0].INTEGER);
	       return;
	   }
   //whal_txSetPacketBursting((BOOL)parameters[0].INTEGER); 
   
	
	_4x.cmd = TIAP_PARAM_4X_PKTBURST;
	_4x.value = (UINT8) parameters[0].INTEGER;

	if (tiap_wcfg_set_4x(wd, &_4x, 0, UPDATE_STORAGE)) 
	{
		PRINT_CONFIG_ERR_MSG("Configuration failed.");
	}


}

/*!	\brief 
  	\param 

  	\return 
  
*/
void setConcatDebug(ConsoleParameter_T parameters[])
{
	PRINT_NOT_SUPPORTED_MSG("Concat debug");
	/*
	if ((parameters[0].INTEGER != 0) && (parameters[0].INTEGER != 1))
	{
		printf("Invalid concat debug mode: %d\n", parameters[0].INTEGER);
		return;
	}
	wdrv_txSetConcatDebug((BOOL)parameters[0].INTEGER); */
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void fourXTable(ConsoleParameter_T parameters[])
{
	DRIVER_LOADED_ASSERT;
	if(dda_print_proc_entry(WLAN_PROC_MGMT_DIR_NAME, WLAN_PROC_MGMT_4XTABLE) == -1)
		PRINT_PROC_READ_ERR(WLAN_PROC_MGMT_DIR_NAME, WLAN_PROC_MGMT_4XTABLE);	

}



/*!	\brief 
  	\param 

  	\return 
  
*/
void set4xFeatureSet(ConsoleParameter_T parameters[])
{
	tiap_param_4x_t _4x;

	if(parameters[0].SHORT != 0 && parameters[0].SHORT != 1)
		printf("Wrong FeatureSet value\n");

	/* when setting/resetting FeatureSet, the ERPProtectionTI must be set/reset too 
	   In apdk5.6 this logic is implemented in the shared library */
	
	//wdrv_dbSetFeatureSet(parameters[0].SHORT);
	
	
	_4x.cmd = TIAP_PARAM_4X_FEATSET;
	_4x.value = (UINT8) parameters[0].SHORT;

	/* the wone index is needed in order to set the ERPProtectionTI */
	if (tiap_wcfg_set_4x(wd, &_4x, currentConfigWoneInex, UPDATE_STORAGE)) 
	{
		PRINT_CONFIG_ERR_MSG("Configuration failed.");
	}
	else
	{
		printf("Wone Index %d: Setting 4x FeatureSet to %d\n",currentConfigWoneInex, parameters[0].SHORT);
		printf("NOTE: Setting the FeatureSet sets automatically the ERPProtectionTI variable to the same value!!!\n"); 
		printf("If you would like a different value for this variable you have to change it manually.\n"); 
		printf("to take effect RESTART AP.\n"); 
	}
}


/*!	\brief 
  	\param 

  	\return 
  
*/
void setBurstLimit(ConsoleParameter_T parameters[])
{/*
      whal_nvram_setTxopLimitParams(parameters[0].INTEGER, IMMEDIATELY);
      printf("\n Set Packet Burst time limit is %d MicroSec\n",parameters[0].INTEGER); */
      
	  #ifdef TIWLAN_DEBUG
		  if (tiap_wcfg_set_basic(wd, TIAP_PARAM_BURST_LIMIT, &(parameters[0].SHORT), UPDATE_STORAGE)) 
		  {
			  PRINT_CONFIG_ERR_MSG("Configuration failed.");
		  }
		  else
		  {
			  printf("\nPacket Burst time limit was set to %d MicroSec\n",parameters[0].SHORT); 
			  printf("to take effect RESTART AP.\n");
		  }
	  #else
		  printf("Burst limit is configurable only in the DEBUG version\n");

	  #endif
		  
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void wlanSet4xMixMode(ConsoleParameter_T parameters[])
{

	tiap_param_4x_t _4x;
	
	_4x.cmd = TIAP_PARAM_4X_MIXMOD;
	_4x.value = parameters[0].SHORT;

	if (tiap_wcfg_set_4x(wd, &_4x, 0, UPDATE_STORAGE)) 
	{
		PRINT_CONFIG_ERR_MSG("Configuration failed.");
	}
	else
	{
	   if( parameters[0].SHORT == 0 )
		   printf("4x mix disabled \n"); 
	   else
	      printf("4x mix enabled\n");
	}


}



/***************************************************************************/
/* Execute the last command                                                */
/***************************************************************************/
/*!	\brief 
  	\param 

  	\return 
  
*/
void consoleLastCommand(ConsoleParameter_T parameters[])
{
	
   unsigned char lastCommand[WAPP_UTILS_MAX_LINE_LENGTH];

   getDevConsoleCommand(lastCommand);
   consoleExecuteCommand(lastCommand, (ConsoleCommand_T *)NULL);
   
}


void enableWds(ConsoleParameter_T parameters[])
{
    tiap_wcfg_enableWds(wd, parameters[0].INTEGER, 1);
}


void applyWdsParams(ConsoleParameter_T parameters[])
{
    if(!wd->storage.ldb.wdsEnable) {
        printf("WDS is disabled.\n");
        return ;
    }
    if(tiap_wcfg_configWds(wd, 1, 1) ) {
        printf("failed.\n");
    }
    else {
        printf("done.\n");
    }
}


void showWdsParams(ConsoleParameter_T parameters[])
{
    __u8 hop, chSel, autoCfg, privacy, bssMode, crudeMode;
    char buff[128];

    tiap_wcfg_wdsHopGet(wd, &hop);
    tiap_wcfg_wdsChSelModeGet(wd, &chSel);
    tiap_wcfg_wdsAutoCfgGet(wd, &autoCfg);
    tiap_wcfg_wdsPrivacyGet(wd, &privacy);
    tiap_wcfg_wdsWirelessIfModeGet(wd, &bssMode);
    tiap_wcfg_wdsCrudeModeGet(wd, &crudeMode);

    printf("WDS %s.\n", ((wd->storage.ldb.wdsEnable) ? "Enabled" : "Disabled"));
    printf("hop : %d\nch select mode : %d\nauto config : %d\nprivacy : %d\nbss mode : %d, crude mode %d\n",
           hop, chSel, autoCfg, privacy, bssMode, crudeMode);
    tiap_wcfg_wdsNameGet(wd, buff);
    printf("WDS name : %s\n", buff);
    tiap_wcfg_wdsSharedSecretGet(wd, buff);
    printf("Shared secret : %s\n", buff);
    tiap_wcfg_wdsUlMacGet(wd, buff);
    printf("Uplink MAC : %hhX:%hhX:%hhX:%hhX:%hhX:%hhX\n",
           buff[0], buff[1], buff[2], buff[3], buff[4], buff[5]);
    tiap_wcfg_wdsDl0MacGet(wd, buff);
    printf("Downlink 0 MAC : %hhX:%hhX:%hhX:%hhX:%hhX:%hhX\n",
           buff[0], buff[1], buff[2], buff[3], buff[4], buff[5]);
    tiap_wcfg_wdsDl1MacGet(wd, buff);
    printf("Downlink 1 MAC : %hhX:%hhX:%hhX:%hhX:%hhX:%hhX\n",
           buff[0], buff[1], buff[2], buff[3], buff[4], buff[5]);
    tiap_wcfg_wdsDl2MacGet(wd, buff);
    printf("Downlink 2 MAC : %hhX:%hhX:%hhX:%hhX:%hhX:%hhX\n",
           buff[0], buff[1], buff[2], buff[3], buff[4], buff[5]);
    tiap_wcfg_wdsDl3MacGet(wd, buff);
    printf("Downlink 3 MAC : %hhX:%hhX:%hhX:%hhX:%hhX:%hhX\n",
           buff[0], buff[1], buff[2], buff[3], buff[4], buff[5]);
}
void showWdsReport(ConsoleParameter_T parameters[])    
{
    tiap_wcfg_wdsReport(wd);
}

void setWdsDefaults(ConsoleParameter_T parameters[])
{
    tiap_wcfg_setWdsDefaults(wd);
}

void showWdsStat(ConsoleParameter_T parameters[])    
{
    tiap_wcfg_wdsStat(wd);
}

void showWdsInfoDb(ConsoleParameter_T parameters[])    
{
    tiap_wcfg_wdsInfoDb(wd);
}

void setWdsHop(ConsoleParameter_T parameters[])        
{
    if(tiap_wcfg_wdsHopSet(wd, parameters[0].INTEGER, UPDATE_STORAGE)) {
        printf("failed.\n");
    }
    else {
        printf("done.\n");
    }
}
void setWdsChSel(ConsoleParameter_T parameters[])      
{
    if(tiap_wcfg_wdsChSelModeSet(wd, parameters[0].INTEGER, UPDATE_STORAGE)) {
        printf("failed.\n");
    }
    else {
        printf("done.\n");
    }
}
void setWdsAutoCfg(ConsoleParameter_T parameters[])    
{
    if(tiap_wcfg_wdsAutoCfgSet(wd, parameters[0].INTEGER, UPDATE_STORAGE)) {
        printf("failed.\n");
    }
    else {
        printf("done.\n");
    }
}
void setWdsPrivacy(ConsoleParameter_T parameters[])    
{
    if(tiap_wcfg_wdsPrivacySet(wd, parameters[0].INTEGER, UPDATE_STORAGE)) {
        printf("failed.\n");
    }
    else {
        printf("done.\n");
    }
}
void setWdsName(ConsoleParameter_T parameters[])       
{
    if(tiap_wcfg_wdsNameSet(wd, parameters[0].STRING, UPDATE_STORAGE)) {
        printf("failed.\n");
    }
    else {
        printf("done.\n");
    }
}
void setWdsBssMode(ConsoleParameter_T parameters[])    
{
    if(tiap_wcfg_wdsWirelessIfModeSet(wd, parameters[0].INTEGER, UPDATE_STORAGE)) {
        printf("failed.\n");
    }
    else {
        printf("done.\n");
    }
}
void setWdsCrudeMode(ConsoleParameter_T parameters[])    
{
    if(tiap_wcfg_wdsCrudeModeSet(wd, parameters[0].INTEGER, UPDATE_STORAGE)) {
        printf("failed.\n");
    }
    else {
        printf("done.\n");
    }
}
void setWdsSharedSecret(ConsoleParameter_T parameters[])
{
    if(tiap_wcfg_wdsSharedSecretSet(wd, parameters[0].STRING, UPDATE_STORAGE)) {
        printf("failed.\n");
    }
    else {
        printf("done.\n");
    }
}
void setWdsUplink(ConsoleParameter_T parameters[])     
{
    if(tiap_wcfg_wdsUlMacSet(wd, parameters[0].MAC_ADDR, UPDATE_STORAGE)) {
        printf("failed.\n");
    }
    else {
        printf("done.\n");
    }
}
void setWdsDownlink0(ConsoleParameter_T parameters[])  
{
    if(tiap_wcfg_wdsDl0MacSet(wd, parameters[0].MAC_ADDR, UPDATE_STORAGE)) {
        printf("failed.\n");
    }
    else {
        printf("done.\n");
    }
}
void setWdsDownlink1(ConsoleParameter_T parameters[])  
{
    if(tiap_wcfg_wdsDl1MacSet(wd, parameters[0].MAC_ADDR, UPDATE_STORAGE)) {
        printf("failed.\n");
    }
    else {
        printf("done.\n");
    }
}
void setWdsDownlink2(ConsoleParameter_T parameters[])  
{
    if(tiap_wcfg_wdsDl2MacSet(wd, parameters[0].MAC_ADDR, UPDATE_STORAGE)) {
        printf("failed.\n");
    }
    else {
        printf("done.\n");
    }
}
void setWdsDownlink3(ConsoleParameter_T parameters[])  
{
    if(tiap_wcfg_wdsDl3MacSet(wd, parameters[0].MAC_ADDR, UPDATE_STORAGE)) {
        printf("failed.\n");
    }
    else {
        printf("done.\n");
    }
}

void setWdsPrintSeverity(ConsoleParameter_T parameters[])
{
    if(!wd->storage.ldb.wdsEnable) {
        printf("WDS is enabled.\n");
        return ;
    }
    tiap_wcfg_wdsSeveritySet(wd, parameters[0].INTEGER, parameters[1].INTEGER, parameters[2].INTEGER);
}

void setWdsRestart(ConsoleParameter_T parameters[])
{
    if(!wd->storage.ldb.wdsEnable) {
        printf("WDS is enabled.\n");
        return ;
    }
    tiap_wcfg_wdsRestart(wd);
}

/*!	
    \brief Sets IP address in the storage
  	\param input parameter structure
*/


#define VALIDATE_IP( octet ) ((octet) < 0 || (octet) > 255)
#define SET_IPADR( ip, b0, b1, b2, b3 ) \
        (ip) = (((b0) & 0xFF) | (((b1) & 0xFF) << 8) | (((b2) & 0xFF) << 16) | (((b3) & 0xFF) << 24))
                                                                                          
void setIpAddr( ConsoleParameter_T parameters[] )
{
    __u32 ipAdr;

    if (VALIDATE_IP( parameters[0].INTEGER )) goto setIpAddrError;
    if (VALIDATE_IP( parameters[1].INTEGER )) goto setIpAddrError;
    if (VALIDATE_IP( parameters[2].INTEGER )) goto setIpAddrError;
    if (VALIDATE_IP( parameters[3].INTEGER )) goto setIpAddrError;

    SET_IPADR( ipAdr, parameters[3].INTEGER, parameters[2].INTEGER, 
               parameters[1].INTEGER, parameters[0].INTEGER );

    tiap_wcfg_set_storage_ipaddr(wd, SIOCSIFADDR, htonl(ipAdr));

    return;

 setIpAddrError:

    printf("IP address is not valid: %d %d %d %d\n", 
           (int) parameters[0].INTEGER, (int) parameters[1].INTEGER, 
           (int) parameters[2].INTEGER, (int) parameters[3].INTEGER );
    return;

}

/*!	
    \brief Sets IP subnet mask in the storage
  	\param input parameter structure
*/

void setIpMask( ConsoleParameter_T parameters[] )
{
    __u32 ipAdr;

    if (VALIDATE_IP( parameters[0].INTEGER )) goto setIpAddrError;
    if (VALIDATE_IP( parameters[1].INTEGER )) goto setIpAddrError;
    if (VALIDATE_IP( parameters[2].INTEGER )) goto setIpAddrError;
    if (VALIDATE_IP( parameters[3].INTEGER )) goto setIpAddrError;

    SET_IPADR( ipAdr, parameters[3].INTEGER, parameters[2].INTEGER, 
               parameters[1].INTEGER, parameters[0].INTEGER );

    tiap_wcfg_set_storage_ipaddr(wd, SIOCSIFNETMASK, htonl(ipAdr));

    return;

 setIpAddrError:

    printf("IP address is not valid: %d %d %d %d\n", 
           (int) parameters[0].INTEGER, (int) parameters[1].INTEGER, 
           (int) parameters[2].INTEGER, (int) parameters[3].INTEGER );
    return;

}

/*!	
    \brief Sets default gateway in the storage
  	\param input parameter structure
*/

void setDefGateway( ConsoleParameter_T parameters[] )
{
    __u32 ipAdr;

    if (VALIDATE_IP( parameters[0].INTEGER )) goto setIpAddrError;
    if (VALIDATE_IP( parameters[1].INTEGER )) goto setIpAddrError;
    if (VALIDATE_IP( parameters[2].INTEGER )) goto setIpAddrError;
    if (VALIDATE_IP( parameters[3].INTEGER )) goto setIpAddrError;

    SET_IPADR( ipAdr, parameters[3].INTEGER, parameters[2].INTEGER, 
               parameters[1].INTEGER, parameters[0].INTEGER );

    tiap_wcfg_set_storage_ipaddr(wd, SIOCADDRT, htonl(ipAdr));

    return;

 setIpAddrError:

    printf("IP address is not valid: %d %d %d %d\n", 
           (int) parameters[0].INTEGER, (int) parameters[1].INTEGER, 
           (int) parameters[2].INTEGER, (int) parameters[3].INTEGER );
    return;

}

/*!	
    \brief Sets IP configuration from the storage
  	\param input parameter structure
*/

void applyIpSettings( ConsoleParameter_T parameters[] )
{
    __u32 gateway;

    /* delete previous default route */
    if( tiap_wcfg_get_default_gateway(&gateway) ) {
        printf( "%s: Error, failed to get default route\n", __FUNCTION__ );
    }
    else if( tiap_wcfg_set_route(wd->br_name, 0, 0, gateway, SIOCDELRT) ) {
        printf( "%s: Error, failed to delete default route\n", __FUNCTION__ );
    }

#if 0
    if( tiap_wcfg_set_ipaddr_or_netmask(wd->br_name, 
                                        SIOCSIFNETMASK, 
                                        wd->storage.ldb.netmask)) {
        printf( "%s: Error, failed to set IP subnet mask=%x\n", __FUNCTION__, 
                wd->storage.ldb.netmask );
        return;
    }

    if( tiap_wcfg_set_ipaddr_or_netmask(wd->br_name, SIOCSIFADDR, wd->storage.ldb.ipaddr)) {
        printf( "%s: Error, failed to set IP address=%x\n", __FUNCTION__, 
                wd->storage.ldb.ipaddr );
        return;
    }
#endif

    if( tiap_wcfg_set_route(wd->br_name, 0, 0, wd->storage.ldb.default_gw, SIOCADDRT) ) {
        printf( "%s: Error, failed to set default gateway=%x\n", __FUNCTION__, 
                wd->storage.ldb.default_gw );
        return;
    }
}

/*!	
    \brief Shows current IP settings
  	\param input parameter structure
*/

void showIpSettings( ConsoleParameter_T parameters[] )
{
    char   dotIpAdr[ INET_ADDR_LEN ];
    char   dotIpMask[ INET_ADDR_LEN ];
    char   dotGateway[ INET_ADDR_LEN ];

    if (tiap_wcfg_get_ipConfig( wd->br_name, dotIpAdr, dotIpMask, dotGateway )) {
		printf("%s: Failed to get IP configuration.\n", __FUNCTION__);
		return;
    }
    printf("IP address:      %s\n", dotIpAdr );
    printf("Subnet mask:     %s\n", dotIpMask );
    printf("Default gateway: %s\n", dotGateway );
}


/***************************************************************************/
/*                      production submenu items                           */
/***************************************************************************/

#ifdef VXWORKS 

void consoleDir(ConsoleParameter_T parameters[])
{
   //cm_serviceDir();
}
void consoleErase(ConsoleParameter_T parameters[])
{
   //cm_serviceErase(parameters[0].INTEGER);
}
void consoleDl(ConsoleParameter_T parameters[])
{
/* cm_serviceDownload(parameters[0].INTEGER,
                      parameters[1].IP_ADDR,
                      parameters[2].STRING); */
}
void consoleSetDef(ConsoleParameter_T parameters[])
{
   //cm_serviceSetDef(parameters[0].INTEGER);
}
void consoleProdShow(ConsoleParameter_T parameters[])
{
   //cm_serviceShow();
}
void consoleProdSet(ConsoleParameter_T parameters[])
{
   //cm_serviceSet();
}
void consoleProdDef(ConsoleParameter_T parameters[])
{/*
   int   wone_index;
   for (wone_index = 0 ;wone_index < MAX_WONE_NET; wone_index++)
   {
       whal_nvram_setDefaults(wone_index);
   }
   cm_serviceSetDefs();*/
}

void consolePassword(ConsoleParameter_T parameters[])
{/*
   char pbuf1[256] = {0};
   char pbuf2[256] = {0};
   printf("Enter new password> ");
	 fgets(pbuf1, 255, stdin);
	 pbuf1[255] = 0;
   if(!strlen(pbuf1))
   {
      return;
   }
   printf("Reenter password> ");
	 fgets(pbuf2, 255, stdin);
	 pbuf2[255] = 0;
   if(strcmp(pbuf1,pbuf2))
   {
      puts("Error: Password isnt match");
      return;
   }
   whal_nvram_setConsolePassword(pbuf1,IMMEDIATELY);
   */
}

#endif /* VXWORKS */

/***************************************************************************/
/* Add sub menu to the console                                             */
/***************************************************************************/
/*!	\brief 
  	\param 

  	\return 
  
*/
long wapp_utilsAddSubMenusToConsole(void)
{
	int ret, servRet, privRet;
	/*int iTestSubmenuContext;
	  int iBringUpSubmenuContext;*/
	ConsoleCommandParameter_T cParams[] =  {NO_PARAM,  NO_PARAM, NO_PARAM, NO_PARAM, NO_PARAM};
	//ConsoleCommandParameter_T subMenuParams[] = {STRING_PRM,NO_PARAM, NO_PARAM, NO_PARAM, NO_PARAM};

	ret = addConsoleSubMenu(
			wapp_utilsConsoleCurrentContext,
			"BIT> ",
			ConsoleBITCommandArray,
			"TNETW Built In Tests (BIT) sub menu",
			cParams,0,
			BITCB);

	if( ret == -1)
		puts("AddConsoleMenu failed while adding 'BIT' sub-menu.");


	ret = addConsoleSubMenu(
			wapp_utilsConsoleCurrentContext,
			"mgmt> ",
			ConsoleMgmtCommandArray,
			"Mgmt parameters",
			cParams,0,
			MgmtCB);


	if (ret == -1)
		printf("AddConsoleMenu failed while adding 'MGMT' sub-menu\n");

	else
	{
		ret = addConsoleSubMenu(
				ret,
				"test-mgmt> ",
				ConsoleMgmtTstCommandArray,
				"Mgmt Test sub menu",
				cParams,0,
				MgmtTstCB);


		if (ret == -1)
			printf("AddConsoleMenu failed while adding 'MGMT-TEST' sub-menu\n");
	}


	privRet = addConsoleSubMenu(
			wapp_utilsConsoleCurrentContext,
			"privacy > ",
			ConsoleWpaCommandArray,
			"Privacy parameters",
			cParams,0,
			WpaCB);


	if (privRet == -1)
		printf("AddConsoleMenu failed while adding 'privacy' sub-menu\n");

	else
	{
        /*  temporary removed */
	  /*  ret = addConsoleSubMenu(
				privRet,
				"test-wpa> ",
				ConsoleWpaTstCommandArray,
				"Wpa Test sub menu",
				cParams,0,
				WpaTstCB);
		if (ret == -1)
			printf("AddConsoleMenu failed while adding 'WPA-TEST' sub-menu\n"); */

		ret = addConsoleSubMenu(
				privRet,
				"Reports> ",
				ConsoleWpaReportCommandArray,
				"Privacy reports sub menu",
				cParams,0,
				WpaRepCB);


		if (ret == -1)
			printf("AddConsoleMenu failed while adding 'Reports' sub-menu\n");
	}


	if (ret == -1)
		printf("AddConsoleMenu failed while adding 'PRIVACY' sub-menu\n");

	servRet = addConsoleSubMenu(
			wapp_utilsConsoleCurrentContext,
			"serv> ",
			ConsoleServiceAppCommandArray,
			"WLAN Port Set-Up",
			cParams,0,
			ServCB);

	if (servRet == -1)
		printf("AddConsoleMenu failed while adding 'SERVICE' sub-menu\n");
	else  {
		ret = addConsoleSubMenu(
				wapp_utilsConsoleCurrentContext,
				"ssidMgmt> ",
				ConsoleSsidAppCommandArray,
				"SSID Library Features set-up",
				cParams,0,
				ServCB);
		if (ret == -1)
			printf("AddConsoleMenu failed while adding 'SSID' sub-menu\n");
		else  {
			ret = addConsoleSubMenu(
					servRet,
					"features> ",
					Console_DH_AppCommandArray,
					"Advanced features (e.g. 802.11d) configuration",
					cParams,0,
					ServCB);
			if (ret == -1)
				printf("AddConsoleMenu failed while adding 'QoS' sub-menu\n");
			else  {
				ret = addConsoleSubMenu(
						servRet,
						"QoS> ",
						ConsoleQoSCommandArray,
						"QoS (WME) configuration",
						cParams,0,
						ServCB);
				if (ret == -1)
					printf("AddConsoleMenu failed while adding 'QoS' sub-menu\n");
			}
		}
	}

	ret = addConsoleSubMenu(
			wapp_utilsConsoleCurrentContext,
			"show> ",
			ConsoleShowCommandArray,
			"Show WLAN statistics",
			cParams,0,
			ShowCB);

	if (ret == -1)
		printf("AddConsoleMenu failed while adding 'SHOW' sub-menu\n");

	ret = addConsoleSubMenu(
			wapp_utilsConsoleCurrentContext,
			"4x> ",
			Console4xCommandArray,
			"Set 4x parameters ",
			cParams,0,
			x4CB);

	if (ret == -1)
		printf("AddConsoleMenu failed while adding 'SHOW' sub-menu\n");


	privRet = addConsoleSubMenu(
			wapp_utilsConsoleCurrentContext,
			"hal> ",
			ConsoleHalCommandArray,
			"Hal parameters",
			cParams,0,
			HalCB);
	if (privRet == -1)
		printf("AddConsoleMenu failed while adding 'HAL' sub-menu\n");
	else
	{
		ret = addConsoleSubMenu(
				privRet,
				"Diag> ",
				ConsoleHalDiagCommandArray,
				"Hal diagnostic parameters",
				cParams,0,
				HalDiagCB);


		if (ret == -1)
			printf("AddConsoleMenu failed while adding 'HAL-DIAG' sub-menu\n");

		ret = addConsoleSubMenu(
				privRet,
				"hwInterfCfg> ",
				ConsoleHalHwInterfCommandArray,
				"HW Interference handling parameters (only for 1150 and higher)",
				cParams,0,
				HwInterfCB);


		if (ret == -1)
			printf("AddConsoleMenu failed while adding 'HAL-DIAG' sub-menu\n");
	}


	ret = addConsoleSubMenu(
			wapp_utilsConsoleCurrentContext,
			"wds> ",
			ConsoleWdsCommandArray,
			"WDS parameters",
			cParams,0,
			WdsCB);
	if (ret == -1)
		printf("AddConsoleMenu failed while adding 'WDS' sub-menu\n");

	ret = addConsoleSubMenu(
			wapp_utilsConsoleCurrentContext,
			"IP> ",
			ConsoleIpSetCommandArray,
			"IP Setting",
			cParams,0,
			IpSetCB);
	if (ret == -1)
		printf("AddConsoleMenu failed while adding 'IP' sub-menu\n");


#ifdef INCLUDE_TESTAPP
	iTestSubmenuContext = addConsoleSubMenu(wapp_utilsConsoleCurrentContext,
			"ta>",
			consoleTestAppCommandArray,
			"Test Application",
			cParams,0,
			TestAppCB);
	if (iTestSubmenuContext == -1)
	{
		puts("AddConsoleMenu failed while adding 'TA' sub-menu");
	}
	else
	{
		iBringUpSubmenuContext = addConsoleSubMenu(iTestSubmenuContext,
				"bringup>",
				consoleBringUpCommandArray ,         
				"bringup <password>",
				subMenuParams,1,
				BringUpCB);
		if (iBringUpSubmenuContext == -1)
		{
			puts("AddConsoleMenu failed while adding 'TA->Bringup' sub-menu");
			return;
		}
	}
	/* Add bringup->tx submenu */
	ret = addConsoleSubMenu(iBringUpSubmenuContext,
			"tx>",
			consoleBringUpTxCommandArray ,         
			"tx",
			cParams,0,
			BringUpTxCB);
	if(ret == -1)
	{
		puts("AddConsoleMenu failed while adding 'Bringup->TX' sub-menu");
		return -1;
	}
	ret = addConsoleSubMenu(iBringUpSubmenuContext,
			"rx>",
			consoleBringUpRxCommandArray ,         
			"rx",
			cParams,0,
			BringUpTxCB);
	if(ret == -1)
	{
		puts("AddConsoleMenu failed while adding 'Bringup->RX' sub-menu");
		return -1;
	}
#endif   /* INCLUDE_TESTAPP */
	return 0;
}  /* END wapp_utilsAddSubMenusToConsole() */



/*!	\brief 
  	\param 

  	\return 
  
*/
void dbg_switch_atu_flushall(void)
{/*
   int is_sw = tnetd53xx_emac_phy_is_switch(0);
   int result;

   if (is_sw <= 0)
   {
      puts("command not supported");
      return;
   }

   // flush all ATU entries 
   result = mv_tnetd53xx_emac_phy_atu_mng(
      0, MV_ATU_OP_FLUSH_ALL_ENTRIES, 0, NULL);

   if( result == 0)
      puts("Flushing all entries");
   else
      printf("Error code %d\n", result);*/
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void dbg_switch_atu_flushunlocked(void)
{/*
   int result;
   int is_sw;

   is_sw = tnetd53xx_emac_phy_is_switch(0);
   if( is_sw <= 0)
   {
      puts( "command not supported");
      return;
   }

   // flush all ATU entries 
   result = mv_tnetd53xx_emac_phy_atu_mng(
      0, MV_ATU_OP_FLUSH_ALL_UNLOCKED_ENTRIES, 0, NULL);

   if( result == 0)
      puts("Flushing all unlocked entries");
   else
      printf("Error code %d\n", result);*/
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void dbg_switch_atu_flushall_db(void)
{/*
   int is_sw = tnetd53xx_emac_phy_is_switch(0);
   int result;
   UINT32 dbn=0; 

   if (is_sw <= 0)
   {
      puts("Command not supported");
      return;
   }

   // flush all ATU entries 
   result = mv_tnetd53xx_emac_phy_atu_mng(
      0, MV_ATU_OP_FLUSH_DB, dbn, NULL);

   if( result == 0)
      printf("Flushing all entries in db %d\n", dbn);
   else
      printf("Error code %d\n", result);*/
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void dbg_switch_atu_flushunlocked_db(void)
{/*
   int is_sw = tnetd53xx_emac_phy_is_switch(0);
   int result;
   UINT32 dbn=0;  //  this should be an argument

   if (is_sw <= 0)
   {
      puts("Command not supported");
      return;
   }

   // flush all ATU entries 
   result = mv_tnetd53xx_emac_phy_atu_mng(
      0, MV_ATU_OP_FLUSH_UNLOCKED_DB, dbn, NULL);

   if( result == 0)
      printf("Flushing all unlocked entries in db %d\n", dbn);
   else
      printf("Error code %d\n", result);*/
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void dbg_switch_datu(void)
{/*
   // Display switch valid entries 

   UINT32 dbn = 0; // this should be an argument 
   int i;
   int result;
   int is_sw = tnetd53xx_emac_phy_is_switch(0);
   MV_SWITCH_ATU_ENTRY entry;

   if (is_sw <= 0)
   {
      puts("command not supported");
      return;
   }

   printf( "Displaying all learned MAC addresses at DB#%d:\n", dbn);
   memset( &(entry.atu_mac[0]), 0xff, sizeof(entry.atu_mac));
   i =0;
   do{
      mv_tnetd53xx_emac_phy_atu_mng( 0, MV_ATU_OP_GETNEXT, dbn, &entry);
      printf( "%5d   %02x-%02x-%02x-%02x-%02x-%02x  pri=0x%x pv=0x%02x state=0x%x %cCAST\n",
         ++i, (UINT)(entry.atu_mac[0]), (UINT)(entry.atu_mac[1]),
         (UINT)(entry.atu_mac[2]), (UINT)(entry.atu_mac[3]),
         (UINT)(entry.atu_mac[4]), (UINT)(entry.atu_mac[5]),
         (UINT)(entry.atu_data_entryPri),
         (UINT)(entry.atu_data_portVec),
         (UINT)(entry.atu_data_entryState),
         mv_tnetd53xx_emac_atu_is_mcast_addr(&entry) ? 'M':'U'
         );
   }while( ((entry.atu_mac[0] & entry.atu_mac[1] & entry.atu_mac[2] & entry.atu_mac[3] &
             entry.atu_mac[4] & entry.atu_mac[5]) != 0xff)
           && (entry.atu_data_entryState != MV_SWITCH_ATU_ENTRY_STATE_INVALID)
           );*/
}

/*!	\brief 
  	\param 

  	\return 
  
*/
void dbg_switch_atu_delMAC(UINT8 *p_buff)
{/*
   int result;
   char* tok = p_buff;
   unsigned int mac[6] = {0};
   UINT32 dbn =0;    // Add this as an argument 
   MV_SWITCH_ATU_ENTRY entry;

   // delete a MAC address from entry: rem <MAC> <dbNum> 
   int is_sw = tnetd53xx_emac_phy_is_switch(0);

   if (is_sw <= 0)
   {
      puts("Command not supported");
      return;
   }

   // validate MAC 
   if(! tok)
   {
      puts("Usage: rem <MAC> <dbNum>");
      return;
   }

   if( (sscanf( tok, fmtMac, &mac[0], &mac[1], &mac[2], &mac[3], &mac[4], &mac[5]) !=  6)
      || (mac[0] > 255) || (mac[1] > 255) || (mac[2] > 255)
      || (mac[3] > 255) || (mac[4] > 255) || (mac[5] > 255) )
   {
      puts("Invalid MAC address");
      return;
   }


   // Build command for switch 
   memset(&entry, 0, sizeof(entry));
   entry.atu_mac[0] = (UINT8)(mac[0]);
   entry.atu_mac[1] = (UINT8)(mac[1]);
   entry.atu_mac[2] = (UINT8)(mac[2]);
   entry.atu_mac[3] = (UINT8)(mac[3]);
   entry.atu_mac[4] = (UINT8)(mac[4]);
   entry.atu_mac[5] = (UINT8)(mac[5]);
   entry.atu_data_entryState = MV_SWITCH_ATU_ENTRY_STATE_INVALID;

   result = mv_tnetd53xx_emac_phy_atu_mng( 0, MV_ATU_OP_LOAD_PURGE_ENTRY, dbn, &entry);
   if( result == 0)
      printf( "Removing %02x-%02x-%02x-%02x-%02x-%02x from DB#%u\n", 
         mac[0],      mac[1],      mac[2],
         mac[3],      mac[4],      mac[5],
         dbn
         );
   else
      puts( "Failed");*/
}

#ifndef VXWORKS
/*!	\brief 
  	\param 

  	\return 
  
*/

int main(int argc, char **argv) {	
	if (tiap_wcfg_start() != OK)
		 return 1;

	wdrv_dbInitRangeTables();

	wapp_utilsConsoleInit();
	consoleTask();

	return 0;
}

int tiap_wcfg_start(void) {
	wcfginit init;

	memset(&init, 0, sizeof(init));
	strcpy(init.storage_file, WCFG_MTDBLOCK);

	if( !(wd = tiap_wcfg_init(&init, &driver_load_failed)) ) {
		if (driver_load_failed) {
			puts("\n\nDriver Init failed, probably due to misconfiguration. \nRunning CLI to fix the problem\n");
			if( !(wd = malloc(sizeof(wcfgd))) )
				return NOK;
			memset(wd, 0, sizeof(wcfgd));
			/*store storage file*/
			strncpy(wd->storage_file, init.storage_file, _POSIX_NAME_MAX+_POSIX_PATH_MAX);
			wd->storage_file[_POSIX_NAME_MAX+_POSIX_PATH_MAX] = '\0';
		} else
			return NOK;		
	}
	return OK;
}

#else
int klogctl( int mode,  void * dummy, int idummy)
{
	return OK;
}
#endif

/*!	\brief 
  	\param 

  	\return 
  
*/
static void access_list_to_str(char *buff, int buff_size, tiap_param_access_list_t access_list, 
			       int *num_entries)
{
	 int i, res, count;
	 
	 char zeromac[ETH_ALEN];

	memset(zeromac, 0, sizeof(zeromac));
	*buff = '\0';
      
	    for (i=0, count=0, *num_entries=0; i<TIAP_PARAM_ACCESS_LIST_SIZE; ++i) 
	    {
	       /* if empty mac, return */

		if(! memcmp(access_list.list[i], zeromac, ETH_ALEN))
		{
		    return;     /* end of table */
		}
	
		res = sprintf(buff+count,   
			      "%2d : %02hhx-%02hhx-%02hhx-%02hhx-%02hhx-%02hhx\n", 
			      i+1, 
			      access_list.list[i][0], access_list.list[i][1],
			      access_list.list[i][2], access_list.list[i][3],
			      access_list.list[i][4], access_list.list[i][5]);
		if (count+5 > buff_size)    /* almost end of given buffer */
		    return;
		if (res<0) 
		    return;
		else
		    count+=res;
		(*num_entries)++;
	    }

}


